@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/amd64/amd64/bios.c,v 1.72 2004/09/24 01:08:34 peter Exp $");
+__FBSDID("$FreeBSD: src/sys/amd64/amd64/bios.c,v 1.73 2005/07/21 09:48:36 phk Exp $");
 
 /*
  * Subset of the i386 bios support code.  We cannot make bios16 nor bios32
@@ -93,18 +93,3 @@ bios_sigsearch(u_int32_t start, u_char *sig, int siglen, int paralen, int sigofs
     }
     return(0);
 }
-
-const u_char *
-bios_string(u_int from, u_int to, const u_char *string, int len)
-{
-	const char *t, *te;
-
-	if (len == 0)
-		len = strlen(string);
-	t = (const char *)(KERNBASE + from);
-	te = (const char *)(KERNBASE + to);
-	for (; t <= te; t++)
-		if (!memcmp(string, t, len))
-			return (t);
-	return (NULL);
-}
