@@ -29,7 +29,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/arm/arm/busdma_machdep.c,v 1.22.2.5 2007/10/16 21:13:59 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/arm/arm/busdma_machdep.c,v 1.35 2007/08/18 16:47:28 cognet Exp $");
 
 /*
  * ARM bus dma support routines
@@ -621,7 +621,7 @@ bus_dmamem_alloc(bus_dma_tag_t dmat, void** vaddr, int flags,
 		void *tmpaddr = arm_remap_nocache(
 		    (void *)((vm_offset_t)*vaddr &~ PAGE_MASK),
 		    dmat->maxsize + ((vm_offset_t)*vaddr & PAGE_MASK));
-		
+
 		if (tmpaddr) {
 			tmpaddr = (void *)((vm_offset_t)(tmpaddr) +
 			    ((vm_offset_t)*vaddr & PAGE_MASK));
@@ -675,7 +675,7 @@ _bus_dmamap_count_pages(bus_dma_tag_t dmat, bus_dmamap_t map, void *buf,
 		CTR4(KTR_BUSDMA, "lowaddr= %d Maxmem= %d, boundary= %d, "
 		    "alignment= %d", dmat->lowaddr, ptoa((vm_paddr_t)Maxmem),
 		    dmat->boundary, dmat->alignment);
-		CTR2(KTR_BUSDMA, "map= %p,  pagesneeded= %d",
+		CTR2(KTR_BUSDMA, "map= %p, pagesneeded= %d",
 		    map, map->pagesneeded);
 		/*
 		 * Count the number of bounce pages
@@ -763,8 +763,12 @@ bus_dmamap_load_buffer(bus_dma_tag_t dmat, bus_dma_segment_t *segs,
 		if (__predict_true(pmap == pmap_kernel())) {
 			(void) pmap_get_pde_pte(pmap, vaddr, &pde, &ptep);
 			if (__predict_false(pmap_pde_section(pde))) {
-				curaddr = (*pde & L1_S_FRAME) |
-				    (vaddr & L1_S_OFFSET);
+				if (*pde & L1_S_SUPERSEC)
+					curaddr = (*pde & L1_SUP_FRAME) |
+					    (vaddr & L1_SUP_OFFSET);
+				else
+					curaddr = (*pde & L1_S_FRAME) |
+					    (vaddr & L1_S_OFFSET);
 				if (*pde & L1_S_CACHE_MASK) {
 					map->flags &=
 					    ~DMAMAP_COHERENT;
@@ -800,6 +804,8 @@ bus_dmamap_load_buffer(bus_dma_tag_t dmat, bus_dma_segment_t *segs,
 		 * Compute the segment size, and adjust counts.
 		 */
 		sgsize = PAGE_SIZE - ((u_long)curaddr & PAGE_MASK);
+		if (sgsize > dmat->maxsegsz)
+			sgsize = dmat->maxsegsz;
 		if (buflen < sgsize)
 			sgsize = buflen;
 
@@ -1004,7 +1010,7 @@ bus_dmamap_load_uio(bus_dma_tag_t dmat, bus_dmamap_t map, struct uio *uio,
     bus_dmamap_callback2_t *callback, void *callback_arg,
     int flags)
 {
-	vm_offset_t lastaddr;
+	vm_offset_t lastaddr = 0;
 #ifdef __CC_SUPPORTS_DYNAMIC_ARRAY_INIT
 	bus_dma_segment_t dm_segments[dmat->nsegments];
 #else
@@ -1080,42 +1086,47 @@ _bus_dmamap_unload(bus_dma_tag_t dmat, bus_dmamap_t map)
 	return;
 }
 
-
-static __inline void
+static void
 bus_dmamap_sync_buf(void *buf, int len, bus_dmasync_op_t op)
 {
 	char _tmp_cl[arm_dcache_align], _tmp_clend[arm_dcache_align];
 
-	if (op & BUS_DMASYNC_PREWRITE)
+	if ((op & BUS_DMASYNC_PREWRITE) && !(op & BUS_DMASYNC_PREREAD)) {
 		cpu_dcache_wb_range((vm_offset_t)buf, len);
+		cpu_l2cache_wb_range((vm_offset_t)buf, len);
+	}
+	if (op & BUS_DMASYNC_PREREAD) {
+		if ((op & BUS_DMASYNC_PREWRITE) ||
+		    ((((vm_offset_t)(buf) | len) & arm_dcache_align_mask) == 0)) {
+			cpu_dcache_inv_range((vm_offset_t)buf, len);
+			cpu_l2cache_inv_range((vm_offset_t)buf, len);
+		} else {
+		    	cpu_dcache_wbinv_range((vm_offset_t)buf, len);
+	    		cpu_l2cache_wbinv_range((vm_offset_t)buf, len);
+		}
+	}
 	if (op & BUS_DMASYNC_POSTREAD) {
-		if ((vm_offset_t)buf & arm_dcache_align_mask)
+		if ((vm_offset_t)buf & arm_dcache_align_mask) {
 			memcpy(_tmp_cl, (void *)((vm_offset_t)buf & ~
 			    arm_dcache_align_mask),
-			    (vm_offset_t)buf - ((vm_offset_t)buf &~
-			    arm_dcache_align_mask));
-		if (((vm_offset_t)buf + len) & arm_dcache_align_mask)
-			memcpy(_tmp_cl, (void *)((vm_offset_t)buf & ~
-			    arm_dcache_align_mask),
-			    (vm_offset_t)buf - ((vm_offset_t)buf &~
-			    arm_dcache_align_mask));
-		if (((vm_offset_t)buf + len) & arm_dcache_align_mask)
-			memcpy(_tmp_clend, (void *)(((vm_offset_t)buf + len) & ~
-			    arm_dcache_align_mask),
-			    (vm_offset_t)buf +len - (((vm_offset_t)buf + len) &~
-			    arm_dcache_align_mask));
+			    (vm_offset_t)buf & arm_dcache_align_mask);
+		}
+		if (((vm_offset_t)buf + len) & arm_dcache_align_mask) {
+			memcpy(_tmp_clend, (void *)((vm_offset_t)buf + len),
+			    arm_dcache_align - (((vm_offset_t)(buf) + len) &
+			   arm_dcache_align_mask));
+		}
 		cpu_dcache_inv_range((vm_offset_t)buf, len);
+		cpu_l2cache_inv_range((vm_offset_t)buf, len);
+
 		if ((vm_offset_t)buf & arm_dcache_align_mask)
 			memcpy((void *)((vm_offset_t)buf &
-			    ~arm_dcache_align_mask),
-			    _tmp_cl, 
-			    (vm_offset_t)buf - ((vm_offset_t)buf &~
-			    arm_dcache_align_mask));
+			    ~arm_dcache_align_mask), _tmp_cl, 
+			    (vm_offset_t)buf & arm_dcache_align_mask);
 		if (((vm_offset_t)buf + len) & arm_dcache_align_mask)
-			memcpy((void *)(((vm_offset_t)buf + len) & ~
-			    arm_dcache_align_mask), _tmp_clend,
-			    (vm_offset_t)buf +len - (((vm_offset_t)buf + len) &~
-			    arm_dcache_align_mask));
+			memcpy((void *)((vm_offset_t)buf + len), _tmp_clend,
+			    arm_dcache_align - (((vm_offset_t)(buf) + len) &
+			   arm_dcache_align_mask));
 	}
 }
 
@@ -1130,14 +1141,20 @@ _bus_dmamap_sync_bp(bus_dma_tag_t dmat, bus_dmamap_t map, bus_dmasync_op_t op)
 			    (void *)(bpage->vaddr_nocache != 0 ? 
 				     bpage->vaddr_nocache : bpage->vaddr),
 			    bpage->datacount);
-			if (bpage->vaddr_nocache == 0)
+			if (bpage->vaddr_nocache == 0) {
 				cpu_dcache_wb_range(bpage->vaddr,
 				    bpage->datacount);
+				cpu_l2cache_wb_range(bpage->vaddr,
+				    bpage->datacount);
+			}
 		}
 		if (op & BUS_DMASYNC_POSTREAD) {
-			if (bpage->vaddr_nocache == 0)
+			if (bpage->vaddr_nocache == 0) {
 				cpu_dcache_inv_range(bpage->vaddr,
 				    bpage->datacount);
+				cpu_l2cache_inv_range(bpage->vaddr,
+				    bpage->datacount);
+			}
 			bcopy((void *)(bpage->vaddr_nocache != 0 ? 
 	       		    bpage->vaddr_nocache : bpage->vaddr),
 			    (void *)bpage->datavaddr, bpage->datacount);
@@ -1174,10 +1191,6 @@ _bus_dmamap_sync(bus_dma_tag_t dmat, bus_dmamap_t map, bus_dmasync_op_t op)
 		_bus_dmamap_sync_bp(dmat, map, op);
 	if (map->flags & DMAMAP_COHERENT)
 		return;
-	if ((op && BUS_DMASYNC_POSTREAD) && (map->len > PAGE_SIZE)) {
-		cpu_dcache_wbinv_all();
-		return;
-	}
 	CTR3(KTR_BUSDMA, "%s: op %x flags %x", __func__, op, map->flags);
 	switch(map->flags & DMAMAP_TYPE_MASK) {
 	case DMAMAP_LINEAR:
