@@ -29,11 +29,11 @@
  *
  * 1) detach
  * 2) Free dma setup
- * 3) Turn on the clock in pmc and turn on pins?  Turn off?
+ * 3) Turn on the clock in pmc?  Turn off?
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/arm/at91/if_ate.c,v 1.8.2.2.4.1 2007/12/02 14:21:26 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/arm/at91/if_ate.c,v 1.20.2.1 2007/12/02 14:19:37 cognet Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -45,6 +45,7 @@ __FBSDID("$FreeBSD: src/sys/arm/at91/if_ate.c,v 1.8.2.2.4.1 2007/12/02 14:21:26
 #include <sys/rman.h>
 #include <sys/socket.h>
 #include <sys/sockio.h>
+#include <sys/sysctl.h>
 #include <machine/bus.h>
 
 #include <net/ethernet.h>
@@ -71,7 +72,7 @@ __FBSDID("$FreeBSD: src/sys/arm/at91/if_ate.c,v 1.8.2.2.4.1 2007/12/02 14:21:26
 
 #include "miibus_if.h"
 
-#define ATE_MAX_TX_BUFFERS 64		/* We have ping-pong tx buffers */
+#define ATE_MAX_TX_BUFFERS 2		/* We have ping-pong tx buffers */
 #define ATE_MAX_RX_BUFFERS 64
 
 struct ate_softc
@@ -131,7 +132,6 @@ static void atestart_locked(struct ifnet *);
 static void ateinit(void *);
 static void atestart(struct ifnet *);
 static void atestop(struct ate_softc *);
-static void atewatchdog(struct ifnet *);
 static int ateioctl(struct ifnet * ifp, u_long, caddr_t);
 
 /* bus entry points */
@@ -146,7 +146,7 @@ static int ate_activate(device_t dev);
 static void ate_deactivate(device_t dev);
 static int ate_ifmedia_upd(struct ifnet *ifp);
 static void ate_ifmedia_sts(struct ifnet *ifp, struct ifmediareq *ifmr);
-static void ate_get_mac(struct ate_softc *sc, u_char *eaddr);
+static int ate_get_mac(struct ate_softc *sc, u_char *eaddr);
 static void ate_set_mac(struct ate_softc *sc, u_char *eaddr);
 
 /*
@@ -167,6 +167,8 @@ ate_attach(device_t dev)
 {
 	struct ate_softc *sc = device_get_softc(dev);
 	struct ifnet *ifp = NULL;
+	struct sysctl_ctx_list *sctx;
+	struct sysctl_oid *soid;
 	int err;
 	u_char eaddr[6];
 
@@ -177,15 +179,23 @@ ate_attach(device_t dev)
 
 	sc->use_rmii = (RD4(sc, ETH_CFG) & ETH_CFG_RMII) == ETH_CFG_RMII;
 
+	/*Sysctls*/
+	sctx = device_get_sysctl_ctx(dev);
+	soid = device_get_sysctl_tree(dev);
+	SYSCTL_ADD_UINT(sctx, SYSCTL_CHILDREN(soid), OID_AUTO, "rmii",
+	    CTLFLAG_RD, &sc->use_rmii, 0, "rmii in use");
+
 	/* calling atestop before ifp is set is OK */
 	atestop(sc);
 	ATE_LOCK_INIT(sc);
 	callout_init_mtx(&sc->tick_ch, &sc->sc_mtx, 0);
 
-	ate_get_mac(sc, eaddr);
+	if ((err = ate_get_mac(sc, eaddr)) != 0) {
+		device_printf(dev, "No MAC address set");
+		goto out;
+	}
 	ate_set_mac(sc, eaddr);
 
-
 	sc->ifp = ifp = if_alloc(IFT_ETHER);
 	if (mii_phy_probe(dev, &sc->miibus, ate_ifmedia_upd, ate_ifmedia_sts)) {
 		device_printf(dev, "Cannot find my PHY.\n");
@@ -195,15 +205,15 @@ ate_attach(device_t dev)
 
 	ifp->if_softc = sc;
 	if_initname(ifp, device_get_name(dev), device_get_unit(dev));
-	ifp->if_mtu = ETHERMTU;
 	ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
+	ifp->if_capabilities |= IFCAP_VLAN_MTU;
+	ifp->if_capenable |= IFCAP_VLAN_MTU; /* the hw bits already set */
 	ifp->if_start = atestart;
 	ifp->if_ioctl = ateioctl;
-	ifp->if_watchdog = atewatchdog;
 	ifp->if_init = ateinit;
 	ifp->if_baudrate = 10000000;
 	IFQ_SET_MAXLEN(&ifp->if_snd, IFQ_MAXLEN);
-	ifp->if_snd.ifq_maxlen = IFQ_MAXLEN;
+	ifp->if_snd.ifq_drv_maxlen = IFQ_MAXLEN;
 	IFQ_SET_READY(&ifp->if_snd);
 	ifp->if_timer = 0;
 	ifp->if_linkmib = &sc->mibdata;
@@ -216,7 +226,7 @@ ate_attach(device_t dev)
 	 * Activate the interrupt
 	 */
 	err = bus_setup_intr(dev, sc->irq_res, INTR_TYPE_NET | INTR_MPSAFE,
-	    ate_intr, sc, &sc->intrhand);
+	    NULL, ate_intr, sc, &sc->intrhand);
 	if (err) {
 		ether_ifdetach(ifp);
 		ATE_LOCK_DESTROY(sc);
@@ -261,10 +271,12 @@ ate_load_rx_buf(void *arg, bus_dma_segment_t *segs, int nsegs, int error)
 	 * For the last buffer, set the wrap bit so the controller
 	 * restarts from the first descriptor.
 	 */
+	bus_dmamap_sync(sc->rx_desc_tag, sc->rx_desc_map, BUS_DMASYNC_PREWRITE);
 	if (i == ATE_MAX_RX_BUFFERS - 1)
 		sc->rx_descs[i].addr = segs[0].ds_addr | ETH_WRAP_BIT;
 	else
 		sc->rx_descs[i].addr = segs[0].ds_addr;
+	bus_dmamap_sync(sc->rx_desc_tag, sc->rx_desc_map, BUS_DMASYNC_POSTWRITE);
 	sc->rx_descs[i].status = 0;
 	/* Flush the memory in the mbuf */
 	bus_dmamap_sync(sc->rxtag, sc->rx_map[i], BUS_DMASYNC_PREREAD);
@@ -323,7 +335,7 @@ ate_activate(device_t dev)
 	rid = 0;
 	sc->irq_res = bus_alloc_resource_any(dev, SYS_RES_IRQ, &rid,
 	    RF_ACTIVE);
-	if (sc->mem_res == NULL)
+	if (sc->irq_res == NULL)
 		goto errout;
 
 	/*
@@ -352,11 +364,6 @@ ate_activate(device_t dev)
 	    busdma_lock_mutex, &sc->sc_mtx, &sc->rxtag);
 	if (err != 0)
 		goto errout;
-	for (i = 0; i < ATE_MAX_RX_BUFFERS; i++) {
-		err = bus_dmamap_create(sc->rxtag, 0, &sc->rx_map[i]);
-		if (err != 0)
-			goto errout;
-	}
 
 	/* Dma TAG and MAP for the rx descriptors. */
 	err = bus_dma_tag_create(NULL, sizeof(eth_rx_desc_t), 0, 
@@ -483,12 +490,31 @@ ate_ifmedia_sts(struct ifnet *ifp, struct ifmediareq *ifmr)
 	ATE_UNLOCK(sc);
 }
 
+static void
+ate_stat_update(struct ate_softc *sc, int active)
+{
+	/*
+	 * The speed and full/half-duplex state needs to be reflected
+	 * in the ETH_CFG register.
+	 */
+	if (IFM_SUBTYPE(active) == IFM_10_T)
+		WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) & ~ETH_CFG_SPD);
+	else
+		WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) | ETH_CFG_SPD);
+	if (active & IFM_FDX)
+		WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) | ETH_CFG_FD);
+	else
+		WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) & ~ETH_CFG_FD);
+}
+
 static void
 ate_tick(void *xsc)
 {
 	struct ate_softc *sc = xsc;
+	struct ifnet *ifp = sc->ifp;
 	struct mii_data *mii;
 	int active;
+	uint32_t c;
 
 	/*
 	 * The KB920x boot loader tests ETH_SR & ETH_SR_LINK and will ask
@@ -501,25 +527,8 @@ ate_tick(void *xsc)
 		active = mii->mii_media_active;
 		mii_tick(mii);
 		if (mii->mii_media_status & IFM_ACTIVE &&
-		     active != mii->mii_media_active) {
-			/*
-			 * The speed and full/half-duplex state needs
-			 * to be reflected in the ETH_CFG register, it
-			 * seems.
-			 */
-			if (IFM_SUBTYPE(mii->mii_media_active) == IFM_10_T)
-				WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) &
-				    ~ETH_CFG_SPD);
-			else
-				WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) |
-				    ETH_CFG_SPD);
-			if (mii->mii_media_active & IFM_FDX)
-				WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) |
-				    ETH_CFG_FD);
-			else
-				WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) &
-				    ~ETH_CFG_FD);
-		}
+		     active != mii->mii_media_active)
+			ate_stat_update(sc, mii->mii_media_active);
 	}
 
 	/*
@@ -530,16 +539,25 @@ ate_tick(void *xsc)
 	 * the dot3Stats mib, so for those we just count them as general
 	 * errors.  Stats for iframes, ibutes, oframes and obytes are
 	 * collected elsewhere.  These registers zero on a read to prevent
-	 * races.
+	 * races.  For all the collision stats, also update the collision
+	 * stats for the interface.
 	 */
 	sc->mibdata.dot3StatsAlignmentErrors += RD4(sc, ETH_ALE);
 	sc->mibdata.dot3StatsFCSErrors += RD4(sc, ETH_SEQE);
-	sc->mibdata.dot3StatsSingleCollisionFrames += RD4(sc, ETH_SCOL);
-	sc->mibdata.dot3StatsMultipleCollisionFrames += RD4(sc, ETH_MCOL);
+	c = RD4(sc, ETH_SCOL);
+	ifp->if_collisions += c;
+	sc->mibdata.dot3StatsSingleCollisionFrames += c;
+	c = RD4(sc, ETH_MCOL);
+	sc->mibdata.dot3StatsMultipleCollisionFrames += c;
+	ifp->if_collisions += c;
 	sc->mibdata.dot3StatsSQETestErrors += RD4(sc, ETH_SQEE);
 	sc->mibdata.dot3StatsDeferredTransmissions += RD4(sc, ETH_DTE);
-	sc->mibdata.dot3StatsLateCollisions += RD4(sc, ETH_LCOL);
-	sc->mibdata.dot3StatsExcessiveCollisions += RD4(sc, ETH_ECOL);
+	c = RD4(sc, ETH_LCOL);
+	sc->mibdata.dot3StatsLateCollisions += c;
+	ifp->if_collisions += c;
+	c = RD4(sc, ETH_ECOL);
+	sc->mibdata.dot3StatsExcessiveCollisions += c;
+	ifp->if_collisions += c;
 	sc->mibdata.dot3StatsCarrierSenseErrors += RD4(sc, ETH_CSE);
 	sc->mibdata.dot3StatsFrameTooLongs += RD4(sc, ETH_ELR);
 	sc->mibdata.dot3StatsInternalMacReceiveErrors += RD4(sc, ETH_DRFC);
@@ -547,7 +565,7 @@ ate_tick(void *xsc)
 	 * not sure where to lump these, so count them against the errors
 	 * for the interface.
 	 */
-	sc->ifp->if_oerrors += RD4(sc, ETH_CSE) + RD4(sc, ETH_TUE);
+	sc->ifp->if_oerrors += RD4(sc, ETH_TUE);
 	sc->ifp->if_ierrors += RD4(sc, ETH_CDE) + RD4(sc, ETH_RJB) +
 	    RD4(sc, ETH_USF);
 
@@ -563,33 +581,36 @@ ate_set_mac(struct ate_softc *sc, u_char *eaddr)
 	WR4(sc, ETH_SA1L, (eaddr[3] << 24) | (eaddr[2] << 16) |
 	    (eaddr[1] << 8) | eaddr[0]);
 	WR4(sc, ETH_SA1H, (eaddr[5] << 8) | (eaddr[4]));
-
 }
 
-static void
+static int
 ate_get_mac(struct ate_softc *sc, u_char *eaddr)
 {
-    uint32_t low, high;
+	uint32_t low, high;
 
-    /*
-     * The KB920x loaders will setup the MAC with an address, if one
-     * is set in the loader.  The TSC loader will also set the MAC address
-     * in a similar way.  Grab the MAC address from the SA1[HL] registers.
-     */
-    low = RD4(sc, ETH_SA1L);
-    high =  RD4(sc, ETH_SA1H);
-    eaddr[0] = low & 0xff;
-    eaddr[1] = (low >> 8) & 0xff;
-    eaddr[2] = (low >> 16) & 0xff;
-    eaddr[3] = (low >> 24) & 0xff;
-    eaddr[4] = high & 0xff;
-    eaddr[5] = (high >> 8) & 0xff;
+	/*
+	 * The boot loader setup the MAC with an address, if one is set in
+	 * the loader.  The TSC loader will also set the MAC address in a
+	 * similar way.  Grab the MAC address from the SA1[HL] registers.
+	 */
+	low = RD4(sc, ETH_SA1L);
+	high =  RD4(sc, ETH_SA1H);
+	if ((low | (high & 0xffff)) == 0)
+		return (ENXIO);
+	eaddr[0] = low & 0xff;
+	eaddr[1] = (low >> 8) & 0xff;
+	eaddr[2] = (low >> 16) & 0xff;
+	eaddr[3] = (low >> 24) & 0xff;
+	eaddr[4] = high & 0xff;
+	eaddr[5] = (high >> 8) & 0xff;
+	return (0);
 }
 
 static void
 ate_intr(void *xsc)
 {
 	struct ate_softc *sc = xsc;
+	struct ifnet *ifp = sc->ifp;
 	int status;
 	int i;
 	void *bp;
@@ -609,15 +630,18 @@ ate_intr(void *xsc)
 			rx_stat = sc->rx_descs[i].status;
 			if ((rx_stat & ETH_LEN_MASK) == 0) {
 				printf("ignoring bogus 0 len packet\n");
-				sc->rx_descs[i].addr &= ~ETH_CPU_OWNER;
 				bus_dmamap_sync(sc->rx_desc_tag, sc->rx_desc_map,
 				    BUS_DMASYNC_PREWRITE);
+				sc->rx_descs[i].addr &= ~ETH_CPU_OWNER;
+				bus_dmamap_sync(sc->rx_desc_tag, sc->rx_desc_map,
+				    BUS_DMASYNC_POSTWRITE);
 				continue;
 			}
 			/* Flush memory for mbuf so we don't get stale bytes */
 			bus_dmamap_sync(sc->rxtag, sc->rx_map[i],
 			    BUS_DMASYNC_POSTREAD);
-			WR4(sc, ETH_RSR, RD4(sc, ETH_RSR));	// XXX WHY? XXX imp
+			WR4(sc, ETH_RSR, RD4(sc, ETH_RSR));
+
 			/*
 			 * The length returned by the device includes the
 			 * ethernet CRC calculation for the packet, but
@@ -625,25 +649,37 @@ ate_intr(void *xsc)
 			 */
 			mb = m_devget(sc->rx_buf[i],
 			    (rx_stat & ETH_LEN_MASK) - ETHER_CRC_LEN,
-			    ETHER_ALIGN, sc->ifp, NULL);
-			sc->rx_descs[i].addr &= ~ETH_CPU_OWNER;
+			    ETHER_ALIGN, ifp, NULL);
 			bus_dmamap_sync(sc->rx_desc_tag, sc->rx_desc_map,
 			    BUS_DMASYNC_PREWRITE);
+			sc->rx_descs[i].addr &= ~ETH_CPU_OWNER;
+			bus_dmamap_sync(sc->rx_desc_tag, sc->rx_desc_map,
+			    BUS_DMASYNC_POSTWRITE);
 			bus_dmamap_sync(sc->rxtag, sc->rx_map[i],
 			    BUS_DMASYNC_PREREAD);
-			if (mb != NULL)
-				(*sc->ifp->if_input)(sc->ifp, mb);
+			if (mb != NULL) {
+				ifp->if_ipackets++;
+				(*ifp->if_input)(ifp, mb);
+			}
+			
 		}
 	}
 	if (status & ETH_ISR_TCOM) {
 		ATE_LOCK(sc);
+		/* XXX TSR register should be cleared */
 		if (sc->sent_mbuf[0]) {
+			bus_dmamap_sync(sc->rxtag, sc->tx_map[0],
+			    BUS_DMASYNC_POSTWRITE);
 			m_freem(sc->sent_mbuf[0]);
+			ifp->if_opackets++;
 			sc->sent_mbuf[0] = NULL;
 		}
 		if (sc->sent_mbuf[1]) {
 			if (RD4(sc, ETH_TSR) & ETH_TSR_IDLE) {
+				bus_dmamap_sync(sc->rxtag, sc->tx_map[1],
+				    BUS_DMASYNC_POSTWRITE);
 				m_freem(sc->sent_mbuf[1]);
+				ifp->if_opackets++;
 				sc->txcur = 0;
 				sc->sent_mbuf[0] = sc->sent_mbuf[1] = NULL;
 			} else {
@@ -679,6 +715,7 @@ ateinit_locked(void *xsc)
 {
 	struct ate_softc *sc = xsc;
 	struct ifnet *ifp = sc->ifp;
+ 	struct mii_data *mii;
 
 	ATE_ASSERT_LOCKED(sc);
 
@@ -720,12 +757,19 @@ ateinit_locked(void *xsc)
 	 */
 	ate_setmcast(sc);
 
+	/* enable big packets */
+	WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) | ETH_CFG_BIG);
+
 	/*
 	 * Set 'running' flag, and clear output active flag
 	 * and attempt to start the output
 	 */
 	ifp->if_drv_flags |= IFF_DRV_RUNNING;
 	ifp->if_drv_flags &= ~IFF_DRV_OACTIVE;
+
+	mii = device_get_softc(sc->miibus);
+	mii_pollstat(mii);
+	ate_stat_update(sc, mii->mii_media_active);
 	atestart_locked(ifp);
 
 	callout_reset(&sc->tick_ch, hz, ate_tick, sc);
@@ -740,7 +784,7 @@ atestart_locked(struct ifnet *ifp)
 	struct ate_softc *sc = ifp->if_softc;
 	struct mbuf *m, *mdefrag;
 	bus_dma_segment_t segs[1];
-	int nseg;
+	int nseg, e;
 
 	ATE_ASSERT_LOCKED(sc);
 	if (ifp->if_drv_flags & IFF_DRV_OACTIVE)
@@ -762,14 +806,19 @@ atestart_locked(struct ifnet *ifp)
 			ifp->if_drv_flags &= ~IFF_DRV_OACTIVE;
 			return;
 		}
-		mdefrag = m_defrag(m, M_DONTWAIT);
-		if (mdefrag == NULL) {
-			IFQ_DRV_PREPEND(&ifp->if_snd, m);
-			return;
+		e = bus_dmamap_load_mbuf_sg(sc->mtag, sc->tx_map[sc->txcur], m,
+		    segs, &nseg, 0);
+		if (e == EFBIG) {
+			mdefrag = m_defrag(m, M_DONTWAIT);
+			if (mdefrag == NULL) {
+				IFQ_DRV_PREPEND(&ifp->if_snd, m);
+				return;
+			}
+			m = mdefrag;
+			e = bus_dmamap_load_mbuf_sg(sc->mtag,
+			    sc->tx_map[sc->txcur], m, segs, &nseg, 0);
 		}
-		m = mdefrag;
-		if (bus_dmamap_load_mbuf_sg(sc->mtag, sc->tx_map[sc->txcur], m,
-		    segs, &nseg, 0) != 0) {
+		if (e != 0) {
 			m_freem(m);
 			continue;
 		}
@@ -867,25 +916,13 @@ atestop(struct ate_softc *sc)
 	 */
 }
 
-static void
-atewatchdog(struct ifnet *ifp)
-{
-	struct ate_softc *sc = ifp->if_softc;
-
-	ATE_LOCK(sc);
-	device_printf(sc->dev, "Device timeout\n");
-	ifp->if_oerrors++;
-	ateinit_locked(sc);
-	ATE_UNLOCK(sc);
-}
-
 static int
 ateioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 {
 	struct ate_softc *sc = ifp->if_softc;
  	struct mii_data *mii;
  	struct ifreq *ifr = (struct ifreq *)data;	
-	int             error = 0;
+	int mask, error = 0;
 
 	switch (cmd) {
 	case SIOCSIFFLAGS:
@@ -915,6 +952,19 @@ ateioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
  		mii = device_get_softc(sc->miibus);
  		error = ifmedia_ioctl(ifp, ifr, &mii->mii_media, cmd);
   		break;
+	case SIOCSIFCAP:
+		mask = ifp->if_capenable ^ ifr->ifr_reqcap;
+		if (mask & IFCAP_VLAN_MTU) {
+			ATE_LOCK(sc);
+			if (ifr->ifr_reqcap & IFCAP_VLAN_MTU) {
+				WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) | ETH_CFG_BIG);
+				ifp->if_capenable |= IFCAP_VLAN_MTU;
+			} else {
+				WR4(sc, ETH_CFG, RD4(sc, ETH_CFG) & ~ETH_CFG_BIG);
+				ifp->if_capenable &= ~IFCAP_VLAN_MTU;
+			}
+			ATE_UNLOCK(sc);
+		}
 	default:
 		error = ether_ioctl(ifp, cmd, data);
 		break;
