@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/arm/at91/uart_dev_at91usart.c,v 1.6.2.6 2006/07/25 23:17:46 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/arm/at91/uart_dev_at91usart.c,v 1.12 2007/04/02 22:00:21 marcel Exp $");
 
 #include "opt_comconsole.h"
 
@@ -45,14 +45,8 @@ __FBSDID("$FreeBSD: src/sys/arm/at91/uart_dev_at91usart.c,v 1.6.2.6 2006/07/25 2
 #include <arm/at91/at91_usartreg.h>
 #include <arm/at91/at91_pdcreg.h>
 
-#ifdef KDB
-#include <sys/kdb.h>
-#endif
-
 #include "uart_if.h"
 
-#define uart_lock(x)	mtx_lock_spin(&(x))
-#define uart_unlock(x)	mtx_unlock_spin(&(x))
 #define DEFAULT_RCLK		AT91C_MASTER_CLOCK
 #define	USART_BUFFER_SIZE	128
 
@@ -100,8 +94,8 @@ static int at91_usart_probe(struct uart_bas *bas);
 static void at91_usart_init(struct uart_bas *bas, int, int, int, int);
 static void at91_usart_term(struct uart_bas *bas);
 static void at91_usart_putc(struct uart_bas *bas, int);
-static int at91_usart_poll(struct uart_bas *bas);
-static int at91_usart_getc(struct uart_bas *bas);
+static int at91_usart_rxready(struct uart_bas *bas);
+static int at91_usart_getc(struct uart_bas *bas, struct mtx *mtx);
 
 extern SLIST_HEAD(uart_devinfo_list, uart_devinfo) uart_sysdevs;
 
@@ -202,12 +196,12 @@ at91_usart_param(struct uart_bas *bas, int baudrate, int databits,
 	return (0);
 }
 
-struct uart_ops at91_usart_ops = {
+static struct uart_ops at91_usart_ops = {
 	.probe = at91_usart_probe,
 	.init = at91_usart_init,
 	.term = at91_usart_term,
 	.putc = at91_usart_putc,
-	.poll = at91_usart_poll,
+	.rxready = at91_usart_rxready,
 	.getc = at91_usart_getc,
 };
 
@@ -258,22 +252,20 @@ at91_usart_putc(struct uart_bas *bas, int c)
 }
 
 /*
- * Poll for a character available
+ * Check for a character available.
  */
 static int
-at91_usart_poll(struct uart_bas *bas)
+at91_usart_rxready(struct uart_bas *bas)
 {
 
-	if (!(RD4(bas, USART_CSR) & USART_CSR_RXRDY))
-		return (-1);
-	return (RD4(bas, USART_RHR) & 0xff);
+	return ((RD4(bas, USART_CSR) & USART_CSR_RXRDY) != 0 ? 1 : 0);
 }
 
 /*
  * Block waiting for a character.
  */
 static int
-at91_usart_getc(struct uart_bas *bas)
+at91_usart_getc(struct uart_bas *bas, struct mtx *mtx)
 {
 	int c;
 
@@ -329,7 +321,10 @@ at91_getaddr(void *arg, bus_dma_segment_t *segs, int nsegs, int error)
 static int
 at91_usart_bus_attach(struct uart_softc *sc)
 {
-	int err, i;
+#ifndef SKYEYE_WORKAROUNDS
+	int err;
+	int i;
+#endif
 	uint32_t cr;
 	struct at91_usart_softc *atsc;
 
@@ -351,6 +346,7 @@ at91_usart_bus_attach(struct uart_softc *sc)
 	sc->sc_rxfifosz = USART_BUFFER_SIZE;
 	sc->sc_hwiflow = 0;
 
+#ifndef SKYEYE_WORKAROUNDS
 	/*
 	 * Allocate DMA tags and maps
 	 */
@@ -380,6 +376,7 @@ at91_usart_bus_attach(struct uart_softc *sc)
 		atsc->ping = &atsc->ping_pong[0];
 		atsc->pong = &atsc->ping_pong[1];
 	}
+#endif
 
 	/*
 	 * Prime the pump with the RX buffer.  We use two 64 byte bounce
@@ -413,9 +410,13 @@ at91_usart_bus_attach(struct uart_softc *sc)
 		WR4(&sc->sc_bas, USART_IER, USART_CSR_RXRDY);
 	}
 	WR4(&sc->sc_bas, USART_IER, USART_CSR_RXBRK);
+#ifndef SKYEYE_WORKAROUNDS
 errout:;
 	// XXX bad
 	return (err);
+#else
+	return (0);
+#endif
 }
 
 static int
@@ -501,7 +502,6 @@ at91_usart_bus_param(struct uart_softc *sc, int baudrate, int databits,
 	    parity));
 }
 
-
 static __inline void
 at91_rx_put(struct uart_softc *sc, int key)
 {
@@ -531,12 +531,12 @@ at91_usart_bus_ipend(struct uart_softc *sc)
 	uart_lock(sc->sc_hwmtx);
 	if (csr & USART_CSR_TXRDY) {
 		if (sc->sc_txbusy)
-			ipend |= UART_IPEND_TXIDLE;
+			ipend |= SER_INT_TXIDLE;
 		WR4(&sc->sc_bas, USART_IDR, USART_CSR_TXRDY);
 	}
 	if (csr & USART_CSR_ENDTX) {
 		if (sc->sc_txbusy)
-			ipend |= UART_IPEND_TXIDLE;
+			ipend |= SER_INT_TXIDLE;
 		WR4(&sc->sc_bas, USART_IDR, USART_CSR_ENDTX);
 	}
 
@@ -559,14 +559,14 @@ at91_usart_bus_ipend(struct uart_softc *sc)
 			at91_rx_put(sc, atsc->ping->buffer[i]);
 		for (i = 0; i < sc->sc_rxfifosz; i++)
 			at91_rx_put(sc, atsc->pong->buffer[i]);
-		at91_rx_put(sc, UART_STAT_OVERRUN);
+		uart_rx_put(sc, UART_STAT_OVERRUN);
 		csr &= ~(USART_CSR_ENDRX | USART_CSR_TIMEOUT);
 		WR4(&sc->sc_bas, PDC_RPR, atsc->ping->pa);
 		WR4(&sc->sc_bas, PDC_RCR, sc->sc_rxfifosz);
 		WR4(&sc->sc_bas, PDC_RNPR, atsc->pong->pa);
 		WR4(&sc->sc_bas, PDC_RNCR, sc->sc_rxfifosz);
 		WR4(&sc->sc_bas, PDC_PTCR, PDC_PTCR_RXTEN);
-		ipend |= UART_IPEND_RXREADY;
+		ipend |= SER_INT_RXREADY;
 	}
 	if ((atsc->flags & HAS_TIMEOUT) && (csr & USART_CSR_ENDRX)) {
 		// Shuffle data from 'ping' of ping pong buffer, but
@@ -582,7 +582,7 @@ at91_usart_bus_ipend(struct uart_softc *sc)
 		atsc->pong = p;
 		WR4(&sc->sc_bas, PDC_RNPR, atsc->pong->pa);
 		WR4(&sc->sc_bas, PDC_RNCR, sc->sc_rxfifosz);
-		ipend |= UART_IPEND_RXREADY;
+		ipend |= SER_INT_RXREADY;
 	}
 	if ((atsc->flags & HAS_TIMEOUT) && (csr & USART_CSR_TIMEOUT)) {
 		// We have one partial buffer.  We need to stop the
@@ -601,19 +601,19 @@ at91_usart_bus_ipend(struct uart_softc *sc)
 		WR4(&sc->sc_bas, PDC_RCR, sc->sc_rxfifosz);
 		WR4(&sc->sc_bas, USART_CR, USART_CR_STTTO);
 		WR4(&sc->sc_bas, PDC_PTCR, PDC_PTCR_RXTEN);
-		ipend |= UART_IPEND_RXREADY;
+		ipend |= SER_INT_RXREADY;
 	}
 	if (!(atsc->flags & HAS_TIMEOUT) && (csr & USART_CSR_RXRDY)) {
 		// We have another charater in a device that doesn't support
 		// timeouts, so we do it one character at a time.
 		at91_rx_put(sc, RD4(&sc->sc_bas, USART_RHR) & 0xff);
-		ipend |= UART_IPEND_RXREADY;
+		ipend |= SER_INT_RXREADY;
 	}
 
 	if (csr & USART_CSR_RXBRK) {
 		unsigned int cr = USART_CR_RSTSTA;
 
-		ipend |= UART_IPEND_BREAK;
+		ipend |= SER_INT_BREAK;
 		WR4(&sc->sc_bas, USART_CR, cr);
 	}
 	uart_unlock(sc->sc_hwmtx);
@@ -642,7 +642,7 @@ at91_usart_bus_getsig(struct uart_softc *sc)
 		sig |= SER_DSR;
 	if (csr & USART_CSR_RI)
 		sig |= SER_RI;
-	new = sig & ~UART_SIGMASK_DELTA;
+	new = sig & ~SER_MASK_DELTA;
 	sc->sc_hwsig = new;
 	uart_unlock(sc->sc_hwmtx);
 	return (sig);
@@ -662,10 +662,12 @@ at91_usart_bus_ioctl(struct uart_softc *sc, int request, intptr_t data)
 	}
 	return (EINVAL);
 }
+
 struct uart_class at91_usart_class = {
-	"at91_usart class",
+	"at91_usart",
 	at91_usart_methods,
 	sizeof(struct at91_usart_softc),
+	.uc_ops = &at91_usart_ops,
 	.uc_range = 8,
 	.uc_rclk = DEFAULT_RCLK
 };
