@@ -1,5 +1,5 @@
 /* $NetBSD: cpu.h,v 1.2 2001/02/23 21:23:52 reinoud Exp $ */
-/* $FreeBSD: src/sys/arm/include/cpu.h,v 1.4 2005/02/01 06:36:27 njl Exp $ */
+/* $FreeBSD: src/sys/arm/include/cpu.h,v 1.6 2006/05/11 17:29:24 phk Exp $ */
 
 #ifndef MACHINE_CPU_H
 #define MACHINE_CPU_H
@@ -21,18 +21,7 @@ get_cyclecount(void)
 }
 #endif
 
-#define CPU_CONSDEV 1
-#define CPU_ADJKERNTZ           2       /* int: timezone offset (seconds) */
-#define CPU_DISRTCSET           3       /* int: disable resettodr() call */
-#define CPU_BOOTINFO            4       /* struct: bootinfo */
-#define CPU_WALLCLOCK           5       /* int: indicates wall CMOS clock */
-#define CPU_MAXID               6       /* number of valid machdep ids */
-
-
-#define CLKF_USERMODE(frame)    ((frame->if_spsr & PSR_MODE) == PSR_USR32_MODE)
-
 #define TRAPF_USERMODE(frame)	((frame->tf_spsr & PSR_MODE) == PSR_USR32_MODE)
-#define CLKF_PC(frame)          (frame->if_pc)
 
 #define TRAPF_PC(tfp)		((tfp)->tf_pc)
 
