@@ -23,7 +23,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/if_npe.c,v 1.5.2.2 2007/11/17 18:22:57 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/if_npe.c,v 1.6 2007/05/24 16:31:22 sam Exp $");
 
 /*
  * Intel XScale NPE Ethernet driver.
@@ -88,6 +88,12 @@ __FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/if_npe.c,v 1.5.2.2 2007/11/17 18:2
 
 #include "miibus_if.h"
 
+/* 
+ * XXX: For the main bus dma tag. Can go away if the new method to get the 
+ * dma tag from the parent got MFC'd into RELENG_6.
+ */
+extern struct ixp425_softc *ixp425_softc;
+
 struct npebuf {
 	struct npebuf	*ix_next;	/* chain to next buffer */
 	void		*ix_m;		/* backpointer to mbuf */
@@ -120,6 +126,7 @@ struct npe_softc {
 	int		sc_debug;	/* DPRINTF* control */
 	int		sc_tickinterval;
 	struct callout	tick_ch;	/* Tick callout */
+	int		npe_watchdog_timer;
 	struct npedma	txdma;
 	struct npebuf	*tx_free;	/* list of free tx buffers */
 	struct npedma	rxdma;
@@ -223,7 +230,7 @@ static void	npeinit(void *);
 static void	npestart_locked(struct ifnet *);
 static void	npestart(struct ifnet *);
 static void	npestop(struct npe_softc *);
-static void	npewatchdog(struct ifnet *);
+static void	npewatchdog(struct npe_softc *);
 static int	npeioctl(struct ifnet * ifp, u_long, caddr_t);
 
 static int	npe_setrxqosentry(struct npe_softc *, int classix,
@@ -318,16 +325,13 @@ npe_attach(device_t dev)
 
 	ifp->if_softc = sc;
 	if_initname(ifp, device_get_name(dev), device_get_unit(dev));
-	ifp->if_mtu = ETHERMTU;
 	ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
 	ifp->if_start = npestart;
 	ifp->if_ioctl = npeioctl;
-	ifp->if_watchdog = npewatchdog;
 	ifp->if_init = npeinit;
 	IFQ_SET_MAXLEN(&ifp->if_snd, sc->txdma.nbuf - 1);
-	ifp->if_snd.ifq_maxlen = IFQ_MAXLEN;
+	ifp->if_snd.ifq_drv_maxlen = IFQ_MAXLEN;
 	IFQ_SET_READY(&ifp->if_snd);
-	ifp->if_timer = 0;
 	ifp->if_linkmib = &sc->mibdata;
 	ifp->if_linkmiblen = sizeof(sc->mibdata);
 	sc->mibdata.dot3Compliance = DOT3COMPLIANCE_STATS;
@@ -447,7 +451,7 @@ npe_dma_setup(struct npe_softc *sc, struct npedma *dma,
 	dma->nbuf = nbuf;
 
 	/* DMA tag for mapped mbufs  */
-	error = bus_dma_tag_create(NULL, 1, 0,
+	error = bus_dma_tag_create(ixp425_softc->sc_dmat, 1, 0,
 	    BUS_SPACE_MAXADDR_32BIT, BUS_SPACE_MAXADDR, NULL, NULL,
 	    MCLBYTES, maxseg, MCLBYTES, 0,
 	    busdma_lock_mutex, &sc->sc_mtx, &dma->mtag);
@@ -458,7 +462,7 @@ npe_dma_setup(struct npe_softc *sc, struct npedma *dma,
 	}
 
 	/* DMA tag and map for the NPE buffers */
-	error = bus_dma_tag_create(NULL, sizeof(uint32_t), 0, 
+	error = bus_dma_tag_create(ixp425_softc->sc_dmat, sizeof(uint32_t), 0, 
 	    BUS_SPACE_MAXADDR_32BIT, BUS_SPACE_MAXADDR, NULL, NULL,
 	    nbuf * sizeof(struct npehwbuf), 1,
 	    nbuf * sizeof(struct npehwbuf), 0,
@@ -598,7 +602,7 @@ npe_activate(device_t dev)
 		return error;
 
 	/* setup statistics block */
-	error = bus_dma_tag_create(NULL, sizeof(uint32_t), 0,
+	error = bus_dma_tag_create(ixp425_softc->sc_dmat, sizeof(uint32_t), 0,
 	    BUS_SPACE_MAXADDR_32BIT, BUS_SPACE_MAXADDR, NULL, NULL,
 	    sizeof(struct npestats), 1, sizeof(struct npestats), 0,
 	    busdma_lock_mutex, &sc->sc_mtx, &sc->sc_stats_tag);
@@ -806,6 +810,8 @@ npe_tick(void *xsc)
 	npe_updatestats(sc);
 	mii_tick(mii);
 
+	npewatchdog(sc);
+
 	/* schedule next poll */
 	callout_reset(&sc->tick_ch, sc->sc_tickinterval * hz, npe_tick, sc);
 #undef ACK
@@ -855,7 +861,7 @@ npe_txdone_finish(struct npe_softc *sc, const struct txdone *td)
 	 */
 	ifp->if_opackets += td->count;
 	ifp->if_drv_flags &= ~IFF_DRV_OACTIVE;
-	ifp->if_timer = 0;
+	sc->npe_watchdog_timer = 0;
 	npestart_locked(ifp);
 	NPE_UNLOCK(sc);
 }
@@ -1114,7 +1120,7 @@ if (ifp->if_drv_flags & IFF_DRV_RUNNING) return;/*XXX*/
 
 	ifp->if_drv_flags |= IFF_DRV_RUNNING;
 	ifp->if_drv_flags &= ~IFF_DRV_OACTIVE;
-	ifp->if_timer = 0;		/* just in case */
+	sc->npe_watchdog_timer = 0;		/* just in case */
 
 	/* enable transmitter and reciver in the MAC */
  	WR4(sc, NPE_MAC_RX_CNTRL1,
@@ -1297,7 +1303,7 @@ npestart_locked(struct ifnet *ifp)
 		/* XXX add vlan priority */
 		ixpqmgr_qwrite(sc->tx_qid, npe->ix_neaddr);
 
-		ifp->if_timer = 5;
+		sc->npe_watchdog_timer = 5;
 	}
 	if (sc->tx_free == NULL)
 		ifp->if_drv_flags |= IFF_DRV_OACTIVE;
@@ -1366,7 +1372,7 @@ npestop(struct npe_softc *sc)
  	WR4(sc, NPE_MAC_TX_CNTRL1,
 	    RD4(sc, NPE_MAC_TX_CNTRL1) &~ NPE_TX_CNTRL1_TX_EN);
 
-	ifp->if_timer = 0;
+	sc->npe_watchdog_timer = 0;
 	ifp->if_drv_flags &= ~(IFF_DRV_RUNNING | IFF_DRV_OACTIVE);
 
 	callout_stop(&sc->tick_ch);
@@ -1388,15 +1394,17 @@ npestop(struct npe_softc *sc)
 }
 
 void
-npewatchdog(struct ifnet *ifp)
+npewatchdog(struct npe_softc *sc)
 {
-	struct npe_softc *sc = ifp->if_softc;
+	NPE_ASSERT_LOCKED(sc);
+
+	if (sc->npe_watchdog_timer == 0 || --sc->npe_watchdog_timer != 0)
+		return;
+
+	device_printf(sc->sc_dev, "watchdog timeout\n");
+	sc->sc_ifp->if_oerrors++;
 
-	NPE_LOCK(sc);
-	if_printf(ifp, "device timeout\n");
-	ifp->if_oerrors++;
 	npeinit_locked(sc);
-	NPE_UNLOCK(sc);
 }
 
 static int
