@@ -57,7 +57,7 @@
  * SUCH DAMAGE.
 */
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/ixp425_npe.c,v 1.3.2.2 2007/11/17 18:22:57 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/ixp425_npe.c,v 1.8 2007/09/27 22:39:49 cognet Exp $");
 
 /*
  * Intel XScale Network Processing Engine (NPE) support.
@@ -291,7 +291,7 @@ ixpnpe_attach(device_t dev)
 	panic("%s: Unable to allocate irq %u", device_get_name(dev), irq);
     /* XXX could be a source of entropy */
     bus_setup_intr(dev, sc->sc_irq, INTR_TYPE_NET | INTR_MPSAFE,
-	ixpnpe_intr, sc, &sc->sc_ih);
+	NULL, ixpnpe_intr, sc, &sc->sc_ih);
     /* enable output fifo interrupts (NB: must also set OFIFO Write Enable) */ 
     npe_reg_write(sc, IX_NPECTL,
 	npe_reg_read(sc, IX_NPECTL) | (IX_NPECTL_OFE | IX_NPECTL_OFWE));
