@@ -57,7 +57,7 @@
  * SUCH DAMAGE.
 */
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/ixp425_qmgr.c,v 1.2.2.1 2007/02/26 23:13:09 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/arm/xscale/ixp425/ixp425_qmgr.c,v 1.2 2007/02/25 22:17:54 cognet Exp $");
 
 /*
  * Intel XScale Queue Manager support.
@@ -225,7 +225,7 @@ ixpqmgr_attach(device_t dev)
 		panic("Unable to allocate the qmgr irqs.\n");
 	/* XXX could be a source of entropy */
 	bus_setup_intr(dev, sc->sc_irq, INTR_TYPE_NET | INTR_MPSAFE,
-		ixpqmgr_intr, NULL, &sc->sc_ih);
+		NULL, ixpqmgr_intr, NULL, &sc->sc_ih);
 
 	/* NB: softc is pre-zero'd */
 	for (i = 0; i < IX_QMGR_MAX_NUM_QUEUES; i++) {
