@@ -14,7 +14,7 @@
  * warranties of merchantability and fitness for a particular
  * purpose.
  *
- * $FreeBSD: src/sys/boot/arm/at91/libat91/printf.c,v 1.1.2.1 2006/08/17 22:57:08 imp Exp $
+ * $FreeBSD: src/sys/boot/arm/at91/libat91/printf.c,v 1.2 2006/10/20 09:12:05 imp Exp $
  */
 
 #include <stdarg.h>
@@ -36,12 +36,13 @@ printf(const char *fmt,...)
 			c = *fmt++;
 			switch (c) {
 			case 'c':
-				putchar(va_arg(ap, int));
+				xputchar(va_arg(ap, int));
 				continue;
 			case 's':
 				for (s = va_arg(ap, char *); *s; s++)
-					putchar(*s);
+					xputchar(*s);
 				continue;
+			case 'd':	/* A lie, always prints unsigned */
 			case 'u':
 				u = va_arg(ap, unsigned);
 				s = buf;
@@ -50,7 +51,7 @@ printf(const char *fmt,...)
 				while (u /= 10U);
 			dumpbuf:;
 				while (--s >= buf)
-					putchar(*s);
+					xputchar(*s);
 				continue;
 			case 'x':
 				u = va_arg(ap, unsigned);
@@ -61,7 +62,7 @@ printf(const char *fmt,...)
 				goto dumpbuf;
 			}
 		}
-		putchar(c);
+		xputchar(c);
 	}
 	va_end(ap);
 
