@@ -1,10 +1,17 @@
-/* $FreeBSD: src/sys/boot/efi/include/efipart.h,v 1.2 2002/05/19 03:17:20 marcel Exp $ */
+/* $FreeBSD: src/sys/boot/efi/include/efipart.h,v 1.3 2006/11/02 02:42:47 marcel Exp $ */
 #ifndef _EFI_PART_H
 #define _EFI_PART_H
 
 /*++
 
-Copyright (c) 1998  Intel Corporation
+Copyright (c)  1999 - 2002 Intel Corporation. All rights reserved
+This software and associated documentation (if any) is furnished
+under a license and may only be used or copied in accordance
+with the terms of the license. Except as permitted by such
+license, no part of this software or documentation may be
+reproduced, stored in a retrieval system, or transmitted in any
+form or by any means without the express written consent of
+Intel Corporation.
 
 Module Name:
 
@@ -20,6 +27,10 @@ Revision History
 
 --*/
 
+//
+//
+//
+
 #define EFI_PARTITION   0xef
 #define MBR_SIZE        512
 
@@ -42,7 +53,7 @@ typedef struct {
 
 #define MBR_SIGNATURE           0xaa55
 #define MIN_MBR_DEVICE_SIZE     0x80000
-#define MBR_ERRATA_PAD          0x40000	/* 128 MB */
+#define MBR_ERRATA_PAD          0x40000 // 128 MB
 
 #define MAX_MBR_PARTITIONS  4   
 typedef struct {
