@@ -49,7 +49,7 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/boot/pc98/boot2/boot.c,v 1.14.2.1 2007/11/01 18:20:37 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/boot/pc98/boot2/boot.c,v 1.17.2.1 2007/11/01 18:19:19 jhb Exp $");
 
 #include "boot.h"
 #include <a.out.h>
@@ -70,7 +70,7 @@ static char kernel_config[KERNEL_CONFIG_SIZE];
 static char kernel_config_namebuf[NAMEBUF_LEN + sizeof "config"];
 static char linebuf[NAMEBUF_LEN];
 static char namebuf[NAMEBUF_LEN];
-static struct bootinfo bootinfo;
+struct bootinfo bootinfo;
 int loadflags;
 
 static void getbootdev(char *ptr, int *howto);
