@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/boot/sparc64/loader/metadata.c,v 1.13.2.2 2006/02/14 06:20:16 ru Exp $");
+__FBSDID("$FreeBSD: src/sys/boot/sparc64/loader/metadata.c,v 1.15 2005/09/22 15:06:58 ru Exp $");
 
 #include <stand.h>
 #include <sys/param.h>
