@@ -28,7 +28,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/cam/cam_periph.c,v 1.60.2.2 2006/10/09 21:44:23 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/cam/cam_periph.c,v 1.69 2007/05/14 21:48:52 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -51,6 +51,7 @@ __FBSDID("$FreeBSD: src/sys/cam/cam_periph.c,v 1.60.2.2 2006/10/09 21:44:23 mjac
 #include <cam/cam_xpt_periph.h>
 #include <cam/cam_periph.h>
 #include <cam/cam_debug.h>
+#include <cam/cam_sim.h>
 
 #include <cam/scsi/scsi_all.h>
 #include <cam/scsi/scsi_message.h>
@@ -101,7 +102,8 @@ periphdriver_register(void *data)
 	int ndrivers;
 
 	ndrivers = nperiph_drivers + 2;
-	newdrivers = malloc(sizeof(*newdrivers) * ndrivers, M_TEMP, M_WAITOK);
+	newdrivers = malloc(sizeof(*newdrivers) * ndrivers, M_CAMPERIPH,
+			    M_WAITOK);
 	if (periph_drivers)
 		bcopy(periph_drivers, newdrivers,
 		      sizeof(*newdrivers) * nperiph_drivers);
@@ -110,7 +112,7 @@ periphdriver_register(void *data)
 	old = periph_drivers;
 	periph_drivers = newdrivers;
 	if (old)
-		free(old, M_TEMP);
+		free(old, M_CAMPERIPH);
 	nperiph_drivers++;
 }
 
@@ -122,6 +124,7 @@ cam_periph_alloc(periph_ctor_t *periph_ctor,
 		 ac_callback_t *ac_callback, ac_code code, void *arg)
 {
 	struct		periph_driver **p_drv;
+	struct		cam_sim *sim;
 	struct		cam_periph *periph;
 	struct		cam_periph *cur_periph;
 	path_id_t	path_id;
@@ -129,7 +132,6 @@ cam_periph_alloc(periph_ctor_t *periph_ctor,
 	lun_id_t	lun_id;
 	cam_status	status;
 	u_int		init_level;
-	int s;
 
 	init_level = 0;
 	/*
@@ -163,11 +165,14 @@ cam_periph_alloc(periph_ctor_t *periph_ctor,
 	
 	init_level++;
 
+	xpt_lock_buses();
 	for (p_drv = periph_drivers; *p_drv != NULL; p_drv++) {
 		if (strcmp((*p_drv)->driver_name, name) == 0)
 			break;
 	}
-	
+	xpt_unlock_buses();
+
+	sim = xpt_path_sim(path);
 	path_id = xpt_path_path_id(path);
 	target_id = xpt_path_target_id(path);
 	lun_id = xpt_path_lun_id(path);
@@ -181,6 +186,7 @@ cam_periph_alloc(periph_ctor_t *periph_ctor,
 	periph->unit_number = camperiphunit(*p_drv, path_id, target_id, lun_id);
 	periph->immediate_priority = CAM_PRIORITY_NONE;
 	periph->refcount = 0;
+	periph->sim = sim;
 	SLIST_INIT(&periph->ccb_list);
 	status = xpt_create_path(&path, periph, path_id, target_id, lun_id);
 	if (status != CAM_REQ_CMP)
@@ -194,7 +200,6 @@ cam_periph_alloc(periph_ctor_t *periph_ctor,
 	if (status != CAM_REQ_CMP)
 		goto failure;
 
-	s = splsoftcam();
 	cur_periph = TAILQ_FIRST(&(*p_drv)->units);
 	while (cur_periph != NULL
 	    && cur_periph->unit_number < periph->unit_number)
@@ -207,8 +212,6 @@ cam_periph_alloc(periph_ctor_t *periph_ctor,
 		(*p_drv)->generation++;
 	}
 
-	splx(s);
-
 	init_level++;
 
 	status = periph_ctor(periph, arg);
@@ -222,9 +225,7 @@ failure:
 		/* Initialized successfully */
 		break;
 	case 3:
-		s = splsoftcam();
 		TAILQ_REMOVE(&(*p_drv)->units, periph, unit_links);
-		splx(s);
 		xpt_remove_periph(periph);
 		/* FALLTHROUGH */
 	case 2:
@@ -252,38 +253,38 @@ cam_periph_find(struct cam_path *path, char *name)
 {
 	struct periph_driver **p_drv;
 	struct cam_periph *periph;
-	int s;
 
+	xpt_lock_buses();
 	for (p_drv = periph_drivers; *p_drv != NULL; p_drv++) {
 
 		if (name != NULL && (strcmp((*p_drv)->driver_name, name) != 0))
 			continue;
 
-		s = splsoftcam();
 		TAILQ_FOREACH(periph, &(*p_drv)->units, unit_links) {
 			if (xpt_path_comp(periph->path, path) == 0) {
-				splx(s);
+				xpt_unlock_buses();
 				return(periph);
 			}
 		}
-		splx(s);
-		if (name != NULL)
+		if (name != NULL) {
+			xpt_unlock_buses();
 			return(NULL);
+		}
 	}
+	xpt_unlock_buses();
 	return(NULL);
 }
 
 cam_status
 cam_periph_acquire(struct cam_periph *periph)
 {
-	int s;
 
 	if (periph == NULL)
 		return(CAM_REQ_CMP_ERR);
 
-	s = splsoftcam();
+	xpt_lock_buses();
 	periph->refcount++;
-	splx(s);
+	xpt_unlock_buses();
 
 	return(CAM_REQ_CMP);
 }
@@ -291,20 +292,68 @@ cam_periph_acquire(struct cam_periph *periph)
 void
 cam_periph_release(struct cam_periph *periph)
 {
-	int s;
 
 	if (periph == NULL)
 		return;
 
-	s = splsoftcam();
+	xpt_lock_buses();
 	if ((--periph->refcount == 0)
 	 && (periph->flags & CAM_PERIPH_INVALID)) {
 		camperiphfree(periph);
 	}
-	splx(s);
+	xpt_unlock_buses();
 
 }
 
+int
+cam_periph_hold(struct cam_periph *periph, int priority)
+{
+	struct mtx *mtx;
+	int error;
+
+	mtx_assert(periph->sim->mtx, MA_OWNED);
+
+	/*
+	 * Increment the reference count on the peripheral
+	 * while we wait for our lock attempt to succeed
+	 * to ensure the peripheral doesn't disappear out
+	 * from user us while we sleep.
+	 */
+
+	if (cam_periph_acquire(periph) != CAM_REQ_CMP)
+		return (ENXIO);
+
+	mtx = periph->sim->mtx;
+	if (mtx == &Giant)
+		mtx = NULL;
+
+	while ((periph->flags & CAM_PERIPH_LOCKED) != 0) {
+		periph->flags |= CAM_PERIPH_LOCK_WANTED;
+		if ((error = msleep(periph, mtx, priority, "caplck", 0)) != 0) {
+			cam_periph_release(periph);
+			return (error);
+		}
+	}
+
+	periph->flags |= CAM_PERIPH_LOCKED;
+	return (0);
+}
+
+void
+cam_periph_unhold(struct cam_periph *periph)
+{
+
+	mtx_assert(periph->sim->mtx, MA_OWNED);
+
+	periph->flags &= ~CAM_PERIPH_LOCKED;
+	if ((periph->flags & CAM_PERIPH_LOCK_WANTED) != 0) {
+		periph->flags &= ~CAM_PERIPH_LOCK_WANTED;
+		wakeup(periph);
+	}
+
+	cam_periph_release(periph);
+}
+
 /*
  * Look for the next unit number that is not currently in use for this
  * peripheral type starting at "newunit".  Also exclude unit numbers that
@@ -319,11 +368,9 @@ camperiphnextunit(struct periph_driver *p_drv, u_int newunit, int wired,
 {
 	struct	cam_periph *periph;
 	char	*periph_name;
-	int	s;
 	int	i, val, dunit, r;
 	const char *dname, *strval;
 
-	s = splsoftcam();
 	periph_name = p_drv->driver_name;
 	for (;;newunit++) {
 
@@ -334,12 +381,12 @@ camperiphnextunit(struct periph_driver *p_drv, u_int newunit, int wired,
 
 		if (periph != NULL && periph->unit_number == newunit) {
 			if (wired != 0) {
-				xpt_print_path(periph->path);
-				printf("Duplicate Wired Device entry!\n");
-				xpt_print_path(periph->path);
-				printf("Second device (%s device at scbus%d "
-				       "target %d lun %d) will not be wired\n",
-				       periph_name, pathid, target, lun);
+				xpt_print(periph->path, "Duplicate Wired "
+				    "Device entry!\n");
+				xpt_print(periph->path, "Second device (%s "
+				    "device at scbus%d target %d lun %d) will "
+				    "not be wired\n", periph_name, pathid,
+				    target, lun);
 				wired = 0;
 			}
 			continue;
@@ -369,7 +416,6 @@ camperiphnextunit(struct periph_driver *p_drv, u_int newunit, int wired,
 		if (r != 0)
 			break;
 	}
-	splx(s);
 	return (newunit);
 }
 
@@ -424,13 +470,10 @@ camperiphunit(struct periph_driver *p_drv, path_id_t pathid,
 void
 cam_periph_invalidate(struct cam_periph *periph)
 {
-	int s;
 
-	s = splsoftcam();
 	/*
 	 * We only call this routine the first time a peripheral is
-	 * invalidated.  The oninvalidate() routine is always called at
-	 * splsoftcam().
+	 * invalidated.
 	 */
 	if (((periph->flags & CAM_PERIPH_INVALID) == 0)
 	 && (periph->periph_oninval != NULL))
@@ -439,17 +482,17 @@ cam_periph_invalidate(struct cam_periph *periph)
 	periph->flags |= CAM_PERIPH_INVALID;
 	periph->flags &= ~CAM_PERIPH_NEW_DEV_FOUND;
 
+	xpt_lock_buses();
 	if (periph->refcount == 0)
 		camperiphfree(periph);
 	else if (periph->refcount < 0)
 		printf("cam_invalidate_periph: refcount < 0!!\n");
-	splx(s);
+	xpt_unlock_buses();
 }
 
 static void
 camperiphfree(struct cam_periph *periph)
 {
-	int s;
 	struct periph_driver **p_drv;
 
 	for (p_drv = periph_drivers; *p_drv != NULL; p_drv++) {
@@ -460,15 +503,13 @@ camperiphfree(struct cam_periph *periph)
 		printf("camperiphfree: attempt to free non-existant periph\n");
 		return;
 	}
-	
-	if (periph->periph_dtor != NULL)
-		periph->periph_dtor(periph);
-	
-	s = splsoftcam();
+
 	TAILQ_REMOVE(&(*p_drv)->units, periph, unit_links);
 	(*p_drv)->generation++;
-	splx(s);
+	xpt_unlock_buses();
 
+	if (periph->periph_dtor != NULL)
+		periph->periph_dtor(periph);
 	xpt_remove_periph(periph);
 
 	if (periph->flags & CAM_PERIPH_NEW_DEV_FOUND) {
@@ -497,50 +538,7 @@ camperiphfree(struct cam_periph *periph)
 	}
 	xpt_free_path(periph->path);
 	free(periph, M_CAMPERIPH);
-}
-
-/*
- * Wait interruptibly for an exclusive lock.
- */
-int
-cam_periph_lock(struct cam_periph *periph, int priority)
-{
-	int error;
-
-	/*
-	 * Increment the reference count on the peripheral
-	 * while we wait for our lock attempt to succeed
-	 * to ensure the peripheral doesn't disappear out
-	 * from under us while we sleep.
-	 */
-	if (cam_periph_acquire(periph) != CAM_REQ_CMP)
-		return(ENXIO);
-
-	while ((periph->flags & CAM_PERIPH_LOCKED) != 0) {
-		periph->flags |= CAM_PERIPH_LOCK_WANTED;
-		if ((error = tsleep(periph, priority, "caplck", 0)) != 0) {
-			cam_periph_release(periph);
-			return error;
-		}
-	}
-
-	periph->flags |= CAM_PERIPH_LOCKED;
-	return 0;
-}
-
-/*
- * Unlock and wake up any waiters.
- */
-void
-cam_periph_unlock(struct cam_periph *periph)
-{
-	periph->flags &= ~CAM_PERIPH_LOCKED;
-	if ((periph->flags & CAM_PERIPH_LOCK_WANTED) != 0) {
-		periph->flags &= ~CAM_PERIPH_LOCK_WANTED;
-		wakeup(periph);
-	}
-
-	cam_periph_release(periph);
+	xpt_lock_buses();
 }
 
 /*
@@ -681,6 +679,16 @@ cam_periph_mapmem(union ccb *ccb, struct cam_periph_map_info *mapinfo)
 		mapinfo->num_bufs_used++;
 	}
 
+	/*
+	 * Now that we've gotten this far, change ownership to the kernel
+	 * of the buffers so that we don't run afoul of returning to user
+	 * space with locks (on the buffer) held.
+	 */
+	for (i = 0; i < numbufs; i++) {
+		BUF_KERNPROC(mapinfo->bp[i]);
+	}
+
+
 	return(0);
 }
 
@@ -742,12 +750,11 @@ union ccb *
 cam_periph_getccb(struct cam_periph *periph, u_int32_t priority)
 {
 	struct ccb_hdr *ccb_h;
-	int s;
+	struct mtx *mtx;
 
+	mtx_assert(periph->sim->mtx, MA_OWNED);
 	CAM_DEBUG(periph->path, CAM_DEBUG_TRACE, ("entering cdgetccb\n"));
 
-	s = splsoftcam();
-	
 	while (SLIST_FIRST(&periph->ccb_list) == NULL) {
 		if (periph->immediate_priority > priority)
 			periph->immediate_priority = priority;
@@ -755,26 +762,33 @@ cam_periph_getccb(struct cam_periph *periph, u_int32_t priority)
 		if ((SLIST_FIRST(&periph->ccb_list) != NULL)
 		 && (SLIST_FIRST(&periph->ccb_list)->pinfo.priority == priority))
 			break;
-		tsleep(&periph->ccb_list, PRIBIO, "cgticb", 0);
+		mtx_assert(periph->sim->mtx, MA_OWNED);
+		if (periph->sim->mtx == &Giant)
+			mtx = NULL;
+		else
+			mtx = periph->sim->mtx;
+		msleep(&periph->ccb_list, mtx, PRIBIO, "cgticb", 0);
 	}
 
 	ccb_h = SLIST_FIRST(&periph->ccb_list);
 	SLIST_REMOVE_HEAD(&periph->ccb_list, periph_links.sle);
-	splx(s);
 	return ((union ccb *)ccb_h);
 }
 
 void
 cam_periph_ccbwait(union ccb *ccb)
 {
-	int s;
+	struct mtx *mtx;
+	struct cam_sim *sim;
 
-	s = splsoftcam();
+	sim = xpt_path_sim(ccb->ccb_h.path);
+	if (sim->mtx == &Giant)
+		mtx = NULL;
+	else
+		mtx = sim->mtx;
 	if ((ccb->ccb_h.pinfo.index != CAM_UNQUEUED_INDEX)
 	 || ((ccb->ccb_h.status & CAM_STATUS_MASK) == CAM_REQ_INPROG))
-		tsleep(&ccb->ccb_h.cbfcnp, PRIBIO, "cbwait", 0);
-
-	splx(s);
+		msleep(&ccb->ccb_h.cbfcnp, mtx, PRIBIO, "cbwait", 0);
 }
 
 int
@@ -847,10 +861,13 @@ cam_periph_runccb(union ccb *ccb,
 		  cam_flags camflags, u_int32_t sense_flags,
 		  struct devstat *ds)
 {
+	struct cam_sim *sim;
 	int error;
  
 	error = 0;
-        
+	sim = xpt_path_sim(ccb->ccb_h.path);
+	mtx_assert(sim->mtx, MA_OWNED);
+
 	/*
 	 * If the user has supplied a stats structure, and if we understand
 	 * this particular type of ccb, record the transaction start.
@@ -985,8 +1002,8 @@ camperiphdone(struct cam_periph *periph, union ccb *done_ccb)
 					saved_ccb->ccb_h.status |=
 					    CAM_AUTOSNS_VALID;
 #if 0
-					xpt_print_path(saved_ccb->ccb_h.path);
-					printf("Recovered Sense\n");
+					xpt_print(saved_ccb->ccb_h.path,
+					    "Recovered Sense\n");
 					scsi_sense_print(&saved_ccb->csio);
 					cam_error_print(saved_ccb, CAM_ESF_ALL,
 							CAM_EPF_ALL);
@@ -1176,11 +1193,8 @@ cam_periph_freeze_after_event(struct cam_periph *periph,
 {
 	struct timeval delta;
 	struct timeval duration_tv;
-	int s;
 
-	s = splclock();
 	microtime(&delta);
-	splx(s);
 	timevalsub(&delta, event_time);
 	duration_tv.tv_sec = duration_ms / 1000;
 	duration_tv.tv_usec = (duration_ms % 1000) * 1000;
@@ -1277,8 +1291,7 @@ camperiphscsistatuserror(union ccb *ccb, cam_flags camflags,
 			*timeout = 0;
 			error = ERESTART;
 			if (bootverbose) {
-				xpt_print_path(ccb->ccb_h.path);
-				printf("Queue Full\n");
+				xpt_print(ccb->ccb_h.path, "Queue Full\n");
 			}
 			break;
 		}
@@ -1290,8 +1303,7 @@ camperiphscsistatuserror(union ccb *ccb, cam_flags camflags,
 		 * command completes or a 1 second timeout.
 		 */
 		if (bootverbose) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Device Busy\n");
+			xpt_print(ccb->ccb_h.path, "Device Busy\n");
 		}
 	 	if (ccb->ccb_h.retry_count > 0) {
 	 		ccb->ccb_h.retry_count--;
@@ -1304,13 +1316,12 @@ camperiphscsistatuserror(union ccb *ccb, cam_flags camflags,
 		}
 		break;
 	case SCSI_STATUS_RESERV_CONFLICT:
-		xpt_print_path(ccb->ccb_h.path);
-		printf("Reservation Conflict\n");
+		xpt_print(ccb->ccb_h.path, "Reservation Conflict\n");
 		error = EIO;
 		break;
 	default:
-		xpt_print_path(ccb->ccb_h.path);
-		printf("SCSI Status 0x%x\n", ccb->csio.scsi_status);
+		xpt_print(ccb->ccb_h.path, "SCSI Status 0x%x\n",
+		    ccb->csio.scsi_status);
 		error = EIO;
 		break;
 	}
@@ -1564,42 +1575,38 @@ cam_periph_error(union ccb *ccb, cam_flags camflags,
 						 &timeout);
 		break;
 	case CAM_AUTOSENSE_FAIL:
-		xpt_print_path(ccb->ccb_h.path);
-		printf("AutoSense Failed\n");
+		xpt_print(ccb->ccb_h.path, "AutoSense Failed\n");
 		error = EIO;	/* we have to kill the command */
 		break;
 	case CAM_REQ_CMP_ERR:
 		if (bootverbose && printed == 0) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Request completed with CAM_REQ_CMP_ERR\n");
+			xpt_print(ccb->ccb_h.path,
+			    "Request completed with CAM_REQ_CMP_ERR\n");
 			printed++;
 		}
 		/* FALLTHROUGH */
 	case CAM_CMD_TIMEOUT:
 		if (bootverbose && printed == 0) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Command timed out\n");
+			xpt_print(ccb->ccb_h.path, "Command timed out\n");
 			printed++;
 		}
 		/* FALLTHROUGH */
 	case CAM_UNEXP_BUSFREE:
 		if (bootverbose && printed == 0) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Unexpected Bus Free\n");
+			xpt_print(ccb->ccb_h.path, "Unexpected Bus Free\n");
 			printed++;
 		}
 		/* FALLTHROUGH */
 	case CAM_UNCOR_PARITY:
 		if (bootverbose && printed == 0) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Uncorrected Parity Error\n");
+			xpt_print(ccb->ccb_h.path,
+			    "Uncorrected Parity Error\n");
 			printed++;
 		}
 		/* FALLTHROUGH */
 	case CAM_DATA_RUN_ERR:
 		if (bootverbose && printed == 0) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Data Overrun\n");
+			xpt_print(ccb->ccb_h.path, "Data Overrun\n");
 			printed++;
 		}
 		error = EIO;	/* we have to kill the command */
@@ -1628,8 +1635,8 @@ cam_periph_error(union ccb *ccb, cam_flags camflags,
 				ccb->ccb_h.retry_count--;
 				error = ERESTART;
 				if (bootverbose && printed == 0) {
-					xpt_print_path(ccb->ccb_h.path);
-					printf("Selection Timeout\n");
+					xpt_print(ccb->ccb_h.path,
+					    "Selection Timeout\n");
 					printed++;
 				}
 
@@ -1691,8 +1698,7 @@ cam_periph_error(union ccb *ccb, cam_flags camflags,
 		/* Unconditional requeue */
 		error = ERESTART;
 		if (bootverbose && printed == 0) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("Request Requeued\n");
+			xpt_print(ccb->ccb_h.path, "Request Requeued\n");
 			printed++;
 		}
 		break;
@@ -1713,8 +1719,8 @@ cam_periph_error(union ccb *ccb, cam_flags camflags,
 			ccb->ccb_h.retry_count--;
 			error = ERESTART;
 			if (bootverbose && printed == 0) {
-				xpt_print_path(ccb->ccb_h.path);
-				printf("CAM Status 0x%x\n", status);
+				xpt_print(ccb->ccb_h.path, "CAM Status 0x%x\n",
+				    status);
 				printed++;
 			}
 		} else {
@@ -1753,11 +1759,9 @@ cam_periph_error(union ccb *ccb, cam_flags camflags,
 		if (action_string == NULL)
 			action_string = "Unretryable Error";
 		if (error != ERESTART) {
-			xpt_print_path(ccb->ccb_h.path);
-			printf("error %d\n", error);
+			xpt_print(ccb->ccb_h.path, "error %d\n", error);
 		}
-		xpt_print_path(ccb->ccb_h.path);
-		printf("%s\n", action_string);
+		xpt_print(ccb->ccb_h.path, "%s\n", action_string);
 	}
 
 	return (error);
