@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_da.c,v 1.180.2.13 2007/05/25 20:14:02 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_da.c,v 1.224 2007/07/01 17:42:07 imp Exp $");
 
 #include <sys/param.h>
 
@@ -37,20 +37,15 @@ __FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_da.c,v 1.180.2.13 2007/05/25 20:14:02
 #include <sys/bio.h>
 #include <sys/sysctl.h>
 #include <sys/taskqueue.h>
-#endif /* _KERNEL */
-
-#include <sys/devicestat.h>
+#include <sys/lock.h>
+#include <sys/mutex.h>
 #include <sys/conf.h>
+#include <sys/devicestat.h>
 #include <sys/eventhandler.h>
 #include <sys/malloc.h>
 #include <sys/cons.h>
-
-#include <machine/md_var.h>
-
-#include <vm/vm.h>
-#include <vm/pmap.h>
-
 #include <geom/geom_disk.h>
+#endif /* _KERNEL */
 
 #ifndef _KERNEL
 #include <stdio.h>
@@ -61,6 +56,7 @@ __FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_da.c,v 1.180.2.13 2007/05/25 20:14:02
 #include <cam/cam_ccb.h>
 #include <cam/cam_periph.h>
 #include <cam/cam_xpt_periph.h>
+#include <cam/cam_sim.h>
 
 #include <cam/scsi/scsi_message.h>
 
@@ -133,6 +129,7 @@ struct da_softc {
 	struct task		sysctl_task;
 	struct sysctl_ctx_list	sysctl_ctx;
 	struct sysctl_oid	*sysctl_tree;
+	struct callout		sendordered_c;
 };
 
 struct da_quirk_entry {
@@ -169,7 +166,6 @@ static struct da_quirk_entry da_quirk_table[] =
 		 */
 		{T_DIRECT, SIP_MEDIA_FIXED, "FUJITSU", "M2954*", "*"},
 		/*quirks*/ DA_Q_NO_SYNC_CACHE
-	
 	},
 	{
 		/*
@@ -241,7 +237,7 @@ static struct da_quirk_entry da_quirk_table[] =
 		 * Reported by: walter@pelissero.de
 		 */
 		{T_DIRECT, SIP_MEDIA_FIXED, "CONNER", "CP3500*", "*"},
-                /*quirks*/ DA_Q_NO_SYNC_CACHE
+		/*quirks*/ DA_Q_NO_SYNC_CACHE
 	},
 	{
 		/*
@@ -266,7 +262,7 @@ static struct da_quirk_entry da_quirk_table[] =
 		 */
 		{T_DIRECT, SIP_MEDIA_REMOVABLE, "Generic*", "USB Flash Disk*",
 		"*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
-	}, 
+	},
  	{
  		/*
  		 * Creative Nomad MUVO mp3 player (USB)
@@ -351,8 +347,8 @@ static struct da_quirk_entry da_quirk_table[] =
 		 * Kingston DataTraveler II+ USB Pen-Drive.
 		 * Reported by: Pawel Jakub Dawidek <pjd@FreeBSD.org>
 		 */
-		{T_DIRECT, SIP_MEDIA_REMOVABLE, "Kingston" , "DataTraveler II+", "*"},
-		/*quirks*/ DA_Q_NO_SYNC_CACHE
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "Kingston" , "DataTraveler II+",
+		"*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
 	},
 	{
 		/*
@@ -458,13 +454,21 @@ static struct da_quirk_entry da_quirk_table[] =
 		 */
 		{T_DIRECT, SIP_MEDIA_REMOVABLE, "iriver", "H10*",
 		"*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
-       },
+	},
+	{
+		/*
+		 * iRiver U10 MP3 player
+		 * PR: usb/92306
+		 */
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "iriver", "U10*",
+		"*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
 	{
 		/*
 		 * X-Micro Flash Disk
 		 * PR: usb/96901
 		 */
-		{T_DIRECT, SIP_MEDIA_REMOVABLE, "X-Micro" , "Flash Disk",
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "X-Micro", "Flash Disk",
 		"*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
 	},
 	{
@@ -475,6 +479,62 @@ static struct da_quirk_entry da_quirk_table[] =
 		{T_DIRECT, SIP_MEDIA_REMOVABLE, "EM732X", "MP3 Player*",
 		"1.0"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
 	},
+	{
+		/*
+		 * Denver MP3 player
+		 * PR: usb/107101
+		 */
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "DENVER", "MP3 PLAYER",
+		 "*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
+	{
+		/*
+		 * Philips USB Key Audio KEY013
+		 * PR: usb/68412
+		 */
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "PHILIPS", "Key*", "*"},
+		/*quirks*/ DA_Q_NO_SYNC_CACHE | DA_Q_NO_PREVENT
+	},
+	{
+		/*
+		 * JNC MP3 Player
+		 * PR: usb/94439
+		 */
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "JNC*" , "MP3 Player*",
+		 "*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
+	{
+		/*
+		 * SAMSUNG MP0402H
+		 * PR: usb/108427
+		 */
+		{T_DIRECT, SIP_MEDIA_FIXED, "SAMSUNG", "MP0402H", "*"},
+		/*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
+	{
+		/*
+		 * I/O Magic USB flash - Giga Bank
+		 * PR: usb/108810
+		 */
+		{T_DIRECT, SIP_MEDIA_FIXED, "GS-Magic", "stor*", "*"},
+		/*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
+	{
+		/*
+		 * JoyFly 128mb USB Flash Drive
+		 * PR: 96133
+		 */
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "USB 2.0", "Flash Disk*",
+		 "*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
+	{
+		/*
+		 * ChipsBnk usb stick
+		 * PR: 103702
+		 */
+		{T_DIRECT, SIP_MEDIA_REMOVABLE, "ChipsBnk", "USB*",
+		 "*"}, /*quirks*/ DA_Q_NO_SYNC_CACHE
+	},
 };
 
 static	disk_strategy_t	dastrategy;
@@ -552,7 +612,7 @@ static struct periph_driver dadriver =
 
 PERIPHDRIVER_DECLARE(da, dadriver);
 
-static SLIST_HEAD(,da_softc) softc_list;
+MALLOC_DEFINE(M_SCSIDA, "scsi_da", "scsi_da buffers");
 
 static int
 daopen(struct disk *dp)
@@ -561,34 +621,35 @@ daopen(struct disk *dp)
 	struct da_softc *softc;
 	int unit;
 	int error;
-	int s;
 
-	s = splsoftcam();
 	periph = (struct cam_periph *)dp->d_drv1;
 	if (periph == NULL) {
-		splx(s);
 		return (ENXIO);	
 	}
-	unit = periph->unit_number;
 
+	if (cam_periph_acquire(periph) != CAM_REQ_CMP) {
+		return(ENXIO);
+	}
+
+	cam_periph_lock(periph);
+	if ((error = cam_periph_hold(periph, PRIBIO|PCATCH)) != 0) {
+		cam_periph_unlock(periph);
+		cam_periph_release(periph);
+		return (error);
+	}
+
+	unit = periph->unit_number;
 	softc = (struct da_softc *)periph->softc;
+	softc->flags |= DA_FLAG_OPEN;
 
 	CAM_DEBUG(periph->path, CAM_DEBUG_TRACE,
 	    ("daopen: disk=%s%d (unit %d)\n", dp->d_name, dp->d_unit,
 	     unit));
 
-	if ((error = cam_periph_lock(periph, PRIBIO|PCATCH)) != 0)
-		return (error); /* error code from tsleep */
-
-	if (cam_periph_acquire(periph) != CAM_REQ_CMP)
-		return(ENXIO);
-	softc->flags |= DA_FLAG_OPEN;
-
 	if ((softc->flags & DA_FLAG_PACK_INVALID) != 0) {
 		/* Invalidate our pack information. */
 		softc->flags &= ~DA_FLAG_PACK_INVALID;
 	}
-	splx(s);
 
 	error = dagetcapacity(periph);
 
@@ -611,6 +672,7 @@ daopen(struct disk *dp)
 		softc->flags &= ~DA_FLAG_OPEN;
 		cam_periph_release(periph);
 	}
+	cam_periph_unhold(periph);
 	cam_periph_unlock(periph);
 	return (error);
 }
@@ -620,18 +682,21 @@ daclose(struct disk *dp)
 {
 	struct	cam_periph *periph;
 	struct	da_softc *softc;
-	int	error;
+	int error;
 
 	periph = (struct cam_periph *)dp->d_drv1;
 	if (periph == NULL)
 		return (ENXIO);	
 
-	softc = (struct da_softc *)periph->softc;
-
-	if ((error = cam_periph_lock(periph, PRIBIO)) != 0) {
-		return (error); /* error code from tsleep */
+	cam_periph_lock(periph);
+	if ((error = cam_periph_hold(periph, PRIBIO)) != 0) {
+		cam_periph_unlock(periph);
+		cam_periph_release(periph);
+		return (error);
 	}
 
+	softc = (struct da_softc *)periph->softc;
+
 	if ((softc->quirks & DA_Q_NO_SYNC_CACHE) == 0) {
 		union	ccb *ccb;
 
@@ -663,11 +728,10 @@ daclose(struct disk *dp)
 				if (sense_key != SSD_KEY_ILLEGAL_REQUEST)
 					scsi_sense_print(&ccb->csio);
 			} else {
-				xpt_print_path(periph->path);
-				printf("Synchronize cache failed, status "
-				       "== 0x%x, scsi status == 0x%x\n",
-				       ccb->csio.ccb_h.status,
-				       ccb->csio.scsi_status);
+				xpt_print(periph->path, "Synchronize cache "
+				    "failed, status == 0x%x, scsi status == "
+				    "0x%x\n", ccb->csio.ccb_h.status,
+				    ccb->csio.scsi_status);
 			}
 		}
 
@@ -694,8 +758,9 @@ daclose(struct disk *dp)
 	}
 
 	softc->flags &= ~DA_FLAG_OPEN;
-	cam_periph_unlock(periph);
+	cam_periph_unhold(periph);
 	cam_periph_release(periph);
+	cam_periph_unlock(periph);
 	return (0);	
 }
 
@@ -709,7 +774,6 @@ dastrategy(struct bio *bp)
 {
 	struct cam_periph *periph;
 	struct da_softc *softc;
-	int    s;
 	
 	periph = (struct cam_periph *)bp->bio_disk->d_drv1;
 	if (periph == NULL) {
@@ -717,6 +781,9 @@ dastrategy(struct bio *bp)
 		return;
 	}
 	softc = (struct da_softc *)periph->softc;
+
+	cam_periph_lock(periph);
+
 #if 0
 	/*
 	 * check it's not too big a transfer for our adapter
@@ -729,13 +796,12 @@ dastrategy(struct bio *bp)
 	 * after we are in the queue.  Otherwise, we might not properly
 	 * clean up one of the buffers.
 	 */
-	s = splbio();
 	
 	/*
 	 * If the device has been made invalid, error out
 	 */
 	if ((softc->flags & DA_FLAG_PACK_INVALID)) {
-		splx(s);
+		cam_periph_unlock(periph);
 		biofinish(bp, NULL, ENXIO);
 		return;
 	}
@@ -745,12 +811,11 @@ dastrategy(struct bio *bp)
 	 */
 	bioq_disksort(&softc->bio_queue, bp);
 
-	splx(s);
-	
 	/*
 	 * Schedule ourselves for performing the work.
 	 */
 	xpt_schedule(periph, /* XXX priority */1);
+	cam_periph_unlock(periph);
 
 	return;
 }
@@ -769,12 +834,16 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 	if (periph == NULL)
 		return (ENXIO);
 	softc = (struct da_softc *)periph->softc;
+	cam_periph_lock(periph);
 	secsize = softc->params.secsize;
 	
-	if ((softc->flags & DA_FLAG_PACK_INVALID) != 0)
+	if ((softc->flags & DA_FLAG_PACK_INVALID) != 0) {
+		cam_periph_unlock(periph);
 		return (ENXIO);
+	}
 
 	if (length > 0) {
+		periph->flags |= CAM_PERIPH_POLLED;
 		xpt_setup_ccb(&csio.ccb_h, periph->path, /*priority*/1);
 		csio.ccb_h.ccb_state = DA_CCB_DUMP;
 		scsi_read_write(&csio,
@@ -800,10 +869,12 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 			else
 				printf("status == 0x%x, scsi status == 0x%x\n",
 				       csio.ccb_h.status, csio.scsi_status);
+			periph->flags |= CAM_PERIPH_POLLED;
 			return(EIO);
 		}
+		cam_periph_unlock(periph);
 		return(0);
-	} 
+	}
 		
 	/*
 	 * Sync the disk cache contents to the physical media.
@@ -835,13 +906,15 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 				if (sense_key != SSD_KEY_ILLEGAL_REQUEST)
 					scsi_sense_print(&csio);
 			} else {
-				xpt_print_path(periph->path);
-				printf("Synchronize cache failed, status "
-				       "== 0x%x, scsi status == 0x%x\n",
-				       csio.ccb_h.status, csio.scsi_status);
+				xpt_print(periph->path, "Synchronize cache "
+				    "failed, status == 0x%x, scsi status == "
+				    "0x%x\n", csio.ccb_h.status,
+				    csio.scsi_status);
 			}
 		}
 	}
+	periph->flags &= ~CAM_PERIPH_POLLED;
+	cam_periph_unlock(periph);
 	return (0);
 }
 
@@ -849,42 +922,18 @@ static void
 dainit(void)
 {
 	cam_status status;
-	struct cam_path *path;
 
-	SLIST_INIT(&softc_list);
-	
 	/*
 	 * Install a global async callback.  This callback will
 	 * receive async callbacks like "new device found".
 	 */
-	status = xpt_create_path(&path, /*periph*/NULL, CAM_XPT_PATH_ID,
-				 CAM_TARGET_WILDCARD, CAM_LUN_WILDCARD);
-
-	if (status == CAM_REQ_CMP) {
-		struct ccb_setasync csa;
-
-                xpt_setup_ccb(&csa.ccb_h, path, /*priority*/5);
-                csa.ccb_h.func_code = XPT_SASYNC_CB;
-                csa.event_enable = AC_FOUND_DEVICE;
-                csa.callback = daasync;
-                csa.callback_arg = NULL;
-                xpt_action((union ccb *)&csa);
-		status = csa.ccb_h.status;
-                xpt_free_path(path);
-        }
+	status = xpt_register_async(AC_FOUND_DEVICE, daasync, NULL, NULL);
 
 	if (status != CAM_REQ_CMP) {
 		printf("da: Failed to attach master async callback "
 		       "due to status 0x%x!\n", status);
 	} else if (da_send_ordered) {
 
-		/*
-		 * Schedule a periodic event to occasionally send an
-		 * ordered tag to a device.
-		 */
-		timeout(dasendorderedtag, NULL,
-			(DA_DEFAULT_TIMEOUT * hz) / DA_ORDEREDTAG_INTERVAL);
-
 		/* Register our shutdown event handler */
 		if ((EVENTHANDLER_REGISTER(shutdown_post_sync, dashutdown, 
 					   NULL, SHUTDOWN_PRI_DEFAULT)) == NULL)
@@ -895,45 +944,26 @@ dainit(void)
 static void
 daoninvalidate(struct cam_periph *periph)
 {
-	int s;
 	struct da_softc *softc;
-	struct ccb_setasync csa;
 
 	softc = (struct da_softc *)periph->softc;
 
 	/*
 	 * De-register any async callbacks.
 	 */
-	xpt_setup_ccb(&csa.ccb_h, periph->path,
-		      /* priority */ 5);
-	csa.ccb_h.func_code = XPT_SASYNC_CB;
-	csa.event_enable = 0;
-	csa.callback = daasync;
-	csa.callback_arg = periph;
-	xpt_action((union ccb *)&csa);
+	xpt_register_async(0, daasync, periph, periph->path);
 
 	softc->flags |= DA_FLAG_PACK_INVALID;
 
-	/*
-	 * Although the oninvalidate() routines are always called at
-	 * splsoftcam, we need to be at splbio() here to keep the buffer
-	 * queue from being modified while we traverse it.
-	 */
-	s = splbio();
-
 	/*
 	 * Return all queued I/O with ENXIO.
 	 * XXX Handle any transactions queued to the card
 	 *     with XPT_ABORT_CCB.
 	 */
 	bioq_flush(&softc->bio_queue, NULL, ENXIO);
-	splx(s);
-
-	SLIST_REMOVE(&softc_list, softc, da_softc, links);
 
 	disk_gone(softc->disk);
-	xpt_print_path(periph->path);
-	printf("lost device\n");
+	xpt_print(periph->path, "lost device\n");
 }
 
 static void
@@ -943,17 +973,19 @@ dacleanup(struct cam_periph *periph)
 
 	softc = (struct da_softc *)periph->softc;
 
-	xpt_print_path(periph->path);
-	printf("removing device entry\n");
+	xpt_print(periph->path, "removing device entry\n");
 	/*
 	 * If we can't free the sysctl tree, oh well...
 	 */
 	if ((softc->flags & DA_FLAG_SCTX_INIT) != 0
 	    && sysctl_ctx_free(&softc->sysctl_ctx) != 0) {
-		xpt_print_path(periph->path);
-		printf("can't remove sysctl context\n");
+		xpt_print(periph->path, "can't remove sysctl context\n");
 	}
+
+	cam_periph_unlock(periph);
 	disk_destroy(softc->disk);
+	callout_drain(&softc->sendordered_c);
+	cam_periph_lock(periph);
 	free(softc, M_DEVBUF);
 }
 
@@ -968,6 +1000,7 @@ daasync(void *callback_arg, u_int32_t code,
 	case AC_FOUND_DEVICE:
 	{
 		struct ccb_getdev *cgd;
+		struct cam_sim *sim;
 		cam_status status;
  
 		cgd = (struct ccb_getdev *)arg;
@@ -984,6 +1017,7 @@ daasync(void *callback_arg, u_int32_t code,
 		 * this device and start the probe
 		 * process.
 		 */
+		sim = xpt_path_sim(cgd->ccb_h.path);
 		status = cam_periph_alloc(daregister, daoninvalidate,
 					  dacleanup, dastart,
 					  "da", CAM_PERIPH_BIO,
@@ -1001,10 +1035,8 @@ daasync(void *callback_arg, u_int32_t code,
 	{
 		struct da_softc *softc;
 		struct ccb_hdr *ccbh;
-		int s;
 
 		softc = (struct da_softc *)periph->softc;
-		s = splsoftcam();
 		/*
 		 * Don't fail on the expected unit attention
 		 * that will occur.
@@ -1012,7 +1044,6 @@ daasync(void *callback_arg, u_int32_t code,
 		softc->flags |= DA_FLAG_RETRY_UA;
 		LIST_FOREACH(ccbh, &softc->pending_ccbs, periph_links.le)
 			ccbh->ccb_state |= DA_CCB_RETRY_UA;
-		splx(s);
 		/* FALLTHROUGH*/
 	}
 	default:
@@ -1029,8 +1060,10 @@ dasysctlinit(void *context, int pending)
 	char tmpstr[80], tmpstr2[80];
 
 	periph = (struct cam_periph *)context;
-	softc = (struct da_softc *)periph->softc;
+	if (cam_periph_acquire(periph) != CAM_REQ_CMP)
+		return;
 
+	softc = (struct da_softc *)periph->softc;
 	snprintf(tmpstr, sizeof(tmpstr), "CAM DA unit %d", periph->unit_number);
 	snprintf(tmpstr2, sizeof(tmpstr2), "%d", periph->unit_number);
 
@@ -1043,6 +1076,7 @@ dasysctlinit(void *context, int pending)
 	if (softc->sysctl_tree == NULL) {
 		printf("dasysctlinit: unable to allocate sysctl tree\n");
 		mtx_unlock(&Giant);
+		cam_periph_release(periph);
 		return;
 	}
 
@@ -1056,6 +1090,7 @@ dasysctlinit(void *context, int pending)
 		"Minimum CDB size");
 
 	mtx_unlock(&Giant);
+	cam_periph_release(periph);
 }
 
 static int
@@ -1093,9 +1128,7 @@ dacmdsizesysctl(SYSCTL_HANDLER_ARGS)
 static cam_status
 daregister(struct cam_periph *periph, void *arg)
 {
-	int s;
 	struct da_softc *softc;
-	struct ccb_setasync csa;
 	struct ccb_pathinq cpi;
 	struct ccb_getdev *cgd;
 	char tmpstr[80];
@@ -1182,18 +1215,11 @@ daregister(struct cam_periph *periph, void *arg)
 	else if (softc->minimum_cmd_size > 12)
 		softc->minimum_cmd_size = 16;
 
-	/*
-	 * Block our timeout handler while we
-	 * add this softc to the dev list.
-	 */
-	s = splsoftclock();
-	SLIST_INSERT_HEAD(&softc_list, softc, links);
-	splx(s);
-
 	/*
 	 * Register this media as a disk
 	 */
 
+	mtx_unlock(periph->sim->mtx);
 	softc->disk = disk_alloc();
 	softc->disk->d_open = daopen;
 	softc->disk->d_close = daclose;
@@ -1203,8 +1229,11 @@ daregister(struct cam_periph *periph, void *arg)
 	softc->disk->d_drv1 = periph;
 	softc->disk->d_maxsize = DFLTPHYS; /* XXX: probably not arbitrary */
 	softc->disk->d_unit = periph->unit_number;
-	softc->disk->d_flags = DISKFLAG_NEEDSGIANT;
+	softc->disk->d_flags = 0;
+	if ((softc->quirks & DA_Q_NO_SYNC_CACHE) == 0)
+		softc->disk->d_flags |= DISKFLAG_CANFLUSHCACHE;
 	disk_create(softc->disk, DISK_VERSION);
+	mtx_lock(periph->sim->mtx);
 
 	/*
 	 * Add async callbacks for bus reset and
@@ -1214,19 +1243,26 @@ daregister(struct cam_periph *periph, void *arg)
 	 * them and the only alternative would be to
 	 * not attach the device on failure.
 	 */
-	xpt_setup_ccb(&csa.ccb_h, periph->path, /*priority*/5);
-	csa.ccb_h.func_code = XPT_SASYNC_CB;
-	csa.event_enable = AC_SENT_BDR | AC_BUS_RESET | AC_LOST_DEVICE;
-	csa.callback = daasync;
-	csa.callback_arg = periph;
-	xpt_action((union ccb *)&csa);
+	xpt_register_async(AC_SENT_BDR | AC_BUS_RESET | AC_LOST_DEVICE,
+			   daasync, periph, periph->path);
+
 	/*
-	 * Lock this peripheral until we are setup.
-	 * This first call can't block
+	 * Take an exclusive refcount on the periph while dastart is called
+	 * to finish the probe.  The reference will be dropped in dadone at
+	 * the end of probe.
 	 */
-	(void)cam_periph_lock(periph, PRIBIO);
+	(void)cam_periph_hold(periph, PRIBIO);
 	xpt_schedule(periph, /*priority*/5);
 
+	/*
+	 * Schedule a periodic event to occasionally send an
+	 * ordered tag to a device.
+	 */
+	callout_init_mtx(&softc->sendordered_c, periph->sim->mtx, 0);
+	callout_reset(&softc->sendordered_c,
+	    (DA_DEFAULT_TIMEOUT * hz) / DA_ORDEREDTAG_INTERVAL,
+	    dasendorderedtag, softc);
+
 	return(CAM_REQ_CMP);
 }
 
@@ -1237,18 +1273,15 @@ dastart(struct cam_periph *periph, union ccb *start_ccb)
 
 	softc = (struct da_softc *)periph->softc;
 
-	
 	switch (softc->state) {
 	case DA_STATE_NORMAL:
 	{
 		/* Pull a buffer from the queue and get going on it */		
 		struct bio *bp;
-		int s;
 
 		/*
 		 * See if there is a buf with work for us to do..
 		 */
-		s = splbio();
 		bp = bioq_first(&softc->bio_queue);
 		if (periph->immediate_priority <= periph->pinfo.priority) {
 			CAM_DEBUG_PRINT(CAM_DEBUG_SUBTRACE,
@@ -1257,13 +1290,10 @@ dastart(struct cam_periph *periph, union ccb *start_ccb)
 			SLIST_INSERT_HEAD(&periph->ccb_list, &start_ccb->ccb_h,
 					  periph_links.sle);
 			periph->immediate_priority = CAM_PRIORITY_NONE;
-			splx(s);
 			wakeup(&periph->ccb_list);
 		} else if (bp == NULL) {
-			splx(s);
 			xpt_release_ccb(start_ccb);
 		} else {
-			int oldspl;
 			u_int8_t tag_code;
 
 			bioq_remove(&softc->bio_queue, bp);
@@ -1275,31 +1305,44 @@ dastart(struct cam_periph *periph, union ccb *start_ccb)
 			} else {
 				tag_code = MSG_SIMPLE_Q_TAG;
 			}
-			scsi_read_write(&start_ccb->csio,
-					/*retries*/da_retry_count,
-					/*cbfcnp*/dadone,
-					/*tag_action*/tag_code,
-					/*read_op*/bp->bio_cmd == BIO_READ,
-					/*byte2*/0,
-					softc->minimum_cmd_size,
-					/*lba*/bp->bio_pblkno,
-					/*block_count*/bp->bio_bcount /
-					softc->params.secsize,
-					/*data_ptr*/ bp->bio_data,
-					/*dxfer_len*/ bp->bio_bcount,
-					/*sense_len*/SSD_FULL_SIZE,
-					/*timeout*/da_default_timeout*1000);
+			switch (bp->bio_cmd) {
+			case BIO_READ:
+			case BIO_WRITE:
+				scsi_read_write(&start_ccb->csio,
+						/*retries*/da_retry_count,
+						/*cbfcnp*/dadone,
+						/*tag_action*/tag_code,
+						/*read_op*/bp->bio_cmd == BIO_READ,
+						/*byte2*/0,
+						softc->minimum_cmd_size,
+						/*lba*/bp->bio_pblkno,
+						/*block_count*/bp->bio_bcount /
+						softc->params.secsize,
+						/*data_ptr*/ bp->bio_data,
+						/*dxfer_len*/ bp->bio_bcount,
+						/*sense_len*/SSD_FULL_SIZE,
+						/*timeout*/da_default_timeout*1000);
+				break;
+			case BIO_FLUSH:
+				scsi_synchronize_cache(&start_ccb->csio,
+						       /*retries*/1,
+						       /*cbfcnp*/dadone,
+						       MSG_SIMPLE_Q_TAG,
+						       /*begin_lba*/0,/* Cover the whole disk */
+						       /*lb_count*/0,
+						       SSD_FULL_SIZE,
+						       /*timeout*/da_default_timeout*1000);
+				break;
+			}
 			start_ccb->ccb_h.ccb_state = DA_CCB_BUFFER_IO;
 
 			/*
 			 * Block out any asyncronous callbacks
 			 * while we touch the pending ccb list.
 			 */
-			oldspl = splcam();
 			LIST_INSERT_HEAD(&softc->pending_ccbs,
 					 &start_ccb->ccb_h, periph_links.le);
 			softc->outstanding_cmds++;
-			splx(oldspl);
 
 			/* We expect a unit attention from this device */
 			if ((softc->flags & DA_FLAG_RETRY_UA) != 0) {
@@ -1309,7 +1352,6 @@ dastart(struct cam_periph *periph, union ccb *start_ccb)
 
 			start_ccb->ccb_h.ccb_bp = bp;
 			bp = bioq_first(&softc->bio_queue);
-			splx(s);
 
 			xpt_action(start_ccb);
 		}
@@ -1326,7 +1368,7 @@ dastart(struct cam_periph *periph, union ccb *start_ccb)
 		struct scsi_read_capacity_data *rcap;
 
 		rcap = (struct scsi_read_capacity_data *)
-		    malloc(sizeof(*rcap), M_TEMP, M_NOWAIT|M_ZERO);
+		    malloc(sizeof(*rcap), M_SCSIDA, M_NOWAIT|M_ZERO);
 		if (rcap == NULL) {
 			printf("dastart: Couldn't malloc read_capacity data\n");
 			/* da_free_periph??? */
@@ -1351,7 +1393,7 @@ dastart(struct cam_periph *periph, union ccb *start_ccb)
 		struct scsi_read_capacity_data_long *rcaplong;
 
 		rcaplong = (struct scsi_read_capacity_data_long *)
-			malloc(sizeof(*rcaplong), M_TEMP, M_NOWAIT|M_ZERO);
+			malloc(sizeof(*rcaplong), M_SCSIDA, M_NOWAIT|M_ZERO);
 		if (rcaplong == NULL) {
 			printf("dastart: Couldn't malloc read_capacity data\n");
 			/* da_free_periph??? */
@@ -1392,9 +1434,8 @@ cmd6workaround(union ccb *ccb)
 	    (*cdb != READ_6 && *cdb != WRITE_6))
 		return 0;
 
-	xpt_print_path(ccb->ccb_h.path);
- 	printf("READ(6)/WRITE(6) not supported, "
-	       "increasing minimum_cmd_size to 10.\n");
+	xpt_print(ccb->ccb_h.path, "READ(6)/WRITE(6) not supported, "
+	    "increasing minimum_cmd_size to 10.\n");
  	softc = (struct da_softc *)xpt_path_periph(ccb->ccb_h.path)->softc;
 	softc->minimum_cmd_size = 10;
 
@@ -1434,12 +1475,10 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 	case DA_CCB_BUFFER_IO:
 	{
 		struct bio *bp;
-		int    oldspl;
 
 		bp = (struct bio *)done_ccb->ccb_h.ccb_bp;
 		if ((done_ccb->ccb_h.status & CAM_STATUS_MASK) != CAM_REQ_CMP) {
 			int error;
-			int s;
 			int sf;
 			
 			if ((csio->ccb_h.ccb_state & DA_CCB_RETRY_UA) != 0)
@@ -1457,18 +1496,17 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 			}
 			if (error != 0) {
 
-				s = splbio();
-
 				if (error == ENXIO) {
 					/*
 					 * Catastrophic error.  Mark our pack as
 					 * invalid.
 					 */
-					/* XXX See if this is really a media
-					 *     change first.
+					/*
+					 * XXX See if this is really a media
+					 * XXX change first?
 					 */
-					xpt_print_path(periph->path);
-					printf("Invalidating pack\n");
+					xpt_print(periph->path,
+					    "Invalidating pack\n");
 					softc->flags |= DA_FLAG_PACK_INVALID;
 				}
 
@@ -1478,7 +1516,6 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 				 * proper order should it attempt to recover.
 				 */
 				bioq_flush(&softc->bio_queue, NULL, EIO);
-				splx(s);
 				bp->bio_error = error;
 				bp->bio_resid = bp->bio_bcount;
 				bp->bio_flags |= BIO_ERROR;
@@ -1506,12 +1543,10 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 		 * Block out any asyncronous callbacks
 		 * while we touch the pending ccb list.
 		 */
-		oldspl = splcam();
 		LIST_REMOVE(&done_ccb->ccb_h, periph_links.le);
 		softc->outstanding_cmds--;
 		if (softc->outstanding_cmds == 0)
 			softc->flags |= DA_FLAG_WENT_IDLE;
-		splx(oldspl);
 
 		biodone(bp);
 		break;
@@ -1550,7 +1585,7 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 				 */
 				if (maxsector == 0xffffffff) {
 					softc->state = DA_STATE_PROBE2;
-					free(rdcap, M_TEMP);
+					free(rdcap, M_SCSIDA);
 					xpt_release_ccb(done_ccb);
 					xpt_schedule(periph, /*priority*/5);
 					return;
@@ -1566,9 +1601,9 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 			 * here.
 			 */
 			if (block_size >= MAXPHYS || block_size == 0) {
-				xpt_print_path(periph->path);
-				printf("unsupportable block size %ju\n",
-				      (uintmax_t) block_size);
+				xpt_print(periph->path,
+				    "unsupportable block size %ju\n",
+				    (uintmax_t) block_size);
 				announce_buf[0] = '\0';
 				cam_periph_invalidate(periph);
 			} else {
@@ -1663,14 +1698,13 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 						scsi_sense_print(
 							&done_ccb->csio);
 					else {
-						xpt_print_path(periph->path);
-						printf("got CAM status %#x\n",
-						       done_ccb->ccb_h.status);
+						xpt_print(periph->path,
+						    "got CAM status %#x\n",
+						    done_ccb->ccb_h.status);
 					}
 
-					xpt_print_path(periph->path);
-					printf("fatal error, failed" 
-					       " to attach to device\n");
+					xpt_print(periph->path, "fatal error, "
+					    "failed to attach to device\n");
 
 					/*
 					 * Free up resources.
@@ -1679,7 +1713,7 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 				} 
 			}
 		}
-		free(csio->data_ptr, M_TEMP);
+		free(csio->data_ptr, M_SCSIDA);
 		if (announce_buf[0] != '\0') {
 			xpt_announce_periph(periph, announce_buf);
 			/*
@@ -1698,7 +1732,7 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 		 * operation.
 		 */
 		xpt_release_ccb(done_ccb);
-		cam_periph_unlock(periph);
+		cam_periph_unhold(periph);
 		return;
 	}
 	case DA_CCB_WAITING:
@@ -1820,8 +1854,10 @@ dagetcapacity(struct cam_periph *periph)
 
 	/* Do a read capacity */
 	rcap = (struct scsi_read_capacity_data *)malloc(sizeof(*rcaplong),
-							M_TEMP,
-							M_WAITOK);
+							M_SCSIDA,
+							M_NOWAIT);
+	if (rcap == NULL)
+		return (ENOMEM);
 		
 	ccb = cam_periph_getccb(periph, /*priority*/1);
 	scsi_read_capacity(&ccb->csio,
@@ -1892,7 +1928,7 @@ done:
 
 	xpt_release_ccb(ccb);
 
-	free(rcap, M_TEMP);
+	free(rcap, M_SCSIDA);
 
 	return (error);
 }
@@ -1947,27 +1983,22 @@ dasetgeom(struct cam_periph *periph, uint32_t block_len, uint64_t maxsector)
 static void
 dasendorderedtag(void *arg)
 {
-	struct da_softc *softc;
-	int s;
-	if (da_send_ordered) {
-		for (softc = SLIST_FIRST(&softc_list);
-		     softc != NULL;
-		     softc = SLIST_NEXT(softc, links)) {
-			s = splsoftcam();
-			if ((softc->ordered_tag_count == 0) 
-			 && ((softc->flags & DA_FLAG_WENT_IDLE) == 0)) {
-				softc->flags |= DA_FLAG_NEED_OTAG;
-			}
-			if (softc->outstanding_cmds > 0)
-				softc->flags &= ~DA_FLAG_WENT_IDLE;
+	struct da_softc *softc = arg;
 
-			softc->ordered_tag_count = 0;
-			splx(s);
+	if (da_send_ordered) {
+		if ((softc->ordered_tag_count == 0) 
+		 && ((softc->flags & DA_FLAG_WENT_IDLE) == 0)) {
+			softc->flags |= DA_FLAG_NEED_OTAG;
 		}
-		/* Queue us up again */
-		timeout(dasendorderedtag, NULL,
-			(da_default_timeout * hz) / DA_ORDEREDTAG_INTERVAL);
+		if (softc->outstanding_cmds > 0)
+			softc->flags &= ~DA_FLAG_WENT_IDLE;
+
+		softc->ordered_tag_count = 0;
 	}
+	/* Queue us up again */
+	callout_reset(&softc->sendordered_c,
+	    (DA_DEFAULT_TIMEOUT * hz) / DA_ORDEREDTAG_INTERVAL,
+	    dasendorderedtag, softc);
 }
 
 /*
@@ -1982,6 +2013,8 @@ dashutdown(void * arg, int howto)
 
 	TAILQ_FOREACH(periph, &dadriver.units, unit_links) {
 		union ccb ccb;
+
+		cam_periph_lock(periph);
 		softc = (struct da_softc *)periph->softc;
 
 		/*
@@ -1989,8 +2022,10 @@ dashutdown(void * arg, int howto)
 		 * if the drive is capable of it..
 		 */
 		if (((softc->flags & DA_FLAG_OPEN) == 0)
-		 || (softc->quirks & DA_Q_NO_SYNC_CACHE))
+		 || (softc->quirks & DA_Q_NO_SYNC_CACHE)) {
+			cam_periph_unlock(periph);
 			continue;
+		}
 
 		xpt_setup_ccb(&ccb.ccb_h, periph->path, /*priority*/1);
 
@@ -2019,10 +2054,10 @@ dashutdown(void * arg, int howto)
 				if (sense_key != SSD_KEY_ILLEGAL_REQUEST)
 					scsi_sense_print(&ccb.csio);
 			} else {
-				xpt_print_path(periph->path);
-				printf("Synchronize cache failed, status "
-				       "== 0x%x, scsi status == 0x%x\n",
-				       ccb.ccb_h.status, ccb.csio.scsi_status);
+				xpt_print(periph->path, "Synchronize "
+				    "cache failed, status == 0x%x, scsi status "
+				    "== 0x%x\n", ccb.ccb_h.status,
+				    ccb.csio.scsi_status);
 			}
 		}
 
@@ -2032,7 +2067,7 @@ dashutdown(void * arg, int howto)
 					 /*reduction*/0,
 					 /*timeout*/0,
 					 /*getcount_only*/0);
-
+		cam_periph_unlock(periph);
 	}
 }
 
