@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_ses.c,v 1.32 2005/07/01 15:21:30 avatar Exp $");
+__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_ses.c,v 1.35 2007/05/16 16:54:23 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/queue.h>
@@ -43,6 +43,7 @@ __FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_ses.c,v 1.32 2005/07/01 15:21:30 avata
 #include <cam/cam_periph.h>
 #include <cam/cam_xpt_periph.h>
 #include <cam/cam_debug.h>
+#include <cam/cam_sim.h>
 
 #include <cam/scsi/scsi_all.h>
 #include <cam/scsi/scsi_message.h>
@@ -182,34 +183,19 @@ static struct cdevsw ses_cdevsw = {
 	.d_close =	sesclose,
 	.d_ioctl =	sesioctl,
 	.d_name =	"ses",
-	.d_flags =	D_NEEDGIANT,
+	.d_flags =	0,
 };
 
 static void
 sesinit(void)
 {
 	cam_status status;
-	struct cam_path *path;
 
 	/*
 	 * Install a global async callback.  This callback will
 	 * receive async callbacks like "new device found".
 	 */
-	status = xpt_create_path(&path, NULL, CAM_XPT_PATH_ID,
-	    CAM_TARGET_WILDCARD, CAM_LUN_WILDCARD);
-
-	if (status == CAM_REQ_CMP) {
-		struct ccb_setasync csa;
-
-                xpt_setup_ccb(&csa.ccb_h, path, 5);
-                csa.ccb_h.func_code = XPT_SASYNC_CB;
-                csa.event_enable = AC_FOUND_DEVICE;
-                csa.callback = sesasync;
-                csa.callback_arg = NULL;
-                xpt_action((union ccb *)&csa);
-		status = csa.ccb_h.status;
-                xpt_free_path(path);
-        }
+	status = xpt_register_async(AC_FOUND_DEVICE, sesasync, NULL, NULL);
 
 	if (status != CAM_REQ_CMP) {
 		printf("ses: Failed to attach master async callback "
@@ -221,24 +207,17 @@ static void
 sesoninvalidate(struct cam_periph *periph)
 {
 	struct ses_softc *softc;
-	struct ccb_setasync csa;
 
 	softc = (struct ses_softc *)periph->softc;
 
 	/*
 	 * Unregister any async callbacks.
 	 */
-	xpt_setup_ccb(&csa.ccb_h, periph->path, 5);
-	csa.ccb_h.func_code = XPT_SASYNC_CB;
-	csa.event_enable = 0;
-	csa.callback = sesasync;
-	csa.callback_arg = periph;
-	xpt_action((union ccb *)&csa);
+	xpt_register_async(0, sesasync, periph, periph->path);
 
 	softc->ses_flags |= SES_FLAG_INVALID;
 
-	xpt_print_path(periph->path);
-	printf("lost device\n");
+	xpt_print(periph->path, "lost device\n");
 }
 
 static void
@@ -250,8 +229,7 @@ sescleanup(struct cam_periph *periph)
 
 	destroy_dev(softc->ses_dev);
 
-	xpt_print_path(periph->path);
-	printf("removing device entry\n");
+	xpt_print(periph->path, "removing device entry\n");
 	free(softc, M_SCSISES);
 }
 
@@ -311,7 +289,6 @@ static cam_status
 sesregister(struct cam_periph *periph, void *arg)
 {
 	struct ses_softc *softc;
-	struct ccb_setasync csa;
 	struct ccb_getdev *cgd;
 	char *tname;
 
@@ -365,21 +342,18 @@ sesregister(struct cam_periph *periph, void *arg)
 		return (CAM_REQ_CMP_ERR);
 	}
 
+	cam_periph_unlock(periph);
 	softc->ses_dev = make_dev(&ses_cdevsw, unit2minor(periph->unit_number),
 	    UID_ROOT, GID_OPERATOR, 0600, "%s%d",
 	    periph->periph_name, periph->unit_number);
+	cam_periph_lock(periph);
 	softc->ses_dev->si_drv1 = periph;
 
 	/*
 	 * Add an async callback so that we get
 	 * notified if this device goes away.
 	 */
-	xpt_setup_ccb(&csa.ccb_h, periph->path, 5);
-	csa.ccb_h.func_code = XPT_SASYNC_CB;
-	csa.event_enable = AC_LOST_DEVICE;
-	csa.callback = sesasync;
-	csa.callback_arg = periph;
-	xpt_action((union ccb *)&csa);
+	xpt_register_async(AC_LOST_DEVICE, sesasync, periph, periph->path);
 
 	switch (softc->ses_type) {
 	default:
@@ -411,25 +385,20 @@ sesopen(struct cdev *dev, int flags, int fmt, struct thread *td)
 {
 	struct cam_periph *periph;
 	struct ses_softc *softc;
-	int error, s;
+	int error = 0;
 
-	s = splsoftcam();
 	periph = (struct cam_periph *)dev->si_drv1;
 	if (periph == NULL) {
-		splx(s);
 		return (ENXIO);
 	}
-	if ((error = cam_periph_lock(periph, PRIBIO | PCATCH)) != 0) {
-		splx(s);
-		return (error);
-	}
-	splx(s);
 
 	if (cam_periph_acquire(periph) != CAM_REQ_CMP) {
 		cam_periph_unlock(periph);
 		return (ENXIO);
 	}
 
+	cam_periph_lock(periph);
+
 	softc = (struct ses_softc *)periph->softc;
 
 	if (softc->ses_flags & SES_FLAG_INVALID) {
@@ -455,10 +424,10 @@ sesopen(struct cdev *dev, int flags, int fmt, struct thread *td)
 	}
 
 out:
+	cam_periph_unlock(periph);
 	if (error) {
 		cam_periph_release(periph);
 	}
-	cam_periph_unlock(periph);
 	return (error);
 }
 
@@ -475,11 +444,9 @@ sesclose(struct cdev *dev, int flag, int fmt, struct thread *td)
 	if (periph == NULL)
 		return (ENXIO);
 
+	cam_periph_lock(periph);
+
 	softc = (struct ses_softc *)periph->softc;
-
-	if ((error = cam_periph_lock(periph, PRIBIO)) != 0)
-		return (error);
-
 	softc->ses_flags &= ~SES_FLAG_OPEN;
 
 	cam_periph_unlock(periph);
@@ -491,13 +458,11 @@ sesclose(struct cdev *dev, int flag, int fmt, struct thread *td)
 static void
 sesstart(struct cam_periph *p, union ccb *sccb)
 {
-	int s = splbio();
 	if (p->immediate_priority <= p->pinfo.priority) {
 		SLIST_INSERT_HEAD(&p->ccb_list, &sccb->ccb_h, periph_links.sle);
 		p->immediate_priority = CAM_PRIORITY_NONE;
 		wakeup(&p->ccb_list);
 	}
-	splx(s);
 }
 
 static void
@@ -541,14 +506,17 @@ sesioctl(struct cdev *dev, u_long cmd, caddr_t arg_addr, int flag, struct thread
 
 	CAM_DEBUG(periph->path, CAM_DEBUG_TRACE, ("entering sesioctl\n"));
 
+	cam_periph_lock(periph);
 	ssc = (struct ses_softc *)periph->softc;
 
 	/*
 	 * Now check to see whether we're initialized or not.
 	 */
 	if ((ssc->ses_flags & SES_FLAG_INITIALIZED) == 0) {
+		cam_periph_unlock(periph);
 		return (ENXIO);
 	}
+	cam_periph_lock(periph);
 
 	error = 0;
 
@@ -578,22 +546,34 @@ sesioctl(struct cdev *dev, u_long cmd, caddr_t arg_addr, int flag, struct thread
 		break;
 		
 	case SESIOC_GETOBJMAP:
+		/*
+		 * XXX Dropping the lock while copying multiple segments is
+		 * bogus.
+		 */
+		cam_periph_lock(periph);
 		for (uobj = addr, i = 0; i != ssc->ses_nobjects; i++, uobj++) {
 			obj.obj_id = i;
 			obj.subencid = ssc->ses_objmap[i].subenclosure;
 			obj.object_type = ssc->ses_objmap[i].enctype;
+			cam_periph_lock(periph);
 			error = copyout(&obj, uobj, sizeof (ses_object));
+			cam_periph_lock(periph);
 			if (error) {
 				break;
 			}
 		}
+		cam_periph_lock(periph);
 		break;
 
 	case SESIOC_GETENCSTAT:
+		cam_periph_lock(periph);
 		error = (*ssc->ses_vec.get_encstat)(ssc, 1);
-		if (error)
+		if (error) {
+			cam_periph_unlock(periph);
 			break;
+		}
 		tmp = ssc->ses_encstat & ~ENCI_SVALID;
+		cam_periph_unlock(periph);
 		error = copyout(&tmp, addr, sizeof (ses_encstat));
 		ssc->ses_encstat = tmp;
 		break;
@@ -602,7 +582,9 @@ sesioctl(struct cdev *dev, u_long cmd, caddr_t arg_addr, int flag, struct thread
 		error = copyin(addr, &tmp, sizeof (ses_encstat));
 		if (error)
 			break;
+		cam_periph_lock(periph);
 		error = (*ssc->ses_vec.set_encstat)(ssc, tmp, 1);
+		cam_periph_unlock(periph);
 		break;
 
 	case SESIOC_GETOBJSTAT:
@@ -613,7 +595,9 @@ sesioctl(struct cdev *dev, u_long cmd, caddr_t arg_addr, int flag, struct thread
 			error = EINVAL;
 			break;
 		}
+		cam_periph_lock(periph);
 		error = (*ssc->ses_vec.get_objstat)(ssc, &objs, 1);
+		cam_periph_unlock(periph);
 		if (error)
 			break;
 		error = copyout(&objs, addr, sizeof (ses_objstat));
@@ -632,7 +616,9 @@ sesioctl(struct cdev *dev, u_long cmd, caddr_t arg_addr, int flag, struct thread
 			error = EINVAL;
 			break;
 		}
+		cam_periph_lock(periph);
 		error = (*ssc->ses_vec.set_objstat)(ssc, &objs, 1);
+		cam_periph_unlock(periph);
 
 		/*
 		 * Always (for now) invalidate entry.
@@ -642,11 +628,15 @@ sesioctl(struct cdev *dev, u_long cmd, caddr_t arg_addr, int flag, struct thread
 
 	case SESIOC_INIT:
 
+		cam_periph_lock(periph);
 		error = (*ssc->ses_vec.init_enc)(ssc);
+		cam_periph_unlock(periph);
 		break;
 
 	default:
+		cam_periph_lock(periph);
 		error = cam_periph_ioctl(periph, cmd, arg_addr, seserror);
+		cam_periph_unlock(periph);
 		break;
 	}
 	return (error);
