@@ -30,7 +30,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_sg.c,v 1.4.2.2 2007/04/17 01:21:08 scottl Exp $");
+__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_sg.c,v 1.9 2007/05/16 16:54:23 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -156,27 +156,12 @@ static void
 sginit(void)
 {
 	cam_status status;
-	struct cam_path *path;
 
 	/*
 	 * Install a global async callback.  This callback will receive aync
 	 * callbacks like "new device found".
 	 */
-	status = xpt_create_path(&path, /*periph*/NULL, CAM_XPT_PATH_ID,
-				 CAM_TARGET_WILDCARD, CAM_LUN_WILDCARD);
-
-	if (status == CAM_REQ_CMP) {
-		struct ccb_setasync csa;
-
-		xpt_setup_ccb(&csa.ccb_h, path, /*priority*/5);
-		csa.ccb_h.func_code = XPT_SASYNC_CB;
-		csa.event_enable = AC_FOUND_DEVICE;
-		csa.callback = sgasync;
-		csa.callback_arg = NULL;
-		xpt_action((union ccb *)&csa);
-		status = csa.ccb_h.status;
-		xpt_free_path(path);
-	}
+	status = xpt_register_async(AC_FOUND_DEVICE, sgasync, NULL, NULL);
 
 	if (status != CAM_REQ_CMP) {
 		printf("sg: Failed to attach master async callbac "
@@ -188,19 +173,13 @@ static void
 sgoninvalidate(struct cam_periph *periph)
 {
 	struct sg_softc *softc;
-	struct ccb_setasync csa;
 
 	softc = (struct sg_softc *)periph->softc;
 
 	/*
 	 * Deregister any async callbacks.
 	 */
-	xpt_setup_ccb(&csa.ccb_h, periph->path, /*priority*/5);
-	csa.ccb_h.func_code = XPT_SASYNC_CB;
-	csa.event_enable = 0;
-	csa.callback = sgasync;
-	csa.callback_arg = periph;
-	xpt_action((union ccb *)&csa);
+	xpt_register_async(0, sgasync, periph, periph->path);
 
 	softc->flags |= SG_FLAG_INVALID;
 
@@ -274,7 +253,6 @@ static cam_status
 sgregister(struct cam_periph *periph, void *arg)
 {
 	struct sg_softc *softc;
-	struct ccb_setasync csa;
 	struct ccb_getdev *cgd;
 	int no_tags;
 
@@ -306,6 +284,7 @@ sgregister(struct cam_periph *periph, void *arg)
 	 * We pass in 0 for all blocksize, since we don't know what the
 	 * blocksize of the device is, if it even has a blocksize.
 	 */
+	cam_periph_unlock(periph);
 	no_tags = (cgd->inq_data.flags & SID_CmdQue) == 0;
 	softc->device_stats = devstat_new_entry("sg",
 			unit2minor(periph->unit_number), 0,
@@ -321,18 +300,14 @@ sgregister(struct cam_periph *periph, void *arg)
 			      UID_ROOT, GID_OPERATOR, 0600, "%s%d",
 			      periph->periph_name, periph->unit_number);
 	(void)make_dev_alias(softc->dev, "sg%c", 'a' + periph->unit_number);
+	cam_periph_lock(periph);
 	softc->dev->si_drv1 = periph;
 
 	/*
 	 * Add as async callback so that we get
 	 * notified if this device goes away.
 	 */
-	xpt_setup_ccb(&csa.ccb_h, periph->path, /*priority*/5);
-	csa.ccb_h.func_code = XPT_SASYNC_CB;
-	csa.event_enable = AC_LOST_DEVICE;
-	csa.callback = sgasync;
-	csa.callback_arg = periph;
-	xpt_action((union ccb *)&csa);
+	xpt_register_async(AC_LOST_DEVICE, sgasync, periph, periph->path);
 
 	if (bootverbose)
 		xpt_announce_periph(periph, NULL);
@@ -407,10 +382,6 @@ sgopen(struct cdev *dev, int flags, int fmt, struct thread *td)
 	if (periph == NULL)
 		return (ENXIO);
 
-	softc = (struct sg_softc *)periph->softc;
-	if (softc->flags & SG_FLAG_INVALID)
-		return (ENXIO);
-
 	/*
 	 * Don't allow access when we're running at a high securelevel.
 	 */
@@ -418,13 +389,19 @@ sgopen(struct cdev *dev, int flags, int fmt, struct thread *td)
 	if (error)
 		return (error);
 
-	if ((error = cam_periph_lock(periph, PRIBIO | PCATCH)) != 0)
-		return (error);
+	cam_periph_lock(periph);
+
+	softc = (struct sg_softc *)periph->softc;
+	if (softc->flags & SG_FLAG_INVALID) {
+		cam_periph_unlock(periph);
+		return (ENXIO);
+	}
 
 	if ((softc->flags & SG_FLAG_OPEN) == 0) {
-		if (cam_periph_acquire(periph) != CAM_REQ_CMP)
-			return (ENXIO);
 		softc->flags |= SG_FLAG_OPEN;
+	} else {
+		/* Device closes aren't symmetrical, fix up the refcount. */
+		cam_periph_release(periph);
 	}
 
 	cam_periph_unlock(periph);
@@ -437,17 +414,14 @@ sgclose(struct cdev *dev, int flag, int fmt, struct thread *td)
 {
 	struct cam_periph *periph;
 	struct sg_softc *softc;
-	int error;
 
 	periph = (struct cam_periph *)dev->si_drv1;
 	if (periph == NULL)
 		return (ENXIO);
 
+	cam_periph_lock(periph);
+
 	softc = (struct sg_softc *)periph->softc;
-
-	if ((error = cam_periph_lock(periph, PRIBIO)) != 0)
-		return (error);
-
 	softc->flags &= ~SG_FLAG_OPEN;
 
 	cam_periph_unlock(periph);
@@ -470,6 +444,8 @@ sgioctl(struct cdev *dev, u_long cmd, caddr_t arg, int flag, struct thread *td)
 	if (periph == NULL)
 		return (ENXIO);
 
+	cam_periph_lock(periph);
+
 	softc = (struct sg_softc *)periph->softc;
 	error = 0;
 
@@ -666,6 +642,7 @@ sgioctl(struct cdev *dev, u_long cmd, caddr_t arg, int flag, struct thread *td)
 		break;
 	}
 
+	cam_periph_unlock(periph);
 	return (error);
 }
 
@@ -683,7 +660,6 @@ sgwrite(struct cdev *dev, struct uio *uio, int ioflag)
 	int error = 0, cdb_len, buf_len, dir;
 
 	periph = dev->si_drv1;
-	sc = periph->softc;
 	rdwr = malloc(sizeof(*rdwr), M_DEVBUF, M_WAITOK | M_ZERO);
 	hdr = &rdwr->hdr.hdr;
 
@@ -701,7 +677,6 @@ sgwrite(struct cdev *dev, struct uio *uio, int ioflag)
 		error = ENOMEM;
 		goto out_hdr;
 	}
-	xpt_setup_ccb(&ccb->ccb_h, periph->path, /*priority*/5);
 	csio = &ccb->csio;
 
 	/*
@@ -748,6 +723,9 @@ sgwrite(struct cdev *dev, struct uio *uio, int ioflag)
 		dir = CAM_DIR_NONE;
 	}
 
+	cam_periph_lock(periph);
+	sc = periph->softc;
+	xpt_setup_ccb(&ccb->ccb_h, periph->path, /*priority*/5);
 	cam_fill_csio(csio,
 		      /*retries*/1,
 		      sgdone,
@@ -771,7 +749,9 @@ sgwrite(struct cdev *dev, struct uio *uio, int ioflag)
 	ccb->ccb_h.ccb_rdwr = rdwr;
 	ccb->ccb_h.ccb_type = SG_CCB_RDWR_IO;
 	TAILQ_INSERT_TAIL(&sc->rdwr_done, rdwr, rdwr_link);
-	return (sgsendrdwr(periph, ccb));
+	error = sgsendrdwr(periph, ccb);
+	cam_periph_unlock(periph);
+	return (error);
 
 out_buf:
 	free(buf, M_DEVBUF);
@@ -794,7 +774,6 @@ sgread(struct cdev *dev, struct uio *uio, int ioflag)
 	int error, pack_len, reply_len, pack_id;
 
 	periph = dev->si_drv1;
-	sc = periph->softc;
 
 	/* XXX The pack len field needs to be updated and written out instead
 	 * of discarded.  Not sure how to do that.
@@ -808,17 +787,20 @@ sgread(struct cdev *dev, struct uio *uio, int ioflag)
 		return (error);
 	uio->uio_rw = UIO_READ;
 
+	cam_periph_lock(periph);
+	sc = periph->softc;
 search:
 	TAILQ_FOREACH(rdwr, &sc->rdwr_done, rdwr_link) {
 		if (rdwr->tag == pack_id)
 			break;
 	}
 	if ((rdwr == NULL) || (rdwr->state != SG_RDWR_DONE)) {
-		if (tsleep(rdwr, PCATCH, "sgread", 0) == ERESTART)
+		if (msleep(rdwr, periph->sim->mtx, PCATCH, "sgread", 0) == ERESTART)
 			return (EAGAIN);
 		goto search;
 	}
 	TAILQ_REMOVE(&sc->rdwr_done, rdwr, rdwr_link);
+	cam_periph_unlock(periph);
 
 	hdr = &rdwr->hdr.hdr;
 	csio = &rdwr->ccb->csio;
@@ -862,7 +844,9 @@ search:
 	if ((error == 0) && (hdr->result == 0))
 		error = uiomove(rdwr->buf, rdwr->buf_len, uio);
 
+	cam_periph_lock(periph);
 	xpt_free_ccb(rdwr->ccb);
+	cam_periph_unlock(periph);
 	free(rdwr->buf, M_DEVBUF);
 	free(rdwr, M_DEVBUF);
 	return (error);
@@ -879,7 +863,15 @@ sgsendccb(struct cam_periph *periph, union ccb *ccb)
 	if (((ccb->ccb_h.flags & CAM_DIR_MASK) != CAM_DIR_NONE)
 	    && (ccb->csio.data_ptr != NULL)) {
 		bzero(&mapinfo, sizeof(mapinfo));
+
+		/*
+		 * cam_periph_mapmem calls into proc and vm functions that can
+		 * sleep as well as trigger I/O, so we can't hold the lock.
+		 * Dropping it here is reasonably safe.
+		 */
+		cam_periph_unlock(periph);
 		error = cam_periph_mapmem(ccb, &mapinfo);
+		cam_periph_lock(periph);
 		if (error)
 			return (error);
 		need_unmap = 1;
