@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_targ_bh.c,v 1.22 2005/07/01 15:21:30 avatar Exp $");
+__FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_targ_bh.c,v 1.25 2007/05/16 16:54:23 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/queue.h>
@@ -46,6 +46,7 @@ __FBSDID("$FreeBSD: src/sys/cam/scsi/scsi_targ_bh.c,v 1.22 2005/07/01 15:21:30 a
 #include <cam/cam_queue.h>
 #include <cam/cam_xpt_periph.h>
 #include <cam/cam_debug.h>
+#include <cam/cam_sim.h>
 
 #include <cam/scsi/scsi_all.h>
 #include <cam/scsi/scsi_message.h>
@@ -155,27 +156,13 @@ static void
 targbhinit(void)
 {
 	cam_status status;
-	struct cam_path *path;
 
 	/*
 	 * Install a global async callback.  This callback will
 	 * receive async callbacks like "new path registered".
 	 */
-	status = xpt_create_path(&path, /*periph*/NULL, CAM_XPT_PATH_ID,
-				 CAM_TARGET_WILDCARD, CAM_LUN_WILDCARD);
-
-	if (status == CAM_REQ_CMP) {
-		struct ccb_setasync csa;
-
-		xpt_setup_ccb(&csa.ccb_h, path, /*priority*/5);
-		csa.ccb_h.func_code = XPT_SASYNC_CB;
-		csa.event_enable = AC_PATH_REGISTERED | AC_PATH_DEREGISTERED;
-		csa.callback = targbhasync;
-		csa.callback_arg = NULL;
-		xpt_action((union ccb *)&csa);
-		status = csa.ccb_h.status;
-		xpt_free_path(path);
-        }
+	status = xpt_register_async(AC_PATH_REGISTERED | AC_PATH_DEREGISTERED,
+				    targbhasync, NULL, NULL);
 
 	if (status != CAM_REQ_CMP) {
 		printf("targbh: Failed to attach master async callback "
@@ -263,9 +250,9 @@ targbhenlun(struct cam_periph *periph)
 	xpt_action(&immed_ccb);
 	status = immed_ccb.ccb_h.status;
 	if (status != CAM_REQ_CMP) {
-		xpt_print_path(periph->path);
-		printf("targbhenlun - Enable Lun Rejected with status 0x%x\n",
-		       status);
+		xpt_print(periph->path,
+		    "targbhenlun - Enable Lun Rejected with status 0x%x\n",
+		    status);
 		return (status);
 	}
 	
@@ -309,9 +296,9 @@ targbhenlun(struct cam_periph *periph)
 	}
 
 	if (i == 0) {
-		xpt_print_path(periph->path);
-		printf("targbhenlun - Could not allocate accept tio CCBs: "
-		       "status = 0x%x\n", status);
+		xpt_print(periph->path,
+		    "targbhenlun - Could not allocate accept tio CCBs: status "
+		    "= 0x%x\n", status);
 		targbhdislun(periph);
 		return (CAM_REQ_CMP_ERR);
 	}
@@ -345,9 +332,9 @@ targbhenlun(struct cam_periph *periph)
 	}
 
 	if (i == 0) {
-		xpt_print_path(periph->path);
-		printf("targbhenlun - Could not allocate immediate notify "
-		       "CCBs: status = 0x%x\n", status);
+		xpt_print(periph->path,
+		    "targbhenlun - Could not allocate immediate notify "
+		    "CCBs: status = 0x%x\n", status);
 		targbhdislun(periph);
 		return (CAM_REQ_CMP_ERR);
 	}
@@ -447,7 +434,7 @@ targbhdtor(struct cam_periph *periph)
 		/* FALLTHROUGH */
 	default:
 		/* XXX Wait for callback of targbhdislun() */
-		tsleep(softc, PRIBIO, "targbh", hz/2);
+		msleep(softc, periph->sim->mtx, PRIBIO, "targbh", hz/2);
 		free(softc, M_SCSIBH);
 		break;
 	}
@@ -462,27 +449,22 @@ targbhstart(struct cam_periph *periph, union ccb *start_ccb)
 	struct targbh_cmd_desc *desc;
 	struct ccb_scsiio *csio;
 	ccb_flags flags;
-	int    s;
 
 	softc = (struct targbh_softc *)periph->softc;
 	
-	s = splbio();
 	ccbh = TAILQ_FIRST(&softc->work_queue);
 	if (periph->immediate_priority <= periph->pinfo.priority) {
 		start_ccb->ccb_h.ccb_type = TARGBH_CCB_WAITING;			
 		SLIST_INSERT_HEAD(&periph->ccb_list, &start_ccb->ccb_h,
 				  periph_links.sle);
 		periph->immediate_priority = CAM_PRIORITY_NONE;
-		splx(s);
 		wakeup(&periph->ccb_list);
 	} else if (ccbh == NULL) {
-		splx(s);
 		xpt_release_ccb(start_ccb);	
 	} else {
 		TAILQ_REMOVE(&softc->work_queue, ccbh, periph_links.tqe);
 		TAILQ_INSERT_HEAD(&softc->pending_queue, ccbh,
 				  periph_links.tqe);
-		splx(s);	
 		atio = (struct ccb_accept_tio*)ccbh;
 		desc = (struct targbh_cmd_desc *)atio->ccb_h.ccb_descr;
 
@@ -543,9 +525,7 @@ targbhstart(struct cam_periph *periph, union ccb *start_ccb)
 					 /*getcount_only*/0); 
 			atio->ccb_h.status &= ~CAM_DEV_QFRZN;
 		}
-		s = splbio();
 		ccbh = TAILQ_FIRST(&softc->work_queue);
-		splx(s);
 	}
 	if (ccbh != NULL)
 		xpt_schedule(periph, /*priority*/1);
