@@ -26,15 +26,9 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/compat/ia32/ia32_signal.h,v 1.7 2005/04/05 22:41:49 peter Exp $
+ * $FreeBSD: src/sys/compat/ia32/ia32_signal.h,v 1.10 2006/10/05 01:56:11 davidxu Exp $
  */
 
-struct ia32_sigaltstack {
-	u_int32_t	ss_sp;		/* signal stack base */
-	u_int32_t	ss_size;	/* signal stack length */
-	int		ss_flags;	/* SS_DISABLE and/or SS_ONSTACK */
-};
-
 struct ia32_mcontext {
 	u_int32_t	mc_onstack;		/* XXX - sigcontext compat. */
 	u_int32_t	mc_gs;			/* machine state (struct trapframe) */
@@ -72,7 +66,7 @@ struct ia32_ucontext {
 	sigset_t		uc_sigmask;
 	struct ia32_mcontext	uc_mcontext;
 	u_int32_t		uc_link;
-	struct ia32_sigaltstack	uc_stack;
+	struct sigaltstack32	uc_stack;
 	u_int32_t		uc_flags;
 	u_int32_t		__spare__[4];
 };
@@ -108,7 +102,7 @@ struct ia32_ucontext4 {
 	sigset_t		uc_sigmask;
 	struct ia32_mcontext4	uc_mcontext;
 	u_int32_t		uc_link;
-	struct ia32_sigaltstack	uc_stack;
+	struct sigaltstack32	uc_stack;
 	u_int32_t		__spare__[8];
 };
 #endif
@@ -142,22 +136,6 @@ struct ia32_sigcontext3 {
 /*
  * Signal frames, arguments passed to application signal handlers.
  */
-union ia32_sigval {
-	int			sigval_int;
-	u_int32_t		sigval_ptr;
-};
-struct ia32_siginfo {
-	int			si_signo;	/* signal number */
-	int			si_errno;	/* errno association */
-	int			si_code;	/* signal code */
-	int32_t			si_pid;		/* sending process */
-	u_int32_t		si_uid;		/* sender's ruid */
-	int			si_status;	/* exit value */
-	u_int32_t		si_addr;	/* faulting instruction */
-	union ia32_sigval	si_value;	/* signal value */
-	int32_t			si_band;	/* band event for SIGPOLL */
-	int			__spare__[7];	/* gimme some slack */
-};
 
 #ifdef COMPAT_FREEBSD4
 struct ia32_sigframe4 {
@@ -167,7 +145,7 @@ struct ia32_sigframe4 {
 	u_int32_t		sf_addr;	/* undocumented 4th arg */
 	u_int32_t		sf_ah;		/* action/handler pointer */
 	struct ia32_ucontext4	sf_uc;		/* = *sf_ucontext */
-	struct ia32_siginfo	sf_si;		/* = *sf_siginfo (SA_SIGINFO case) */
+	struct siginfo32	sf_si;		/* = *sf_siginfo (SA_SIGINFO case) */
 };
 #endif
 
@@ -179,7 +157,7 @@ struct ia32_sigframe {
 	u_int32_t		sf_ah;		/* action/handler pointer */
 	/* Beware, hole due to ucontext being 16 byte aligned! */
 	struct ia32_ucontext	sf_uc;		/* = *sf_ucontext */
-	struct ia32_siginfo	sf_si;		/* = *sf_siginfo (SA_SIGINFO case) */
+	struct siginfo32	sf_si;		/* = *sf_siginfo (SA_SIGINFO case) */
 };
 
 #ifdef COMPAT_FREEBSD3
@@ -187,7 +165,7 @@ struct ia32_siginfo3 {
 	struct ia32_sigcontext3 si_sc;
 	int			si_signo;
 	int			si_code;
-	union ia32_sigval si_value;
+	union sigval32		si_value;
 };
 struct ia32_sigframe3 {
 	int			sf_signum;
@@ -199,10 +177,11 @@ struct ia32_sigframe3 {
 };
 #endif
 
+struct ksiginfo;
 extern char ia32_sigcode[];
 extern char freebsd4_ia32_sigcode[];
 extern int sz_ia32_sigcode;
 extern int sz_freebsd4_ia32_sigcode;
-extern void ia32_sendsig(sig_t, int, sigset_t *, u_long);
+extern void ia32_sendsig(sig_t, struct ksiginfo *, sigset_t *);
 extern void ia32_setregs(struct thread *td, u_long entry, u_long stack,
     u_long ps_strings);
