@@ -37,7 +37,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/compat/linux/linux_getcwd.c,v 1.19.2.3 2007/07/08 08:41:09 netchild Exp $");
+__FBSDID("$FreeBSD: src/sys/compat/linux/linux_getcwd.c,v 1.27 2006/11/18 17:27:39 kib Exp $");
 
 #include "opt_compat.h"
 #include "opt_mac.h"
@@ -54,13 +54,10 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_getcwd.c,v 1.19.2.3 2007/07/08 08
 #include <sys/mount.h>
 #include <sys/proc.h>
 #include <sys/uio.h>
-#include <sys/mac.h>
 #include <sys/malloc.h>
 #include <sys/dirent.h>
 #include <ufs/ufs/dir.h>	/* XXX only for DIRBLKSIZ */
 
-#include "opt_compat.h"
-
 #ifdef COMPAT_LINUX32
 #include <machine/../linux32/linux.h>
 #include <machine/../linux32/linux32_proto.h>
@@ -70,6 +67,8 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_getcwd.c,v 1.19.2.3 2007/07/08 08
 #endif
 #include <compat/linux/linux_util.h>
 
+#include <security/mac/mac_framework.h>
+
 static int
 linux_getcwd_scandir(struct vnode **, struct vnode **,
     char **, char *, struct thread *);
