@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/compat/linux/linux_ioctl.c,v 1.127.2.6 2007/09/03 13:55:05 netchild Exp $");
+__FBSDID("$FreeBSD: src/sys/compat/linux/linux_ioctl.c,v 1.138.2.1 2007/11/15 10:38:06 kib Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -43,7 +43,9 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_ioctl.c,v 1.127.2.6 2007/09/03 13
 #include <sys/filedesc.h>
 #include <sys/filio.h>
 #include <sys/kbio.h>
+#include <sys/kernel.h>
 #include <sys/linker_set.h>
+#include <sys/lock.h>
 #include <sys/malloc.h>
 #include <sys/proc.h>
 #include <sys/sbuf.h>
@@ -51,6 +53,7 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_ioctl.c,v 1.127.2.6 2007/09/03 13
 #include <sys/sockio.h>
 #include <sys/soundcard.h>
 #include <sys/stdint.h>
+#include <sys/sx.h>
 #include <sys/tty.h>
 #include <sys/uio.h>
 #include <net/if.h>
@@ -130,6 +133,8 @@ struct handler_element
 
 static TAILQ_HEAD(, handler_element) handlers =
 	TAILQ_HEAD_INITIALIZER(handlers);
+static struct sx linux_ioctl_sx;
+SX_SYSINIT(linux_ioctl, &linux_ioctl_sx, "linux ioctl handlers");
 
 /*
  * hdio related ioctls for VMWare support
@@ -280,15 +285,8 @@ struct linux_termios {
 	unsigned int c_oflag;
 	unsigned int c_cflag;
 	unsigned int c_lflag;
-#ifdef __alpha__
-	unsigned char c_cc[LINUX_NCCS];
-	unsigned char c_line;
-	unsigned int  c_ispeed;
-	unsigned int  c_ospeed;
-#else
 	unsigned char c_line;
 	unsigned char c_cc[LINUX_NCCS];
-#endif
 };
 
 struct linux_winsize {
@@ -638,20 +636,7 @@ bsd_to_linux_termio(struct termios *bios, struct linux_termio *lio)
 	lio->c_cflag = lios.c_cflag;
 	lio->c_lflag = lios.c_lflag;
 	lio->c_line  = lios.c_line;
-#ifdef __alpha__
-	lio->c_cc[LINUX__VINTR] = lios.c_cc[LINUX_VINTR];
-	lio->c_cc[LINUX__VQUIT] = lios.c_cc[LINUX_VQUIT];
-	lio->c_cc[LINUX__VERASE] = lios.c_cc[LINUX_VERASE];
-	lio->c_cc[LINUX__VKILL] = lios.c_cc[LINUX_VKILL];
-	lio->c_cc[LINUX__VEOF] =
-	    lios.c_cc[(lios.c_lflag & ICANON) ? LINUX_VEOF : LINUX_VMIN];
-	lio->c_cc[LINUX__VEOL] =
-	    lios.c_cc[(lios.c_lflag & ICANON) ? LINUX_VEOL : LINUX_VTIME];
-	lio->c_cc[LINUX__VEOL2] = lios.c_cc[LINUX_VEOL2];
-	lio->c_cc[LINUX__VSWTC] = lios.c_cc[LINUX_VSWTC];
-#else
 	memcpy(lio->c_cc, lios.c_cc, LINUX_NCC);
-#endif
 }
 
 static void
@@ -664,24 +649,9 @@ linux_to_bsd_termio(struct linux_termio *lio, struct termios *bios)
 	lios.c_oflag = lio->c_oflag;
 	lios.c_cflag = lio->c_cflag;
 	lios.c_lflag = lio->c_lflag;
-#ifdef __alpha__
-	for (i=0; i<LINUX_NCCS; i++)
-		lios.c_cc[i] = LINUX_POSIX_VDISABLE;
-	lios.c_cc[LINUX_VINTR] = lio->c_cc[LINUX__VINTR];
-	lios.c_cc[LINUX_VQUIT] = lio->c_cc[LINUX__VQUIT];
-	lios.c_cc[LINUX_VERASE] = lio->c_cc[LINUX__VERASE];
-	lios.c_cc[LINUX_VKILL] = lio->c_cc[LINUX__VKILL];
-	lios.c_cc[LINUX_VEOL2] = lio->c_cc[LINUX__VEOL2];
-	lios.c_cc[LINUX_VSWTC] = lio->c_cc[LINUX__VSWTC];
-	lios.c_cc[(lio->c_lflag & ICANON) ? LINUX_VEOF : LINUX_VMIN] =
-	    lio->c_cc[LINUX__VEOF];
-	lios.c_cc[(lio->c_lflag & ICANON) ? LINUX_VEOL : LINUX_VTIME] =
-	    lio->c_cc[LINUX__VEOL];
-#else
 	for (i=LINUX_NCC; i<LINUX_NCCS; i++)
 		lios.c_cc[i] = LINUX_POSIX_VDISABLE;
 	memcpy(lios.c_cc, lio->c_cc, LINUX_NCC);
-#endif
 	linux_to_bsd_termios(&lios, bios);
 }
 
@@ -1017,7 +987,7 @@ linux_ioctl_termio(struct thread *td, struct linux_ioctl_args *args)
 	case LINUX_TIOCGPTN: {
 		int nb;
 		
-		error = fo_ioctl(fp, LINUX_TIOCGPTN, (caddr_t)&nb, td->td_ucred, td);
+		error = fo_ioctl(fp, TIOCGPTN, (caddr_t)&nb, td->td_ucred, td);
 		if (!error)
 			error = copyout(&nb, (void *)args->arg,
 			    sizeof(int));
@@ -2353,6 +2323,7 @@ linux_ioctl_socket(struct thread *td, struct linux_ioctl_args *args)
 	case LINUX_SIOCGIFCONF:
 	case LINUX_SIOCGPGRP:
 	case LINUX_SIOCSPGRP:
+	case LINUX_SIOCGIFCOUNT:
 		/* these ioctls don't take an interface name */
 #ifdef DEBUG
 		printf("%s(): ioctl %d\n", __func__,
@@ -2374,6 +2345,7 @@ linux_ioctl_socket(struct thread *td, struct linux_ioctl_args *args)
 	case LINUX_SIOCSIFHWADDR:
 	case LINUX_SIOCDEVPRIVATE:
 	case LINUX_SIOCDEVPRIVATE+1:
+	case LINUX_SIOCGIFINDEX:
 		/* copy in the interface name and translate it. */
 		error = copyin((void *)args->arg, lifname, LINUX_IFNAMSIZ);
 		if (error != 0)
@@ -2508,6 +2480,15 @@ linux_ioctl_socket(struct thread *td, struct linux_ioctl_args *args)
 		error = ioctl(td, (struct ioctl_args *)args);
 		break;
 
+	case LINUX_SIOCGIFINDEX:
+		args->cmd = SIOCGIFINDEX;
+		error = ioctl(td, (struct ioctl_args *)args);
+		break;
+
+	case LINUX_SIOCGIFCOUNT:
+		error = 0;
+		break;
+
 	/*
 	 * XXX This is slightly bogus, but these ioctls are currently
 	 * XXX only used by the aironet (if_an) network driver.
@@ -2633,15 +2614,21 @@ linux_ioctl(struct thread *td, struct linux_ioctl_args *args)
 
 	/* Iterate over the ioctl handlers */
 	cmd = args->cmd & 0xffff;
+	sx_slock(&linux_ioctl_sx);
+	mtx_lock(&Giant);
 	TAILQ_FOREACH(he, &handlers, list) {
 		if (cmd >= he->low && cmd <= he->high) {
 			error = (*he->func)(td, args);
 			if (error != ENOIOCTL) {
+				mtx_unlock(&Giant);
+				sx_sunlock(&linux_ioctl_sx);
 				fdrop(fp, td);
 				return (error);
 			}
 		}
 	}
+	mtx_unlock(&Giant);
+	sx_sunlock(&linux_ioctl_sx);
 	fdrop(fp, td);
 
 	linux_msg(td, "ioctl fd=%d, cmd=0x%x ('%c',%d) is not implemented",
@@ -2663,6 +2650,7 @@ linux_ioctl_register_handler(struct linux_ioctl_handler *h)
 	 * Reuse the element if the handler is already on the list, otherwise
 	 * create a new element.
 	 */
+	sx_xlock(&linux_ioctl_sx);
 	TAILQ_FOREACH(he, &handlers, list) {
 		if (he->func == h->func)
 			break;
@@ -2683,10 +2671,12 @@ linux_ioctl_register_handler(struct linux_ioctl_handler *h)
 	TAILQ_FOREACH(cur, &handlers, list) {
 		if (cur->span > he->span) {
 			TAILQ_INSERT_BEFORE(cur, he, list);
+			sx_xunlock(&linux_ioctl_sx);
 			return (0);
 		}
 	}
 	TAILQ_INSERT_TAIL(&handlers, he, list);
+	sx_xunlock(&linux_ioctl_sx);
 
 	return (0);
 }
@@ -2699,13 +2689,16 @@ linux_ioctl_unregister_handler(struct linux_ioctl_handler *h)
 	if (h == NULL || h->func == NULL)
 		return (EINVAL);
 
+	sx_xlock(&linux_ioctl_sx);
 	TAILQ_FOREACH(he, &handlers, list) {
 		if (he->func == h->func) {
 			TAILQ_REMOVE(&handlers, he, list);
+			sx_xunlock(&linux_ioctl_sx);
 			FREE(he, M_LINUX);
 			return (0);
 		}
 	}
+	sx_xunlock(&linux_ioctl_sx);
 
 	return (EINVAL);
 }
