@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/compat/linux/linux_ioctl.h,v 1.22.2.2 2007/09/03 13:18:42 netchild Exp $
+ * $FreeBSD: src/sys/compat/linux/linux_ioctl.h,v 1.26.2.1 2007/11/15 10:38:06 kib Exp $
  */
 
 #ifndef _LINUX_IOCTL_H_
@@ -230,9 +230,12 @@
 #define	LINUX_SIOCGIFHWADDR	0x8927
 #define	LINUX_SIOCADDMULTI	0x8931
 #define	LINUX_SIOCDELMULTI	0x8932
+#define	LINUX_SIOCGIFINDEX	0x8933
+#define	LINUX_SIOGIFINDEX	LINUX_SIOCGIFINDEX
+#define	LINUX_SIOCGIFCOUNT	0x8938
 
 #define	LINUX_IOCTL_SOCKET_MIN	LINUX_FIOSETOWN
-#define	LINUX_IOCTL_SOCKET_MAX	LINUX_SIOCDELMULTI
+#define	LINUX_IOCTL_SOCKET_MAX	LINUX_SIOCGIFCOUNT
 
 /*
  * Device private ioctl calls 
@@ -312,19 +315,6 @@
 /*
  * termio
  */
-#ifdef __alpha__
-#define	LINUX_TCGETS		0x7413
-#define	LINUX_TCSETS		0x7414
-#define	LINUX_TCSETSW		0x7415
-#define	LINUX_TCSETSF		0x7416
-#define	LINUX_TCGETA		0x7417
-#define	LINUX_TCSETA		0x7418
-#define	LINUX_TCSETAW		0x7419
-#define	LINUX_TCSETAF		0x741c
-#define	LINUX_TCSBRK		0x741d
-#define	LINUX_TCXONC		0x741e
-#define	LINUX_TCFLSH		0x741f
-#else
 #define	LINUX_TCGETS		0x5401
 #define	LINUX_TCSETS		0x5402
 #define	LINUX_TCSETSW		0x5403
@@ -336,30 +326,19 @@
 #define	LINUX_TCSBRK		0x5409
 #define	LINUX_TCXONC		0x540A
 #define	LINUX_TCFLSH		0x540B
-#endif
 
 #define	LINUX_TIOCEXCL		0x540C
 #define	LINUX_TIOCNXCL		0x540D
 #define	LINUX_TIOCSCTTY		0x540E
 
-#ifdef __alpha__
-#define	LINUX_TIOCSPGRP		0x7476
-#define	LINUX_TIOCGPGRP		0x7477
-#else
 #define	LINUX_TIOCGPGRP		0x540F
 #define	LINUX_TIOCSPGRP		0x5410
-#endif
 
 #define	LINUX_TIOCOUTQ		0x5411
 #define	LINUX_TIOCSTI		0x5412
 
-#ifdef __alpha__
-#define	LINUX_TIOCSWINSZ	0x7467
-#define	LINUX_TIOCGWINSZ	0x7468
-#else
 #define	LINUX_TIOCGWINSZ	0x5413
 #define	LINUX_TIOCSWINSZ	0x5414
-#endif
 
 #define	LINUX_TIOCMGET		0x5415
 #define	LINUX_TIOCMBIS		0x5416
@@ -368,11 +347,7 @@
 #define	LINUX_TIOCGSOFTCAR	0x5419
 #define	LINUX_TIOCSSOFTCAR	0x541A
 
-#ifdef __alpha__
-#define	LINUX_FIONREAD		0x667f
-#else
 #define	LINUX_FIONREAD		0x541B
-#endif
 
 #define	LINUX_TIOCINQ		FIONREAD
 #define	LINUX_TIOCLINUX		0x541C
@@ -381,11 +356,7 @@
 #define	LINUX_TIOCSSERIAL	0x541F
 #define	LINUX_TIOCPKT		0x5420
 
-#ifdef __alpha__
-#define	LINUX_FIONBIO		0x667e
-#else
 #define	LINUX_FIONBIO		0x5421
-#endif
 
 #define	LINUX_TIOCNOTTY		0x5422
 #define	LINUX_TIOCSETD		0x5423
@@ -396,17 +367,11 @@
 #define	LINUX_TIOCSBRK		0x5427
 #define	LINUX_TIOCCBRK		0x5428
 
-#define	LINUX_TIOCGPTN		0x5430
+#define LINUX_TIOCGPTN		0x5430
 
-#ifdef __alpha__
-#define	LINUX_FIOCLEX		0x6601
-#define	LINUX_FIONCLEX		0x6602
-#define	LINUX_FIOASYNC		0x667d
-#else
 #define	LINUX_FIONCLEX		0x5450
 #define	LINUX_FIOCLEX		0x5451
 #define	LINUX_FIOASYNC		0x5452
-#endif
 
 #define	LINUX_TIOCSERCONFIG	0x5453
 #define	LINUX_TIOCSERGWILD	0x5454
@@ -414,13 +379,8 @@
 #define	LINUX_TIOCGLCKTRMIOS	0x5456
 #define	LINUX_TIOCSLCKTRMIOS	0x5457
 
-#ifdef __alpha__
-#define	LINUX_IOCTL_TERMIO_MIN	LINUX_TIOCEXCL
-#define	LINUX_IOCTL_TERMIO_MAX	LINUX_TIOCGPGRP
-#else
 #define	LINUX_IOCTL_TERMIO_MIN	LINUX_TCGETS
 #define	LINUX_IOCTL_TERMIO_MAX	LINUX_TIOCSLCKTRMIOS
-#endif
 
 /* arguments for tcflow() and LINUX_TCXONC */
 #define	LINUX_TCOOFF		0
@@ -440,18 +400,6 @@
 #define	LINUX_N_PPP		3
 
 /* Linux termio c_cc values */
-#ifdef __alpha__
-#define	LINUX__VINTR		0
-#define	LINUX__VQUIT		1
-#define	LINUX__VERASE		2
-#define	LINUX__VKILL		3
-#define	LINUX__VEOF		4
-#define	LINUX__VMIN		4
-#define	LINUX__VEOL		5
-#define	LINUX__VTIME		5
-#define	LINUX__VEOL2		6
-#define	LINUX__VSWTC		7
-#else
 #define	LINUX_VINTR		0
 #define	LINUX_VQUIT		1
 #define	LINUX_VERASE		2
@@ -460,29 +408,9 @@
 #define	LINUX_VTIME		5
 #define	LINUX_VMIN		6
 #define	LINUX_VSWTC		7
-#endif
 #define	LINUX_NCC		8
 
 /* Linux termios c_cc values */
-#ifdef __alpha__
-#define	LINUX_VEOF		0
-#define	LINUX_VEOL		1
-#define	LINUX_VEOL2		2
-#define	LINUX_VERASE		3
-#define	LINUX_VWERASE		4
-#define	LINUX_VKILL		5
-#define	LINUX_VREPRINT		6
-#define	LINUX_VSWTC		7
-#define	LINUX_VINTR		8
-#define	LINUX_VQUIT		9
-#define	LINUX_VSUSP		10
-#define	LINUX_VSTART		12
-#define	LINUX_VSTOP		13
-#define	LINUX_VLNEXT		14
-#define	LINUX_VDISCARD		15
-#define	LINUX_VMIN		16
-#define	LINUX_VTIME		17
-#else
 /* In addition to the termio values */
 #define	LINUX_VSTART		8
 #define	LINUX_VSTOP		9
@@ -493,7 +421,6 @@
 #define	LINUX_VWERASE		14
 #define	LINUX_VLNEXT		15
 #define	LINUX_VEOL2		16
-#endif
 #define	LINUX_NCCS		19
 
 #define	LINUX_POSIX_VDISABLE	'\0'
@@ -509,30 +436,18 @@
 #define	LINUX_IGNCR		0x0000080
 #define	LINUX_ICRNL		0x0000100
 
-#ifdef __alpha__
-#define	LINUX_IXON		0x0000200
-#define	LINUX_IXOFF		0x0000400
-#define	LINUX_IXANY		0x0000800
-#define	LINUX_IUCLC		0x0001000
-#else
 #define	LINUX_IUCLC		0x0000200
 #define	LINUX_IXON		0x0000400
 #define	LINUX_IXANY		0x0000800
 #define	LINUX_IXOFF		0x0001000
-#endif
 
 #define	LINUX_IMAXBEL		0x0002000
 
 /* Linux c_oflag masks */
 #define	LINUX_OPOST		0x0000001
 
-#ifdef __alpha__
-#define	LINUX_ONLCR		0x0000002
-#define	LINUX_OLCUC		0x0000004
-#else
 #define	LINUX_OLCUC		0x0000002
 #define	LINUX_ONLCR		0x0000004
-#endif
 
 #define	LINUX_OCRNL		0x0000008
 #define	LINUX_ONOCR		0x0000010
@@ -540,33 +455,6 @@
 #define	LINUX_OFILL		0x0000040
 #define	LINUX_OFDEL		0x0000080
 
-#ifdef __alpha__
-#define	LINUX_NLDLY		0x0000300
-#define	LINUX_NL0		0x0000000
-#define	LINUX_NL1		0x0000100
-#define	LINUX_NL2		0x0000200
-#define	LINUX_NL3		0x0000300
-#define	LINUX_TABDLY		0x000C000
-#define	LINUX_TAB0		0x0000000
-#define	LINUX_TAB1		0x0004000
-#define	LINUX_TAB2		0x0008000
-#define	LINUX_TAB3		0x000C000
-#define	LINUX_CRDLY		0x0030000
-#define	LINUX_CR0		0x0000000
-#define	LINUX_CR1		0x0010000
-#define	LINUX_CR2		0x0020000
-#define	LINUX_CR3		0x0030000
-#define	LINUX_FFDLY		0x0040000
-#define	LINUX_FF0		0x0000000
-#define	LINUX_FF1		0x0040000
-#define	LINUX_BSDLY		0x0080000
-#define	LINUX_BS0		0x0000000
-#define	LINUX_BS1		0x0080000
-#define	LINUX_VTDLY		0x0100000
-#define	LINUX_VT0		0x0000000
-#define	LINUX_VT1		0x0100000
-#define	LINUX_XTABS		0x0200000
-#else
 #define	LINUX_NLDLY		0x0000100
 #define	LINUX_NL0		0x0000000
 #define	LINUX_NL1		0x0000100
@@ -590,13 +478,8 @@
 #define	LINUX_FFDLY		0x0008000
 #define	LINUX_FF0		0x0000000
 #define	LINUX_FF1		0x0008000
-#endif
 
-#ifdef __alpha__
-#define	LINUX_CBAUD		0x0000001f
-#else
 #define	LINUX_CBAUD		0x0000100f
-#endif
 
 #define	LINUX_B0		0x00000000
 #define	LINUX_B50		0x00000001
@@ -617,29 +500,10 @@
 #define	LINUX_EXTA		LINUX_B19200
 #define	LINUX_EXTB		LINUX_B38400
 
-#ifdef __alpha__
-#define	LINUX_CBAUDEX		0x00000000
-#define	LINUX_B57600		0x00000010
-#define	LINUX_B115200		0x00000011
-#else
 #define	LINUX_CBAUDEX		0x00001000
 #define	LINUX_B57600		0x00001001
 #define	LINUX_B115200		0x00001002
-#endif
 
-#ifdef __alpha__
-#define	LINUX_CSIZE		0x00000300
-#define	LINUX_CS5		0x00000000
-#define	LINUX_CS6		0x00000100
-#define	LINUX_CS7		0x00000200
-#define	LINUX_CS8		0x00000300
-#define	LINUX_CSTOPB		0x00000400
-#define	LINUX_CREAD		0x00000800
-#define	LINUX_PARENB		0x00001000
-#define	LINUX_PARODD		0x00002000
-#define	LINUX_HUPCL		0x00004000
-#define	LINUX_CLOCAL		0x00008000
-#else
 #define	LINUX_CSIZE		0x00000030
 #define	LINUX_CS5		0x00000000
 #define	LINUX_CS6		0x00000010
@@ -651,28 +515,10 @@
 #define	LINUX_PARODD		0x00000200
 #define	LINUX_HUPCL		0x00000400
 #define	LINUX_CLOCAL		0x00000800
-#endif
 
 #define	LINUX_CRTSCTS		0x80000000
 
 /* Linux c_lflag masks */
-#ifdef __alpha__
-#define	LINUX_ECHOKE		0x00000001
-#define	LINUX_ECHOE		0x00000002
-#define	LINUX_ECHOK		0x00000004
-#define	LINUX_ECHO		0x00000008
-#define	LINUX_ECHONL		0x00000010
-#define	LINUX_ECHOPRT		0x00000020
-#define	LINUX_ECHOCTL		0x00000040
-#define	LINUX_ISIG		0x00000080
-#define	LINUX_ICANON		0x00000100
-#define	LINUX_IEXTEN		0x00000400
-#define	LINUX_XCASE		0x00004000
-#define	LINUX_TOSTOP		0x00400000
-#define	LINUX_FLUSHO		0x00800000
-#define	LINUX_PENDIN		0x20000000
-#define	LINUX_NOFLSH		0x80000000
-#else
 #define	LINUX_ISIG		0x00000001
 #define	LINUX_ICANON		0x00000002
 #define	LINUX_XCASE		0x00000004
@@ -688,7 +534,6 @@
 #define	LINUX_FLUSHO		0x00001000
 #define	LINUX_PENDIN		0x00002000
 #define	LINUX_IEXTEN		0x00008000
-#endif
 
 /* serial_struct values for TIOC[GS]SERIAL ioctls */
 #define	LINUX_ASYNC_CLOSING_WAIT_INF  0
