@@ -27,16 +27,12 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/compat/linux/linux_socket.c,v 1.59.2.4 2007/09/03 13:18:42 netchild Exp $");
+__FBSDID("$FreeBSD: src/sys/compat/linux/linux_socket.c,v 1.74 2007/08/06 14:25:59 rwatson Exp $");
 
 /* XXX we use functions that might not exist. */
 #include "opt_compat.h"
 #include "opt_inet6.h"
 
-#ifndef COMPAT_43
-#error "Unable to compile Linux-emulator due to missing COMPAT_43 option!"
-#endif
-
 #include <sys/param.h>
 #include <sys/proc.h>
 #include <sys/systm.h>
@@ -63,8 +59,6 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_socket.c,v 1.59.2.4 2007/09/03 13
 #include <netinet6/ip6_var.h>
 #endif
 
-#include "opt_compat.h"
-
 #ifdef COMPAT_LINUX32
 #include <machine/../linux32/linux.h>
 #include <machine/../linux32/linux32_proto.h>
@@ -200,7 +194,6 @@ linux_to_bsd_domain(int domain)
 	return (-1);
 }
 
-#ifndef __alpha__
 static int
 bsd_to_linux_domain(int domain)
 {
@@ -632,6 +625,7 @@ linux_bind(struct thread *td, struct linux_bind_args *args)
 		return (error);
 
 	error = kern_bind(td, linux_args.s, sa);
+	free(sa, M_SONAME);
 	if (error == EADDRNOTAVAIL && linux_args.namelen != sizeof(struct sockaddr_in))
 	   	return (EINVAL);
 	return (error);
@@ -643,7 +637,6 @@ struct linux_connect_args {
 	int namelen;
 };
 int linux_connect(struct thread *, struct linux_connect_args *);
-#endif /* !__alpha__*/
 
 int
 linux_connect(struct thread *td, struct linux_connect_args *args)
@@ -654,12 +647,8 @@ linux_connect(struct thread *td, struct linux_connect_args *args)
 	u_int fflag;
 	int error;
 
-#ifdef __alpha__
-	bcopy(args, &linux_args, sizeof(linux_args));
-#else
 	if ((error = copyin(args, &linux_args, sizeof(linux_args))))
 		return (error);
-#endif /* __alpha__ */
 
 	error = linux_getsockaddr(&sa,
 	    (struct osockaddr *)PTRIN(linux_args.name),
@@ -668,6 +657,7 @@ linux_connect(struct thread *td, struct linux_connect_args *args)
 		return (error);
 
 	error = kern_connect(td, linux_args.s, sa);
+	free(sa, M_SONAME);
 	if (error != EISCONN)
 		return (error);
 
@@ -680,7 +670,6 @@ linux_connect(struct thread *td, struct linux_connect_args *args)
 	 * socket and use the file descriptor reference instead of
 	 * creating a new one.
 	 */
-	NET_LOCK_GIANT();
 	error = fgetsock(td, linux_args.s, &so, &fflag);
 	if (error == 0) {
 		error = EISCONN;
@@ -693,12 +682,9 @@ linux_connect(struct thread *td, struct linux_connect_args *args)
 		}
 		fputsock(so);
 	}
-	NET_UNLOCK_GIANT();
 	return (error);
 }
 
-#ifndef __alpha__
-
 struct linux_listen_args {
 	int s;
 	int backlog;
@@ -1299,4 +1285,3 @@ linux_socketcall(struct thread *td, struct linux_socketcall_args *args)
 	uprintf("LINUX: 'socket' typ=%d not implemented\n", args->what);
 	return (ENOSYS);
 }
-#endif	/*!__alpha__*/
