@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/compat/linux/linux_stats.c,v 1.72.2.6.2.1 2007/12/08 13:43:47 dwmalone Exp $");
+__FBSDID("$FreeBSD: src/sys/compat/linux/linux_stats.c,v 1.88.4.1 2008/01/09 16:03:02 kib Exp $");
 
 #include "opt_compat.h"
 #include "opt_mac.h"
@@ -38,7 +38,6 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_stats.c,v 1.72.2.6.2.1 2007/12/08
 #include <sys/filedesc.h>
 #include <sys/proc.h>
 #include <sys/jail.h>
-#include <sys/mac.h>
 #include <sys/malloc.h>
 #include <sys/mount.h>
 #include <sys/namei.h>
@@ -59,6 +58,8 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_stats.c,v 1.72.2.6.2.1 2007/12/08
 
 #include <compat/linux/linux_util.h>
 
+#include <security/mac/mac_framework.h>
+
 /*
  * XXX: This was removed from newstat_copyout(), and almost identical
  * XXX: code was in stat64_copyout().  findcdev() needs to be replaced
@@ -99,23 +100,16 @@ static void
 translate_fd_major_minor(struct thread *td, int fd, struct stat *buf)
 {
 	struct file *fp;
-	int error;
 	int major, minor;
 
-	if ((error = fget(td, fd, &fp)) != 0)
+	if ((!S_ISCHR(buf->st_mode) && !S_ISBLK(buf->st_mode)) ||
+	    fget(td, fd, &fp) != 0)
 		return;
-	if (fp->f_vnode) {
-		if (fp->f_vnode->v_type == VCHR
-		    || fp->f_vnode->v_type == VBLK) {
-			if (fp->f_vnode->v_un.vu_cdev) {
-				if (linux_driver_get_major_minor(
-				    fp->f_vnode->v_un.vu_cdev->si_name,
-				    &major, &minor) == 0) {
-					buf->st_rdev = (major << 8 | minor);
-				}
-			}
-		}
-	}
+	if (fp->f_vnode != NULL &&
+	    fp->f_vnode->v_un.vu_cdev != NULL &&
+	    linux_driver_get_major_minor(fp->f_vnode->v_un.vu_cdev->si_name,
+					 &major, &minor) == 0)
+		buf->st_rdev = (major << 8 | minor);
 	fdrop(fp, td);
 }
 
@@ -124,19 +118,17 @@ translate_path_major_minor(struct thread *td, char *path, struct stat *buf)
 {
 	struct proc *p = td->td_proc;	
 	struct filedesc *fdp = p->p_fd;
-	struct file *fp;
 	int fd;
 	int temp;
 
+	if (!S_ISCHR(buf->st_mode) && !S_ISBLK(buf->st_mode))
+		return;
 	temp = td->td_retval[0];
 	if (kern_open(td, path, UIO_SYSSPACE, O_RDONLY, 0) != 0)
 		return;
 	fd = td->td_retval[0];
 	td->td_retval[0] = temp;
 	translate_fd_major_minor(td, fd, buf);
-	FILEDESC_LOCK(fdp);
-	fp = fdp->fd_ofiles[fd];
-	FILEDESC_UNLOCK(fdp);
 	fdclose(fdp, fdp->fd_ofiles[fd], fd, td);
 }
 
@@ -178,18 +170,19 @@ linux_newstat(struct thread *td, struct linux_newstat_args *args)
 #endif
 
 	error = kern_stat(td, path, UIO_SYSSPACE, &buf);
-	  if (!error && strlen(path) > strlen("/dev/pts/") &&
-	      !strncmp(path, "/dev/pts/", strlen("/dev/pts/"))
-	      && path[9] >= '0' && path[9] <= '9') {
-		  /*
-		   * Linux checks major and minors of the slave device to make
-		   * sure it's a pty device, so let's make him believe it is.
-		   */
-		  buf.st_rdev = (136 << 8);
-	  }
-
-	translate_path_major_minor(td, path, &buf);
-
+	if (!error) {
+		if (strlen(path) > strlen("/dev/pts/") &&
+		    !strncmp(path, "/dev/pts/", strlen("/dev/pts/")) &&
+		    path[9] >= '0' && path[9] <= '9') {
+			/*
+			 * Linux checks major and minors of the slave device
+			 * to make sure it's a pty device, so let's make him
+			 * believe it is.
+			 */
+			buf.st_rdev = (136 << 8);
+		} else
+			translate_path_major_minor(td, path, &buf);
+	}
 	LFREEPATH(path);
 	if (error)
 		return (error);
@@ -270,15 +263,22 @@ int
 linux_stat(struct thread *td, struct linux_stat_args *args)
 {
 	struct stat buf;
+	char *path;
 	int error;
+
+	LCONVPATHEXIST(td, args->path, &path);
+
 #ifdef DEBUG
 	if (ldebug(stat))
-	printf(ARGS(stat, "%s, *"), args->path);
+		printf(ARGS(stat, "%s, *"), path);
 #endif
-	error = kern_stat(td, args->path, UIO_SYSSPACE, &buf);
-	if (error)
+	error = kern_stat(td, path, UIO_SYSSPACE, &buf);
+	if (error) {
+		LFREEPATH(path);
 		return (error);
-	translate_path_major_minor(td, args->path, &buf);
+	}
+	translate_path_major_minor(td, path, &buf);
+	LFREEPATH(path);
 	return(stat_copyout(&buf, args->up));
 }
 
@@ -286,16 +286,22 @@ int
 linux_lstat(struct thread *td, struct linux_lstat_args *args)
 {
 	struct stat buf;
+	char *path;
 	int error;
 
+	LCONVPATHEXIST(td, args->path, &path);
+
 #ifdef DEBUG
 	if (ldebug(lstat))
-	printf(ARGS(lstat, "%s, *"), args->path);
+		printf(ARGS(lstat, "%s, *"), path);
 #endif
-	error = kern_lstat(td, args->path, UIO_SYSSPACE, &buf);
-	if (error)
+	error = kern_lstat(td, path, UIO_SYSSPACE, &buf);
+	if (error) {
+		LFREEPATH(path);
 		return (error);
-	translate_path_major_minor(td, args->path, &buf);
+	}
+	translate_path_major_minor(td, path, &buf);
+	LFREEPATH(path);
 	return(stat_copyout(&buf, args->up));
 }
 
@@ -323,7 +329,7 @@ struct l_statfs {
 #define	LINUX_NTFS_SUPER_MAGIC	0x5346544EL
 #define	LINUX_PROC_SUPER_MAGIC	0x9fa0L
 #define	LINUX_UFS_SUPER_MAGIC	0x00011954L	/* XXX - UFS_MAGIC in Linux */
-#define	LINUX_DEVFS_SUPER_MAGIC	0x1373L
+#define LINUX_DEVFS_SUPER_MAGIC	0x1373L
 
 static long
 bsd_to_linux_ftype(const char *fstypename)
