@@ -30,7 +30,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/compat/linux/linux_util.c,v 1.29.2.1 2006/06/15 15:52:04 ambrisko Exp $");
+__FBSDID("$FreeBSD: src/sys/compat/linux/linux_util.c,v 1.32 2007/02/24 16:49:24 netchild Exp $");
+
+#include "opt_compat.h"
 
 #include <sys/param.h>
 #include <sys/bus.h>
@@ -47,16 +49,20 @@ __FBSDID("$FreeBSD: src/sys/compat/linux/linux_util.c,v 1.29.2.1 2006/06/15 15:5
 #include <machine/stdarg.h>
 
 #include <compat/linux/linux_util.h>
+#ifdef COMPAT_LINUX32
+#include <machine/../linux32/linux.h>
+#else
+#include <machine/../linux/linux.h>
+#endif
 
 const char      linux_emul_path[] = "/compat/linux";
 
 /*
- * Search an alternate path before passing pathname arguments on
- * to system calls. Useful for keeping a separate 'emulation tree'.
+ * Search an alternate path before passing pathname arguments on to
+ * system calls. Useful for keeping a separate 'emulation tree'.
  *
- * If cflag is set, we check if an attempt can be made to create
- * the named file, i.e. we check if the directory it should
- * be in exists.
+ * If cflag is set, we check if an attempt can be made to create the
+ * named file, i.e. we check if the directory it should be in exists.
  */
 int
 linux_emul_convpath(td, path, pathseg, pbuf, cflag)
@@ -85,8 +91,6 @@ linux_msg(const struct thread *td, const char *fmt, ...)
 	printf("\n");
 }
 
-MALLOC_DECLARE(M_LINUX);
-
 struct device_element
 {
 	TAILQ_ENTRY(device_element) list;
