@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
- * $FreeBSD: src/sys/compat/svr4/svr4_siginfo.h,v 1.4 2005/01/05 22:34:36 imp Exp $
+ * $FreeBSD: src/sys/compat/svr4/svr4_siginfo.h,v 1.5 2005/10/19 09:33:15 davidxu Exp $
  */
 
 #ifndef	_SVR4_SIGINFO_H_
@@ -96,16 +96,16 @@ typedef union svr4_siginfo {
 	} _info;
 } svr4_siginfo_t;
 
-#define	si_signo	_info._signo
-#define	si_code		_info._code
-#define	si_errno	_info._errno
+#define	svr4_si_signo	_info._signo
+#define	svr4_si_code	_info._code
+#define	svr4_si_errno	_info._errno
 
-#define si_pid		_info._reason._child._pid
-#define si_stime	_info._reason._child._stime
-#define si_status	_info._reason._child._status
-#define si_utime	_info._reason._child._utime
+#define svr4_si_pid	_info._reason._child._pid
+#define svr4_si_stime	_info._reason._child._stime
+#define svr4_si_status	_info._reason._child._status
+#define svr4_si_utime	_info._reason._child._utime
 
-#define si_addr		_info._reason._fault._addr
-#define si_trap		_info._reason._fault._trap
+#define svr4_si_addr	_info._reason._fault._addr
+#define svr4_si_trap	_info._reason._fault._trap
 
 #endif /* !_SVR4_SIGINFO_H_ */
