@@ -36,11 +36,10 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/compat/svr4/svr4_stream.c,v 1.53.2.1 2005/07/18 19:48:13 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/compat/svr4/svr4_stream.c,v 1.62 2006/08/05 22:04:21 rwatson Exp $");
 
 #include "opt_compat.h"
 #include "opt_ktrace.h"
-#include "opt_mac.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -50,7 +49,6 @@ __FBSDID("$FreeBSD: src/sys/compat/svr4/svr4_stream.c,v 1.53.2.1 2005/07/18 19:4
 #include <sys/lock.h>
 #include <sys/malloc.h>
 #include <sys/file.h> 		/* Must come after sys/malloc.h */
-#include <sys/mac.h>
 #include <sys/mbuf.h>
 #include <sys/mutex.h>
 #include <sys/proc.h>
@@ -133,237 +131,6 @@ static int ti_getinfo(struct file *, int, struct svr4_strioctl *,
 			      struct thread *);
 static int ti_bind(struct file *, int, struct svr4_strioctl *, struct thread *);
 
-/* infrastructure */
-static int svr4_sendit(struct thread *td, int s, struct msghdr *mp, int flags);
-
-static int svr4_recvit(struct thread *td, int s, struct msghdr *mp,
-			    caddr_t namelenp);
-
-/* <sigh>  Ok, so we shouldn't use sendit() in uipc_syscalls.c because
- * it isn't part of a "public" interface;  We're supposed to use
- * pru_sosend instead.  Same goes for recvit()/pru_soreceive() for
- * that matter.  Solution:  Suck sendit()/recvit() into here where we
- * can do what we like.
- * 
- * I hate code duplication. 
- * 
- * I will take out all the #ifdef COMPAT_OLDSOCK gumph, though.
- */
-static int
-svr4_sendit(td, s, mp, flags)
-	register struct thread *td;
-	int s;
-	register struct msghdr *mp;
-	int flags;
-{
-	struct uio auio;
-	register struct iovec *iov;
-	register int i;
-	struct mbuf *control;
-	struct sockaddr *to;
-	int len, error;
-	struct socket *so;
-#ifdef KTRACE
-	struct uio *ktruio = NULL;
-#endif
-
-	if ((error = fgetsock(td, s, &so, NULL)) != 0)
-		return (error);
-
-#ifdef MAC
-	SOCK_LOCK(so);
-	error = mac_check_socket_send(td->td_ucred, so);
-	SOCK_UNLOCK(so);
-	if (error)
-		goto done1;
-#endif
-
-	auio.uio_iov = mp->msg_iov;
-	auio.uio_iovcnt = mp->msg_iovlen;
-	auio.uio_segflg = UIO_USERSPACE;
-	auio.uio_rw = UIO_WRITE;
-	auio.uio_td = td;
-	auio.uio_offset = 0;			/* XXX */
-	auio.uio_resid = 0;
-	iov = mp->msg_iov;
-	for (i = 0; i < mp->msg_iovlen; i++, iov++) {
-		if ((auio.uio_resid += iov->iov_len) < 0) {
-			error = EINVAL;
-			goto done1;
-		}
-	}
-	if (mp->msg_name) {
-		error = getsockaddr(&to, mp->msg_name, mp->msg_namelen);
-		if (error)
-			goto done1;
-	} else {
-		to = 0;
-	}
-	if (mp->msg_control) {
-		if (mp->msg_controllen < sizeof(struct cmsghdr)) {
-			error = EINVAL;
-			goto bad;
-		}
-		error = sockargs(&control, mp->msg_control,
-		    mp->msg_controllen, MT_CONTROL);
-		if (error)
-			goto bad;
-	} else {
-		control = 0;
-	}
-#ifdef KTRACE
-	if (KTRPOINT(td, KTR_GENIO))
-		ktruio = cloneuio(&auio);
-#endif
-	len = auio.uio_resid;
-	error = so->so_proto->pr_usrreqs->pru_sosend(so, to, &auio, 0, control,
-						     flags, td);
-	if (error) {
-		if (auio.uio_resid != len && (error == ERESTART ||
-		    error == EINTR || error == EWOULDBLOCK))
-			error = 0;
-		if (error == EPIPE) {
-			PROC_LOCK(td->td_proc);
-			psignal(td->td_proc, SIGPIPE);
-			PROC_UNLOCK(td->td_proc);
-		}
-	}
-	if (error == 0)
-		td->td_retval[0] = len - auio.uio_resid;
-#ifdef KTRACE
-	if (ktruio != NULL) {
-		ktruio->uio_resid = td->td_retval[0];
-		ktrgenio(s, UIO_WRITE, ktruio, error);
-	}
-#endif
-bad:
-	if (to)
-		FREE(to, M_SONAME);
-done1:
-	fputsock(so);
-	return (error);
-}
-
-static int
-svr4_recvit(td, s, mp, namelenp)
-	register struct thread *td;
-	int s;
-	register struct msghdr *mp;
-	caddr_t namelenp;
-{
-	struct uio auio;
-	register struct iovec *iov;
-	register int i;
-	int len, error;
-	struct mbuf *m, *control = 0;
-	caddr_t ctlbuf;
-	struct socket *so;
-	struct sockaddr *fromsa = 0;
-#ifdef KTRACE
-	struct uio *ktruio = NULL;
-#endif
-
-	if ((error = fgetsock(td, s, &so, NULL)) != 0)
-		return (error);
-
-#ifdef MAC
-	SOCK_LOCK(so);
-	error = mac_check_socket_receive(td->td_ucred, so);
-	SOCK_UNLOCK(so);
-	if (error)
-		goto done1;
-#endif
-
-	auio.uio_iov = mp->msg_iov;
-	auio.uio_iovcnt = mp->msg_iovlen;
-	auio.uio_segflg = UIO_USERSPACE;
-	auio.uio_rw = UIO_READ;
-	auio.uio_td = td;
-	auio.uio_offset = 0;			/* XXX */
-	auio.uio_resid = 0;
-	iov = mp->msg_iov;
-	for (i = 0; i < mp->msg_iovlen; i++, iov++) {
-		if ((auio.uio_resid += iov->iov_len) < 0) {
-			error = EINVAL;
-			goto done1;
-		}
-	}
-#ifdef KTRACE
-	if (KTRPOINT(td, KTR_GENIO)) 
-		ktruio = cloneuio(&auio);
-#endif
-	len = auio.uio_resid;
-	error = so->so_proto->pr_usrreqs->pru_soreceive(so, &fromsa, &auio,
-	    (struct mbuf **)0, mp->msg_control ? &control : (struct mbuf **)0,
-	    &mp->msg_flags);
-	if (error) {
-		if (auio.uio_resid != len && (error == ERESTART ||
-		    error == EINTR || error == EWOULDBLOCK))
-			error = 0;
-	}
-#ifdef KTRACE
-	if (ktruio != NULL) {
-		ktruio->uio_resid = len - auio.uio_resid;
-		ktrgenio(s, UIO_READ, ktruio, error);
-	}
-#endif
-	if (error)
-		goto out;
-	td->td_retval[0] = len - auio.uio_resid;
-	if (mp->msg_name) {
-		len = mp->msg_namelen;
-		if (len <= 0 || fromsa == 0)
-			len = 0;
-		else {
-			/* save sa_len before it is destroyed by MSG_COMPAT */
-			len = MIN(len, fromsa->sa_len);
-			error = copyout(fromsa,
-			    (caddr_t)mp->msg_name, (unsigned)len);
-			if (error)
-				goto out;
-		}
-		mp->msg_namelen = len;
-		if (namelenp &&
-		    (error = copyout((caddr_t)&len, namelenp, sizeof (int)))) {
-			goto out;
-		}
-	}
-	if (mp->msg_control) {
-		len = mp->msg_controllen;
-		m = control;
-		mp->msg_controllen = 0;
-		ctlbuf = (caddr_t) mp->msg_control;
-
-		while (m && len > 0) {
-			unsigned int tocopy;
-
-			if (len >= m->m_len) 
-				tocopy = m->m_len;
-			else {
-				mp->msg_flags |= MSG_CTRUNC;
-				tocopy = len;
-			}
-		
-			if ((error = copyout((caddr_t)mtod(m, caddr_t),
-					ctlbuf, tocopy)) != 0)
-				goto out;
-
-			ctlbuf += tocopy;
-			len -= tocopy;
-			m = m->m_next;
-		}
-		mp->msg_controllen = ctlbuf - (caddr_t)mp->msg_control;
-	}
-out:
-	if (fromsa)
-		FREE(fromsa, M_SONAME);
-	if (control)
-		m_freem(control);
-done1:
-	fputsock(so);
-	return (error);
-}
-
 #ifdef DEBUG_SVR4
 static void bufprint(u_char *, size_t);
 static int show_ioc(const char *, struct svr4_strioctl *);
@@ -981,11 +748,9 @@ ti_bind(fp, fd, ioc, td)
 	struct svr4_strm *st = svr4_stream_get(fp);
 	struct sockaddr_in sain;
 	struct sockaddr_un saun;
-	caddr_t sg;
-	void *skp, *sup = NULL;
+	struct sockaddr *skp;
 	int sasize;
 	struct svr4_strmcmd bnd;
-	struct bind_args ba;
 
 	if (st == NULL) {
 		DPRINTF(("ti_bind: bad file descriptor\n"));
@@ -1005,11 +770,11 @@ ti_bind(fp, fd, ioc, td)
 
 	switch (st->s_family) {
 	case AF_INET:
-		skp = &sain;
+		skp = (struct sockaddr *)&sain;
 		sasize = sizeof(sain);
 
 		if (bnd.offs == 0)
-			goto reply;
+			goto error;
 
 		netaddr_to_sockaddr_in(&sain, &bnd);
 
@@ -1019,15 +784,15 @@ ti_bind(fp, fd, ioc, td)
 		break;
 
 	case AF_LOCAL:
-		skp = &saun;
+		skp = (struct sockaddr *)&saun;
 		sasize = sizeof(saun);
 		if (bnd.offs == 0)
-			goto reply;
+			goto error;
 
 		netaddr_to_sockaddr_un(&saun, &bnd);
 
 		if (saun.sun_path[0] == '\0')
-			goto reply;
+			goto error;
 
 		DPRINTF(("TI_BIND: fam %d, path %s\n",
 			 saun.sun_family, saun.sun_path));
@@ -1044,29 +809,20 @@ ti_bind(fp, fd, ioc, td)
 		return ENOSYS;
 	}
 
-	sg = stackgap_init();
-	sup = stackgap_alloc(&sg, sasize);
-
-	if ((error = copyout(skp, sup, sasize)) != 0)
-		return error;
-
-	ba.s = fd;
 	DPRINTF(("TI_BIND: fileno %d\n", fd));
-	ba.name = (void *) sup;
-	ba.namelen = sasize;
 
-	if ((error = bind(td, &ba)) != 0) {
+	if ((error = kern_bind(td, fd, skp)) != 0) {
 		DPRINTF(("TI_BIND: bind failed %d\n", error));
 		return error;
 	}
+	goto reply;
+
+error:
+	memset(&bnd, 0, sizeof(bnd));
+	bnd.len = sasize + 4;
+	bnd.offs = 0x10;	/* XXX */
 
 reply:
-	if (sup == NULL) {
-		memset(&bnd, 0, sizeof(bnd));
-		bnd.len = sasize + 4;
-		bnd.offs = 0x10;	/* XXX */
-	}
-
 	bnd.cmd = SVR4_TI_BIND_REPLY;
 
 	if ((error = copyout(&bnd, ioc->buf, ioc->len)) != 0)
@@ -1119,13 +875,9 @@ svr4_stream_ti_ioctl(fp, td, retval, fd, cmd, dat)
 	struct svr4_strbuf skb, *sub = (struct svr4_strbuf *) dat;
 	struct svr4_strm *st = svr4_stream_get(fp);
 	int error;
-	void *skp, *sup;
-	struct sockaddr_in sain;
-	struct sockaddr_un saun;
+	struct sockaddr *sa;
+	socklen_t sasize, oldsasize;
 	struct svr4_strmcmd sc;
-	int sasize, oldsasize;
-	caddr_t sg;
-	int *lenp;
 
 	DPRINTF(("svr4_stream_ti_ioctl\n"));
 
@@ -1141,13 +893,11 @@ svr4_stream_ti_ioctl(fp, td, retval, fd, cmd, dat)
 
 	switch (st->s_family) {
 	case AF_INET:
-		skp = &sain;
-		sasize = sizeof(sain);
+		sasize = sizeof(struct sockaddr_in);
 		break;
 
 	case AF_LOCAL:
-		skp = &saun;
-		sasize = sizeof(saun);
+		sasize = sizeof(struct sockaddr_un);
 		break;
 
 	default:
@@ -1155,25 +905,14 @@ svr4_stream_ti_ioctl(fp, td, retval, fd, cmd, dat)
 			 st->s_family));
 		return ENOSYS;
 	}
-
-	sg = stackgap_init();
-	sup = stackgap_alloc(&sg, sasize);
-	lenp = stackgap_alloc(&sg, sizeof(*lenp));
-
-	if ((error = copyout(&sasize, lenp, sizeof(*lenp))) != 0) {
-		DPRINTF(("ti_ioctl: error copying out lenp\n"));
-		return error;
-	}
+	oldsasize = sasize;
 
 	switch (cmd) {
 	case SVR4_TI_GETMYNAME:
 		DPRINTF(("TI_GETMYNAME\n"));
 		{
-			struct getsockname_args ap;
-			ap.fdes = fd;
-			ap.asa = sup;
-			ap.alen = lenp;
-			if ((error = getsockname(td, &ap)) != 0) {
+			error = kern_getsockname(td, fd, &sa, &sasize);
+			if (error) {
 				DPRINTF(("ti_ioctl: getsockname error\n"));
 				return error;
 			}
@@ -1183,11 +922,8 @@ svr4_stream_ti_ioctl(fp, td, retval, fd, cmd, dat)
 	case SVR4_TI_GETPEERNAME:
 		DPRINTF(("TI_GETPEERNAME\n"));
 		{
-			struct getpeername_args ap;
-			ap.fdes = fd;
-			ap.asa = sup;
-			ap.alen = lenp;
-			if ((error = getpeername(td, &ap)) != 0) {
+			error = kern_getpeername(td, fd, &sa, &sasize);
+			if (error) {
 				DPRINTF(("ti_ioctl: getpeername error\n"));
 				return error;
 			}
@@ -1206,36 +942,27 @@ svr4_stream_ti_ioctl(fp, td, retval, fd, cmd, dat)
 		return ENOSYS;
 	}
 
-	if ((error = copyin(sup, skp, sasize)) != 0) {
-		DPRINTF(("ti_ioctl: error copying in socket data\n"));
-		return error;
-	}
-
-	oldsasize = sasize;
-
-	if ((error = copyin(lenp, &sasize, sizeof(*lenp))) != 0) {
-		DPRINTF(("ti_ioctl: error copying in socket size\n"));
-		return error;
-	}
-
-	if (sasize < 0 || sasize > oldsasize)
+	if (sasize < 0 || sasize > oldsasize) {
+		free(sa, M_SONAME);
 		return EINVAL;
+	}
 
 	switch (st->s_family) {
 	case AF_INET:
-		sockaddr_to_netaddr_in(&sc, &sain);
+		sockaddr_to_netaddr_in(&sc, (struct sockaddr_in *)sa);
 		skb.len = sasize;
 		break;
 
 	case AF_LOCAL:
-		sockaddr_to_netaddr_un(&sc, &saun);
+		sockaddr_to_netaddr_un(&sc, (struct sockaddr_un *)sa);
 		skb.len = sasize + 4;
 		break;
 
 	default:
+		free(sa, M_SONAME);
 		return ENOSYS;
 	}
-
+	free(sa, M_SONAME);
 
 	if ((error = copyout(SVR4_ADDROF(&sc), skb.buf, sasize)) != 0) {
 		DPRINTF(("ti_ioctl: error copying out socket data\n"));
@@ -1306,20 +1033,22 @@ i_fdinsert(fp, td, retval, fd, cmd, dat)
 	struct svr4_strm *st = svr4_stream_get(fp);
 	struct svr4_strfdinsert fdi;
 	struct dup2_args d2p;
-	struct close_args clp;
 
 	if (st == NULL) {
 		DPRINTF(("fdinsert: bad file type\n"));
 		return EINVAL;
 	}
 
+	mtx_lock(&Giant);
 	if (st->s_afd == -1) {
 		DPRINTF(("fdinsert: accept fd not found\n"));
+		mtx_unlock(&Giant);
 		return ENOENT;
 	}
 
 	if ((error = copyin(dat, &fdi, sizeof(fdi))) != 0) {
 		DPRINTF(("fdinsert: copyin failed %d\n", error));
+		mtx_unlock(&Giant);
 		return error;
 	}
 
@@ -1329,18 +1058,19 @@ i_fdinsert(fp, td, retval, fd, cmd, dat)
 	if ((error = dup2(td, &d2p)) != 0) {
 		DPRINTF(("fdinsert: dup2(%d, %d) failed %d\n", 
 		    st->s_afd, fdi.fd, error));
+		mtx_unlock(&Giant);
 		return error;
 	}
 
-	clp.fd = st->s_afd;
-
-	if ((error = close(td, &clp)) != 0) {
+	if ((error = kern_close(td, st->s_afd)) != 0) {
 		DPRINTF(("fdinsert: close(%d) failed %d\n", 
 		    st->s_afd, error));
+		mtx_unlock(&Giant);
 		return error;
 	}
 
 	st->s_afd = -1;
+	mtx_unlock(&Giant);
 
 	*retval = 0;
 	return 0;
@@ -1468,6 +1198,7 @@ i_setsig(fp, td, retval, fd, cmd, dat)
 	oflags = td->td_retval[0];
 
 	/* update the flags */
+	mtx_lock(&Giant);
 	if (dat != NULL) {
 		int mask;
 
@@ -1486,6 +1217,7 @@ i_setsig(fp, td, retval, fd, cmd, dat)
 		flags = oflags & ~O_ASYNC;
 		st->s_eventmask = 0;
 	}
+	mtx_unlock(&Giant);
 
 	/* set the new flags, if changed */
 	if (flags != oflags) {
@@ -1510,7 +1242,7 @@ i_getsig(fp, td, retval, fd, cmd, dat)
 	u_long cmd;
 	caddr_t dat;
 {
-	int error;
+	int error, eventmask;
 
 	if (dat != NULL) {
 		struct svr4_strm *st = svr4_stream_get(fp);
@@ -1519,8 +1251,11 @@ i_getsig(fp, td, retval, fd, cmd, dat)
 			DPRINTF(("i_getsig: bad file descriptor\n"));
 			return EINVAL;
 		}
-		if ((error = copyout(&st->s_eventmask, dat, 
-				     sizeof(st->s_eventmask))) != 0) {
+		mtx_lock(&Giant);
+		eventmask = st->s_eventmask;
+		mtx_unlock(&Giant);		
+		if ((error = copyout(&eventmask, dat,
+				     sizeof(eventmask))) != 0) {
 			DPRINTF(("i_getsig: bad eventmask pointer\n"));
 			return error;
 		}
@@ -1734,11 +1469,10 @@ svr4_do_putmsg(td, uap, fp)
 	struct svr4_strmcmd sc;
 	struct sockaddr_in sain;
 	struct sockaddr_un saun;
-	void *skp, *sup;
+	struct sockaddr *sa;
 	int sasize, *retval;
 	struct svr4_strm *st;
 	int error;
-	caddr_t sg;
 
 	retval = td->td_retval;
 
@@ -1812,9 +1546,10 @@ svr4_do_putmsg(td, uap, fp)
 	                return EINVAL;
 	        }
 	        netaddr_to_sockaddr_in(&sain, &sc);
-	        skp = &sain;
+		sa = (struct sockaddr *)&sain;
 	        sasize = sizeof(sain);
-	        error = sain.sin_family != st->s_family;
+		if (sain.sin_family != st->s_family)
+			error = EINVAL;
 		break;
 
 	case AF_LOCAL:
@@ -1828,12 +1563,11 @@ svr4_do_putmsg(td, uap, fp)
 			/* Maybe we've been given a device/inode pair */
 			dev_t *dev = SVR4_ADDROF(&sc);
 			ino_t *ino = (ino_t *) &dev[1];
-			skp = svr4_find_socket(td, fp, *dev, *ino);
-			if (skp == NULL) {
-				skp = &saun;
+			if (svr4_find_socket(td, fp, *dev, *ino, &saun) != 0) {
 				/* I guess we have it by name */
-				netaddr_to_sockaddr_un(skp, &sc);
+				netaddr_to_sockaddr_un(&saun, &sc);
 			}
+			sa = (struct sockaddr *)&saun;
 			sasize = sizeof(saun);
 		}
 		break;
@@ -1844,22 +1578,14 @@ svr4_do_putmsg(td, uap, fp)
 		return ENOSYS;
 	}
 
-	sg = stackgap_init();
-	sup = stackgap_alloc(&sg, sasize);
-
-	if ((error = copyout(skp, sup, sasize)) != 0)
-		return error;
-
-	switch (st->s_cmd = sc.cmd) {
+	mtx_lock(&Giant);
+	st->s_cmd = sc.cmd;
+	mtx_unlock(&Giant);
+	switch (sc.cmd) {
 	case SVR4_TI_CONNECT_REQUEST:	/* connect 	*/
 		{
-			struct connect_args co;
 
-			co.s = uap->fd;
-			co.name = (void *) sup;
-			co.namelen = (int) sasize;
-			
-			return connect(td, &co);
+			return (kern_connect(td, uap->fd, sa));
 		}
 
 	case SVR4_TI_SENDTO_REQUEST:	/* sendto 	*/
@@ -1867,7 +1593,7 @@ svr4_do_putmsg(td, uap, fp)
 			struct msghdr msg;
 			struct iovec aiov;
 
-			msg.msg_name = (caddr_t) sup;
+			msg.msg_name = sa;
 			msg.msg_namelen = sasize;
 			msg.msg_iov = &aiov;
 			msg.msg_iovlen = 1;
@@ -1875,12 +1601,8 @@ svr4_do_putmsg(td, uap, fp)
 			msg.msg_flags = 0;
 			aiov.iov_base = dat.buf;
 			aiov.iov_len = dat.len;
-#if 0
-			error = so->so_proto->pr_usrreqs->pru_sosend(so, 0, 
-					      uio, 0, 0, 0, uio->uio_td);
-#endif
-			error = svr4_sendit(td, uap->fd, &msg,
-				       uap->flags);
+			error = kern_sendit(td, uap->fd, &msg, uap->flags,
+			    NULL, UIO_USERSPACE);
 			DPRINTF(("sendto_request error: %d\n", error));
 			*retval = 0;
 			return error;
@@ -1917,8 +1639,6 @@ svr4_do_getmsg(td, uap, fp)
 	struct svr4_sys_getmsg_args *uap;
 	struct file *fp;
 {
-	struct getpeername_args ga;
-	struct accept_args aa;
 	struct svr4_strbuf dat, ctl;
 	struct svr4_strmcmd sc;
 	int error, *retval;
@@ -1926,14 +1646,15 @@ svr4_do_getmsg(td, uap, fp)
 	struct iovec aiov;
 	struct sockaddr_in sain;
 	struct sockaddr_un saun;
-	void *skp, *sup;
-	int sasize;
+	struct sockaddr *sa;
+	socklen_t sasize;
 	struct svr4_strm *st;
-	int *flen;
+	struct file *afp;
 	int fl;
-	caddr_t sg;
 
 	retval = td->td_retval;
+	error = 0;
+	afp = NULL;
 
 	FILE_LOCK_ASSERT(fp, MA_NOTOWNED);
 
@@ -1979,12 +1700,10 @@ svr4_do_getmsg(td, uap, fp)
 
 	switch (st->s_family) {
 	case AF_INET:
-		skp = &sain;
 		sasize = sizeof(sain);
 		break;
 
 	case AF_LOCAL:
-		skp = &saun;
 		sasize = sizeof(saun);
 		break;
 
@@ -1994,14 +1713,7 @@ svr4_do_getmsg(td, uap, fp)
 		return ENOSYS;
 	}
 
-	sg = stackgap_init();
-	sup = stackgap_alloc(&sg, sasize);
-	flen = (int *) stackgap_alloc(&sg, sizeof(*flen));
-
-	fl = sasize;
-	if ((error = copyout(&fl, flen, sizeof(fl))) != 0)
-		return error;
-
+	mtx_lock(&Giant);
 	switch (st->s_cmd) {
 	case SVR4_TI_CONNECT_REQUEST:
 		DPRINTF(("getmsg: TI_CONNECT_REQUEST\n"));
@@ -2025,18 +1737,13 @@ svr4_do_getmsg(td, uap, fp)
 		 * a connect verification.
 		 */
 
-		ga.fdes = uap->fd;
-		ga.asa = (void *) sup;
-		ga.alen = flen;
-		
-		if ((error = getpeername(td, &ga)) != 0) {
+		error = kern_getpeername(td, uap->fd, &sa, &sasize);
+		if (error) {
+			mtx_unlock(&Giant);
 			DPRINTF(("getmsg: getpeername failed %d\n", error));
 			return error;
 		}
 
-		if ((error = copyin(sup, skp, sasize)) != 0)
-			return error;
-		
 		sc.cmd = SVR4_TI_CONNECT_REPLY;
 		sc.pad[0] = 0x4;
 		sc.offs = 0x18;
@@ -2046,17 +1753,20 @@ svr4_do_getmsg(td, uap, fp)
 		switch (st->s_family) {
 		case AF_INET:
 			sc.len = sasize;
-			sockaddr_to_netaddr_in(&sc, &sain);
+			sockaddr_to_netaddr_in(&sc, (struct sockaddr_in *)sa);
 			break;
 
 		case AF_LOCAL:
 			sc.len = sasize + 4;
-			sockaddr_to_netaddr_un(&sc, &saun);
+			sockaddr_to_netaddr_un(&sc, (struct sockaddr_un *)sa);
 			break;
 
 		default:
+			mtx_unlock(&Giant);
+			free(sa, M_SONAME);
 			return ENOSYS;
 		}
+		free(sa, M_SONAME);
 
 		ctl.len = 40;
 		dat.len = -1;
@@ -2084,11 +1794,10 @@ svr4_do_getmsg(td, uap, fp)
 		/*
 		 * We are after a listen, so we try to accept...
 		 */
-		aa.s = uap->fd;
-		aa.name = (void *) sup;
-		aa.anamelen = flen;
-		
-		if ((error = accept(td, &aa)) != 0) {
+
+		error = kern_accept(td, uap->fd, &sa, &sasize, &afp);
+		if (error) {
+			mtx_unlock(&Giant);
 			DPRINTF(("getmsg: accept failed %d\n", error));
 			return error;
 		}
@@ -2097,9 +1806,6 @@ svr4_do_getmsg(td, uap, fp)
 
 		DPRINTF(("getmsg: Accept fd = %d\n", st->s_afd));
 
-		if ((error = copyin(sup, skp, sasize)) != 0)
-			return error;
-		
 		sc.cmd = SVR4_TI_ACCEPT_REPLY;
 		sc.offs = 0x18;
 		sc.pad[0] = 0x0;
@@ -2107,7 +1813,7 @@ svr4_do_getmsg(td, uap, fp)
 		switch (st->s_family) {
 		case AF_INET:
 			sc.pad[1] = 0x28;
-			sockaddr_to_netaddr_in(&sc, &sain);
+			sockaddr_to_netaddr_in(&sc, (struct sockaddr_in *)&sa);
 			ctl.len = 40;
 			sc.len = sasize;
 			break;
@@ -2121,8 +1827,14 @@ svr4_do_getmsg(td, uap, fp)
 			break;
 
 		default:
+			fdclose(td->td_proc->p_fd, afp, st->s_afd, td);
+			fdrop(afp, td);
+			st->s_afd = -1;
+			mtx_unlock(&Giant);
+			free(sa, M_SONAME);
 			return ENOSYS;
 		}
+		free(sa, M_SONAME);
 
 		dat.len = -1;
 		fl = 0;
@@ -2137,23 +1849,28 @@ svr4_do_getmsg(td, uap, fp)
 		if (ctl.len > sizeof(sc))
 			ctl.len = sizeof(sc);
 
-		if ((error = copyin(ctl.buf, &sc, ctl.len)) != 0)
+		if ((error = copyin(ctl.buf, &sc, ctl.len)) != 0) {
+			mtx_unlock(&Giant);
 			return error;
+		}
 
 		switch (st->s_family) {
 		case AF_INET:
+			sa = (struct sockaddr *)&sain;
 			sockaddr_to_netaddr_in(&sc, &sain);
 			break;
 
 		case AF_LOCAL:
+			sa = (struct sockaddr *)&saun;
 			sockaddr_to_netaddr_un(&sc, &saun);
 			break;
 
 		default:
+			mtx_unlock(&Giant);
 			return ENOSYS;
 		}
 
-		msg.msg_name = (caddr_t) sup;
+		msg.msg_name = sa;
 		msg.msg_namelen = sasize;
 		msg.msg_iov = &aiov;
 		msg.msg_iovlen = 1;
@@ -2162,16 +1879,14 @@ svr4_do_getmsg(td, uap, fp)
 		aiov.iov_len = dat.maxlen;
 		msg.msg_flags = 0;
 
-		error = svr4_recvit(td, uap->fd, &msg, (caddr_t) flen);
+		error = kern_recvit(td, uap->fd, &msg, UIO_SYSSPACE, NULL);
 
 		if (error) {
+			mtx_unlock(&Giant);
 			DPRINTF(("getmsg: recvit failed %d\n", error));
 			return error;
 		}
 
-		if ((error = copyin(msg.msg_name, skp, sasize)) != 0)
-			return error;
-
 		sc.cmd = SVR4_TI_RECVFROM_IND;
 
 		switch (st->s_family) {
@@ -2186,6 +1901,7 @@ svr4_do_getmsg(td, uap, fp)
 			break;
 
 		default:
+			mtx_unlock(&Giant);
 			return ENOSYS;
 		}
 
@@ -2214,6 +1930,7 @@ svr4_do_getmsg(td, uap, fp)
 			ra.buf = dat.buf;
 			ra.nbyte = dat.maxlen;
 			if ((error = read(td, &ra)) != 0) {
+				mtx_unlock(&Giant);
 			        return error;
 			}
 			dat.len = *retval;
@@ -2221,6 +1938,7 @@ svr4_do_getmsg(td, uap, fp)
 			st->s_cmd = SVR4_TI_SENDTO_REQUEST;
 			break;
 		}
+		mtx_unlock(&Giant);
 		DPRINTF(("getmsg: Unknown state %x\n", st->s_cmd));
 		return EINVAL;
 	}
@@ -2229,23 +1947,35 @@ svr4_do_getmsg(td, uap, fp)
 		if (ctl.len > sizeof(sc))
 			ctl.len = sizeof(sc);
 		if (ctl.len != -1)
-			if ((error = copyout(&sc, ctl.buf, ctl.len)) != 0)
-				return error;
+			error = copyout(&sc, ctl.buf, ctl.len);
 
-		if ((error = copyout(&ctl, uap->ctl, sizeof(ctl))) != 0)
-			return error;
+		if (error == 0)
+			error = copyout(&ctl, uap->ctl, sizeof(ctl));
 	}
 
 	if (uap->dat) {
-		if ((error = copyout(&dat, uap->dat, sizeof(dat))) != 0)
-			return error;
+		if (error == 0)
+			error = copyout(&dat, uap->dat, sizeof(dat));
 	}
 
 	if (uap->flags) { /* XXX: Need translation */
-		if ((error = copyout(&fl, uap->flags, sizeof(fl))) != 0)
-			return error;
+		if (error == 0)
+			error = copyout(&fl, uap->flags, sizeof(fl));
 	}
 
+	if (error) {
+		if (afp) {
+			fdclose(td->td_proc->p_fd, afp, st->s_afd, td);
+			fdrop(afp, td);
+			st->s_afd = -1;
+		}
+		mtx_unlock(&Giant);
+		return (error);
+	}
+	mtx_unlock(&Giant);
+	if (afp)
+		fdrop(afp, td);
+
 	*retval = 0;
 
 #ifdef DEBUG_SVR4
