@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * Module Name: evrgnini- ACPI AddressSpace (OpRegion) init
- *              $Revision: 74 $
+ *              $Revision: 1.88 $
  *
  *****************************************************************************/
 
@@ -9,7 +9,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2004, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2007, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -117,13 +117,23 @@
 
 #define __EVRGNINI_C__
 
-#include "acpi.h"
-#include "acevents.h"
-#include "acnamesp.h"
+#include <contrib/dev/acpica/acpi.h>
+#include <contrib/dev/acpica/acevents.h>
+#include <contrib/dev/acpica/acnamesp.h>
 
 #define _COMPONENT          ACPI_EVENTS
         ACPI_MODULE_NAME    ("evrgnini")
 
+/* Local prototypes */
+
+static BOOLEAN
+AcpiEvMatchPciRootBridge (
+    char                    *Id);
+
+static BOOLEAN
+AcpiEvIsPciRootBridge (
+    ACPI_NAMESPACE_NODE     *Node);
+
 
 /*******************************************************************************
  *
@@ -136,7 +146,7 @@
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Do any prep work for region handling, a nop for now
+ * DESCRIPTION: Setup a SystemMemory operation region
  *
  ******************************************************************************/
 
@@ -151,14 +161,23 @@ AcpiEvSystemMemoryRegionSetup (
     ACPI_MEM_SPACE_CONTEXT  *LocalRegionContext;
 
 
-    ACPI_FUNCTION_TRACE ("EvSystemMemoryRegionSetup");
+    ACPI_FUNCTION_TRACE (EvSystemMemoryRegionSetup);
 
 
     if (Function == ACPI_REGION_DEACTIVATE)
     {
         if (*RegionContext)
         {
-            ACPI_MEM_FREE (*RegionContext);
+            LocalRegionContext = (ACPI_MEM_SPACE_CONTEXT *) *RegionContext;
+
+            /* Delete a cached mapping if present */
+
+            if (LocalRegionContext->MappedLength)
+            {
+                AcpiOsUnmapMemory (LocalRegionContext->MappedLogicalAddress,
+                    LocalRegionContext->MappedLength);
+            }
+            ACPI_FREE (LocalRegionContext);
             *RegionContext = NULL;
         }
         return_ACPI_STATUS (AE_OK);
@@ -166,7 +185,7 @@ AcpiEvSystemMemoryRegionSetup (
 
     /* Create a new context */
 
-    LocalRegionContext = ACPI_MEM_CALLOCATE (sizeof (ACPI_MEM_SPACE_CONTEXT));
+    LocalRegionContext = ACPI_ALLOCATE_ZEROED (sizeof (ACPI_MEM_SPACE_CONTEXT));
     if (!(LocalRegionContext))
     {
         return_ACPI_STATUS (AE_NO_MEMORY);
@@ -193,7 +212,7 @@ AcpiEvSystemMemoryRegionSetup (
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Do any prep work for region handling
+ * DESCRIPTION: Setup a IO operation region
  *
  ******************************************************************************/
 
@@ -204,7 +223,7 @@ AcpiEvIoSpaceRegionSetup (
     void                    *HandlerContext,
     void                    **RegionContext)
 {
-    ACPI_FUNCTION_TRACE ("EvIoSpaceRegionSetup");
+    ACPI_FUNCTION_TRACE (EvIoSpaceRegionSetup);
 
 
     if (Function == ACPI_REGION_DEACTIVATE)
@@ -224,14 +243,14 @@ AcpiEvIoSpaceRegionSetup (
  *
  * FUNCTION:    AcpiEvPciConfigRegionSetup
  *
- * PARAMETERS:  Handle             - Region we are interested in
+ * PARAMETERS:  Handle              - Region we are interested in
  *              Function            - Start or stop
  *              HandlerContext      - Address space handler context
  *              RegionContext       - Region specific context
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Do any prep work for region handling
+ * DESCRIPTION: Setup a PCI_Config operation region
  *
  * MUTEX:       Assumes namespace is not locked
  *
@@ -250,11 +269,11 @@ AcpiEvPciConfigRegionSetup (
     ACPI_OPERAND_OBJECT     *HandlerObj;
     ACPI_NAMESPACE_NODE     *ParentNode;
     ACPI_NAMESPACE_NODE     *PciRootNode;
+    ACPI_NAMESPACE_NODE     *PciDeviceNode;
     ACPI_OPERAND_OBJECT     *RegionObj = (ACPI_OPERAND_OBJECT  *) Handle;
-    ACPI_DEVICE_ID          ObjectHID;
 
 
-    ACPI_FUNCTION_TRACE ("EvPciConfigRegionSetup");
+    ACPI_FUNCTION_TRACE (EvPciConfigRegionSetup);
 
 
     HandlerObj = RegionObj->Region.Handler;
@@ -274,7 +293,7 @@ AcpiEvPciConfigRegionSetup (
     {
         if (PciId)
         {
-            ACPI_MEM_FREE (PciId);
+            ACPI_FREE (PciId);
         }
         return_ACPI_STATUS (Status);
     }
@@ -301,39 +320,35 @@ AcpiEvPciConfigRegionSetup (
         PciRootNode = ParentNode;
         while (PciRootNode != AcpiGbl_RootNode)
         {
-            Status = AcpiUtExecute_HID (PciRootNode, &ObjectHID);
-            if (ACPI_SUCCESS (Status))
+            /* Get the _HID/_CID in order to detect a RootBridge */
+
+            if (AcpiEvIsPciRootBridge (PciRootNode))
             {
-                /* Got a valid _HID, check if this is a PCI root */
+                /* Install a handler for this PCI root bridge */
 
-                if (!(ACPI_STRNCMP (ObjectHID.Value, PCI_ROOT_HID_STRING,
-                                    sizeof (PCI_ROOT_HID_STRING))))
+                Status = AcpiInstallAddressSpaceHandler (
+                            (ACPI_HANDLE) PciRootNode,
+                            ACPI_ADR_SPACE_PCI_CONFIG,
+                            ACPI_DEFAULT_HANDLER, NULL, NULL);
+                if (ACPI_FAILURE (Status))
                 {
-                    /* Install a handler for this PCI root bridge */
-
-                    Status = AcpiInstallAddressSpaceHandler ((ACPI_HANDLE) PciRootNode,
-                                        ACPI_ADR_SPACE_PCI_CONFIG,
-                                        ACPI_DEFAULT_HANDLER, NULL, NULL);
-                    if (ACPI_FAILURE (Status))
+                    if (Status == AE_SAME_HANDLER)
                     {
-                        if (Status == AE_SAME_HANDLER)
-                        {
-                            /*
-                             * It is OK if the handler is already installed on the root
-                             * bridge.  Still need to return a context object for the
-                             * new PCI_Config operation region, however.
-                             */
-                            Status = AE_OK;
-                        }
-                        else
-                        {
-                            ACPI_REPORT_ERROR ((
-                                "Could not install PciConfig handler for Root Bridge %4.4s, %s\n",
-                                AcpiUtGetNodeName (PciRootNode), AcpiFormatException (Status)));
-                        }
+                        /*
+                         * It is OK if the handler is already installed on the root
+                         * bridge.  Still need to return a context object for the
+                         * new PCI_Config operation region, however.
+                         */
+                        Status = AE_OK;
+                    }
+                    else
+                    {
+                        ACPI_EXCEPTION ((AE_INFO, Status,
+                            "Could not install PciConfig handler for Root Bridge %4.4s",
+                            AcpiUtGetNodeName (PciRootNode)));
                     }
-                    break;
                 }
+                break;
             }
 
             PciRootNode = AcpiNsGetParentNode (PciRootNode);
@@ -357,7 +372,7 @@ AcpiEvPciConfigRegionSetup (
 
     /* Region is still not initialized. Create a new context */
 
-    PciId = ACPI_MEM_CALLOCATE (sizeof (ACPI_PCI_ID));
+    PciId = ACPI_ALLOCATE_ZEROED (sizeof (ACPI_PCI_ID));
     if (!PciId)
     {
         return_ACPI_STATUS (AE_NO_MEMORY);
@@ -366,13 +381,26 @@ AcpiEvPciConfigRegionSetup (
     /*
      * For PCI_Config space access, we need the segment, bus,
      * device and function numbers.  Acquire them here.
+     *
+     * Find the parent device object. (This allows the operation region to be
+     * within a subscope under the device, such as a control method.)
      */
+    PciDeviceNode = RegionObj->Region.Node;
+    while (PciDeviceNode && (PciDeviceNode->Type != ACPI_TYPE_DEVICE))
+    {
+        PciDeviceNode = AcpiNsGetParentNode (PciDeviceNode);
+    }
+
+    if (!PciDeviceNode)
+    {
+        return_ACPI_STATUS (AE_AML_OPERAND_TYPE);
+    }
 
     /*
      * Get the PCI device and function numbers from the _ADR object
      * contained in the parent's scope.
      */
-    Status = AcpiUtEvaluateNumericObject (METHOD_NAME__ADR, ParentNode, &PciValue);
+    Status = AcpiUtEvaluateNumericObject (METHOD_NAME__ADR, PciDeviceNode, &PciValue);
 
     /*
      * The default is zero, and since the allocation above zeroed
@@ -409,6 +437,105 @@ AcpiEvPciConfigRegionSetup (
 }
 
 
+/*******************************************************************************
+ *
+ * FUNCTION:    AcpiEvMatchPciRootBridge
+ *
+ * PARAMETERS:  Id              - The HID/CID in string format
+ *
+ * RETURN:      TRUE if the Id is a match for a PCI/PCI-Express Root Bridge
+ *
+ * DESCRIPTION: Determine if the input ID is a PCI Root Bridge ID.
+ *
+ ******************************************************************************/
+
+static BOOLEAN
+AcpiEvMatchPciRootBridge (
+    char                    *Id)
+{
+
+    /*
+     * Check if this is a PCI root.
+     * ACPI 3.0+: check for a PCI Express root also.
+     */
+    if (!(ACPI_STRNCMP (Id,
+            PCI_ROOT_HID_STRING,
+            sizeof (PCI_ROOT_HID_STRING)))      ||
+
+        !(ACPI_STRNCMP (Id,
+            PCI_EXPRESS_ROOT_HID_STRING,
+            sizeof (PCI_EXPRESS_ROOT_HID_STRING))))
+    {
+        return (TRUE);
+    }
+
+    return (FALSE);
+}
+
+
+/*******************************************************************************
+ *
+ * FUNCTION:    AcpiEvIsPciRootBridge
+ *
+ * PARAMETERS:  Node            - Device node being examined
+ *
+ * RETURN:      TRUE if device is a PCI/PCI-Express Root Bridge
+ *
+ * DESCRIPTION: Determine if the input device represents a PCI Root Bridge by
+ *              examining the _HID and _CID for the device.
+ *
+ ******************************************************************************/
+
+static BOOLEAN
+AcpiEvIsPciRootBridge (
+    ACPI_NAMESPACE_NODE     *Node)
+{
+    ACPI_STATUS             Status;
+    ACPI_DEVICE_ID          Hid;
+    ACPI_COMPATIBLE_ID_LIST *Cid;
+    ACPI_NATIVE_UINT        i;
+
+
+    /*
+     * Get the _HID and check for a PCI Root Bridge
+     */
+    Status = AcpiUtExecute_HID (Node, &Hid);
+    if (ACPI_FAILURE (Status))
+    {
+        return (FALSE);
+    }
+
+    if (AcpiEvMatchPciRootBridge (Hid.Value))
+    {
+        return (TRUE);
+    }
+
+    /*
+     * The _HID did not match.
+     * Get the _CID and check for a PCI Root Bridge
+     */
+    Status = AcpiUtExecute_CID (Node, &Cid);
+    if (ACPI_FAILURE (Status))
+    {
+        return (FALSE);
+    }
+
+    /* Check all _CIDs in the returned list */
+
+    for (i = 0; i < Cid->Count; i++)
+    {
+        if (AcpiEvMatchPciRootBridge (Cid->Id[i].Value))
+        {
+            ACPI_FREE (Cid);
+            return (TRUE);
+        }
+    }
+
+    ACPI_FREE (Cid);
+    return (FALSE);
+}
+
+
 /*******************************************************************************
  *
  * FUNCTION:    AcpiEvPciBarRegionSetup
@@ -420,7 +547,7 @@ AcpiEvPciConfigRegionSetup (
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Do any prep work for region handling
+ * DESCRIPTION: Setup a PciBAR operation region
  *
  * MUTEX:       Assumes namespace is not locked
  *
@@ -433,7 +560,7 @@ AcpiEvPciBarRegionSetup (
     void                    *HandlerContext,
     void                    **RegionContext)
 {
-    ACPI_FUNCTION_TRACE ("EvPciBarRegionSetup");
+    ACPI_FUNCTION_TRACE (EvPciBarRegionSetup);
 
 
     return_ACPI_STATUS (AE_OK);
@@ -451,7 +578,7 @@ AcpiEvPciBarRegionSetup (
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Do any prep work for region handling
+ * DESCRIPTION: Setup a CMOS operation region
  *
  * MUTEX:       Assumes namespace is not locked
  *
@@ -464,7 +591,7 @@ AcpiEvCmosRegionSetup (
     void                    *HandlerContext,
     void                    **RegionContext)
 {
-    ACPI_FUNCTION_TRACE ("EvCmosRegionSetup");
+    ACPI_FUNCTION_TRACE (EvCmosRegionSetup);
 
 
     return_ACPI_STATUS (AE_OK);
@@ -482,7 +609,7 @@ AcpiEvCmosRegionSetup (
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Do any prep work for region handling
+ * DESCRIPTION: Default region initialization
  *
  ******************************************************************************/
 
@@ -493,7 +620,7 @@ AcpiEvDefaultRegionSetup (
     void                    *HandlerContext,
     void                    **RegionContext)
 {
-    ACPI_FUNCTION_TRACE ("EvDefaultRegionSetup");
+    ACPI_FUNCTION_TRACE (EvDefaultRegionSetup);
 
 
     if (Function == ACPI_REGION_DEACTIVATE)
@@ -529,6 +656,9 @@ AcpiEvDefaultRegionSetup (
  *              a PCI address in the scope of the definition.  This address is
  *              required to perform an access to PCI config space.
  *
+ * MUTEX:       Interpreter should be unlocked, because we may run the _REG
+ *              method for this region.
+ *
  ******************************************************************************/
 
 ACPI_STATUS
@@ -546,7 +676,7 @@ AcpiEvInitializeRegion (
     ACPI_OPERAND_OBJECT     *RegionObj2;
 
 
-    ACPI_FUNCTION_TRACE_U32 ("EvInitializeRegion", AcpiNsLocked);
+    ACPI_FUNCTION_TRACE_U32 (EvInitializeRegion, AcpiNsLocked);
 
 
     if (!RegionObj)
@@ -577,8 +707,8 @@ AcpiEvInitializeRegion (
 
     /* Find any "_REG" method associated with this region definition */
 
-    Status = AcpiNsSearchNode (*RegNamePtr, Node,
-                                ACPI_TYPE_METHOD, &MethodNode);
+    Status = AcpiNsSearchOneScope (
+                *RegNamePtr, Node, ACPI_TYPE_METHOD, &MethodNode);
     if (ACPI_SUCCESS (Status))
     {
         /*
