@@ -2,7 +2,7 @@
 /******************************************************************************
  *
  * Name: hwsleep.c - ACPI Hardware Sleep/Wake Interface
- *              $Revision: 70 $
+ *              $Revision: 1.87 $
  *
  *****************************************************************************/
 
@@ -10,7 +10,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2004, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2007, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -115,26 +115,14 @@
  *
  *****************************************************************************/
 
-#include "acpi.h"
+#include <contrib/dev/acpica/acpi.h>
+#include <contrib/dev/acpica/actables.h>
 
 #define _COMPONENT          ACPI_HARDWARE
         ACPI_MODULE_NAME    ("hwsleep")
 
 
-#define METHOD_NAME__BFS        "\\_BFS"
-#define METHOD_NAME__GTS        "\\_GTS"
-#define METHOD_NAME__PTS        "\\_PTS"
-#define METHOD_NAME__SST        "\\_SI._SST"
-#define METHOD_NAME__WAK        "\\_WAK"
-
-#define ACPI_SST_INDICATOR_OFF  0
-#define ACPI_SST_WORKING        1
-#define ACPI_SST_WAKING         2
-#define ACPI_SST_SLEEPING       3
-#define ACPI_SST_SLEEP_CONTEXT  4
-
-
-/******************************************************************************
+/*******************************************************************************
  *
  * FUNCTION:    AcpiSetFirmwareWakingVector
  *
@@ -143,55 +131,76 @@
  *
  * RETURN:      Status
  *
- * DESCRIPTION: Access function for dFirmwareWakingVector field in FACS
+ * DESCRIPTION: Access function for the FirmwareWakingVector field in FACS
  *
  ******************************************************************************/
 
 ACPI_STATUS
 AcpiSetFirmwareWakingVector (
-    ACPI_PHYSICAL_ADDRESS PhysicalAddress)
+    ACPI_PHYSICAL_ADDRESS   PhysicalAddress)
 {
+    ACPI_TABLE_FACS         *Facs;
+    ACPI_STATUS             Status;
 
-    ACPI_FUNCTION_TRACE ("AcpiSetFirmwareWakingVector");
 
+    ACPI_FUNCTION_TRACE (AcpiSetFirmwareWakingVector);
+
+    /* Get the FACS */
+
+    Status = AcpiGetTableByIndex (ACPI_TABLE_INDEX_FACS,
+        ACPI_CAST_INDIRECT_PTR (ACPI_TABLE_HEADER, &Facs));
+    if (ACPI_FAILURE (Status))
+    {
+        return_ACPI_STATUS (Status);
+    }
 
     /* Set the vector */
 
-    if (AcpiGbl_CommonFACS.VectorWidth == 32)
+    if ((Facs->Length < 32) ||
+        (!(Facs->XFirmwareWakingVector)))
     {
-        *(ACPI_CAST_PTR (UINT32, AcpiGbl_CommonFACS.FirmwareWakingVector))
-                = (UINT32) PhysicalAddress;
+        /*
+         * ACPI 1.0 FACS or short table or optional X_ field is zero
+         */
+        Facs->FirmwareWakingVector = (UINT32) PhysicalAddress;
     }
     else
     {
-        *AcpiGbl_CommonFACS.FirmwareWakingVector
-                = PhysicalAddress;
+        /*
+         * ACPI 2.0 FACS with valid X_ field
+         */
+        Facs->XFirmwareWakingVector = PhysicalAddress;
     }
 
     return_ACPI_STATUS (AE_OK);
 }
 
+ACPI_EXPORT_SYMBOL (AcpiSetFirmwareWakingVector)
 
-/******************************************************************************
+
+/*******************************************************************************
  *
  * FUNCTION:    AcpiGetFirmwareWakingVector
  *
- * PARAMETERS:  *PhysicalAddress    - Output buffer where contents of
+ * PARAMETERS:  *PhysicalAddress    - Where the contents of
  *                                    the FirmwareWakingVector field of
- *                                    the FACS will be stored.
+ *                                    the FACS will be returned.
  *
- * RETURN:      Status
+ * RETURN:      Status, vector
  *
- * DESCRIPTION: Access function for FirmwareWakingVector field in FACS
+ * DESCRIPTION: Access function for the FirmwareWakingVector field in FACS
  *
  ******************************************************************************/
 
 ACPI_STATUS
 AcpiGetFirmwareWakingVector (
-    ACPI_PHYSICAL_ADDRESS *PhysicalAddress)
+    ACPI_PHYSICAL_ADDRESS   *PhysicalAddress)
 {
+    ACPI_TABLE_FACS         *Facs;
+    ACPI_STATUS             Status;
 
-    ACPI_FUNCTION_TRACE ("AcpiGetFirmwareWakingVector");
+
+    ACPI_FUNCTION_TRACE (AcpiGetFirmwareWakingVector);
 
 
     if (!PhysicalAddress)
@@ -199,24 +208,41 @@ AcpiGetFirmwareWakingVector (
         return_ACPI_STATUS (AE_BAD_PARAMETER);
     }
 
+    /* Get the FACS */
+
+    Status = AcpiGetTableByIndex (ACPI_TABLE_INDEX_FACS,
+        ACPI_CAST_INDIRECT_PTR (ACPI_TABLE_HEADER, &Facs));
+    if (ACPI_FAILURE (Status))
+    {
+        return_ACPI_STATUS (Status);
+    }
+
     /* Get the vector */
 
-    if (AcpiGbl_CommonFACS.VectorWidth == 32)
+    if ((Facs->Length < 32) ||
+        (!(Facs->XFirmwareWakingVector)))
     {
-        *PhysicalAddress = (ACPI_PHYSICAL_ADDRESS)
-            *(ACPI_CAST_PTR (UINT32, AcpiGbl_CommonFACS.FirmwareWakingVector));
+        /*
+         * ACPI 1.0 FACS or short table or optional X_ field is zero
+         */
+        *PhysicalAddress =
+            (ACPI_PHYSICAL_ADDRESS) Facs->FirmwareWakingVector;
     }
     else
     {
-        *PhysicalAddress =
-            *AcpiGbl_CommonFACS.FirmwareWakingVector;
+        /*
+         * ACPI 2.0 FACS with valid X_ field
+         */
+        *PhysicalAddress = (ACPI_PHYSICAL_ADDRESS) Facs->XFirmwareWakingVector;
     }
 
     return_ACPI_STATUS (AE_OK);
 }
 
+ACPI_EXPORT_SYMBOL (AcpiGetFirmwareWakingVector)
 
-/******************************************************************************
+
+/*******************************************************************************
  *
  * FUNCTION:    AcpiEnterSleepStatePrep
  *
@@ -240,7 +266,7 @@ AcpiEnterSleepStatePrep (
     ACPI_OBJECT         Arg;
 
 
-    ACPI_FUNCTION_TRACE ("AcpiEnterSleepStatePrep");
+    ACPI_FUNCTION_TRACE (AcpiEnterSleepStatePrep);
 
 
     /*
@@ -294,7 +320,7 @@ AcpiEnterSleepStatePrep (
         break;
 
     default:
-        Arg.Integer.Value = ACPI_SST_INDICATOR_OFF; /* Default is indicator off */
+        Arg.Integer.Value = ACPI_SST_INDICATOR_OFF; /* Default is off */
         break;
     }
 
@@ -303,14 +329,16 @@ AcpiEnterSleepStatePrep (
     Status = AcpiEvaluateObject (NULL, METHOD_NAME__SST, &ArgList, NULL);
     if (ACPI_FAILURE (Status) && Status != AE_NOT_FOUND)
     {
-         ACPI_REPORT_ERROR (("Method _SST failed, %s\n", AcpiFormatException (Status)));
+        ACPI_EXCEPTION ((AE_INFO, Status, "While executing method _SST"));
     }
 
     return_ACPI_STATUS (AE_OK);
 }
 
+ACPI_EXPORT_SYMBOL (AcpiEnterSleepStatePrep)
 
-/******************************************************************************
+
+/*******************************************************************************
  *
  * FUNCTION:    AcpiEnterSleepState
  *
@@ -336,13 +364,13 @@ AcpiEnterSleepState (
     ACPI_STATUS             Status;
 
 
-    ACPI_FUNCTION_TRACE ("AcpiEnterSleepState");
+    ACPI_FUNCTION_TRACE (AcpiEnterSleepState);
 
 
     if ((AcpiGbl_SleepTypeA > ACPI_SLEEP_TYPE_MAX) ||
         (AcpiGbl_SleepTypeB > ACPI_SLEEP_TYPE_MAX))
     {
-        ACPI_REPORT_ERROR (("Sleep values out of range: A=%X B=%X\n",
+        ACPI_ERROR ((AE_INFO, "Sleep values out of range: A=%X B=%X",
             AcpiGbl_SleepTypeA, AcpiGbl_SleepTypeB));
         return_ACPI_STATUS (AE_AML_OPERAND_VALUE);
     }
@@ -352,7 +380,7 @@ AcpiEnterSleepState (
 
     /* Clear wake status */
 
-    Status = AcpiSetRegister (ACPI_BITREG_WAKE_STATUS, 1, ACPI_MTX_DO_NOT_LOCK);
+    Status = AcpiSetRegister (ACPI_BITREG_WAKE_STATUS, 1);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -360,7 +388,7 @@ AcpiEnterSleepState (
 
     /* Clear all fixed and general purpose status bits */
 
-    Status = AcpiHwClearAcpiStatus (ACPI_MTX_DO_NOT_LOCK);
+    Status = AcpiHwClearAcpiStatus ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -370,7 +398,7 @@ AcpiEnterSleepState (
     {
         /* Disable BM arbitration */
 
-        Status = AcpiSetRegister (ACPI_BITREG_ARB_DISABLE, 1, ACPI_MTX_DO_NOT_LOCK);
+        Status = AcpiSetRegister (ACPI_BITREG_ARB_DISABLE, 1);
         if (ACPI_FAILURE (Status))
         {
             return_ACPI_STATUS (Status);
@@ -381,14 +409,14 @@ AcpiEnterSleepState (
      * 1) Disable/Clear all GPEs
      * 2) Enable all wakeup GPEs
      */
-    Status = AcpiHwDisableAllGpes (ACPI_ISR);
+    Status = AcpiHwDisableAllGpes ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
     AcpiGbl_SystemAwakeAndRunning = FALSE;
 
-    Status = AcpiHwEnableAllWakeupGpes (ACPI_ISR);
+    Status = AcpiHwEnableAllWakeupGpes ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -396,16 +424,19 @@ AcpiEnterSleepState (
 
     /* Get current value of PM1A control */
 
-    Status = AcpiHwRegisterRead (ACPI_MTX_DO_NOT_LOCK, ACPI_REGISTER_PM1_CONTROL, &PM1AControl);
+    Status = AcpiHwRegisterRead (ACPI_MTX_DO_NOT_LOCK,
+                ACPI_REGISTER_PM1_CONTROL, &PM1AControl);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
-    ACPI_DEBUG_PRINT ((ACPI_DB_INIT, "Entering sleep state [S%d]\n", SleepState));
+    ACPI_DEBUG_PRINT ((ACPI_DB_INIT,
+        "Entering sleep state [S%d]\n", SleepState));
 
     /* Clear SLP_EN and SLP_TYP fields */
 
-    PM1AControl &= ~(SleepTypeRegInfo->AccessBitMask | SleepEnableRegInfo->AccessBitMask);
+    PM1AControl &= ~(SleepTypeRegInfo->AccessBitMask |
+                     SleepEnableRegInfo->AccessBitMask);
     PM1BControl = PM1AControl;
 
     /* Insert SLP_TYP bits */
@@ -420,13 +451,15 @@ AcpiEnterSleepState (
 
     /* Write #1: fill in SLP_TYP data */
 
-    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK, ACPI_REGISTER_PM1A_CONTROL, PM1AControl);
+    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK,
+                ACPI_REGISTER_PM1A_CONTROL, PM1AControl);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
 
-    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK, ACPI_REGISTER_PM1B_CONTROL, PM1BControl);
+    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK,
+                ACPI_REGISTER_PM1B_CONTROL, PM1BControl);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -441,13 +474,15 @@ AcpiEnterSleepState (
 
     ACPI_FLUSH_CPU_CACHE ();
 
-    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK, ACPI_REGISTER_PM1A_CONTROL, PM1AControl);
+    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK,
+                ACPI_REGISTER_PM1A_CONTROL, PM1AControl);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
 
-    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK, ACPI_REGISTER_PM1B_CONTROL, PM1BControl);
+    Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK,
+                ACPI_REGISTER_PM1B_CONTROL, PM1BControl);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -456,10 +491,11 @@ AcpiEnterSleepState (
     if (SleepState > ACPI_STATE_S3)
     {
         /*
-         * We wanted to sleep > S3, but it didn't happen (by virtue of the fact that
-         * we are still executing!)
+         * We wanted to sleep > S3, but it didn't happen (by virtue of the
+         * fact that we are still executing!)
          *
-         * Wait ten seconds, then try again. This is to get S4/S5 to work on all machines.
+         * Wait ten seconds, then try again. This is to get S4/S5 to work on
+         * all machines.
          *
          * We wait so long to allow chipsets that poll this reg very slowly to
          * still read the right value. Ideally, this block would go
@@ -467,7 +503,8 @@ AcpiEnterSleepState (
          */
         AcpiOsStall (10000000);
 
-        Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK, ACPI_REGISTER_PM1_CONTROL,
+        Status = AcpiHwRegisterWrite (ACPI_MTX_DO_NOT_LOCK,
+                    ACPI_REGISTER_PM1_CONTROL,
                     SleepEnableRegInfo->AccessBitMask);
         if (ACPI_FAILURE (Status))
         {
@@ -480,7 +517,7 @@ AcpiEnterSleepState (
     Retry = 1000;
     do
     {
-        Status = AcpiGetRegister (ACPI_BITREG_WAKE_STATUS, &InValue, ACPI_MTX_DO_NOT_LOCK);
+        Status = AcpiGetRegister (ACPI_BITREG_WAKE_STATUS, &InValue);
         if (ACPI_FAILURE (Status))
         {
             return_ACPI_STATUS (Status);
@@ -502,8 +539,10 @@ AcpiEnterSleepState (
     return_ACPI_STATUS (AE_OK);
 }
 
+ACPI_EXPORT_SYMBOL (AcpiEnterSleepState)
 
-/******************************************************************************
+
+/*******************************************************************************
  *
  * FUNCTION:    AcpiEnterSleepStateS4bios
  *
@@ -524,16 +563,16 @@ AcpiEnterSleepStateS4bios (
     ACPI_STATUS             Status;
 
 
-    ACPI_FUNCTION_TRACE ("AcpiEnterSleepStateS4bios");
+    ACPI_FUNCTION_TRACE (AcpiEnterSleepStateS4bios);
 
 
-    Status = AcpiSetRegister (ACPI_BITREG_WAKE_STATUS, 1, ACPI_MTX_DO_NOT_LOCK);
+    Status = AcpiSetRegister (ACPI_BITREG_WAKE_STATUS, 1);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
 
-    Status = AcpiHwClearAcpiStatus (ACPI_MTX_DO_NOT_LOCK);
+    Status = AcpiHwClearAcpiStatus ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -543,14 +582,14 @@ AcpiEnterSleepStateS4bios (
      * 1) Disable/Clear all GPEs
      * 2) Enable all wakeup GPEs
      */
-    Status = AcpiHwDisableAllGpes (ACPI_ISR);
+    Status = AcpiHwDisableAllGpes ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
     AcpiGbl_SystemAwakeAndRunning = FALSE;
 
-    Status = AcpiHwEnableAllWakeupGpes (ACPI_ISR);
+    Status = AcpiHwEnableAllWakeupGpes ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -558,11 +597,12 @@ AcpiEnterSleepStateS4bios (
 
     ACPI_FLUSH_CPU_CACHE ();
 
-    Status = AcpiOsWritePort (AcpiGbl_FADT->SmiCmd, (UINT32) AcpiGbl_FADT->S4BiosReq, 8);
+    Status = AcpiOsWritePort (AcpiGbl_FADT.SmiCommand,
+                (UINT32) AcpiGbl_FADT.S4BiosRequest, 8);
 
     do {
         AcpiOsStall(1000);
-        Status = AcpiGetRegister (ACPI_BITREG_WAKE_STATUS, &InValue, ACPI_MTX_DO_NOT_LOCK);
+        Status = AcpiGetRegister (ACPI_BITREG_WAKE_STATUS, &InValue);
         if (ACPI_FAILURE (Status))
         {
             return_ACPI_STATUS (Status);
@@ -572,8 +612,10 @@ AcpiEnterSleepStateS4bios (
     return_ACPI_STATUS (AE_OK);
 }
 
+ACPI_EXPORT_SYMBOL (AcpiEnterSleepStateS4bios)
 
-/******************************************************************************
+
+/*******************************************************************************
  *
  * FUNCTION:    AcpiLeaveSleepState
  *
@@ -599,7 +641,7 @@ AcpiLeaveSleepState (
     UINT32                  PM1BControl;
 
 
-    ACPI_FUNCTION_TRACE ("AcpiLeaveSleepState");
+    ACPI_FUNCTION_TRACE (AcpiLeaveSleepState);
 
 
     /*
@@ -656,20 +698,20 @@ AcpiLeaveSleepState (
     Status = AcpiEvaluateObject (NULL, METHOD_NAME__SST, &ArgList, NULL);
     if (ACPI_FAILURE (Status) && Status != AE_NOT_FOUND)
     {
-        ACPI_REPORT_ERROR (("Method _SST failed, %s\n", AcpiFormatException (Status)));
+        ACPI_EXCEPTION ((AE_INFO, Status, "During Method _SST"));
     }
 
     Arg.Integer.Value = SleepState;
     Status = AcpiEvaluateObject (NULL, METHOD_NAME__BFS, &ArgList, NULL);
     if (ACPI_FAILURE (Status) && Status != AE_NOT_FOUND)
     {
-        ACPI_REPORT_ERROR (("Method _BFS failed, %s\n", AcpiFormatException (Status)));
+        ACPI_EXCEPTION ((AE_INFO, Status, "During Method _BFS"));
     }
 
     Status = AcpiEvaluateObject (NULL, METHOD_NAME__WAK, &ArgList, NULL);
     if (ACPI_FAILURE (Status) && Status != AE_NOT_FOUND)
     {
-        ACPI_REPORT_ERROR (("Method _WAK failed, %s\n", AcpiFormatException (Status)));
+        ACPI_EXCEPTION ((AE_INFO, Status, "During Method _WAK"));
     }
     /* TBD: _WAK "sometimes" returns stuff - do we want to look at it? */
 
@@ -678,14 +720,14 @@ AcpiLeaveSleepState (
      * 1) Disable/Clear all GPEs
      * 2) Enable all runtime GPEs
      */
-    Status = AcpiHwDisableAllGpes (ACPI_NOT_ISR);
+    Status = AcpiHwDisableAllGpes ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
     AcpiGbl_SystemAwakeAndRunning = TRUE;
 
-    Status = AcpiHwEnableAllRuntimeGpes (ACPI_NOT_ISR);
+    Status = AcpiHwEnableAllRuntimeGpes ();
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -693,14 +735,15 @@ AcpiLeaveSleepState (
 
     /* Enable power button */
 
-    (void) AcpiSetRegister(AcpiGbl_FixedEventInfo[ACPI_EVENT_POWER_BUTTON].EnableRegisterId,
-            1, ACPI_MTX_DO_NOT_LOCK);
-    (void) AcpiSetRegister(AcpiGbl_FixedEventInfo[ACPI_EVENT_POWER_BUTTON].StatusRegisterId,
-            1, ACPI_MTX_DO_NOT_LOCK);
+    (void) AcpiSetRegister(
+            AcpiGbl_FixedEventInfo[ACPI_EVENT_POWER_BUTTON].EnableRegisterId, 1);
+
+    (void) AcpiSetRegister(
+            AcpiGbl_FixedEventInfo[ACPI_EVENT_POWER_BUTTON].StatusRegisterId, 1);
 
     /* Enable BM arbitration */
 
-    Status = AcpiSetRegister (ACPI_BITREG_ARB_DISABLE, 0, ACPI_MTX_LOCK);
+    Status = AcpiSetRegister (ACPI_BITREG_ARB_DISABLE, 0);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -710,8 +753,11 @@ AcpiLeaveSleepState (
     Status = AcpiEvaluateObject (NULL, METHOD_NAME__SST, &ArgList, NULL);
     if (ACPI_FAILURE (Status) && Status != AE_NOT_FOUND)
     {
-        ACPI_REPORT_ERROR (("Method _SST failed, %s\n", AcpiFormatException (Status)));
+        ACPI_EXCEPTION ((AE_INFO, Status, "During Method _SST"));
     }
 
     return_ACPI_STATUS (Status);
 }
+
+ACPI_EXPORT_SYMBOL (AcpiLeaveSleepState)
+
