@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * Module Name: nsload - namespace loading/expanding/contracting procedures
- *              $Revision: 69 $
+ *              $Revision: 1.80 $
  *
  *****************************************************************************/
 
@@ -9,7 +9,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2004, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2007, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -119,19 +119,31 @@
 #include <contrib/dev/acpica/acpi.h>
 #include <contrib/dev/acpica/acnamesp.h>
 #include <contrib/dev/acpica/acdispat.h>
+#include <contrib/dev/acpica/actables.h>
 
 
 #define _COMPONENT          ACPI_NAMESPACE
         ACPI_MODULE_NAME    ("nsload")
 
+/* Local prototypes */
+
+#ifdef ACPI_FUTURE_IMPLEMENTATION
+ACPI_STATUS
+AcpiNsUnloadNamespace (
+    ACPI_HANDLE             Handle);
+
+static ACPI_STATUS
+AcpiNsDeleteSubtree (
+    ACPI_HANDLE             StartHandle);
+#endif
+
 
 #ifndef ACPI_NO_METHOD_EXECUTION
-
 /*******************************************************************************
  *
  * FUNCTION:    AcpiNsLoadTable
  *
- * PARAMETERS:  TableDesc       - Descriptor for table to be loaded
+ * PARAMETERS:  TableIndex      - Index for table to be loaded
  *              Node            - Owning NS node
  *
  * RETURN:      Status
@@ -142,44 +154,15 @@
 
 ACPI_STATUS
 AcpiNsLoadTable (
-    ACPI_TABLE_DESC         *TableDesc,
+    ACPI_NATIVE_UINT        TableIndex,
     ACPI_NAMESPACE_NODE     *Node)
 {
     ACPI_STATUS             Status;
 
 
-    ACPI_FUNCTION_TRACE ("NsLoadTable");
+    ACPI_FUNCTION_TRACE (NsLoadTable);
 
 
-    /* Check if table contains valid AML (must be DSDT, PSDT, SSDT, etc.) */
-
-    if (!(AcpiGbl_TableData[TableDesc->Type].Flags & ACPI_TABLE_EXECUTABLE))
-    {
-        /* Just ignore this table */
-
-        return_ACPI_STATUS (AE_OK);
-    }
-
-    /* Check validity of the AML start and length */
-
-    if (!TableDesc->AmlStart)
-    {
-        ACPI_DEBUG_PRINT ((ACPI_DB_ERROR, "Null AML pointer\n"));
-        return_ACPI_STATUS (AE_BAD_PARAMETER);
-    }
-
-    ACPI_DEBUG_PRINT ((ACPI_DB_INFO, "AML block at %p\n",
-        TableDesc->AmlStart));
-
-    /* Ignore table if there is no AML contained within */
-
-    if (!TableDesc->AmlLength)
-    {
-        ACPI_REPORT_WARNING (("Zero-length AML block in table [%4.4s]\n",
-            TableDesc->Pointer->Signature));
-        return_ACPI_STATUS (AE_OK);
-    }
-
     /*
      * Parse the table and load the namespace with all named
      * objects found within.  Control methods are NOT parsed
@@ -189,16 +172,40 @@ AcpiNsLoadTable (
      * to another control method, we can't continue parsing
      * because we don't know how many arguments to parse next!
      */
-    ACPI_DEBUG_PRINT ((ACPI_DB_INFO,
-        "**** Loading table into namespace ****\n"));
-
     Status = AcpiUtAcquireMutex (ACPI_MTX_NAMESPACE);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
     }
 
-    Status = AcpiNsParseTable (TableDesc, Node->Child);
+    /* If table already loaded into namespace, just return */
+
+    if (AcpiTbIsTableLoaded (TableIndex))
+    {
+        Status = AE_ALREADY_EXISTS;
+        goto Unlock;
+    }
+
+    ACPI_DEBUG_PRINT ((ACPI_DB_INFO,
+        "**** Loading table into namespace ****\n"));
+
+    Status = AcpiTbAllocateOwnerId (TableIndex);
+    if (ACPI_FAILURE (Status))
+    {
+        goto Unlock;
+    }
+
+    Status = AcpiNsParseTable (TableIndex, Node->Child);
+    if (ACPI_SUCCESS (Status))
+    {
+        AcpiTbSetTableLoadedFlag (TableIndex, TRUE);
+    }
+    else
+    {
+        AcpiTbReleaseOwnerId (TableIndex);
+    }
+
+Unlock:
     (void) AcpiUtReleaseMutex (ACPI_MTX_NAMESPACE);
 
     if (ACPI_FAILURE (Status))
@@ -215,7 +222,7 @@ AcpiNsLoadTable (
     ACPI_DEBUG_PRINT ((ACPI_DB_INFO,
         "**** Begin Table Method Parsing and Object Initialization ****\n"));
 
-    Status = AcpiDsInitializeObjects (TableDesc, Node);
+    Status = AcpiDsInitializeObjects (TableIndex, Node);
 
     ACPI_DEBUG_PRINT ((ACPI_DB_INFO,
         "**** Completed Table Method Parsing and Object Initialization ****\n"));
@@ -224,140 +231,7 @@ AcpiNsLoadTable (
 }
 
 
-/*******************************************************************************
- *
- * FUNCTION:    AcpiNsLoadTableByType
- *
- * PARAMETERS:  TableType           - Id of the table type to load
- *
- * RETURN:      Status
- *
- * DESCRIPTION: Load an ACPI table or tables into the namespace.  All tables
- *              of the given type are loaded.  The mechanism allows this
- *              routine to be called repeatedly.
- *
- ******************************************************************************/
-
-ACPI_STATUS
-AcpiNsLoadTableByType (
-    ACPI_TABLE_TYPE         TableType)
-{
-    UINT32                  i;
-    ACPI_STATUS             Status;
-    ACPI_TABLE_DESC         *TableDesc;
-
-
-    ACPI_FUNCTION_TRACE ("NsLoadTableByType");
-
-
-    Status = AcpiUtAcquireMutex (ACPI_MTX_TABLES);
-    if (ACPI_FAILURE (Status))
-    {
-        return_ACPI_STATUS (Status);
-    }
-
-    /*
-     * Table types supported are:
-     * DSDT (one), SSDT/PSDT (multiple)
-     */
-    switch (TableType)
-    {
-    case ACPI_TABLE_DSDT:
-
-        ACPI_DEBUG_PRINT ((ACPI_DB_INFO, "Loading DSDT\n"));
-
-        TableDesc = AcpiGbl_TableLists[ACPI_TABLE_DSDT].Next;
-
-        /* If table already loaded into namespace, just return */
-
-        if (TableDesc->LoadedIntoNamespace)
-        {
-            goto UnlockAndExit;
-        }
-
-        /* Now load the single DSDT */
-
-        Status = AcpiNsLoadTable (TableDesc, AcpiGbl_RootNode);
-        if (ACPI_SUCCESS (Status))
-        {
-            TableDesc->LoadedIntoNamespace = TRUE;
-        }
-        break;
-
-
-    case ACPI_TABLE_SSDT:
-
-        ACPI_DEBUG_PRINT ((ACPI_DB_INFO, "Loading %d SSDTs\n",
-            AcpiGbl_TableLists[ACPI_TABLE_SSDT].Count));
-
-        /*
-         * Traverse list of SSDT tables
-         */
-        TableDesc = AcpiGbl_TableLists[ACPI_TABLE_SSDT].Next;
-        for (i = 0; i < AcpiGbl_TableLists[ACPI_TABLE_SSDT].Count; i++)
-        {
-            /*
-             * Only attempt to load table if it is not
-             * already loaded!
-             */
-            if (!TableDesc->LoadedIntoNamespace)
-            {
-                Status = AcpiNsLoadTable (TableDesc, AcpiGbl_RootNode);
-                if (ACPI_FAILURE (Status))
-                {
-                    break;
-                }
-
-                TableDesc->LoadedIntoNamespace = TRUE;
-            }
-
-            TableDesc = TableDesc->Next;
-        }
-        break;
-
-
-    case ACPI_TABLE_PSDT:
-
-        ACPI_DEBUG_PRINT ((ACPI_DB_INFO, "Loading %d PSDTs\n",
-            AcpiGbl_TableLists[ACPI_TABLE_PSDT].Count));
-
-        /*
-         * Traverse list of PSDT tables
-         */
-        TableDesc = AcpiGbl_TableLists[ACPI_TABLE_PSDT].Next;
-
-        for (i = 0; i < AcpiGbl_TableLists[ACPI_TABLE_PSDT].Count; i++)
-        {
-            /* Only attempt to load table if it is not already loaded! */
-
-            if (!TableDesc->LoadedIntoNamespace)
-            {
-                Status = AcpiNsLoadTable (TableDesc, AcpiGbl_RootNode);
-                if (ACPI_FAILURE (Status))
-                {
-                    break;
-                }
-
-                TableDesc->LoadedIntoNamespace = TRUE;
-            }
-
-            TableDesc = TableDesc->Next;
-        }
-        break;
-
-
-    default:
-        Status = AE_SUPPORT;
-        break;
-    }
-
-
-UnlockAndExit:
-    (void) AcpiUtReleaseMutex (ACPI_MTX_TABLES);
-    return_ACPI_STATUS (Status);
-}
-
-
+#ifdef ACPI_OBSOLETE_FUNCTIONS
 /*******************************************************************************
  *
  * FUNCTION:    AcpiLoadNamespace
@@ -378,14 +252,14 @@ AcpiNsLoadNamespace (
     ACPI_STATUS             Status;
 
 
-    ACPI_FUNCTION_TRACE ("AcpiLoadNameSpace");
+    ACPI_FUNCTION_TRACE (AcpiLoadNameSpace);
 
 
     /* There must be at least a DSDT installed */
 
     if (AcpiGbl_DSDT == NULL)
     {
-        ACPI_DEBUG_PRINT ((ACPI_DB_ERROR, "DSDT is not in memory\n"));
+        ACPI_ERROR ((AE_INFO, "DSDT is not in memory"));
         return_ACPI_STATUS (AE_NO_ACPI_TABLES);
     }
 
@@ -393,7 +267,7 @@ AcpiNsLoadNamespace (
      * Load the namespace.  The DSDT is required,
      * but the SSDT and PSDT tables are optional.
      */
-    Status = AcpiNsLoadTableByType (ACPI_TABLE_DSDT);
+    Status = AcpiNsLoadTableByType (ACPI_TABLE_ID_DSDT);
     if (ACPI_FAILURE (Status))
     {
         return_ACPI_STATUS (Status);
@@ -401,8 +275,8 @@ AcpiNsLoadNamespace (
 
     /* Ignore exceptions from these */
 
-    (void) AcpiNsLoadTableByType (ACPI_TABLE_SSDT);
-    (void) AcpiNsLoadTableByType (ACPI_TABLE_PSDT);
+    (void) AcpiNsLoadTableByType (ACPI_TABLE_ID_SSDT);
+    (void) AcpiNsLoadTableByType (ACPI_TABLE_ID_PSDT);
 
     ACPI_DEBUG_PRINT_RAW ((ACPI_DB_INIT,
         "ACPI Namespace successfully loaded at root %p\n",
@@ -410,8 +284,9 @@ AcpiNsLoadNamespace (
 
     return_ACPI_STATUS (Status);
 }
+#endif
 
-
+#ifdef ACPI_FUTURE_IMPLEMENTATION
 /*******************************************************************************
  *
  * FUNCTION:    AcpiNsDeleteSubtree
@@ -428,7 +303,7 @@ AcpiNsLoadNamespace (
  *
  ******************************************************************************/
 
-ACPI_STATUS
+static ACPI_STATUS
 AcpiNsDeleteSubtree (
     ACPI_HANDLE             StartHandle)
 {
@@ -440,7 +315,7 @@ AcpiNsDeleteSubtree (
     UINT32                  Level;
 
 
-    ACPI_FUNCTION_TRACE ("NsDeleteSubtree");
+    ACPI_FUNCTION_TRACE (NsDeleteSubtree);
 
 
     ParentHandle = StartHandle;
@@ -528,7 +403,7 @@ AcpiNsUnloadNamespace (
     ACPI_STATUS             Status;
 
 
-    ACPI_FUNCTION_TRACE ("NsUnloadNameSpace");
+    ACPI_FUNCTION_TRACE (NsUnloadNameSpace);
 
 
     /* Parameter validation */
@@ -549,6 +424,6 @@ AcpiNsUnloadNamespace (
 
     return_ACPI_STATUS (Status);
 }
-
+#endif
 #endif
 
