@@ -1,11 +1,11 @@
-/*	$FreeBSD: src/sys/contrib/ipfilter/netinet/ip_log.c,v 1.29.2.2 2007/11/18 11:03:28 darrenr Exp $	*/
+/*	$FreeBSD: src/sys/contrib/ipfilter/netinet/ip_log.c,v 1.33.2.1 2007/10/31 05:00:38 darrenr Exp $	*/
 
 /*
  * Copyright (C) 1997-2003 by Darren Reed.
  *
  * See the IPFILTER.LICENCE file for details on licencing.
  *
- * $FreeBSD: src/sys/contrib/ipfilter/netinet/ip_log.c,v 1.29.2.2 2007/11/18 11:03:28 darrenr Exp $
+ * $FreeBSD: src/sys/contrib/ipfilter/netinet/ip_log.c,v 1.33.2.1 2007/10/31 05:00:38 darrenr Exp $
  * Id: ip_log.c,v 2.75.2.19 2007/09/09 11:32:06 darrenr Exp $
  */
 #include <sys/param.h>
@@ -52,8 +52,7 @@ struct file;
 # undef _KERNEL
 # undef KERNEL
 #endif
-#if (defined(__FreeBSD_version) && (__FreeBSD_version >= 220000)) && \
-    defined(_KERNEL)
+#if __FreeBSD_version >= 220000 && defined(_KERNEL)
 # include <sys/fcntl.h>
 # include <sys/filio.h>
 #else
@@ -62,14 +61,14 @@ struct file;
 #include <sys/time.h>
 #if defined(_KERNEL)
 # include <sys/systm.h>
-# if (defined(NetBSD) && (__NetBSD_Version__ >= 104000000))
+# if defined(NetBSD) && (__NetBSD_Version__ >= 104000000)
 #  include <sys/proc.h>
 # endif
 #endif /* _KERNEL */
 #if !SOLARIS && !defined(__hpux) && !defined(linux)
-# if (defined(NetBSD) && (NetBSD > 199609)) || \
-     (defined(OpenBSD) && (OpenBSD > 199603)) || \
-     (defined(__FreeBSD_version) && (__FreeBSD_version >= 300000))
+# if (defined(NetBSD) && NetBSD > 199609) || \
+     (defined(OpenBSD) && OpenBSD > 199603) || \
+     (__FreeBSD_version >= 300000)
 #  include <sys/dirent.h>
 # else
 #  include <sys/dir.h>
