@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/ddb/db_main.c,v 1.4.2.1 2006/10/16 09:00:30 bde Exp $");
+__FBSDID("$FreeBSD: src/sys/ddb/db_main.c,v 1.5 2006/11/06 11:10:57 kib Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -47,8 +47,9 @@ __FBSDID("$FreeBSD: src/sys/ddb/db_main.c,v 1.4.2.1 2006/10/16 09:00:30 bde Exp
 
 static dbbe_init_f db_init;
 static dbbe_trap_f db_trap;
+static dbbe_trace_f db_trace_self_wrapper;
 
-KDB_BACKEND(ddb, db_init, db_trace_self, db_trap);
+KDB_BACKEND(ddb, db_init, db_trace_self_wrapper, db_trap);
 
 vm_offset_t ksym_start, ksym_end;
 
@@ -198,7 +199,6 @@ db_trap(int type, int code)
 	if (cnunavailable())
 		return (0);
 
-	cndbctl(TRUE);
 	bkpt = IS_BREAKPOINT_TRAP(type, code);
 	watchpt = IS_WATCHPOINT_TRAP(type, code);
 
@@ -225,6 +225,17 @@ db_trap(int type, int code)
 
 	db_restart_at_pc(watchpt);
 
-	cndbctl(FALSE);
 	return (1);
 }
+
+static void
+db_trace_self_wrapper(void)
+{
+	jmp_buf jb;
+	void *prev_jb;
+
+	prev_jb = kdb_jmpbuf(jb);
+	if (setjmp(jb) == 0)
+		db_trace_self();
+	(void)kdb_jmpbuf(prev_jb);
+}
