@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/ddb/db_thread.c,v 1.4.2.1 2006/08/17 19:53:05 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/ddb/db_thread.c,v 1.7 2007/01/17 15:05:51 delphij Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -93,13 +93,9 @@ db_show_threads(db_expr_t addr, boolean_t hasaddr, db_expr_t cnt, char *mod)
 	jmp_buf jb;
 	void *prev_jb;
 	struct thread *thr;
-	int pager_quit;
 
-	db_setup_paging(db_simple_pager, &pager_quit, db_lines_per_page);
-
-	pager_quit = 0;
 	thr = kdb_thr_first();
-	while (!pager_quit && thr != NULL) {
+	while (!db_pager_quit && thr != NULL) {
 		db_printf("  %6ld (%p)  ", (long)thr->td_tid, thr);
 		prev_jb = kdb_jmpbuf(jb);
 		if (setjmp(jb) == 0) {
@@ -163,7 +159,7 @@ db_lookup_thread(db_expr_t addr, boolean_t check_pid)
 	if (td != NULL)
 		return (td);
 	if (check_pid) {
-		LIST_FOREACH(p, &allproc, p_list) {
+		FOREACH_PROC_IN_SYSTEM(p) {
 			if (p->p_pid == decaddr)
 				return (FIRST_THREAD_IN_PROC(p));
 		}
@@ -189,7 +185,7 @@ db_lookup_proc(db_expr_t addr)
 
 	decaddr = hex2dec(addr);
 	if (decaddr != -1) {
-		LIST_FOREACH(p, &allproc, p_list) {
+		FOREACH_PROC_IN_SYSTEM(p) {
 			if (p->p_pid == decaddr)
 				return (p);
 		}
