@@ -28,7 +28,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/acpi_support/acpi_fujitsu.c,v 1.2.2.2 2006/06/11 11:34:09 philip Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/acpi_support/acpi_fujitsu.c,v 1.5 2007/03/22 18:16:39 jkim Exp $");
 
 #include "opt_acpi.h"
 #include <sys/param.h>
@@ -320,7 +320,7 @@ acpi_fujitsu_notify_handler(ACPI_HANDLE h, uint32_t notify, void *context)
 
 	switch (notify) {
 	case ACPI_NOTIFY_STATUS_CHANGED:
-		AcpiOsQueueForExecution(OSD_PRIORITY_LO,
+		AcpiOsExecute(OSL_NOTIFY_HANDLER,
 		    acpi_fujitsu_notify_status_changed, sc);
 		break;
 	default:
