@@ -30,7 +30,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/acpica/Osd/OsdDebug.c,v 1.9.2.3 2007/03/14 19:50:07 njl Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/acpica/Osd/OsdDebug.c,v 1.13 2007/03/22 18:16:41 jkim Exp $");
 
 #include "opt_ddb.h"
 #include <sys/param.h>
@@ -62,14 +62,6 @@ AcpiOsGetLine(char *Buffer)
 #endif /* DDB */
 }
 
-void
-AcpiOsDbgAssert(void *FailedAssertion, void *FileName, UINT32 LineNumber,
-    char *Message)
-{
-    printf("ACPI: %s:%d - %s\n", (char *)FileName, LineNumber, Message);
-    printf("ACPI: assertion  %s\n", (char *)FailedAssertion);
-}
-
 ACPI_STATUS
 AcpiOsSignal(UINT32 Function, void *Info)
 {
