@@ -30,7 +30,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/acpica/Osd/OsdSchedule.c,v 1.32.2.6 2006/07/06 08:32:49 glebius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/acpica/Osd/OsdSchedule.c,v 1.39.4.1 2008/01/16 01:22:15 njl Exp $");
 
 #include "opt_acpi.h"
 #include <sys/param.h>
@@ -42,7 +42,6 @@ __FBSDID("$FreeBSD: src/sys/dev/acpica/Osd/OsdSchedule.c,v 1.32.2.6 2006/07/06 0
 #include <sys/malloc.h>
 #include <sys/proc.h>
 #include <sys/taskqueue.h>
-#include <machine/clock.h>
 
 #include <contrib/dev/acpica/acpi.h>
 #include <dev/acpica/acpivar.h>
@@ -66,7 +65,8 @@ struct acpi_task_ctx {
 };
 
 TASKQUEUE_DEFINE(acpi, taskqueue_thread_enqueue, &taskqueue_acpi,
-    taskqueue_start_threads(&taskqueue_acpi, 3, PWAIT, "acpi_task"));
+    taskqueue_start_threads(&taskqueue_acpi, acpi_max_threads, PWAIT,
+    "acpi_task"));
 
 /*
  * Bounce through this wrapper function since ACPI-CA doesn't understand
@@ -87,7 +87,7 @@ acpi_task_execute(void *context, int pending)
  * We allocate and queue a task for one of our taskqueue threads to process.
  */
 ACPI_STATUS
-AcpiOsQueueForExecution(UINT32 Priority, ACPI_OSD_EXEC_CALLBACK Function,
+AcpiOsExecute(ACPI_EXECUTE_TYPE Type, ACPI_OSD_EXEC_CALLBACK Function,
     void *Context)
 {
     struct acpi_task_ctx *at;
@@ -104,18 +104,24 @@ AcpiOsQueueForExecution(UINT32 Priority, ACPI_OSD_EXEC_CALLBACK Function,
 
     at->at_function = Function;
     at->at_context = Context;
-    switch (Priority) {
-    case OSD_PRIORITY_GPE:
-	pri = 4;
+    switch (Type) {
+    case OSL_GPE_HANDLER:
+    case OSL_NOTIFY_HANDLER:
+	/*
+	 * Run GPEs and Notifies at the same priority.  This allows
+	 * Notifies that are generated by running a GPE's method (e.g., _L00)
+	 * to not be pre-empted by a later GPE that arrives during the
+	 * Notify handler execution.
+	 */
+	pri = 10;
 	break;
-    case OSD_PRIORITY_HIGH:
-	pri = 3;
+    case OSL_GLOBAL_LOCK_HANDLER:
+    case OSL_EC_POLL_HANDLER:
+    case OSL_EC_BURST_HANDLER:
+	pri = 5;
 	break;
-    case OSD_PRIORITY_MED:
-	pri = 2;
-	break;
-    case OSD_PRIORITY_LO:
-	pri = 1;
+    case OSL_DEBUGGER_THREAD:
+	pri = 0;
 	break;
     default:
 	free(at, M_ACPITASK);
@@ -132,7 +138,6 @@ void
 AcpiOsSleep(ACPI_INTEGER Milliseconds)
 {
     int		timo;
-    static int	dummy;
 
     ACPI_FUNCTION_TRACE((char *)(uintptr_t)__func__);
 
@@ -143,7 +148,7 @@ AcpiOsSleep(ACPI_INTEGER Milliseconds)
      * DELAY instead for better granularity.
      */
     if (timo > 0)
-	tsleep(&dummy, 0, "acpislp", timo);
+	pause("acpislp", timo);
     else
 	DELAY(Milliseconds * 1000);
 
@@ -179,7 +184,7 @@ AcpiOsStall(UINT32 Microseconds)
     return_VOID;
 }
 
-UINT32
+ACPI_THREAD_ID
 AcpiOsGetThreadId(void)
 {
     struct proc *p;
