@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/acpica/acpi_pci_link.c,v 1.44.2.8 2007/11/08 20:24:53 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/acpica/acpi_pci_link.c,v 1.56.2.1 2007/11/08 20:21:40 jhb Exp $");
 
 #include "opt_acpi.h"
 #include <sys/param.h>
@@ -119,7 +119,7 @@ struct link_res_request {
 	int	link_index;
 };
 
-MALLOC_DEFINE(M_PCI_LINK, "PCI Link", "ACPI PCI Link structures");
+MALLOC_DEFINE(M_PCI_LINK, "pci_link", "ACPI PCI Link structures");
 
 static int pci_link_interrupt_weights[NUM_ACPI_INTERRUPTS];
 static int pci_link_bios_isa_irqs;
@@ -169,8 +169,8 @@ acpi_count_irq_resources(ACPI_RESOURCE *res, void *context)
 	struct link_count_request *req;
 
 	req = (struct link_count_request *)context;
-	switch (res->Id) {
-	case ACPI_RSTYPE_START_DPF:
+	switch (res->Type) {
+	case ACPI_RESOURCE_TYPE_START_DEPENDENT:
 		switch (req->in_dpf) {
 		case DPF_OUTSIDE:
 			/* We've started the first DPF. */
@@ -182,14 +182,14 @@ acpi_count_irq_resources(ACPI_RESOURCE *res, void *context)
 			break;
 		}
 		break;
-	case ACPI_RSTYPE_END_DPF:
+	case ACPI_RESOURCE_TYPE_END_DEPENDENT:
 		/* We are finished with DPF parsing. */
 		KASSERT(req->in_dpf != DPF_OUTSIDE,
 		    ("%s: end dpf when not parsing a dpf", __func__));
 		req->in_dpf = DPF_OUTSIDE;
 		break;
-	case ACPI_RSTYPE_IRQ:
-	case ACPI_RSTYPE_EXT_IRQ:
+	case ACPI_RESOURCE_TYPE_IRQ:
+	case ACPI_RESOURCE_TYPE_EXTENDED_IRQ:
 		/*
 		 * Don't count resources if we are in a DPF set that we are
 		 * ignoring.
@@ -208,8 +208,8 @@ link_add_crs(ACPI_RESOURCE *res, void *context)
 
 	ACPI_SERIAL_ASSERT(pci_link);
 	req = (struct link_res_request *)context;
-	switch (res->Id) {
-	case ACPI_RSTYPE_START_DPF:
+	switch (res->Type) {
+	case ACPI_RESOURCE_TYPE_START_DEPENDENT:
 		switch (req->in_dpf) {
 		case DPF_OUTSIDE:
 			/* We've started the first DPF. */
@@ -223,14 +223,14 @@ link_add_crs(ACPI_RESOURCE *res, void *context)
 			break;
 		}
 		break;
-	case ACPI_RSTYPE_END_DPF:
+	case ACPI_RESOURCE_TYPE_END_DEPENDENT:
 		/* We are finished with DPF parsing. */
 		KASSERT(req->in_dpf != DPF_OUTSIDE,
 		    ("%s: end dpf when not parsing a dpf", __func__));
 		req->in_dpf = DPF_OUTSIDE;
 		break;
-	case ACPI_RSTYPE_IRQ:
-	case ACPI_RSTYPE_EXT_IRQ:
+	case ACPI_RESOURCE_TYPE_IRQ:
+	case ACPI_RESOURCE_TYPE_EXTENDED_IRQ:
 		KASSERT(req->link_index < req->sc->pl_num_links,
 		    ("%s: array boundary violation", __func__));
 		link = &req->sc->pl_links[req->link_index];
@@ -243,10 +243,10 @@ link_add_crs(ACPI_RESOURCE *res, void *context)
 		 * systems return multiple IRQs (which is nonsense for _CRS)
 		 * when the link hasn't been programmed.
 		 */
-		if (res->Id == ACPI_RSTYPE_IRQ) {
-			if (res->Data.Irq.NumberOfInterrupts == 1)
+		if (res->Type == ACPI_RESOURCE_TYPE_IRQ) {
+			if (res->Data.Irq.InterruptCount == 1)
 				link->l_irq = res->Data.Irq.Interrupts[0];
-		} else if (res->Data.ExtendedIrq.NumberOfInterrupts == 1)
+		} else if (res->Data.ExtendedIrq.InterruptCount == 1)
 			link->l_irq = res->Data.ExtendedIrq.Interrupts[0];
 
 		/*
@@ -269,13 +269,14 @@ link_add_prs(ACPI_RESOURCE *res, void *context)
 {
 	struct link_res_request *req;
 	struct link *link;
-	UINT32 *irqs;
-	int i;
+	UINT8 *irqs = NULL;
+	UINT32 *ext_irqs = NULL;
+	int i, is_ext_irq = 1;
 
 	ACPI_SERIAL_ASSERT(pci_link);
 	req = (struct link_res_request *)context;
-	switch (res->Id) {
-	case ACPI_RSTYPE_START_DPF:
+	switch (res->Type) {
+	case ACPI_RESOURCE_TYPE_START_DEPENDENT:
 		switch (req->in_dpf) {
 		case DPF_OUTSIDE:
 			/* We've started the first DPF. */
@@ -287,14 +288,16 @@ link_add_prs(ACPI_RESOURCE *res, void *context)
 			break;
 		}
 		break;
-	case ACPI_RSTYPE_END_DPF:
+	case ACPI_RESOURCE_TYPE_END_DEPENDENT:
 		/* We are finished with DPF parsing. */
 		KASSERT(req->in_dpf != DPF_OUTSIDE,
 		    ("%s: end dpf when not parsing a dpf", __func__));
 		req->in_dpf = DPF_OUTSIDE;
 		break;
-	case ACPI_RSTYPE_IRQ:
-	case ACPI_RSTYPE_EXT_IRQ:
+	case ACPI_RESOURCE_TYPE_IRQ:
+		is_ext_irq = 0;
+		/* fall through */
+	case ACPI_RESOURCE_TYPE_EXTENDED_IRQ:
 		/*
 		 * Don't parse resources if we are in a DPF set that we are
 		 * ignoring.
@@ -318,13 +321,13 @@ link_add_prs(ACPI_RESOURCE *res, void *context)
 		 * _SRS.
 		 */
 		bcopy(res, &link->l_prs_template, sizeof(ACPI_RESOURCE));
-		if (res->Id == ACPI_RSTYPE_IRQ) {
-			link->l_num_irqs = res->Data.Irq.NumberOfInterrupts;
-			irqs = res->Data.Irq.Interrupts;
-		} else {
+		if (is_ext_irq) {
 			link->l_num_irqs =
-			    res->Data.ExtendedIrq.NumberOfInterrupts;
-			irqs = res->Data.ExtendedIrq.Interrupts;
+			    res->Data.ExtendedIrq.InterruptCount;
+			ext_irqs = res->Data.ExtendedIrq.Interrupts;
+		} else {
+			link->l_num_irqs = res->Data.Irq.InterruptCount;
+			irqs = res->Data.Irq.Interrupts;
 		}
 		if (link->l_num_irqs == 0)
 			break;
@@ -338,9 +341,15 @@ link_add_prs(ACPI_RESOURCE *res, void *context)
 		link->l_irqs = malloc(sizeof(int) * link->l_num_irqs,
 		    M_PCI_LINK, M_WAITOK | M_ZERO);
 		for (i = 0; i < link->l_num_irqs; i++) {
-			link->l_irqs[i] = irqs[i];
-			if (irqs[i] >= NUM_ISA_INTERRUPTS)
-				link->l_isa_irq = FALSE;
+			if (is_ext_irq) {
+				link->l_irqs[i] = ext_irqs[i];
+				if (ext_irqs[i] >= NUM_ISA_INTERRUPTS)
+					link->l_isa_irq = FALSE;
+			} else {
+				link->l_irqs[i] = irqs[i];
+				if (irqs[i] >= NUM_ISA_INTERRUPTS)
+					link->l_isa_irq = FALSE;
+			}
 		}
 		break;
 	default:
@@ -375,7 +384,7 @@ link_valid_irq(struct link *link, int irq)
 	 * For links routed via an ISA interrupt, if the SCI is routed via
 	 * an ISA interrupt, the SCI is always treated as a valid IRQ.
 	 */
-	if (link->l_isa_irq && AcpiGbl_FADT->SciInt == irq &&
+	if (link->l_isa_irq && AcpiGbl_FADT.SciInterrupt == irq &&
 	    irq < NUM_ISA_INTERRUPTS)
 		return (TRUE);
 
@@ -707,8 +716,8 @@ acpi_pci_link_srs_from_crs(struct acpi_pci_link_softc *sc, ACPI_BUFFER *srsbuf)
 	resource = (ACPI_RESOURCE *)crsbuf.Pointer;
 	end = (ACPI_RESOURCE *)((char *)crsbuf.Pointer + crsbuf.Length);
 	for (;;) {
-		switch (resource->Id) {
-		case ACPI_RSTYPE_START_DPF:
+		switch (resource->Type) {
+		case ACPI_RESOURCE_TYPE_START_DEPENDENT:
 			switch (in_dpf) {
 			case DPF_OUTSIDE:
 				/* We've started the first DPF. */
@@ -723,19 +732,19 @@ acpi_pci_link_srs_from_crs(struct acpi_pci_link_softc *sc, ACPI_BUFFER *srsbuf)
 			}
 			resptr = NULL;
 			break;
-		case ACPI_RSTYPE_END_DPF:
+		case ACPI_RESOURCE_TYPE_END_DEPENDENT:
 			/* We are finished with DPF parsing. */
 			KASSERT(in_dpf != DPF_OUTSIDE,
 			    ("%s: end dpf when not parsing a dpf", __func__));
 			in_dpf = DPF_OUTSIDE;
 			resptr = NULL;
 			break;
-		case ACPI_RSTYPE_IRQ:
+		case ACPI_RESOURCE_TYPE_IRQ:
 			MPASS(i < sc->pl_num_links);
-			MPASS(link->l_prs_template.Id == ACPI_RSTYPE_IRQ);
+			MPASS(link->l_prs_template.Type == ACPI_RESOURCE_TYPE_IRQ);
 			newres = link->l_prs_template;
 			resptr = &newres;
-			resptr->Data.Irq.NumberOfInterrupts = 1;
+			resptr->Data.Irq.InterruptCount = 1;
 			if (PCI_INTERRUPT_VALID(link->l_irq)) {
 				KASSERT(link->l_irq < NUM_ISA_INTERRUPTS,
 		("%s: can't put non-ISA IRQ %d in legacy IRQ resource type",
@@ -746,12 +755,12 @@ acpi_pci_link_srs_from_crs(struct acpi_pci_link_softc *sc, ACPI_BUFFER *srsbuf)
 			link++;
 			i++;
 			break;
-		case ACPI_RSTYPE_EXT_IRQ:
+		case ACPI_RESOURCE_TYPE_EXTENDED_IRQ:
 			MPASS(i < sc->pl_num_links);
-			MPASS(link->l_prs_template.Id == ACPI_RSTYPE_EXT_IRQ);
+			MPASS(link->l_prs_template.Type == ACPI_RESOURCE_TYPE_EXTENDED_IRQ);
 			newres = link->l_prs_template;
 			resptr = &newres;
-			resptr->Data.ExtendedIrq.NumberOfInterrupts = 1;
+			resptr->Data.ExtendedIrq.InterruptCount = 1;
 			if (PCI_INTERRUPT_VALID(link->l_irq))
 				resptr->Data.ExtendedIrq.Interrupts[0] =
 				    link->l_irq;
@@ -775,7 +784,7 @@ acpi_pci_link_srs_from_crs(struct acpi_pci_link_softc *sc, ACPI_BUFFER *srsbuf)
 				return (status);
 			}
 		}
-		if (resource->Id == ACPI_RSTYPE_END_TAG)
+		if (resource->Type == ACPI_RESOURCE_TYPE_END_TAG)
 			break;
 		resource = ACPI_NEXT_RESOURCE(resource);
 		if (resource >= end)
@@ -802,10 +811,10 @@ acpi_pci_link_srs_from_links(struct acpi_pci_link_softc *sc,
 		/* Add a new IRQ resource from each link. */
 		link = &sc->pl_links[i];
 		newres = link->l_prs_template;
-		if (newres.Id == ACPI_RSTYPE_IRQ) {
+		if (newres.Type == ACPI_RESOURCE_TYPE_IRQ) {
 
 			/* Build an IRQ resource. */
-			newres.Data.Irq.NumberOfInterrupts = 1;
+			newres.Data.Irq.InterruptCount = 1;
 			if (PCI_INTERRUPT_VALID(link->l_irq)) {
 				KASSERT(link->l_irq < NUM_ISA_INTERRUPTS,
 		("%s: can't put non-ISA IRQ %d in legacy IRQ resource type",
@@ -816,7 +825,7 @@ acpi_pci_link_srs_from_links(struct acpi_pci_link_softc *sc,
 		} else {
 
 			/* Build an ExtIRQ resuorce. */
-			newres.Data.ExtendedIrq.NumberOfInterrupts = 1;
+			newres.Data.ExtendedIrq.InterruptCount = 1;
 			if (PCI_INTERRUPT_VALID(link->l_irq))
 				newres.Data.ExtendedIrq.Interrupts[0] =
 				    link->l_irq;
@@ -873,11 +882,11 @@ acpi_pci_link_route_irqs(device_t dev)
 	resource = (ACPI_RESOURCE *)srsbuf.Pointer;
 	end = (ACPI_RESOURCE *)((char *)srsbuf.Pointer + srsbuf.Length);
 	for (;;) {
-		if (resource->Id == ACPI_RSTYPE_END_TAG)
+		if (resource->Type == ACPI_RESOURCE_TYPE_END_TAG)
 			break;
-		switch (resource->Id) {
-		case ACPI_RSTYPE_IRQ:
-		case ACPI_RSTYPE_EXT_IRQ:
+		switch (resource->Type) {
+		case ACPI_RESOURCE_TYPE_IRQ:
+		case ACPI_RESOURCE_TYPE_EXTENDED_IRQ:
 			MPASS(i < sc->pl_num_links);
 
 			/*
@@ -948,19 +957,27 @@ acpi_pci_link_choose_irq(device_t dev, struct link *link)
 	KASSERT(!PCI_INTERRUPT_VALID(link->l_irq),
 	    ("%s: link already has an IRQ", __func__));
 
-	/* Check for a tunable override and use it if it is valid. */
+	/* Check for a tunable override. */
 	if (ACPI_SUCCESS(acpi_short_name(acpi_get_handle(dev), link_name,
 	    sizeof(link_name)))) {
 		snprintf(tunable_buffer, sizeof(tunable_buffer),
 		    "hw.pci.link.%s.%d.irq", link_name, link->l_res_index);
-		if (getenv_int(tunable_buffer, &i) &&
-		    PCI_INTERRUPT_VALID(i) && link_valid_irq(link, i))
+		if (getenv_int(tunable_buffer, &i) && PCI_INTERRUPT_VALID(i)) {
+			if (!link_valid_irq(link, i))
+				device_printf(dev,
+				    "Warning, IRQ %d is not listed as valid\n",
+				    i);
 			return (i);
+		}
 		snprintf(tunable_buffer, sizeof(tunable_buffer),
 		    "hw.pci.link.%s.irq", link_name);
-		if (getenv_int(tunable_buffer, &i) &&
-		    PCI_INTERRUPT_VALID(i) && link_valid_irq(link, i))
+		if (getenv_int(tunable_buffer, &i) && PCI_INTERRUPT_VALID(i)) {
+			if (!link_valid_irq(link, i))
+				device_printf(dev,
+				    "Warning, IRQ %d is not listed as valid\n",
+				    i);
 			return (i);
+		}
 	}
 
 	/*
@@ -1001,7 +1018,7 @@ acpi_pci_link_choose_irq(device_t dev, struct link *link)
 	 * interrupt as a fallback.
 	 */
 	if (link->l_isa_irq) {
-		pos_irq = AcpiGbl_FADT->SciInt;
+		pos_irq = AcpiGbl_FADT.SciInterrupt;
 		pos_weight = pci_link_interrupt_weights[pos_irq];
 		if (pos_weight < best_weight) {
 			best_weight = pos_weight;
@@ -1078,8 +1095,8 @@ acpi_pci_link_identify(driver_t *driver, device_t parent)
 	 * if we are using the APIC, we also shouldn't be having any PCI
 	 * interrupts routed via ISA IRQs, so this is probably ok.
 	 */
-	if (AcpiGbl_FADT->SciInt < NUM_ISA_INTERRUPTS)
-		pci_link_bios_isa_irqs |= (1 << AcpiGbl_FADT->SciInt);
+	if (AcpiGbl_FADT.SciInterrupt < NUM_ISA_INTERRUPTS)
+		pci_link_bios_isa_irqs |= (1 << AcpiGbl_FADT.SciInterrupt);
 }
 
 static device_method_t acpi_pci_link_methods[] = {
