@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/acpica/acpi_thermal.c,v 1.53.2.7.2.1 2008/01/07 21:27:50 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/acpica/acpi_thermal.c,v 1.64.4.1 2008/01/07 21:26:38 jhb Exp $");
 
 #include "opt_acpi.h"
 #include <sys/param.h>
