@@ -44,7 +44,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/advansys/adwcam.c,v 1.21.2.1 2006/12/14 02:57:04 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/advansys/adwcam.c,v 1.28 2007/06/17 05:55:46 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -526,6 +526,8 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 		break;
 	case XPT_SET_TRAN_SETTINGS:
 	{
+		struct ccb_trans_settings_scsi *scsi;
+		struct ccb_trans_settings_spi *spi;
 		struct	  ccb_trans_settings *cts;
 		u_int	  target_mask;
 		int	  s;
@@ -534,17 +536,19 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 		target_mask = 0x01 << ccb->ccb_h.target_id;
 
 		s = splcam();
-		if ((cts->flags & CCB_TRANS_CURRENT_SETTINGS) != 0) {
+		scsi = &cts->proto_specific.scsi;
+		spi = &cts->xport_specific.spi;
+		if (cts->type == CTS_TYPE_CURRENT_SETTINGS) {
 			u_int sdtrdone;
 
 			sdtrdone = adw_lram_read_16(adw, ADW_MC_SDTR_DONE);
-			if ((cts->valid & CCB_TRANS_DISC_VALID) != 0) {
+			if ((spi->valid & CTS_SPI_VALID_DISC) != 0) {
 				u_int discenb;
 
 				discenb =
 				    adw_lram_read_16(adw, ADW_MC_DISC_ENABLE);
 
-				if ((cts->flags & CCB_TRANS_DISC_ENB) != 0)
+				if ((spi->flags & CTS_SPI_FLAGS_DISC_ENB) != 0)
 					discenb |= target_mask;
 				else
 					discenb &= ~target_mask;
@@ -553,15 +557,15 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 						  discenb);
 			}
 		
-			if ((cts->valid & CCB_TRANS_TQ_VALID) != 0) {
+			if ((scsi->valid & CTS_SCSI_VALID_TQ) != 0) {
 
-				if ((cts->flags & CCB_TRANS_TAG_ENB) != 0)
+				if ((scsi->flags & CTS_SCSI_FLAGS_TAG_ENB) != 0)
 					adw->tagenb |= target_mask;
 				else
 					adw->tagenb &= ~target_mask;
 			}	
 
-			if ((cts->valid & CCB_TRANS_BUS_WIDTH_VALID) != 0) {
+			if ((spi->valid & CTS_SPI_VALID_BUS_WIDTH) != 0) {
 				u_int wdtrenb_orig;
 				u_int wdtrenb;
 				u_int wdtrdone;
@@ -571,7 +575,7 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 				wdtrenb = wdtrenb_orig;
 				wdtrdone = adw_lram_read_16(adw,
 							    ADW_MC_WDTR_DONE);
-				switch (cts->bus_width) {
+				switch (spi->bus_width) {
 				case MSG_EXT_WDTR_BUS_32_BIT:
 				case MSG_EXT_WDTR_BUS_16_BIT:
 					wdtrenb |= target_mask;
@@ -597,8 +601,8 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 				}
 			}
 
-			if (((cts->valid & CCB_TRANS_SYNC_RATE_VALID) != 0)
-			 || ((cts->valid & CCB_TRANS_SYNC_OFFSET_VALID) != 0)) {
+			if (((spi->valid & CTS_SPI_VALID_SYNC_RATE) != 0)
+			 || ((spi->valid & CTS_SPI_VALID_SYNC_OFFSET) != 0)) {
 				u_int sdtr_orig;
 				u_int sdtr;
 				u_int sdtrable_orig;
@@ -611,17 +615,17 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 							    ADW_MC_SDTR_ABLE);
 				sdtrable_orig = sdtrable;
 
-				if ((cts->valid
-				   & CCB_TRANS_SYNC_RATE_VALID) != 0) {
+				if ((spi->valid
+				   & CTS_SPI_VALID_SYNC_RATE) != 0) {
 
 					sdtr =
 					    adw_find_sdtr(adw,
-							  cts->sync_period);
+							  spi->sync_period);
 				}
 					
-				if ((cts->valid
-				   & CCB_TRANS_SYNC_OFFSET_VALID) != 0) {
-					if (cts->sync_offset == 0)
+				if ((spi->valid
+				   & CTS_SPI_VALID_SYNC_OFFSET) != 0) {
+					if (spi->sync_offset == 0)
 						sdtr = ADW_MC_SDTR_ASYNC;
 				}
 
@@ -651,49 +655,53 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 	case XPT_GET_TRAN_SETTINGS:
 	/* Get default/user set transfer settings for the target */
 	{
+		struct ccb_trans_settings_scsi *scsi;
+		struct ccb_trans_settings_spi *spi;
 		struct	ccb_trans_settings *cts;
 		u_int	target_mask;
  
 		cts = &ccb->cts;
 		target_mask = 0x01 << ccb->ccb_h.target_id;
-		if ((cts->flags & CCB_TRANS_USER_SETTINGS) != 0) { 
+		cts->protocol = PROTO_SCSI;
+		cts->protocol_version = SCSI_REV_2;
+		cts->transport = XPORT_SPI;
+		cts->transport_version = 2;
+
+		scsi = &cts->proto_specific.scsi;
+		spi = &cts->xport_specific.spi;
+		if (cts->type == CTS_TYPE_CURRENT_SETTINGS) {
 			u_int mc_sdtr;
 
-			cts->flags = 0;
+			spi->flags = 0;
 			if ((adw->user_discenb & target_mask) != 0)
-				cts->flags |= CCB_TRANS_DISC_ENB;
+				spi->flags |= CTS_SPI_FLAGS_DISC_ENB;
 
 			if ((adw->user_tagenb & target_mask) != 0)
-				cts->flags |= CCB_TRANS_TAG_ENB;
+				scsi->flags |= CTS_SCSI_FLAGS_TAG_ENB;
 
 			if ((adw->user_wdtr & target_mask) != 0)
-				cts->bus_width = MSG_EXT_WDTR_BUS_16_BIT;
+				spi->bus_width = MSG_EXT_WDTR_BUS_16_BIT;
 			else
-				cts->bus_width = MSG_EXT_WDTR_BUS_8_BIT;
+				spi->bus_width = MSG_EXT_WDTR_BUS_8_BIT;
 
 			mc_sdtr = adw_get_user_sdtr(adw, ccb->ccb_h.target_id);
-			cts->sync_period = adw_find_period(adw, mc_sdtr);
-			if (cts->sync_period != 0)
-				cts->sync_offset = 15; /* XXX ??? */
+			spi->sync_period = adw_find_period(adw, mc_sdtr);
+			if (spi->sync_period != 0)
+				spi->sync_offset = 15; /* XXX ??? */
 			else
-				cts->sync_offset = 0;
+				spi->sync_offset = 0;
+
 
-			cts->valid = CCB_TRANS_SYNC_RATE_VALID
-				   | CCB_TRANS_SYNC_OFFSET_VALID
-				   | CCB_TRANS_BUS_WIDTH_VALID
-				   | CCB_TRANS_DISC_VALID
-				   | CCB_TRANS_TQ_VALID;
-			ccb->ccb_h.status = CAM_REQ_CMP;
 		} else {
 			u_int targ_tinfo;
 
-			cts->flags = 0;
+			spi->flags = 0;
 			if ((adw_lram_read_16(adw, ADW_MC_DISC_ENABLE)
 			  & target_mask) != 0)
-				cts->flags |= CCB_TRANS_DISC_ENB;
+				spi->flags |= CTS_SPI_FLAGS_DISC_ENB;
 
 			if ((adw->tagenb & target_mask) != 0)
-				cts->flags |= CCB_TRANS_TAG_ENB;
+				scsi->flags |= CTS_SCSI_FLAGS_TAG_ENB;
 
 			targ_tinfo =
 			    adw_lram_read_16(adw,
@@ -701,25 +709,26 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 					     + (2 * ccb->ccb_h.target_id));
 
 			if ((targ_tinfo & ADW_HSHK_CFG_WIDE_XFR) != 0)
-				cts->bus_width = MSG_EXT_WDTR_BUS_16_BIT;
+				spi->bus_width = MSG_EXT_WDTR_BUS_16_BIT;
 			else
-				cts->bus_width = MSG_EXT_WDTR_BUS_8_BIT;
+				spi->bus_width = MSG_EXT_WDTR_BUS_8_BIT;
 
-			cts->sync_period =
+			spi->sync_period =
 			    adw_hshk_cfg_period_factor(targ_tinfo);
 
-			cts->sync_offset = targ_tinfo & ADW_HSHK_CFG_OFFSET;
-			if (cts->sync_period == 0)
-				cts->sync_offset = 0;
+			spi->sync_offset = targ_tinfo & ADW_HSHK_CFG_OFFSET;
+			if (spi->sync_period == 0)
+				spi->sync_offset = 0;
 
-			if (cts->sync_offset == 0)
-				cts->sync_period = 0;
+			if (spi->sync_offset == 0)
+				spi->sync_period = 0;
 		}
-		cts->valid = CCB_TRANS_SYNC_RATE_VALID
-			   | CCB_TRANS_SYNC_OFFSET_VALID
-			   | CCB_TRANS_BUS_WIDTH_VALID
-			   | CCB_TRANS_DISC_VALID
-			   | CCB_TRANS_TQ_VALID;
+
+		spi->valid = CTS_SPI_VALID_SYNC_RATE
+			   | CTS_SPI_VALID_SYNC_OFFSET
+			   | CTS_SPI_VALID_BUS_WIDTH
+			   | CTS_SPI_VALID_DISC;
+		scsi->valid = CTS_SCSI_VALID_TQ;
 		ccb->ccb_h.status = CAM_REQ_CMP;
 		xpt_done(ccb);
 		break;
@@ -774,6 +783,10 @@ adw_action(struct cam_sim *sim, union ccb *ccb)
 		strncpy(cpi->hba_vid, "AdvanSys", HBA_IDLEN);
 		strncpy(cpi->dev_name, cam_sim_name(sim), DEV_IDLEN);
 		cpi->unit_number = cam_sim_unit(sim);
+                cpi->transport = XPORT_SPI;
+                cpi->transport_version = 2;
+                cpi->protocol = PROTO_SCSI;
+                cpi->protocol_version = SCSI_REV_2;
 		cpi->ccb_h.status = CAM_REQ_CMP;
 		xpt_done(ccb);
 		break;
@@ -1200,8 +1213,8 @@ adw_attach(struct adw_softc *adw)
 	s = splcam();
 	/* Hook up our interrupt handler */
 	if ((error = bus_setup_intr(adw->device, adw->irq,
-				    INTR_TYPE_CAM | INTR_ENTROPY, adw_intr,
-				    adw, &adw->ih)) != 0) {
+				    INTR_TYPE_CAM | INTR_ENTROPY, NULL, 
+				    adw_intr, adw, &adw->ih)) != 0) {				    
 		device_printf(adw->device, "bus_setup_intr() failed: %d\n",
 			      error);
 		goto fail;
@@ -1221,7 +1234,7 @@ adw_attach(struct adw_softc *adw)
 	 * Construct our SIM entry.
 	 */
 	adw->sim = cam_sim_alloc(adw_action, adw_poll, "adw", adw, adw->unit,
-				 1, adw->max_acbs, devq);
+				 &Giant, 1, adw->max_acbs, devq);
 	if (adw->sim == NULL) {
 		error = ENOMEM;
 		goto fail;
@@ -1230,7 +1243,7 @@ adw_attach(struct adw_softc *adw)
 	/*
 	 * Register the bus.
 	 */
-	if (xpt_bus_register(adw->sim, 0) != CAM_SUCCESS) {
+	if (xpt_bus_register(adw->sim, adw->device, 0) != CAM_SUCCESS) {
 		cam_sim_free(adw->sim, /*free devq*/TRUE);
 		error = ENOMEM;
 		goto fail;
@@ -1291,7 +1304,7 @@ adw_intr(void *arg)
 			 * Handle RDMA failure by resetting the
 			 * SCSI Bus and chip.
 			 */
-#if XXX
+#if 0 /* XXX */
 			AdvResetChipAndSB(adv_dvc_varp);
 #endif
 			break;
