@@ -39,7 +39,7 @@
  *
  * $Id: //depot/aic7xxx/aic7xxx/aic7xxx_inline.h#47 $
  *
- * $FreeBSD: src/sys/dev/aic7xxx/aic7xxx_inline.h,v 1.24 2005/01/06 01:42:26 imp Exp $
+ * $FreeBSD: src/sys/dev/aic7xxx/aic7xxx_inline.h,v 1.25 2007/04/19 18:53:52 scottl Exp $
  */
 
 #ifndef _AIC7XXX_INLINE_H_
@@ -363,7 +363,8 @@ ahc_get_scb(struct ahc_softc *ahc)
 	struct scb *scb;
 
 	if ((scb = SLIST_FIRST(&ahc->scb_data->free_scbs)) == NULL) {
-		ahc_alloc_scbs(ahc);
+		if (ahc_alloc_scbs(ahc) == 0)
+			return (NULL);
 		scb = SLIST_FIRST(&ahc->scb_data->free_scbs);
 		if (scb == NULL)
 			return (NULL);
