@@ -48,7 +48,7 @@
 #include "aic7xxx_93cx6.h"
 #else
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/aic7xxx/aic7xxx_pci.c,v 1.34 2005/01/06 01:42:26 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/aic7xxx/aic7xxx_pci.c,v 1.36 2007/04/17 06:26:25 scottl Exp $");
 #include <dev/aic7xxx/aic7xxx_osm.h>
 #include <dev/aic7xxx/aic7xxx_inline.h>
 #include <dev/aic7xxx/aic7xxx_93cx6.h>
@@ -785,7 +785,6 @@ ahc_find_pci_device(aic_dev_softc_t pci)
 int
 ahc_pci_config(struct ahc_softc *ahc, struct ahc_pci_identity *entry)
 {
-	u_long	 l;
 	u_int	 command;
 	u_int	 our_id;
 	u_int	 sxfrctl1;
@@ -1045,12 +1044,12 @@ ahc_pci_config(struct ahc_softc *ahc, struct ahc_pci_identity *entry)
 	if (error != 0)
 		return (error);
 
-	ahc_list_lock(&l);
+	ahc_lock(ahc);
 	/*
 	 * Link this softc in with all other ahc instances.
 	 */
 	ahc_softc_insert(ahc);
-	ahc_list_unlock(&l);
+	ahc_unlock(ahc);
 	return (0);
 }
 
@@ -2023,11 +2022,15 @@ ahc_pci_intr(struct ahc_softc *ahc)
 	status1 = aic_pci_read_config(ahc->dev_softc,
 				      PCIR_STATUS + 1, /*bytes*/1);
 
-	printf("%s: PCI error Interrupt at seqaddr = 0x%x\n",
-	      ahc_name(ahc),
-	      ahc_inb(ahc, SEQADDR0) | (ahc_inb(ahc, SEQADDR1) << 8));
+	if ((status1 & ~DPE) != 0
+	 || (ahc->flags & AHC_DISABLE_PCI_PERR) == 0) {
+		printf("%s: PCI error Interrupt at seqaddr = 0x%x\n",
+		       ahc_name(ahc),
+		       ahc_inb(ahc, SEQADDR0) | (ahc_inb(ahc, SEQADDR1) << 8));
+	}
 
-	if (status1 & DPE) {
+	if (status1 & DPE
+	 && (ahc->flags & AHC_DISABLE_PCI_PERR) == 0) {
 		ahc->pci_target_perr_count++;
 		printf("%s: Data Parity Error Detected during address "
 		       "or write data phase\n", ahc_name(ahc));
@@ -2060,17 +2063,20 @@ ahc_pci_intr(struct ahc_softc *ahc)
 		ahc_outb(ahc, CLRINT, CLRPARERR);
 	}
 
-	if (ahc->pci_target_perr_count > AHC_PCI_TARGET_PERR_THRESH) {
+	if (ahc->pci_target_perr_count > AHC_PCI_TARGET_PERR_THRESH
+	 && (ahc->flags & AHC_DISABLE_PCI_PERR) == 0) {
 		printf(
 "%s: WARNING WARNING WARNING WARNING\n"
 "%s: Too many PCI parity errors observed as a target.\n"
-"%s: Some device on this bus is generating bad parity.\n"
-"%s: This is an error *observed by*, not *generated by*, this controller.\n"
+"%s: Some device on this PCI bus is generating bad parity.\n"
+"%s: This is an error *observed by*, not *generated by*, %s.\n"
 "%s: PCI parity error checking has been disabled.\n"
 "%s: WARNING WARNING WARNING WARNING\n",
 		       ahc_name(ahc), ahc_name(ahc), ahc_name(ahc),
-		       ahc_name(ahc), ahc_name(ahc), ahc_name(ahc));
+		       ahc_name(ahc), ahc_name(ahc), ahc_name(ahc),
+		       ahc_name(ahc));
 		ahc->seqctl |= FAILDIS;
+		ahc->flags |= AHC_DISABLE_PCI_PERR;
 		ahc_outb(ahc, SEQCTL, ahc->seqctl);
 	}
 	ahc_unpause(ahc);
