@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1998 - 2006 Sren Schmidt <sos@FreeBSD.org>
+ * Copyright (c) 1998 - 2007 Sren Schmidt <sos@FreeBSD.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/ata-all.c,v 1.252.2.12 2007/11/09 09:58:07 delphij Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/ata-all.c,v 1.280 2007/10/04 19:17:15 sos Exp $");
 
 #include "opt_ata.h"
 #include <sys/param.h>
@@ -47,9 +47,6 @@ __FBSDID("$FreeBSD: src/sys/dev/ata/ata-all.c,v 1.252.2.12 2007/11/09 09:58:07 d
 #include <machine/resource.h>
 #include <machine/bus.h>
 #include <sys/rman.h>
-#ifdef __alpha__
-#include <machine/md_var.h>
-#endif
 #include <dev/ata/ata-all.h>
 #include <ata_if.h>
 
@@ -125,7 +122,7 @@ ata_attach(device_t dev)
 
     /* reset the controller HW, the channel and device(s) */
     while (ATA_LOCKING(dev, ATA_LF_LOCK) != ch->unit)
-	tsleep(&error, PRIBIO, "ataatch", 1);
+	pause("ataatch", 1);
     ATA_RESET(dev);
     ATA_LOCKING(dev, ATA_LF_UNLOCK);
 
@@ -137,7 +134,7 @@ ata_attach(device_t dev)
 	device_printf(dev, "unable to allocate interrupt\n");
 	return ENXIO;
     }
-    if ((error = bus_setup_intr(dev, ch->r_irq, ATA_INTR_FLAGS,
+    if ((error = bus_setup_intr(dev, ch->r_irq, ATA_INTR_FLAGS, NULL,
 				(driver_intr_t *)ata_interrupt, ch, &ch->ih))) {
 	device_printf(dev, "unable to setup interrupt\n");
 	return error;
@@ -199,7 +196,7 @@ ata_reinit(device_t dev)
 
     /* poll for locking the channel */
     while (ATA_LOCKING(dev, ATA_LF_LOCK) != ch->unit)
-	tsleep(&dev, PRIBIO, "atarini", 1);
+	pause("atarini", 1);
 
     /* catch eventual request in ch->running */
     mtx_lock(&ch->state_mtx);
@@ -827,7 +824,7 @@ ata_udelay(int interval)
     if (1 || interval < (1000000/hz) || ata_delayed_attach)
 	DELAY(interval);
     else
-	tsleep(&interval, PRIBIO, "ataslp", interval/(1000000/hz));
+	pause("ataslp", interval/(1000000/hz));
 }
 
 char *
