@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1998 - 2006 Sren Schmidt <sos@FreeBSD.org>
+ * Copyright (c) 1998 - 2007 Sren Schmidt <sos@FreeBSD.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/ata-dma.c,v 1.137.2.4 2007/11/22 07:50:36 sos Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/ata-dma.c,v 1.147.2.1.2.1 2008/01/09 08:55:10 delphij Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -75,7 +75,7 @@ ata_dmainit(device_t dev)
 	ch->dma->load = ata_dmaload;
 	ch->dma->unload = ata_dmaunload;
 	ch->dma->alignment = 2;
-	ch->dma->boundary = 128 * DEV_BSIZE;
+	ch->dma->boundary = 65536;
 	ch->dma->segsize = 128 * DEV_BSIZE;
 	ch->dma->max_iosize = 128 * DEV_BSIZE;
 	ch->dma->max_address = BUS_SPACE_MAXADDR_32BIT;
@@ -97,7 +97,7 @@ ata_dmaalloc(device_t dev)
     struct ata_channel *ch = device_get_softc(dev);
     struct ata_dc_cb_args ccba;
 
-    if (bus_dma_tag_create(NULL, ch->dma->alignment, 0,
+    if (bus_dma_tag_create(bus_get_dma_tag(dev), ch->dma->alignment, 0,
 			   ch->dma->max_address, BUS_SPACE_MAXADDR,
 			   NULL, NULL, ch->dma->max_iosize,
 			   ATA_DMA_ENTRIES, ch->dma->segsize,
