@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/ata-pci.c,v 1.105.2.7 2007/11/22 07:50:36 sos Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/ata-pci.c,v 1.121.2.2 2007/11/21 21:15:00 sos Exp $");
 
 #include "opt_ata.h"
 #include <sys/param.h>
@@ -42,9 +42,6 @@ __FBSDID("$FreeBSD: src/sys/dev/ata/ata-pci.c,v 1.105.2.7 2007/11/22 07:50:36 so
 #include <machine/stdarg.h>
 #include <machine/resource.h>
 #include <machine/bus.h>
-#ifdef __alpha__
-#include <machine/md_var.h>
-#endif
 #include <sys/rman.h>
 #include <dev/pci/pcivar.h>
 #include <dev/pci/pcireg.h>
@@ -80,7 +77,7 @@ ata_pci_probe(device_t dev)
 	return ENXIO;
 
     /* if this is an AHCI chipset grab it */
-    if (pci_get_subclass(dev) == 0x06) {
+    if (pci_get_subclass(dev) == PCIS_STORAGE_SATA) {
 	if (!ata_ahci_ident(dev))
 	    return ATA_PROBE_OK;
     }
@@ -299,14 +296,10 @@ ata_pci_alloc_resource(device_t dev, device_t child, int type, int *rid,
     }
     if (type == SYS_RES_IRQ && *rid == ATA_IRQ_RID) {
 	if (ata_legacy(dev)) {
-#ifdef __alpha__
-	    res = alpha_platform_alloc_ide_intr(unit);
-#else
 	    int irq = (unit == 0 ? 14 : 15);
 	    
 	    res = BUS_ALLOC_RESOURCE(device_get_parent(dev), child,
 				     SYS_RES_IRQ, rid, irq, irq, 1, flags);
-#endif
 	}
 	else
 	    res = controller->r_irq;
@@ -342,12 +335,8 @@ ata_pci_release_resource(device_t dev, device_t child, int type, int rid,
 	    return ENOENT;
 
 	if (ata_legacy(dev)) {
-#ifdef __alpha__
-	    return alpha_platform_release_ide_intr(unit, r);
-#else
 	    return BUS_RELEASE_RESOURCE(device_get_parent(dev), child,
 					SYS_RES_IRQ, rid, r);
-#endif
 	}
 	else  
 	    return 0;
@@ -357,22 +346,21 @@ ata_pci_release_resource(device_t dev, device_t child, int type, int rid,
 
 int
 ata_pci_setup_intr(device_t dev, device_t child, struct resource *irq, 
-		   int flags, driver_intr_t *function, void *argument,
-		   void **cookiep)
+		   int flags, driver_filter_t *filter, driver_intr_t *function, 
+		   void *argument, void **cookiep)
 {
     if (ata_legacy(dev)) {
-#ifdef __alpha__
-	return alpha_platform_setup_ide_intr(child, irq, function, argument,
-					     cookiep);
-#else
 	return BUS_SETUP_INTR(device_get_parent(dev), child, irq,
-			      flags, function, argument, cookiep);
-#endif
+			      flags, filter, function, argument, cookiep);
     }
     else {
 	struct ata_pci_controller *controller = device_get_softc(dev);
 	int unit = ((struct ata_channel *)device_get_softc(child))->unit;
 
+	if (filter != NULL) {
+		printf("ata-pci.c: we cannot use a filter here\n");
+		return (EINVAL);
+	}
 	controller->interrupt[unit].function = function;
 	controller->interrupt[unit].argument = argument;
 	*cookiep = controller;
@@ -385,11 +373,7 @@ ata_pci_teardown_intr(device_t dev, device_t child, struct resource *irq,
 		      void *cookie)
 {
     if (ata_legacy(dev)) {
-#ifdef __alpha__
-	return alpha_platform_teardown_ide_intr(child, irq, cookie);
-#else
 	return BUS_TEARDOWN_INTR(device_get_parent(dev), child, irq, cookie);
-#endif
     }
     else {
 	struct ata_pci_controller *controller = device_get_softc(dev);
