@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1998 - 2006 Sren Schmidt <sos@FreeBSD.org>
+ * Copyright (c) 1998 - 2007 Sren Schmidt <sos@FreeBSD.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/ata-queue.c,v 1.50.2.8 2007/03/29 19:14:39 thomas Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/ata-queue.c,v 1.69 2007/03/13 20:31:56 thomas Exp $");
 
 #include "opt_ata.h"
 #include <sys/param.h>
