@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2006 Sren Schmidt <sos@FreeBSD.org>
+ * Copyright (c) 2006 - 2007 Sren Schmidt <sos@FreeBSD.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/ata-usb.c,v 1.5.2.1 2006/10/09 18:11:19 sos Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/ata-usb.c,v 1.7 2007/06/23 21:52:05 imp Exp $");
 
 #include "opt_ata.h"
 #include <sys/param.h>
@@ -45,6 +45,7 @@ __FBSDID("$FreeBSD: src/sys/dev/ata/ata-usb.c,v 1.5.2.1 2006/10/09 18:11:19 sos
 #include <machine/resource.h>
 #include <machine/bus.h>
 #include <sys/rman.h>
+#include <dev/usb/usb_port.h>
 #include <dev/usb/usb.h>
 #include <dev/usb/usbdi.h>
 #include <dev/usb/usbdi_util.h>
