@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/atapi-cd.c,v 1.179.2.9 2007/11/22 07:50:36 sos Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/atapi-cd.c,v 1.193.2.2 2007/11/21 21:15:00 sos Exp $");
 
 #include "opt_ata.h"
 #include <sys/param.h>
@@ -34,6 +34,7 @@ __FBSDID("$FreeBSD: src/sys/dev/ata/atapi-cd.c,v 1.179.2.9 2007/11/22 07:50:36 s
 #include <sys/kernel.h>
 #include <sys/module.h>
 #include <sys/malloc.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/bio.h>
 #include <sys/bus.h>
@@ -254,8 +255,11 @@ acd_geom_ioctl(struct g_provider *pp, u_long cmd, void *addr, int fflag, struct
 	cdp->flags |= F_LOCKED;
 	break;
 
+    /*
+     * XXXRW: Why does this require privilege?
+     */
     case CDIOCRESET:
-	error = suser(td);
+	error = priv_check(td, PRIV_DRIVER);
 	if (error)
 	    break;
 	error = acd_test_ready(dev);
@@ -703,7 +707,7 @@ acd_geom_access(struct g_provider *pp, int dr, int dw, int de)
 	     request->u.atapi.sense.key == 7) &&
 	    request->u.atapi.sense.asc == 4 &&
 	    request->u.atapi.sense.ascq == 1)
-	    tsleep(&timeout, PRIBIO, "acdld", hz / 2);
+	    pause("acdld", hz / 2);
 	else
 	    break;
     }
@@ -1088,7 +1092,7 @@ acd_fixate(device_t dev, int multisession)
     /* some drives just return ready, wait for the expected fixate time */
     if ((error = acd_test_ready(dev)) != EBUSY) {
 	timeout = timeout / (cdp->cap.cur_write_speed / 177);
-	tsleep(&error, PRIBIO, "acdfix", timeout * hz / 2);
+	pause("acdfix", timeout * hz / 2);
 	return acd_test_ready(dev);
     }
 
@@ -1097,7 +1101,7 @@ acd_fixate(device_t dev, int multisession)
 	    return error;
 	if ((error = acd_test_ready(dev)) != EBUSY)
 	    return error;
-	tsleep(&error, PRIBIO, "acdcld", hz / 2);
+	pause("acdcld", hz / 2);
     }
     return EIO;
 }
