@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1998 - 2006 Sren Schmidt <sos@FreeBSD.org>
+ * Copyright (c) 1998 - 2007 Sren Schmidt <sos@FreeBSD.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ata/atapi-tape.c,v 1.99.2.2 2007/11/22 07:50:37 sos Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ata/atapi-tape.c,v 1.103.2.1 2007/11/21 21:15:00 sos Exp $");
 
 #include "opt_ata.h"
 #include <sys/param.h>
@@ -609,7 +609,7 @@ ast_load_unload(device_t dev, u_int8_t function)
     error = ata_atapicmd(dev, ccb, NULL, 0, 0, 10);
     if (error)
 	return error;
-    tsleep((caddr_t)&error, PRIBIO, "astlu", 1 * hz);
+    pause("astlu", 1 * hz);
     if (function == ATAPI_SS_EJECT)
 	return 0;
     return ast_wait_dsc(dev, 60*60);
@@ -662,7 +662,7 @@ ast_wait_dsc(device_t dev, int timeout)
 	error = ata_atapicmd(dev, ccb, NULL, 0, 0, 0);
 	if (error != EBUSY)
 	    break;
-	tsleep(&error, PRIBIO, "atpwt", hz / 2);
+	pause("atpwt", hz / 2);
 	timeout -= (hz / 2);
     }
     return error;
