@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/atkbdc/atkbdc.c,v 1.21.2.2 2006/09/22 01:05:30 dwhite Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/atkbdc/atkbdc.c,v 1.25 2006/09/04 00:19:31 dwhite Exp $");
 
 #include "opt_kbd.h"
 
@@ -174,8 +174,6 @@ atkbdc_configure(void)
 	tag = I386_BUS_SPACE_IO;
 #elif defined(__amd64__)
 	tag = AMD64_BUS_SPACE_IO;
-#elif defined(__alpha__)
-	tag = busspace_isa_io;
 #elif defined(__ia64__)
 	tag = IA64_BUS_SPACE_IO;
 #elif defined(__sparc64__)
@@ -215,7 +213,7 @@ atkbdc_configure(void)
 	port0 = IO_KBD;
 	resource_int_value("atkbdc", 0, "port", &port0);
 	port1 = IO_KBD + KBD_STATUS_PORT;
-#if notyet
+#ifdef notyet
 	bus_space_map(tag, port0, IO_KBDSIZE, 0, &h0);
 	bus_space_map(tag, port1, IO_KBDSIZE, 0, &h1);
 #else
