@@ -30,7 +30,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/bge/if_bgereg.h,v 1.36.2.10 2007/03/21 22:53:22 jkim Exp $
+ * $FreeBSD: src/sys/dev/bge/if_bgereg.h,v 1.73 2007/05/22 19:22:58 jkim Exp $
  */
 
 /*
@@ -316,7 +316,9 @@
 #define	BGE_PCIDMARWCTL_MINDMA		0x000000FF
 #define	BGE_PCIDMARWCTL_RDADRR_BNDRY	0x00000700
 #define	BGE_PCIDMARWCTL_WRADDR_BNDRY	0x00003800
-#define	BGE_PCIDMARWCTL_ONEDMA_ATONCE	0x00004000
+#define	BGE_PCIDMARWCTL_ONEDMA_ATONCE	0x0000C000
+#define	BGE_PCIDMARWCTL_ONEDMA_ATONCE_GLOBAL	0x00004000
+#define	BGE_PCIDMARWCTL_ONEDMA_ATONCE_LOCAL	0x00008000
 #define	BGE_PCIDMARWCTL_RD_WAT		0x00070000
 #define	BGE_PCIDMARWCTL_WR_WAT		0x00380000
 #define	BGE_PCIDMARWCTL_USE_MRM		0x00400000
@@ -2079,6 +2081,11 @@ struct bge_status_block {
 #define	APPLE_VENDORID			0x106b
 #define	APPLE_DEVICE_BCM5701		0x1645
 
+/*
+ * Sun PCI vendor ID
+ */
+#define	SUN_VENDORID			0x108e
+
 /*
  * Offset of MAC address inside EEPROM.
  */
@@ -2101,9 +2108,6 @@ struct bge_status_block {
 #define	BGE_MEDIA_COPPER		0x00000010
 #define	BGE_MEDIA_FIBER			0x00000020
 
-#define	BGE_PCI_READ_CMD		0x06000000
-#define	BGE_PCI_WRITE_CMD		0x70000000
-
 #define	BGE_TICKS_PER_SEC		1000000
 
 /*
@@ -2454,6 +2458,7 @@ struct bge_softc {
 	uint32_t		bge_flags;
 #define	BGE_FLAG_TBI		0x00000001
 #define	BGE_FLAG_JUMBO		0x00000002
+#define	BGE_FLAG_EEPROM		0x00000004
 #define	BGE_FLAG_MSI		0x00000100
 #define	BGE_FLAG_PCIX		0x00000200
 #define	BGE_FLAG_PCIE		0x00000400
