@@ -32,7 +32,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/bktr/bktr_os.c,v 1.51.2.2 2006/10/17 03:45:54 jmg Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/bktr/bktr_os.c,v 1.54 2007/02/23 12:18:34 piso Exp $");
 
 /*
  * This is part of the Driver for Video Capture Cards (Frame grabbers)
@@ -384,7 +384,7 @@ bktr_attach( device_t dev )
 	}
 
 	error = bus_setup_intr(dev, bktr->res_irq, INTR_TYPE_TTY,
-                               bktr_intr, bktr, &bktr->res_ih);
+                               NULL, bktr_intr, bktr, &bktr->res_ih);
 	if (error) {
 		device_printf(dev, "could not setup irq\n");
 		goto fail;
