@@ -16,7 +16,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ce/if_ce.c,v 1.3.6.3 2007/05/12 06:10:09 nyan Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ce/if_ce.c,v 1.9 2007/07/27 11:59:56 rwatson Exp $");
 
 #include <sys/param.h>
 
@@ -29,6 +29,7 @@ __FBSDID("$FreeBSD: src/sys/dev/ce/if_ce.c,v 1.3.6.3 2007/05/12 06:10:09 nyan Ex
 #if NPCI > 0
 
 #include <sys/ucred.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/systm.h>
 #include <sys/mbuf.h>
@@ -68,13 +69,6 @@ __FBSDID("$FreeBSD: src/sys/dev/ce/if_ce.c,v 1.3.6.3 2007/05/12 06:10:09 nyan Ex
 #   include <net/if_types.h>
 #   include <net/if_sppp.h>
 #   define PP_CISCO IFF_LINK2
-#   if __FreeBSD_version < 500000
-#	include <bpf.h>
-#	define NBPFILTER NBPF
-#   else
-#	include "opt_bpf.h"
-#	define NBPFILTER DEV_BPF
-#   endif
 #   include <net/bpf.h>
 #endif
 #include <dev/cx/machdep.h>
@@ -666,7 +660,7 @@ static int ce_attach (device_t dev)
 #else
 				INTR_TYPE_NET,
 #endif
-				ce_intr, bd, &bd->ce_intrhand);
+				NULL, ce_intr, bd, &bd->ce_intrhand);
 	if (error) {
 		printf ("ce%d: cannot set up irq\n", unit);
 		bus_release_resource (dev, SYS_RES_IRQ, 0, bd->ce_irq);
@@ -843,10 +837,9 @@ static int ce_detach (device_t dev)
 		if (! d || ! d->chan)
 			continue;
 #ifndef NETGRAPH
-#if __FreeBSD_version >= 410000 && NBPFILTER > 0
 		/* Detach from the packet filter list of interfaces. */
 		bpfdetach (d->ifp);
-#endif
+
 		/* Detach from the sync PPP list. */
 		sppp_detach (d->ifp);
 
@@ -1340,9 +1333,11 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else /* __FreeBSD_version >= 500000 */
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
-#endif /* __FreeBSD_version >= 500000 */
+#else
+		error = priv_check (td, PRIV_DRIVER);
+#endif
 		if (error)
 			return error;
 #if __FreeBSD_version >= 600034
@@ -1379,8 +1374,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1407,8 +1404,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1425,8 +1424,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		CE_DEBUG2 (d, ("ioctl: setcfg\n"));
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1525,8 +1526,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1559,8 +1562,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1585,8 +1590,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1607,8 +1614,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1633,8 +1642,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1657,8 +1668,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1685,8 +1698,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1707,8 +1722,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1733,8 +1750,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1757,8 +1776,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1783,8 +1804,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1809,8 +1832,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1835,8 +1860,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1866,8 +1893,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1891,8 +1920,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1908,8 +1939,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -1944,8 +1977,10 @@ static int ce_ioctl (struct cdev *dev, u_long cmd, caddr_t data, int flag, struc
 		/* Only for superuser! */
 #if __FreeBSD_version < 500000
 		error = suser (p);
-#else
+#elsif __FreeBSD_version < 700000
 		error = suser (td);
+#else
+		error = priv_check (td, PRIV_DRIVER);
 #endif
 		if (error)
 			return error;
@@ -2569,13 +2604,6 @@ static int ce_modevent (module_t mod, int type, void *unused)
 #if __FreeBSD_version < 500000
 	dev = makedev (CDEV_MAJOR, 0);
 #endif
-#if __FreeBSD_version >= 501114
-	if (!debug_mpsafenet && ce_mpsafenet) {
-		printf ("WORNING! Network stack is not MPSAFE. "
-			"Turning off debug.ce.mpsafenet.\n");
-		ce_mpsafenet = 0;
-	}
-#endif
 #if __FreeBSD_version >= 502103
 	if (ce_mpsafenet)
 		ce_cdevsw.d_flags &= ~D_NEEDGIANT;
