@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/ciss/cissreg.h,v 1.15.2.1 2005/09/16 17:59:48 ps Exp $
+ *	$FreeBSD: src/sys/dev/ciss/cissreg.h,v 1.16 2005/08/09 20:53:51 ps Exp $
  */
 
 /*
