@@ -1,3 +1,4 @@
+
 /**************************************************************************
 
 Copyright (c) 2007, Chelsio Inc.
@@ -28,7 +29,7 @@ POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/cxgb/common/cxgb_xgmac.c,v 1.1.2.5.2.1 2007/12/06 00:02:00 kmacy Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/cxgb/common/cxgb_xgmac.c,v 1.7 2007/09/09 01:28:03 kmacy Exp $");
 
 #ifdef CONFIG_DEFINED
 #include <cxgb_include.h>
