@@ -28,7 +28,7 @@ POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/cxgb/cxgb_main.c,v 1.3.2.9.2.1 2007/12/05 23:46:44 kmacy Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/cxgb/cxgb_main.c,v 1.36 2007/09/11 23:49:27 kmacy Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -412,6 +412,7 @@ cxgb_controller_attach(device_t dev)
 		pectl = (pectl & ~0x7000) | (5 << 12);
 		pci_write_config(dev, reg + 0x8, pectl, 2);
 	}
+
 	if (sc->link_width != 0 && sc->link_width <= 4 &&
 	    (ai->nports0 + ai->nports1) <= 2) {
 		device_printf(sc->dev,
@@ -1201,7 +1202,7 @@ setup_rss(adapter_t *adap)
 	u_int nq[2]; 
 	uint8_t cpus[SGE_QSETS + 1];
 	uint16_t rspq_map[RSS_TABLE_SIZE];
-
+	
 	for (i = 0; i < SGE_QSETS; ++i)
 		cpus[i] = i;
 	cpus[SGE_QSETS] = 0xff;
@@ -1334,7 +1335,7 @@ update_tpeeprom(struct adapter *adap)
 	const struct firmware *tpeeprom;
 #else
 	struct firmware *tpeeprom;
-#endif	
+#endif
 
 	char buf[64];
 	uint32_t version;
