@@ -28,7 +28,7 @@ POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/cxgb/cxgb_sge.c,v 1.2.2.8.2.1 2007/12/05 23:46:44 kmacy Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/cxgb/cxgb_sge.c,v 1.30 2007/09/09 04:34:03 kmacy Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -523,25 +523,20 @@ refill_fl_cb(void *arg, bus_dma_segment_t *segs, int nseg, int error)
  *	The caller must assure that @n does not exceed the queue's capacity.
  */
 static void
-refill_fl(adapter_t *sc, struct sge_fl *q, int n, int blocking)
+refill_fl(adapter_t *sc, struct sge_fl *q, int n)
 {
 	struct rx_sw_desc *sd = &q->sdesc[q->pidx];
 	struct rx_desc *d = &q->desc[q->pidx];
 	struct refill_fl_cb_arg cb_arg;
 	void *cl;
 	int err;
-#ifdef notyet	
-	int flags = blocking ? M_TRYWAIT : M_DONTWAIT;
-#else
-	int flags = M_DONTWAIT;
-#endif
-	
+
 	cb_arg.error = 0;
 	while (n--) {
 		/*
 		 * We only allocate a cluster, mbuf allocation happens after rx
 		 */
-		if ((cl = m_cljget(NULL, flags, q->buf_size)) == NULL) {
+		if ((cl = m_cljget(NULL, M_DONTWAIT, q->buf_size)) == NULL) {
 			log(LOG_WARNING, "Failed to allocate cluster\n");
 			goto done;
 		}
@@ -618,7 +613,7 @@ free_rx_bufs(adapter_t *sc, struct sge_fl *q)
 static __inline void
 __refill_fl(adapter_t *adap, struct sge_fl *fl)
 {
-	refill_fl(adap, fl, min(16U, fl->size - fl->credits), 0);
+	refill_fl(adap, fl, min(16U, fl->size - fl->credits));
 }
 
 #ifndef DISABLE_MBUF_IOVEC
@@ -898,8 +893,7 @@ sge_timer_reclaim(void *arg, int ncount)
 		
 		lock = (sc->flags & USING_MSIX) ? &qs->rspq.lock :
 			    &sc->sge.qs[0].rspq.lock;
-		
-	retry:
+
 		if (mtx_trylock(lock)) {
 			/* XXX currently assume that we are *NOT* polling */
 			uint32_t status = t3_read_reg(sc, A_SG_RSPQ_FL_STATUS);
@@ -919,9 +913,6 @@ sge_timer_reclaim(void *arg, int ncount)
 			}
 			mtx_unlock(lock);
 		}
-		if ((qs->fl[0].credits < qs->fl[0].size - 16) ||
-		    (qs->fl[1].credits < qs->fl[1].size - 16))
-			goto retry;
 	}
 }
 
@@ -2226,8 +2217,8 @@ t3_sge_alloc_qset(adapter_t *sc, u_int id, int nports, int irq_vec_idx,
 	t3_update_qset_coalesce(q, p);
 	q->port = pi;
 	
-	refill_fl(sc, &q->fl[0], q->fl[0].size, 1);
-	refill_fl(sc, &q->fl[1], q->fl[1].size, 1);
+	refill_fl(sc, &q->fl[0], q->fl[0].size);
+	refill_fl(sc, &q->fl[1], q->fl[1].size);
 	refill_rspq(sc, &q->rspq, q->rspq.size - 1);
 
 	t3_write_reg(sc, A_SG_GTS, V_RSPQ(q->rspq.cntxt_id) |
@@ -2271,9 +2262,7 @@ t3_rx_eth(struct adapter *adap, struct sge_rspq *rq, struct mbuf *m, int ethpad)
 	
 	m->m_pkthdr.rcvif = ifp;
 	m->m_pkthdr.header = mtod(m, uint8_t *) + sizeof(*cpl) + ethpad;
-#ifndef DISABLE_MBUF_IOVEC
 	m_explode(m);
-#endif	   
 	/*
 	 * adjust after conversion to mbuf chain
 	 */
@@ -2607,8 +2596,8 @@ process_responses(adapter_t *adap, struct sge_qset *qs, int budget)
 			if (eth) {				
 				t3_rx_eth_lro(adap, rspq, rspq->rspq_mh.mh_head, ethpad,
 				    rss_hash, rss_csum, lro);
-				
-				rspq->rspq_mh.mh_tail = rspq->rspq_mh.mh_head = NULL;
+
+				rspq->rspq_mh.mh_head = NULL;
 			} else {
 				rspq->rspq_mh.mh_head->m_pkthdr.csum_data = rss_csum;
 				/*
@@ -2619,12 +2608,10 @@ process_responses(adapter_t *adap, struct sge_qset *qs, int budget)
 				ngathered = rx_offload(&adap->tdev, rspq,
 				    rspq->rspq_mh.mh_head, offload_mbufs, ngathered);
 			}
-
 			__refill_fl(adap, &qs->fl[0]);
 			__refill_fl(adap, &qs->fl[1]);
+
 		}
-
-
 		--budget_left;
 	}
 
@@ -2639,10 +2626,6 @@ process_responses(adapter_t *adap, struct sge_qset *qs, int budget)
 		restart_tx(qs);
 
 	budget -= budget_left;
-#if 0	
-	refill_fl(adap, &qs->fl[0], &qs->fl[0].size - &qs->fl[0].credits, 1);
-	refill_fl(adap, &qs->fl[1], &qs->fl[1].size - &qs->fl[1].credits, 1);
-#endif	
 	return (budget);
 }
 
