@@ -24,10 +24,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/digi/CX.fepos.h,v 1.1 2001/05/02 01:08:02 brian Exp $
+ * $FreeBSD: src/sys/dev/digi/CX.fepos.h,v 1.2 2005/05/06 20:53:33 anholt Exp $
  */
 
-const u_char CX_fepos[] = {
+static const u_char CX_fepos[] = {
   0x4f, 0x53, 0x31, 0x5f, 0xe9, 0xf9, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x28, 0x23, 0x29, 0x63, 0x78, 0x66,
