@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/digi/digi.c,v 1.58.2.3 2007/02/15 19:49:27 luigi Exp $
+ * $FreeBSD: src/sys/dev/digi/digi.c,v 1.63 2006/09/27 19:56:58 ru Exp $
  */
 
 /*-
@@ -360,7 +360,7 @@ digi_init(struct digi_softc *sc)
 	DLOG(DIGIDB_INIT, (sc->dev, "BIOS started after %d us\n", i));
 
 	for (i = 0; vW(ptr) != *(u_short *)"GD"; i++) {
-		if (i > 2*hz) {
+		if (i > 5*hz) {
 			log(LOG_ERR, "digi%d: BIOS boot failed "
 			    "(0x%02x != 0x%02x)\n",
 			    sc->res.unit, vW(ptr), *(u_short *)"GD");
@@ -588,7 +588,7 @@ digi_init(struct digi_softc *sc)
 
 		ttyinitmode(tp, 0, 0);
 		port->send_ring = 1;	/* Default action on signal RI */
-		ttycreate(tp, NULL, 0, MINOR_CALLOUT, "D%r%r", sc->res.unit, i);
+		ttycreate(tp, TS_CALLOUT, "D%r%r", sc->res.unit, i);
 	}
 
 	sc->hidewin(sc);
@@ -913,7 +913,10 @@ digiioctl(struct tty *tp, u_long cmd, void *data, int flag, struct thread *td)
 {
 	struct digi_softc *sc;
 	struct digi_p *port;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
 	int ival;
+#endif
 
 	port = tp->t_sc;
 	sc = port->sc;
@@ -943,10 +946,13 @@ digiioctl(struct tty *tp, u_long cmd, void *data, int flag, struct thread *td)
 			}
 		}
 		return (0);
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
 	case _IO('e', 'C'):
 		ival = IOCPARM_IVAL(data);
 		data = &ival;
 		/* FALLTHROUGH */
+#endif
 	case DIGIIO_RING:
 		port->send_ring = (u_char)*(int *)data;
 		break;
