@@ -26,7 +26,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/dpt/dpt_eisa.c,v 1.20.2.1 2006/12/14 02:57:06 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/dpt/dpt_eisa.c,v 1.24 2007/06/17 05:55:49 scottl Exp $");
+
+#include "opt_eisa.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -107,6 +109,7 @@ dpt_eisa_attach (device_t dev)
 	int		error = 0;
 
 	dpt = device_get_softc(dev);
+	dpt->dev = dev;
 
 	dpt->io_rid = 0;
 	dpt->io_type = SYS_RES_IOPORT;
@@ -153,7 +156,7 @@ dpt_eisa_attach (device_t dev)
 	splx(s);
 
 	if (bus_setup_intr(dev, dpt->irq_res, INTR_TYPE_CAM | INTR_ENTROPY,
-			   dpt_intr, dpt, &dpt->ih)) {
+			   NULL, dpt_intr, dpt, &dpt->ih)) {
 		device_printf(dev, "Unable to register interrupt handler\n");
 		error = ENXIO;
 		goto bad;
