@@ -28,7 +28,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/dpt/dpt_pci.c,v 1.34.2.1 2006/12/14 02:57:06 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/dpt/dpt_pci.c,v 1.37 2007/06/17 05:55:49 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -81,6 +81,7 @@ dpt_pci_attach (device_t dev)
 	u_int32_t	command;
 
 	dpt = device_get_softc(dev);
+	dpt->dev = dev;
 
 	command = pci_read_config(dev, PCIR_COMMAND, /*bytes*/1);
 
@@ -163,7 +164,7 @@ dpt_pci_attach (device_t dev)
 	splx(s);
 
 	if (bus_setup_intr(dev, dpt->irq_res, INTR_TYPE_CAM | INTR_ENTROPY,
-			   dpt_intr, dpt, &dpt->ih)) {
+			   NULL, dpt_intr, dpt, &dpt->ih)) {
 		device_printf(dev, "Unable to register interrupt handler\n");
 		error = ENXIO;
 		goto bad;
