@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/drm/i915_dma.c,v 1.1.2.4 2007/09/10 12:51:06 kib Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/drm/i915_dma.c,v 1.6 2007/08/21 12:52:57 kib Exp $");
 
 #include "dev/drm/drmP.h"
 #include "dev/drm/drm.h"
@@ -39,6 +39,7 @@ __FBSDID("$FreeBSD: src/sys/dev/drm/i915_dma.c,v 1.1.2.4 2007/09/10 12:51:06 kib
 			dev->pci_device == 0x2992 || \
 			dev->pci_device == 0x29A2)
 
+
 /* Really want an OS-independent resettable timer.  Would like to have
  * this loop run for (eg) 3 sec, but have the timer reset every time
  * the head pointer changes, so that EBUSY only happens if the ring
