@@ -30,10 +30,9 @@
   POSSIBILITY OF SUCH DAMAGE.
 
 *******************************************************************************/
-/* $FreeBSD: src/sys/dev/em/e1000_80003es2lan.c,v 1.3.2.1 2007/10/05 22:49:06 jfv Exp $ */
+/* $FreeBSD: src/sys/dev/em/e1000_80003es2lan.c,v 1.3.4.1 2007/11/28 23:24:37 jfv Exp $ */
 
-/*
- * e1000_80003es2lan
+/* e1000_80003es2lan
  */
 
 #include "e1000_api.h"
@@ -71,6 +70,7 @@ static s32  e1000_cfg_kmrn_1000_80003es2lan(struct e1000_hw *hw);
 static s32  e1000_copper_link_setup_gg82563_80003es2lan(struct e1000_hw *hw);
 static void e1000_initialize_hw_bits_80003es2lan(struct e1000_hw *hw);
 static void e1000_release_swfw_sync_80003es2lan(struct e1000_hw *hw, u16 mask);
+STATIC void e1000_power_down_phy_copper_80003es2lan(struct e1000_hw *hw);
 
 /*
  * A table for the GG82563 cable length where the range is defined
@@ -100,6 +100,9 @@ STATIC s32 e1000_init_phy_params_80003es2lan(struct e1000_hw *hw)
 	if (hw->phy.media_type != e1000_media_type_copper) {
 		phy->type        = e1000_phy_none;
 		goto out;
+	} else {
+		func->power_up_phy = e1000_power_up_phy_copper;
+		func->power_down_phy = e1000_power_down_phy_copper_80003es2lan;
 	}
 
 	phy->addr                = 1;
@@ -177,6 +180,10 @@ STATIC s32 e1000_init_nvm_params_80003es2lan(struct e1000_hw *hw)
 	 * for setting word_size.
 	 */
 	size += NVM_WORD_SIZE_BASE_SHIFT;
+
+	/* EEPROM access above 16k is unsupported */
+	if (size > 14)
+		size = 14;
 	nvm->word_size	= 1 << size;
 
 	/* Function Pointers */
@@ -1051,7 +1058,7 @@ static s32 e1000_copper_link_setup_gg82563_80003es2lan(struct e1000_hw *hw)
 
 	}
 
-	/* Bypass RX and TX FIFO's */
+	/* Bypass Rx and Tx FIFO's */
 	ret_val = e1000_write_kmrn_reg(hw,
 	                        E1000_KMRNCTRLSTA_OFFSET_FIFO_CTRL,
 	                        E1000_KMRNCTRLSTA_FIFO_CTRL_RX_BYPASS |
@@ -1296,6 +1303,22 @@ out:
 	return ret_val;
 }
 
+/**
+ * e1000_power_down_phy_copper_80003es2lan - Remove link during PHY power down
+ * @hw: pointer to the HW structure
+ *
+ * In the case of a PHY power down to save power, or to turn off link during a
+ * driver unload, or wake on lan is not enabled, remove the link.
+ **/
+STATIC void e1000_power_down_phy_copper_80003es2lan(struct e1000_hw *hw)
+{
+	/* If the management interface is not enabled, then power down */
+	if (!(e1000_check_mng_mode(hw) || e1000_check_reset_block(hw)))
+		e1000_power_down_phy_copper(hw);
+
+	return;
+}
+
 /**
  *  e1000_clear_hw_cntrs_80003es2lan - Clear device specific hardware counters
  *  @hw: pointer to the HW structure
