@@ -30,7 +30,8 @@
   POSSIBILITY OF SUCH DAMAGE.
 
 *******************************************************************************/
-/* $FreeBSD: src/sys/dev/em/e1000_api.c,v 1.3.2.1 2007/10/05 22:49:06 jfv Exp $ */
+/* $FreeBSD: src/sys/dev/em/e1000_api.c,v 1.3.4.1 2007/11/28 23:24:37 jfv Exp $ */
+
 
 #include "e1000_api.h"
 #include "e1000_mac.h"
@@ -1053,18 +1054,18 @@ s32 e1000_read_mac_addr(struct e1000_hw *hw)
 }
 
 /**
- *  e1000_read_part_num - Read device part number
+ *  e1000_read_pba_num - Read device part number
  *  @hw: pointer to the HW structure
- *  @part_num: pointer to device part number
+ *  @pba_num: pointer to device part number
  *
  *  Reads the product board assembly (PBA) number from the EEPROM and stores
- *  the value in part_num.
+ *  the value in pba_num.
  *  Currently no func pointer exists and all implementations are handled in the
  *  generic version of this function.
  **/
-s32 e1000_read_part_num(struct e1000_hw *hw, u32 *part_num)
+s32 e1000_read_pba_num(struct e1000_hw *hw, u32 *pba_num)
 {
-	return e1000_read_part_num_generic(hw, part_num);
+	return e1000_read_pba_num_generic(hw, pba_num);
 }
 
 /**
@@ -1160,3 +1161,32 @@ s32 e1000_write_8bit_ctrl_reg(struct e1000_hw *hw, u32 reg, u32 offset, u8 data)
 {
 	return e1000_write_8bit_ctrl_reg_generic(hw, reg, offset, data);
 }
+
+/**
+ * e1000_power_up_phy - Restores link in case of PHY power down
+ * @hw: pointer to the HW structure
+ *
+ * The phy may be powered down to save power, to turn off link when the
+ * driver is unloaded, or wake on lan is not enabled (among others).
+ **/
+void e1000_power_up_phy(struct e1000_hw *hw)
+{
+	if (hw->func.power_up_phy)
+		hw->func.power_up_phy(hw);
+
+	e1000_setup_link(hw);
+}
+
+/**
+ * e1000_power_down_phy - Power down PHY 
+ * @hw: pointer to the HW structure
+ *
+ * The phy may be powered down to save power, to turn off link when the
+ * driver is unloaded, or wake on lan is not enabled (among others).
+ **/
+void e1000_power_down_phy(struct e1000_hw *hw)
+{
+	if (hw->func.power_down_phy)
+		hw->func.power_down_phy(hw);
+}
+
