@@ -30,7 +30,7 @@
   POSSIBILITY OF SUCH DAMAGE.
 
 *******************************************************************************/
-/* $FreeBSD: src/sys/dev/em/e1000_hw.h,v 1.3.2.1 2007/10/05 22:49:07 jfv Exp $ */
+/* $FreeBSD: src/sys/dev/em/e1000_hw.h,v 1.3.4.1 2007/11/28 23:24:37 jfv Exp $ */
 
 
 #ifndef _E1000_HW_H_
@@ -566,6 +566,8 @@ struct e1000_functions {
 	s32  (*set_d0_lplu_state)(struct e1000_hw *, bool);
 	s32  (*set_d3_lplu_state)(struct e1000_hw *, bool);
 	s32  (*write_phy_reg)(struct e1000_hw *, u32, u16);
+	void (*power_up_phy)(struct e1000_hw *);
+	void (*power_down_phy)(struct e1000_hw *);
 
 	/* Function pointers for the NVM. */
 	s32  (*init_nvm_params)(struct e1000_hw *);
