@@ -30,7 +30,7 @@
   POSSIBILITY OF SUCH DAMAGE.
 
 *******************************************************************************/
-/* $FreeBSD: src/sys/dev/em/e1000_ich8lan.c,v 1.3.2.1 2007/10/05 22:49:07 jfv Exp $ */
+/* $FreeBSD: src/sys/dev/em/e1000_ich8lan.c,v 1.3.4.1 2007/11/28 23:24:37 jfv Exp $ */
 
 
 /* e1000_ich8lan
@@ -93,6 +93,7 @@ STATIC s32  e1000_write_flash_byte_ich8lan(struct e1000_hw *hw,
 static s32  e1000_write_flash_data_ich8lan(struct e1000_hw *hw, u32 offset,
                                            u8 size, u16 data);
 STATIC s32 e1000_get_cfg_done_ich8lan(struct e1000_hw *hw);
+STATIC void e1000_power_down_phy_copper_ich8lan(struct e1000_hw *hw);
 
 /* ICH GbE Flash Hardware Sequencing Flash Status Register bit breakdown */
 /* Offset 04h HSFSTS */
@@ -176,6 +177,8 @@ STATIC s32 e1000_init_phy_params_ich8lan(struct e1000_hw *hw)
 	func->set_d0_lplu_state         = e1000_set_d0_lplu_state_ich8lan;
 	func->set_d3_lplu_state         = e1000_set_d3_lplu_state_ich8lan;
 	func->write_phy_reg             = e1000_write_phy_reg_igp;
+	func->power_up_phy              = e1000_power_up_phy_copper;
+	func->power_down_phy            = e1000_power_down_phy_copper_ich8lan;
 
 
 	phy->id = 0;
@@ -990,6 +993,24 @@ out:
 	return ret_val;
 }
 
+/**
+ *  e1000_valid_nvm_bank_detect_ich8lan - finds out the valid bank 0 or 1
+ *  @hw: pointer to the HW structure
+ *  @bank:  pointer to the variable that returns the active bank
+ *
+ *  Reads signature byte from the NVM using the flash access registers.
+ **/
+STATIC s32 e1000_valid_nvm_bank_detect_ich8lan(struct e1000_hw *hw, u32 *bank)
+{
+	s32 ret_val = E1000_SUCCESS;
+	if (E1000_READ_REG(hw, E1000_EECD) & E1000_EECD_SEC1VAL)
+		*bank = 1;
+	else
+		*bank = 0;
+	
+	return ret_val;
+}
+
 /**
  *  e1000_read_nvm_ich8lan - Read word(s) from the NVM
  *  @hw: pointer to the HW structure
@@ -1006,6 +1027,7 @@ STATIC s32 e1000_read_nvm_ich8lan(struct e1000_hw *hw, u16 offset, u16 words,
 	struct e1000_dev_spec_ich8lan *dev_spec;
 	u32 act_offset;
 	s32 ret_val = E1000_SUCCESS;
+	u32 bank = 0;
 	u16 i, word;
 
 	DEBUGFUNC("e1000_read_nvm_ich8lan");
@@ -1029,10 +1051,11 @@ STATIC s32 e1000_read_nvm_ich8lan(struct e1000_hw *hw, u16 offset, u16 words,
 	if (ret_val)
 		goto out;
 
-	/* Start with the bank offset, then add the relative offset. */
-	act_offset = (E1000_READ_REG(hw, E1000_EECD) & E1000_EECD_SEC1VAL)
-		     ? nvm->flash_bank_size
-		     : 0;
+	ret_val = e1000_valid_nvm_bank_detect_ich8lan(hw, &bank);
+	if (ret_val != E1000_SUCCESS)
+		goto out;
+
+	act_offset = (bank) ? nvm->flash_bank_size : 0;
 	act_offset += offset;
 
 	for (i = 0; i < words; i++) {
@@ -1347,7 +1370,7 @@ STATIC s32 e1000_update_nvm_checksum_ich8lan(struct e1000_hw *hw)
 {
 	struct e1000_nvm_info *nvm = &hw->nvm;
 	struct e1000_dev_spec_ich8lan *dev_spec;
-	u32 i, act_offset, new_bank_offset, old_bank_offset;
+	u32 i, act_offset, new_bank_offset, old_bank_offset, bank;
 	s32 ret_val;
 	u16 data;
 
@@ -1371,7 +1394,11 @@ STATIC s32 e1000_update_nvm_checksum_ich8lan(struct e1000_hw *hw)
 	 * write to bank 0 etc.  We also need to erase the segment that
 	 * is going to be written
 	 */
-	if (!(E1000_READ_REG(hw, E1000_EECD) & E1000_EECD_SEC1VAL)) {
+	ret_val =  e1000_valid_nvm_bank_detect_ich8lan(hw, &bank);
+	if (ret_val != E1000_SUCCESS)
+		goto out;
+
+	if (bank == 0) {
 		new_bank_offset = nvm->flash_bank_size;
 		old_bank_offset = 0;
 		e1000_erase_flash_bank_ich8lan(hw, 1);
@@ -2149,6 +2176,29 @@ STATIC s32 e1000_setup_copper_link_ich8lan(struct e1000_hw *hw)
 			goto out;
 	}
 
+	if (hw->phy.type == e1000_phy_ife) {
+		ret_val = e1000_read_phy_reg(hw, IFE_PHY_MDIX_CONTROL, &reg_data);
+		if (ret_val)
+			goto out;
+
+		reg_data &= ~IFE_PMC_AUTO_MDIX;
+
+		switch (hw->phy.mdix) {
+		case 1:
+			reg_data &= ~IFE_PMC_FORCE_MDIX;
+			break;
+		case 2:
+			reg_data |= IFE_PMC_FORCE_MDIX;
+			break;
+		case 0:
+		default:
+			reg_data |= IFE_PMC_AUTO_MDIX;
+			break;
+		}
+		ret_val = e1000_write_phy_reg(hw, IFE_PHY_MDIX_CONTROL, reg_data);
+		if (ret_val)
+			goto out;
+	}
 	ret_val = e1000_setup_copper_link_generic(hw);
 
 out:
@@ -2479,6 +2529,8 @@ STATIC s32 e1000_led_off_ich8lan(struct e1000_hw *hw)
  **/
 STATIC s32 e1000_get_cfg_done_ich8lan(struct e1000_hw *hw)
 {
+	s32 ret_val = E1000_SUCCESS;
+
 	e1000_get_cfg_done_generic(hw);
 
 	/* If EEPROM is not marked present, init the IGP 3 PHY manually */
@@ -2487,7 +2539,23 @@ STATIC s32 e1000_get_cfg_done_ich8lan(struct e1000_hw *hw)
 		e1000_phy_init_script_igp3(hw);
 	}
 
-	return E1000_SUCCESS;
+	return ret_val;
+}
+
+/**
+ * e1000_power_down_phy_copper_ich8lan - Remove link during PHY power down
+ * @hw: pointer to the HW structure
+ *
+ * In the case of a PHY power down to save power, or to turn off link during a
+ * driver unload, or wake on lan is not enabled, remove the link.
+ **/
+STATIC void e1000_power_down_phy_copper_ich8lan(struct e1000_hw *hw)
+{
+	/* If the management interface is not enabled, then power down */
+	if (!(e1000_check_mng_mode(hw) || e1000_check_reset_block(hw)))
+		e1000_power_down_phy_copper(hw);
+
+	return;
 }
 
 /**
