@@ -30,13 +30,18 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 ***************************************************************************/
-/* $FreeBSD: src/sys/dev/em/if_em.h,v 1.32.2.8 2007/11/07 01:33:28 jfv Exp $ */
+/* $FreeBSD: src/sys/dev/em/if_em.h,v 1.62.2.1 2007/11/28 23:24:38 jfv Exp $ */
 
 #ifndef _EM_H_DEFINED_
 #define _EM_H_DEFINED_
 
 /* Tunables */
 
+/* Set FAST handling on by default */
+#if __FreeBSD_version > 700000
+#define EM_FAST_IRQ
+#endif
+
 /*
  * EM_TXD: Maximum number of Transmit Descriptors
  * Valid Range: 80-256 for 82542 and 82543-based adapters
@@ -135,7 +140,7 @@ POSSIBILITY OF SUCH DAMAGE.
 /*
  * This parameter controls the duration of transmit watchdog timer.
  */
-#define EM_TX_TIMEOUT                   10    /* set to 10 seconds */
+#define EM_TX_TIMEOUT                   5
 
 /*
  * This parameter controls when the driver calls the routine to reclaim
@@ -185,7 +190,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #define MAX_NUM_MULTICAST_ADDRESSES     128
 #define PCI_ANY_ID                      (~0U)
 #define ETHER_ALIGN                     2
-#define EM_TX_BUFFER_SIZE		((uint32_t) 1514)
 #define EM_FC_PAUSE_TIME		0x0680
 #define EM_EEPROM_APME			0x400;
 
@@ -231,7 +235,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #define HW_DEBUGOUT2(S, A, B)       if (DEBUG_HW) printf(S "\n", A, B)
 
 #define EM_MAX_SCATTER		64
-#define EM_TSO_SIZE		65535	/* maxsize of a dma transfer */
+#define EM_TSO_SIZE		(65535 + sizeof(struct ether_vlan_header))
 #define EM_TSO_SEG_SIZE		4096	/* Max dma segment size */
 #define ETH_ZLEN		60
 #define ETH_ADDR_LEN		6
@@ -421,9 +425,9 @@ typedef struct _DESCRIPTOR_PAIR
 } DESC_ARRAY, *PDESC_ARRAY;
 
 #define	EM_CORE_LOCK_INIT(_sc, _name) \
-	mtx_init(&(_sc)->core_mtx, _name, MTX_NETWORK_LOCK, MTX_DEF)
+	mtx_init(&(_sc)->core_mtx, _name, "EM Core Lock", MTX_DEF)
 #define	EM_TX_LOCK_INIT(_sc, _name) \
-	mtx_init(&(_sc)->tx_mtx, _name, MTX_NETWORK_LOCK, MTX_DEF)
+	mtx_init(&(_sc)->tx_mtx, _name, "EM TX Lock", MTX_DEF)
 #define	EM_CORE_LOCK_DESTROY(_sc)	mtx_destroy(&(_sc)->core_mtx)
 #define	EM_TX_LOCK_DESTROY(_sc)		mtx_destroy(&(_sc)->tx_mtx)
 #define	EM_CORE_LOCK(_sc)		mtx_lock(&(_sc)->core_mtx)
