@@ -32,7 +32,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/en/midway.c,v 1.65.2.2 2007/01/19 23:01:28 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/en/midway.c,v 1.73 2007/05/29 11:28:27 rwatson Exp $");
 
 /*
  *
@@ -90,7 +90,7 @@ __FBSDID("$FreeBSD: src/sys/dev/en/midway.c,v 1.65.2.2 2007/01/19 23:01:28 jhb E
  */
 #define DBG(SC, FL, PRINT) do {						\
 	if ((SC)->debug & DBG_##FL) {					\
-		if_printf((SC)->ifp, "%s: "#FL": ", __func__);	\
+		device_printf((SC)->dev, "%s: "#FL": ", __func__);	\
 		printf PRINT;						\
 		printf("\n");						\
 	}								\
@@ -126,7 +126,6 @@ enum {
 
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/kdb.h>
 #include <sys/queue.h>
 #include <sys/sockio.h>
 #include <sys/socket.h>
@@ -395,7 +394,7 @@ en_dump_packet(struct en_softc *sc, struct mbuf *m)
 	int len;
 	u_char *ptr;
 
-	if_printf(sc->ifp, "packet len=%d", plen);
+	device_printf(sc->dev, "packet len=%d", plen);
 	while (m != NULL) {
 		totlen += m->m_len;
 		ptr = mtod(m, u_char *);
@@ -445,7 +444,7 @@ en_map_ctor(void *mem, int size, void *arg, int flags)
 
 	err = bus_dmamap_create(sc->txtag, 0, &map->map);
 	if (err != 0) {
-		if_printf(sc->ifp, "cannot create DMA map %d\n", err);
+		device_printf(sc->dev, "cannot create DMA map %d\n", err);
 		return (err);
 	}
 	map->flags = ENMAP_ALLOC;
@@ -754,7 +753,7 @@ en_txdma(struct en_softc *sc, struct en_txslot *slot)
 		lastm->m_next = NULL;
 
 	if (error != 0) {
-		if_printf(sc->ifp, "loading TX map failed %d\n",
+		device_printf(sc->dev, "loading TX map failed %d\n",
 		    error);
 		goto dequeue_drop;
 	}
@@ -1098,7 +1097,7 @@ en_start(struct ifnet *ifp)
 		tx = vc->txslot;
 
 		if (m->m_pkthdr.len > EN_TXSZ * 1024) {
-			DBG(sc, TX, ("tx%zu: packet larger than xmit buffer "
+			DBG(sc, TX, ("tx%td: packet larger than xmit buffer "
 			    "(%d > %d)\n", tx - sc->txslot, m->m_pkthdr.len,
 			    EN_TXSZ * 1024));
 			EN_UNLOCK(sc);
@@ -1109,7 +1108,7 @@ en_start(struct ifnet *ifp)
 
 		if (tx->mbsize > EN_TXHIWAT) {
 			EN_COUNT(sc->stats.txmbovr);
-			DBG(sc, TX, ("tx%d: buffer space shortage",
+			DBG(sc, TX, ("tx%td: buffer space shortage",
 			    tx - sc->txslot));
 			EN_UNLOCK(sc);
 			m_freem(m);
@@ -1120,7 +1119,7 @@ en_start(struct ifnet *ifp)
 		/* commit */
 		tx->mbsize += m->m_pkthdr.len;
 
-		DBG(sc, TX, ("tx%zu: VCI=%d, speed=0x%x, buflen=%d, mbsize=%d",
+		DBG(sc, TX, ("tx%td: VCI=%d, speed=0x%x, buflen=%d, mbsize=%d",
 		    tx - sc->txslot, vci, sc->vccs[vci]->txspeed,
 		    m->m_pkthdr.len, tx->mbsize));
 
@@ -1164,7 +1163,7 @@ en_loadvc(struct en_softc *sc, struct en_vcc *vc)
 
 	vc->rxslot->cur = vc->rxslot->start;
 
-	DBG(sc, VC, ("rx%d: assigned to VCI %d", vc->rxslot - sc->rxslot,
+	DBG(sc, VC, ("rx%td: assigned to VCI %d", vc->rxslot - sc->rxslot,
 	    vc->vcc.vci));
 }
 
@@ -1217,7 +1216,7 @@ en_open_vcc(struct en_softc *sc, struct atmio_openvcc *op)
 	slot->vcc = vc;
 
 	KASSERT (_IF_QLEN(&slot->indma) == 0 && _IF_QLEN(&slot->q) == 0,
-	    ("en_rxctl: left over mbufs on enable slot=%tu",
+	    ("en_rxctl: left over mbufs on enable slot=%td",
 	    vc->rxslot - sc->rxslot));
 
 	vc->txspeed = 0;
@@ -1349,7 +1348,7 @@ en_reset_ul(struct en_softc *sc)
 	struct en_rxslot *rx;
 	int lcv;
 
-	if_printf(sc->ifp, "reset\n");
+	device_printf(sc->dev, "reset\n");
 	sc->ifp->if_drv_flags &= ~IFF_DRV_RUNNING;
 
 	if (sc->en_busreset)
@@ -1819,7 +1818,7 @@ en_rx_drain(struct en_softc *sc, u_int drq)
 	m = NULL;	/* assume "JK" trash DMA */
 	if (EN_DQ_LEN(drq) != 0) {
 		_IF_DEQUEUE(&slot->indma, m);
-		KASSERT(m != NULL, ("drqsync: %s: lost mbuf in slot %zu!",
+		KASSERT(m != NULL, ("drqsync: %s: lost mbuf in slot %td!",
 		    sc->ifp->if_xname, slot - sc->rxslot));
 		uma_zfree(sc->map_zone, (struct en_map *)m->m_pkthdr.rcvif);
 	}
@@ -1852,7 +1851,7 @@ en_rx_drain(struct en_softc *sc, u_int drq)
 		ATM_PH_VPI(&ah) = 0;
 		ATM_PH_SETVCI(&ah, vc->vcc.vci);
 
-		DBG(sc, INTR, ("rx%zu: rxvci%d: atm_input, mbuf %p, len %d, "
+		DBG(sc, INTR, ("rx%td: rxvci%d: atm_input, mbuf %p, len %d, "
 		    "hand %p", slot - sc->rxslot, vc->vcc.vci, m,
 		    EN_DQ_LEN(drq), vc->rxhand));
 
@@ -1869,7 +1868,9 @@ en_rx_drain(struct en_softc *sc, u_int drq)
 #ifdef ENABLE_BPF
 		BPF_MTAP(sc->ifp, m);
 #endif
+		EN_UNLOCK(sc);
 		atm_input(sc->ifp, &ah, m, vc->rxhand);
+		EN_LOCK(sc);
 	}
 }
 
@@ -2250,13 +2251,13 @@ en_service(struct en_softc *sc)
 
 		if (MID_RBD_CNT(rbd) * MID_ATMDATASZ <
 		    MID_PDU_LEN(pdu)) {
-			if_printf(sc->ifp, "invalid AAL5 length\n");
+			device_printf(sc->dev, "invalid AAL5 length\n");
 			rx.post_skip = MID_RBD_CNT(rbd) * MID_ATMDATASZ;
 			mlen = 0;
 			sc->ifp->if_ierrors++;
 
 		} else if (rbd & MID_RBD_CRCERR) {
-			if_printf(sc->ifp, "CRC error\n");
+			device_printf(sc->dev, "CRC error\n");
 			rx.post_skip = MID_RBD_CNT(rbd) * MID_ATMDATASZ;
 			mlen = 0;
 			sc->ifp->if_ierrors++;
@@ -2334,7 +2335,7 @@ en_service(struct en_softc *sc)
 		    en_rxdma_load, &rx, BUS_DMA_NOWAIT);
 
 		if (error != 0) {
-			if_printf(sc->ifp, "loading RX map failed "
+			device_printf(sc->dev, "loading RX map failed "
 			    "%d\n", error);
 			uma_zfree(sc->map_zone, map);
 			m_freem(m);
@@ -2430,11 +2431,10 @@ en_intr(void *arg)
 	 * unexpected errors that need a reset
 	 */
 	if ((reg & (MID_INT_IDENT | MID_INT_LERR | MID_INT_DMA_ERR)) != 0) {
-		if_printf(sc->ifp, "unexpected interrupt=0x%b, "
+		device_printf(sc->dev, "unexpected interrupt=0x%b, "
 		    "resetting\n", reg, MID_INTBITS);
 #ifdef EN_DEBUG
-		kdb_enter("en: unexpected error");
-		sc->ifp->if_drv_flags &= ~IFF_DRV_RUNNING; /* FREEZE! */
+		panic("en: unexpected error");
 #else
 		en_reset_ul(sc);
 		en_init(sc);
@@ -2853,7 +2853,7 @@ en_attach(struct en_softc *sc)
 
 	reg = en_read(sc, MID_RESID);
 
-	if_printf(sc->ifp, "ATM midway v%d, board IDs %d.%d, %s%s%s, "
+	device_printf(sc->dev, "ATM midway v%d, board IDs %d.%d, %s%s%s, "
 	    "%ldKB on-board RAM\n", MID_VER(reg), MID_MID(reg), MID_DID(reg), 
 	    (MID_IS_SABRE(reg)) ? "sabre controller, " : "",
 	    (MID_IS_SUNI(reg)) ? "SUNI" : "Utopia",
@@ -2879,15 +2879,15 @@ en_attach(struct en_softc *sc)
 	if (sc->is_adaptec) {
 		IFP2IFATM(sc->ifp)->mib.device = ATM_DEVICE_ADP155P;
 		if (sc->bestburstlen == 64 && sc->alburst == 0)
-			if_printf(sc->ifp,
+			device_printf(sc->dev,
 			    "passed 64 byte DMA test\n");
 		else
-			if_printf(sc->ifp, "FAILED DMA TEST: "
+			device_printf(sc->dev, "FAILED DMA TEST: "
 			    "burst=%d, alburst=%d\n", sc->bestburstlen,
 			    sc->alburst);
 	} else {
 		IFP2IFATM(sc->ifp)->mib.device = ATM_DEVICE_ENI155P;
-		if_printf(sc->ifp, "maximum DMA burst length = %d "
+		device_printf(sc->dev, "maximum DMA burst length = %d "
 		    "bytes%s\n", sc->bestburstlen, sc->alburst ?
 		    sc->noalbursts ?  " (no large bursts)" : " (must align)" :
 		    "");
@@ -2961,7 +2961,7 @@ en_attach(struct en_softc *sc)
 	ptr = roundup(ptr, EN_TXSZ * 1024);	/* align */
 	sz = sz - (ptr - sav);
 	if (EN_TXSZ*1024 * EN_NTX > sz) {
-		if_printf(sc->ifp, "EN_NTX/EN_TXSZ too big\n");
+		device_printf(sc->dev, "EN_NTX/EN_TXSZ too big\n");
 		goto fail;
 	}
 	for (lcv = 0 ;lcv < EN_NTX ;lcv++) {
@@ -2980,7 +2980,7 @@ en_attach(struct en_softc *sc)
 	sz = sz - (ptr - sav);
 	sc->en_nrx = sz / (EN_RXSZ * 1024);
 	if (sc->en_nrx <= 0) {
-		if_printf(sc->ifp, "EN_NTX/EN_TXSZ/EN_RXSZ too big\n");
+		device_printf(sc->dev, "EN_NTX/EN_TXSZ/EN_RXSZ too big\n");
 		goto fail;
 	}
 
@@ -3011,9 +3011,9 @@ en_attach(struct en_softc *sc)
 		    sc->rxslot[lcv].mode));
 	}
 
-	if_printf(sc->ifp, "%d %dKB receive buffers, %d %dKB transmit "
+	device_printf(sc->dev, "%d %dKB receive buffers, %d %dKB transmit "
 	    "buffers\n", sc->en_nrx, EN_RXSZ, EN_NTX, EN_TXSZ);
-	if_printf(sc->ifp, "end station identifier (mac address) "
+	device_printf(sc->dev, "end station identifier (mac address) "
 	    "%6D\n", IFP2IFATM(sc->ifp)->mib.esi, ":");
 
 	/*
@@ -3195,7 +3195,7 @@ en_dump_mregs(struct en_softc *sc)
 	printf("  rxvc slot mappings:");
 	for (cnt = 0 ; cnt < MID_N_VC ; cnt++)
 		if (sc->vccs[cnt]->rxslot != NULL)
-			printf("  %d->%zu", cnt,
+			printf("  %d->%td", cnt,
 			    sc->vccs[cnt]->rxslot - sc->rxslot);
 	printf("\n");
 }
@@ -3229,7 +3229,7 @@ en_dump_rx(struct en_softc *sc)
 
 	printf("  recv slots:\n");
 	for (slot = sc->rxslot ; slot < &sc->rxslot[sc->en_nrx]; slot++) {
-		printf("rx%zu: start/stop/cur=0x%x/0x%x/0x%x mode=0x%x ",
+		printf("rx%td: start/stop/cur=0x%x/0x%x/0x%x mode=0x%x ",
 		    slot - sc->rxslot, slot->start, slot->stop, slot->cur,
 		    slot->mode);
 		if (slot->vcc != NULL) {
@@ -3303,7 +3303,7 @@ en_dump(int unit, int level)
 		if (unit != -1 && unit != lcv)
 			continue;
 
-		if_printf(sc->ifp, "dumping device at level 0x%b\n",
+		device_printf(sc->dev, "dumping device at level 0x%b\n",
 		    level, END_BITS);
 
 		if (sc->dtq_us == 0) {
