@@ -29,7 +29,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ep/if_ep.c,v 1.140.2.3 2006/12/04 00:51:14 mlaier Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ep/if_ep.c,v 1.148 2006/12/01 20:29:55 mlaier Exp $");
 
 /*
  *	Modified from the FreeBSD 1.1.5.1 version by:
@@ -72,7 +72,7 @@ __FBSDID("$FreeBSD: src/sys/dev/ep/if_ep.c,v 1.140.2.3 2006/12/04 00:51:14 mlaie
 #include <sys/rman.h>
 
 #include <net/if.h>
-#include <net/if_arp.h>
+#include <net/if_dl.h>
 #include <net/if_media.h>
 #include <net/if_types.h>
 #include <net/ethernet.h>
@@ -352,20 +352,18 @@ ep_detach(device_t dev)
 	struct ifnet *ifp;
 
 	sc = device_get_softc(dev);
-	EP_ASSERT_UNLOCKED(sc);
 	ifp = sc->ifp;
-
-	if (sc->gone)
-		return (0);
+	EP_ASSERT_UNLOCKED(sc);
+	EP_LOCK(sc);
 	if (bus_child_present(dev))
 		epstop(sc);
-
-	ifp->if_drv_flags &= ~IFF_DRV_RUNNING;
-	ether_ifdetach(ifp);
-	if_free(ifp);
-
 	sc->gone = 1;
+	ifp->if_drv_flags &= ~IFF_DRV_RUNNING;
+	EP_UNLOCK(sc);
+	ether_ifdetach(ifp);
 	ep_free(dev);
+
+	if_free(ifp);
 	EP_LOCK_DESTROY(sc);
 
 	return (0);
@@ -410,7 +408,7 @@ epinit_locked(struct ep_softc *sc)
 
 	GO_WINDOW(sc, 2);
 	/* Reload the ether_addr. */
-	ep_setup_station(sc, IFP2ENADDR(sc->ifp));
+	ep_setup_station(sc, IF_LLADDR(sc->ifp));
 
 	CSR_WRITE_2(sc, EP_COMMAND, RX_RESET);
 	CSR_WRITE_2(sc, EP_COMMAND, TX_RESET);
@@ -920,12 +918,6 @@ epioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 			epinit_locked(sc);
 		EP_UNLOCK(sc);
 		break;
-#ifdef notdef
-	case SIOCGHWADDR:
-		bcopy((caddr_t)sc->sc_addr, (caddr_t)&ifr->ifr_data,
-		    sizeof(sc->sc_addr));
-		break;
-#endif
 	case SIOCADDMULTI:
 	case SIOCDELMULTI:
 		/*
@@ -966,8 +958,6 @@ epwatchdog(struct ifnet *ifp)
 static void
 epstop(struct ep_softc *sc)
 {
-	if (sc->gone)
-		return;
 	CSR_WRITE_2(sc, EP_COMMAND, RX_DISABLE);
 	CSR_WRITE_2(sc, EP_COMMAND, RX_DISCARD_TOP_PACK);
 	EP_BUSY_WAIT(sc);
