@@ -104,7 +104,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/esp/ncr53c9x.c,v 1.12 2005/05/19 14:51:10 marius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/esp/ncr53c9x.c,v 1.17 2007/06/17 05:55:49 scottl Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -325,14 +325,14 @@ ncr53c9x_attach(struct ncr53c9x_softc *sc)
 	}
 
 	sim = cam_sim_alloc(ncr53c9x_action, ncr53c9x_poll, "esp", sc,
-			    device_get_unit(sc->sc_dev), 1,
+			    device_get_unit(sc->sc_dev), &Giant, 1,
 			    NCR_TAG_DEPTH, devq);
 	if (sim == NULL) {
 		device_printf(sc->sc_dev, "cannot allocate SIM entry\n");
 		error = ENOMEM;
 		goto fail_devq;
 	}
-	if (xpt_bus_register(sim, 0) != CAM_SUCCESS) {
+	if (xpt_bus_register(sim, sc->sc_dev, 0) != CAM_SUCCESS) {
 		device_printf(sc->sc_dev, "cannot register bus\n");
 		error = EIO;
 		goto fail_sim;
@@ -928,6 +928,10 @@ ncr53c9x_action(struct cam_sim *sim, union ccb *ccb)
 		strncpy(cpi->hba_vid, "Sun", HBA_IDLEN);
 		strncpy(cpi->dev_name, cam_sim_name(sim), DEV_IDLEN);
 		cpi->unit_number = cam_sim_unit(sim);
+		cpi->transport = XPORT_SPI;
+		cpi->transport_version = 2;
+		cpi->protocol = PROTO_SCSI;
+		cpi->protocol_version = SCSI_REV_2;
 		ccb->ccb_h.status = CAM_REQ_CMP;
 		mtx_unlock(&sc->sc_lock);
 		xpt_done(ccb);
@@ -936,31 +940,41 @@ ncr53c9x_action(struct cam_sim *sim, union ccb *ccb)
 	case XPT_GET_TRAN_SETTINGS:
 	{
 		struct ccb_trans_settings *cts = &ccb->cts;
-		struct ncr53c9x_tinfo *ti;
+		struct ncr53c9x_tinfo *ti = &sc->sc_tinfo[ccb->ccb_h.target_id];
+		struct ccb_trans_settings_scsi *scsi =
+		    &cts->proto_specific.scsi;
+		struct ccb_trans_settings_spi *spi =
+		    &cts->xport_specific.spi;
 
-		ti = &sc->sc_tinfo[ccb->ccb_h.target_id];
+		cts->protocol = PROTO_SCSI;
+		cts->protocol_version = SCSI_REV_2;
+		cts->transport = XPORT_SPI;
+		cts->transport_version = 2;
 
-		if ((cts->flags & CCB_TRANS_CURRENT_SETTINGS) != 0) {
-			cts->sync_period = ti->period;
-			cts->sync_offset = ti->offset;
-			cts->bus_width = ti->width;
-			if ((ti->flags & T_TAG) != 0)
-				cts->flags |=
-				    (CCB_TRANS_DISC_ENB | CCB_TRANS_TAG_ENB);
-			else
-				cts->flags &=
-				    ~(CCB_TRANS_DISC_ENB | CCB_TRANS_TAG_ENB);
+		if (cts->type == CTS_TYPE_CURRENT_SETTINGS) {
+			spi->sync_period = ti->period;
+			spi->sync_offset = ti->offset;
+			spi->bus_width = ti->width;
+			if ((ti->flags & T_TAG) != 0) {
+				spi->flags |= CTS_SPI_FLAGS_DISC_ENB;
+				scsi->flags |= CTS_SCSI_FLAGS_TAG_ENB;
+			} else {
+				spi->flags &= ~CTS_SPI_FLAGS_DISC_ENB;
+				scsi->flags &= ~CTS_SCSI_FLAGS_TAG_ENB;
+			}
 		} else {
-			cts->sync_period = sc->sc_maxsync;
-			cts->sync_offset = sc->sc_maxoffset;
-			cts->bus_width = sc->sc_maxwidth;
-			cts->flags |= (CCB_TRANS_DISC_ENB | CCB_TRANS_TAG_ENB);
+			spi->sync_period = sc->sc_maxsync;
+			spi->sync_offset = sc->sc_maxoffset;
+			spi->bus_width = sc->sc_maxwidth;
+			spi->flags |= CTS_SPI_FLAGS_DISC_ENB;
+			scsi->flags |= CTS_SCSI_FLAGS_TAG_ENB;
 		}
-		cts->valid = CCB_TRANS_BUS_WIDTH_VALID |
-			     CCB_TRANS_SYNC_RATE_VALID |
-			     CCB_TRANS_SYNC_OFFSET_VALID |
-			     CCB_TRANS_DISC_VALID |
-			     CCB_TRANS_TQ_VALID;
+		spi->valid =
+		    CTS_SPI_VALID_BUS_WIDTH |
+		    CTS_SPI_VALID_SYNC_RATE |
+		    CTS_SPI_VALID_SYNC_OFFSET |
+		    CTS_SPI_VALID_DISC;
+		scsi->valid = CTS_SCSI_VALID_TQ;
 		ccb->ccb_h.status = CAM_REQ_CMP;
 		mtx_unlock(&sc->sc_lock);
 		xpt_done(ccb);
@@ -1038,15 +1052,17 @@ ncr53c9x_action(struct cam_sim *sim, union ccb *ccb)
 
 	case XPT_SET_TRAN_SETTINGS:
 	{
-		struct ncr53c9x_tinfo *ti;
 		struct ccb_trans_settings *cts = &ccb->cts;
 		int target = ccb->ccb_h.target_id;
+		struct ncr53c9x_tinfo *ti = &sc->sc_tinfo[target];
+		struct ccb_trans_settings_scsi *scsi =
+		    &cts->proto_specific.scsi;
+		struct ccb_trans_settings_spi *spi =
+		    &cts->xport_specific.spi;
 
-		ti = &sc->sc_tinfo[target];
-
-		if ((cts->valid & CCB_TRANS_TQ_VALID) != 0) {
+		if ((scsi->valid & CTS_SCSI_VALID_TQ) != 0) {
 			if ((sc->sc_cfflags & (1<<((target & 7) + 16))) == 0 &&
-			    (cts->flags & CCB_TRANS_TAG_ENB)) {
+			    (scsi->flags & CTS_SCSI_FLAGS_TAG_ENB)) {
 				NCR_MISC(("%s: target %d: tagged queuing\n",
 				    device_get_nameunit(sc->sc_dev), target));
 				ti->flags |= T_TAG;
@@ -1054,8 +1070,8 @@ ncr53c9x_action(struct cam_sim *sim, union ccb *ccb)
 				ti->flags &= ~T_TAG;
 		}
 
-		if ((cts->valid & CCB_TRANS_BUS_WIDTH_VALID) != 0) {
-			if (cts->bus_width != 0) {
+		if ((spi->valid & CTS_SPI_VALID_BUS_WIDTH) != 0) {
+			if (spi->bus_width != 0) {
 				NCR_MISC(("%s: target %d: wide negotiation\n",
 				    device_get_nameunit(sc->sc_dev), target));
 				if (sc->sc_rev == NCR_VARIANT_FAS366) {
@@ -1069,18 +1085,18 @@ ncr53c9x_action(struct cam_sim *sim, union ccb *ccb)
 			ti->flags |= T_NEGOTIATE;
 		}
 
-		if ((cts->valid & CCB_TRANS_SYNC_RATE_VALID) != 0) {
+		if ((spi->valid & CTS_SPI_VALID_SYNC_RATE) != 0) {
 			NCR_MISC(("%s: target %d: sync period negotiation\n",
 			    device_get_nameunit(sc->sc_dev), target));
 			ti->flags |= T_NEGOTIATE;
-			ti->period = cts->sync_period;
+			ti->period = spi->sync_period;
 		}
 
-		if ((cts->valid & CCB_TRANS_SYNC_OFFSET_VALID) != 0) {
+		if ((spi->valid & CTS_SPI_VALID_SYNC_OFFSET) != 0) {
 			NCR_MISC(("%s: target %d: sync offset negotiation\n",
 			    device_get_nameunit(sc->sc_dev), target));
 			ti->flags |= T_NEGOTIATE;
-			ti->offset = cts->sync_offset;
+			ti->offset = spi->sync_offset;
 		}
 
 		mtx_unlock(&sc->sc_lock);
@@ -2958,7 +2974,7 @@ ncr53c9x_timeout(void *arg)
 	    sc->sc_phase, sc->sc_prevphase,
 	    (long)sc->sc_dleft, sc->sc_msgpriq, sc->sc_msgout,
 	    NCRDMA_ISACTIVE(sc) ? "DMA active" : "");
-#if NCR53C9X_DEBUG > 1
+#if defined(NCR53C9X_DEBUG) && NCR53C9X_DEBUG > 1
 	printf("TRACE: %s.", ecb->trace);
 #endif
 
