@@ -53,7 +53,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/exca/exca.c,v 1.19 2005/01/11 00:32:43 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/exca/exca.c,v 1.28 2007/03/23 17:15:07 jhb Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -86,7 +86,6 @@ __FBSDID("$FreeBSD: src/sys/dev/exca/exca.c,v 1.19 2005/01/11 00:32:43 imp Exp $
 #define DPRINTF(fmt, args...)
 #endif
 
-#if 0
 static const char *chip_names[] = 
 {
 	"CardBus socket",
@@ -105,7 +104,6 @@ static const char *chip_names[] =
 	"IBM clone",
 	"IBM KING PCMCIA Controller"
 };
-#endif
 
 static exca_getb_fn exca_mem_getb;
 static exca_putb_fn exca_mem_putb;
@@ -181,6 +179,7 @@ exca_do_mem_map(struct exca_softc *sc, int win)
 	struct mem_map_index_st *map;
 	struct pccard_mem_handle *mem;
 	uint32_t offset;
+	int mem8 = 1 /* mem->kind == PCCARD_A_MEM_ATTR */;
 	
 	map = &mem_map_index[win];
 	mem = &sc->mem[win];
@@ -190,7 +189,8 @@ exca_do_mem_map(struct exca_softc *sc, int win)
 	    (mem->addr >> EXCA_SYSMEM_ADDRX_SHIFT) & 0xff);
 	exca_putb(sc, map->sysmem_start_msb,
 	    ((mem->addr >> (EXCA_SYSMEM_ADDRX_SHIFT + 8)) &
-	    EXCA_SYSMEM_ADDRX_START_MSB_ADDR_MASK));
+	    EXCA_SYSMEM_ADDRX_START_MSB_ADDR_MASK) |
+	    (mem8 ? 0 : EXCA_SYSMEM_ADDRX_START_MSB_DATASIZE_16BIT));
 
 	exca_putb(sc, map->sysmem_stop_lsb,
 	    ((mem->addr + mem->realsize - 1) >>
@@ -259,11 +259,11 @@ exca_mem_map(struct exca_softc *sc, int kind, struct resource *res)
 		}
 	}
 	if (win >= EXCA_MEM_WINS)
-		return (1);
+		return (ENOSPC);
 	if (((rman_get_start(res) >> EXCA_MEMREG_WIN_SHIFT) & 0xff) != 0 &&
 	    (sc->flags & EXCA_HAS_MEMREG_WIN) == 0) {
 		device_printf(sc->dev, "Does not support mapping above 24M.");
-		return (1);
+		return (EINVAL);
 	}
 
 	sc->mem[win].cardaddr = 0;
@@ -301,7 +301,7 @@ exca_mem_unmap(struct exca_softc *sc, int window)
 }
 
 /*
- * Find the map that we're using to hold the resoruce.  This works well
+ * Find the map that we're using to hold the resource.  This works well
  * so long as the client drivers don't do silly things like map the same
  * area mutliple times, or map both common and attribute memory at the
  * same time.  This latter restriction is a bug.  We likely should just
@@ -365,9 +365,9 @@ exca_mem_unmap_res(struct exca_softc *sc, struct resource *res)
 	
 /*
  * Set the offset of the memory.  We use this for reading the CIS and
- * frobbing the pccard's pccard registers (POR, etc).  Some drivers
- * need to access this functionality as well, since they have receive
- * buffers defined in the attribute memory.
+ * frobbing the pccard's pccard registers (CCR, etc).  Some drivers
+ * need to access arbitrary attribute and common memory during their
+ * initialization and operation.
  */
 int
 exca_mem_set_offset(struct exca_softc *sc, struct resource *res,
@@ -477,7 +477,7 @@ exca_io_map(struct exca_softc *sc, int width, struct resource *r)
 		}
 	}
 	if (win >= EXCA_IO_WINS)
-		return (1);
+		return (ENOSPC);
 
 	sc->io[win].iot = rman_get_bustag(r);
 	sc->io[win].ioh = rman_get_bushandle(r);
@@ -569,7 +569,7 @@ exca_wait_ready(struct exca_softc *sc)
  * do anything but a busy wait here.  One could argue that these
  * such cards are broken, or that the bridge that allows this sort
  * of interrupt through isn't quite what you'd want (and may be a standards
- * violation).  However, such arguing would leave a huge class of pc cards
+ * violation).  However, such arguing would leave a huge class of PC Cards
  * and bridges out of reach for use in the system.
  *
  * Maybe I should reevaluate the above based on the power bug I fixed
@@ -588,7 +588,7 @@ exca_reset(struct exca_softc *sc, device_t child)
 	DELAY(30*1000);
 	/* clear the reset flag */
 	exca_setb(sc, EXCA_INTR, EXCA_INTR_RESET);
-	/* wait 20ms as per pc card standard (r2.01) section 4.3.6 */
+	/* wait 20ms as per PC Card standard (r2.01) section 4.3.6 */
 	DELAY(20*1000);
 
 	exca_wait_ready(sc);
@@ -643,6 +643,7 @@ exca_valid_slot(struct exca_softc *exca)
 	 * IBM clone chips use 0x88 and 0x89, apparently
 	 */
 	c = exca_getb(exca, EXCA_IDENT);
+	DEVPRINTF(exca->dev, "Ident is %x\n", c);
 	if ((c & EXCA_IDENT_IFTYPE_MASK) != EXCA_IDENT_IFTYPE_MEM_AND_IO)
 		return (0);
 	if ((c & EXCA_IDENT_ZERO) != 0)
@@ -754,8 +755,10 @@ exca_probe_slots(device_t dev, struct exca_softc *exca, bus_space_tag_t iot,
 		exca_init(&exca[i], dev, iot, ioh, i * EXCA_SOCKET_SIZE);
 		exca->getb = exca_io_getb;
 		exca->putb = exca_io_putb;
-		if (exca_valid_slot(&exca[i]))
+		if (exca_valid_slot(&exca[i])) {
+			device_set_desc(dev, chip_names[exca[i].chipset]);
 			err = 0;
+		}
 	}
 	return (err);
 }
@@ -763,7 +766,7 @@ exca_probe_slots(device_t dev, struct exca_softc *exca, bus_space_tag_t iot,
 void
 exca_insert(struct exca_softc *exca)
 {
-	if (exca->pccarddev != NULL) {
+	if (device_is_attached(exca->pccarddev)) {
 		if (CARD_ATTACH_CARD(exca->pccarddev) != 0)
 			device_printf(exca->dev,
 			    "PC Card card activation failed\n");
@@ -777,7 +780,7 @@ exca_insert(struct exca_softc *exca)
 void
 exca_removal(struct exca_softc *exca)
 {
-	if (exca->pccarddev != NULL)
+	if (device_is_attached(exca->pccarddev))
 		CARD_DETACH_CARD(exca->pccarddev);
 }
 
@@ -786,24 +789,25 @@ exca_activate_resource(struct exca_softc *exca, device_t child, int type,
     int rid, struct resource *res)
 {
 	int err;
-	if (!(rman_get_flags(res) & RF_ACTIVE)) { /* not already activated */
-		switch (type) {
-		case SYS_RES_IOPORT:
-			err = exca_io_map(exca, PCCARD_WIDTH_AUTO, res);
-			break;
-		case SYS_RES_MEMORY:
-			err = exca_mem_map(exca, PCCARD_A_MEM_COM, res);
-			break;
-		default:
-			err = 0;
-			break;
-		}
-		if (err)
-			return (err);
 
+	if (rman_get_flags(res) & RF_ACTIVE)
+		return (0);
+	err = BUS_ACTIVATE_RESOURCE(device_get_parent(exca->dev), child,
+	    type, rid, res);
+	if (err)
+		return (err);
+	switch (type) {
+	case SYS_RES_IOPORT:
+		err = exca_io_map(exca, PCCARD_WIDTH_AUTO, res);
+		break;
+	case SYS_RES_MEMORY:
+		err = exca_mem_map(exca, PCCARD_A_MEM_COM, res);
+		break;
 	}
-	return (BUS_ACTIVATE_RESOURCE(device_get_parent(exca->dev), child,
-		  type, rid, res));
+	if (err)
+		BUS_DEACTIVATE_RESOURCE(device_get_parent(exca->dev), child,
+		    type, rid, res);
+	return (err);
 }
 
 int
