@@ -1,5 +1,5 @@
 /*	$NetBSD: i82365reg.h,v 1.3 1998/12/20 17:53:28 nathanw Exp $	*/
-/* $FreeBSD: src/sys/dev/exca/excareg.h,v 1.5 2005/01/06 01:42:40 imp Exp $ */
+/* $FreeBSD: src/sys/dev/exca/excareg.h,v 1.6 2006/08/12 08:53:54 imp Exp $ */
 
 /*-
  * Copyright (c) 2002 M Warner Losh.  All rights reserved.
@@ -402,6 +402,12 @@
 #define EXCA_O2CC_IREQ_INTC			0x80
 #define EXCA_O2CC_STSCHG_INTC			0x20
 
+/*
+ * TOPIC specific registers
+ */
+#define EXCA_TOPIC97_CTRL			0x3e
+#define EXCA_TOPIC97_CTRL_LV_MASK		0x03
+
 /* Plug and play */
 #define EXCA_PNP_ACTIONTEC	0x1802A904	/* AEI0218 */
 #define EXCA_PNP_IBM3765	0x65374d24	/* IBM3765 */
