@@ -31,7 +31,7 @@
  * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  * 
- * $FreeBSD: src/sys/dev/firewire/fwohci_pci.c,v 1.52.2.3 2007/04/03 22:41:43 simokawa Exp $
+ * $FreeBSD: src/sys/dev/firewire/fwohci_pci.c,v 1.60 2007/06/06 14:31:36 simokawa Exp $
  */
 
 #define BOUNCE_BUFFER_TEST	0
@@ -169,8 +169,8 @@ fwohci_pci_probe( device_t dev )
 		return BUS_PROBE_DEFAULT;
 	}
 	if (id == (FW_VENDORID_SONY | FW_DEVICE_CXD1947)) {
-		device_set_desc(dev, "Sony i.LINK (CXD1947)");
-		return BUS_PROBE_DEFAULT;
+		device_printf(dev, "Sony i.LINK (CXD1947) not supported\n");
+		return ENXIO;
 	}
 	if (id == (FW_VENDORID_SONY | FW_DEVICE_CXD3222)) {
 		device_set_desc(dev, "Sony i.LINK (CXD3222)");
@@ -302,6 +302,7 @@ fwohci_pci_attach(device_t self)
 		firewire_debug = bootverbose;
 #endif
 
+	mtx_init(FW_GMTX(&sc->fc), "firewire", NULL, MTX_DEF);
 	fwohci_pci_init(self);
 
 	rid = PCI_CBMEM;
@@ -335,12 +336,12 @@ fwohci_pci_attach(device_t self)
 
 
 	err = bus_setup_intr(self, sc->irq_res,
-#if FWOHCI_TASKQUEUE
 			INTR_TYPE_NET | INTR_MPSAFE,
+#if FWOHCI_INTFILT
+		     fwohci_filt, NULL, sc, &sc->ih);
 #else
-			INTR_TYPE_NET,
+		     NULL, (driver_intr_t *) fwohci_intr, sc, &sc->ih);
 #endif
-		     (driver_intr_t *) fwohci_intr, sc, &sc->ih);
 #if defined(__DragonFly__) || __FreeBSD_version < 500000
 	/* XXX splcam() should mask this irq for sbp.c*/
 	err = bus_setup_intr(self, sc->irq_res, INTR_TYPE_CAM,
@@ -355,7 +356,13 @@ fwohci_pci_attach(device_t self)
 		return ENXIO;
 	}
 
-	err = bus_dma_tag_create(/*parent*/NULL, /*alignment*/1,
+	err = bus_dma_tag_create(
+#if defined(__FreeBSD__) && __FreeBSD_version >= 700020
+				/*parent*/bus_get_dma_tag(self),
+#else
+				/*parent*/NULL,
+#endif
+				/*alignment*/1,
 				/*boundary*/0,
 #if BOUNCE_BUFFER_TEST
 				/*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
@@ -370,7 +377,7 @@ fwohci_pci_attach(device_t self)
 				/*flags*/BUS_DMA_ALLOCNOW,
 #if defined(__FreeBSD__) && __FreeBSD_version >= 501102
 				/*lockfunc*/busdma_lock_mutex,
-				/*lockarg*/&Giant,
+				/*lockarg*/FW_GMTX(&sc->fc),
 #endif
 				&sc->fc.dmat);
 	if (err != 0) {
@@ -442,6 +449,7 @@ fwohci_pci_detach(device_t self)
 	}
 
 	fwohci_detach(sc, self);
+	mtx_destroy(FW_GMTX(&sc->fc));
 	splx(s);
 
 	return 0;
@@ -486,7 +494,7 @@ fwohci_pci_add_child(device_t dev, int order, const char *name, int unit)
 {
 	struct fwohci_softc *sc;
 	device_t child;
-	int s, err = 0;
+	int err = 0;
 
 	sc = (struct fwohci_softc *)device_get_softc(dev);
 	child = device_add_child(dev, name, unit);
@@ -510,6 +518,7 @@ fwohci_pci_add_child(device_t dev, int order, const char *name, int unit)
 	 * interrupt is disabled during the boot process.
 	 */
 	if (cold) {
+		int s;
 		DELAY(250); /* 2 cycles */
 		s = splfw();
 		fwohci_poll((void *)sc, 0, -1);
