@@ -24,9 +24,9 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	from: NetBSD: gemreg.h,v 1.15 2002/05/11 00:36:02 matt Exp
+ *	from: NetBSD: gemreg.h,v 1.8 2005/12/11 12:21:26 christos Exp
  *
- * $FreeBSD: src/sys/dev/gem/if_gemreg.h,v 1.3 2005/01/06 01:42:42 imp Exp $
+ * $FreeBSD: src/sys/dev/gem/if_gemreg.h,v 1.5 2007/09/26 21:14:17 marius Exp $
  */
 
 #ifndef	_IF_GEMREG_H
@@ -55,10 +55,15 @@
 
 
 /* Bits in GEM_CONFIG register */
-#define	GEM_CONFIG_BURST_64	0x000000000	/* 0->infininte, 1->64KB */
-#define	GEM_CONFIG_BURST_INF	0x000000001	/* 0->infininte, 1->64KB */
+#define	GEM_CONFIG_BURST_64	0x000000000	/* maximum burst size 64KB */
+#define	GEM_CONFIG_BURST_INF	0x000000001	/* infinite for entire packet */
 #define	GEM_CONFIG_TXDMA_LIMIT	0x00000003e
 #define	GEM_CONFIG_RXDMA_LIMIT	0x0000007c0
+/* GEM_CONFIG_RONPAULBIT and GEM_CONFIG_BUG2FIX are Apple only. */
+#define	GEM_CONFIG_RONPAULBIT	0x000000800	/* after infinite burst use */
+						/* memory read multiple for */
+						/* PCI commands */
+#define	GEM_CONFIG_BUG2FIX	0x000001000	/* fix RX hang after overflow */
 
 #define	GEM_CONFIG_TXDMA_LIMIT_SHIFT	1
 #define	GEM_CONFIG_RXDMA_LIMIT_SHIFT	6
@@ -68,31 +73,35 @@
 #define	GEM_STATUS_TX_COMPL	0xfff800000	/* TX completion reg. */
 
 
-/* Interrupt bits, for both the GEM_STATUS and GEM_INTMASK regs. */
+/*
+ * Interrupt bits, for both the GEM_STATUS and GEM_INTMASK regs.
+ * Bits 0-6 auto-clear when read.
+ */
 #define	GEM_INTR_TX_INTME	0x000000001	/* Frame w/INTME bit set sent */
 #define	GEM_INTR_TX_EMPTY	0x000000002	/* TX ring empty */
 #define	GEM_INTR_TX_DONE	0x000000004	/* TX complete */
 #define	GEM_INTR_RX_DONE	0x000000010	/* Got a packet */
 #define	GEM_INTR_RX_NOBUF	0x000000020
 #define	GEM_INTR_RX_TAG_ERR	0x000000040
-#define	GEM_INTR_PCS		0x000002000
+#define	GEM_INTR_PERR		0x000000080	/* Parity error */
+#define	GEM_INTR_PCS		0x000002000	/* Physical Code Sub-layer */
 #define	GEM_INTR_TX_MAC		0x000004000
 #define	GEM_INTR_RX_MAC		0x000008000
 #define	GEM_INTR_MAC_CONTROL	0x000010000	/* MAC control interrupt */
 #define	GEM_INTR_MIF		0x000020000
 #define	GEM_INTR_BERR		0x000040000	/* Bus error interrupt */
-#define GEM_INTR_BITS	"\177\020"					\
+#define	GEM_INTR_BITS	"\177\020"					\
 			"b\0INTME\0b\1TXEMPTY\0b\2TXDONE\0"		\
 			"b\4RXDONE\0b\5RXNOBUF\0b\6RX_TAG_ERR\0"	\
-			"b\15PCS\0b\16TXMAC\0b\17RXMAC\0"		\
-			"b\20MAC_CONTROL\0b\21MIF\0b\22BERR\0\0"	\
-
+			"b\xdPCS\0b\xeTXMAC\0b\xfRXMAC\0"		\
+			"b\x10MAC_CONTROL\0b\x11MIF\0b\x12IBERR\0\0"
 
 
 /* GEM_ERROR_STATUS and GEM_ERROR_MASK PCI error bits */
 #define	GEM_ERROR_STAT_BADACK	0x000000001	/* No ACK64# */
 #define	GEM_ERROR_STAT_DTRTO	0x000000002	/* Delayed xaction timeout */
 #define	GEM_ERROR_STAT_OTHERS	0x000000004
+#define	GEM_ERROR_BITS		"\177\020b\0ACKBAD\0b\1DTRTO\0b\2OTHER\0\0"
 
 
 /* GEM_BIF_CONFIG register bits */
@@ -100,6 +109,8 @@
 #define	GEM_BIF_CONFIG_HOST_64	0x000000002	/* 64-bit host */
 #define	GEM_BIF_CONFIG_B64D_DIS	0x000000004	/* no 64-bit data cycle */
 #define	GEM_BIF_CONFIG_M66EN	0x000000008
+#define	GEM_BIF_CONFIG_BITS	"\177\020b\0SLOWCLK\0b\1HOST64\0"	\
+				"b\2B64DIS\0b\3M66EN\0\0"
 
 
 /* GEM_RESET register bits -- TX and RX self clear when complete. */
@@ -188,7 +199,7 @@
 #define	GEM_RX_CONFIG_RXRING_SZ	0x0000001e	/* RX ring size */
 #define	GEM_RX_CONFIG_BATCH_DIS	0x00000020	/* desc batching disable */
 #define	GEM_RX_CONFIG_FBOFF	0x00001c00	/* first byte offset */
-#define	GEM_RX_CONFIG_CXM_START	0x000fe000	/* checksum start offset */
+#define	GEM_RX_CONFIG_CXM_START	0x000fe000	/* cksum start offset bytes */
 #define	GEM_RX_CONFIG_FIFO_THRS	0x07000000	/* fifo threshold size */
 
 #define	GEM_THRSH_64	0
@@ -232,7 +243,7 @@
 #define	GEM_MAC_IPG0		0x6040		/* inter packet gap 0 */
 #define	GEM_MAC_IPG1		0x6044		/* inter packet gap 1 */
 #define	GEM_MAC_IPG2		0x6048		/* inter packet gap 2 */
-#define	GEM_MAC_SLOT_TIME	0x604c
+#define	GEM_MAC_SLOT_TIME	0x604c		/* slot time, bits 0-7 */
 #define	GEM_MAC_MAC_MIN_FRAME	0x6050
 #define	GEM_MAC_MAC_MAX_FRAME	0x6054
 #define	GEM_MAC_PREAMBLE_LEN	0x6058
@@ -285,7 +296,7 @@
 #define	GEM_MAC_RX_CRC_ERR_CNT	0x6124
 #define	GEM_MAC_RX_CODE_VIOL	0x6128
 #define	GEM_MAC_RANDOM_SEED	0x6130
-#define	GEM_MAC_MAC_STATE	0x6134		/* MAC sstate machine reg */
+#define	GEM_MAC_MAC_STATE	0x6134		/* MAC state machine reg */
 
 
 /* GEM_MAC_SEND_PAUSE_CMD register bits */
@@ -319,7 +330,12 @@
 #define	GEM_MAC_PAUSED		0x00000001	/* Pause received */
 #define	GEM_MAC_PAUSE		0x00000002	/* enter pause state */
 #define	GEM_MAC_RESUME		0x00000004	/* exit pause state */
-#define	GEM_MAC_PAUSE_TIME	0xffff0000
+#define	GEM_MAC_PAUSE_TIME_SLTS	0xffff0000	/* pause time in slots */
+#define	GEM_MAC_STATUS_BITS	"\177\020b\0PAUSED\0b\1PAUSE\0b\2RESUME\0\0"
+
+#define	GEM_MAC_PAUSE_TIME_SHFT	16
+#define	GEM_MAC_PAUSE_TIME(x)						\
+	(((x) & GEM_MAC_PAUSE_TIME_SLTS) >> GEM_MAC_PAUSE_TIME_SHFT)
 
 /* GEM_MAC_XIF_CONFIG register bits */
 #define	GEM_MAC_XIF_TX_MII_ENA	0x00000001	/* Enable XIF output drivers */
@@ -329,11 +345,22 @@
 #define	GEM_MAC_XIF_MII_BUF_ENA	0x00000010	/* Enable MII recv buffers */
 #define	GEM_MAC_XIF_LINK_LED	0x00000020	/* force link LED active */
 #define	GEM_MAC_XIF_FDPLX_LED	0x00000040	/* force FDPLX LED active */
+#define	GEM_MAC_XIF_BITS	"\177\020b\0TXMIIENA\0b\1MIILOOP\0b\2NOECHO" \
+				"\0b\3GMII\0b\4MIIBUFENA\0b\5LINKLED\0" \
+				"b\6FDLED\0\0"
+
+/*
+ * GEM_MAC_SLOT_TIME register
+ * The slot time is used as PAUSE time unit, value depends on whether carrier
+ * extension is enabled.
+ */
+#define	GEM_MAC_SLOT_TIME_CARR_EXTEND	0x200
+#define	GEM_MAC_SLOT_TIME_NORMAL	0x40
 
 /* GEM_MAC_TX_CONFIG register bits */
 #define	GEM_MAC_TX_ENABLE	0x00000001	/* TX enable */
 #define	GEM_MAC_TX_IGN_CARRIER	0x00000002	/* Ignore carrier sense */
-#define	GEM_MAC_TX_IGN_COLLIS	0x00000004	/* ignore collitions */
+#define	GEM_MAC_TX_IGN_COLLIS	0x00000004	/* ignore collisions */
 #define	GEM_MAC_TX_ENA_IPG0	0x00000008	/* extend Rx-to-TX IPG */
 #define	GEM_MAC_TX_NGU		0x00000010	/* Never give up */
 #define	GEM_MAC_TX_NGU_LIMIT	0x00000020	/* Never give up limit */
@@ -342,6 +369,11 @@
 #define	GEM_MAC_TX_NO_FCS	0x00000100	/* no FCS will be generated */
 #define	GEM_MAC_TX_CARR_EXTEND	0x00000200	/* Ena TX Carrier Extension */
 /* Carrier Extension is required for half duplex Gbps operation */
+#define	GEM_MAC_TX_CONFIG_BITS	"\177\020" \
+				"b\0TXENA\0b\1IGNCAR\0b\2IGNCOLLIS\0" \
+				"b\3IPG0ENA\0b\4TXNGU\0b\5TXNGULIM\0" \
+				"b\6NOBKOFF\0b\7SLOWDN\0b\x8NOFCS\0" \
+				"b\x9TXCARREXT\0\0"
 
 
 /* GEM_MAC_RX_CONFIG register bits */
@@ -358,12 +390,17 @@
  * Carrier Extension enables reception of packet bursts generated by
  * senders with carrier extension enabled.
  */
+#define	GEM_MAC_RX_CONFIG_BITS	"\177\020" \
+				"b\0RXENA\0b\1STRPAD\0b\2STRCRC\0" \
+				"b\3PROMIS\0b\4PROMISCGRP\0b\5HASHFLTR\0" \
+				"b\6ADDRFLTR\0b\7ERRCHKDIS\0b\x9TXCARREXT\0\0"
 
 
 /* GEM_MAC_CONTROL_CONFIG bits */
 #define	GEM_MAC_CC_TX_PAUSE	0x00000001	/* send pause enabled */
 #define	GEM_MAC_CC_RX_PAUSE	0x00000002	/* receive pause enabled */
 #define	GEM_MAC_CC_PASS_PAUSE	0x00000004	/* pass pause up */
+#define	GEM_MAC_CC_BITS		"\177\020b\0TXPAUSE\0b\1RXPAUSE\0b\2NOPAUSE\0\0"
 
 
 /* GEM MIF registers */
@@ -395,14 +432,16 @@
 
 
 /* GEM_MIF_CONFIG register bits */
-#define	GEM_MIF_CONFIG_PHY_SEL	0x00000001	/* PHY select */
+#define	GEM_MIF_CONFIG_PHY_SEL	0x00000001	/* PHY select, 0=MDIO0 */
 #define	GEM_MIF_CONFIG_POLL_ENA	0x00000002	/* poll enable */
 #define	GEM_MIF_CONFIG_BB_ENA	0x00000004	/* bit bang enable */
 #define	GEM_MIF_CONFIG_REG_ADR	0x000000f8	/* poll register address */
 #define	GEM_MIF_CONFIG_MDI0	0x00000100	/* MDIO_0 Data/MDIO_0 atached */
 #define	GEM_MIF_CONFIG_MDI1	0x00000200	/* MDIO_1 Data/MDIO_1 atached */
 #define	GEM_MIF_CONFIG_PHY_ADR	0x00007c00	/* poll PHY address */
-/* MDI0 is onboard tranciever MID1 is external, PHYAD for both is 0 */
+/* MDI0 is onboard transceiver MDI1 is external, PHYAD for both is 0 */
+#define	GEM_MIF_CONFIG_BITS	"\177\020b\0PHYSEL\0b\1POLL\0b\2BBENA\0" \
+				"b\x8MDIO0\0b\x9MDIO1\0\0"
 
 
 /* GEM_MIF_BASIC_STATUS and GEM_MIF_INTERRUPT_MASK bits */
@@ -416,47 +455,52 @@
  */
 
 
-/* The GEM PCS/Serial link register. */
+/* The GEM PCS/Serial link registers. */
+/* DO NOT TOUCH THESE REGISTERS ON ERI -- IT HARD HANGS. */
 #define	GEM_MII_CONTROL		0x9000
 #define	GEM_MII_STATUS		0x9004
 #define	GEM_MII_ANAR		0x9008		/* MII advertisement reg */
-#define	GEM_MII_ANLPAR		0x900c		/* LP ability reg */
+#define	GEM_MII_ANLPAR		0x900c		/* Link Partner Ability Reg */
 #define	GEM_MII_CONFIG		0x9010
 #define	GEM_MII_STATE_MACHINE	0x9014
-#define	GEM_MII_INTERRUP_STATUS	0x9018
+#define	GEM_MII_INTERRUP_STATUS	0x9018		/* PCS interrupt state */
 #define	GEM_MII_DATAPATH_MODE	0x9050
 #define	GEM_MII_SLINK_CONTROL	0x9054		/* Serial link control */
 #define	GEM_MII_OUTPUT_SELECT	0x9058
 #define	GEM_MII_SLINK_STATUS	0x905c		/* serial link status */
 
 
-/* GEM_MII_CONTROL bits */
-/*
- * DO NOT TOUCH THIS REGISTER ON ERI -- IT HARD HANGS.
- */
+/* GEM_MII_CONTROL bits - PCS "BMCR" (Basic Mode Control Reg) */
 #define	GEM_MII_CONTROL_RESET	0x00008000
 #define	GEM_MII_CONTROL_LOOPBK	0x00004000	/* 10-bit i/f loopback */
 #define	GEM_MII_CONTROL_1000M	0x00002000	/* speed select, always 0 */
 #define	GEM_MII_CONTROL_AUTONEG	0x00001000	/* auto negotiation enabled */
 #define	GEM_MII_CONTROL_POWERDN	0x00000800
 #define	GEM_MII_CONTROL_ISOLATE	0x00000400	/* isolate phy from mii */
-#define	GEM_MII_CONTROL_RAN	0x00000200	/* restart auto negotioation */
+#define	GEM_MII_CONTROL_RAN	0x00000200	/* restart auto negotiation */
 #define	GEM_MII_CONTROL_FDUPLEX	0x00000100	/* full duplex, always 0 */
 #define	GEM_MII_CONTROL_COL_TST	0x00000080	/* collision test */
+#define	GEM_MII_CONTROL_BITS	"\177\020b\7COLTST\0b\x8_FD\0b\x9RAN\0" \
+				"b\xaISOLATE\0b\xbPWRDWN\0b\xc_ANEG\0" \
+				"b\xdGIGE\0b\xeLOOP\0b\xfRESET\0\0"
 
 
-/* GEM_MII_STATUS reg */
+/* GEM_MII_STATUS reg - PCS "BMSR" (Basic Mode Status Reg) */
 #define	GEM_MII_STATUS_GB_FDX	0x00000400	/* can perform GBit FDX */
 #define	GEM_MII_STATUS_GB_HDX	0x00000200	/* can perform GBit HDX */
+#define	GEM_MII_STATUS_UNK	0x00000100
 #define	GEM_MII_STATUS_ANEG_CPT	0x00000020	/* auto negotiate compete */
 #define	GEM_MII_STATUS_REM_FLT	0x00000010	/* remote fault detected */
 #define	GEM_MII_STATUS_ACFG	0x00000008	/* can auto negotiate */
 #define	GEM_MII_STATUS_LINK_STS	0x00000004	/* link status */
 #define	GEM_MII_STATUS_JABBER	0x00000002	/* jabber condition detected */
 #define	GEM_MII_STATUS_EXTCAP	0x00000001	/* extended register capability */
+#define	GEM_MII_STATUS_BITS	"\177\020b\0EXTCAP\0b\1JABBER\0b\2LINKSTS\0" \
+				"b\3ACFG\0b\4REMFLT\0b\5ANEGCPT\0b\x9GBHDX\0" \
+				"b\xaGBFDX\0\0"
 
 
-/* GEM_MII_ANAR and GEM_MII_ANLAR reg bits */
+/* GEM_MII_ANAR and GEM_MII_ANLPAR reg bits */
 #define	GEM_MII_ANEG_NP		0x00008000	/* next page bit */
 #define	GEM_MII_ANEG_ACK	0x00004000	/* ack reception of */
 						/* Link Partner Capability */
@@ -465,27 +509,59 @@
 #define	GEM_MII_ANEG_SYM_PAUSE	0x00000080	/* symmetric pause */
 #define	GEM_MII_ANEG_HLF_DUPLX	0x00000040
 #define	GEM_MII_ANEG_FUL_DUPLX	0x00000020
+#define	GEM_MII_ANEG_BITS	"\177\020b\5FDX\0b\6HDX\0b\7SYMPAUSE\0" \
+				"\b\x8_ASYMPAUSE\0\b\xdREMFLT\0\b\xeLPACK\0" \
+				"\b\xfNPBIT\0\0"
 
 
 /* GEM_MII_CONFIG reg */
-#define	GEM_MII_CONFIG_TIMER	0x0000001c	/* link monitor timer values */
+#define	GEM_MII_CONFIG_TIMER	0x0000000e	/* link monitor timer values */
+#define	GEM_MII_CONFIG_ANTO	0x00000020	/* 10ms ANEG timer override */
+#define	GEM_MII_CONFIG_JS	0x00000018	/* Jitter Study, 0 normal
+						 * 1 high freq, 2 low freq */
+#define	GEM_MII_CONFIG_SDL	0x00000004	/* Signal Detect active low */
+#define	GEM_MII_CONFIG_SDO	0x00000002	/* Signal Detect Override */
 #define	GEM_MII_CONFIG_ENABLE	0x00000001	/* Enable PCS */
+#define	GEM_MII_CONFIG_BITS	"\177\020b\0PCSENA\0\0"
+
+
+/*
+ * GEM_MII_STATE_MACHINE
+ * XXX These are best guesses from observed behavior.
+ */
+#define	GEM_MII_FSM_STOP	0x00000000	/* stopped */
+#define	GEM_MII_FSM_RUN		0x00000001	/* running */
+#define	GEM_MII_FSM_UNKWN	0x00000100	/* unknown */
+#define	GEM_MII_FSM_DONE	0x00000101	/* complete */
+
+
+/*
+ * GEM_MII_INTERRUP_STATUS reg
+ * No mask register; mask with the global interrupt mask register.
+ */
+#define	GEM_MII_INTERRUP_LINK	0x00000004	/* PCS link status change */
 
 
 /* GEM_MII_DATAPATH_MODE reg */
 #define	GEM_MII_DATAPATH_SERIAL	0x00000001	/* Serial link */
 #define	GEM_MII_DATAPATH_SERDES	0x00000002	/* Use PCS via 10bit interfac */
-#define	GEM_MII_DATAPATH_MII	0x00000004	/* Use MII, not PCS */
+#define	GEM_MII_DATAPATH_MII	0x00000004	/* Use {G}MII, not PCS */
 #define	GEM_MII_DATAPATH_MIIOUT	0x00000008	/* enable serial output on GMII */
+#define	GEM_MII_DATAPATH_BITS	"\177\020"	\
+				"b\0SERIAL\0b\1SERDES\0b\2MII\0b\3MIIOUT\0\0"
 
 
 /* GEM_MII_SLINK_CONTROL reg */
-#define	GEM_MII_SLINK_LOOPBACK	0x00000001	/* enable loopback at sl */
+#define	GEM_MII_SLINK_LOOPBACK	0x00000001	/* enable loopback at sl, logic
+						 * reversed for SERDES */
 #define	GEM_MII_SLINK_EN_SYNC_D	0x00000002	/* enable sync detection */
 #define	GEM_MII_SLINK_LOCK_REF	0x00000004	/* lock reference clock */
 #define	GEM_MII_SLINK_EMPHASIS	0x00000008	/* enable emphasis */
 #define	GEM_MII_SLINK_SELFTEST	0x000001c0
 #define	GEM_MII_SLINK_POWER_OFF	0x00000200	/* Power down serial link */
+#define	GEM_MII_SLINK_CONTROL_BITS		\
+				"\177\020b\0LOOP\0b\1ENASYNC\0b\2LOCKREF" \
+				"\0b\3EMPHASIS\0b\x9PWRDWN\0\0"
 
 
 /* GEM_MII_SLINK_STATUS reg */
@@ -494,6 +570,13 @@
 #define	GEM_MII_SLINK_COMMA	0x00000002	/* waiting for comma detect */
 #define	GEM_MII_SLINK_SYNC	0x00000003	/* recv data synchronized */
 
+/*
+ * PCI Expansion ROM runtime access
+ * Sun GEMs map a 1MB space for the PCI Expansion ROM as the second half
+ * of the first register bank, although they only support up to 64KB ROMs.
+ */
+#define	GEM_PCI_ROM_OFFSET	0x100000
+#define	GEM_PCI_ROM_SIZE	0x10000
 
 /* Wired GEM PHY addresses */
 #define	GEM_PHYAD_INTERNAL	1
@@ -510,7 +593,9 @@ struct gem_desc {
 /* Transmit flags */
 #define	GEM_TD_BUFSIZE		0x0000000000007fffLL
 #define	GEM_TD_CXSUM_START	0x00000000001f8000LL	/* Cxsum start offset */
+#define	GEM_TD_CXSUM_STARTSHFT	15
 #define	GEM_TD_CXSUM_STUFF	0x000000001fe00000LL	/* Cxsum stuff offset */
+#define	GEM_TD_CXSUM_STUFFSHFT	21
 #define	GEM_TD_CXSUM_ENABLE	0x0000000020000000LL	/* Cxsum generation enable */
 #define	GEM_TD_END_OF_PACKET	0x0000000040000000LL
 #define	GEM_TD_START_OF_PACKET	0x0000000080000000LL
@@ -522,7 +607,7 @@ struct gem_desc {
  */
 
 /* Receive flags */
-#define	GEM_RD_CHECKSUM		0x000000000000ffffLL
+#define	GEM_RD_CHECKSUM		0x000000000000ffffLL	/* is the complement */
 #define	GEM_RD_BUFSIZE		0x000000007fff0000LL
 #define	GEM_RD_OWN		0x0000000080000000LL	/* 1 - owned by h/w */
 #define	GEM_RD_HASHVAL		0x0ffff00000000000LL
@@ -531,9 +616,6 @@ struct gem_desc {
 #define	GEM_RD_BAD_CRC		0x4000000000000000LL
 
 #define	GEM_RD_BUFSHIFT		16
-#define	GEM_RD_BUFLEN(x)	(((x)&GEM_RD_BUFSIZE)>>GEM_RD_BUFSHIFT)
-
-/* PCI support */
-#define PCI_GEM_BASEADDR	0x10
+#define	GEM_RD_BUFLEN(x)	(((x) & GEM_RD_BUFSIZE) >> GEM_RD_BUFSHIFT)
 
 #endif
