@@ -77,7 +77,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/idt/idt.c,v 1.9.2.1 2005/01/30 00:59:46 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/idt/idt.c,v 1.12 2006/05/16 14:36:26 phk Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -95,7 +95,6 @@ __FBSDID("$FreeBSD: src/sys/dev/idt/idt.c,v 1.9.2.1 2005/01/30 00:59:46 imp Exp
 #include <sys/proc.h>
 #include <sys/signalvar.h>
 #include <sys/mman.h>
-#include <machine/clock.h>
 #include <machine/cpu.h>	/* bootverbose */
 
 #include <sys/bus.h>
@@ -1556,7 +1555,7 @@ idt_queue_put(CONNECTION * connection, struct mbuf * m)
 		return (1);
 	}
 	m->m_nextpkt = NULL;
-	m->m_pkthdr.rcvif = (struct ifnet *) connection;
+	m->m_pkthdr.rcvif = (void *) connection;
 
 	s = splimp();
 
@@ -1599,7 +1598,7 @@ idt_queue_flush(CONNECTION * connection)
 	m0 = &txqueue->mget;
 	m1 = *m0;
 	while (m1 != NULL) {
-		if (m1->m_pkthdr.rcvif == (struct ifnet *) connection) {
+		if (m1->m_pkthdr.rcvif == (void *) connection) {
 			*m0 = m1->m_nextpkt;
 			m_freem(m1);
 			m1 = *m0;
