@@ -34,21 +34,23 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/if_ndis/if_ndis.c,v 1.99.2.6 2005/12/16 17:33:47 wpaul Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/if_ndis/if_ndis.c,v 1.124.2.1 2007/12/05 02:53:44 thompsa Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/sockio.h>
 #include <sys/mbuf.h>
 #include <sys/malloc.h>
+#include <sys/endian.h>
+#include <sys/priv.h>
 #include <sys/kernel.h>
 #include <sys/socket.h>
 #include <sys/queue.h>
 #include <sys/module.h>
 #include <sys/proc.h>
-#if __FreeBSD_version < 502113
 #include <sys/sysctl.h>
-#endif
+#include <sys/kthread.h>
+#include <sys/taskqueue.h>
 
 
 #include <net/if.h>
@@ -68,8 +70,11 @@ __FBSDID("$FreeBSD: src/sys/dev/if_ndis/if_ndis.c,v 1.99.2.6 2005/12/16 17:33:47
 
 #include <net80211/ieee80211_var.h>
 #include <net80211/ieee80211_ioctl.h>
+#include <net80211/ieee80211_regdomain.h>
 
+#if __FreeBSD_version < 700046
 #include <dev/wi/if_wavelan_ieee.h>
+#endif
 
 #include <dev/pci/pcireg.h>
 #include <dev/pci/pcivar.h>
@@ -82,6 +87,16 @@ __FBSDID("$FreeBSD: src/sys/dev/if_ndis/if_ndis.c,v 1.99.2.6 2005/12/16 17:33:47
 #include <compat/ndis/ndis_var.h>
 #include <dev/if_ndis/if_ndisvar.h>
 
+#define NDIS_DEBUG
+#ifdef NDIS_DEBUG
+#define DPRINTF(x)	do { if (ndis_debug > 0) printf x; } while (0)
+int ndis_debug = 0;
+SYSCTL_INT(_debug, OID_AUTO, ndis, CTLFLAG_RW, &ndis_debug, 0,
+    "if_ndis debug level");
+#else
+#define DPRINTF(x)
+#endif
+
 MODULE_DEPEND(ndis, ether, 1, 1, 1);
 MODULE_DEPEND(ndis, wlan, 1, 1, 1);
 MODULE_DEPEND(ndis, ndisapi, 1, 1, 1);
@@ -129,10 +144,23 @@ static void ndis_starttask	(device_object *, void *);
 static void ndis_resettask	(device_object *, void *);
 static void ndis_inputtask	(device_object *, void *);
 static int ndis_ioctl		(struct ifnet *, u_long, caddr_t);
+#if __FreeBSD_version < 700046
 static int ndis_wi_ioctl_get	(struct ifnet *, u_long, caddr_t);
 static int ndis_wi_ioctl_set	(struct ifnet *, u_long, caddr_t);
+#endif
 static int ndis_80211_ioctl_get	(struct ifnet *, u_long, caddr_t);
 static int ndis_80211_ioctl_set	(struct ifnet *, u_long, caddr_t);
+static int ndis_newstate	(struct ieee80211com *, enum ieee80211_state,
+	int);
+static int ndis_nettype_chan	(uint32_t);
+static int ndis_nettype_mode	(uint32_t);
+static void ndis_scan		(void *, int);
+static void ndis_scan_results	(struct ndis_softc *);
+static void ndis_scan_start	(struct ieee80211com *);
+static void ndis_scan_end	(struct ieee80211com *);
+static void ndis_set_channel	(struct ieee80211com *);
+static void ndis_scan_curchan	(struct ieee80211com *, unsigned long);
+static void ndis_scan_mindwell	(struct ieee80211com *);
 static void ndis_init		(void *);
 static void ndis_stop		(struct ndis_softc *);
 static void ndis_watchdog	(struct ifnet *);
@@ -463,6 +491,32 @@ ndis_probe_offload(sc)
 	return(0);
 }
 
+static int
+ndis_nettype_chan(uint32_t type)
+{
+	switch (type) {
+	case NDIS_80211_NETTYPE_11FH:		return (IEEE80211_CHAN_FHSS);
+	case NDIS_80211_NETTYPE_11DS:		return (IEEE80211_CHAN_B);
+	case NDIS_80211_NETTYPE_11OFDM5:	return (IEEE80211_CHAN_A);
+	case NDIS_80211_NETTYPE_11OFDM24:	return (IEEE80211_CHAN_G);
+	}
+	DPRINTF(("unknown channel nettype %d\n", type));
+	return (IEEE80211_CHAN_B);	/* Default to 11B chan */
+}
+
+static int
+ndis_nettype_mode(uint32_t type)
+{
+	switch (type) {
+	case NDIS_80211_NETTYPE_11FH:		return (IEEE80211_MODE_FH);
+	case NDIS_80211_NETTYPE_11DS:		return (IEEE80211_MODE_11B);
+	case NDIS_80211_NETTYPE_11OFDM5:	return (IEEE80211_MODE_11A);
+	case NDIS_80211_NETTYPE_11OFDM24:	return (IEEE80211_MODE_11G);
+	}
+	DPRINTF(("unknown mode nettype %d\n", type));
+	return (IEEE80211_MODE_AUTO);
+}
+
 /*
  * Attach the interface. Allocate softc structures, do ifmedia
  * setup and ethernet/BPF attach.
@@ -476,7 +530,7 @@ ndis_attach(dev)
 	driver_object		*pdrv;
 	device_object		*pdo;
 	struct ifnet		*ifp = NULL;
-	int			error = 0, len;
+	int			error = 0, len, mode, bands = 0;
 	int			i;
 
 	sc = device_get_softc(dev);
@@ -552,6 +606,21 @@ ndis_attach(dev)
 	sc->ndis_block->nmb_ethrxdone_func = ndis_rxeof_done_wrap;
 	sc->ndis_block->nmb_tdcond_func = ndis_rxeof_xfr_done_wrap;
 
+	/* Override the status handler so we can detect link changes. */
+	sc->ndis_block->nmb_status_func = ndis_linksts_wrap;
+	sc->ndis_block->nmb_statusdone_func = ndis_linksts_done_wrap;
+
+	/* Set up work item handlers. */
+	sc->ndis_tickitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
+	sc->ndis_startitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
+	sc->ndis_resetitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
+	sc->ndis_inputitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
+	KeInitializeDpc(&sc->ndis_rxdpc, ndis_rxeof_xfr_wrap, sc->ndis_block);
+
+        /* make sure drv flags are all cleared before initing the NIC. */
+
+        ifp->if_drv_flags = 0;
+
 	/* Call driver's init routine. */
 	if (ndis_init_nic(sc)) {
 		device_printf (dev, "init handler failed\n");
@@ -664,14 +733,26 @@ ndis_attach(dev)
 		uint32_t		arg;
 		int			r;
 
+#if __FreeBSD_version >= 700000
+		sc->ndis_tq = taskqueue_create("nids_taskq", M_NOWAIT | M_ZERO,
+		    taskqueue_thread_enqueue, &sc->ndis_tq);
+		taskqueue_start_threads(&sc->ndis_tq, 1, PI_NET, "%s taskq",
+		    device_get_nameunit(dev));
+#else
+		sc->ndis_tq = taskqueue_create("ndis_taskq", M_NOWAIT | M_ZERO,
+		    taskqueue_thread_enqueue, &sc->ndis_tq, &sc->sc_tqproc);
+		kthread_create(taskqueue_thread_loop, &sc->ndis_tq, &sc->sc_tqproc,
+		    0, 0, "%s taskq", device_get_nameunit(dev));
+#endif
+		TASK_INIT(&sc->ndis_scantask, 0, ndis_scan, sc);
+
 #if __FreeBSD_version >= 600007
 		ic->ic_ifp = ifp;
 #endif
 	        ic->ic_phytype = IEEE80211_T_DS;
 		ic->ic_opmode = IEEE80211_M_STA;
 		ic->ic_caps = IEEE80211_C_IBSS;
-		ic->ic_state = IEEE80211_S_ASSOC;
-		ic->ic_modecaps = (1<<IEEE80211_MODE_AUTO);
+		setbit(ic->ic_modecaps, IEEE80211_MODE_AUTO);
 		len = 0;
 		r = ndis_get_info(sc, OID_802_11_NETWORK_TYPES_SUPPORTED,
 		    NULL, &len);
@@ -686,23 +767,21 @@ ndis_attach(dev)
 		}
 
 		for (i = 0; i < ntl->ntl_items; i++) {
-			switch (ntl->ntl_type[i]) {
-			case NDIS_80211_NETTYPE_11FH:
-			case NDIS_80211_NETTYPE_11DS:
-				ic->ic_modecaps |= (1<<IEEE80211_MODE_11B);
-				break;
-			case NDIS_80211_NETTYPE_11OFDM5:
-				ic->ic_modecaps |= (1<<IEEE80211_MODE_11A);
-				break;
-			case NDIS_80211_NETTYPE_11OFDM24:
-				ic->ic_modecaps |= (1<<IEEE80211_MODE_11G);
-				break;
-			default:
-				break;
-			}
+			mode = ndis_nettype_mode(ntl->ntl_type[i]);
+			if (mode) {
+				setbit(ic->ic_modecaps, mode);
+				setbit(&bands, mode);
+			} else
+				device_printf(dev, "Unknown nettype %d\n",
+				    ntl->ntl_type[i]);
 		}
 		free(ntl, M_DEVBUF);
 nonettypes:
+		/* Default to 11b channels if the card did not supply any */
+		if (bands == 0) {
+			setbit(ic->ic_modecaps, IEEE80211_MODE_11B);
+			setbit(&bands, IEEE80211_MODE_11B);
+		}
 		len = sizeof(rates);
 		bzero((char *)&rates, len);
 		r = ndis_get_info(sc, OID_802_11_SUPPORTED_RATES,
@@ -734,11 +813,11 @@ nonettypes:
 	ic->ic_sup_rates[x].rs_nrates++
 
 		ic->ic_curmode = IEEE80211_MODE_AUTO;
-		if (ic->ic_modecaps & (1<<IEEE80211_MODE_11A))
+		if (isset(ic->ic_modecaps, IEEE80211_MODE_11A))
 			ic->ic_sup_rates[IEEE80211_MODE_11A].rs_nrates = 0;
-		if (ic->ic_modecaps & (1<<IEEE80211_MODE_11B))
+		if (isset(ic->ic_modecaps, IEEE80211_MODE_11B))
 			ic->ic_sup_rates[IEEE80211_MODE_11B].rs_nrates = 0;
-		if (ic->ic_modecaps & (1<<IEEE80211_MODE_11G))
+		if (isset(ic->ic_modecaps, IEEE80211_MODE_11G))
 			ic->ic_sup_rates[IEEE80211_MODE_11G].rs_nrates = 0;
 		for (i = 0; i < len; i++) {
 			switch (rates[i] & IEEE80211_RATE_VAL) {
@@ -747,11 +826,10 @@ nonettypes:
 			case 11:
 			case 10:
 			case 22:
-				if (!(ic->ic_modecaps &
-				    (1<<IEEE80211_MODE_11B))) {
+				if (isclr(ic->ic_modecaps, IEEE80211_MODE_11B)) {
 					/* Lazy-init 802.11b. */
-					ic->ic_modecaps |=
-					    (1<<IEEE80211_MODE_11B);
+					setbit(ic->ic_modecaps,
+					    IEEE80211_MODE_11B);
 					ic->ic_sup_rates[IEEE80211_MODE_11B].
 					    rs_nrates = 0;
 				}
@@ -759,11 +837,11 @@ nonettypes:
 				INCRATE(IEEE80211_MODE_11B);
 				break;
 			default:
-				if (ic->ic_modecaps & (1<<IEEE80211_MODE_11A)) {
+				if (isset(ic->ic_modecaps, IEEE80211_MODE_11A)) {
 					SETRATE(IEEE80211_MODE_11A, rates[i]);
 					INCRATE(IEEE80211_MODE_11A);
 				}
-				if (ic->ic_modecaps & (1<<IEEE80211_MODE_11G)) {
+				if (isset(ic->ic_modecaps, IEEE80211_MODE_11G)) {
 					SETRATE(IEEE80211_MODE_11G, rates[i]);
 					INCRATE(IEEE80211_MODE_11G);
 				}
@@ -778,7 +856,7 @@ nonettypes:
 		 * just cheat here.  Just how in the heck do
 		 * we detect turbo modes, though?
 		 */
-		if (ic->ic_modecaps & (1<<IEEE80211_MODE_11B)) {
+		if (isset(ic->ic_modecaps, IEEE80211_MODE_11B)) {
 			TESTSETRATE(IEEE80211_MODE_11B,
 			    IEEE80211_RATE_BASIC|2);
 			TESTSETRATE(IEEE80211_MODE_11B,
@@ -788,13 +866,13 @@ nonettypes:
 			TESTSETRATE(IEEE80211_MODE_11B,
 			    IEEE80211_RATE_BASIC|22);
 		}
-		if (ic->ic_modecaps & (1<<IEEE80211_MODE_11G)) {
+		if (isset(ic->ic_modecaps, IEEE80211_MODE_11G)) {
 			TESTSETRATE(IEEE80211_MODE_11G, 47);
 			TESTSETRATE(IEEE80211_MODE_11G, 72);
 			TESTSETRATE(IEEE80211_MODE_11G, 96);
 			TESTSETRATE(IEEE80211_MODE_11G, 108);
 		}
-		if (ic->ic_modecaps & (1<<IEEE80211_MODE_11A)) {
+		if (isset(ic->ic_modecaps, IEEE80211_MODE_11A)) {
 			TESTSETRATE(IEEE80211_MODE_11A, 47);
 			TESTSETRATE(IEEE80211_MODE_11A, 72);
 			TESTSETRATE(IEEE80211_MODE_11A, 96);
@@ -802,25 +880,7 @@ nonettypes:
 		}
 #undef SETRATE
 #undef INCRATE
-		/*
-		 * Taking yet more guesses here.
-		 */
-		for (i = 1; i < IEEE80211_CHAN_MAX; i++) {
-			int chanflag = 0;
-
-			if (ic->ic_sup_rates[IEEE80211_MODE_11G].rs_nrates)
-				chanflag |= IEEE80211_CHAN_G;
-			if (i <= 14)
-				chanflag |= IEEE80211_CHAN_B;
-			if (ic->ic_sup_rates[IEEE80211_MODE_11A].rs_nrates &&
-			    i > 14)
-				chanflag = IEEE80211_CHAN_A;
-			if (chanflag == 0)
-				break;
-			ic->ic_channels[i].ic_freq =
-			    ieee80211_ieee2mhz(i, chanflag);
-			ic->ic_channels[i].ic_flags = chanflag;
-		}
+		ieee80211_init_channels(ic, 0, CTRY_DEFAULT, bands, 0, 1);
 
 		/*
 		 * To test for WPA support, we need to see if we can
@@ -882,8 +942,16 @@ got_crypto:
 		ieee80211_media_init(ic, ieee80211_media_change,
 		    ndis_media_status);
 #endif
-		ic->ic_ibss_chan = IEEE80211_CHAN_ANYC;
-		ic->ic_bss->ni_chan = ic->ic_ibss_chan;
+		ic->ic_scan_start = ndis_scan_start;
+		ic->ic_scan_end = ndis_scan_end;
+		ic->ic_set_channel = ndis_set_channel;
+		ic->ic_scan_curchan = ndis_scan_curchan;
+		ic->ic_scan_mindwell = ndis_scan_mindwell;
+		ic->ic_bsschan = IEEE80211_CHAN_ANYC;
+		ic->ic_bss->ni_chan = ic->ic_bsschan;
+		/* override state transition machine */
+		sc->ndis_newstate = ic->ic_newstate;
+		ic->ic_newstate = ndis_newstate;
 #ifdef IEEE80211_F_WPA
 		/* install key handing routines */
 		ic->ic_crypto.cs_key_set = ndis_add_key;
@@ -902,18 +970,6 @@ got_crypto:
 		ether_ifattach(ifp, eaddr);
 	}
 
-	/* Override the status handler so we can detect link changes. */
-	sc->ndis_block->nmb_status_func = ndis_linksts_wrap;
-	sc->ndis_block->nmb_statusdone_func = ndis_linksts_done_wrap;
-
-	/* Set up work item handlers. */
-	sc->ndis_tickitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
-	sc->ndis_startitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
-	sc->ndis_resetitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
-	sc->ndis_inputitem = IoAllocateWorkItem(sc->ndis_block->nmb_deviceobj);
-	KeInitializeDpc(&sc->ndis_rxdpc, ndis_rxeof_xfr_wrap, sc->ndis_block);
-
-
 fail:
 	if (error)
 		ndis_detach(dev);
@@ -958,6 +1014,8 @@ ndis_detach(dev)
 	} else
 		NDIS_UNLOCK(sc);
 
+	taskqueue_drain(sc->ndis_tq, &sc->ndis_scantask);
+
 	if (sc->ndis_tickitem != NULL)
 		IoFreeWorkItem(sc->ndis_tickitem);
 	if (sc->ndis_startitem != NULL)
@@ -968,6 +1026,7 @@ ndis_detach(dev)
 		IoFreeWorkItem(sc->ndis_inputitem);
 
 	bus_generic_detach(dev);
+	ndis_unload_driver(sc);
 
 	if (sc->ndis_irq)
 		bus_release_resource(dev, SYS_RES_IRQ, 0, sc->ndis_irq);
@@ -1001,8 +1060,6 @@ ndis_detach(dev)
 	if (sc->ndis_txpool != NULL)
 		NdisFreePacketPool(sc->ndis_txpool);
 
-	ndis_unload_driver(sc);
-
 	/* Destroy the PDO for this device. */
 	
 	if (sc->ndis_iftype == PCIBus)
@@ -1018,6 +1075,7 @@ ndis_detach(dev)
 	if (sc->ndis_iftype == PCIBus)
 		bus_dma_tag_destroy(sc->ndis_parent_tag);
 
+	taskqueue_free(sc->ndis_tq);
 #if __FreeBSD_version < 502113
 	sysctl_ctx_free(&sc->ndis_ctx);
 #endif
@@ -1079,7 +1137,7 @@ ndis_rxeof_eth(adapter, ctx, addr, hdr, hdrlen, lookahead, lookaheadlen, pktlen)
 	uint32_t		pktlen;
 {
 	ndis_miniport_block	*block;
-	uint8_t			irql;
+	uint8_t			irql = 0;
 	uint32_t		status;
 	ndis_buffer		*b;
 	ndis_packet		*p;
@@ -1317,6 +1375,23 @@ ndis_rxeof(adapter, packets, pktcnt)
 	sc = device_get_softc(block->nmb_physdeviceobj->do_devext);
 	ifp = sc->ifp;
 
+	/*
+	 * There's a slim chance the driver may indicate some packets
+	 * before we're completely ready to handle them. If we detect this,
+	 * we need to return them to the miniport and ignore them.
+	 */
+
+        if (!(ifp->if_drv_flags & IFF_DRV_RUNNING)) {
+		for (i = 0; i < pktcnt; i++) {
+			p = packets[i];
+			if (p->np_oob.npo_status == NDIS_STATUS_SUCCESS) {
+				p->np_refcnt++;
+				ndis_return_packet(p, block);
+			}
+		}
+		return;
+        }
+
 	for (i = 0; i < pktcnt; i++) {
 		p = packets[i];
 		/* Stash the softc here so ptom can use it. */
@@ -1324,7 +1399,7 @@ ndis_rxeof(adapter, packets, pktcnt)
 		if (ndis_ptom(&m0, p)) {
 			device_printf (sc->ndis_dev, "ptom failed\n");
 			if (p->np_oob.npo_status == NDIS_STATUS_SUCCESS)
-				ndis_return_packet(sc, p);
+				ndis_return_packet(p, block);
 		} else {
 #ifdef notdef
 			if (p->np_oob.npo_status == NDIS_STATUS_RESOURCES) {
@@ -1380,7 +1455,7 @@ ndis_rxeof(adapter, packets, pktcnt)
 			}
 
 			KeAcquireSpinLockAtDpcLevel(&sc->ndis_rxlock);
-			_IF_ENQUEUE(&sc->ndis_rxqueue, m);
+			_IF_ENQUEUE(&sc->ndis_rxqueue, m0);
 			KeReleaseSpinLockFromDpcLevel(&sc->ndis_rxlock);
 			IoQueueWorkItem(sc->ndis_inputitem,
 			    (io_workitem_func)ndis_inputtask_wrap,
@@ -1620,7 +1695,7 @@ ndis_ticktask(d, xsc)
 		NDIS_UNLOCK(sc);
 		if (sc->ndis_80211) {
 			ndis_getstate_80211(sc);
-			ic->ic_state = IEEE80211_S_RUN;
+			ieee80211_new_state(ic, IEEE80211_S_RUN, -1);
 		}
 		NDIS_LOCK(sc);
 #ifdef LINK_STATE_UP
@@ -1639,7 +1714,7 @@ ndis_ticktask(d, xsc)
 	    sc->ndis_sts == NDIS_STATUS_MEDIA_DISCONNECT) {
 		sc->ndis_link = 0;
 		if (sc->ndis_80211)
-			ic->ic_state = IEEE80211_S_ASSOC;
+			ieee80211_new_state(ic, IEEE80211_S_SCAN, 0);
 #ifdef LINK_STATE_DOWN
 #if __FreeBSD_version > 600006
 		if_link_state_change(sc->ifp, LINK_STATE_DOWN);
@@ -1860,6 +1935,7 @@ ndis_init(xsc)
 	void			*xsc;
 {
 	struct ndis_softc	*sc = xsc;
+	struct ieee80211com	*ic = (void *)&sc->ic;
 	struct ifnet		*ifp = sc->ifp;
 	int			i, len, error;
 
@@ -1934,6 +2010,18 @@ ndis_init(xsc)
 #endif
 #endif /* LINK_STATE_UNKNOWN */
 
+	if (ic->ic_opmode != IEEE80211_M_MONITOR) {
+		/*
+		 * NB: When restarting the adapter clock the state
+		 * machine regardless of the roaming mode; otherwise
+		 * we need to notify user apps so they can manually
+		 * get us going again.
+		 */
+		if (ic->ic_roaming != IEEE80211_ROAMING_MANUAL)
+			ieee80211_new_state(ic, IEEE80211_S_SCAN, 0);
+	} else
+		ieee80211_new_state(ic, IEEE80211_S_RUN, -1);
+
 	ifp->if_drv_flags |= IFF_DRV_RUNNING;
 	ifp->if_drv_flags &= ~IFF_DRV_OACTIVE;
 
@@ -2053,6 +2141,7 @@ ndis_set_cipher(sc, cipher)
 		arg = NDIS_80211_WEPSTAT_ENC3ENABLED;
 	}
 
+	DPRINTF(("Setting cipher to %d\n", arg));
 	save = arg;
 	rval = ndis_set_info(sc, OID_802_11_ENCRYPTION_STATUS, &arg, &len);
 
@@ -2103,8 +2192,10 @@ ndis_set_wpa(sc)
 	w = (struct ieee80211_ie_wpa *)ic->ic_opt_ie;
 
 	/* Check for the right kind of IE. */
-	if (w->wpa_id != IEEE80211_ELEMID_VENDOR)
+	if (w->wpa_id != IEEE80211_ELEMID_VENDOR) {
+		DPRINTF(("Incorrect IE type %d\n", w->wpa_id));
 		return(EINVAL);
+	}
 
 	/* Skip over the ucast cipher OIDs. */
 	pos = (char *)&w->wpa_uciphers[0];
@@ -2128,6 +2219,7 @@ ndis_set_wpa(sc)
 	if (n->ni_val == WPA_ASE_8021X_PSK)
 		arg = NDIS_80211_AUTHMODE_WPAPSK;
 
+	DPRINTF(("Setting WPA auth mode to %d\n", arg));
 	i = sizeof(arg);
 	if (ndis_set_info(sc, OID_802_11_AUTHENTICATION_MODE, &arg, &i))
 		return(ENOTSUP);
@@ -2161,6 +2253,7 @@ ndis_setstate_80211(sc)
 	struct ndis_softc	*sc;
 {
 	struct ieee80211com	*ic;
+	struct ieee80211_node	*ni;
 	ndis_80211_ssid		ssid;
 	ndis_80211_macaddr	bssid;
 	ndis_80211_config	config;
@@ -2171,9 +2264,12 @@ ndis_setstate_80211(sc)
 
 	ic = &sc->ic;
 	ifp = sc->ifp;
+	ni = ic->ic_bss;
 
-	if (!NDIS_INITIALIZED(sc))
+	if (!NDIS_INITIALIZED(sc)) {
+		DPRINTF(("%s: NDIS not initialized\n", __func__));
 		return;
+	}
 
 	/* Disassociate and turn off radio. */
 
@@ -2246,6 +2342,15 @@ ndis_setstate_80211(sc)
 #endif
 		int			keys_set = 0;
 
+#if __FreeBSD_version >= 600007
+		if (ic->ic_bss->ni_authmode == IEEE80211_AUTH_SHARED) {
+			len = sizeof(arg);
+			arg = NDIS_80211_AUTHMODE_SHARED;
+			DPRINTF(("Setting shared auth\n"));
+			ndis_set_info(sc, OID_802_11_AUTHENTICATION_MODE,
+			    &arg, &len);
+		}
+#endif
 		for (i = 0; i < IEEE80211_WEP_NKID; i++) {
 			if (ic->ic_nw_keys[i].wk_keylen) {
 #if __FreeBSD_version >= 600007
@@ -2279,6 +2384,7 @@ ndis_setstate_80211(sc)
 				bcopy(ic->ic_nw_keys[i].wk_key,
 				    wep.nw_keydata, wep.nw_length);
 				len = sizeof(wep);
+				DPRINTF(("Setting WEP key %d\n", i));
 				rval = ndis_set_info(sc,
 				    OID_802_11_ADD_WEP, &wep, &len);
 				if (rval)
@@ -2288,6 +2394,7 @@ ndis_setstate_80211(sc)
 			}
 		}
 		if (keys_set) {
+			DPRINTF(("Setting WEP on\n"));
 			arg = NDIS_80211_WEPSTAT_ENABLED;
 			len = sizeof(arg);
 			rval = ndis_set_info(sc,
@@ -2338,6 +2445,7 @@ ndis_setstate_80211(sc)
 	}
 
 	if (arg) {
+		DPRINTF(("Setting network type to %d\n", arg));
 		len = sizeof(arg);
 		rval = ndis_set_info(sc, OID_802_11_NETWORK_TYPE_IN_USE,
 		    &arg, &len);
@@ -2364,20 +2472,21 @@ ndis_setstate_80211(sc)
 		config.nc_atimwin = 100;
 	if (config.nc_fhconfig.ncf_dwelltime == 0)
 		config.nc_fhconfig.ncf_dwelltime = 200;
-	if (rval == 0 && ic->ic_ibss_chan != IEEE80211_CHAN_ANYC) { 
+	if (rval == 0 && ic->ic_bsschan != IEEE80211_CHAN_ANYC) { 
 		int chan, chanflag;
 
-		chan = ieee80211_chan2ieee(ic, ic->ic_ibss_chan);
+		chan = ieee80211_chan2ieee(ic, ic->ic_bsschan);
 		chanflag = config.nc_dsconfig > 2500000 ? IEEE80211_CHAN_2GHZ :
 		    IEEE80211_CHAN_5GHZ;
 		if (chan != ieee80211_mhz2ieee(config.nc_dsconfig / 1000, 0)) {
 			config.nc_dsconfig =
-			    ic->ic_ibss_chan->ic_freq * 1000;
-			ic->ic_bss->ni_chan = ic->ic_ibss_chan;
+				ic->ic_bsschan->ic_freq * 1000;
+			ic->ic_bss->ni_chan = ic->ic_bsschan;
 			len = sizeof(config);
 			config.nc_length = len;
 			config.nc_fhconfig.ncf_length =
 			    sizeof(ndis_80211_config_fh);
+			DPRINTF(("Setting channel to %ukHz\n", config.nc_dsconfig));
 			rval = ndis_set_info(sc, OID_802_11_CONFIGURATION,
 			    &config, &len);
 			if (rval)
@@ -2406,10 +2515,11 @@ ndis_setstate_80211(sc)
 	len = IEEE80211_ADDR_LEN;
 	if (ic->ic_flags & IEEE80211_F_DESBSSID &&
 	    ic->ic_opmode != IEEE80211_M_IBSS)
-		bcopy(ic->ic_des_bssid, bssid, len);
+		bcopy(ni->ni_bssid, bssid, len);
 	else
 		bcopy(ifp->if_broadcastaddr, bssid, len);
 
+	DPRINTF(("Setting BSSID to %6D\n", &bssid, ":"));
 	rval = ndis_set_info(sc, OID_802_11_BSSID, &bssid, &len);
 	if (rval)
 		device_printf(sc->ndis_dev,
@@ -2417,20 +2527,29 @@ ndis_setstate_80211(sc)
 
 	/* Set SSID -- always do this last. */
 
+#ifdef NDIS_DEBUG
+	if (ndis_debug > 0) {
+		printf("Setting ESSID to ");
+		ieee80211_print_essid(ni->ni_essid, ni->ni_esslen);
+		printf("\n");
+	}
+#endif
+
 	len = sizeof(ssid);
 	bzero((char *)&ssid, len);
-	ssid.ns_ssidlen = ic->ic_des_esslen;
+	ssid.ns_ssidlen = ni->ni_esslen;
 	if (ssid.ns_ssidlen == 0) {
 		ssid.ns_ssidlen = 1;
 	} else
-		bcopy(ic->ic_des_essid, ssid.ns_ssid, ssid.ns_ssidlen);
+		bcopy(ni->ni_essid, ssid.ns_ssid, ssid.ns_ssidlen);
 
 	rval = ndis_set_info(sc, OID_802_11_SSID, &ssid, &len);
 
 	if (rval)
 		device_printf (sc->ndis_dev, "set ssid failed: %d\n", rval);
 
-	ic->ic_state = IEEE80211_S_ASSOC;
+	if (ic->ic_state == IEEE80211_S_AUTH)
+		ieee80211_new_state(ic, IEEE80211_S_ASSOC, 0);
 
 	return;
 }
@@ -2469,6 +2588,9 @@ ndis_media_status(struct ifnet *ifp, struct ifmediareq *imr)
         case IEEE80211_M_MONITOR:
                 imr->ifm_active |= IFM_IEEE80211_MONITOR;
                 break;
+	case IEEE80211_M_WDS:
+		printf("WARNING: WDS operation mode not supported by NDIS\n");
+		break;
         }
         switch (ic->ic_curmode) {
         case IEEE80211_MODE_11A:
@@ -2548,7 +2670,7 @@ ndis_getstate_80211(sc)
 	struct ieee80211com	*ic;
 	ndis_wlan_bssid_ex	*bs;
 	int			rval, len, i = 0;
-	int			chan;
+	int			chanflag;
 	uint32_t		arg;
 	struct ifnet		*ifp;
 
@@ -2558,38 +2680,15 @@ ndis_getstate_80211(sc)
 	if (!NDIS_INITIALIZED(sc))
 		return;
 
-	if (sc->ndis_link)
-		ic->ic_state = IEEE80211_S_RUN;
-	else
-		ic->ic_state = IEEE80211_S_ASSOC;
-
-
-	/*
-	 * If we're associated, retrieve info on the current bssid.
-	 */
-	if ((rval = ndis_get_assoc(sc, &bs)) == 0) {
-		switch(bs->nwbx_nettype) {
-		case NDIS_80211_NETTYPE_11FH:
-		case NDIS_80211_NETTYPE_11DS:
-			ic->ic_curmode = IEEE80211_MODE_11B;
-			break;
-		case NDIS_80211_NETTYPE_11OFDM5:
-			ic->ic_curmode = IEEE80211_MODE_11A;
-			break;
-		case NDIS_80211_NETTYPE_11OFDM24:
-			ic->ic_curmode = IEEE80211_MODE_11G;
-			break;
-		default:
-			device_printf(sc->ndis_dev,
-			    "unknown nettype %d\n", arg);
-			break;
-		}
-		IEEE80211_ADDR_COPY(ic->ic_bss->ni_bssid, bs->nwbx_macaddr);
-	} else
+	if ((rval = ndis_get_assoc(sc, &bs)) != 0)
 		return;
 
-	/* Get SSID from current association info. */
+	/* We're associated, retrieve info on the current bssid. */
+	ic->ic_curmode = ndis_nettype_mode(bs->nwbx_nettype);
+	chanflag = ndis_nettype_chan(bs->nwbx_nettype);
+	IEEE80211_ADDR_COPY(ic->ic_bss->ni_bssid, bs->nwbx_macaddr);
 
+	/* Get SSID from current association info. */
 	bcopy(bs->nwbx_ssid.ns_ssid, ic->ic_bss->ni_essid,
 	    bs->nwbx_ssid.ns_ssidlen);
 	ic->ic_bss->ni_esslen = bs->nwbx_ssid.ns_ssidlen;
@@ -2600,7 +2699,7 @@ ndis_getstate_80211(sc)
 		device_printf (sc->ndis_dev, "get link speed failed: %d\n",
 		    rval);
 
-	if (ic->ic_modecaps & (1<<IEEE80211_MODE_11B)) {
+	if (isset(ic->ic_modecaps, IEEE80211_MODE_11B)) {
 		ic->ic_bss->ni_rates = ic->ic_sup_rates[IEEE80211_MODE_11B];
 		for (i = 0; i < ic->ic_bss->ni_rates.rs_nrates; i++) {
 			if ((ic->ic_bss->ni_rates.rs_rates[i] &
@@ -2610,7 +2709,7 @@ ndis_getstate_80211(sc)
 	}
 
 	if (i == ic->ic_bss->ni_rates.rs_nrates &&
-	    ic->ic_modecaps & (1<<IEEE80211_MODE_11G)) {
+	    isset(ic->ic_modecaps, IEEE80211_MODE_11G)) {
 		ic->ic_bss->ni_rates = ic->ic_sup_rates[IEEE80211_MODE_11G];
 		for (i = 0; i < ic->ic_bss->ni_rates.rs_nrates; i++) {
 			if ((ic->ic_bss->ni_rates.rs_rates[i] &
@@ -2642,22 +2741,12 @@ ndis_getstate_80211(sc)
 	 * Use the current association information to reflect
 	 * what channel we're on.
 	 */
-
-	chan = ieee80211_mhz2ieee(bs->nwbx_config.nc_dsconfig / 1000, 0);
-	if (chan < 0 || chan >= IEEE80211_CHAN_MAX) {
-		if (ifp->if_flags & IFF_DEBUG)
-			device_printf(sc->ndis_dev, "current channel "
-			    "(%uMHz) out of bounds\n", 
-			    bs->nwbx_config.nc_dsconfig / 1000);
-		chan = 1;
-	}
-
-	ic->ic_bss->ni_chan = &ic->ic_channels[chan];
-	ic->ic_des_chan = &ic->ic_channels[chan];
-	ic->ic_ibss_chan = &ic->ic_channels[chan];
-#if __FreeBSD_version >= 600007
-	ic->ic_curchan = &ic->ic_channels[chan];
-#endif
+	ic->ic_curchan = ieee80211_find_channel(ic,
+	    bs->nwbx_config.nc_dsconfig / 1000, chanflag);
+	if (ic->ic_curchan == NULL)
+		ic->ic_curchan = &ic->ic_channels[0];
+	ic->ic_bss->ni_chan = ic->ic_curchan;
+	ic->ic_bsschan = ic->ic_curchan;
 
 	free(bs, M_TEMP);
 
@@ -2816,7 +2905,7 @@ ndis_ioctl(ifp, command, data)
 			error = ENOTTY;
 		break;
 	case SIOCGDRVSPEC:
-		if ((error = suser(curthread)))
+		if ((error = priv_check(curthread, PRIV_DRIVER)))
 			break;
 		error =  copyin(ifr->ifr_data, &oid, sizeof(oid));
 		if (error)
@@ -2845,7 +2934,7 @@ ndis_ioctl(ifp, command, data)
 		free(oidbuf, M_TEMP);
 		break;
 	case SIOCSDRVSPEC:
-		if ((error = suser(curthread)))
+		if ((error = priv_check(curthread, PRIV_DRIVER)))
 			break;
 		error =  copyin(ifr->ifr_data, &oid, sizeof(oid));
 		if (error)
@@ -2874,7 +2963,7 @@ ndis_ioctl(ifp, command, data)
 		free(oidbuf, M_TEMP);
 		break;
 	case SIOCGPRIVATE_0:
-		if ((error = suser(curthread)))
+		if ((error = priv_check(curthread, PRIV_DRIVER)))
 			break;
 		NDIS_LOCK(sc);
 		if (sc->ndis_evt[sc->ndis_evtcidx].ne_sts == 0) {
@@ -2914,6 +3003,7 @@ ndis_ioctl(ifp, command, data)
 		NDIS_EVTINC(sc->ndis_evtcidx);
 		NDIS_UNLOCK(sc);
 		break;
+#if __FreeBSD_version < 700046
 	case SIOCGIFGENERIC:
 	case SIOCSIFGENERIC:
 		if (sc->ndis_80211 && NDIS_INITIALIZED(sc)) {
@@ -2925,6 +3015,7 @@ ndis_ioctl(ifp, command, data)
 			error = ENOTTY;
 		if (error != ENOTTY)
 			break;
+#endif
 	default:
 do_80211:
 		sc->ndis_skip = 1;
@@ -2949,6 +3040,7 @@ do_80211:
 	return(error);
 }
 
+#if __FreeBSD_version < 700046
 static int
 ndis_wi_ioctl_get(ifp, command, data)
 	struct ifnet		*ifp;
@@ -3042,7 +3134,7 @@ ndis_wi_ioctl_set(ifp, command, data)
 	uint32_t		foo;
 	int			error, len;
 
-	error = suser(curthread);
+	error = priv_check(curthread, PRIV_DRIVER);
 	if (error)
 		return (error);
 
@@ -3066,19 +3158,13 @@ ndis_wi_ioctl_set(ifp, command, data)
 	}
 	return (error);
 }
+#endif
 
 static int
 ndis_80211_ioctl_get(struct ifnet *ifp, u_long command, caddr_t data)
 {
 	struct ndis_softc	*sc;
 	struct ieee80211req	*ireq;
-#ifdef RTM_IEEE80211_SCAN
-	ndis_80211_bssid_list_ex *bl;
-	ndis_wlan_bssid_ex	*wb;
-	struct ieee80211req_scan_result	*sr, *bsr;
-	int			i, j;
-	char			*cp;
-#endif
 	int			error, len;
 	uint16_t		nodename_u[IEEE80211_NWID_LEN + 1];
 	unicode_string		us;
@@ -3092,89 +3178,6 @@ ndis_80211_ioctl_get(struct ifnet *ifp, u_long command, caddr_t data)
 	case IEEE80211_IOC_MLME:
 		error = 0;
 		break;
-#endif
-#ifdef RTM_IEEE80211_SCAN
-	case IEEE80211_IOC_SCAN_RESULTS:
-		len = 0;
-		error = ndis_get_info(sc, OID_802_11_BSSID_LIST, NULL, &len);
-		if (error != ENOSPC)
-			len = 65536;	
-		bl = malloc(len, M_DEVBUF, M_NOWAIT | M_ZERO);
-		error = ndis_get_info(sc, OID_802_11_BSSID_LIST, bl, &len);
-		if (error) {
-			free(bl, M_DEVBUF);
-			break;
-		}
-		sr = bsr = malloc(ireq->i_len, M_DEVBUF, M_NOWAIT | M_ZERO);
-		wb = &bl->nblx_bssid[0];
-		len = 0;
-		for (i = 0; i < bl->nblx_items; i++) {
-			/*
-			 * Check if we have enough space left for this ap.
-			 * Note that the size of the SSID list structure
-		 	 * can vary: the extended structure with info
-			 * elements is only supported by NDIS 5.1 drivers.
-			 */
-			if (wb->nwbx_len > sizeof(ndis_wlan_bssid)) {
-				j = roundup(sizeof(*sr) +
-				    wb->nwbx_ssid.ns_ssidlen +
-				    wb->nwbx_ielen -
-				    sizeof(struct ndis_80211_fixed_ies),
-				    sizeof(uint32_t));
-			} else {
-				j = roundup(sizeof(*sr) +
-				    wb->nwbx_ssid.ns_ssidlen +
-				    sizeof(struct ndis_80211_fixed_ies),
-				    sizeof(uint32_t));
-			}
-			if (len + j > ireq->i_len)
-				break;
-			bcopy(&wb->nwbx_macaddr, &sr->isr_bssid,
-			    sizeof(sr->isr_bssid));
-			if (wb->nwbx_privacy)
-				sr->isr_capinfo |= IEEE80211_CAPINFO_PRIVACY;
-			sr->isr_rssi = wb->nwbx_rssi + 200;
-			sr->isr_freq = wb->nwbx_config.nc_dsconfig / 1000;
-			sr->isr_intval = wb->nwbx_config.nc_beaconperiod;
-			switch (wb->nwbx_netinfra) {
-			case NDIS_80211_NET_INFRA_IBSS:
-				sr->isr_capinfo |= IEEE80211_CAPINFO_IBSS;
-				break;
-			case NDIS_80211_NET_INFRA_BSS:
-				sr->isr_capinfo |= IEEE80211_CAPINFO_ESS;
-				break;
-			}
-			for (j = 0; j < sizeof(sr->isr_rates); j++) {
-				/* XXX - check units */
-				if (wb->nwbx_supportedrates[j] == 0)
-					break;
-				sr->isr_rates[j] =
-				    wb->nwbx_supportedrates[j] & 0x7f;
-			}
-			sr->isr_nrates = j;
-			sr->isr_ssid_len = wb->nwbx_ssid.ns_ssidlen;
-			cp = (char *)sr + sizeof(*sr);
-			bcopy(&wb->nwbx_ssid.ns_ssid, cp, sr->isr_ssid_len);
-			cp += sr->isr_ssid_len;
-			if (wb->nwbx_len > sizeof(ndis_wlan_bssid)) {
-				sr->isr_ie_len = wb->nwbx_ielen
-				    - sizeof(struct ndis_80211_fixed_ies);
-				bcopy((char *)wb->nwbx_ies +
-				    sizeof(struct ndis_80211_fixed_ies),
-				    cp, sr->isr_ie_len);
-			}
-			sr->isr_len = roundup(sizeof(*sr) + sr->isr_ssid_len
-			    + sr->isr_ie_len, sizeof(uint32_t));
-			len += sr->isr_len;
-			sr = (struct ieee80211req_scan_result *)((char *)sr +
-			    sr->isr_len);
-			wb = (ndis_wlan_bssid_ex *)((char *)wb + wb->nwbx_len);
-		}
-		ireq->i_len = len;
-		error = copyout(bsr, ireq->i_data, len);
-		free(bl, M_DEVBUF);
-		free(bsr, M_DEVBUF);
-		break;
 #endif
 	case IEEE80211_IOC_STATIONNAME:
 		error = ndis_get_info(sc, OID_GEN_MACHINE_NAME,
@@ -3332,25 +3335,14 @@ ndis_80211_ioctl_set(struct ifnet *ifp, u_long command, caddr_t data)
 	ireq = (struct ieee80211req *) data;
 
 	switch (ireq->i_type) {
-#ifdef IEEE80211_IOC_MLME
-	case IEEE80211_IOC_MLME:
-	case IEEE80211_IOC_ROAMING:
+#ifdef IEEE80211_IOC_COUNTERMEASURES
 	case IEEE80211_IOC_COUNTERMEASURES:
 	case IEEE80211_IOC_DROPUNENCRYPTED:
 		error = 0;
 		break;
-#endif
-#ifdef RTM_IEEE80211_SCAN
-	case IEEE80211_IOC_SCAN_REQ:
-		len = 0;
-		error = ndis_set_info(sc, OID_802_11_BSSID_LIST_SCAN,
-		    NULL, &len);
-		tsleep(&error, PPAUSE|PCATCH, "ssidscan", hz * 2);
-		rt_ieee80211msg(ifp, RTM_IEEE80211_SCAN, NULL, 0);
-		break;
 #endif
 	case IEEE80211_IOC_STATIONNAME:
-		error = suser(curthread);
+		error = priv_check(curthread, PRIV_NET80211_MANAGE);
 		if (error)
 			break;
 		if (ireq->i_val != 0 ||
@@ -3433,7 +3425,7 @@ ndis_stop(sc)
 
 	ic = &sc->ic;
 	if (sc->ndis_80211)
-		ic->ic_state = IEEE80211_S_INIT;
+		ieee80211_new_state(ic, IEEE80211_S_INIT, -1);
 
 	ifp = sc->ifp;
 	untimeout(ndis_tick, sc, sc->ndis_stat_ch);
@@ -3475,3 +3467,222 @@ ndis_shutdown(dev)
 
 	return;
 }
+
+static int
+ndis_newstate(struct ieee80211com *ic, enum ieee80211_state nstate, int arg)
+{
+	struct ifnet *ifp = ic->ic_ifp;
+	struct ndis_softc *sc = ifp->if_softc;
+	 enum ieee80211_state ostate;
+
+	DPRINTF(("%s: %s -> %s\n", __func__,
+		ieee80211_state_name[ic->ic_state],
+		ieee80211_state_name[nstate]));
+
+	ostate = ic->ic_state;
+	ic->ic_state = nstate;
+
+	switch (nstate) {
+	/* pass on to net80211 */
+	case IEEE80211_S_INIT:
+	case IEEE80211_S_SCAN:
+		return (sc->ndis_newstate(ic, nstate, arg));
+
+	case IEEE80211_S_ASSOC:
+		if (ostate != IEEE80211_S_AUTH)
+			ndis_setstate_80211(sc);
+		break;
+
+	case IEEE80211_S_AUTH:
+		ndis_setstate_80211(sc);
+		break;
+
+	default:
+		break;
+	}
+	return (0);
+}
+
+static void
+ndis_scan(void *arg, int npending)
+{
+	struct ndis_softc *sc = arg;
+	struct ieee80211com *ic = (void *)&sc->ic;
+	struct ieee80211_scan_state *ss = ic->ic_scan;
+	ndis_80211_ssid ssid;
+	int error, len;
+
+	if (!NDIS_INITIALIZED(sc)) {
+		DPRINTF(("%s: scan aborted\n", __func__));
+		ieee80211_cancel_scan(ic);
+		return;
+	}
+
+	if (ss->ss_nssid != 0) {
+		/* Perform a directed scan */
+		len = sizeof(ssid);
+		bzero((char *)&ssid, len);
+		ssid.ns_ssidlen = ss->ss_ssid[0].len;
+		bcopy(ss->ss_ssid[0].ssid, ssid.ns_ssid, ssid.ns_ssidlen);
+
+		error = ndis_set_info(sc, OID_802_11_SSID, &ssid, &len);
+		if (error)
+			DPRINTF(("%s: set ESSID failed\n", __func__));
+	}
+
+	len = 0;
+	error = ndis_set_info(sc, OID_802_11_BSSID_LIST_SCAN,
+	    NULL, &len);
+	if (error) {
+		DPRINTF(("%s: scan command failed\n", __func__));
+		ieee80211_cancel_scan(ic);
+		return;
+	}
+
+	tsleep(&error, PWAIT, "ssidscan", hz * 3);
+	if (!NDIS_INITIALIZED(sc))
+		/* The interface was downed while we were sleeping */
+		return;
+
+	ndis_scan_results(sc);
+	ieee80211_scan_done(ic);
+}
+
+static void
+ndis_scan_results(struct ndis_softc *sc)
+{
+	struct ieee80211com *ic = (void *)&sc->ic;
+	ndis_80211_bssid_list_ex *bl;
+	ndis_wlan_bssid_ex	*wb;
+	struct ieee80211_scanparams sp;
+	struct ieee80211_frame wh;
+	int i, j;
+	int error, len, rssi, noise, freq, chanflag;
+	static long rstamp;
+	uint8_t ssid[2+IEEE80211_NWID_LEN];
+	uint8_t rates[2+IEEE80211_RATE_MAXSIZE];
+	uint8_t *frm, *efrm;
+
+	len = 0;
+	noise = -96;
+	error = ndis_get_info(sc, OID_802_11_BSSID_LIST, NULL, &len);
+	if (error != ENOSPC)
+		len = 65536;
+	bl = malloc(len, M_DEVBUF, M_NOWAIT | M_ZERO);
+	error = ndis_get_info(sc, OID_802_11_BSSID_LIST, bl, &len);
+	if (error) {
+		DPRINTF(("%s: failed to read\n", __func__));
+		free(bl, M_DEVBUF);
+		return;;
+	}
+
+	DPRINTF(("%s: %d results\n", __func__, bl->nblx_items));
+	rstamp++;
+	wb = &bl->nblx_bssid[0];
+	for (i = 0; i < bl->nblx_items; i++) {
+		memset(&sp, 0, sizeof(sp));
+
+		memcpy(wh.i_addr2, wb->nwbx_macaddr, sizeof(wh.i_addr2));
+		memcpy(wh.i_addr3, wb->nwbx_macaddr, sizeof(wh.i_addr3));
+		rssi = 100 * (wb->nwbx_rssi - noise) / (-32 - noise);
+		rssi = max(0, min(rssi, 100));	/* limit 0 <= rssi <= 100 */
+		if (wb->nwbx_privacy)
+			sp.capinfo |= IEEE80211_CAPINFO_PRIVACY;
+		sp.bintval = wb->nwbx_config.nc_beaconperiod;
+		switch (wb->nwbx_netinfra) {
+			case NDIS_80211_NET_INFRA_IBSS:
+				sp.capinfo |= IEEE80211_CAPINFO_IBSS;
+				break;
+			case NDIS_80211_NET_INFRA_BSS:
+				sp.capinfo |= IEEE80211_CAPINFO_ESS;
+				break;
+		}
+		sp.rates = &rates[0];
+		for (j = 0; j < IEEE80211_RATE_MAXSIZE; j++) {
+			/* XXX - check units */
+			if (wb->nwbx_supportedrates[j] == 0)
+				break;
+			rates[2 + j] =
+			wb->nwbx_supportedrates[j] & 0x7f;
+		}
+		rates[1] = j;
+		sp.ssid = (uint8_t *)&ssid[0];
+		memcpy(sp.ssid + 2, &wb->nwbx_ssid.ns_ssid,
+		    wb->nwbx_ssid.ns_ssidlen);
+		sp.ssid[1] = wb->nwbx_ssid.ns_ssidlen;
+
+		chanflag = ndis_nettype_chan(wb->nwbx_nettype);
+		freq = wb->nwbx_config.nc_dsconfig / 1000;
+		sp.bchan = ieee80211_mhz2ieee(freq, chanflag);
+		sp.curchan = ieee80211_find_channel(ic, freq, chanflag);
+		if (sp.curchan == NULL)
+			sp.curchan = &ic->ic_channels[0];
+
+		/* Process extended info from AP */
+		if (wb->nwbx_len > sizeof(ndis_wlan_bssid)) {
+			frm = (uint8_t *)&wb->nwbx_ies;
+			efrm = frm + wb->nwbx_ielen;
+			if (efrm - frm < 12)
+				goto done;
+			sp.tstamp = frm;
+			frm += 8;
+			sp.bintval = le16toh(*(uint16_t *)frm);
+			frm += 2;
+			sp.capinfo = le16toh(*(uint16_t *)frm);
+			frm += 2;
+
+			/* Grab variable length ies */
+			while (efrm - frm > 1) {
+				if (efrm - frm < frm[1] + 2)
+					break;
+				switch (*frm) {
+				case IEEE80211_ELEMID_RSN:
+					sp.rsn = frm;
+					break;
+				}
+				frm += frm[1] + 2;
+			}
+		}
+done:
+		DPRINTF(("scan: bssid %s chan %dMHz (%d/%d) rssi %d\n",
+		    ether_sprintf(wb->nwbx_macaddr), freq, sp.bchan, chanflag,
+		    rssi));
+		ieee80211_add_scan(ic, &sp, &wh, 0, rssi, noise, rstamp);
+		wb = (ndis_wlan_bssid_ex *)((char *)wb + wb->nwbx_len);
+	}
+	free(bl, M_DEVBUF);
+}
+
+static void
+ndis_scan_start(struct ieee80211com *ic)
+{
+	struct ifnet *ifp = ic->ic_ifp;
+	struct ndis_softc *sc = ifp->if_softc;
+
+	taskqueue_enqueue(sc->ndis_tq, &sc->ndis_scantask);
+}
+
+static void
+ndis_set_channel(struct ieee80211com *ic)
+{
+	/* ignore */
+}
+
+static void
+ndis_scan_curchan(struct ieee80211com *ic, unsigned long maxdwell)
+{
+	/* ignore */
+}
+
+static void
+ndis_scan_mindwell(struct ieee80211com *ic)
+{
+	/* NB: don't try to abort scan; wait for firmware to finish */
+}
+
+static void
+ndis_scan_end(struct ieee80211com *ic)
+{
+	/* ignore */
+}
+
