@@ -25,10 +25,11 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/iicbus/iiconf.c,v 1.14.10.1 2007/02/26 23:17:51 cognet Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/iicbus/iiconf.c,v 1.17 2007/03/23 23:03:54 imp Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/malloc.h>
 #include <sys/module.h>
 #include <sys/bus.h>
 
@@ -234,7 +235,7 @@ iicbus_stop(device_t bus)
  * iicbus_start() call
  */
 int
-iicbus_write(device_t bus, char *buf, int len, int *sent, int timeout)
+iicbus_write(device_t bus, const char *buf, int len, int *sent, int timeout)
 {
 	struct iicbus_softc *sc = (struct iicbus_softc *)device_get_softc(bus);
 	
@@ -356,13 +357,16 @@ iicbus_transfer(device_t bus, struct iic_msg *msgs, uint32_t nmsgs)
  * buffer addresses.
  */
 int
-iicbus_transfer_gen(device_t bus, struct iic_msg *msgs, uint32_t nmsgs)
+iicbus_transfer_gen(device_t dev, struct iic_msg *msgs, uint32_t nmsgs)
 {
-	int i, error, max, lenread, lenwrote;
+	int i, error, lenread, lenwrote, nkid;
+	device_t *children, bus;
 
-	for (i = 0, max = 0; i < nmsgs; i++)
-		if (max < msgs[i].len)
-			max = msgs[i].len;
+	device_get_children(dev, &children, &nkid);
+	if (nkid != 1)
+		return EIO;
+	bus = children[0];
+	free(children, M_TEMP);
 	for (i = 0, error = 0; i < nmsgs && error == 0; i++) {
 		if (msgs[i].flags & IIC_M_RD)
 			error = iicbus_block_read(bus, msgs[i].slave,
