@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/iicbus/iicsmb.c,v 1.12.10.1 2006/09/22 19:19:16 jhb Exp $
+ * $FreeBSD: src/sys/dev/iicbus/iicsmb.c,v 1.14 2006/09/11 20:52:41 jhb Exp $
  *
  */
 
@@ -153,8 +153,6 @@ iicsmb_attach(device_t dev)
 {
 	struct iicsmb_softc *sc = (struct iicsmb_softc *)device_get_softc(dev);
 
-	bzero(sc, sizeof(*sc));
-
 	sc->smbus = device_add_child(dev, "smbus", -1);
 
 	/* probe and attach the smbus */
