@@ -31,7 +31,7 @@
 
 #ident "$Id: iir_pci.c 1.2 2003/08/26 12:29:55 achim Exp $"
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/iir/iir_pci.c,v 1.15.2.2 2006/12/14 02:57:06 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/iir/iir_pci.c,v 1.20 2007/06/17 05:55:50 scottl Exp $");
 
 /*
  *  iir_pci.c:  PCI Bus Attachment for Intel Integrated RAID Controller driver
@@ -56,7 +56,6 @@ __FBSDID("$FreeBSD: src/sys/dev/iir/iir_pci.c,v 1.15.2.2 2006/12/14 02:57:06 mja
 
 #include <machine/bus.h> 
 #include <machine/resource.h>
-#include <machine/clock.h>
 #include <sys/rman.h>
 
 #include <dev/pci/pcireg.h>
@@ -210,7 +209,7 @@ iir_pci_attach(device_t dev)
     }
 
     gdt = device_get_softc(dev);
-    bzero(gdt, sizeof(struct gdt_softc));
+    gdt->sc_devnode = dev;
     gdt->sc_init_level = 0;
     gdt->sc_dpmemt = rman_get_bustag(io);
     gdt->sc_dpmemh = rman_get_bushandle(io);
@@ -345,7 +344,7 @@ iir_pci_attach(device_t dev)
 
     /* associate interrupt handler */
     if (bus_setup_intr( dev, irq, INTR_TYPE_CAM, 
-                        iir_intr, gdt, &ih )) {
+                        NULL, iir_intr, gdt, &ih )) {
         device_printf(dev, "Unable to register interrupt handler\n");
         error = ENXIO;
         goto err;
