@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ipmi/ipmi.c,v 1.3.2.5 2007/02/20 22:56:27 n_hibma Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ipmi/ipmi.c,v 1.10 2007/03/27 21:03:36 n_hibma Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -843,7 +843,7 @@ ipmi_attach(device_t dev)
 
 	if (sc->ipmi_irq_res != NULL && sc->ipmi_intr != NULL) {
 		error = bus_setup_intr(dev, sc->ipmi_irq_res, INTR_TYPE_MISC,
-		    sc->ipmi_intr, sc, &sc->ipmi_irq);
+		    NULL, sc->ipmi_intr, sc, &sc->ipmi_irq);
 		if (error) {
 			device_printf(dev, "can't set up interrupt\n");
 			return (error);
