@@ -28,7 +28,7 @@
  * Platform (FreeBSD) dependent common attachment code for Qlogic adapters.
  */
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/isp/isp_freebsd.c,v 1.105.2.11 2007/08/27 16:56:28 jkim Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/isp/isp_freebsd.c,v 1.146 2007/08/23 15:57:13 jkim Exp $");
 #include <dev/isp/isp_freebsd.h>
 #include <sys/unistd.h>
 #include <sys/kthread.h>
@@ -146,14 +146,17 @@ isp_attach(ispsoftc_t *isp)
 
 	isp->isp_osinfo.ehook.ich_func = isp_intr_enable;
 	isp->isp_osinfo.ehook.ich_arg = isp;
+	ISP_UNLOCK(isp);
 	if (config_intrhook_establish(&isp->isp_osinfo.ehook) != 0) {
+		ISP_LOCK(isp);
 		cam_sim_free(sim, TRUE);
 		isp_prt(isp, ISP_LOGERR,
 		    "could not establish interrupt enable hook");
 		return;
 	}
+	ISP_LOCK(isp);
 
-	if (xpt_bus_register(sim, primary) != CAM_SUCCESS) {
+	if (xpt_bus_register(sim, isp->isp_dev, primary) != CAM_SUCCESS) {
 		cam_sim_free(sim, TRUE);
 		return;
 	}
@@ -188,7 +191,8 @@ isp_attach(ispsoftc_t *isp)
 			config_intrhook_disestablish(&isp->isp_osinfo.ehook);
 			return;
 		}
-		if (xpt_bus_register(sim, secondary) != CAM_SUCCESS) {
+		if (xpt_bus_register(sim, isp->isp_dev, secondary) !=
+		    CAM_SUCCESS) {
 			xpt_bus_deregister(cam_sim_path(isp->isp_sim));
 			xpt_free_path(isp->isp_path);
 			cam_sim_free(sim, TRUE);
@@ -304,11 +308,6 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 {
 	ispsoftc_t *isp;
 	int nr, retval = ENOTTY;
-#if __FreeBSD_version < 500000  
-	int s = splcam();
-#else
-	GIANT_REQUIRED;
-#endif
 
 	isp = isplist;
 	while (isp) {
@@ -318,9 +317,6 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		isp = isp->isp_osinfo.next;
 	}
 	if (isp == NULL) {
-#if __FreeBSD_version < 500000  
-		splx(s);
-#endif
 		return (ENXIO);
 	}
 	
@@ -351,10 +347,12 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		break;
 	case ISP_FORCE_CRASH_DUMP:
 		if (IS_FC(isp)) {
+			ISP_LOCK(isp);
 			isp_freeze_loopdown(isp,
 			    "ispioctl(ISP_FORCE_CRASH_DUMP)");
 			isp_fw_dump(isp);
 			isp_reinit(isp);
+			ISP_UNLOCK(isp);
 			retval = 0;
 		}
 		break;
@@ -377,37 +375,35 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 			retval = EINVAL;
 			break;
 		}
-		/*
-		 * XXX: Current
-		 */
-		if (nr == ISP_ROLE_BOTH) {
-			isp_prt(isp, ISP_LOGERR, "dual roles not supported");
-			retval = EINVAL;
-			break;
-		}
 		*(int *)addr = isp->isp_role;
 		isp->isp_role = nr;
 		/* FALLTHROUGH */
 	case ISP_RESETHBA:
+		ISP_LOCK(isp);
 		isp_reinit(isp);
+		ISP_UNLOCK(isp);
 		retval = 0;
 		break;
 	case ISP_RESCAN:
 		if (IS_FC(isp)) {
+			ISP_LOCK(isp);
 			if (isp_fc_runstate(isp, 5 * 1000000)) {
 				retval = EIO;
 			} else {
 				retval = 0;
 			}
+			ISP_UNLOCK(isp);
 		}
 		break;
 	case ISP_FC_LIP:
 		if (IS_FC(isp)) {
+			ISP_LOCK(isp);
 			if (isp_control(isp, ISPCTL_SEND_LIP, 0)) {
 				retval = EIO;
 			} else {
 				retval = 0;
 			}
+			ISP_UNLOCK(isp);
 		}
 		break;
 	case ISP_FC_GETDINFO:
@@ -418,7 +414,7 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		if (IS_SCSI(isp)) {
 			break;
 		}
-		if (ifc->loopid < 0 || ifc->loopid >= MAX_FC_TARG) {
+		if (ifc->loopid >= MAX_FC_TARG) {
 			retval = EINVAL;
 			break;
 		}
@@ -443,6 +439,7 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		sp->isp_stat_version = ISP_STATS_VERSION;
 		sp->isp_type = isp->isp_type;
 		sp->isp_revision = isp->isp_revision;
+		ISP_LOCK(isp);
 		sp->isp_stats[ISP_INTCNT] = isp->isp_intcnt;
 		sp->isp_stats[ISP_INTBOGUS] = isp->isp_intbogus;
 		sp->isp_stats[ISP_INTMBOXC] = isp->isp_intmboxc;
@@ -451,10 +448,12 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		sp->isp_stats[ISP_FPHCCMCPLT] = isp->isp_fphccmplt;
 		sp->isp_stats[ISP_RSCCHIWAT] = isp->isp_rscchiwater;
 		sp->isp_stats[ISP_FPCCHIWAT] = isp->isp_fpcchiwater;
+		ISP_UNLOCK(isp);
 		retval = 0;
 		break;
 	}
 	case ISP_CLR_STATS:
+		ISP_LOCK(isp);
 		isp->isp_intcnt = 0;
 		isp->isp_intbogus = 0;
 		isp->isp_intmboxc = 0;
@@ -463,6 +462,7 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		isp->isp_fphccmplt = 0;
 		isp->isp_rscchiwater = 0;
 		isp->isp_fpcchiwater = 0;
+		ISP_UNLOCK(isp);
 		retval = 0;
 		break;
 	case ISP_FC_GETHINFO:
@@ -486,92 +486,6 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 		retval = 0;
 		break;
 	}
-	case ISP_GET_FC_PARAM:
-	{
-		struct isp_fc_param *f = (struct isp_fc_param *) addr;
-
-		if (IS_SCSI(isp)) {
-			break;
-		}
-		f->parameter = 0;
-		if (strcmp(f->param_name, "framelength") == 0) {
-			f->parameter = FCPARAM(isp)->isp_maxfrmlen;
-			retval = 0;
-			break;
-		}
-		if (strcmp(f->param_name, "exec_throttle") == 0) {
-			f->parameter = FCPARAM(isp)->isp_execthrottle;
-			retval = 0;
-			break;
-		}
-		if (strcmp(f->param_name, "fullduplex") == 0) {
-			if (FCPARAM(isp)->isp_fwoptions & ICBOPT_FULL_DUPLEX)
-				f->parameter = 1;
-			retval = 0;
-			break;
-		}
-		if (strcmp(f->param_name, "loopid") == 0) {
-			f->parameter = FCPARAM(isp)->isp_loopid;
-			retval = 0;
-			break;
-		}
-		retval = EINVAL;
-		break;
-	}
-	case ISP_SET_FC_PARAM:
-	{
-		struct isp_fc_param *f = (struct isp_fc_param *) addr;
-		uint32_t param = f->parameter;
-
-		if (IS_SCSI(isp)) {
-			break;
-		}
-		f->parameter = 0;
-		if (strcmp(f->param_name, "framelength") == 0) {
-			if (param != 512 && param != 1024 && param != 1024) {
-				retval = EINVAL;
-				break;
-			}
-			FCPARAM(isp)->isp_maxfrmlen = param;
-			retval = 0;
-			break;
-		}
-		if (strcmp(f->param_name, "exec_throttle") == 0) {
-			if (param < 16 || param > 255) {
-				retval = EINVAL;
-				break;
-			}
-			FCPARAM(isp)->isp_execthrottle = param;
-			retval = 0;
-			break;
-		}
-		if (strcmp(f->param_name, "fullduplex") == 0) {
-			if (param != 0 && param != 1) {
-				retval = EINVAL;
-				break;
-			}
-			if (param) {
-				FCPARAM(isp)->isp_fwoptions |=
-				    ICBOPT_FULL_DUPLEX;
-			} else {
-				FCPARAM(isp)->isp_fwoptions &=
-				    ~ICBOPT_FULL_DUPLEX;
-			}
-			retval = 0;
-			break;
-		}
-		if (strcmp(f->param_name, "loopid") == 0) {
-			if (param < 0 || param > 125) {
-				retval = EINVAL;
-				break;
-			}
-			FCPARAM(isp)->isp_loopid = param;
-			retval = 0;
-			break;
-		}
-		retval = EINVAL;
-		break;
-	}
 	case ISP_TSK_MGMT:
 	{
 		int needmarker;
@@ -626,7 +540,9 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 			if (needmarker) {
 				isp->isp_sendmarker |= 1;
 			}
+			ISP_LOCK(isp);
 			retval = isp_control(isp, ISPCTL_RUN_MBOXCMD, &mbs);
+			ISP_UNLOCK(isp);
 			if (retval)
 				retval = EIO;
 		}
@@ -635,9 +551,6 @@ ispioctl(_DEV dev, u_long c, caddr_t addr, int flags, _IOP *td)
 	default:
 		break;
 	}
-#if __FreeBSD_version < 500000  
-	splx(s);
-#endif
 	return (retval);
 }
 
@@ -813,7 +726,7 @@ create_lun_state(ispsoftc_t *isp, int bus,
 	tstate_t *tptr, *new;
 
 	lun = xpt_path_lun_id(path);
-	if (lun < 0) {
+	if (lun >= ISP_MAX_LUNS(isp)) {
 		return (CAM_LUN_INVALID);
 	}
 	if (is_lun_enabled(isp, bus, lun)) {
@@ -889,7 +802,7 @@ static int
 isp_en_lun(ispsoftc_t *isp, union ccb *ccb)
 {
 	struct ccb_en_lun *cel = &ccb->cel;
-	tstate_t *tptr;
+	tstate_t *tptr = NULL;
 	uint32_t seq;
 	int bus, cmd, av, wildcard, tm_on;
 	lun_id_t lun;
@@ -910,7 +823,7 @@ isp_en_lun(ispsoftc_t *isp, union ccb *ccb)
 	}
 
 	if ((lun != CAM_LUN_WILDCARD) &&
-	    (lun < 0 || lun >= (lun_id_t) isp->isp_maxluns)) {
+	    (lun >= (lun_id_t) isp->isp_maxluns)) {
 		ccb->ccb_h.status = CAM_LUN_INVALID;
 		return (-1);
 	}
