@@ -28,7 +28,7 @@
  * FreeBSD Version.
  */
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/isp/isp_pci.c,v 1.104.2.12 2007/05/11 06:29:00 mjacob Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/isp/isp_pci.c,v 1.148 2007/06/26 23:08:57 mjacob Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -1712,11 +1712,15 @@ isp_pci_mbxdma(ispsoftc_t *isp)
 
 	hlim = BUS_SPACE_MAXADDR;
 	if (IS_ULTRA2(isp) || IS_FC(isp) || IS_1240(isp)) {
-		slim = (bus_size_t) (1ULL << 32);
+		if (sizeof (bus_size_t) > 4) {
+			slim = (bus_size_t) (1ULL << 32);
+		} else {
+			slim = (bus_size_t) (1UL << 31);
+		}
 		llim = BUS_SPACE_MAXADDR;
 	} else {
 		llim = BUS_SPACE_MAXADDR_32BIT;
-		slim = (1 << 24);
+		slim = (1UL << 24);
 	}
 
 	len = isp->isp_maxcmds * sizeof (struct isp_pcmd);
@@ -2843,10 +2847,22 @@ mbxsync:
 		    (ispextreq_t *)qep);
 		break;
 	case RQSTYPE_T2RQS:
-		isp_put_request_t2(isp, (ispreqt2_t *) rq, (ispreqt2_t *) qep);
+		if (FCPARAM(isp)->isp_2klogin) {
+			isp_put_request_t2e(isp,
+			    (ispreqt2e_t *) rq, (ispreqt2e_t *) qep);
+		} else {
+			isp_put_request_t2(isp,
+			    (ispreqt2_t *) rq, (ispreqt2_t *) qep);
+		}
 		break;
-	case RQSTYPE_A64:
 	case RQSTYPE_T3RQS:
+		if (FCPARAM(isp)->isp_2klogin) {
+			isp_put_request_t3e(isp,
+			    (ispreqt3e_t *) rq, (ispreqt3e_t *) qep);
+			break;
+		}
+		/* FALLTHROUGH */
+	case RQSTYPE_A64:
 		isp_put_request_t3(isp, (ispreqt3_t *) rq, (ispreqt3_t *) qep);
 		break;
 	case RQSTYPE_T7RQS:
