@@ -1,4 +1,4 @@
-/* $FreeBSD: src/sys/dev/isp/ispvar.h,v 1.71.2.7 2007/04/01 17:15:33 mjacob Exp $ */
+/* $FreeBSD: src/sys/dev/isp/ispvar.h,v 1.85 2007/07/02 20:08:20 mjacob Exp $ */
 /*-
  *  Copyright (c) 1997-2007 by Matthew Jacob
  *  All rights reserved.
@@ -1015,6 +1015,9 @@ int isp_async(ispsoftc_t *, ispasync_t, void *);
  *	ISP_IOXGET_32(ispsoftc_t *, uint32_t *srcptr, uint32_t dstrval)
  *
  *	ISP_SWIZZLE_NVRAM_WORD(ispsoftc_t *, uint16_t *)
+ *	ISP_SWIZZLE_NVRAM_LONG(ispsoftc_t *, uint32_t *)
+ *	ISP_SWAP16(ispsoftc_t *, uint16_t srcval)
+ *	ISP_SWAP32(ispsoftc_t *, uint32_t srcval)
  */
 
 #endif	/* _ISPVAR_H */
