@@ -28,9 +28,10 @@
  * SUCH DAMAGE.
  *
  * $Id: kbdmux.c,v 1.4 2005/07/14 17:38:35 max Exp $
- * $FreeBSD: src/sys/dev/kbdmux/kbdmux.c,v 1.2.2.9 2007/07/19 17:12:02 emax Exp $
+ * $FreeBSD: src/sys/dev/kbdmux/kbdmux.c,v 1.15 2007/07/11 18:57:15 emax Exp $
  */
 
+#include "opt_compat.h"
 #include "opt_kbd.h"
 
 #include <sys/param.h>
@@ -943,7 +944,9 @@ kbdmux_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 	kbdmux_kbd_t	*k;
 	keyboard_info_t	*ki;
 	int		 error = 0, mode;
+#ifdef COMPAT_FREEBSD6
 	int		 ival;
+#endif
 
 	if (state == NULL)
 		return (ENXIO);
@@ -1052,10 +1055,12 @@ kbdmux_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		KBDMUX_UNLOCK(state);
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 7):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSKBMODE: /* set keyboard mode */
 		KBDMUX_LOCK(state);
 
@@ -1090,10 +1095,12 @@ kbdmux_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		KBDMUX_UNLOCK(state);
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 66):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSETLED: /* set keyboard LED */
 		KBDMUX_LOCK(state);
 
@@ -1119,10 +1126,12 @@ kbdmux_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		KBDMUX_UNLOCK(state);
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 20):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSKBSTATE: /* set lock key state */
 		KBDMUX_LOCK(state);
 
@@ -1144,11 +1153,13 @@ kbdmux_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		return (kbdmux_ioctl(kbd, KDSETLED, arg));
 		/* NOT REACHED */
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 67):
 		cmd = KDSETRAD;
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSETREPEAT: /* set keyboard repeat rate (new interface) */
 	case KDSETRAD: /* set keyboard repeat rate (old interface) */
 		KBDMUX_LOCK(state);
