@@ -25,7 +25,7 @@
  *
  *	from: NetBSD: mc146818var.h,v 1.3 2003/11/24 06:20:40 tsutsui Exp
  *
- * $FreeBSD: src/sys/dev/mc146818/mc146818var.h,v 1.3 2005/05/19 21:20:42 marius Exp $
+ * $FreeBSD: src/sys/dev/mc146818/mc146818var.h,v 1.4 2007/06/16 23:10:00 marius Exp $
  */
 
 struct mc146818_softc {
@@ -51,6 +51,10 @@ struct mc146818_softc {
 	void (*sc_setcent)(device_t, u_int);
 };
 
+/* Default read/write functions */
+u_int mc146818_def_read(device_t, u_int);
+void mc146818_def_write(device_t, u_int, u_int);
+
 /* Chip attach function */
 int mc146818_attach(device_t);
 
