@@ -6,7 +6,7 @@
  * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
  * ----------------------------------------------------------------------------
  *
- * $FreeBSD: src/sys/dev/md/md.c,v 1.153.2.8 2007/01/05 09:51:14 kib Exp $
+ * $FreeBSD: src/sys/dev/md/md.c,v 1.169 2007/06/05 00:00:51 jeff Exp $
  *
  */
 
@@ -96,8 +96,8 @@
 #define MD_NSECT (10000 * 2)
 #endif
 
-static MALLOC_DEFINE(M_MD, "MD disk", "Memory Disk");
-static MALLOC_DEFINE(M_MDSECT, "MD sectors", "Memory Disk Sectors");
+static MALLOC_DEFINE(M_MD, "md_disk", "Memory Disk");
+static MALLOC_DEFINE(M_MDSECT, "md_sectors", "Memory Disk Sectors");
 
 static int md_debug;
 SYSCTL_INT(_debug, OID_AUTO, mddebug, CTLFLAG_RW, &md_debug, 0, "");
@@ -122,6 +122,8 @@ static g_init_t g_md_init;
 static g_fini_t g_md_fini;
 static g_start_t g_md_start;
 static g_access_t g_md_access;
+static void g_md_dumpconf(struct sbuf *sb, const char *indent, struct g_geom *gp, 
+    struct g_consumer *cp __unused, struct g_provider *pp);
 
 static int	mdunits;
 static struct cdev *status_dev = 0;
@@ -142,6 +144,7 @@ struct g_class g_md_class = {
 	.fini = g_md_fini,
 	.start = g_md_start,
 	.access = g_md_access,
+	.dumpconf = g_md_dumpconf,
 };
 
 DECLARE_GEOM_CLASS(g_md_class, g_md);
@@ -403,6 +406,15 @@ mdstart_malloc(struct md_s *sc, struct bio *bp)
 	off_t secno, nsec, uc;
 	uintptr_t sp, osp;
 
+	switch (bp->bio_cmd) {
+	case BIO_READ:
+	case BIO_WRITE:
+	case BIO_DELETE:
+		break;
+	default:
+		return (EOPNOTSUPP);
+	}
+
 	nsec = bp->bio_length / sc->sectorsize;
 	secno = bp->bio_offset / sc->sectorsize;
 	dst = bp->bio_data;
@@ -488,6 +500,20 @@ mdstart_vnode(struct md_s *sc, struct bio *bp)
 	struct uio auio;
 	struct iovec aiov;
 	struct mount *mp;
+	struct vnode *vp;
+	struct thread *td;
+
+	switch (bp->bio_cmd) {
+	case BIO_READ:
+	case BIO_WRITE:
+	case BIO_FLUSH:
+		break;
+	default:
+		return (EOPNOTSUPP);
+	}
+
+	td = curthread;
+	vp = sc->vnode;
 
 	/*
 	 * VNODE I/O
@@ -497,6 +523,17 @@ mdstart_vnode(struct md_s *sc, struct bio *bp)
 	 * still valid.
 	 */
 
+	if (bp->bio_cmd == BIO_FLUSH) {
+		vfslocked = VFS_LOCK_GIANT(vp->v_mount);
+		(void) vn_start_write(vp, &mp, V_WAIT);
+		vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, td);
+		error = VOP_FSYNC(vp, MNT_WAIT, td);
+		VOP_UNLOCK(vp, 0, td);
+		vn_finished_write(mp);
+		VFS_UNLOCK_GIANT(vfslocked);
+		return (error);
+	}
+
 	bzero(&auio, sizeof(auio));
 
 	aiov.iov_base = bp->bio_data;
@@ -505,29 +542,29 @@ mdstart_vnode(struct md_s *sc, struct bio *bp)
 	auio.uio_iovcnt = 1;
 	auio.uio_offset = (vm_ooffset_t)bp->bio_offset;
 	auio.uio_segflg = UIO_SYSSPACE;
-	if(bp->bio_cmd == BIO_READ)
+	if (bp->bio_cmd == BIO_READ)
 		auio.uio_rw = UIO_READ;
-	else if(bp->bio_cmd == BIO_WRITE)
+	else if (bp->bio_cmd == BIO_WRITE)
 		auio.uio_rw = UIO_WRITE;
 	else
 		panic("wrong BIO_OP in mdstart_vnode");
 	auio.uio_resid = bp->bio_length;
-	auio.uio_td = curthread;
+	auio.uio_td = td;
 	/*
 	 * When reading set IO_DIRECT to try to avoid double-caching
 	 * the data.  When writing IO_DIRECT is not optimal.
 	 */
-	vfslocked = VFS_LOCK_GIANT(sc->vnode->v_mount);
+	vfslocked = VFS_LOCK_GIANT(vp->v_mount);
 	if (bp->bio_cmd == BIO_READ) {
-		vn_lock(sc->vnode, LK_EXCLUSIVE | LK_RETRY, curthread);
-		error = VOP_READ(sc->vnode, &auio, IO_DIRECT, sc->cred);
-		VOP_UNLOCK(sc->vnode, 0, curthread);
+		vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, td);
+		error = VOP_READ(vp, &auio, IO_DIRECT, sc->cred);
+		VOP_UNLOCK(vp, 0, td);
 	} else {
-		(void) vn_start_write(sc->vnode, &mp, V_WAIT);
-		vn_lock(sc->vnode, LK_EXCLUSIVE | LK_RETRY, curthread);
-		error = VOP_WRITE(sc->vnode, &auio,
-		    sc->flags & MD_ASYNC ? 0 : IO_SYNC, sc->cred);
-		VOP_UNLOCK(sc->vnode, 0, curthread);
+		(void) vn_start_write(vp, &mp, V_WAIT);
+		vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, td);
+		error = VOP_WRITE(vp, &auio, sc->flags & MD_ASYNC ? 0 : IO_SYNC,
+		    sc->cred);
+		VOP_UNLOCK(vp, 0, td);
 		vn_finished_write(mp);
 	}
 	VFS_UNLOCK_GIANT(vfslocked);
@@ -544,6 +581,15 @@ mdstart_swap(struct md_s *sc, struct bio *bp)
 	vm_page_t m;
 	u_char *p;
 
+	switch (bp->bio_cmd) {
+	case BIO_READ:
+	case BIO_WRITE:
+	case BIO_DELETE:
+		break;
+	default:
+		return (EOPNOTSUPP);
+	}
+
 	p = bp->bio_data;
 
 	/*
@@ -644,9 +690,9 @@ md_kthread(void *arg)
 	int error;
 
 	sc = arg;
-	mtx_lock_spin(&sched_lock);
+	thread_lock(curthread);
 	sched_prio(curthread, PRIBIO);
-	mtx_unlock_spin(&sched_lock);
+	thread_unlock(curthread);
 	if (sc->type == MD_VNODE)
 		curthread->td_pflags |= TDP_NORUNNINGBUF;
 
@@ -866,12 +912,12 @@ mdcreate_vnode(struct md_s *sc, struct md_ioctl *mdio, struct thread *td)
 	 */
 	if ((mdio->md_options & MD_READONLY) != 0)
 		flags &= ~FWRITE;
-	NDINIT(&nd, LOOKUP, FOLLOW, UIO_SYSSPACE, sc->file, td);
-	error = vn_open(&nd, &flags, 0, -1);
+	NDINIT(&nd, LOOKUP, FOLLOW | MPSAFE, UIO_SYSSPACE, sc->file, td);
+	error = vn_open(&nd, &flags, 0, NULL);
 	if (error != 0)
 		return (error);
+	vfslocked = NDHASGIANT(&nd);
 	NDFREE(&nd, NDF_ONLY_PNBUF);
-	vfslocked = VFS_LOCK_GIANT(nd.ni_vp->v_mount);
 	if (nd.ni_vp->v_type != VREG ||
 	    (error = VOP_GETATTR(nd.ni_vp, &vattr, td->td_ucred, td))) {
 		VOP_UNLOCK(nd.ni_vp, 0, td);
@@ -1180,6 +1226,65 @@ g_md_init(struct g_class *mp __unused)
 	g_topology_lock();
 }
 
+static void
+g_md_dumpconf(struct sbuf *sb, const char *indent, struct g_geom *gp, 
+    struct g_consumer *cp __unused, struct g_provider *pp)
+{
+	struct md_s *mp;
+	char *type;
+
+	mp = gp->softc;
+	if (mp == NULL)
+		return;
+
+	switch (mp->type) {
+	case MD_MALLOC:
+		type = "malloc";
+		break;
+	case MD_PRELOAD:
+		type = "preload";
+		break;
+	case MD_VNODE:
+		type = "vnode";
+		break;
+	case MD_SWAP:
+		type = "swap";
+		break;
+	default:
+		type = "unknown";
+		break;
+	}
+
+	if (pp != NULL) {
+		if (indent == NULL) {
+			sbuf_printf(sb, " u %d", mp->unit);
+			sbuf_printf(sb, " s %ju", (uintmax_t) mp->sectorsize);
+			sbuf_printf(sb, " f %ju", (uintmax_t) mp->fwheads);
+			sbuf_printf(sb, " fs %ju", (uintmax_t) mp->fwsectors);
+			sbuf_printf(sb, " l %ju", (uintmax_t) mp->mediasize);
+			sbuf_printf(sb, " t %s", type);
+			if (mp->type == MD_VNODE && mp->vnode != NULL)
+				sbuf_printf(sb, " file %s", mp->file);
+		} else {
+			sbuf_printf(sb, "%s<unit>%d</unit>\n", indent,
+			    mp->unit);
+			sbuf_printf(sb, "%s<sectorsize>%ju</sectorsize>\n",
+			    indent, (uintmax_t) mp->sectorsize);
+			sbuf_printf(sb, "%s<fwheads>%ju</fwheads>\n",
+			    indent, (uintmax_t) mp->fwheads);
+			sbuf_printf(sb, "%s<fwsectors>%ju</fwsectors>\n",
+			    indent, (uintmax_t) mp->fwsectors);
+			sbuf_printf(sb, "%s<length>%ju</length>\n",
+			    indent, (uintmax_t) mp->mediasize);
+			sbuf_printf(sb, "%s<type>%s</type>\n", indent,
+			    type);
+			if (mp->type == MD_VNODE && mp->vnode != NULL)
+				sbuf_printf(sb, "%s<file>%s</file>\n",
+				    indent, mp->file);
+		}
+	}
+}
+
 static void
 g_md_fini(struct g_class *mp __unused)
 {
