@@ -54,7 +54,7 @@
 #define _MFIREG_H
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mfi/mfireg.h,v 1.1.2.8 2007/08/26 02:36:31 scottl Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mfi/mfireg.h,v 1.10.4.1 2008/02/04 14:54:21 ambrisko Exp $");
 
 /*
  * MegaRAID SAS MFI firmware definitions
@@ -352,6 +352,8 @@ struct mfi_frame_header {
 	uint32_t	context;
 	uint32_t	pad0;
 	uint16_t	flags;
+#define MFI_FRAME_DATAOUT	0x08
+#define MFI_FRAME_DATAIN	0x10
 	uint16_t	timeout;
 	uint32_t	data_len;
 } __packed;
