@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mii/brgphy.c,v 1.34.2.15 2007/06/14 21:07:19 davidch Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mii/brgphy.c,v 1.70 2007/06/08 02:34:44 davidch Exp $");
 
 /*
  * Driver for the Broadcom BCM54xx/57xx 1000baseTX PHY.
@@ -286,13 +286,6 @@ brgphy_attach(device_t dev)
 
 #undef ADD
 	MIIBUS_MEDIAINIT(sc->mii_dev);
-
-	/* If MFW running on bce, bring the PHY up now. */
-	if (bce_sc && (bce_sc->bce_flags & BCE_MFW_ENABLE_FLAG)) {
-		brgphy_mii_phy_auto(sc);
-		brgphy_status(sc);
-	}
-
 	return (0);
 }
 
