@@ -38,7 +38,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mii/mii_physubr.c,v 1.22.2.5 2007/02/25 16:57:43 marius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mii/mii_physubr.c,v 1.29 2007/01/13 00:14:45 marius Exp $");
 
 /*
  * Subroutines common to all PHYs.
@@ -325,24 +325,6 @@ mii_anar(int media)
 	return (rv);
 }
 
-/*
- * Given a BMCR value, return the corresponding ifmedia word.
- */
-int
-mii_media_from_bmcr(int bmcr)
-{
-	int rv = IFM_ETHER;
-
-	if (bmcr & BMCR_S100)
-		rv |= IFM_100_TX;
-	else
-		rv |= IFM_10_T;
-	if (bmcr & BMCR_FDX)
-		rv |= IFM_FDX;
-
-	return (rv);
-}
-
 /*
  * Initialize generic PHY media based on BMSR, called when a PHY is
  * attached.  We expect to be set up to print a comma-separated list
@@ -542,10 +524,12 @@ mii_phy_detach(device_t dev)
 }
 
 const struct mii_phydesc *
-mii_phy_match(const struct mii_attach_args *ma, const struct mii_phydesc *mpd)
+mii_phy_match_gen(const struct mii_attach_args *ma,
+  const struct mii_phydesc *mpd, size_t len)
 {
 
-	for (; mpd->mpd_name != NULL; mpd++) {
+	for (; mpd->mpd_name != NULL;
+	     mpd = (const struct mii_phydesc *) ((const char *) mpd + len)) {
 		if (MII_OUI(ma->mii_id1, ma->mii_id2) == mpd->mpd_oui &&
 		    MII_MODEL(ma->mii_id2) == mpd->mpd_model)
 			return (mpd);
@@ -553,6 +537,13 @@ mii_phy_match(const struct mii_attach_args *ma, const struct mii_phydesc *mpd)
 	return (NULL);
 }
 
+const struct mii_phydesc *
+mii_phy_match(const struct mii_attach_args *ma, const struct mii_phydesc *mpd)
+{
+
+	return (mii_phy_match_gen(ma, mpd, sizeof(struct mii_phydesc)));
+}
+
 int
 mii_phy_dev_probe(device_t dev, const struct mii_phydesc *mpd, int mrv)
 {
