@@ -38,7 +38,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mii/nsgphy.c,v 1.19.2.4 2007/02/25 16:57:43 marius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mii/nsgphy.c,v 1.24 2006/12/02 15:32:33 marius Exp $");
 
 /*
  * Driver for the National Semiconductor DP83891 and DP83861
@@ -61,8 +61,6 @@ __FBSDID("$FreeBSD: src/sys/dev/mii/nsgphy.c,v 1.19.2.4 2007/02/25 16:57:43 mari
 #include <sys/socket.h>
 #include <sys/bus.h>
 
-#include <machine/clock.h>
-
 #include <net/if.h>
 #include <net/if_media.h>
 
