@@ -67,7 +67,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mii/nsphy.c,v 1.23.2.4 2007/02/25 16:57:43 marius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mii/nsphy.c,v 1.28 2007/01/12 22:27:46 marius Exp $");
 
 /*
  * driver for National Semiconductor's DP83840A ethernet 10/100 PHY
