@@ -23,10 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/mlx/mlxreg.h,v 1.8 2002/09/23 18:54:30 alfred Exp $
+ *	$FreeBSD: src/sys/dev/mlx/mlxreg.h,v 1.9 2004/11/25 12:15:49 scottl Exp $
  */
 
 #define MLX_BLKSIZE	512		/* fixed feature */
+#define MLX_PAGE_SIZE	4096		/* controller, not cpu, attribute */
 
 /*
  * Selected command codes.
