@@ -21,10 +21,37 @@
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Portions of this software may have been developed with reference to
+ * the SD Simplified Specification.  The following disclaimer may apply:
+ *
+ * The following conditions apply to the release of the simplified
+ * specification ("Simplified Specification") by the SD Card Association and
+ * the SD Group. The Simplified Specification is a subset of the complete SD
+ * Specification which is owned by the SD Card Association and the SD
+ * Group. This Simplified Specification is provided on a non-confidential
+ * basis subject to the disclaimers below. Any implementation of the
+ * Simplified Specification may require a license from the SD Card
+ * Association, SD Group, SD-3C LLC or other third parties.
+ *
+ * Disclaimers:
+ *
+ * The information contained in the Simplified Specification is presented only
+ * as a standard specification for SD Cards and SD Host/Ancillary products and
+ * is provided "AS-IS" without any representations or warranties of any
+ * kind. No responsibility is assumed by the SD Group, SD-3C LLC or the SD
+ * Card Association for any damages, any infringements of patents or other
+ * right of the SD Group, SD-3C LLC, the SD Card Association or any third
+ * parties, which may result from its use. No license is granted by
+ * implication, estoppel or otherwise under any patent or other rights of the
+ * SD Group, SD-3C LLC, the SD Card Association or any third party. Nothing
+ * herein shall be construed as an obligation by the SD Group, the SD-3C LLC
+ * or the SD Card Association to disclose or distribute any technical
+ * information, know-how or other confidential information to any third party.
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mmc/mmc.c,v 1.1.2.1 2006/11/20 07:16:28 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mmc/mmc.c,v 1.4 2007/06/05 17:04:44 imp Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -117,7 +144,26 @@ mmc_attach(device_t dev)
 static int
 mmc_detach(device_t dev)
 {
-	return (EBUSY);	/* XXX */
+	struct mmc_softc *sc = device_get_softc(dev);
+	device_t *kids;
+	int i, nkid;
+
+	/* kill children [ph33r].  -sorbo */
+	if (device_get_children(sc->dev, &kids, &nkid) != 0)
+		return 0;
+	for (i = 0; i < nkid; i++) {
+		device_t kid = kids[i];
+		void *ivar = device_get_ivars(kid);
+		
+		device_detach(kid);
+		device_delete_child(sc->dev, kid);
+		free(ivar, M_DEVBUF);
+	}
+	free(kids, M_TEMP);
+
+	MMC_LOCK_DESTROY(sc);
+
+	return 0;
 }
 
 static int
@@ -347,7 +393,7 @@ mmc_send_app_op_cond(struct mmc_softc *sc, uint32_t ocr, uint32_t *rocr)
 	cmd.arg = ocr;
 	cmd.flags = MMC_RSP_R3 | MMC_CMD_BCR;
 
-	for (i = 0; i < 10; i++) {
+	for (i = 0; i < 100; i++) {
 		err = mmc_wait_for_app_cmd(sc, 0, &cmd, CMD_RETRIES);
 		if (err != MMC_ERR_NONE)
 			break;
@@ -553,6 +599,8 @@ mmc_discover_cards(struct mmc_softc *sc)
 
 	while (1) {
 		ivar = malloc(sizeof(struct mmc_ivars), M_DEVBUF, M_WAITOK);
+		if (!ivar)
+			return;
 		err = mmc_all_send_cid(sc, ivar->raw_cid);
 		if (err == MMC_ERR_TIMEOUT)
 			break;
@@ -568,13 +616,15 @@ mmc_discover_cards(struct mmc_softc *sc)
 			// RO check
 			mmc_send_csd(sc, ivar->rca, ivar->raw_csd);
 			mmc_decode_csd(1, ivar->raw_csd, &ivar->csd);
-			printf("SD CARD: %lld bytes\n", ivar->csd.capacity);
+			printf("SD CARD: %lld bytes\n", (long long)
+			    ivar->csd.capacity);
 			child = device_add_child(sc->dev, NULL, -1);
 			device_set_ivars(child, ivar);
-			break;
+			return;
 		}
 		panic("Write MMC card code here");
 	}
+	free(ivar, M_DEVBUF);
 }
 
 static void
@@ -679,9 +729,6 @@ mmc_read_ivar(device_t bus, device_t child, int which, u_char *result)
 	case MMC_IVAR_MEDIA_SIZE:
 		*(int *)result = ivar->csd.capacity;
 		break;
-	case MMC_IVAR_MODE:
-		*(int *)result = ivar->mode;
-		break;
 	case MMC_IVAR_RCA:
 		*(int *)result = ivar->rca;
 		break;
@@ -743,3 +790,4 @@ static devclass_t mmc_devclass;
 
 
 DRIVER_MODULE(mmc, at91_mci, mmc_driver, mmc_devclass, 0, 0);
+DRIVER_MODULE(mmc, sdh, mmc_driver, mmc_devclass, 0, 0);
