@@ -96,7 +96,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/mpt/mpt.c,v 1.12.2.6 2007/06/05 21:32:56 scottl Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/mpt/mpt.c,v 1.44.2.2 2007/12/15 18:39:07 delphij Exp $");
 
 #include <dev/mpt/mpt.h>
 #include <dev/mpt/mpt_cam.h> /* XXX For static handler registration */
@@ -562,7 +562,8 @@ mpt_event_reply_handler(struct mpt_softc *mpt, request_t *req,
 				"Event %#x (ACK %sequired).\n",
 				msg->Event, msg->AckRequired? "r" : "not r");
 		} else if (handled == 0) {
-			mpt_lprt(mpt, MPT_PRT_WARN,
+			mpt_lprt(mpt,
+				msg->AckRequired? MPT_PRT_WARN : MPT_PRT_INFO,
 				"Unhandled Event Notify Frame. Event %#x "
 				"(ACK %sequired).\n",
 				msg->Event, msg->AckRequired? "r" : "not r");
@@ -706,6 +707,8 @@ mpt_intr(void *arg)
 
 	mpt = (struct mpt_softc *)arg;
 	mpt_lprt(mpt, MPT_PRT_DEBUG2, "enter mpt_intr\n");
+	MPT_LOCK_ASSERT(mpt);
+
 	while ((reply_desc = mpt_pop_reply_queue(mpt)) != MPT_REPLY_EMPTY) {
 		request_t	  *req;
 		MSG_DEFAULT_REPLY *reply_frame;
@@ -1173,7 +1176,7 @@ mpt_free_request(struct mpt_softc *mpt, request_t *req)
 	}
 	KASSERT(req->state != REQ_STATE_FREE, ("freeing free request"));
 	KASSERT(!(req->state & REQ_STATE_LOCKED), ("freeing locked request"));
-	KASSERT(MPT_OWNED(mpt), ("mpt_free_request: mpt not locked\n"));
+	MPT_LOCK_ASSERT(mpt);
 	KASSERT(mpt_req_on_free_list(mpt, req) == 0,
 	    ("mpt_free_request: req %p:%u func %x already on freelist",
 	    req, req->serno, ((MSG_REQUEST_HEADER *)req->req_vbuf)->Function));
@@ -1222,7 +1225,7 @@ mpt_get_request(struct mpt_softc *mpt, int sleep_ok)
 	request_t *req;
 
 retry:
-	KASSERT(MPT_OWNED(mpt), ("mpt_get_request: mpt not locked\n"));
+	MPT_LOCK_ASSERT(mpt);
 	req = TAILQ_FIRST(&mpt->request_free_list);
 	if (req != NULL) {
 		KASSERT(req == &mpt->request_pool[req->index],
@@ -1235,6 +1238,7 @@ retry:
 		req->state = REQ_STATE_ALLOCATED;
 		req->chain = NULL;
 		mpt_assign_serno(mpt, req);
+		mpt_callout_init(&req->callout);
 	} else if (sleep_ok != 0) {
 		mpt->getreqwaiter = 1;
 		mpt_sleep(mpt, &mpt->request_free_list, PUSER, "mptgreq", 0);
@@ -1353,7 +1357,7 @@ mpt_send_handshake_cmd(struct mpt_softc *mpt, size_t len, void *cmd)
 	len = (len + 3) >> 2;
 	data32 = cmd;
 
-	/* Clear any left over pending doorbell interupts */
+	/* Clear any left over pending doorbell interrupts */
 	if (MPT_DB_INTR(mpt_rd_intr(mpt)))
 		mpt_write(mpt, MPT_OFFSET_INTR_STATUS, 0);
 
@@ -2092,7 +2096,7 @@ mpt_send_event_request(struct mpt_softc *mpt, int onoff)
 }
 
 /*
- * Un-mask the interupts on the chip.
+ * Un-mask the interrupts on the chip.
  */
 void
 mpt_enable_ints(struct mpt_softc *mpt)
@@ -2102,7 +2106,7 @@ mpt_enable_ints(struct mpt_softc *mpt)
 }
 
 /*
- * Mask the interupts on the chip.
+ * Mask the interrupts on the chip.
  */
 void
 mpt_disable_ints(struct mpt_softc *mpt)
@@ -2232,18 +2236,20 @@ mpt_core_load(struct mpt_personality *pers)
 int
 mpt_core_attach(struct mpt_softc *mpt)
 {
-        int val;
+        int val, error;
 
 	LIST_INIT(&mpt->ack_frames);
 	/* Put all request buffers on the free list */
 	TAILQ_INIT(&mpt->request_pending_list);
 	TAILQ_INIT(&mpt->request_free_list);
 	TAILQ_INIT(&mpt->request_timeout_list);
+	MPT_LOCK(mpt);
 	for (val = 0; val < MPT_MAX_REQUESTS(mpt); val++) {
 		request_t *req = &mpt->request_pool[val];
 		req->state = REQ_STATE_ALLOCATED;
 		mpt_free_request(mpt, req);
 	}
+	MPT_UNLOCK(mpt);
 	for (val = 0; val < MPT_MAX_LUNS; val++) {
 		STAILQ_INIT(&mpt->trt[val].atios);
 		STAILQ_INIT(&mpt->trt[val].inots);
@@ -2257,7 +2263,12 @@ mpt_core_attach(struct mpt_softc *mpt)
 	mpt_sysctl_attach(mpt);
 	mpt_lprt(mpt, MPT_PRT_DEBUG, "doorbell req = %s\n",
 	    mpt_ioc_diag(mpt_read(mpt, MPT_OFFSET_DOORBELL)));
-	return (mpt_configure_ioc(mpt, 0, 0));
+
+	MPT_LOCK(mpt);
+	error = mpt_configure_ioc(mpt, 0, 0);
+	MPT_UNLOCK(mpt);
+
+	return (error);
 }
 
 int
@@ -2268,6 +2279,7 @@ mpt_core_enable(struct mpt_softc *mpt)
 	 * not enabled, ports not enabled and interrupts
 	 * not enabled.
 	 */
+	MPT_LOCK(mpt);
 
 	/*
 	 * Enable asynchronous event reporting- all personalities
@@ -2302,8 +2314,10 @@ mpt_core_enable(struct mpt_softc *mpt)
 	 */
 	if (mpt_send_port_enable(mpt, 0) != MPT_OK) {
 		mpt_prt(mpt, "failed to enable port 0\n");
+		MPT_UNLOCK(mpt);
 		return (ENXIO);
 	}
+	MPT_UNLOCK(mpt);
 	return (0);
 }
 
