@@ -25,7 +25,7 @@
  *
  * Written by: yen_cw@myson.com.tw  available at: http://www.myson.com.tw/
  *
- * $FreeBSD: src/sys/dev/my/if_myreg.h,v 1.3.2.1 2005/08/29 20:00:31 jhb Exp $
+ * $FreeBSD: src/sys/dev/my/if_myreg.h,v 1.6 2006/09/15 15:16:12 glebius Exp $
  *
  * Myson MTD80x register definitions.
  *
@@ -351,6 +351,7 @@ struct my_type {
 
 struct my_softc {
         struct ifnet            *my_ifp;
+	device_t		my_dev;
         struct ifmedia          ifmedia;        /* media info */
         bus_space_handle_t      my_bhandle;
         bus_space_tag_t         my_btag;
