@@ -37,7 +37,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ncv/ncr53c500.c,v 1.10.2.1 2005/01/30 00:59:55 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ncv/ncr53c500.c,v 1.13 2006/05/16 14:36:29 phk Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -73,9 +73,7 @@ __FBSDID("$FreeBSD: src/sys/dev/ncv/ncr53c500.c,v 1.10.2.1 2005/01/30 00:59:55 i
 #endif /* __NetBSD__ */
 
 #ifdef __FreeBSD__
-#include <machine/clock.h>
 #include <machine/cpu.h>
-#include <machine/bus_pio.h>
 #include <machine/bus.h>
 
 #include <compat/netbsd/dvcfg.h>
