@@ -37,7 +37,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ncv/ncr53c500_pccard.c,v 1.26 2005/06/24 14:36:53 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ncv/ncr53c500_pccard.c,v 1.29 2007/02/23 12:18:47 piso Exp $");
 
 #include <sys/param.h>
 #include <sys/errno.h>
@@ -200,7 +200,8 @@ ncv_alloc_resource(DEVPORT_PDEVICE dev)
 	return(0);
 }
 
-static int ncv_pccard_match(device_t dev)
+static int
+ncv_pccard_probe(device_t dev)
 {
 	const struct ncv_product *pp;
 	const char *vendorstr;
@@ -228,7 +229,7 @@ static int ncv_pccard_match(device_t dev)
 }
 
 static int
-ncv_pccard_probe(DEVPORT_PDEVICE dev)
+ncv_pccard_attach(device_t dev)
 {
 	struct ncv_softc	*sc = device_get_softc(dev);
 	int			error;
@@ -244,25 +245,8 @@ ncv_pccard_probe(DEVPORT_PDEVICE dev)
 		ncv_release_resource(dev);
 		return(ENXIO);
 	}
-
-	ncv_release_resource(dev);
-
-	return(0);
-}
-
-static int
-ncv_pccard_attach(DEVPORT_PDEVICE dev)
-{
-	struct ncv_softc	*sc = device_get_softc(dev);
-	int			error;
-
-	error = ncv_alloc_resource(dev);
-	if (error) {
-		return(error);
-	}
-
 	error = bus_setup_intr(dev, sc->irq_res, INTR_TYPE_CAM | INTR_ENTROPY,
-			       ncv_pccard_intr, (void *)sc, &sc->ncv_intrhand);
+			       NULL, ncv_pccard_intr, (void *)sc, &sc->ncv_intrhand);
 	if (error) {
 		ncv_release_resource(dev);
 		return(error);
@@ -277,7 +261,7 @@ ncv_pccard_attach(DEVPORT_PDEVICE dev)
 }
 
 static	void
-ncv_pccard_detach(DEVPORT_PDEVICE dev)
+ncv_pccard_detach(device_t dev)
 {
 	ncv_card_unload(dev);
 	ncv_release_resource(dev);
@@ -285,15 +269,10 @@ ncv_pccard_detach(DEVPORT_PDEVICE dev)
 
 static device_method_t ncv_pccard_methods[] = {
 	/* Device interface */
-	DEVMETHOD(device_probe,		pccard_compat_probe),
-	DEVMETHOD(device_attach,	pccard_compat_attach),
+	DEVMETHOD(device_probe,		ncv_pccard_probe),
+	DEVMETHOD(device_attach,	ncv_pccard_attach),
 	DEVMETHOD(device_detach,	ncv_pccard_detach),
 
-	/* Card interface */
-	DEVMETHOD(card_compat_match,	ncv_pccard_match),
-	DEVMETHOD(card_compat_probe,	ncv_pccard_probe),
-	DEVMETHOD(card_compat_attach,	ncv_pccard_attach),
-
 	{ 0, 0 }
 };
 
@@ -314,7 +293,6 @@ ncv_card_unload(DEVPORT_PDEVICE devi)
 	struct ncv_softc *sc = DEVPORT_PDEVGET_SOFTC(devi);
 	intrmask_t s;
 
-	printf("%s: unload\n", sc->sc_sclow.sl_xname);
 	s = splcam();
 	scsi_low_deactivate((struct scsi_low_softc *)sc);
         scsi_low_dettach(&sc->sc_sclow);
