@@ -28,7 +28,7 @@
  * SUCH DAMAGE.
  */
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/pcf/pcf_ebus.c,v 1.4.2.1 2007/05/12 06:10:10 nyan Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/pcf/pcf_ebus.c,v 1.7 2007/03/21 03:38:36 nyan Exp $");
 
 /*
  * Device specific driver for the EBus i2c devices found on some sun4u
@@ -193,8 +193,8 @@ pcf_ebus_attach(device_t dev)
 	pcf_rst_card(dev, IIC_FASTEST, own_addr, NULL);
 
 	if (sc->res_irq) {
-		rv = BUS_SETUP_INTR(device_get_parent(dev), dev, sc->res_irq,
-		    INTR_TYPE_NET /* | INTR_ENTROPY */, pcf_intr, sc,
+		rv = bus_setup_intr(dev, sc->res_irq,
+		    INTR_TYPE_NET /* | INTR_ENTROPY */, NULL, pcf_intr, sc,
 		    &sc->intr_cookie);
 		if (rv) {
 			device_printf(dev, "could not setup IRQ\n");
@@ -237,7 +237,7 @@ pcf_ebus_detach(device_t dev)
 		return (rv);
 
 	if (sc->res_irq != 0) {
-		BUS_TEARDOWN_INTR(device_get_parent(dev), dev, sc->res_irq,
+		bus_teardown_intr(dev, sc->res_irq,
 		    sc->intr_cookie);
 		bus_release_resource(dev, SYS_RES_IRQ, sc->rid_irq,
 		    sc->res_irq);
