@@ -23,7 +23,7 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 #
-# $FreeBSD: src/sys/dev/pci/pci_if.m,v 1.7.2.2 2007/03/31 14:50:51 jhb Exp $
+# $FreeBSD: src/sys/dev/pci/pci_if.m,v 1.12 2007/05/02 17:50:35 jhb Exp $
 #
 
 #include <sys/bus.h>
@@ -65,6 +65,19 @@ METHOD int set_powerstate {
 	int		state;
 };
 
+METHOD int get_vpd_ident {
+	device_t	dev;
+	device_t	child;
+	const char	**identptr;
+};
+
+METHOD int get_vpd_readonly {
+	device_t	dev;
+	device_t	child;
+	const char	*kw;
+	const char	**vptr;
+};
+
 METHOD int enable_busmaster {
 	device_t	dev;
 	device_t	child;
@@ -111,6 +124,13 @@ METHOD int alloc_msix {
 	int		*count;
 };
 
+METHOD int remap_msix {
+	device_t	dev;
+	device_t	child;
+	int		count;
+	const u_int	*vectors;
+};
+
 METHOD int release_msi {
 	device_t	dev;
 	device_t	child;
