@@ -23,7 +23,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/pci/pcireg.h,v 1.45.2.9 2007/10/15 02:43:39 sepotvin Exp $
+ * $FreeBSD: src/sys/dev/pci/pcireg.h,v 1.64 2007/09/19 13:05:58 sepotvin Exp $
  *
  */
 
@@ -116,11 +116,25 @@
 /* config registers for header type 0 devices */
 
 #define	PCIR_BARS	0x10
-#define	PCIR_BAR(x)	(PCIR_BARS + (x) * 4)
-#define	PCI_RID2BAR(rid) (((rid)-PCIR_BARS)/4)
+#define	PCIR_BAR(x)		(PCIR_BARS + (x) * 4)
+#define	PCI_MAX_BAR_0		5	/* Number of standard bars */
+#define	PCI_RID2BAR(rid)	(((rid) - PCIR_BARS) / 4)
+#define	PCI_BAR_IO(x)		(((x) & PCIM_BAR_SPACE) == PCIM_BAR_IO_SPACE)
+#define	PCI_BAR_MEM(x)		(((x) & PCIM_BAR_SPACE) == PCIM_BAR_MEM_SPACE)
+#define	PCIM_BAR_SPACE		0x00000001
+#define	PCIM_BAR_MEM_SPACE	0
+#define	PCIM_BAR_IO_SPACE	1
+#define	PCIM_BAR_MEM_TYPE	0x00000006
+#define	PCIM_BAR_MEM_32		0
+#define	PCIM_BAR_MEM_1MB	2	/* Locate below 1MB in PCI <= 2.1 */
+#define	PCIM_BAR_MEM_64		4
+#define	PCIM_BAR_MEM_PREFETCH	0x00000008
+#define	PCIM_BAR_MEM_BASE	0xfffffff0
+#define	PCIM_BAR_IO_RESERVED	0x00000002
+#define	PCIM_BAR_IO_BASE	0xfffffffc
 #define	PCIR_CIS	0x28
 #define	PCIM_CIS_ASI_MASK	0x7
-#define	PCIM_CIS_ASI_TUPLE	0
+#define	PCIM_CIS_ASI_CONFIG	0
 #define	PCIM_CIS_ASI_BAR0	1
 #define	PCIM_CIS_ASI_BAR1	2
 #define	PCIM_CIS_ASI_BAR2	3
@@ -130,6 +144,7 @@
 #define	PCIM_CIS_ASI_ROM	7
 #define	PCIM_CIS_ADDR_MASK	0x0ffffff8
 #define	PCIM_CIS_ROM_MASK	0xf0000000
+#define PCIM_CIS_CONFIG_MASK	0xff
 #define	PCIR_SUBVEND_0	0x2c
 #define	PCIR_SUBDEV_0	0x2e
 #define	PCIR_BIOS	0x30
@@ -211,6 +226,10 @@
 #define	PCIS_STORAGE_FLOPPY	0x02
 #define	PCIS_STORAGE_IPI	0x03
 #define	PCIS_STORAGE_RAID	0x04
+#define	PCIS_STORAGE_ATA_ADMA	0x05
+#define	PCIS_STORAGE_SATA	0x06
+#define	PCIP_STORAGE_SATA_AHCI_1_0	0x01
+#define	PCIS_STORAGE_SAS	0x07
 #define	PCIS_STORAGE_OTHER	0x80
 
 #define	PCIC_NETWORK	0x02
@@ -219,6 +238,8 @@
 #define	PCIS_NETWORK_FDDI	0x02
 #define	PCIS_NETWORK_ATM	0x03
 #define	PCIS_NETWORK_ISDN	0x04
+#define	PCIS_NETWORK_WORLDFIP	0x05
+#define	PCIS_NETWORK_PICMG	0x06
 #define	PCIS_NETWORK_OTHER	0x80
 
 #define	PCIC_DISPLAY	0x03
@@ -244,10 +265,13 @@
 #define	PCIS_BRIDGE_EISA	0x02
 #define	PCIS_BRIDGE_MCA		0x03
 #define	PCIS_BRIDGE_PCI		0x04
+#define	PCIP_BRIDGE_PCI_SUBTRACTIVE	0x01
 #define	PCIS_BRIDGE_PCMCIA	0x05
 #define	PCIS_BRIDGE_NUBUS	0x06
 #define	PCIS_BRIDGE_CARDBUS	0x07
 #define	PCIS_BRIDGE_RACEWAY	0x08
+#define	PCIS_BRIDGE_PCI_TRANSPARENT 0x09
+#define	PCIS_BRIDGE_INFINIBAND	0x0a
 #define	PCIS_BRIDGE_OTHER	0x80
 
 #define	PCIC_SIMPLECOMM	0x07
@@ -262,6 +286,8 @@
 #define	PCIS_SIMPLECOMM_PAR	0x01
 #define	PCIS_SIMPLECOMM_MULSER	0x02
 #define	PCIS_SIMPLECOMM_MODEM	0x03
+#define	PCIS_SIMPLECOMM_GPIB	0x04
+#define	PCIS_SIMPLECOMM_SMART_CARD 0x05
 #define	PCIS_SIMPLECOMM_OTHER	0x80
 
 #define	PCIC_BASEPERIPH	0x08
@@ -304,16 +330,28 @@
 #define	PCIS_SERIALBUS_ACCESS	0x01
 #define	PCIS_SERIALBUS_SSA	0x02
 #define	PCIS_SERIALBUS_USB	0x03
-#define	PCIP_SERIALBUS_USB_UHCI	0x00
-#define	PCIP_SERIALBUS_USB_OHCI	0x10
-#define	PCIP_SERIALBUS_USB_EHCI	0x20
+#define	PCIP_SERIALBUS_USB_UHCI		0x00
+#define	PCIP_SERIALBUS_USB_OHCI		0x10
+#define	PCIP_SERIALBUS_USB_EHCI		0x20
+#define	PCIP_SERIALBUS_USB_DEVICE	0xfe
 #define	PCIS_SERIALBUS_FC	0x04
 #define	PCIS_SERIALBUS_SMBUS	0x05
+#define	PCIS_SERIALBUS_INFINIBAND 0x06
+#define	PCIS_SERIALBUS_IPMI	0x07
+#define	PCIP_SERIALBUS_IPMI_SMIC	0x00
+#define	PCIP_SERIALBUS_IPMI_KCS		0x01
+#define	PCIP_SERIALBUS_IPMI_BT		0x02
+#define	PCIS_SERIALBUS_SERCOS	0x08
+#define	PCIS_SERIALBUS_CANBUS	0x09
 
 #define	PCIC_WIRELESS	0x0d
 #define	PCIS_WIRELESS_IRDA	0x00
 #define	PCIS_WIRELESS_IR	0x01
 #define	PCIS_WIRELESS_RF	0x10
+#define	PCIS_WIRELESS_BLUETOOTH	0x11
+#define	PCIS_WIRELESS_BROADBAND	0x12
+#define	PCIS_WIRELESS_80211A	0x20
+#define	PCIS_WIRELESS_80211B	0x21
 #define	PCIS_WIRELESS_OTHER	0x80
 
 #define	PCIC_INTELLIIO	0x0e
@@ -331,8 +369,11 @@
 #define	PCIS_CRYPTO_OTHER	0x80
 
 #define	PCIC_DASP	0x11
-#define	PCIS_DASP_DPIO	0x00
-#define	PCIS_DASP_OTHER	0x80
+#define	PCIS_DASP_DPIO		0x00
+#define	PCIS_DASP_PERFCNTRS	0x01
+#define	PCIS_DASP_COMM_SYNC	0x10
+#define	PCIS_DASP_MGMT_CARD	0x20
+#define	PCIS_DASP_OTHER		0x80
 
 #define	PCIC_OTHER	0xff
 
