@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ppbus/if_plip.c,v 1.37.2.3 2007/01/19 23:01:30 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ppbus/if_plip.c,v 1.44 2007/02/23 12:18:49 piso Exp $");
 
 /*
  * Parallel port TCP/IP interfaces added.  I looked at the driver from
@@ -185,7 +185,7 @@ lp_identify(driver_t *driver, device_t parent)
 {
 	device_t dev;
 
-	dev = device_find_child(parent, "plip", 0);
+	dev = device_find_child(parent, "plip", -1);
 	if (!dev)
 		BUS_ADD_CHILD(parent, 0, "plip", -1);
 }
@@ -357,8 +357,8 @@ lpioctl (struct ifnet *ifp, u_long cmd, caddr_t data)
 	    }
 
 	    /* attach our interrupt handler, later detached when the bus is released */
-	    if ((error = BUS_SETUP_INTR(ppbus, dev, sc->res_irq,
-					INTR_TYPE_NET, lp_intr, dev, &ih))) {
+	    if ((error = bus_setup_intr(dev, sc->res_irq,
+					INTR_TYPE_NET, NULL, lp_intr, dev, &ih))) {
 		ppb_release_bus(ppbus, dev);
 		return (error);
 	    }
