@@ -51,7 +51,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ppbus/lpt.c,v 1.36.2.2 2007/03/23 19:45:53 njl Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ppbus/lpt.c,v 1.40 2007/02/23 12:18:49 piso Exp $");
 
 /*
  * Device Driver for AT parallel printer port
@@ -342,7 +342,7 @@ lpt_identify(driver_t *driver, device_t parent)
 
 	device_t dev;
 
-	dev = device_find_child(parent, LPT_NAME, 0);
+	dev = device_find_child(parent, LPT_NAME, -1);
 	if (!dev)
 		BUS_ADD_CHILD(parent, 0, LPT_NAME, -1);
 }
@@ -759,8 +759,8 @@ lptwrite(struct cdev *dev, struct uio *uio, int ioflag)
 	/* if interrupts are working, register the handler */
 	if (sc->sc_irq & LP_USE_IRQ) {
 		/* register our interrupt handler */
-		err = BUS_SETUP_INTR(ppbus, lptdev, sc->intr_resource,
-			       INTR_TYPE_TTY, lpt_intr, lptdev,
+		err = bus_setup_intr(lptdev, sc->intr_resource,
+			       INTR_TYPE_TTY, NULL, lpt_intr, lptdev,
 			       &sc->intr_cookie);
 		if (err) {
 			device_printf(lptdev, "handler registration failed, polled mode.\n");
