@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ppbus/ppi.c,v 1.38.8.1 2006/07/19 16:31:12 kib Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ppbus/ppi.c,v 1.41 2007/02/23 19:32:21 imp Exp $");
 #include "opt_ppb_1284.h"
 
 #include <sys/param.h>
@@ -135,7 +135,7 @@ ppi_identify(driver_t *driver, device_t parent)
 
 	device_t dev;
 
-	dev = device_find_child(parent, "ppi", 0);
+	dev = device_find_child(parent, "ppi", -1);
 	if (!dev)
 		BUS_ADD_CHILD(parent, 0, "ppi", -1);
 }
@@ -276,8 +276,9 @@ ppiopen(struct cdev *dev, int flags, int fmt, struct thread *td)
 #ifdef PERIPH_1284
 		if (ppi->intr_resource) {
 			/* register our interrupt handler */
-			BUS_SETUP_INTR(device_get_parent(ppidev), ppidev, ppi->intr_resource,
-				       INTR_TYPE_TTY, ppiintr, dev, &ppi->intr_cookie);
+			bus_setup_intr(ppidev, ppi->intr_resource, 
+				       INTR_TYPE_TTY, NULL, ppiintr, dev,
+				       &ppi->intr_cookie);
 		}
 #endif /* PERIPH_1284 */
 	}
