@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/pst/pst-iop.h,v 1.4.6.1 2004/10/02 14:14:27 scottl Exp $
+ * $FreeBSD: src/sys/dev/pst/pst-iop.h,v 1.6 2004/10/03 16:06:46 stefanf Exp $
  */
 
 /* misc defines */
@@ -631,7 +631,7 @@ struct i2o_bsa_cache_flush_message {
 
 /* prototypes */
 int iop_init(struct iop_softc *);
-void iop_attach(struct iop_softc *);
+void iop_attach(void *);
 void iop_intr(void *);
 int iop_reset(struct iop_softc *);
 int iop_init_outqueue(struct iop_softc *);
