@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/random/probe.c,v 1.4.2.1 2006/07/21 15:12:02 mr Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/random/probe.c,v 1.7 2006/07/13 11:47:36 mr Exp $");
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -56,7 +56,7 @@ random_ident_hardware(struct random_systat *systat)
 
 	/* Then go looking for hardware */
 #if defined(__i386__) && !defined(PC98)
-	if( via_feature_rng & VIA_HAS_RNG ) {
+	if (via_feature_rng & VIA_HAS_RNG) {
 		*systat = random_nehemiah;
 	}
 #endif
