@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/random/randomdev.c,v 1.59.2.1 2006/02/08 05:58:17 ps Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/random/randomdev.c,v 1.61 2006/11/06 13:41:55 rwatson Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -41,6 +41,7 @@ __FBSDID("$FreeBSD: src/sys/dev/random/randomdev.c,v 1.59.2.1 2006/02/08 05:58:1
 #include <sys/module.h>
 #include <sys/mutex.h>
 #include <sys/poll.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/selinfo.h>
 #include <sys/uio.h>
@@ -85,7 +86,7 @@ static int
 random_close(struct cdev *dev __unused, int flags, int fmt __unused,
     struct thread *td)
 {
-	if ((flags & FWRITE) && (suser(td) == 0)
+	if ((flags & FWRITE) && (priv_check(td, PRIV_RANDOM_RESEED) == 0)
 	    && (securelevel_gt(td->td_ucred, 0) == 0)) {
 		(*random_systat.reseed)();
 		random_systat.seeded = 1;
