@@ -32,7 +32,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/rp/rp.c,v 1.67.2.3 2007/07/03 20:35:07 remko Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/rp/rp.c,v 1.74 2007/07/05 06:51:49 peter Exp $");
 
 /* 
  * rp.c - for RocketPort FreeBSD
@@ -436,6 +436,7 @@ int sWriteTxPrioByte(CHANNEL_T *ChP, Byte_t Data)
       le16enc(DWBuf,ChP->TxPrioBuf);   /* data byte address */
 
       DWBuf[2] = Data;		       /* data byte value */
+      DWBuf[3] = 0;		       /* priority buffer pointer */
       rp_writech4(ChP,_INDX_ADDR,le32dec(DWBuf)); /* write it out */
 
       le16enc(DWBuf,ChP->TxPrioCnt);   /* Tx priority count address */
@@ -825,8 +826,7 @@ rp_attachcommon(CONTROLLER_T *ctlp, int num_aiops, int num_ports)
 			rp->rp_cts = (ChanStatus & CTS_ACT) != 0;
 			line = (unit << 5) | (aiop << 3) | chan;
 			rp_table(line) = rp;
-			ttycreate(tp, NULL, 0, MINOR_CALLOUT, "R%r%r", unit,
-			    port);
+			ttycreate(tp, TS_CALLOUT, "R%r%r", unit, port);
 		}
 	}
 
