@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/safe/safe.c,v 1.8.2.1 2006/09/04 15:18:50 pjd Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/safe/safe.c,v 1.18.2.1 2007/11/08 07:14:37 sam Exp $");
 
 /*
  * SafeNet SafeXcel-1141 hardware crypto accelerator
@@ -59,6 +59,9 @@ __FBSDID("$FreeBSD: src/sys/dev/safe/safe.c,v 1.8.2.1 2006/09/04 15:18:50 pjd Ex
 #include <opencrypto/cryptosoft.h>
 #include <sys/md5.h>
 #include <sys/random.h>
+#include <sys/kobj.h>
+
+#include "cryptodev_if.h"
 
 #include <dev/pci/pcivar.h>
 #include <dev/pci/pcireg.h>
@@ -83,6 +86,10 @@ static	int safe_suspend(device_t);
 static	int safe_resume(device_t);
 static	void safe_shutdown(device_t);
 
+static	int safe_newsession(device_t, u_int32_t *, struct cryptoini *);
+static	int safe_freesession(device_t, u_int64_t);
+static	int safe_process(device_t, struct cryptop *, int);
+
 static device_method_t safe_methods[] = {
 	/* Device interface */
 	DEVMETHOD(device_probe,		safe_probe),
@@ -96,6 +103,11 @@ static device_method_t safe_methods[] = {
 	DEVMETHOD(bus_print_child,	bus_generic_print_child),
 	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
 
+	/* crypto device methods */
+	DEVMETHOD(cryptodev_newsession,	safe_newsession),
+	DEVMETHOD(cryptodev_freesession,safe_freesession),
+	DEVMETHOD(cryptodev_process,	safe_process),
+
 	{ 0, 0 }
 };
 static driver_t safe_driver = {
@@ -112,9 +124,6 @@ MODULE_DEPEND(safe, rndtest, 1, 1, 1);
 #endif
 
 static	void safe_intr(void *);
-static	int safe_newsession(void *, u_int32_t *, struct cryptoini *);
-static	int safe_freesession(void *, u_int64_t);
-static	int safe_process(void *, struct cryptop *, int);
 static	void safe_callback(struct safe_softc *, struct safe_ringentry *);
 static	void safe_feed(struct safe_softc *, struct safe_ringentry *);
 static	void safe_mcopy(struct mbuf *, struct mbuf *, u_int);
@@ -265,12 +274,12 @@ safe_attach(device_t dev)
 	 *     so make sure the IRQ is mapped appropriately.
 	 */
 	if (bus_setup_intr(dev, sc->sc_irq, INTR_TYPE_NET | INTR_MPSAFE,
-			   safe_intr, sc, &sc->sc_ih)) {
+			   NULL, safe_intr, sc, &sc->sc_ih)) {
 		device_printf(dev, "could not establish interrupt\n");
 		goto bad2;
 	}
 
-	sc->sc_cid = crypto_get_driverid(0);
+	sc->sc_cid = crypto_get_driverid(dev, CRYPTOCAP_F_HARDWARE);
 	if (sc->sc_cid < 0) {
 		device_printf(dev, "could not get crypto driver id\n");
 		goto bad3;
@@ -298,7 +307,7 @@ safe_attach(device_t dev)
 		goto bad4;
 	}
 	if (bus_dma_tag_create(NULL,			/* parent */
-			       sizeof(u_int32_t),	/* alignment */
+			       1,			/* alignment */
 			       SAFE_MAX_DSIZE,		/* boundary */
 			       BUS_SPACE_MAXADDR_32BIT,	/* lowaddr */
 			       BUS_SPACE_MAXADDR,	/* highaddr */
@@ -388,39 +397,30 @@ safe_attach(device_t dev)
 #if 0
 		printf(" key");
 		sc->sc_flags |= SAFE_FLAGS_KEY;
-		crypto_kregister(sc->sc_cid, CRK_MOD_EXP, 0,
-			safe_kprocess, sc);
-		crypto_kregister(sc->sc_cid, CRK_MOD_EXP_CRT, 0,
-			safe_kprocess, sc);
+		crypto_kregister(sc->sc_cid, CRK_MOD_EXP, 0);
+		crypto_kregister(sc->sc_cid, CRK_MOD_EXP_CRT, 0);
 #endif
 	}
 	if (devinfo & SAFE_DEVINFO_DES) {
 		printf(" des/3des");
-		crypto_register(sc->sc_cid, CRYPTO_3DES_CBC, 0, 0,
-			safe_newsession, safe_freesession, safe_process, sc);
-		crypto_register(sc->sc_cid, CRYPTO_DES_CBC, 0, 0,
-			safe_newsession, safe_freesession, safe_process, sc);
+		crypto_register(sc->sc_cid, CRYPTO_3DES_CBC, 0, 0);
+		crypto_register(sc->sc_cid, CRYPTO_DES_CBC, 0, 0);
 	}
 	if (devinfo & SAFE_DEVINFO_AES) {
 		printf(" aes");
-		crypto_register(sc->sc_cid, CRYPTO_AES_CBC, 0, 0,
-			safe_newsession, safe_freesession, safe_process, sc);
+		crypto_register(sc->sc_cid, CRYPTO_AES_CBC, 0, 0);
 	}
 	if (devinfo & SAFE_DEVINFO_MD5) {
 		printf(" md5");
-		crypto_register(sc->sc_cid, CRYPTO_MD5_HMAC, 0, 0,
-			safe_newsession, safe_freesession, safe_process, sc);
+		crypto_register(sc->sc_cid, CRYPTO_MD5_HMAC, 0, 0);
 	}
 	if (devinfo & SAFE_DEVINFO_SHA1) {
 		printf(" sha1");
-		crypto_register(sc->sc_cid, CRYPTO_SHA1_HMAC, 0, 0,
-			safe_newsession, safe_freesession, safe_process, sc);
+		crypto_register(sc->sc_cid, CRYPTO_SHA1_HMAC, 0, 0);
 	}
 	printf(" null");
-	crypto_register(sc->sc_cid, CRYPTO_NULL_CBC, 0, 0,
-		safe_newsession, safe_freesession, safe_process, sc);
-	crypto_register(sc->sc_cid, CRYPTO_NULL_HMAC, 0, 0,
-		safe_newsession, safe_freesession, safe_process, sc);
+	crypto_register(sc->sc_cid, CRYPTO_NULL_CBC, 0, 0);
+	crypto_register(sc->sc_cid, CRYPTO_NULL_HMAC, 0, 0);
 	/* XXX other supported algorithms */
 	printf("\n");
 
@@ -710,10 +710,10 @@ safe_setup_mackey(struct safe_session *ses, int algo, caddr_t key, int klen)
  * id on successful allocation.
  */
 static int
-safe_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
+safe_newsession(device_t dev, u_int32_t *sidp, struct cryptoini *cri)
 {
+	struct safe_softc *sc = device_get_softc(dev);
 	struct cryptoini *c, *encini = NULL, *macini = NULL;
-	struct safe_softc *sc = arg;
 	struct safe_session *ses = NULL;
 	int sesn;
 
@@ -826,9 +826,9 @@ safe_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
  * Deallocate a session.
  */
 static int
-safe_freesession(void *arg, u_int64_t tid)
+safe_freesession(device_t dev, u_int64_t tid)
 {
-	struct safe_softc *sc = arg;
+	struct safe_softc *sc = device_get_softc(dev);
 	int session, ret;
 	u_int32_t sid = ((u_int32_t) tid) & 0xffffffff;
 
@@ -845,38 +845,24 @@ safe_freesession(void *arg, u_int64_t tid)
 }
 
 static void
-safe_op_cb(struct safe_operand *op, bus_dma_segment_t *seg, int nsegs,
-    int error)
+safe_op_cb(void *arg, bus_dma_segment_t *seg, int nsegs, bus_size_t mapsize, int error)
 {
+	struct safe_operand *op = arg;
 
 	DPRINTF(("%s: mapsize %u nsegs %d error %d\n", __func__,
-		(u_int)op->mapsize, nsegs, error));
+		(u_int) mapsize, nsegs, error));
 	if (error != 0)
 		return;
+	op->mapsize = mapsize;
 	op->nsegs = nsegs;
 	bcopy(seg, op->segs, nsegs * sizeof (seg[0]));
 }
 
-static void
-safe_op_cb1(void *arg, bus_dma_segment_t *seg, int nsegs, int error)
-{
-
-	safe_op_cb(arg, seg, nsegs, error);
-}
-
-static void
-safe_op_cb2(void *arg, bus_dma_segment_t *seg, int nsegs,
-    bus_size_t mapsize __unused, int error)
-{
-
-	safe_op_cb(arg, seg, nsegs, error);
-}
-
 static int
-safe_process(void *arg, struct cryptop *crp, int hint)
+safe_process(device_t dev, struct cryptop *crp, int hint)
 {
+	struct safe_softc *sc = device_get_softc(dev);
 	int err = 0, i, nicealign, uniform;
-	struct safe_softc *sc = arg;
 	struct cryptodesc *crd1, *crd2, *maccrd, *enccrd;
 	int bypass, oplen, ivsize;
 	caddr_t iv;
@@ -912,8 +898,6 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 
 	re->re_crp = crp;
 	re->re_sesn = SAFE_SESSION(crp->crp_sid);
-	re->re_src.mapsize = crp->crp_ilen;
-	re->re_dst.mapsize = crp->crp_ilen;
 
 	if (crp->crp_flags & CRYPTO_F_IMBUF) {
 		re->re_src_m = (struct mbuf *)crp->crp_buf;
@@ -922,8 +906,9 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 		re->re_src_io = (struct uio *)crp->crp_buf;
 		re->re_dst_io = (struct uio *)crp->crp_buf;
 	} else {
-		re->re_src_buf = (void *)crp->crp_buf;
-		re->re_dst_buf = (void *)crp->crp_buf;
+		safestats.st_badflags++;
+		err = EINVAL;
+		goto errout;	/* XXX we don't handle contiguous blocks! */
 	}
 
 	sa = &re->re_sa;
@@ -1184,22 +1169,25 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 		goto errout;
 	}
 	if (crp->crp_flags & CRYPTO_F_IMBUF) {
-		err = bus_dmamap_load_mbuf(sc->sc_srcdmat, re->re_src_map,
-		    re->re_src_m, safe_op_cb2, &re->re_src, BUS_DMA_NOWAIT);
+		if (bus_dmamap_load_mbuf(sc->sc_srcdmat, re->re_src_map,
+		    re->re_src_m, safe_op_cb,
+		    &re->re_src, BUS_DMA_NOWAIT) != 0) {
+			bus_dmamap_destroy(sc->sc_srcdmat, re->re_src_map);
+			re->re_src_map = NULL;
+			safestats.st_noload++;
+			err = ENOMEM;
+			goto errout;
+		}
 	} else if (crp->crp_flags & CRYPTO_F_IOV) {
-		err = bus_dmamap_load_uio(sc->sc_srcdmat, re->re_src_map,
-		    re->re_src_io, safe_op_cb2, &re->re_src, BUS_DMA_NOWAIT);
-	} else {
-		err = bus_dmamap_load(sc->sc_srcdmat, re->re_src_map,
-		    re->re_src_buf, crp->crp_ilen, safe_op_cb1, &re->re_src,
-		    BUS_DMA_NOWAIT);
-	}
-	if (err != 0) {
-		bus_dmamap_destroy(sc->sc_srcdmat, re->re_src_map);
-		re->re_src_map = NULL;
-		safestats.st_noload++;
-		err = ENOMEM;
-		goto errout;
+		if (bus_dmamap_load_uio(sc->sc_srcdmat, re->re_src_map,
+		    re->re_src_io, safe_op_cb,
+		    &re->re_src, BUS_DMA_NOWAIT) != 0) {
+			bus_dmamap_destroy(sc->sc_srcdmat, re->re_src_map);
+			re->re_src_map = NULL;
+			safestats.st_noload++;
+			err = ENOMEM;
+			goto errout;
+		}
 	}
 	nicealign = safe_dmamap_aligned(&re->re_src);
 	uniform = safe_dmamap_uniform(&re->re_src);
@@ -1257,7 +1245,7 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 				}
 				if (bus_dmamap_load_uio(sc->sc_dstdmat,
 				    re->re_dst_map, re->re_dst_io,
-				    safe_op_cb2, &re->re_dst,
+				    safe_op_cb, &re->re_dst,
 				    BUS_DMA_NOWAIT) != 0) {
 					bus_dmamap_destroy(sc->sc_dstdmat,
 						re->re_dst_map);
@@ -1311,7 +1299,7 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 				}
 				if (bus_dmamap_load_mbuf(sc->sc_dstdmat,
 				    re->re_dst_map, re->re_dst_m,
-				    safe_op_cb2, &re->re_dst,
+				    safe_op_cb, &re->re_dst,
 				    BUS_DMA_NOWAIT) != 0) {
 					bus_dmamap_destroy(sc->sc_dstdmat,
 						re->re_dst_map);
@@ -1407,7 +1395,7 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 				}
 				if (bus_dmamap_load_mbuf(sc->sc_dstdmat,
 				    re->re_dst_map, re->re_dst_m,
-				    safe_op_cb2, &re->re_dst,
+				    safe_op_cb, &re->re_dst,
 				    BUS_DMA_NOWAIT) != 0) {
 					bus_dmamap_destroy(sc->sc_dstdmat,
 					re->re_dst_map);
@@ -1435,33 +1423,9 @@ safe_process(void *arg, struct cryptop *crp, int hint)
 				}
 			}
 		} else {
-			KASSERT(nicealign, ("no nicealign"));
-			if (uniform != 1) {
-				/*
-				 * Source is not suitable for direct use as
-				 * the destination.  Create a new scatter/gather
-				 * list based on the destination requirements
-				 * and check if that's ok.
-				 */
-				if (bus_dmamap_create(sc->sc_dstdmat,
-				    BUS_DMA_NOWAIT, &re->re_dst_map)) {
-					safestats.st_nomap++;
-					err = ENOMEM;
-					goto errout;
-				}
-				if (bus_dmamap_load(sc->sc_dstdmat,
-				    re->re_dst_map, re->re_dst_buf,
-				    crp->crp_ilen, safe_op_cb1, &re->re_dst,
-				    BUS_DMA_NOWAIT) != 0) {
-					bus_dmamap_destroy(sc->sc_dstdmat,
-						re->re_dst_map);
-					re->re_dst_map = NULL;
-					safestats.st_noload++;
-					err = ENOMEM;
-					goto errout;
-				}
-			} else
-				re->re_dst = re->re_src;
+			safestats.st_badflags++;
+			err = EINVAL;
+			goto errout;
 		}
 
 		if (re->re_dst.nsegs > 1) {
