@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/safe/safevar.h,v 1.1.12.1 2006/09/04 15:18:50 pjd Exp $
+ * $FreeBSD: src/sys/dev/safe/safevar.h,v 1.2 2006/05/17 18:34:26 pjd Exp $
  */
 #ifndef _SAFE_SAFEVAR_H_
 #define	_SAFE_SAFEVAR_H_
@@ -80,7 +80,6 @@ struct safe_operand {
 	union {
 		struct mbuf *m;
 		struct uio *io;
-		void *buf;
 	} u;
 	bus_dmamap_t		map;
 	bus_size_t		mapsize;
@@ -121,7 +120,6 @@ struct safe_ringentry {
 
 #define	re_src_m	re_src.u.m
 #define	re_src_io	re_src.u.io
-#define	re_src_buf	re_src.u.buf
 #define	re_src_map	re_src.map
 #define	re_src_nsegs	re_src.nsegs
 #define	re_src_segs	re_src.segs
@@ -129,7 +127,6 @@ struct safe_ringentry {
 
 #define	re_dst_m	re_dst.u.m
 #define	re_dst_io	re_dst.u.io
-#define	re_dst_buf	re_dst.u.buf
 #define	re_dst_map	re_dst.map
 #define	re_dst_nsegs	re_dst.nsegs
 #define	re_dst_segs	re_dst.segs
