@@ -1,7 +1,7 @@
 /*-
  * Copyright (c) 1999 Cameron Grant <cg@freebsd.org>
- * Copyright Luigi Rizzo, 1997,1998
- * Copyright by Hannu Savolainen 1994, 1995
+ * Copyright (c) 1997,1998 Luigi Rizzo
+ * Copyright (c) 1994,1995 Hannu Savolainen
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 
 #include "mixer_if.h"
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/ad1816.c,v 1.37.2.2 2006/04/04 17:23:24 ariff Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/ad1816.c,v 1.45 2007/06/17 06:10:40 ariff Exp $");
 
 struct ad1816_info;
 
@@ -318,7 +318,7 @@ ad1816chan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channe
 	ch->parent = ad1816;
 	ch->channel = c;
 	ch->buffer = b;
-	if (sndbuf_alloc(ch->buffer, ad1816->parent_dmat, ad1816->bufsize) != 0)
+	if (sndbuf_alloc(ch->buffer, ad1816->parent_dmat, 0, ad1816->bufsize) != 0)
 		return NULL;
 	return ch;
 }
@@ -411,7 +411,7 @@ ad1816chan_trigger(kobj_t obj, void *data, int go)
     	struct ad1816_info *ad1816 = ch->parent;
     	int wr, reg;
 
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD)
+	if (!PCMTRIG_COMMON(go))
 		return 0;
 
 	sndbuf_dma(ch->buffer, go);
@@ -598,10 +598,9 @@ ad1816_attach(device_t dev)
 	struct ad1816_info *ad1816;
     	char status[SND_STATUSLEN], status2[SND_STATUSLEN];
 
-	ad1816 = (struct ad1816_info *)malloc(sizeof *ad1816, M_DEVBUF, M_NOWAIT | M_ZERO);
-	if (!ad1816) return ENXIO;
-
-	ad1816->lock = snd_mtxcreate(device_get_nameunit(dev), "sound softc");
+	ad1816 = malloc(sizeof(*ad1816), M_DEVBUF, M_WAITOK | M_ZERO);
+	ad1816->lock = snd_mtxcreate(device_get_nameunit(dev),
+	    "snd_ad1816 softc");
 	ad1816->io_rid = 2;
 	ad1816->irq_rid = 0;
 	ad1816->drq1_rid = 0;
@@ -613,7 +612,8 @@ ad1816_attach(device_t dev)
     	if (mixer_init(dev, &ad1816mixer_class, ad1816)) goto no;
 
 	snd_setup_intr(dev, ad1816->irq, 0, ad1816_intr, ad1816, &ad1816->ih);
-    	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+    	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+			/*boundary*/0,
 			/*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
 			/*highaddr*/BUS_SPACE_MAXADDR,
 			/*filter*/NULL, /*filterarg*/NULL,
