@@ -41,11 +41,8 @@
 
 #include <isa/isavar.h>
 #include <isa/isa_common.h>
-#ifdef __alpha__		/* XXX workaround a stupid warning */
-#include <alpha/isa/isavar.h>
-#endif
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/gusc.c,v 1.16 2005/01/06 01:43:17 imp Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/gusc.c,v 1.19 2007/02/23 19:40:13 ariff Exp $");
 
 #define LOGICALID_NOPNP 0
 #define LOGICALID_PCM   0x0000561e
@@ -319,7 +316,7 @@ gusc_attach(device_t dev)
 	}
 
 	if (scp->irq != NULL)
-		bus_setup_intr(dev, scp->irq, INTR_TYPE_AV, gusc_intr, scp, &ih);
+		snd_setup_intr(dev, scp->irq, 0, gusc_intr, scp, &ih);
 	bus_generic_attach(dev);
 
 	return (0);
@@ -421,12 +418,21 @@ gusc_release_resource(device_t bus, device_t child, int type, int rid,
 }
 
 static int
-gusc_setup_intr(device_t dev, device_t child, struct resource *irq,
-		int flags, driver_intr_t *intr, void *arg, void **cookiep)
+gusc_setup_intr(device_t dev, device_t child, struct resource *irq, int flags,
+#if __FreeBSD_version >= 700031
+		driver_filter_t *filter,
+#endif
+		driver_intr_t *intr, void *arg, void **cookiep)
 {
 	sc_p scp = (sc_p)device_get_softc(dev);
 	devclass_t devclass;
 
+#if __FreeBSD_version >= 700031
+	if (filter != NULL) {
+		printf("gusc.c: we cannot use a filter here\n");
+		return (EINVAL);
+	}
+#endif
 	devclass = device_get_devclass(child);
 	if (strcmp(devclass_get_name(devclass), "midi") == 0) {
 		scp->midi_intr.intr = intr;
@@ -437,8 +443,11 @@ gusc_setup_intr(device_t dev, device_t child, struct resource *irq,
 		scp->pcm_intr.arg = arg;
 		return 0;
 	}
-	return bus_generic_setup_intr(dev, child, irq, flags, intr,
-				      arg, cookiep);
+	return bus_generic_setup_intr(dev, child, irq, flags,
+#if __FreeBSD_version >= 700031
+				filter,
+#endif
+				intr, arg, cookiep);
 }
 
 static device_t
