@@ -1,8 +1,8 @@
 /*-
  * Copyright (c) 2001 George Reid <greid@ukug.uk.freebsd.org>
  * Copyright (c) 1999 Cameron Grant <cg@freebsd.org>
- * Copyright Luigi Rizzo, 1997,1998
- * Copyright by Hannu Savolainen 1994, 1995
+ * Copyright (c) 1997,1998 Luigi Rizzo
+ * Copyright (c) 1994,1995 Hannu Savolainen
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,7 @@
 
 #include <dev/sound/pcm/sound.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/mss.c,v 1.95.2.3 2006/04/04 17:30:59 ariff Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/mss.c,v 1.112 2007/06/17 06:10:40 ariff Exp $");
 
 /* board-specific include files */
 #include <dev/sound/isa/mss.h>
@@ -1152,7 +1152,7 @@ msschan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *
 	ch->channel = c;
 	ch->buffer = b;
 	ch->dir = dir;
-	if (sndbuf_alloc(ch->buffer, mss->parent_dmat, mss->bufsize) != 0)
+	if (sndbuf_alloc(ch->buffer, mss->parent_dmat, 0, mss->bufsize) != 0)
 		return NULL;
 	sndbuf_dmasetup(ch->buffer, (dir == PCMDIR_PLAY)? mss->drq1 : mss->drq2);
 	return ch;
@@ -1201,7 +1201,7 @@ msschan_trigger(kobj_t obj, void *data, int go)
 	struct mss_chinfo *ch = data;
 	struct mss_info *mss = ch->parent;
 
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD)
+	if (!PCMTRIG_COMMON(go))
 		return 0;
 
 	sndbuf_dma(ch->buffer, go);
@@ -1320,7 +1320,7 @@ mss_probe(device_t dev)
 		goto mss_probe_end;
     	}
     	tmp &= 0x3f;
-    	if (!(tmp == 0x04 || tmp == 0x0f || tmp == 0x00)) {
+    	if (!(tmp == 0x04 || tmp == 0x0f || tmp == 0x00 || tmp == 0x05)) {
 		BVDDB(printf("No MSS signature detected on port 0x%lx (0x%x)\n",
 		     	rman_get_start(mss->io_base), tmpx));
 		goto no;
@@ -1696,7 +1696,7 @@ mss_doattach(device_t dev, struct mss_info *mss)
     	int pdma, rdma, flags = device_get_flags(dev);
     	char status[SND_STATUSLEN], status2[SND_STATUSLEN];
 
-	mss->lock = snd_mtxcreate(device_get_nameunit(dev), "sound softc");
+	mss->lock = snd_mtxcreate(device_get_nameunit(dev), "snd_mss softc");
 	mss->bufsize = pcm_getbuffersize(dev, 4096, MSS_DEFAULT_BUFSZ, 65536);
     	if (!mss_alloc_resources(mss, dev)) goto no;
     	mss_init(mss, dev);
@@ -1744,7 +1744,8 @@ mss_doattach(device_t dev, struct mss_info *mss)
     	}
     	if (pdma == rdma)
 		pcm_setflags(dev, pcm_getflags(dev) | SD_F_SIMPLEX);
-    	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+    	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+			/*boundary*/0,
 			/*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
 			/*highaddr*/BUS_SPACE_MAXADDR,
 			/*filter*/NULL, /*filterarg*/NULL,
@@ -1978,10 +1979,7 @@ pnpmss_attach(device_t dev)
 {
 	struct mss_info *mss;
 
-	mss = (struct mss_info *)malloc(sizeof *mss, M_DEVBUF, M_NOWAIT | M_ZERO);
-	if (!mss)
-	    return ENXIO;
-
+	mss = malloc(sizeof(*mss), M_DEVBUF, M_WAITOK | M_ZERO);
 	mss->io_rid = 0;
 	mss->conf_rid = -1;
 	mss->irq_rid = 0;
