@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 1999 Cameron Grant <cg@freebsd.org>
- * Copyright 1997,1998 Luigi Rizzo.
+ * Copyright (c) 1997,1998 Luigi Rizzo
  *
  * Derived from files in the Voxware 3.5 distribution,
  * Copyright by Hannu Savolainen 1994, under the same copyright
@@ -38,7 +38,7 @@
 
 #include "mixer_if.h"
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/sb8.c,v 1.79.2.1 2005/12/30 19:55:53 netchild Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/sb8.c,v 1.86 2007/06/17 06:10:41 ariff Exp $");
 
 #define SB_DEFAULT_BUFSZ	4096
 
@@ -599,7 +599,7 @@ sbchan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *c
 	ch->channel = c;
 	ch->dir = dir;
 	ch->buffer = b;
-	if (sndbuf_alloc(ch->buffer, sb->parent_dmat, sb->bufsize) != 0)
+	if (sndbuf_alloc(ch->buffer, sb->parent_dmat, 0, sb->bufsize) != 0)
 		return NULL;
 	sndbuf_dmasetup(ch->buffer, sb->drq);
 	return ch;
@@ -637,7 +637,7 @@ sbchan_trigger(kobj_t obj, void *data, int go)
 {
 	struct sb_chinfo *ch = data;
 
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD)
+	if (!PCMTRIG_COMMON(go))
 		return 0;
 
 	sndbuf_dma(ch->buffer, go);
@@ -714,10 +714,7 @@ sb_attach(device_t dev)
     	char status[SND_STATUSLEN];
 	uintptr_t ver;
 
-    	sb = (struct sb_info *)malloc(sizeof *sb, M_DEVBUF, M_NOWAIT | M_ZERO);
-    	if (!sb)
-		return ENXIO;
-
+    	sb = malloc(sizeof(*sb), M_DEVBUF, M_WAITOK | M_ZERO);
 	sb->parent_dev = device_get_parent(dev);
 	BUS_READ_IVAR(device_get_parent(dev), dev, 1, &ver);
 	sb->bd_id = ver & 0x0000ffff;
@@ -735,7 +732,8 @@ sb_attach(device_t dev)
 
 	pcm_setflags(dev, pcm_getflags(dev) | SD_F_SIMPLEX);
 
-    	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+    	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+			/*boundary*/0,
 			/*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
 			/*highaddr*/BUS_SPACE_MAXADDR,
 			/*filter*/NULL, /*filterarg*/NULL,
