@@ -30,7 +30,7 @@
 
 #include <isa/isavar.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/sbc.c,v 1.44.2.1 2005/12/30 19:55:53 netchild Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/isa/sbc.c,v 1.48 2007/03/15 16:41:25 ariff Exp $");
 
 #define IO_MAX	3
 #define IRQ_MAX	1
@@ -80,8 +80,12 @@ static struct resource *sbc_alloc_resource(device_t bus, device_t child, int typ
 static int sbc_release_resource(device_t bus, device_t child, int type, int rid,
 				struct resource *r);
 static int sbc_setup_intr(device_t dev, device_t child, struct resource *irq,
-   	       int flags, driver_intr_t *intr, void *arg,
-   	       void **cookiep);
+   	       int flags,
+#if __FreeBSD_version >= 700031
+	       driver_filter_t *filter,
+#endif
+	       driver_intr_t *intr, 
+   	       void *arg, void **cookiep);
 static int sbc_teardown_intr(device_t dev, device_t child, struct resource *irq,
   		  void *cookie);
 
@@ -116,7 +120,8 @@ static void sb_setmixer(struct resource *io, u_int port, u_int value);
 static void
 sbc_lockinit(struct sbc_softc *scp)
 {
-	scp->lock = snd_mtxcreate(device_get_nameunit(scp->dev), "sound softc");
+	scp->lock = snd_mtxcreate(device_get_nameunit(scp->dev),
+	    "snd_sbc softc");
 }
 
 static void
@@ -502,14 +507,23 @@ sbc_intr(void *p)
 }
 
 static int
-sbc_setup_intr(device_t dev, device_t child, struct resource *irq,
-   	       int flags, driver_intr_t *intr, void *arg,
-   	       void **cookiep)
+sbc_setup_intr(device_t dev, device_t child, struct resource *irq, int flags,
+#if __FreeBSD_version >= 700031
+   	       driver_filter_t *filter,
+#endif
+	       driver_intr_t *intr, 
+   	       void *arg, void **cookiep)
 {
 	struct sbc_softc *scp = device_get_softc(dev);
 	struct sbc_ihl *ihl = NULL;
 	int i, ret;
 
+#if __FreeBSD_version >= 700031
+	if (filter != NULL) {
+		printf("sbc.c: we cannot use a filter here\n");
+		return (EINVAL);
+	}
+#endif
 	sbc_lock(scp);
 	i = 0;
 	while (i < IRQ_MAX) {
