@@ -42,7 +42,7 @@
 
 #include "mixer_if.h"
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/als4000.c,v 1.18.2.1 2005/12/30 19:55:53 netchild Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/als4000.c,v 1.27 2007/06/17 06:10:41 ariff Exp $");
 
 /* Debugging macro's */
 #undef DEB
@@ -221,7 +221,7 @@ alschan_init(kobj_t obj, void *devinfo,
 	ch->buffer = b;
 	snd_mtxunlock(sc->lock);
 
-	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, sc->bufsz) != 0)
+	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, 0, sc->bufsz) != 0)
 		return NULL;
 
 	return ch;
@@ -391,14 +391,20 @@ alspchan_trigger(kobj_t obj, void *data, int go)
 	struct	sc_chinfo *ch = data;
 	struct sc_info *sc = ch->parent;
 
+	if (!PCMTRIG_COMMON(go))
+		return 0;
+
 	snd_mtxlock(sc->lock);
 	switch(go) {
 	case PCMTRIG_START:
 		als_playback_start(ch);
 		break;
+	case PCMTRIG_STOP:
 	case PCMTRIG_ABORT:
 		als_playback_stop(ch);
 		break;
+	default:
+		break;
 	}
 	snd_mtxunlock(sc->lock);
 	return 0;
@@ -489,6 +495,7 @@ alsrchan_trigger(kobj_t obj, void *data, int go)
 	case PCMTRIG_START:
 		als_capture_start(ch);
 		break;
+	case PCMTRIG_STOP:
 	case PCMTRIG_ABORT:
 		als_capture_stop(ch);
 		break;
@@ -773,7 +780,7 @@ als_resource_grab(device_t dev, struct sc_info *sc)
 
 	sc->bufsz = pcm_getbuffersize(dev, 4096, ALS_DEFAULT_BUFSZ, 65536);
 
-	if (bus_dma_tag_create(/*parent*/NULL,
+	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev),
 			       /*alignment*/2, /*boundary*/0,
 			       /*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
 			       /*highaddr*/BUS_SPACE_MAXADDR,
@@ -798,12 +805,8 @@ als_pci_attach(device_t dev)
 	u_int32_t data;
 	char status[SND_STATUSLEN];
 
-	if ((sc = malloc(sizeof(*sc), M_DEVBUF, M_NOWAIT | M_ZERO)) == NULL) {
-		device_printf(dev, "cannot allocate softc\n");
-		return ENXIO;
-	}
-
-	sc->lock = snd_mtxcreate(device_get_nameunit(dev), "sound softc");
+	sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
+	sc->lock = snd_mtxcreate(device_get_nameunit(dev), "snd_als4000 softc");
 	sc->dev = dev;
 
 	data = pci_read_config(dev, PCIR_COMMAND, 2);
