@@ -37,7 +37,7 @@
 
 #include <dev/sound/pci/cs4281.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/cs4281.c,v 1.20.2.1 2005/01/30 01:00:03 imp Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/cs4281.c,v 1.26 2007/06/17 06:10:41 ariff Exp $");
 
 #define CS4281_DEFAULT_BUFSZ 16384
 
@@ -314,7 +314,7 @@ cs4281chan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channe
     struct sc_chinfo *ch = (dir == PCMDIR_PLAY) ? &sc->pch : &sc->rch;
 
     ch->buffer = b;
-    if (sndbuf_alloc(ch->buffer, sc->parent_dmat, sc->bufsz) != 0) {
+    if (sndbuf_alloc(ch->buffer, sc->parent_dmat, 0, sc->bufsz) != 0) {
 	return NULL;
     }
     ch->parent = sc;
@@ -425,6 +425,7 @@ cs4281chan_trigger(kobj_t obj, void *data, int go)
 	adcdac_prog(ch);
 	adcdac_go(ch, 1);
 	break;
+    case PCMTRIG_STOP:
     case PCMTRIG_ABORT:
 	adcdac_go(ch, 0);
 	break;
@@ -742,7 +743,7 @@ cs4281_pci_probe(device_t dev)
 
     if (s)
 	device_set_desc(dev, s);
-    return s ? 0 : ENXIO;
+    return s ? BUS_PROBE_DEFAULT : ENXIO;
 }
 
 static int
@@ -753,11 +754,7 @@ cs4281_pci_attach(device_t dev)
     u_int32_t data;
     char status[SND_STATUSLEN];
 
-    if ((sc = malloc(sizeof(*sc), M_DEVBUF, M_NOWAIT | M_ZERO)) == NULL) {
-	device_printf(dev, "cannot allocate softc\n");
-	return ENXIO;
-    }
-
+    sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
     sc->dev = dev;
     sc->type = pci_get_devid(dev);
 
@@ -824,7 +821,8 @@ cs4281_pci_attach(device_t dev)
 
     sc->bufsz = pcm_getbuffersize(dev, 4096, CS4281_DEFAULT_BUFSZ, 65536);
 
-    if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+    if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+			   /*boundary*/0,
 			   /*lowaddr*/BUS_SPACE_MAXADDR_32BIT,
 			   /*highaddr*/BUS_SPACE_MAXADDR,
 			   /*filter*/NULL, /*filterarg*/NULL,
