@@ -48,7 +48,7 @@
 
 #include <gnu/dev/sound/pci/csaimg.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/csa.c,v 1.33.2.1 2005/12/30 19:55:53 netchild Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/csa.c,v 1.37 2007/03/17 19:37:09 ariff Exp $");
 
 /* This is the pci device id. */
 #define CS4610_PCI_ID 0x60011013
@@ -82,7 +82,10 @@ static int csa_release_resource(device_t bus, device_t child, int type, int rid,
 				   struct resource *r);
 static int csa_setup_intr(device_t bus, device_t child,
 			  struct resource *irq, int flags,
-			  driver_intr_t *intr, void *arg, void **cookiep);
+#if __FreeBSD_version >= 700031
+			  driver_filter_t *filter,
+#endif
+			  driver_intr_t *intr,  void *arg, void **cookiep);
 static int csa_teardown_intr(device_t bus, device_t child,
 			     struct resource *irq, void *cookie);
 static driver_intr_t csa_intr;
@@ -336,23 +339,31 @@ csa_detach(device_t dev)
 {
 	csa_res *resp;
 	sc_p scp;
+	struct sndcard_func *func;
 	int err;
 
 	scp = device_get_softc(dev);
 	resp = &scp->res;
 
-	err = 0;
-	if (scp->midi != NULL)
+	if (scp->midi != NULL) {
+		func = device_get_ivars(scp->midi);
 		err = device_delete_child(dev, scp->midi);
-	if (err)
-		return err;
-	scp->midi = NULL;
+		if (err != 0)
+			return err;
+		if (func != NULL)
+			free(func, M_DEVBUF);
+		scp->midi = NULL;
+	}
 
-	if (scp->pcm != NULL)
+	if (scp->pcm != NULL) {
+		func = device_get_ivars(scp->pcm);
 		err = device_delete_child(dev, scp->pcm);
-	if (err)
-		return err;
-	scp->pcm = NULL;
+		if (err != 0)
+			return err;
+		if (func != NULL)
+			free(func, M_DEVBUF);
+		scp->pcm = NULL;
+	}
 
 	bus_teardown_intr(dev, resp->irq, scp->ih);
 	bus_release_resource(dev, SYS_RES_IRQ, resp->irq_rid, resp->irq);
@@ -439,12 +450,21 @@ csa_release_resource(device_t bus, device_t child, int type, int rid,
 static int
 csa_setup_intr(device_t bus, device_t child,
 	       struct resource *irq, int flags,
+#if __FreeBSD_version >= 700031
+	       driver_filter_t *filter,
+#endif
 	       driver_intr_t *intr, void *arg, void **cookiep)
 {
 	sc_p scp;
 	csa_res *resp;
 	struct sndcard_func *func;
 
+#if __FreeBSD_version >= 700031
+	if (filter != NULL) {
+		printf("ata-csa.c: we cannot use a filter here\n");
+		return (EINVAL);
+	}
+#endif
 	scp = device_get_softc(bus);
 	resp = &scp->res;
 
