@@ -28,14 +28,16 @@
 
 #include <dev/sound/pcm/sound.h>
 #include <dev/sound/pcm/ac97.h>
-#include <gnu/dev/sound/pci/emu10k1.h>
 #include "emu10k1-alsa%diked.h"
 
 #include <dev/pci/pcireg.h>
 #include <dev/pci/pcivar.h>
 #include <sys/queue.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/emu10k1.c,v 1.55.2.1 2005/12/30 19:55:53 netchild Exp $");
+#include <dev/sound/midi/mpu401.h>
+#include "mpufoi_if.h"
+
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/emu10k1.c,v 1.69 2007/06/17 06:10:42 ariff Exp $");
 
 /* -------------------------------------------------------------------- */
 
@@ -85,7 +87,7 @@ struct emu_mem {
 
 struct emu_voice {
 	int vnum;
-	int b16:1, stereo:1, busy:1, running:1, ismaster:1;
+	unsigned int b16:1, stereo:1, busy:1, running:1, ismaster:1;
 	int speed;
 	int start, end, vol;
 	int fxrt1;	/* FX routing */
@@ -137,6 +139,9 @@ struct sc_info {
 	struct emu_voice voice[64];
 	struct sc_pchinfo pch[EMU_MAX_CHANS];
 	struct sc_rchinfo rch[3];
+	struct mpu401   *mpu;
+	mpu401_intr_t           *mpu_intr;
+	int mputx;
 };
 
 /* -------------------------------------------------------------------- */
@@ -272,7 +277,7 @@ emu_wrptr(struct sc_info *sc, int chn, int reg, u_int32_t data)
 static void
 emu_wrefx(struct sc_info *sc, unsigned int pc, unsigned int data)
 {
-	pc += sc->audigy ? AUDIGY_CODEBASE : MICROCODEBASE;
+	pc += sc->audigy ? A_MICROCODEBASE : MICROCODEBASE;
 	emu_wrptr(sc, 0, pc, data);
 }
 
@@ -790,7 +795,7 @@ emupchan_trigger(kobj_t obj, void *data, int go)
 	struct sc_pchinfo *ch = data;
 	struct sc_info *sc = ch->parent;
 
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD)
+	if (!PCMTRIG_COMMON(go))
 		return 0;
 
 	snd_mtxlock(sc->lock);
@@ -890,7 +895,7 @@ emurchan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b,
 		break;
 	}
 	sc->rnum++;
-	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, sc->bufsz) != 0)
+	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, 0, sc->bufsz) != 0)
 		return NULL;
 	else {
 		snd_mtxlock(sc->lock);
@@ -953,6 +958,9 @@ emurchan_trigger(kobj_t obj, void *data, int go)
 	struct sc_info *sc = ch->parent;
 	u_int32_t val, sz;
 
+	if (!PCMTRIG_COMMON(go))
+		return 0;
+
 	switch(sc->bufsz) {
 	case 4096:
 		sz = ADCBS_BUFSIZE_4096;
@@ -1059,8 +1067,65 @@ static kobj_method_t emurchan_methods[] = {
 };
 CHANNEL_DECLARE(emurchan);
 
+static unsigned char
+emu_mread(void *arg, struct sc_info *sc, int reg)
+{	
+	unsigned int d;
+
+	d = emu_rd(sc, 0x18 + reg, 1); 
+	return d;
+}
+
+static void
+emu_mwrite(void *arg, struct sc_info *sc, int reg, unsigned char b)
+{
+
+	emu_wr(sc, 0x18 + reg, b, 1);
+}
+
+static int
+emu_muninit(void *arg, struct sc_info *sc)
+{
+
+	snd_mtxlock(sc->lock);
+	sc->mpu_intr = 0;
+	snd_mtxunlock(sc->lock);
+
+	return 0;
+}
+
+static kobj_method_t emu_mpu_methods[] = {
+    	KOBJMETHOD(mpufoi_read,		emu_mread),
+    	KOBJMETHOD(mpufoi_write,	emu_mwrite),
+    	KOBJMETHOD(mpufoi_uninit,	emu_muninit),
+	{ 0, 0 }
+};
+
+static DEFINE_CLASS(emu_mpu, emu_mpu_methods, 0);
+
+static void
+emu_intr2(void *p)
+{
+	struct sc_info *sc = (struct sc_info *)p;
+
+	if (sc->mpu_intr)
+	    (sc->mpu_intr)(sc->mpu);
+}
+
+static void
+emu_midiattach(struct sc_info *sc)
+{
+	int i;
+
+	i = emu_rd(sc, INTE, 4);
+	i |= INTE_MIDIRXENABLE;
+	emu_wr(sc, INTE, i, 4);
+
+	sc->mpu = mpu401_init(&emu_mpu_class, sc, emu_intr2, &sc->mpu_intr);
+}
 /* -------------------------------------------------------------------- */
 /* The interrupt handler */
+
 static void
 emu_intr(void *data)
 {
@@ -1100,6 +1165,11 @@ emu_intr(void *data)
 #endif
 		}
 
+	    if (stat & IPR_MIDIRECVBUFEMPTY)
+		if (sc->mpu_intr) {
+		    (sc->mpu_intr)(sc->mpu);
+		    ack |= IPR_MIDIRECVBUFEMPTY | IPR_MIDITRANSBUFEMPTY;
+ 		}
 		if (stat & ~ack)
 			device_printf(sc->dev, "dodgy irq: %x (harmless)\n",
 			    stat & ~ack);
@@ -1312,12 +1382,12 @@ audigy_initefx(struct sc_info *sc)
 
 	/* Audigy 2 (EMU10K2) DSP Registers:
 	   FX Bus
-		0x000-0x00f : 16 registers (???)
+		0x000-0x00f : 16 registers (?)
 	   Input
 		0x040/0x041 : AC97 Codec (l/r)
 		0x042/0x043 : ADC, S/PDIF (l/r)
 		0x044/0x045 : Optical S/PDIF in (l/r)
-		0x046/0x047 : ???
+		0x046/0x047 : ?
 		0x048/0x049 : Line/Mic 2 (l/r)
 		0x04a/0x04b : RCA S/PDIF (l/r)
 		0x04c/0x04d : Aux 2 (l/r)
@@ -1328,11 +1398,11 @@ audigy_initefx(struct sc_info *sc)
 		0x066/0x067 : Digital Rear (l/r)
 		0x068/0x069 : Analog Front (l/r)
 		0x06a/0x06b : Analog Center/LFE
-		0x06c/0x06d : ???
+		0x06c/0x06d : ?
 		0x06e/0x06f : Analog Rear (l/r)
 		0x070/0x071 : AC97 Output (l/r)
-		0x072/0x073 : ???
-		0x074/0x075 : ???
+		0x072/0x073 : ?
+		0x074/0x075 : ?
 		0x076/0x077 : ADC Recording Buffer (l/r)
 	   Constants
 		0x0c0 - 0x0c4 = 0 - 4
@@ -1341,9 +1411,9 @@ audigy_initefx(struct sc_info *sc)
 		0x0cb = 0x10000000, 0x0cc = 0x20000000, 0x0cd = 0x40000000
 		0x0ce = 0x80000000, 0x0cf = 0x7fffffff, 0x0d0 = 0xffffffff
 		0x0d1 = 0xfffffffe, 0x0d2 = 0xc0000000, 0x0d3 = 0x41fbbcdc
-		0x0d4 = 0x5a7ef9db, 0x0d5 = 0x00100000, 0x0dc = 0x00000001 (???)
+		0x0d4 = 0x5a7ef9db, 0x0d5 = 0x00100000, 0x0dc = 0x00000001 (?)
 	   Temporary Values
-		0x0d6 : Accumulator (???)
+		0x0d6 : Accumulator (?)
 		0x0d7 : Condition Register
 		0x0d8 : Noise source
 		0x0d9 : Noise source
@@ -1560,10 +1630,10 @@ emu_initefx(struct sc_info *sc)
 	emu_addefxop(sc, iACC3, EXTOUT(EXTOUT_ALFE), C_00000000,
 			GPR(1), GPR(2), &pc);
 	/* Digital Center = GPR[0] + GPR[2] */
-	emu_addefxop(sc, iACC3, EXTOUT(EXTOUT_CENTER), C_00000000,
+	emu_addefxop(sc, iACC3, EXTOUT(EXTOUT_AC97_CENTER), C_00000000,
 			GPR(0), GPR(2), &pc);
 	/* Digital Sub = GPR[1] + GPR[2] */
-	emu_addefxop(sc, iACC3, EXTOUT(EXTOUT_LFE), C_00000000,
+	emu_addefxop(sc, iACC3, EXTOUT(EXTOUT_AC97_LFE), C_00000000,
 			GPR(1), GPR(2), &pc);
 
 	/* Headphones[l/r] = GPR[0/1] */
@@ -1871,6 +1941,8 @@ emu_uninit(struct sc_info *sc)
 	emu_free(sc, sc->mem.ptb_pages);
 	emu_free(sc, sc->mem.silent_page);
 
+	if(sc->mpu)
+	    mpu401_uninit(sc->mpu);
 	return 0;
 }
 
@@ -1900,7 +1972,7 @@ emu_pci_probe(device_t dev)
 	}
 
 	device_set_desc(dev, s);
-	return BUS_PROBE_DEFAULT;
+	return BUS_PROBE_LOW_PRIORITY;
 }
 
 static int
@@ -1912,12 +1984,8 @@ emu_pci_attach(device_t dev)
 	int i, gotmic;
 	char status[SND_STATUSLEN];
 
-	if ((sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO)) == NULL) {
-		device_printf(dev, "cannot allocate softc\n");
-		return ENXIO;
-	}
-
-	sc->lock = snd_mtxcreate(device_get_nameunit(dev), "sound softc");
+	sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
+	sc->lock = snd_mtxcreate(device_get_nameunit(dev), "snd_emu10k1 softc");
 	sc->dev = dev;
 	sc->type = pci_get_devid(dev);
 	sc->rev = pci_get_revid(dev);
@@ -1942,7 +2010,8 @@ emu_pci_attach(device_t dev)
 
 	sc->bufsz = pcm_getbuffersize(dev, 4096, EMU_DEFAULT_BUFSZ, 65536);
 
-	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+		/*boundary*/0,
 		/*lowaddr*/1 << 31, /* can only access 0-2gb */
 		/*highaddr*/BUS_SPACE_MAXADDR,
 		/*filter*/NULL, /*filterarg*/NULL,
@@ -1963,6 +2032,8 @@ emu_pci_attach(device_t dev)
 	gotmic = (ac97_getcaps(codec) & AC97_CAP_MICCHANNEL) ? 1 : 0;
 	if (mixer_init(dev, ac97_getmixerclass(), codec) == -1) goto bad;
 
+	emu_midiattach(sc);
+
 	i = 0;
 	sc->irq = bus_alloc_resource_any(dev, SYS_RES_IRQ, &i,
 	    RF_ACTIVE | RF_SHAREABLE);
@@ -2041,6 +2112,7 @@ DRIVER_MODULE(snd_emu10k1, pci, emu_driver, pcm_devclass, 0, 0);
 DRIVER_MODULE(snd_emu10k1, cardbus, emu_driver, pcm_devclass, 0, 0);
 MODULE_DEPEND(snd_emu10k1, sound, SOUND_MINVER, SOUND_PREFVER, SOUND_MAXVER);
 MODULE_VERSION(snd_emu10k1, 1);
+MODULE_DEPEND(snd_emu10k1, midi, 1, 1, 1);
 
 /* dummy driver to silence the joystick device */
 static int
