@@ -29,11 +29,11 @@
 #include <dev/pci/pcireg.h>
 #include <dev/pci/pcivar.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/fm801.c,v 1.27.2.1 2006/01/10 01:01:24 ariff Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/fm801.c,v 1.33 2007/06/17 06:10:42 ariff Exp $");
 
 #define PCI_VENDOR_FORTEMEDIA	0x1319
-#define PCI_DEVICE_FORTEMEDIA1	0x08011319
-#define PCI_DEVICE_FORTEMEDIA2	0x08021319	/* ??? have no idea what's this... */
+#define PCI_DEVICE_FORTEMEDIA1	0x08011319	/* Audio controller */
+#define PCI_DEVICE_FORTEMEDIA2	0x08021319	/* Joystick controller */
 
 #define FM_PCM_VOLUME           0x00
 #define FM_FM_VOLUME            0x02
@@ -334,7 +334,7 @@ fm801ch_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *
 	ch->channel = c;
 	ch->buffer = b;
 	ch->dir = dir;
-	if (sndbuf_alloc(ch->buffer, fm801->parent_dmat, fm801->bufsz) != 0)
+	if (sndbuf_alloc(ch->buffer, fm801->parent_dmat, 0, fm801->bufsz) != 0)
 		return NULL;
 	return (void *)ch;
 }
@@ -417,15 +417,16 @@ fm801ch_setblocksize(kobj_t obj, void *data, u_int32_t blocksize)
 	struct fm801_chinfo *ch = data;
 	struct fm801_info *fm801 = ch->parent;
 
-	if(ch->dir == PCMDIR_PLAY) {
-		if(fm801->play_flip) return fm801->play_blksize;
+	/*
+	 * Don't mind for play_flip, set the blocksize to the
+	 * desired values in any case - otherwise sound playback
+	 * breaks here.
+	 */
+	if(ch->dir == PCMDIR_PLAY)
 		fm801->play_blksize = blocksize;
-	}
 
-	if(ch->dir == PCMDIR_REC) {
-		if(fm801->rec_flip) return fm801->rec_blksize;
+	if(ch->dir == PCMDIR_REC)
 		fm801->rec_blksize = blocksize;
-	}
 
 	DPRINT("fm801ch_setblocksize %d (dir %d)\n",blocksize, ch->dir);
 
@@ -442,9 +443,8 @@ fm801ch_trigger(kobj_t obj, void *data, int go)
 
 	DPRINT("fm801ch_trigger go %d , ", go);
 
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD) {
+	if (!PCMTRIG_COMMON(go))
 		return 0;
-	}
 
 	if (ch->dir == PCMDIR_PLAY) {
 		if (go == PCMTRIG_START) {
@@ -575,11 +575,7 @@ fm801_pci_attach(device_t dev)
 	int 			mapped = 0;
 	char 			status[SND_STATUSLEN];
 
-	if ((fm801 = (struct fm801_info *)malloc(sizeof(*fm801), M_DEVBUF, M_NOWAIT | M_ZERO)) == NULL) {
-		device_printf(dev, "cannot allocate softc\n");
-		return ENXIO;
-	}
-
+	fm801 = malloc(sizeof(*fm801), M_DEVBUF, M_WAITOK | M_ZERO);
 	fm801->type = pci_get_devid(dev);
 
 	data = pci_read_config(dev, PCIR_COMMAND, 2);
@@ -630,7 +626,8 @@ fm801_pci_attach(device_t dev)
 		goto oops;
 	}
 
-	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+		/*boundary*/0,
 		/*lowaddr*/BUS_SPACE_MAXADDR_32BIT,
 		/*highaddr*/BUS_SPACE_MAXADDR,
 		/*filter*/NULL, /*filterarg*/NULL,
