@@ -32,7 +32,7 @@
 #include <dev/pci/pcireg.h>
 #include <dev/pci/pcivar.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/ich.c,v 1.53.2.12 2007/07/12 06:39:38 ariff Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/ich.c,v 1.78.2.1 2007/12/15 04:06:52 ariff Exp $");
 
 /* -------------------------------------------------------------------- */
 
@@ -92,19 +92,6 @@ SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/ich.c,v 1.53.2.12 2007/07/12 0
 #define ICH_DMA_NOCACHE		(1 << 4)
 #define ICH_HIGH_LATENCY	(1 << 5)
 
-#if defined(__i386__) || defined(__amd64__)
-#include <machine/specialreg.h>
-#define ICH_DMA_ATTR(sc, v, s, attr)	do {			\
-	vm_offset_t va = (vm_offset_t)(v);			\
-	vm_size_t sz = (vm_size_t)(s);				\
-	if ((sc) != NULL && ((sc)->flags & ICH_DMA_NOCACHE) &&	\
-	    va != 0 && sz != 0)					\
-		(void)pmap_change_attr(va, sz, (attr));	\
-} while(0)
-#else
-#define ICH_DMA_ATTR(...)
-#endif
-
 static const struct ich_type {
         uint16_t	vendor;
         uint16_t	devid;
@@ -424,12 +411,11 @@ ichchan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *
 		ch->spdreg = 0;
 
 	ICH_UNLOCK(sc);
-	if (sndbuf_alloc(ch->buffer, sc->chan_dmat, sc->bufsz) != 0)
+	if (sndbuf_alloc(ch->buffer, sc->chan_dmat,
+	    ((sc->flags & ICH_DMA_NOCACHE) ? BUS_DMA_NOCACHE : 0),
+	    sc->bufsz) != 0)
 		return (NULL);
 
-	ICH_DMA_ATTR(sc, sndbuf_getbuf(ch->buffer),
-	    sndbuf_getmaxsize(ch->buffer), PAT_UNCACHEABLE);
-
 	ICH_LOCK(sc);
 	ich_wr(sc, ch->regbase + ICH_REG_X_BDBAR, (uint32_t)(ch->desc_addr), 4);
 	ICH_UNLOCK(sc);
@@ -437,22 +423,6 @@ ichchan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *
 	return (ch);
 }
 
-static int
-ichchan_free(kobj_t obj, void *data)
-{
-	struct sc_chinfo *ch;
-	struct sc_info *sc;
-
-	ch = (struct sc_chinfo *)data;
-	sc = (ch != NULL) ? ch->parent : NULL;
-	if (ch != NULL && sc != NULL) {
-		ICH_DMA_ATTR(sc, sndbuf_getbuf(ch->buffer),
-		    sndbuf_getmaxsize(ch->buffer), PAT_WRITE_BACK);
-	}
-
-	return (1);
-}
-
 static int
 ichchan_setformat(kobj_t obj, void *data, uint32_t format)
 {
@@ -607,7 +577,6 @@ ichchan_getcaps(kobj_t obj, void *data)
 
 static kobj_method_t ichchan_methods[] = {
 	KOBJMETHOD(channel_init,		ichchan_init),
-	KOBJMETHOD(channel_free,		ichchan_free),
 	KOBJMETHOD(channel_setformat,		ichchan_setformat),
 	KOBJMETHOD(channel_setspeed,		ichchan_setspeed),
 	KOBJMETHOD(channel_setblocksize,	ichchan_setblocksize),
@@ -697,8 +666,11 @@ static int
 ich_initsys(struct sc_info* sc)
 {
 #ifdef SND_DYNSYSCTL
-	SYSCTL_ADD_INT(snd_sysctl_tree(sc->dev),
-		       SYSCTL_CHILDREN(snd_sysctl_tree_top(sc->dev)),
+	/* XXX: this should move to a device specific sysctl "dev.pcm.X.yyy"
+	   via device_get_sysctl_*() as discussed on multimedia@ in msg-id
+	   <861wujij2q.fsf@xps.des.no> */
+	SYSCTL_ADD_INT(device_get_sysctl_ctx(sc->dev),
+		       SYSCTL_CHILDREN(device_get_sysctl_tree(sc->dev)),
 		       OID_AUTO, "ac97rate", CTLFLAG_RW,
 		       &sc->ac97rate, 48000,
 		       "AC97 link rate (default = 48000)");
@@ -1008,6 +980,10 @@ ich_pci_attach(device_t dev)
 	    device_get_unit(dev), "fixedrate", &i) == 0 && i != 0)
 		sc->flags |= ICH_FIXED_RATE;
 
+	if (resource_int_value(device_get_name(dev),
+	    device_get_unit(dev), "micchannel_enabled", &i) == 0 && i != 0)
+		sc->hasmic = 1;
+
 	sc->irqid = 0;
 	sc->irq = bus_alloc_resource_any(dev, SYS_RES_IRQ, &sc->irqid,
 	    RF_ACTIVE | RF_SHAREABLE);
@@ -1054,7 +1030,8 @@ ich_pci_attach(device_t dev)
 	extcaps = ac97_getextcaps(sc->codec);
 	sc->hasvra = extcaps & AC97_EXTCAP_VRA;
 	sc->hasvrm = extcaps & AC97_EXTCAP_VRM;
-	sc->hasmic = ac97_getcaps(sc->codec) & AC97_CAP_MICCHANNEL;
+	sc->hasmic = (sc->hasmic != 0 &&
+	    (ac97_getcaps(sc->codec) & AC97_CAP_MICCHANNEL)) ? 1 : 0;
 	ac97_setextmode(sc->codec, sc->hasvra | sc->hasvrm);
 
 	sc->dtbl_size = sizeof(struct ich_desc) * ICH_DTBL_LENGTH *
