@@ -61,7 +61,7 @@
 #include <gnu/dev/sound/pci/maestro3_reg.h>
 #include <gnu/dev/sound/pci/maestro3_dsp.h>
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/maestro3.c,v 1.28.2.2 2007/03/12 02:03:25 ariff Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/maestro3.c,v 1.35 2007/06/17 06:10:42 ariff Exp $");
 
 /* -------------------------------------------------------------------- */
 
@@ -407,7 +407,7 @@ m3_pchan_init(kobj_t kobj, void *devinfo, struct snd_dbuf *b, struct pcm_channel
 	ch->fmt = AFMT_U8;
 	ch->spd = DSP_DEFAULT_SPEED;
 	M3_UNLOCK(sc); /* XXX */
-	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, sc->bufsz) != 0) {
+	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, 0, sc->bufsz) != 0) {
 		device_printf(sc->dev, "m3_pchan_init chn_allocbuf failed\n");
 		return (NULL);
 	}
@@ -573,6 +573,9 @@ m3_pchan_trigger(kobj_t kobj, void *chdata, int go)
 	struct sc_info *sc = ch->parent;
 	int ret;
 
+	if (!PCMTRIG_COMMON(go))
+		return (0);
+
 	M3_LOCK(sc);
 	ret = m3_pchan_trigger_locked(kobj, chdata, go);
 	M3_UNLOCK(sc);
@@ -756,7 +759,7 @@ m3_rchan_init(kobj_t kobj, void *devinfo, struct snd_dbuf *b, struct pcm_channel
 	ch->fmt = AFMT_U8;
 	ch->spd = DSP_DEFAULT_SPEED;
 	M3_UNLOCK(sc); /* XXX */
-	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, sc->bufsz) != 0) {
+	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, 0, sc->bufsz) != 0) {
 		device_printf(sc->dev, "m3_rchan_init chn_allocbuf failed\n");
 		return (NULL);
 	}
@@ -916,6 +919,9 @@ m3_rchan_trigger(kobj_t kobj, void *chdata, int go)
 	struct sc_info *sc = ch->parent;
 	int ret;
 
+	if (!PCMTRIG_COMMON(go))
+		return (0);
+
 	M3_LOCK(sc);
 	ret = m3_rchan_trigger_locked(kobj, chdata, go);
 	M3_UNLOCK(sc);
@@ -1259,20 +1265,11 @@ m3_pci_attach(device_t dev)
 
 	M3_DEBUG(CALL, ("m3_pci_attach\n"));
 
-	if ((sc = malloc(sizeof(*sc), M_DEVBUF, M_NOWAIT | M_ZERO)) == NULL) {
-		device_printf(dev, "cannot allocate softc\n");
-		return ENXIO;
-	}
-
+	sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
 	sc->dev = dev;
 	sc->type = pci_get_devid(dev);
 	sc->sc_lock = snd_mtxcreate(device_get_nameunit(dev),
-	    "sound softc");
-	if (sc->sc_lock == NULL) {
-		device_printf(dev, "cannot create mutex\n");
-		free(sc, M_DEVBUF);
-		return (ENXIO);
-	}
+	    "snd_maestro3 softc");
 	for (card = m3_card_types ; card->pci_id ; card++) {
 		if (sc->type == card->pci_id) {
 			sc->which = card->which;
@@ -1332,7 +1329,7 @@ m3_pci_attach(device_t dev)
 	    M3_BUFSIZE_MAX);
 
 	if (bus_dma_tag_create(
-	    NULL,		/* parent */
+	    bus_get_dma_tag(dev),	/* parent */
 	    2, 0,		/* alignment, boundary */
 	    M3_MAXADDR,		/* lowaddr */
 	    BUS_SPACE_MAXADDR,	/* highaddr */
@@ -1401,11 +1398,7 @@ m3_pci_attach(device_t dev)
 	/* Create the buffer for saving the card state during suspend */
 	len = sizeof(u_int16_t) * (REV_B_CODE_MEMORY_LENGTH +
 	    REV_B_DATA_MEMORY_LENGTH);
-	sc->savemem = (u_int16_t*)malloc(len, M_DEVBUF, M_NOWAIT | M_ZERO);
-	if (sc->savemem == NULL) {
-		device_printf(dev, "Failed to create suspend buffer\n");
-		goto bad;
-	}
+	sc->savemem = (u_int16_t*)malloc(len, M_DEVBUF, M_WAITOK | M_ZERO);
 
 	return 0;
 
