@@ -33,7 +33,7 @@
 
 #include "mixer_if.h"
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/solo.c,v 1.35.2.4 2006/07/13 01:53:54 yongari Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/solo.c,v 1.45 2007/06/17 06:10:42 ariff Exp $");
 
 #define SOLO_DEFAULT_BUFSZ 16384
 #define ABS(x) (((x) < 0)? -(x) : (x))
@@ -92,7 +92,8 @@ struct ess_info {
 	void		*ih;
     	bus_dma_tag_t parent_dmat;
 
-    	int simplex_dir, type, duplex:1, newspeed:1, dmasz[2];
+    	int simplex_dir, type, dmasz[2];
+	unsigned int duplex:1, newspeed:1;
 	unsigned int bufsz;
 
     	struct ess_chinfo pch, rch;
@@ -538,7 +539,7 @@ esschan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *
 	ch->channel = c;
 	ch->buffer = b;
 	ch->dir = dir;
-	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, sc->bufsz) != 0)
+	if (sndbuf_alloc(ch->buffer, sc->parent_dmat, 0, sc->bufsz) != 0)
 		return NULL;
 	ch->hwch = 1;
 	if ((dir == PCMDIR_PLAY) && (sc->duplex))
@@ -584,10 +585,11 @@ esschan_trigger(kobj_t obj, void *data, int go)
 	struct ess_chinfo *ch = data;
 	struct ess_info *sc = ch->parent;
 
-	DEB(printf("esschan_trigger: %d\n",go));
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD)
+	if (!PCMTRIG_COMMON(go))
 		return 0;
 
+	DEB(printf("esschan_trigger: %d\n",go));
+
 	ess_lock(sc);
 	switch (go) {
 	case PCMTRIG_START:
@@ -897,7 +899,7 @@ ess_alloc_resources(struct ess_info *sc, device_t dev)
 		RF_ACTIVE | RF_SHAREABLE);
 
 #if ESS18XX_MPSAFE == 1
-	sc->lock = snd_mtxcreate(device_get_nameunit(dev), "sound softc");
+	sc->lock = snd_mtxcreate(device_get_nameunit(dev), "snd_solo softc");
 
 	return (sc->irq && sc->io && sc->sb && sc->vc &&
 				sc->mpu && sc->gp && sc->lock)? 0 : ENXIO;
@@ -984,10 +986,7 @@ ess_attach(device_t dev)
 	u_int16_t ddma;
 	u_int32_t data;
 
-	sc = (struct ess_info *)malloc(sizeof *sc, M_DEVBUF, M_NOWAIT | M_ZERO);
-    	if (!sc)
-		return ENXIO;
-
+	sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
 	data = pci_read_config(dev, PCIR_COMMAND, 2);
 	data |= PCIM_CMD_PORTEN | PCIM_CMD_BUSMASTEREN;
 	pci_write_config(dev, PCIR_COMMAND, data, 2);
@@ -1030,9 +1029,9 @@ ess_attach(device_t dev)
 		pcm_setflags(dev, pcm_getflags(dev) | SD_F_SIMPLEX);
 
 #if 0
-    	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/65536, /*boundary*/0,
+    	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/65536, /*boundary*/0,
 #endif
-    	if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+    	if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2, /*boundary*/0,
 			/*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
 			/*highaddr*/BUS_SPACE_MAXADDR,
 			/*filter*/NULL, /*filterarg*/NULL,
