@@ -36,7 +36,7 @@
 
 #include "mixer_if.h"
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/vibes.c,v 1.19.2.1 2006/01/24 18:54:22 joel Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pci/vibes.c,v 1.26 2007/06/17 06:10:42 ariff Exp $");
 
 /* ------------------------------------------------------------------------- */
 /* Constants */
@@ -192,7 +192,7 @@ svchan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *c
 	ch->channel = c;
 	ch->dir = dir;
 
-	if (sndbuf_alloc(b, sc->parent_dmat, sc->bufsz) != 0) {
+	if (sndbuf_alloc(b, sc->parent_dmat, 0, sc->bufsz) != 0) {
 		DEB(printf("svchan_init failed\n"));
 		return NULL;
 	}
@@ -336,6 +336,7 @@ svrchan_trigger(kobj_t obj, void *data, int go)
 		sv_indirect_set(sc, SV_REG_ENABLE, enable);
 		ch->dma_active = 1;
 		break;
+	case PCMTRIG_STOP:
 	case PCMTRIG_ABORT:
 		enable = sv_indirect_get(sc, SV_REG_ENABLE) & ~SV_RECORD_ENABLE;
 		sv_indirect_set(sc, SV_REG_ENABLE, enable);
@@ -412,6 +413,7 @@ svpchan_trigger(kobj_t obj, void *data, int go)
 		sv_indirect_set(sc, SV_REG_ENABLE, enable);
 		ch->dma_active = 1;
 		break;
+	case PCMTRIG_STOP:
 	case PCMTRIG_ABORT:
 		enable = sv_indirect_get(sc, SV_REG_ENABLE) & ~SV_PLAY_ENABLE;
 		sv_indirect_set(sc, SV_REG_ENABLE, enable);
@@ -717,11 +719,7 @@ sv_attach(device_t dev) {
 	char		status[SND_STATUSLEN];
 	u_long		midi_start, games_start, count, sdmaa, sdmac, ml, mu;
 
-	sc = malloc(sizeof(struct sc_info), M_DEVBUF, M_NOWAIT | M_ZERO);
-	if (sc == NULL) {
-		device_printf(dev, "cannot allocate softc");
-		return ENXIO;
-	}
+	sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
 	sc->dev = dev;
 
 	data = pci_read_config(dev, PCIR_COMMAND, 2);
@@ -762,13 +760,14 @@ sv_attach(device_t dev) {
         sc->irq   = bus_alloc_resource(dev, SYS_RES_IRQ, &sc->irqid,
 				       0, ~0, 1, RF_ACTIVE | RF_SHAREABLE);
         if (!sc->irq ||
-	    bus_setup_intr(dev, sc->irq, INTR_TYPE_AV, sv_intr, sc, &sc->ih)) {
+	    snd_setup_intr(dev, sc->irq, 0, sv_intr, sc, &sc->ih)) {
                 device_printf(dev, "sv_attach: Unable to map interrupt\n");
                 goto fail;
         }
 
 	sc->bufsz = pcm_getbuffersize(dev, 4096, SV_DEFAULT_BUFSZ, 65536);
-        if (bus_dma_tag_create(/*parent*/NULL, /*alignment*/2, /*boundary*/0,
+        if (bus_dma_tag_create(/*parent*/bus_get_dma_tag(dev), /*alignment*/2,
+			       /*boundary*/0,
                                /*lowaddr*/BUS_SPACE_MAXADDR_24BIT,
                                /*highaddr*/BUS_SPACE_MAXADDR,
                                /*filter*/NULL, /*filterarg*/NULL,
