@@ -32,12 +32,12 @@
 
 #include "mixer_if.h"
 
-SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pcm/ac97.c,v 1.53.2.6 2007/10/31 04:00:07 ariff Exp $");
+SND_DECLARE_FILE("$FreeBSD: src/sys/dev/sound/pcm/ac97.c,v 1.73.2.1 2007/10/29 18:47:47 ariff Exp $");
 
 MALLOC_DEFINE(M_AC97, "ac97", "ac97 codec");
 
 struct ac97mixtable_entry {
-	int	 reg:8;		/* register index		*/
+	int reg;		/* register index		*/
 				/* reg < 0 if inverted polarity	*/
 	unsigned bits:4;	/* width of control field	*/
 	unsigned ofs:4;		/* offset (only if stereo=0)	*/
@@ -48,7 +48,8 @@ struct ac97mixtable_entry {
 	unsigned enable:1;	/* entry is enabled		*/
 };
 
-#define AC97_NAMELEN	16
+#define AC97_MIXER_SIZE		SOUND_MIXER_NRDEVICES
+
 struct ac97_info {
 	kobj_t methods;
 	device_t dev;
@@ -57,8 +58,8 @@ struct ac97_info {
 	u_int32_t subvendor;
 	unsigned count, caps, se, extcaps, extid, extstat, noext:1;
 	u_int32_t flags;
-	struct ac97mixtable_entry mix[32];
-	char name[AC97_NAMELEN];
+	struct ac97mixtable_entry mix[AC97_MIXER_SIZE];
+	char name[16];
 	struct mtx *lock;
 };
 
@@ -75,7 +76,7 @@ struct ac97_codecid {
 	ac97_patch patch;
 };
 
-static const struct ac97mixtable_entry ac97mixtable_default[32] = {
+static const struct ac97mixtable_entry ac97mixtable_default[AC97_MIXER_SIZE] = {
     /*	[offset]			reg	     bits of st mu re mk en */
 	[SOUND_MIXER_VOLUME]	= { AC97_MIX_MASTER, 	5, 0, 1, 1, 6, 0, 1 },
 	[SOUND_MIXER_OGAIN]	= { AC97_MIX_AUXOUT, 	5, 0, 1, 1, 0, 0, 0 },
@@ -614,12 +615,13 @@ ac97_initmixer(struct ac97_info *codec)
 
 	i = ac97_rdcd(codec, AC97_REG_RESET);
 	j = ac97_rdcd(codec, AC97_REG_RESET);
+	k = ac97_rdcd(codec, AC97_REG_RESET);
 	/*
 	 * Let see if this codec can return consistent value.
 	 * If not, turn on aggressive read workaround
 	 * (STAC9704 comes in mind).
 	 */
-	if (i != j) {
+	if (i != j || j != k) {
 		codec->flags |= AC97_F_RDCD_BUG;
 		i = ac97_rdcd(codec, AC97_REG_RESET);
 	}
@@ -674,7 +676,7 @@ ac97_initmixer(struct ac97_info *codec)
 		}
 	}
 
-	for (i = 0; i < 32; i++) {
+	for (i = 0; i < AC97_MIXER_SIZE; i++) {
 		codec->mix[i] = ac97mixtable_default[i];
 	}
 	ac97_fix_auxout(codec);
@@ -682,7 +684,7 @@ ac97_initmixer(struct ac97_info *codec)
 	if (codec_patch)
 		codec_patch(codec);
 
-	for (i = 0; i < 32; i++) {
+	for (i = 0; i < AC97_MIXER_SIZE; i++) {
 		k = codec->noext? codec->mix[i].enable : 1;
 		reg = codec->mix[i].reg;
 		if (reg < 0)
@@ -820,24 +822,21 @@ struct ac97_info *
 ac97_create(device_t dev, void *devinfo, kobj_class_t cls)
 {
 	struct ac97_info *codec;
+	int eapdinv;
 
-	codec = (struct ac97_info *)malloc(sizeof *codec, M_AC97, M_NOWAIT);
-	if (codec == NULL)
-		return NULL;
-
-	snprintf(codec->name, AC97_NAMELEN, "%s:ac97", device_get_nameunit(dev));
+	codec = malloc(sizeof(*codec), M_AC97, M_WAITOK | M_ZERO);
+	snprintf(codec->name, sizeof(codec->name), "%s:ac97",
+	    device_get_nameunit(dev));
 	codec->lock = snd_mtxcreate(codec->name, "ac97 codec");
-	codec->methods = kobj_create(cls, M_AC97, M_WAITOK);
-	if (codec->methods == NULL) {
-		snd_mtxlock(codec->lock);
-		snd_mtxfree(codec->lock);
-		free(codec, M_AC97);
-		return NULL;
-	}
-
+	codec->methods = kobj_create(cls, M_AC97, M_WAITOK | M_ZERO);
 	codec->dev = dev;
 	codec->devinfo = devinfo;
 	codec->flags = 0;
+	if (resource_int_value(device_get_name(dev), device_get_unit(dev),
+		    "eapdinv", &eapdinv) == 0) {
+		if (eapdinv != 0)
+			codec->flags |= AC97_F_EAPD_INV;
+	}
 	return codec;
 }
 
@@ -865,6 +864,60 @@ ac97_getflags(struct ac97_info *codec)
 
 /* -------------------------------------------------------------------- */
 
+#ifdef SND_DYNSYSCTL
+static int
+sysctl_hw_snd_ac97_eapd(SYSCTL_HANDLER_ARGS)
+{
+	struct ac97_info *codec;
+	int ea, inv, err = 0;
+	u_int16_t val;
+
+	codec = oidp->oid_arg1;
+	if (codec == NULL || codec->id == 0 || codec->lock == NULL)
+		return EINVAL;
+	snd_mtxlock(codec->lock);
+	val = ac97_rdcd(codec, AC97_REG_POWER);
+	inv = (codec->flags & AC97_F_EAPD_INV) ? 0 : 1;
+	ea = (val >> 15) ^ inv;
+	snd_mtxunlock(codec->lock);
+	err = sysctl_handle_int(oidp, &ea, 0, req);
+	if (err == 0 && req->newptr != NULL) {
+		if (ea != 0 && ea != 1)
+			return EINVAL;
+		if (ea != ((val >> 15) ^ inv)) {
+			snd_mtxlock(codec->lock);
+			ac97_wrcd(codec, AC97_REG_POWER, val ^ 0x8000);
+			snd_mtxunlock(codec->lock);
+		}
+	}
+	return err;
+}
+#endif
+
+static void
+ac97_init_sysctl(struct ac97_info *codec)
+{
+#ifdef SND_DYNSYSCTL
+	u_int16_t orig, val;
+
+	if (codec == NULL || codec->dev == NULL)
+		return;
+	snd_mtxlock(codec->lock);
+	orig = ac97_rdcd(codec, AC97_REG_POWER);
+	ac97_wrcd(codec, AC97_REG_POWER, orig ^ 0x8000);
+	val = ac97_rdcd(codec, AC97_REG_POWER);
+	ac97_wrcd(codec, AC97_REG_POWER, orig);
+	snd_mtxunlock(codec->lock);
+	if ((val & 0x8000) == (orig & 0x8000))
+		return;
+	SYSCTL_ADD_PROC(device_get_sysctl_ctx(codec->dev),
+	    SYSCTL_CHILDREN(device_get_sysctl_tree(codec->dev)),
+            OID_AUTO, "eapd", CTLTYPE_INT | CTLFLAG_RW,
+	    codec, sizeof(codec), sysctl_hw_snd_ac97_eapd,
+	    "I", "AC97 External Amplifier");
+#endif
+}
+
 static int
 ac97mix_init(struct snd_mixer *m)
 {
@@ -951,14 +1004,17 @@ ac97mix_init(struct snd_mixer *m)
 #endif
 
 	mask = 0;
-	for (i = 0; i < 32; i++)
+	for (i = 0; i < AC97_MIXER_SIZE; i++)
 		mask |= codec->mix[i].enable? 1 << i : 0;
 	mix_setdevs(m, mask);
 
 	mask = 0;
-	for (i = 0; i < 32; i++)
+	for (i = 0; i < AC97_MIXER_SIZE; i++)
 		mask |= codec->mix[i].recidx? 1 << i : 0;
 	mix_setrecdevs(m, mask);
+
+	ac97_init_sysctl(codec);
+
 	return 0;
 }
 
@@ -992,7 +1048,7 @@ ac97mix_set(struct snd_mixer *m, unsigned dev, unsigned left, unsigned right)
 {
 	struct ac97_info *codec = mix_getdevinfo(m);
 
-	if (codec == NULL)
+	if (codec == NULL || dev >= AC97_MIXER_SIZE)
 		return -1;
 	return ac97_setmixer(codec, dev, left, right);
 }
@@ -1005,7 +1061,7 @@ ac97mix_setrecsrc(struct snd_mixer *m, u_int32_t src)
 
 	if (codec == NULL)
 		return -1;
-	for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
+	for (i = 0; i < AC97_MIXER_SIZE; i++)
 		if ((src & (1 << i)) != 0)
 			break;
 	return (ac97_setrecsrc(codec, i) == 0)? 1 << i : -1;
