@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/sound/pcm/buffer.h,v 1.10.2.2 2007/05/13 20:50:31 ariff Exp $
+ * $FreeBSD: src/sys/dev/sound/pcm/buffer.h,v 1.17 2007/06/14 11:15:51 ariff Exp $
  */
 
 #define SND_DMA(b) (sndbuf_getflags((b)) & SNDBUF_F_DMA)
@@ -39,7 +39,9 @@
 struct snd_dbuf {
 	device_t dev;
 	u_int8_t *buf, *tmpbuf;
-	unsigned int bufsize, maxsize;
+	u_int8_t *shadbuf; /**< shadow buffer used w/ S_D_SILENCE/SKIP */
+	volatile int sl; /**< shadbuf ready length in # of bytes */
+	unsigned int bufsize, maxsize, allocsize;
 	volatile int dl; /* transfer size */
 	volatile int rp; /* pointers to the ready area */
 	volatile int rl; /* length of ready area */
@@ -53,6 +55,7 @@ struct snd_dbuf {
 	bus_dmamap_t dmamap;
 	bus_dma_tag_t dmatag;
 	bus_addr_t buf_addr;
+	int dmaflags;
 	struct selinfo sel;
 	struct pcm_channel *channel;
 	char name[SNDBUF_NAMELEN];
@@ -63,7 +66,7 @@ void sndbuf_destroy(struct snd_dbuf *b);
 
 void sndbuf_dump(struct snd_dbuf *b, char *s, u_int32_t what);
 
-int sndbuf_alloc(struct snd_dbuf *b, bus_dma_tag_t dmatag, unsigned int size);
+int sndbuf_alloc(struct snd_dbuf *b, bus_dma_tag_t dmatag, int dmaflags, unsigned int size);
 int sndbuf_setup(struct snd_dbuf *b, void *buf, unsigned int size);
 void sndbuf_free(struct snd_dbuf *b);
 int sndbuf_resize(struct snd_dbuf *b, unsigned int blkcnt, unsigned int blksz);
@@ -71,6 +74,8 @@ int sndbuf_remalloc(struct snd_dbuf *b, unsigned int blkcnt, unsigned int blksz)
 void sndbuf_reset(struct snd_dbuf *b);
 void sndbuf_clear(struct snd_dbuf *b, unsigned int length);
 void sndbuf_fillsilence(struct snd_dbuf *b);
+void sndbuf_softreset(struct snd_dbuf *b);
+void sndbuf_clearshadow(struct snd_dbuf *b);
 
 u_int32_t sndbuf_getfmt(struct snd_dbuf *b);
 int sndbuf_setfmt(struct snd_dbuf *b, u_int32_t fmt);
@@ -84,6 +89,7 @@ void *sndbuf_getbuf(struct snd_dbuf *b);
 void *sndbuf_getbufofs(struct snd_dbuf *b, unsigned int ofs);
 unsigned int sndbuf_getsize(struct snd_dbuf *b);
 unsigned int sndbuf_getmaxsize(struct snd_dbuf *b);
+unsigned int sndbuf_getallocsize(struct snd_dbuf *b);
 unsigned int sndbuf_getalign(struct snd_dbuf *b);
 unsigned int sndbuf_getblkcnt(struct snd_dbuf *b);
 void sndbuf_setblkcnt(struct snd_dbuf *b, unsigned int blkcnt);
@@ -94,7 +100,7 @@ void sndbuf_setrun(struct snd_dbuf *b, int go);
 struct selinfo *sndbuf_getsel(struct snd_dbuf *b);
 
 unsigned int sndbuf_getxrun(struct snd_dbuf *b);
-void sndbuf_setxrun(struct snd_dbuf *b, unsigned int cnt);
+void sndbuf_setxrun(struct snd_dbuf *b, unsigned int xrun);
 unsigned int sndbuf_gethwptr(struct snd_dbuf *b);
 void sndbuf_sethwptr(struct snd_dbuf *b, unsigned int ptr);
 unsigned int sndbuf_getfree(struct snd_dbuf *b);
@@ -104,6 +110,9 @@ unsigned int sndbuf_getreadyptr(struct snd_dbuf *b);
 unsigned int sndbuf_getblocks(struct snd_dbuf *b);
 unsigned int sndbuf_getprevblocks(struct snd_dbuf *b);
 unsigned int sndbuf_gettotal(struct snd_dbuf *b);
+unsigned int snd_xbytes(unsigned int v, unsigned int from, unsigned int to);
+unsigned int sndbuf_xbytes(unsigned int v, struct snd_dbuf *from, struct snd_dbuf *to);
+u_int8_t sndbuf_zerodata(u_int32_t fmt);
 void sndbuf_updateprevtotal(struct snd_dbuf *b);
 
 int sndbuf_acquire(struct snd_dbuf *b, u_int8_t *from, unsigned int count);
@@ -118,3 +127,7 @@ int sndbuf_dmasetdir(struct snd_dbuf *b, int dir);
 void sndbuf_dma(struct snd_dbuf *b, int go);
 int sndbuf_dmaptr(struct snd_dbuf *b);
 void sndbuf_dmabounce(struct snd_dbuf *b);
+
+#ifdef OSSV4_EXPERIMENT
+void sndbuf_getpeaks(struct snd_dbuf *b, int *lp, int *rp);
+#endif
