@@ -23,7 +23,20 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/sound/pcm/dsp.h,v 1.9 2005/01/06 01:43:20 imp Exp $
+ * $FreeBSD: src/sys/dev/sound/pcm/dsp.h,v 1.13 2007/06/16 03:37:28 ariff Exp $
  */
 
+#ifndef _PCMDSP_H_
+#define _PCMDSP_H_
+
 extern struct cdevsw dsp_cdevsw;
+
+struct dsp_cdevinfo;
+
+char *dsp_unit2name(char *, size_t, int);
+int dsp_oss_audioinfo(struct cdev *, oss_audioinfo *);
+
+void dsp_cdevinfo_init(struct snddev_info *);
+void dsp_cdevinfo_flush(struct snddev_info *);
+
+#endif /* !_PCMDSP_H_ */
