@@ -37,7 +37,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/sound/sbus/cs4231.c,v 1.1.2.2 2005/01/30 01:00:05 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/sound/sbus/cs4231.c,v 1.9 2007/06/17 06:10:43 ariff Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -125,7 +125,6 @@ struct cs4231_softc {
 	struct cs4231_channel	sc_pch;
 	struct cs4231_channel	sc_rch;
 	int			sc_enabled;
-	int			sc_rtype;
 	int			sc_nmres;
 	int			sc_nires;
 	int			sc_codecv;
@@ -307,12 +306,14 @@ MIXER_DECLARE(cs4231_mixer);
 static int
 cs4231_bus_probe(device_t dev)
 {
-	const char *name;
+	const char *compat, *name;
 
+	compat = ofw_bus_get_compat(dev);
 	name = ofw_bus_get_name(dev);
-	if (strcmp("SUNW,CS4231", name) == 0) {
+	if (strcmp("SUNW,CS4231", name) == 0 ||
+	    (compat != NULL && strcmp("SUNW,CS4231", compat) == 0)) {
 		device_set_desc(dev, "Sun Audiocs");
-		return (0);
+		return (BUS_PROBE_DEFAULT);
 	}
 	return (ENXIO);
 }
@@ -320,16 +321,10 @@ cs4231_bus_probe(device_t dev)
 static int
 cs4231_sbus_attach(device_t dev)
 {
-	struct snddev_info *d;
 	struct cs4231_softc *sc;
 	int burst;
 
-	d = device_get_softc(dev);
-	sc = malloc(sizeof(struct cs4231_softc), M_DEVBUF, M_NOWAIT | M_ZERO);
-	if (sc == NULL) {
-		device_printf(dev, "cannot allocate softc\n");
-		return (ENOMEM);
-	}
+	sc = malloc(sizeof(*sc), M_DEVBUF, M_WAITOK | M_ZERO);
 	sc->sc_dev = dev;
 	/*
 	 * XXX
@@ -345,7 +340,6 @@ cs4231_sbus_attach(device_t dev)
 	else
 		sc->sc_burst = 0;
 	sc->sc_flags = CS4231_SBUS;
-	sc->sc_rtype = SYS_RES_MEMORY;
 	sc->sc_nmres = 1;
 	sc->sc_nires = 1;
 	return cs4231_attach_common(sc);
@@ -354,10 +348,8 @@ cs4231_sbus_attach(device_t dev)
 static int
 cs4231_ebus_attach(device_t dev)
 {
-	struct snddev_info *d;
 	struct cs4231_softc *sc;
 
-	d = device_get_softc(dev);
 	sc = malloc(sizeof(struct cs4231_softc), M_DEVBUF, M_NOWAIT | M_ZERO);
 	if (sc == NULL) {
 		device_printf(dev, "cannot allocate softc\n");
@@ -365,7 +357,6 @@ cs4231_ebus_attach(device_t dev)
 	}
 	sc->sc_dev = dev;
 	sc->sc_burst = EBDCSR_BURST_1;
-	sc->sc_rtype = SYS_RES_IOPORT;
 	sc->sc_nmres = CS4231_RES_MEM_MAX;
 	sc->sc_nires = CS4231_RES_IRQ_MAX;
 	sc->sc_flags = CS4231_EBUS;
@@ -380,17 +371,12 @@ cs4231_attach_common(struct cs4231_softc *sc)
 	int i;
 
 	sc->sc_lock = snd_mtxcreate(device_get_nameunit(sc->sc_dev),
-	    "sound softc");
-	if (sc->sc_lock == NULL) {
-		device_printf(sc->sc_dev, "cannot create mutex\n");
-		free(sc, M_DEVBUF);
-		return (ENXIO);
-	}
+	    "snd_cs4231 softc");
 
 	for (i = 0; i < sc->sc_nmres; i++) {
 		sc->sc_rid[i] = i;
 		if ((sc->sc_res[i] = bus_alloc_resource_any(sc->sc_dev,
-		    sc->sc_rtype, &sc->sc_rid[i], RF_ACTIVE)) == NULL) {
+		    SYS_RES_MEMORY, &sc->sc_rid[i], RF_ACTIVE)) == NULL) {
 			device_printf(sc->sc_dev,
 			    "cannot map register %d\n", i);
 			goto fail;
@@ -439,7 +425,7 @@ cs4231_attach_common(struct cs4231_softc *sc)
 	    CS4231_DEFAULT_BUF_SZ, CS4231_MAX_BUF_SZ);
 	for (i = 0; i < sc->sc_nires; i++) {
 		if (bus_dma_tag_create(
-		    NULL,			/* parent */
+		    bus_get_dma_tag(sc->sc_dev),/* parent */
 		    64, 0,			/* alignment, boundary */
 		    BUS_SPACE_MAXADDR_32BIT,	/* lowaddr */
 		    BUS_SPACE_MAXADDR,		/* highaddr */
@@ -718,7 +704,7 @@ cs4231_free_resource(struct cs4231_softc *sc)
 	}
 	for (i = 0; i < sc->sc_nmres; i++) {
 		if (sc->sc_res[i])
-			bus_release_resource(sc->sc_dev, sc->sc_rtype,
+			bus_release_resource(sc->sc_dev, SYS_RES_MEMORY,
 			    sc->sc_rid[i], sc->sc_res[i]);
 	}
 	snd_mtxfree(sc->sc_lock);
@@ -1046,7 +1032,7 @@ cs4231_chan_init(kobj_t obj, void *dev, struct snd_dbuf *b,
 		else
 			dmat = sc->sc_dmat[0];
 	}
-	if (sndbuf_alloc(ch->buffer, dmat, sc->sc_bufsz) != 0)
+	if (sndbuf_alloc(ch->buffer, dmat, 0, sc->sc_bufsz) != 0)
 		return (NULL);
 	DPRINTF(("%s channel addr: 0x%lx\n", dir == PCMDIR_PLAY ? "playback" :
 	    "capture", sndbuf_getbufaddr(ch->buffer)));
