@@ -1,4 +1,4 @@
-/* $FreeBSD: src/sys/dev/sound/usb/uaudio_pcm.c,v 1.15.2.2 2007/05/13 20:53:40 ariff Exp $ */
+/* $FreeBSD: src/sys/dev/sound/usb/uaudio_pcm.c,v 1.24 2007/06/17 06:10:43 ariff Exp $ */
 
 /*-
  * Copyright (c) 2000-2002 Hiroyuki Aizu <aizu@navi.org>
@@ -54,10 +54,20 @@ struct ua_info {
 	u_int32_t ua_recfmt[FORMAT_NUM*2+1]; /* FORMAT_NUM format * (stereo or mono) + endptr */
 	struct pcmchan_caps ua_playcaps;
 	struct pcmchan_caps ua_reccaps;
+	int vendor, product, release;
 };
 
 #define UAUDIO_DEFAULT_BUFSZ		16*1024
 
+static const struct {
+	int vendor;
+	int product;
+	int release;
+	uint32_t dflags;
+} ua_quirks[] = {
+	{ 0x1130, 0xf211, 0x0101, SD_F_PSWAPLR },
+};
+
 /************************************************************/
 static void *
 ua_chan_init(kobj_t obj, void *devinfo, struct snd_dbuf *b, struct pcm_channel *c, int dir)
@@ -144,20 +154,47 @@ ua_chan_setspeed(kobj_t obj, void *data, u_int32_t speed)
 }
 
 static int
-ua_chan_setblocksize(kobj_t obj, void *data, u_int32_t blocksize)
+ua_chan_setfragments(kobj_t obj, void *data, u_int32_t blksz, u_int32_t blkcnt)
 {
 	device_t pa_dev;
 	struct ua_chinfo *ch = data;
 	struct ua_info *ua = ch->parent;
 
-	if (blocksize) {
-		RANGE(blocksize, 128, ua->bufsz / 2);
-		if (sndbuf_resize(ch->buffer, ua->bufsz/blocksize, blocksize) != 0) {
-			ch->blksz = blocksize;
-		}
+	RANGE(blksz, 128, sndbuf_getmaxsize(ch->buffer) / 2);
+	RANGE(blkcnt, 2, 512);
+
+	while ((blksz * blkcnt) > sndbuf_getmaxsize(ch->buffer)) {
+		if ((blkcnt >> 1) >= 2)
+			blkcnt >>= 1;
+		else if ((blksz >> 1) >= 128)
+			blksz >>= 1;
+		else
+			break;
 	}
+
+	if ((sndbuf_getblksz(ch->buffer) != blksz ||
+	    sndbuf_getblkcnt(ch->buffer) != blkcnt) &&
+	    sndbuf_resize(ch->buffer, blkcnt, blksz) != 0)
+		device_printf(ua->sc_dev, "%s: failed blksz=%u blkcnt=%u\n",
+		    __func__, blksz, blkcnt);
+
+	ch->blksz = sndbuf_getblksz(ch->buffer);
+
 	pa_dev = device_get_parent(ua->sc_dev);
-	uaudio_chan_set_param_blocksize(pa_dev, blocksize, ch->dir);
+	uaudio_chan_set_param_pcm_dma_buff(pa_dev, ch->buf,
+	    ch->buf + sndbuf_getsize(ch->buffer), ch->channel, ch->dir);
+	uaudio_chan_set_param_blocksize(pa_dev, ch->blksz, ch->dir);
+
+	return 1;
+}
+
+static int
+ua_chan_setblocksize(kobj_t obj, void *data, u_int32_t blksz)
+{
+	struct ua_chinfo *ch = data;
+
+	ua_chan_setfragments(obj, data, blksz,
+	    sndbuf_getmaxsize(ch->buffer) / blksz);
 
 	return ch->blksz;
 }
@@ -169,7 +206,7 @@ ua_chan_trigger(kobj_t obj, void *data, int go)
 	struct ua_info *ua;
 	struct ua_chinfo *ch = data;
 
-	if (go == PCMTRIG_EMLDMAWR || go == PCMTRIG_EMLDMARD)
+	if (!PCMTRIG_COMMON(go))
 		return 0;
 
 	ua = ch->parent;
@@ -222,6 +259,7 @@ static kobj_method_t ua_chan_methods[] = {
 	KOBJMETHOD(channel_setformat,		ua_chan_setformat),
 	KOBJMETHOD(channel_setspeed,		ua_chan_setspeed),
 	KOBJMETHOD(channel_setblocksize,	ua_chan_setblocksize),
+	KOBJMETHOD(channel_setfragments,	ua_chan_setfragments),
 	KOBJMETHOD(channel_trigger,		ua_chan_trigger),
 	KOBJMETHOD(channel_getptr,		ua_chan_getptr),
 	KOBJMETHOD(channel_getcaps,		ua_chan_getcaps),
@@ -236,27 +274,22 @@ ua_mixer_init(struct snd_mixer *m)
 {
 	u_int32_t mask;
 	device_t pa_dev;
-	struct snddev_info *d;
 	struct ua_info *ua = mix_getdevinfo(m);
 
 	pa_dev = device_get_parent(ua->sc_dev);
-	d = device_get_softc(ua->sc_dev);
 
 	mask = uaudio_query_mix_info(pa_dev);
-	if (d != NULL) {
-		if (!(mask & SOUND_MASK_PCM)) {
-			/*
-			 * Emulate missing pcm mixer controller
-			 * through FEEDER_VOLUME
-			 */
-			 d->flags |= SD_F_SOFTPCMVOL;
-		}
-		if (!(mask & SOUND_MASK_VOLUME)) {
-			mix_setparentchild(m, SOUND_MIXER_VOLUME,
-			    SOUND_MASK_PCM);
-			mix_setrealdev(m, SOUND_MIXER_VOLUME,
-			    SOUND_MIXER_NONE);
-		}
+	if (!(mask & SOUND_MASK_PCM)) {
+		/*
+		 * Emulate missing pcm mixer controller
+		 * through FEEDER_VOLUME
+		 */
+		pcm_setflags(ua->sc_dev, pcm_getflags(ua->sc_dev) |
+		    SD_F_SOFTPCMVOL);
+	}
+	if (!(mask & SOUND_MASK_VOLUME)) {
+		mix_setparentchild(m, SOUND_MIXER_VOLUME, SOUND_MASK_PCM);
+		mix_setrealdev(m, SOUND_MIXER_VOLUME, SOUND_MIXER_NONE);
 	}
 	mix_setdevs(m,	mask);
 
@@ -321,18 +354,30 @@ static int
 ua_attach(device_t dev)
 {
 	struct ua_info *ua;
+	struct sndcard_func *func;
 	char status[SND_STATUSLEN];
 	device_t pa_dev;
-	u_int32_t nplay, nrec;
+	u_int32_t nplay, nrec, flags;
 	int i;
 
-	ua = (struct ua_info *)malloc(sizeof *ua, M_DEVBUF, M_ZERO | M_NOWAIT);
-	if (ua == NULL)
-		return ENXIO;
-
+	ua = malloc(sizeof(*ua), M_DEVBUF, M_WAITOK | M_ZERO);
 	ua->sc_dev = dev;
 
+	/* Mark for existence */
+	func = device_get_ivars(dev);
+	if (func != NULL)
+		func->varinfo = (void *)ua;
+
 	pa_dev = device_get_parent(dev);
+	ua->vendor = uaudio_get_vendor(pa_dev);
+	ua->product = uaudio_get_product(pa_dev);
+	ua->release = uaudio_get_release(pa_dev);
+
+	if (bootverbose)
+		device_printf(dev,
+		    "USB Audio: "
+		    "vendor=0x%04x, product=0x%04x, release=0x%04x\n",
+		    ua->vendor, ua->product, ua->release);
 
 	ua->bufsz = pcm_getbuffersize(dev, 4096, UAUDIO_DEFAULT_BUFSZ, 65536);
 	if (bootverbose)
@@ -354,6 +399,15 @@ ua_attach(device_t dev)
 	if (nrec > 1)
 		nrec = 1;
 
+	flags = pcm_getflags(dev);
+	for (i = 0; i < (sizeof(ua_quirks) / sizeof(ua_quirks[0])); i++) {
+		if (ua->vendor == ua_quirks[i].vendor &&
+		    ua->product == ua_quirks[i].product &&
+		    ua->release == ua_quirks[i].release)
+			flags |= ua_quirks[i].dflags;
+	}
+	pcm_setflags(dev, flags);
+
 #ifndef NO_RECORDING
 	if (pcm_register(dev, ua, nplay, nrec)) {
 #else
@@ -384,8 +438,9 @@ bad:	free(ua, M_DEVBUF);
 static int
 ua_detach(device_t dev)
 {
-	int r;
 	struct ua_info *sc;
+	struct sndcard_func *func;
+	int r;
 
 	r = pcm_unregister(dev);
 	if (r)
@@ -394,6 +449,11 @@ ua_detach(device_t dev)
 	sc = pcm_getdevinfo(dev);
 	free(sc, M_DEVBUF);
 
+	/* Mark for deletion */
+	func = device_get_ivars(dev);
+	if (func != NULL)
+		func->varinfo = NULL;
+
 	return 0;
 }
 
