@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/syscons/scmouse.c,v 1.37.2.3 2005/10/05 21:48:02 marius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/syscons/scmouse.c,v 1.42 2007/05/25 13:13:12 delphij Exp $");
 
 #include "opt_syscons.h"
 
@@ -178,13 +178,13 @@ sc_draw_mouse_image(scr_stat *scp)
     if (ISGRAPHSC(scp))
 	return;
 
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
     (*scp->rndr->draw_mouse)(scp, scp->mouse_xpos, scp->mouse_ypos, TRUE);
     scp->mouse_oldpos = scp->mouse_pos;
     scp->mouse_oldxpos = scp->mouse_xpos;
     scp->mouse_oldypos = scp->mouse_ypos;
     scp->status |= MOUSE_VISIBLE;
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 void
@@ -196,7 +196,7 @@ sc_remove_mouse_image(scr_stat *scp)
     if (ISGRAPHSC(scp))
 	return;
 
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
     (*scp->rndr->draw_mouse)(scp,
 			     (scp->mouse_oldpos%scp->xsize + scp->xoff)
 			         * scp->font_width,
@@ -217,7 +217,7 @@ sc_remove_mouse_image(scr_stat *scp)
     }
 #endif /* PC98 */
     scp->status &= ~MOUSE_VISIBLE;
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 int
@@ -444,7 +444,8 @@ mouse_cut_start(scr_stat *scp)
 
     if (scp->status & MOUSE_VISIBLE) {
 	sc_remove_all_cutmarkings(scp->sc);
-	if (scp->mouse_pos == scp->mouse_cut_start == scp->mouse_cut_end) {
+	if ((scp->mouse_pos == scp->mouse_cut_start) &&
+	    (scp->mouse_pos == scp->mouse_cut_end)) {
 	    cut_buffer[0] = '\0';
 	    return;
 	} else if (skip_spc_right(scp, scp->mouse_pos) >= scp->xsize) {
