@@ -28,8 +28,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/syscons/scvidctl.c,v 1.34.2.4 2006/10/04 06:09:09 ru Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/syscons/scvidctl.c,v 1.38 2006/09/27 19:56:59 ru Exp $");
 
+#include "opt_compat.h"
 #include "opt_syscons.h"
 
 #include <sys/param.h>
@@ -482,7 +483,10 @@ sc_vid_ioctl(struct tty *tp, u_long cmd, caddr_t data, int flag, struct thread *
     video_adapter_info_t adp_info;
     int error;
     int s;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     int ival;
+#endif
 
     scp = SC_STAT(tp->t_dev);
     if (scp == NULL)		/* tp == SC_MOUSE */
@@ -585,10 +589,13 @@ sc_vid_ioctl(struct tty *tp, u_long cmd, caddr_t data, int flag, struct thread *
     case FBIO_FINDMODE:
 	return fb_ioctl(adp, FBIO_FINDMODE, data);
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('c', 104):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case CONS_SETWINORG:	/* set frame buffer window origin */
     case FBIO_SETWINORG:
 	if (scp != scp->sc->cur_scp)
@@ -680,10 +687,13 @@ sc_vid_ioctl(struct tty *tp, u_long cmd, caddr_t data, int flag, struct thread *
 	return sc_set_graphics_mode(scp, tp, cmd & 0xff);
 #endif /* SC_NO_MODE_CHANGE */
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 10):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDSETMODE:     	/* set current mode of this (virtual) console */
 	switch (*(int *)data) {
 	case KD_TEXT:   	/* switch to TEXT (known) mode */
@@ -813,10 +823,13 @@ sc_vid_ioctl(struct tty *tp, u_long cmd, caddr_t data, int flag, struct thread *
 	*data = ISGRAPHSC(scp) ? KD_GRAPHICS : KD_TEXT;
 	return 0;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 13):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDSBORDER:     	/* set border color of this (virtual) console */
 	scp->border = *(int *)data;
 	if (scp == scp->sc->cur_scp)
