@@ -30,8 +30,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/syscons/syscons.c,v 1.436.2.10 2007/10/25 06:09:15 simokawa Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/syscons/syscons.c,v 1.453.4.1 2008/01/28 12:49:33 kib Exp $");
 
+#include "opt_compat.h"
 #include "opt_syscons.h"
 #include "opt_splash.h"
 #include "opt_ddb.h"
@@ -49,6 +50,7 @@ __FBSDID("$FreeBSD: src/sys/dev/syscons/syscons.c,v 1.436.2.10 2007/10/25 06:09:
 #include <sys/lock.h>
 #include <sys/malloc.h>
 #include <sys/mutex.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/random.h>
 #include <sys/reboot.h>
@@ -58,7 +60,7 @@ __FBSDID("$FreeBSD: src/sys/dev/syscons/syscons.c,v 1.436.2.10 2007/10/25 06:09:
 #include <sys/power.h>
 
 #include <machine/clock.h>
-#if __sparc64__ || __powerpc__
+#if defined(__sparc64__) || defined(__powerpc__)
 #include <machine/sc_machdep.h>
 #else
 #include <machine/pc/display.h>
@@ -217,16 +219,13 @@ static int update_kbd_state(scr_stat *scp, int state, int mask);
 static int update_kbd_leds(scr_stat *scp, int which);
 static timeout_t blink_screen;
 
-static cn_probe_t	sccnprobe;
-static cn_init_t	sccninit;
-static cn_getc_t	sccngetc;
-static cn_checkc_t	sccncheckc;
-static cn_putc_t	sccnputc;
-static cn_dbctl_t	sccndbctl;
-static cn_term_t	sccnterm;
+static cn_probe_t	sc_cnprobe;
+static cn_init_t	sc_cninit;
+static cn_term_t	sc_cnterm;
+static cn_getc_t	sc_cngetc;
+static cn_putc_t	sc_cnputc;
 
-CONS_DRIVER(sc, sccnprobe, sccninit, sccnterm, sccngetc, sccncheckc, sccnputc,
-	    sccndbctl);
+CONSOLE_DRIVER(sc);
 
 static	d_open_t	scopen;
 static	d_close_t	scclose;
@@ -519,7 +518,7 @@ scopen(struct cdev *dev, int flag, int mode, struct thread *td)
 	ttyld_modem(tp, 1);
     }
     else
-	if (tp->t_state & TS_XCLUDE && suser(td))
+	if (tp->t_state & TS_XCLUDE && priv_check(td, PRIV_TTY_EXCLUSIVE))
 	    return(EBUSY);
 
     error = ttyld_open(tp, dev);
@@ -556,7 +555,7 @@ scclose(struct cdev *dev, int flag, int mode, struct thread *td)
 	    DPRINTF(5, ("reset WAIT_REL, "));
 	if (finish_vt_acq(scp) == 0)		/* force acknowledge */
 	    DPRINTF(5, ("reset WAIT_ACQ, "));
-#if not_yet_done
+#ifdef not_yet_done
 	if (scp == &main_console) {
 	    scp->pid = 0;
 	    scp->proc = NULL;
@@ -683,7 +682,10 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
     sc_softc_t *sc;
     scr_stat *scp;
     int s;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     int ival;
+#endif
 
     tp = dev->si_tty;
 
@@ -984,10 +986,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	bcopy(&scp->smode, data, sizeof(struct vt_mode));
 	return 0;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('v', 4):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case VT_RELDISP:    	/* screen switcher ioctl */
 	s = spltty();
 	/*
@@ -1033,21 +1038,29 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	}
 	return EINVAL;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('v', 5):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case VT_ACTIVATE:   	/* switch to screen *data */
 	i = (*(int *)data == 0) ? scp->index : (*(int *)data - 1);
 	s = spltty();
-	sc_clean_up(sc->cur_scp);
+	error = sc_clean_up(sc->cur_scp);
 	splx(s);
+	if (error)
+	    return error;
 	return sc_switch_scr(sc, i);
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('v', 6):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case VT_WAITACTIVE: 	/* wait for switch to occur */
 	i = (*(int *)data == 0) ? scp->index : (*(int *)data - 1);
 	if ((i < sc->first_vty) || (i >= sc->first_vty + sc->vtys))
@@ -1058,6 +1071,8 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	if (error)
 	    return error;
 	scp = sc_get_stat(SC_DEV(sc, i));
+	if (scp == NULL)
+		return (ENXIO);
 	if (scp == scp->sc->cur_scp)
 	    return 0;
 	error = tsleep(&scp->smode, PZERO | PCATCH, "waitvt", 0);
@@ -1079,7 +1094,7 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	return 0;
 
     case KDENABIO:      	/* allow io operations */
-	error = suser(td);
+	error = priv_check(td, PRIV_IO);
 	if (error != 0)
 	    return error;
 	error = securelevel_gt(td->td_ucred, 0);
@@ -1100,10 +1115,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 #endif
 	return 0;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 20):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDSKBSTATE:    	/* set keyboard state (locks) */
 	if (*(int *)data & ~LOCK_MASK)
 	    return EINVAL;
@@ -1126,10 +1144,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	    error = ENODEV;
 	return error;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 67):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDSETRAD:      	/* set keyboard repeat & delay rates (old) */
 	if (*(int *)data & ~0x7f)
 	    return EINVAL;
@@ -1138,10 +1159,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	    error = ENODEV;
 	return error;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 7):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDSKBMODE:     	/* set keyboard mode */
 	switch (*(int *)data) {
 	case K_XLATE:   	/* switch to XLT ascii mode */
@@ -1166,10 +1190,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	    error = ENODEV;
 	return error;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 8):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDMKTONE:      	/* sound the bell */
 	if (*(int*)data)
 	    sc_bell(scp, (*(int*)data)&0xffff,
@@ -1178,10 +1205,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	    sc_bell(scp, scp->bell_pitch, scp->bell_duration);
 	return 0;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 63):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KIOCSOUND:     	/* make tone (*data) hz */
 	if (scp == sc->cur_scp) {
 	    if (*(int *)data)
@@ -1199,10 +1229,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	}
 	return 0;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('K', 66):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case KDSETLED:      	/* set keyboard LED status */
 	if (*(int *)data & ~LED_MASK)	/* FIXME: LOCK_MASK? */
 	    return EINVAL;
@@ -1225,10 +1258,13 @@ scioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td)
 	    error = ENODEV;
 	return error;
 
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5) || \
+    defined(COMPAT_FREEBSD4) || defined(COMPAT_43)
     case _IO('c', 110):
 	ival = IOCPARM_IVAL(data);
 	data = (caddr_t)&ival;
 	/* FALLTHROUGH */
+#endif
     case CONS_SETKBD: 		/* set the new keyboard */
 	{
 	    keyboard_t *newkbd;
@@ -1443,7 +1479,7 @@ scstart(struct tty *tp)
 }
 
 static void
-sccnprobe(struct consdev *cp)
+sc_cnprobe(struct consdev *cp)
 {
     int unit;
     int flags;
@@ -1465,7 +1501,7 @@ sccnprobe(struct consdev *cp)
 }
 
 static void
-sccninit(struct consdev *cp)
+sc_cninit(struct consdev *cp)
 {
     int unit;
     int flags;
@@ -1478,7 +1514,7 @@ sccninit(struct consdev *cp)
 }
 
 static void
-sccnterm(struct consdev *cp)
+sc_cnterm(struct consdev *cp)
 {
     /* we are not the kernel console any more, release everything */
 
@@ -1496,7 +1532,7 @@ sccnterm(struct consdev *cp)
 }
 
 static void
-sccnputc(struct consdev *cd, int c)
+sc_cnputc(struct consdev *cd, int c)
 {
     u_char buf[1];
     scr_stat *scp = sc_console;
@@ -1538,48 +1574,11 @@ sccnputc(struct consdev *cd, int c)
 }
 
 static int
-sccngetc(struct consdev *cd)
-{
-    return sccngetch(0);
-}
-
-static int
-sccncheckc(struct consdev *cd)
+sc_cngetc(struct consdev *cd)
 {
     return sccngetch(SCGETC_NONBLOCK);
 }
 
-static void
-sccndbctl(struct consdev *cd, int on)
-{
-    /* assert(sc_console_unit >= 0) */
-    /* try to switch to the kernel console screen */
-    if (on && debugger == 0) {
-	/*
-	 * TRY to make sure the screen saver is stopped, 
-	 * and the screen is updated before switching to 
-	 * the vty0.
-	 */
-	scrn_timer(NULL);
-	if (!cold
-	    && sc_console->sc->cur_scp->smode.mode == VT_AUTO
-	    && sc_console->smode.mode == VT_AUTO) {
-	    sc_console->sc->cur_scp->status |= MOUSE_HIDDEN;
-	    ++debugger;		/* XXX */
-#ifdef DDB
-	    /* unlock vty switching */
-	    sc_console->sc->flags &= ~SC_SCRN_VTYLOCK;
-#endif
-	    sc_switch_scr(sc_console->sc, sc_console->index);
-	    --debugger;		/* XXX */
-	}
-    }
-    if (on)
-	++debugger;
-    else
-	--debugger;
-}
-
 static int
 sccngetch(int flags)
 {
@@ -1656,7 +1655,7 @@ sccnupdate(scr_stat *scp)
 {
     /* this is a cut-down version of scrn_timer()... */
 
-    if (scp->sc->font_loading_in_progress || scp->sc->videoio_in_progress)
+    if (scp->sc->font_loading_in_progress)
 	return;
 
     if (debugger > 0 || panicstr || shutdown_in_progress) {
@@ -1706,7 +1705,7 @@ scrn_timer(void *arg)
 	return;
 
     /* don't do anything when we are performing some I/O operations */
-    if (sc->font_loading_in_progress || sc->videoio_in_progress) {
+    if (sc->font_loading_in_progress) {
 	if (again)
 	    timeout(scrn_timer, sc, hz / 10);
 	return;
@@ -1800,7 +1799,7 @@ scrn_update(scr_stat *scp, int show_cursor)
 
     /* assert(scp == scp->sc->cur_scp) */
 
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
 
 #ifndef SC_NO_CUTPASTE
     /* remove the previous mouse pointer image if necessary */
@@ -1870,7 +1869,7 @@ scrn_update(scr_stat *scp, int show_cursor)
     if (!show_cursor) {
         scp->end = 0;
         scp->start = scp->xsize*scp->ysize - 1;
-	--scp->sc->videoio_in_progress;
+	SC_VIDEO_UNLOCK(scp->sc);
 	return;
     }
 
@@ -1909,7 +1908,7 @@ scrn_update(scr_stat *scp, int show_cursor)
     scp->end = 0;
     scp->start = scp->xsize*scp->ysize - 1;
 
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 #ifdef DEV_SPLASH
@@ -2172,7 +2171,7 @@ sc_switch_scr(sc_softc_t *sc, u_int next_scr)
 
     /* delay switch if the screen is blanked or being updated */
     if ((sc->flags & SC_SCRN_BLANKED) || sc->write_in_progress
-	|| sc->blink_in_progress || sc->videoio_in_progress) {
+	|| sc->blink_in_progress) {
 	sc->delayed_next_scr = next_scr + 1;
 	sc_touch_scrn_saver();
 	DPRINTF(5, ("switch delayed\n"));
@@ -2328,8 +2327,8 @@ sc_switch_scr(sc_softc_t *sc, u_int next_scr)
     if (sc->new_scp == sc->old_scp) {
 	sc->switch_in_progress = 0;
 	/*
-	 * XXX wakeup() calls mtx_lock(&sched_lock) which will hang if
-	 * sched_lock is in an in-between state, e.g., when we stop at
+	 * XXX wakeup() locks the scheduler lock which will hang if
+	 * the lock is in an in-between state, e.g., when we stop at
 	 * a breakpoint at fork_exit.  It has always been wrong to call
 	 * wakeup() when the debugger is active.  In RELENG_4, wakeup()
 	 * is supposed to be locked by splhigh(), but the debugger may
@@ -2539,23 +2538,23 @@ void
 sc_draw_cursor_image(scr_stat *scp)
 {
     /* assert(scp == scp->sc->cur_scp); */
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
     (*scp->rndr->draw_cursor)(scp, scp->cursor_pos,
 			      scp->curs_attr.flags & CONS_BLINK_CURSOR, TRUE,
 			      sc_inside_cutmark(scp, scp->cursor_pos));
     scp->cursor_oldpos = scp->cursor_pos;
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 void
 sc_remove_cursor_image(scr_stat *scp)
 {
     /* assert(scp == scp->sc->cur_scp); */
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
     (*scp->rndr->draw_cursor)(scp, scp->cursor_oldpos,
 			      scp->curs_attr.flags & CONS_BLINK_CURSOR, FALSE,
 			      sc_inside_cutmark(scp, scp->cursor_oldpos));
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 static void
@@ -2586,10 +2585,10 @@ sc_set_cursor_image(scr_stat *scp)
     }
 
     /* assert(scp == scp->sc->cur_scp); */
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
     (*scp->rndr->set_cursor)(scp, scp->curs_attr.base, scp->curs_attr.height,
 			     scp->curs_attr.flags & CONS_BLINK_CURSOR);
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 static void
@@ -2693,6 +2692,9 @@ scinit(int unit, int flags)
      * disappeared...
      */
     sc = sc_get_softc(unit, flags & SC_KERNEL_CONSOLE);
+    if ((sc->flags & SC_INIT_DONE) == 0)
+	SC_VIDEO_LOCKINIT(sc);
+
     adp = NULL;
     if (sc->adapter >= 0) {
 	vid_release(sc->adp, (void *)&sc->adapter);
@@ -3548,9 +3550,9 @@ set_mode(scr_stat *scp)
 void
 sc_set_border(scr_stat *scp, int color)
 {
-    ++scp->sc->videoio_in_progress;
+    SC_VIDEO_LOCK(scp->sc);
     (*scp->rndr->draw_border)(scp, color);
-    --scp->sc->videoio_in_progress;
+    SC_VIDEO_UNLOCK(scp->sc);
 }
 
 #ifndef SC_NO_FONT_LOADING
