@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/syscons/syscons.h,v 1.83.2.3 2007/10/25 06:09:15 simokawa Exp $
+ * $FreeBSD: src/sys/dev/syscons/syscons.h,v 1.88 2007/09/20 04:05:59 simokawa Exp $
  */
 
 #ifndef _DEV_SYSCONS_SYSCONS_H_
@@ -43,8 +43,6 @@
 #include <pc98/cbus/sc_machdep.h>
 #elif defined(__i386__)
 /* nothing for the moment */
-#elif defined(__alpha__)
-/* nothing for the moment */
 #endif
 
 /* default values for configuration options */
@@ -230,9 +228,9 @@ typedef struct sc_softc {
 
 	char        	font_loading_in_progress;
 	char        	switch_in_progress;
-	char        	videoio_in_progress;
 	char        	write_in_progress;
 	char        	blink_in_progress;
+	struct mtx	video_mtx;
 
 	long		scrn_time_stamp;
 
@@ -538,6 +536,19 @@ typedef struct {
 #define kbd_poll(kbd, on)						\
 		(*kbdsw[(kbd)->kb_index]->poll)((kbd), (on))
 
+#define SC_VIDEO_LOCKINIT(sc)						\
+		mtx_init(&(sc)->video_mtx, "syscons video lock", NULL,MTX_SPIN);
+#define SC_VIDEO_LOCK(sc)						\
+		do {							\
+			if (!cold)					\
+				mtx_lock_spin(&(sc)->video_mtx);	\
+		} while(0)
+#define SC_VIDEO_UNLOCK(sc)						\
+		do {							\
+			if (!cold)					\
+				mtx_unlock_spin(&(sc)->video_mtx);	\
+		} while(0)
+
 /* syscons.c */
 extern int 	(*sc_user_ioctl)(struct cdev *dev, u_long cmd, caddr_t data,
 				 int flag, struct thread *td);
