@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/twa/tw_cl_misc.c,v 1.1.4.3 2007/10/26 23:51:45 scottl Exp $
+ *	$FreeBSD: src/sys/dev/twa/tw_cl_misc.c,v 1.4 2007/10/09 17:43:57 scottl Exp $
  */
 
 /*
@@ -32,6 +32,7 @@
  *
  * Author: Vinod Kashyap
  * Modifications by: Adam Radford
+ * Modifications by: Manjunath Ranganathaiah
  */
 
 
