@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/twa/tw_cl_share.h,v 1.1.4.3 2007/10/26 23:51:45 scottl Exp $
+ *	$FreeBSD: src/sys/dev/twa/tw_cl_share.h,v 1.4 2007/10/09 17:43:57 scottl Exp $
  */
 
 /*
@@ -32,6 +32,7 @@
  *
  * Author: Vinod Kashyap
  * Modifications by: Adam Radford
+ * Modifications by: Manjunath Ranganathaiah
  */
 
 
@@ -55,6 +56,7 @@
 #define TW_CL_DEVICE_ID_9K_X		0x1003	/* 9000 PCI-X series device id */
 #define TW_CL_DEVICE_ID_9K_E		0x1004  /* 9000 PCIe series device id */
 #define TW_CL_DEVICE_ID_9K_SA		0x1005	/* 9000 PCIe SAS series device id */
+
 #define TW_CL_BAR_TYPE_IO		1	/* I/O base address */
 #define TW_CL_BAR_TYPE_MEM		2	/* memory base address */
 #define TW_CL_BAR_TYPE_SBUF		3	/* SBUF base address */
