@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/twa/tw_osl.h,v 1.1.4.3 2007/10/26 23:51:45 scottl Exp $
+ *	$FreeBSD: src/sys/dev/twa/tw_osl.h,v 1.5.2.1 2007/10/19 15:21:16 scottl Exp $
  */
 
 /*
@@ -53,7 +53,9 @@
 #define TW_OSLI_MAX_NUM_IOS		TW_CL_MAX_SIMULTANEOUS_REQUESTS
 #define TW_OSLI_MAX_NUM_AENS		0x100
 
-//#define TW_OSLI_DEFERRED_INTR_USED
+/* Disabled, doesn't work yet.
+#define TW_OSLI_DEFERRED_INTR_USED
+*/
 
 /* Possible values of req->state. */
 #define TW_OSLI_REQ_STATE_INIT		0x0	/* being initialized */
@@ -144,6 +146,8 @@ struct twa_softc {
 	struct mtx		*io_lock;/* ptr to general purpose lock */
 	struct mtx		q_lock_handle;	/* queue manipulation lock */
 	struct mtx		*q_lock;/* ptr to queue manipulation lock */
+	struct mtx		sim_lock_handle;/* sim lock shared with cam */
+	struct mtx		*sim_lock;/* ptr to sim lock */
 
 #ifdef TW_OSL_DEBUG
 	struct tw_osli_q_stats	q_stats[TW_OSLI_Q_COUNT];/* queue statistics */
