@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/twa/tw_osl_includes.h,v 1.1.4.2 2007/05/19 10:39:30 scottl Exp $
+ *	$FreeBSD: src/sys/dev/twa/tw_osl_includes.h,v 1.5 2007/05/09 04:16:32 scottl Exp $
  */
 
 /*
@@ -54,6 +54,7 @@
 #include <sys/sysctl.h>
 #include <sys/bus.h>
 #include <sys/conf.h>
+#include <sys/clock.h>
 #include <sys/disk.h>
 #include <sys/stat.h>
 #include <sys/devicestat.h>
@@ -61,7 +62,6 @@
 
 #include <machine/bus.h>
 #include <machine/resource.h>
-#include <machine/clock.h>
 #include <machine/stdarg.h>
 
 #include <vm/vm.h>
