@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/twa/tw_osl_share.h,v 1.2.2.3 2007/10/26 23:51:46 scottl Exp $
+ *	$FreeBSD: src/sys/dev/twa/tw_osl_share.h,v 1.9 2007/10/09 17:43:57 scottl Exp $
  */
 
 /*
@@ -50,6 +50,7 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
+#include <sys/clock.h>
 #include <sys/lock.h>
 #include <sys/mutex.h>
 #include <sys/sysctl.h>
@@ -57,7 +58,6 @@
 #include <sys/taskqueue.h>
 
 #include <machine/bus.h>
-#include <machine/clock.h>
 #include <machine/endian.h>
 #include <machine/stdarg.h>
 
@@ -75,7 +75,7 @@
 #define TW_OSL_ENCLOSURE_SUPPORT
 #endif
 
-#define TW_OSL_DRIVER_VERSION_STRING	"3.60.04.003"
+#define TW_OSL_DRIVER_VERSION_STRING	"3.70.05.001"
 
 #define	TW_OSL_CAN_SLEEP
 
