@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$FreeBSD: src/sys/dev/twa/tw_osl_types.h,v 1.1.4.1 2007/05/19 10:39:31 scottl Exp $
+ *	$FreeBSD: src/sys/dev/twa/tw_osl_types.h,v 1.2 2007/05/09 04:16:32 scottl Exp $
  */
 
 /*
