@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/uart/uart_cpu_pc98.c,v 1.15 2005/01/06 01:43:26 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/uart/uart_cpu_pc98.c,v 1.17 2007/04/02 22:00:22 marcel Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -49,10 +49,15 @@ uart_cpu_eqres(struct uart_bas *b1, struct uart_bas *b2)
 int
 uart_cpu_getdev(int devtype, struct uart_devinfo *di)
 {
+	struct uart_class *class;
 	unsigned int i, ivar, flags;
 
+	class = &uart_ns8250_class;
+	if (class == NULL)
+		return (ENXIO);
+
 	/* Check the environment. */
-	if (uart_getenv(devtype, di) == 0)
+	if (uart_getenv(devtype, di, class) == 0)
 		return (0);
 
 	/*
@@ -80,10 +85,11 @@ uart_cpu_getdev(int devtype, struct uart_devinfo *di)
 		    ivar == 0)
 			continue;
 
-		di->ops = uart_ns8250_ops;
+		di->ops = uart_getops(class);
 		di->bas.chan = 0;
 		di->bas.bst = uart_bus_space_io;
-		if (bus_space_map(di->bas.bst, ivar, 8, 0, &di->bas.bsh) != 0)
+		if (bus_space_map(di->bas.bst, ivar, uart_getrange(class), 0,
+		    &di->bas.bsh) != 0)
 			continue;
 		di->bas.regshft = 0;
 		di->bas.rclk = 0;
