@@ -25,8 +25,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/uart/uart_kbd_sun.c,v 1.5.2.4 2007/09/30 19:33:19 marius Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/uart/uart_kbd_sun.c,v 1.13 2007/05/01 14:14:52 marius Exp $");
 
+#include "opt_compat.h"
 #include "opt_kbd.h"
 #include "opt_sunkbd.h"
 
@@ -42,8 +43,6 @@ __FBSDID("$FreeBSD: src/sys/dev/uart/uart_kbd_sun.c,v 1.5.2.4 2007/09/30 19:33:1
 #include <sys/kbio.h>
 #include <sys/kernel.h>
 #include <sys/limits.h>
-#include <sys/lock.h>
-#include <sys/mutex.h>
 
 #include <machine/bus.h>
 
@@ -73,8 +72,6 @@ struct sunkbd_softc {
 	struct uart_softc	*sc_uart;
 	struct uart_devinfo	*sc_sysdev;
 
-	int			sc_checked_key;
-
 	struct callout		sc_repeat_callout;
 	int			sc_repeat_key;
 
@@ -307,10 +304,10 @@ sunkbd_uart_intr(void *arg)
 		return;
 
 	pend = atomic_readandclear_32(&sc->sc_uart->sc_ttypend);
-	if (!(pend & UART_IPEND_MASK))
+	if (!(pend & SER_INT_MASK))
 		return;
 
-	if (pend & UART_IPEND_RXREADY) {
+	if (pend & SER_INT_RXREADY) {
 		if (KBD_IS_ACTIVE(&sc->sc_kbd) && KBD_IS_BUSY(&sc->sc_kbd)) {
 			sc->sc_kbd.kb_callback.kc_func(&sc->sc_kbd,
 			    KBDIO_KEYINPUT, sc->sc_kbd.kb_callback.kc_arg);
@@ -404,7 +401,7 @@ sunkbd_check(keyboard_t *kbd)
 		return (TRUE);
 
 	if (sc->sc_polling != 0 && sc->sc_sysdev != NULL &&
-	    (sc->sc_checked_key = uart_poll(sc->sc_sysdev)) != -1)
+	    uart_rxready(sc->sc_sysdev))
 		return (TRUE);
 
 	return (FALSE);
@@ -442,12 +439,6 @@ sunkbd_read_char(keyboard_t *kbd, int wait)
 		goto process_code;
 	}
 
-	if (sc->sc_checked_key != -1) {
-		suncode = sc->sc_checked_key;
-		sc->sc_checked_key = -1;
-		goto process_code;
-	}
-
 	for (;;) {
  next_code:
 		if (!(sc->sc_flags & KPCOMPOSE) && (sc->sc_composed_char > 0)) {
@@ -659,7 +650,10 @@ static int
 sunkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t data)
 {
 	struct sunkbd_softc *sc;
-	int c, error, ival;
+	int c, error;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5)
+	int ival;
+#endif
 
 	sc = (struct sunkbd_softc *)kbd;
 	error = 0;
@@ -667,10 +661,12 @@ sunkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t data)
 	case KDGKBMODE:
 		*(int *)data = sc->sc_mode;
 		break;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5)
 	case _IO('K', 7):
 		ival = IOCPARM_IVAL(data);
 		data = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSKBMODE:
 		switch (*(int *)data) {
 		case K_XLATE:
@@ -695,10 +691,12 @@ sunkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t data)
 	case KDGETLED:
 		*(int *)data = KBD_LED_VAL(kbd);
 		break;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5)
 	case _IO('K', 66):
 		ival = IOCPARM_IVAL(data);
 		data = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSETLED:
 		if (*(int *)data & ~LOCK_MASK) {
 			error = EINVAL;
@@ -713,19 +711,21 @@ sunkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t data)
 			c |= SKBD_LED_NUMLOCK;
 		if (*(int *)data & SLKED)
 			c |= SKBD_LED_SCROLLLOCK;
-		mtx_lock_spin(&sc->sc_uart->sc_hwmtx);
-		sc->sc_sysdev->ops.putc(&sc->sc_sysdev->bas, SKBD_CMD_SETLED);
-		sc->sc_sysdev->ops.putc(&sc->sc_sysdev->bas, c);
-		mtx_unlock_spin(&sc->sc_uart->sc_hwmtx);
+		uart_lock(sc->sc_sysdev->hwmtx);
+		sc->sc_sysdev->ops->putc(&sc->sc_sysdev->bas, SKBD_CMD_SETLED);
+		sc->sc_sysdev->ops->putc(&sc->sc_sysdev->bas, c);
+		uart_unlock(sc->sc_sysdev->hwmtx);
 		KBD_LED_VAL(kbd) = *(int *)data;
 		break;
 	case KDGKBSTATE:
 		*(int *)data = sc->sc_state & LOCK_MASK;
 		break;
+#if defined(COMPAT_FREEBSD6) || defined(COMPAT_FREEBSD5)
 	case _IO('K', 20):
 		ival = IOCPARM_IVAL(data);
 		data = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSKBSTATE:
 		if (*(int *)data & ~LOCK_MASK) {
 			error = EINVAL;
@@ -761,7 +761,6 @@ sunkbd_clear_state(keyboard_t *kbd)
 	struct sunkbd_softc *sc;
 
 	sc = (struct sunkbd_softc *)kbd;
-	sc->sc_checked_key = -1;
 	sc->sc_repeat_key = -1;
 	sc->sc_accents = 0;
 	sc->sc_composed_char = 0;
