@@ -4,7 +4,7 @@
  * Copyright (c) 2000 Jason L. Wright (jason@thought.net)
  * Copyright (c) 2000 Theo de Raadt (deraadt@openbsd.org)
  * Copyright (c) 2001 Patrik Lindergren (patrik@ipunplugged.com)
- * 
+ *
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,7 +39,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/ubsec/ubsec.c,v 1.31.2.1 2006/09/04 15:18:50 pjd Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/ubsec/ubsec.c,v 1.49 2007/03/21 03:42:50 sam Exp $");
 
 /*
  * uBsec 5[56]01, 58xx hardware crypto accelerator
@@ -73,6 +73,9 @@ __FBSDID("$FreeBSD: src/sys/dev/ubsec/ubsec.c,v 1.31.2.1 2006/09/04 15:18:50 pjd
 #include <opencrypto/cryptosoft.h>
 #include <sys/md5.h>
 #include <sys/random.h>
+#include <sys/kobj.h>
+
+#include "cryptodev_if.h"
 
 #include <dev/pci/pcivar.h>
 #include <dev/pci/pcireg.h>
@@ -84,7 +87,7 @@ __FBSDID("$FreeBSD: src/sys/dev/ubsec/ubsec.c,v 1.31.2.1 2006/09/04 15:18:50 pjd
 #define	SIMPLEQ_INSERT_TAIL	STAILQ_INSERT_TAIL
 #define	SIMPLEQ_EMPTY		STAILQ_EMPTY
 #define	SIMPLEQ_FIRST		STAILQ_FIRST
-#define	SIMPLEQ_REMOVE_HEAD	STAILQ_REMOVE_HEAD_UNTIL
+#define	SIMPLEQ_REMOVE_HEAD	STAILQ_REMOVE_HEAD
 #define	SIMPLEQ_FOREACH		STAILQ_FOREACH
 /* ditto for endian.h */
 #define	letoh16(x)		le16toh(x)
@@ -106,6 +109,11 @@ static	int ubsec_suspend(device_t);
 static	int ubsec_resume(device_t);
 static	void ubsec_shutdown(device_t);
 
+static	int ubsec_newsession(device_t, u_int32_t *, struct cryptoini *);
+static	int ubsec_freesession(device_t, u_int64_t);
+static	int ubsec_process(device_t, struct cryptop *, int);
+static	int ubsec_kprocess(device_t, struct cryptkop *, int);
+
 static device_method_t ubsec_methods[] = {
 	/* Device interface */
 	DEVMETHOD(device_probe,		ubsec_probe),
@@ -119,6 +127,12 @@ static device_method_t ubsec_methods[] = {
 	DEVMETHOD(bus_print_child,	bus_generic_print_child),
 	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
 
+	/* crypto device methods */
+	DEVMETHOD(cryptodev_newsession,	ubsec_newsession),
+	DEVMETHOD(cryptodev_freesession,ubsec_freesession),
+	DEVMETHOD(cryptodev_process,	ubsec_process),
+	DEVMETHOD(cryptodev_kprocess,	ubsec_kprocess),
+
 	{ 0, 0 }
 };
 static driver_t ubsec_driver = {
@@ -135,9 +149,6 @@ MODULE_DEPEND(ubsec, rndtest, 1, 1, 1);
 #endif
 
 static	void ubsec_intr(void *);
-static	int ubsec_newsession(void *, u_int32_t *, struct cryptoini *);
-static	int ubsec_freesession(void *, u_int64_t);
-static	int ubsec_process(void *, struct cryptop *, int);
 static	void ubsec_callback(struct ubsec_softc *, struct ubsec_q *);
 static	void ubsec_feed(struct ubsec_softc *);
 static	void ubsec_mcopy(struct mbuf *, struct mbuf *, int, int);
@@ -158,7 +169,6 @@ static	void ubsec_totalreset(struct ubsec_softc *sc);
 
 static	int ubsec_free_q(struct ubsec_softc *sc, struct ubsec_q *q);
 
-static	int ubsec_kprocess(void*, struct cryptkop *, int);
 static	int ubsec_kprocess_modexp_hw(struct ubsec_softc *, struct cryptkop *, int);
 static	int ubsec_kprocess_modexp_sw(struct ubsec_softc *, struct cryptkop *, int);
 static	int ubsec_kprocess_rsapriv(struct ubsec_softc *, struct cryptkop *, int);
@@ -301,7 +311,7 @@ ubsec_attach(device_t dev)
 		sc->sc_flags |= UBS_FLAGS_KEY | UBS_FLAGS_RNG |
 		    UBS_FLAGS_LONGCTX | UBS_FLAGS_HWNORM | UBS_FLAGS_BIGKEY;
 	}
- 
+
 	cmd = pci_read_config(dev, PCIR_COMMAND, 4);
 	cmd |= PCIM_CMD_MEMEN | PCIM_CMD_BUSMASTEREN;
 	pci_write_config(dev, PCIR_COMMAND, cmd, 4);
@@ -317,7 +327,7 @@ ubsec_attach(device_t dev)
 		goto bad;
 	}
 
-	/* 
+	/*
 	 * Setup memory-mapping of PCI registers.
 	 */
 	rid = BS_BAR;
@@ -345,12 +355,12 @@ ubsec_attach(device_t dev)
 	 *     so make sure the IRQ is mapped appropriately.
 	 */
 	if (bus_setup_intr(dev, sc->sc_irq, INTR_TYPE_NET | INTR_MPSAFE,
-			   ubsec_intr, sc, &sc->sc_ih)) {
+			   NULL, ubsec_intr, sc, &sc->sc_ih)) {
 		device_printf(dev, "could not establish interrupt\n");
 		goto bad2;
 	}
 
-	sc->sc_cid = crypto_get_driverid(0);
+	sc->sc_cid = crypto_get_driverid(dev, CRYPTOCAP_F_HARDWARE);
 	if (sc->sc_cid < 0) {
 		device_printf(dev, "could not get crypto driver id\n");
 		goto bad3;
@@ -405,14 +415,10 @@ ubsec_attach(device_t dev)
 
 	device_printf(sc->sc_dev, "%s\n", ubsec_partname(sc));
 
-	crypto_register(sc->sc_cid, CRYPTO_3DES_CBC, 0, 0,
-	    ubsec_newsession, ubsec_freesession, ubsec_process, sc);
-	crypto_register(sc->sc_cid, CRYPTO_DES_CBC, 0, 0,
-	     ubsec_newsession, ubsec_freesession, ubsec_process, sc);
-	crypto_register(sc->sc_cid, CRYPTO_MD5_HMAC, 0, 0,
-	     ubsec_newsession, ubsec_freesession, ubsec_process, sc);
-	crypto_register(sc->sc_cid, CRYPTO_SHA1_HMAC, 0, 0,
-	     ubsec_newsession, ubsec_freesession, ubsec_process, sc);
+	crypto_register(sc->sc_cid, CRYPTO_3DES_CBC, 0, 0);
+	crypto_register(sc->sc_cid, CRYPTO_DES_CBC, 0, 0);
+	crypto_register(sc->sc_cid, CRYPTO_MD5_HMAC, 0, 0);
+	crypto_register(sc->sc_cid, CRYPTO_SHA1_HMAC, 0, 0);
 
 	/*
 	 * Reset Broadcom chip
@@ -475,11 +481,9 @@ skip_rng:
 	if (sc->sc_flags & UBS_FLAGS_KEY) {
 		sc->sc_statmask |= BS_STAT_MCR2_DONE;
 
-		crypto_kregister(sc->sc_cid, CRK_MOD_EXP, 0,
-			ubsec_kprocess, sc);
+		crypto_kregister(sc->sc_cid, CRK_MOD_EXP, 0);
 #if 0
-		crypto_kregister(sc->sc_cid, CRK_MOD_EXP_CRT, 0,
-			ubsec_kprocess, sc);
+		crypto_kregister(sc->sc_cid, CRK_MOD_EXP_CRT, 0);
 #endif
 	}
 	return (0);
@@ -522,7 +526,7 @@ ubsec_detach(device_t dev)
 		struct ubsec_q *q;
 
 		q = SIMPLEQ_FIRST(&sc->sc_freequeue);
-		SIMPLEQ_REMOVE_HEAD(&sc->sc_freequeue, q, q_next);
+		SIMPLEQ_REMOVE_HEAD(&sc->sc_freequeue, q_next);
 		ubsec_dma_free(sc, &q->q_dma->d_alloc);
 		free(q, M_DEVBUF);
 	}
@@ -618,7 +622,7 @@ ubsec_intr(void *arg)
 			if ((dmap->d_dma->d_mcr.mcr_flags & htole16(UBS_MCR_DONE)) == 0)
 				break;
 
-			SIMPLEQ_REMOVE_HEAD(&sc->sc_qchip, q, q_next);
+			SIMPLEQ_REMOVE_HEAD(&sc->sc_qchip, q_next);
 
 			npkts = q->q_nstacked_mcrs;
 			sc->sc_nqchip -= 1+npkts;
@@ -666,7 +670,7 @@ ubsec_intr(void *arg)
 				    BUS_DMASYNC_PREREAD|BUS_DMASYNC_PREWRITE);
 				break;
 			}
-			SIMPLEQ_REMOVE_HEAD(&sc->sc_qchip2, q2, q_next);
+			SIMPLEQ_REMOVE_HEAD(&sc->sc_qchip2, q_next);
 			ubsec_callback2(sc, q2);
 			/*
 			 * Don't send any more packet to chip if there has been
@@ -771,7 +775,7 @@ ubsec_feed(struct ubsec_softc *sc)
 #endif /* UBSEC_DEBUG */
 
 	q = SIMPLEQ_FIRST(&sc->sc_queue);
-	SIMPLEQ_REMOVE_HEAD(&sc->sc_queue, q, q_next);
+	SIMPLEQ_REMOVE_HEAD(&sc->sc_queue, q_next);
 	--sc->sc_nqueue;
 
 	bus_dmamap_sync(sc->sc_dmat, q->q_src_map, BUS_DMASYNC_PREWRITE);
@@ -787,7 +791,7 @@ ubsec_feed(struct ubsec_softc *sc)
 		if (q2->q_dst_map != NULL)
 			bus_dmamap_sync(sc->sc_dmat, q2->q_dst_map,
 			    BUS_DMASYNC_PREREAD);
-		SIMPLEQ_REMOVE_HEAD(&sc->sc_queue, q2, q_next);
+		SIMPLEQ_REMOVE_HEAD(&sc->sc_queue, q_next);
 		--sc->sc_nqueue;
 
 		v = (void*)(((char *)&q2->q_dma->d_dma->d_mcr) + sizeof(struct ubsec_mcr) -
@@ -822,7 +826,7 @@ feed1:
 		      q, (u_int32_t)vtophys(&q->q_dma->d_dma->d_mcr),
 		      stat);
 #endif /* UBSEC_DEBUG */
-	SIMPLEQ_REMOVE_HEAD(&sc->sc_queue, q, q_next);
+	SIMPLEQ_REMOVE_HEAD(&sc->sc_queue, q_next);
 	--sc->sc_nqueue;
 	SIMPLEQ_INSERT_TAIL(&sc->sc_qchip, q, q_next);
 	sc->sc_nqchip++;
@@ -900,10 +904,10 @@ ubsec_setup_mackey(struct ubsec_session *ses, int algo, caddr_t key, int klen)
  * id on successful allocation.
  */
 static int
-ubsec_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
+ubsec_newsession(device_t dev, u_int32_t *sidp, struct cryptoini *cri)
 {
+	struct ubsec_softc *sc = device_get_softc(dev);
 	struct cryptoini *c, *encini = NULL, *macini = NULL;
-	struct ubsec_softc *sc = arg;
 	struct ubsec_session *ses = NULL;
 	int sesn;
 
@@ -995,9 +999,9 @@ ubsec_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
  * Deallocate a session.
  */
 static int
-ubsec_freesession(void *arg, u_int64_t tid)
+ubsec_freesession(device_t dev, u_int64_t tid)
 {
-	struct ubsec_softc *sc = arg;
+	struct ubsec_softc *sc = device_get_softc(dev);
 	int session, ret;
 	u_int32_t sid = CRYPTO_SESID2LID(tid);
 
@@ -1016,44 +1020,30 @@ ubsec_freesession(void *arg, u_int64_t tid)
 }
 
 static void
-ubsec_op_cb(struct ubsec_operand *op, bus_dma_segment_t *seg, int nsegs,
-    int error)
+ubsec_op_cb(void *arg, bus_dma_segment_t *seg, int nsegs, bus_size_t mapsize, int error)
 {
+	struct ubsec_operand *op = arg;
 
 	KASSERT(nsegs <= UBS_MAX_SCATTER,
 		("Too many DMA segments returned when mapping operand"));
 #ifdef UBSEC_DEBUG
 	if (ubsec_debug)
-		printf("ubsec_op_cb: mapsize %u nsegs %d error=%d\n",
-			(u_int)op->mapsize, nsegs, error);
+		printf("ubsec_op_cb: mapsize %u nsegs %d error %d\n",
+			(u_int) mapsize, nsegs, error);
 #endif
 	if (error != 0)
 		return;
+	op->mapsize = mapsize;
 	op->nsegs = nsegs;
 	bcopy(seg, op->segs, nsegs * sizeof (seg[0]));
 }
 
-static void
-ubsec_op_cb1(void *arg, bus_dma_segment_t *seg, int nsegs, int error)
-{
-
-	ubsec_op_cb(arg, seg, nsegs, error);
-}
-
-static void
-ubsec_op_cb2(void *arg, bus_dma_segment_t *seg, int nsegs,
-    bus_size_t mapsize __unused, int error)
-{
-
-	ubsec_op_cb(arg, seg, nsegs, error);
-}
-
 static int
-ubsec_process(void *arg, struct cryptop *crp, int hint)
+ubsec_process(device_t dev, struct cryptop *crp, int hint)
 {
+	struct ubsec_softc *sc = device_get_softc(dev);
 	struct ubsec_q *q = NULL;
 	int err = 0, i, j, nicealign;
-	struct ubsec_softc *sc = arg;
 	struct cryptodesc *crd1, *crd2, *maccrd, *enccrd;
 	int encoffset = 0, macoffset = 0, cpskip, cpoffset;
 	int sskip, dskip, stheend, dtheend;
@@ -1079,7 +1069,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 		return (ERESTART);
 	}
 	q = SIMPLEQ_FIRST(&sc->sc_freequeue);
-	SIMPLEQ_REMOVE_HEAD(&sc->sc_freequeue, q, q_next);
+	SIMPLEQ_REMOVE_HEAD(&sc->sc_freequeue, q_next);
 	mtx_unlock(&sc->sc_freeqlock);
 
 	dmap = q->q_dma; /* Save dma pointer */
@@ -1097,8 +1087,9 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 		q->q_src_io = (struct uio *)crp->crp_buf;
 		q->q_dst_io = (struct uio *)crp->crp_buf;
 	} else {
-		q->q_src_buf = (void *)crp->crp_buf;
-		q->q_dst_buf = (void *)crp->crp_buf;
+		ubsecstats.hst_badflags++;
+		err = EINVAL;
+		goto errout;	/* XXX we don't handle contiguous blocks! */
 	}
 
 	bzero(&dmap->d_dma->d_mcr, sizeof(struct ubsec_mcr));
@@ -1106,8 +1097,6 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 	dmap->d_dma->d_mcr.mcr_pkts = htole16(1);
 	dmap->d_dma->d_mcr.mcr_flags = 0;
 	q->q_crp = crp;
-	q->q_src.mapsize = crp->crp_ilen;
-	q->q_dst.mapsize = crp->crp_ilen;
 
 	crd1 = crp->crp_desc;
 	if (crd1 == NULL) {
@@ -1271,21 +1260,23 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 		goto errout;
 	}
 	if (crp->crp_flags & CRYPTO_F_IMBUF) {
-		err = bus_dmamap_load_mbuf(sc->sc_dmat, q->q_src_map,
-		    q->q_src_m, ubsec_op_cb2, &q->q_src, BUS_DMA_NOWAIT);
+		if (bus_dmamap_load_mbuf(sc->sc_dmat, q->q_src_map,
+		    q->q_src_m, ubsec_op_cb, &q->q_src, BUS_DMA_NOWAIT) != 0) {
+			bus_dmamap_destroy(sc->sc_dmat, q->q_src_map);
+			q->q_src_map = NULL;
+			ubsecstats.hst_noload++;
+			err = ENOMEM;
+			goto errout;
+		}
 	} else if (crp->crp_flags & CRYPTO_F_IOV) {
-		err = bus_dmamap_load_uio(sc->sc_dmat, q->q_src_map,
-		    q->q_src_io, ubsec_op_cb2, &q->q_src, BUS_DMA_NOWAIT);
-	} else {
-		err = bus_dmamap_load(sc->sc_dmat, q->q_src_map, q->q_src_buf,
-		    crp->crp_ilen, ubsec_op_cb1, &q->q_src, BUS_DMA_NOWAIT);
-	}
-	if (err != 0) {
-		bus_dmamap_destroy(sc->sc_dmat, q->q_src_map);
-		q->q_src_map = NULL;
-		ubsecstats.hst_noload++;
-		err = ENOMEM;
-		goto errout;
+		if (bus_dmamap_load_uio(sc->sc_dmat, q->q_src_map,
+		    q->q_src_io, ubsec_op_cb, &q->q_src, BUS_DMA_NOWAIT) != 0) {
+			bus_dmamap_destroy(sc->sc_dmat, q->q_src_map);
+			q->q_src_map = NULL;
+			ubsecstats.hst_noload++;
+			err = ENOMEM;
+			goto errout;
+		}
 	}
 	nicealign = ubsec_dmamap_aligned(&q->q_src);
 
@@ -1366,8 +1357,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 				goto errout;
 			}
 			if (bus_dmamap_load_uio(sc->sc_dmat, q->q_dst_map,
-			    q->q_dst_io, ubsec_op_cb2, &q->q_dst,
-			    BUS_DMA_NOWAIT) != 0) {
+			    q->q_dst_io, ubsec_op_cb, &q->q_dst, BUS_DMA_NOWAIT) != 0) {
 				bus_dmamap_destroy(sc->sc_dmat, q->q_dst_map);
 				q->q_dst_map = NULL;
 				ubsecstats.hst_noload++;
@@ -1408,7 +1398,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 				totlen -= len;
 				top = m;
 				mp = &top;
-				
+
 				while (totlen > 0) {
 					if (totlen >= MINCLSIZE) {
 						m = m_getcl(M_DONTWAIT,
@@ -1432,7 +1422,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 				q->q_dst_m = top;
 				ubsec_mcopy(q->q_src_m, q->q_dst_m,
 				    cpskip, cpoffset);
-				if (bus_dmamap_create(sc->sc_dmat, 
+				if (bus_dmamap_create(sc->sc_dmat,
 				    BUS_DMA_NOWAIT, &q->q_dst_map) != 0) {
 					ubsecstats.hst_nomap++;
 					err = ENOMEM;
@@ -1440,7 +1430,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 				}
 				if (bus_dmamap_load_mbuf(sc->sc_dmat,
 				    q->q_dst_map, q->q_dst_m,
-				    ubsec_op_cb2, &q->q_dst,
+				    ubsec_op_cb, &q->q_dst,
 				    BUS_DMA_NOWAIT) != 0) {
 					bus_dmamap_destroy(sc->sc_dmat,
 					q->q_dst_map);
@@ -1451,22 +1441,9 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 				}
 			}
 		} else {
-			KASSERT(nicealign, ("no nicealign"));
-			if (bus_dmamap_create(sc->sc_dmat, BUS_DMA_NOWAIT,
-			     &q->q_dst_map)) {
-				ubsecstats.hst_nomap++;
-				err = ENOMEM;
-				goto errout;
-			}
-			if (bus_dmamap_load(sc->sc_dmat, q->q_dst_map,
-			    q->q_dst_buf, crp->crp_ilen, ubsec_op_cb1,
-			    &q->q_dst, BUS_DMA_NOWAIT) != 0) {
-				bus_dmamap_destroy(sc->sc_dmat, q->q_dst_map);
-				q->q_dst_map = NULL;
-				ubsecstats.hst_noload++;
-				err = ENOMEM;
-				goto errout;
-			}
+			ubsecstats.hst_badflags++;
+			err = EINVAL;
+			goto errout;
 		}
 
 #ifdef UBSEC_DEBUG
@@ -1531,7 +1508,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 
 		ctxl = (struct ubsec_pktctx_long *)(dmap->d_alloc.dma_vaddr +
 		    offsetof(struct ubsec_dmachunk, d_ctx));
-		
+
 		/* transform small context into long context */
 		ctxl->pc_len = htole16(sizeof(struct ubsec_pktctx_long));
 		ctxl->pc_type = htole16(UBS_PKTCTX_TYPE_IPSEC);
@@ -1542,7 +1519,7 @@ ubsec_process(void *arg, struct cryptop *crp, int hint)
 		for (i = 0; i < 5; i++)
 			ctxl->pc_hminner[i] = ctx.pc_hminner[i];
 		for (i = 0; i < 5; i++)
-			ctxl->pc_hmouter[i] = ctx.pc_hmouter[i];   
+			ctxl->pc_hmouter[i] = ctx.pc_hmouter[i];
 		ctxl->pc_iv[0] = ctx.pc_iv[0];
 		ctxl->pc_iv[1] = ctx.pc_iv[1];
 	} else
@@ -1699,7 +1676,7 @@ ubsec_feed2(struct ubsec_softc *sc)
 		ubsec_dma_sync(&q->q_ctx, BUS_DMASYNC_PREWRITE);
 
 		WRITE_REG(sc, BS_MCR2, q->q_mcr.dma_paddr);
-		SIMPLEQ_REMOVE_HEAD(&sc->sc_queue2, q, q_next);
+		SIMPLEQ_REMOVE_HEAD(&sc->sc_queue2, q_next);
 		--sc->sc_nqueue2;
 		SIMPLEQ_INSERT_TAIL(&sc->sc_qchip2, q, q_next);
 	}
@@ -2024,7 +2001,7 @@ ubsec_cleanchip(struct ubsec_softc *sc)
 
 	while (!SIMPLEQ_EMPTY(&sc->sc_qchip)) {
 		q = SIMPLEQ_FIRST(&sc->sc_qchip);
-		SIMPLEQ_REMOVE_HEAD(&sc->sc_qchip, q, q_next);
+		SIMPLEQ_REMOVE_HEAD(&sc->sc_qchip, q_next);
 		ubsec_free_q(sc, q);
 	}
 	sc->sc_nqchip = 0;
@@ -2048,13 +2025,13 @@ ubsec_free_q(struct ubsec_softc *sc, struct ubsec_q *q)
 		if(q->q_stacked_mcr[i]) {
 			q2 = q->q_stacked_mcr[i];
 
-			if ((q2->q_dst_m != NULL) && (q2->q_src_m != q2->q_dst_m)) 
+			if ((q2->q_dst_m != NULL) && (q2->q_src_m != q2->q_dst_m))
 				m_freem(q2->q_dst_m);
 
 			crp = (struct cryptop *)q2->q_crp;
-			
+
 			SIMPLEQ_INSERT_TAIL(&sc->sc_freequeue, q2, q_next);
-			
+
 			crp->crp_etype = EFAULT;
 			crypto_done(crp);
 		} else {
@@ -2069,9 +2046,9 @@ ubsec_free_q(struct ubsec_softc *sc, struct ubsec_q *q)
 		m_freem(q->q_dst_m);
 
 	crp = (struct cryptop *)q->q_crp;
-	
+
 	SIMPLEQ_INSERT_TAIL(&sc->sc_freequeue, q, q_next);
-	
+
 	crp->crp_etype = EFAULT;
 	crypto_done(crp);
 	return(0);
@@ -2137,9 +2114,9 @@ ubsec_kfree(struct ubsec_softc *sc, struct ubsec_q2 *q)
 }
 
 static int
-ubsec_kprocess(void *arg, struct cryptkop *krp, int hint)
+ubsec_kprocess(device_t dev, struct cryptkop *krp, int hint)
 {
-	struct ubsec_softc *sc = arg;
+	struct ubsec_softc *sc = device_get_softc(dev);
 	int r;
 
 	if (krp == NULL || krp->krp_callback == NULL)
@@ -2149,7 +2126,7 @@ ubsec_kprocess(void *arg, struct cryptkop *krp, int hint)
 		struct ubsec_q2 *q;
 
 		q = SIMPLEQ_FIRST(&sc->sc_q2free);
-		SIMPLEQ_REMOVE_HEAD(&sc->sc_q2free, q, q_next);
+		SIMPLEQ_REMOVE_HEAD(&sc->sc_q2free, q_next);
 		ubsec_kfree(sc, q);
 	}
 
