@@ -1,4 +1,4 @@
-/* $FreeBSD: src/sys/dev/ubsec/ubsecvar.h,v 1.8.2.1 2006/09/04 15:18:50 pjd Exp $ */
+/* $FreeBSD: src/sys/dev/ubsec/ubsecvar.h,v 1.9 2006/05/17 18:34:26 pjd Exp $ */
 /*	$OpenBSD: ubsecvar.h,v 1.35 2002/09/24 18:33:26 jason Exp $	*/
 
 /*-
@@ -139,7 +139,6 @@ struct ubsec_operand {
 	union {
 		struct mbuf *m;
 		struct uio *io;
-		void *buf;
 	} u;
 	bus_dmamap_t		map;
 	bus_size_t		mapsize;
@@ -163,7 +162,6 @@ struct ubsec_q {
 
 #define	q_src_m		q_src.u.m
 #define	q_src_io	q_src.u.io
-#define	q_src_buf	q_src.u.buf
 #define	q_src_map	q_src.map
 #define	q_src_nsegs	q_src.nsegs
 #define	q_src_segs	q_src.segs
@@ -171,7 +169,6 @@ struct ubsec_q {
 
 #define	q_dst_m		q_dst.u.m
 #define	q_dst_io	q_dst.u.io
-#define	q_dst_buf	q_dst.u.buf
 #define	q_dst_map	q_dst.map
 #define	q_dst_nsegs	q_dst.nsegs
 #define	q_dst_segs	q_dst.segs
