@@ -1,6 +1,6 @@
 /*	$NetBSD: if_udav.c,v 1.2 2003/09/04 15:17:38 tsutsui Exp $	*/
 /*	$nabe: if_udav.c,v 1.3 2003/08/21 16:57:19 nabe Exp $	*/
-/*	$FreeBSD: src/sys/dev/usb/if_udav.c,v 1.14.2.5 2007/05/21 18:10:48 brueffer Exp $	*/
+/*	$FreeBSD: src/sys/dev/usb/if_udav.c,v 1.33 2007/07/24 14:44:23 imp Exp $	*/
 /*-
  * Copyright (c) 2003
  *     Shingo WATANABE <nabe@nabechan.org>.  All rights reserved.
@@ -45,7 +45,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/usb/if_udav.c,v 1.14.2.5 2007/05/21 18:10:48 brueffer Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/usb/if_udav.c,v 1.33 2007/07/24 14:44:23 imp Exp $");
 
 #include "opt_inet.h"
 #if defined(__NetBSD__)
@@ -78,7 +78,7 @@ __FBSDID("$FreeBSD: src/sys/dev/usb/if_udav.c,v 1.14.2.5 2007/05/21 18:10:48 bru
 #include <sys/device.h>
 #endif
 
-#if NRND > 0
+#if defined(NRND) && NRND > 0
 #include <sys/rnd.h>
 #endif
 
@@ -126,6 +126,7 @@ __FBSDID("$FreeBSD: src/sys/dev/usb/if_udav.c,v 1.14.2.5 2007/05/21 18:10:48 bru
 #include <dev/mii/mii.h>
 #include <dev/mii/miivar.h>
 
+#include <dev/usb/usb_port.h>
 #include <dev/usb/usb.h>
 #include <dev/usb/usbdi.h>
 #include <dev/usb/usbdi_util.h>
@@ -141,7 +142,7 @@ MODULE_DEPEND(udav, ether, 1, 1, 1);
 MODULE_DEPEND(udav, miibus, 1, 1, 1);
 #endif
 
-/* "controller miibus0" required.  See GENERIC if you get errors here. */
+/* "device miibus" required.  See GENERIC if you get errors here. */
 #include "miibus_if.h"
 
 #if !defined(__FreeBSD__)
@@ -150,54 +151,57 @@ USB_DECLARE_DRIVER(udav);
 #endif
 
 #if defined(__FreeBSD__)
-Static int udav_match(device_ptr_t);
-Static int udav_attach(device_ptr_t);
-Static int udav_detach(device_ptr_t);
-Static void udav_shutdown(device_ptr_t);
+static device_probe_t udav_match;
+static device_attach_t udav_attach;
+static device_detach_t udav_detach;
+static device_shutdown_t udav_shutdown;
+static miibus_readreg_t udav_miibus_readreg;
+static miibus_writereg_t udav_miibus_writereg;
+static miibus_statchg_t udav_miibus_statchg;
 #endif
 
-Static int udav_openpipes(struct udav_softc *);
-Static void udav_start(struct ifnet *);
-Static int udav_send(struct udav_softc *, struct mbuf *, int);
-Static void udav_txeof(usbd_xfer_handle, usbd_private_handle, usbd_status);
+static int udav_openpipes(struct udav_softc *);
+static void udav_start(struct ifnet *);
+static int udav_send(struct udav_softc *, struct mbuf *, int);
+static void udav_txeof(usbd_xfer_handle, usbd_private_handle, usbd_status);
 #if defined(__FreeBSD__)
-Static void udav_rxstart(struct ifnet *ifp);
+static void udav_rxstart(struct ifnet *ifp);
 #endif
-Static void udav_rxeof(usbd_xfer_handle, usbd_private_handle, usbd_status);
-Static void udav_tick(void *);
-Static void udav_tick_task(void *);
-Static int udav_ioctl(struct ifnet *, u_long, caddr_t);
-Static void udav_stop_task(struct udav_softc *);
-Static void udav_stop(struct ifnet *, int);
-Static void udav_watchdog(struct ifnet *);
-Static int udav_ifmedia_change(struct ifnet *);
-Static void udav_ifmedia_status(struct ifnet *, struct ifmediareq *);
-Static void udav_lock_mii(struct udav_softc *);
-Static void udav_unlock_mii(struct udav_softc *);
-Static int udav_miibus_readreg(device_ptr_t, int, int);
-Static void udav_miibus_writereg(device_ptr_t, int, int, int);
-Static void udav_miibus_statchg(device_ptr_t);
+static void udav_rxeof(usbd_xfer_handle, usbd_private_handle, usbd_status);
+static void udav_tick(void *);
+static void udav_tick_task(void *);
+static int udav_ioctl(struct ifnet *, u_long, caddr_t);
+static void udav_stop_task(struct udav_softc *);
+static void udav_stop(struct ifnet *, int);
+static void udav_watchdog(struct ifnet *);
+static int udav_ifmedia_change(struct ifnet *);
+static void udav_ifmedia_status(struct ifnet *, struct ifmediareq *);
+static void udav_lock_mii(struct udav_softc *);
+static void udav_unlock_mii(struct udav_softc *);
 #if defined(__NetBSD__)
-Static int udav_init(struct ifnet *);
+static int udav_miibus_readreg(device_t, int, int);
+static void udav_miibus_writereg(device_t, int, int, int);
+static void udav_miibus_statchg(device_t);
+static int udav_init(struct ifnet *);
 #elif defined(__FreeBSD__)
-Static void udav_init(void *);
+static void udav_init(void *);
 #endif
-Static void udav_setmulti(struct udav_softc *);
-Static void udav_reset(struct udav_softc *);
+static void udav_setmulti(struct udav_softc *);
+static void udav_reset(struct udav_softc *);
 
-Static int udav_csr_read(struct udav_softc *, int, void *, int);
-Static int udav_csr_write(struct udav_softc *, int, void *, int);
-Static int udav_csr_read1(struct udav_softc *, int);
-Static int udav_csr_write1(struct udav_softc *, int, unsigned char);
+static int udav_csr_read(struct udav_softc *, int, void *, int);
+static int udav_csr_write(struct udav_softc *, int, void *, int);
+static int udav_csr_read1(struct udav_softc *, int);
+static int udav_csr_write1(struct udav_softc *, int, unsigned char);
 
 #if 0
-Static int udav_mem_read(struct udav_softc *, int, void *, int);
-Static int udav_mem_write(struct udav_softc *, int, void *, int);
-Static int udav_mem_write1(struct udav_softc *, int, unsigned char);
+static int udav_mem_read(struct udav_softc *, int, void *, int);
+static int udav_mem_write(struct udav_softc *, int, void *, int);
+static int udav_mem_write1(struct udav_softc *, int, unsigned char);
 #endif
 
 #if defined(__FreeBSD__)
-Static device_method_t udav_methods[] = {
+static device_method_t udav_methods[] = {
 	/* Device interface */
 	DEVMETHOD(device_probe,		udav_match),
 	DEVMETHOD(device_attach,	udav_attach),
@@ -216,13 +220,13 @@ Static device_method_t udav_methods[] = {
 	{ 0, 0 }
 };
 
-Static driver_t udav_driver = {
+static driver_t udav_driver = {
 	"udav",
 	udav_methods,
 	sizeof(struct udav_softc)
 };
 
-Static devclass_t udav_devclass;
+static devclass_t udav_devclass;
 
 DRIVER_MODULE(udav, uhub, udav_driver, udav_devclass, usbd_driver_load, 0);
 DRIVER_MODULE(miibus, udav, miibus_driver, miibus_devclass, 0, 0);
@@ -231,8 +235,8 @@ DRIVER_MODULE(miibus, udav, miibus_driver, miibus_devclass, 0, 0);
 
 /* Macros */
 #ifdef UDAV_DEBUG
-#define DPRINTF(x)	if (udavdebug) logprintf x
-#define DPRINTFN(n,x)	if (udavdebug >= (n)) logprintf x
+#define DPRINTF(x)	if (udavdebug) printf x
+#define DPRINTFN(n,x)	if (udavdebug >= (n)) printf x
 int udavdebug = 0;
 #else
 #define DPRINTF(x)
@@ -254,6 +258,8 @@ static const struct udav_type {
 } udav_devs [] = {
 	/* Corega USB-TXC */
 	{{ USB_VENDOR_COREGA, USB_PRODUCT_COREGA_FETHER_USB_TXC }, 0},
+	/* ShanTou ST268 USB NIC */
+	{{ USB_VENDOR_SHANTOU, USB_PRODUCT_SHANTOU_ST268 }, 0},
 #if 0
 	/* DAVICOM DM9601 Generic? */
 	/*  XXX: The following ids was obtained from the data sheet. */
@@ -264,9 +270,10 @@ static const struct udav_type {
 
 
 /* Probe */
-USB_MATCH(udav)
+static int
+udav_match(device_t self)
 {
-	USB_MATCH_START(udav, uaa);
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 
 	if (uaa->iface != NULL)
 		return (UMATCH_NONE);
@@ -276,7 +283,8 @@ USB_MATCH(udav)
 }
 
 /* Attach */
-USB_ATTACH(udav)
+static int
+udav_attach(device_t self)
 {
 	USB_ATTACH_START(udav, sc, uaa);
 	usbd_device_handle dev = uaa->device;
@@ -284,7 +292,6 @@ USB_ATTACH(udav)
 	usbd_status err;
 	usb_interface_descriptor_t *id;
 	usb_endpoint_descriptor_t *ed;
-	char devinfo[1024];
 	const char *devname ;
 	struct ifnet *ifp;
 #if defined(__NetBSD__)
@@ -296,13 +303,8 @@ USB_ATTACH(udav)
 	int s;
 #endif
 
-	bzero(sc, sizeof(struct udav_softc));
-
-	usbd_devinfo(dev, 0, devinfo);
-	USB_ATTACH_SETUP;
-        devname = USBDEVNAME(sc->sc_dev);
-	printf("%s: %s\n", devname, devinfo);
-
+	sc->sc_dev = self;
+	devname = device_get_nameunit(self);
 	/* Move the device into the configured state. */
 	err = usbd_set_config_no(dev, UDAV_CONFIG_NO, 1);
 	if (err) {
@@ -444,11 +446,11 @@ USB_ATTACH(udav)
 #elif defined(__FreeBSD__)
 	if (mii_phy_probe(self, &sc->sc_miibus,
 	    udav_ifmedia_change, udav_ifmedia_status)) {
-		printf("%s: MII without any PHY!\n", USBDEVNAME(sc->sc_dev));
+		printf("%s: MII without any PHY!\n", device_get_nameunit(sc->sc_dev));
 		if_free(ifp);
 		UDAV_UNLOCK(sc);
 		mtx_destroy(&sc->sc_mtx);
-		USB_ATTACH_ERROR_RETURN;
+		return ENXIO;
 	}
 
 	sc->sc_qdat.ifp = ifp;
@@ -461,7 +463,7 @@ USB_ATTACH(udav)
 	ether_ifattach(ifp, eaddr);
 #endif
 
-#if NRND > 0
+#if defined(NRND) && NRND > 0
 	rnd_attach_source(&sc->rnd_source, devname, RND_TYPE_NET, 0);
 #endif
 
@@ -476,17 +478,18 @@ USB_ATTACH(udav)
         UDAV_UNLOCK(sc);
 #endif
 
-	usbd_add_drv_event(USB_EVENT_DRIVER_ATTACH, dev, USBDEV(sc->sc_dev));
+	usbd_add_drv_event(USB_EVENT_DRIVER_ATTACH, dev, sc->sc_dev);
 
-	USB_ATTACH_SUCCESS_RETURN;
+	return 0;
 
  bad:
 	sc->sc_dying = 1;
-	USB_ATTACH_ERROR_RETURN;
+	return ENXIO;
 }
 
 /* detach */
-USB_DETACH(udav)
+static int
+udav_detach(device_t self)
 {
 	USB_DETACH_START(udav, sc);
 	struct ifnet *ifp = GET_IFP(sc);
@@ -494,7 +497,7 @@ USB_DETACH(udav)
 	int s;
 #endif
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	/* Detached before attached finished */
 	if (!sc->sc_attached)
@@ -516,7 +519,7 @@ USB_DETACH(udav)
 
 	if (--sc->sc_refcnt >= 0) {
 		/* Wait for processes to go away */
-		usb_detach_wait(USBDEV(sc->sc_dev));
+		usb_detach_wait(sc->sc_dev);
 	}
 #if defined(__FreeBSD__)
 	if (ifp->if_drv_flags & IFF_DRV_RUNNING)
@@ -525,7 +528,7 @@ USB_DETACH(udav)
 #endif
 		udav_stop(GET_IFP(sc), 1);
 
-#if NRND > 0
+#if defined(NRND) && NRND > 0
 	rnd_detach_source(&sc->rnd_source);
 #endif
 #if defined(__NetBSD__)
@@ -543,13 +546,13 @@ USB_DETACH(udav)
 #ifdef DIAGNOSTIC
 	if (sc->sc_pipe_tx != NULL)
 		printf("%s: detach has active tx endpoint.\n",
-		       USBDEVNAME(sc->sc_dev));
+		       device_get_nameunit(sc->sc_dev));
 	if (sc->sc_pipe_rx != NULL)
 		printf("%s: detach has active rx endpoint.\n",
-		       USBDEVNAME(sc->sc_dev));
+		       device_get_nameunit(sc->sc_dev));
 	if (sc->sc_pipe_intr != NULL)
 		printf("%s: detach has active intr endpoint.\n",
-		       USBDEVNAME(sc->sc_dev));
+		       device_get_nameunit(sc->sc_dev));
 #endif
 	sc->sc_attached = 0;
 
@@ -563,14 +566,13 @@ USB_DETACH(udav)
 	mtx_destroy(&sc->sc_mtx);
 #endif
 
-	usbd_add_drv_event(USB_EVENT_DRIVER_DETACH, sc->sc_udev,
-			   USBDEV(sc->sc_dev));
+	usbd_add_drv_event(USB_EVENT_DRIVER_DETACH, sc->sc_udev, sc->sc_dev);
 	return (0);
 }
 
 #if 0
 /* read memory */
-Static int
+static int
 udav_mem_read(struct udav_softc *sc, int offset, void *buf, int len)
 {
 	usb_device_request_t req;
@@ -580,7 +582,7 @@ udav_mem_read(struct udav_softc *sc, int offset, void *buf, int len)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -597,17 +599,17 @@ udav_mem_read(struct udav_softc *sc, int offset, void *buf, int len)
 	sc->sc_refcnt++;
 	err = usbd_do_request(sc->sc_udev, &req, buf);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err) {
 		DPRINTF(("%s: %s: read failed. off=%04x, err=%d\n",
-			 USBDEVNAME(sc->sc_dev), __func__, offset, err));
+			 device_get_nameunit(sc->sc_dev), __func__, offset, err));
 	}
 
 	return (err);
 }
 
 /* write memory */
-Static int
+static int
 udav_mem_write(struct udav_softc *sc, int offset, void *buf, int len)
 {
 	usb_device_request_t req;
@@ -617,7 +619,7 @@ udav_mem_write(struct udav_softc *sc, int offset, void *buf, int len)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -634,17 +636,17 @@ udav_mem_write(struct udav_softc *sc, int offset, void *buf, int len)
 	sc->sc_refcnt++;
 	err = usbd_do_request(sc->sc_udev, &req, buf);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err) {
 		DPRINTF(("%s: %s: write failed. off=%04x, err=%d\n",
-			 USBDEVNAME(sc->sc_dev), __func__, offset, err));
+			 device_get_nameunit(sc->sc_dev), __func__, offset, err));
 	}
 
 	return (err);
 }
 
 /* write memory */
-Static int
+static int
 udav_mem_write1(struct udav_softc *sc, int offset, unsigned char ch)
 {
 	usb_device_request_t req;
@@ -654,7 +656,7 @@ udav_mem_write1(struct udav_softc *sc, int offset, unsigned char ch)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -670,10 +672,10 @@ udav_mem_write1(struct udav_softc *sc, int offset, unsigned char ch)
 	sc->sc_refcnt++;
 	err = usbd_do_request(sc->sc_udev, &req, NULL);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err) {
 		DPRINTF(("%s: %s: write failed. off=%04x, err=%d\n",
-			 USBDEVNAME(sc->sc_dev), __func__, offset, err));
+			 device_get_nameunit(sc->sc_dev), __func__, offset, err));
 	}
 
 	return (err);
@@ -681,7 +683,7 @@ udav_mem_write1(struct udav_softc *sc, int offset, unsigned char ch)
 #endif
 
 /* read register(s) */
-Static int
+static int
 udav_csr_read(struct udav_softc *sc, int offset, void *buf, int len)
 {
 	usb_device_request_t req;
@@ -691,7 +693,7 @@ udav_csr_read(struct udav_softc *sc, int offset, void *buf, int len)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -708,17 +710,17 @@ udav_csr_read(struct udav_softc *sc, int offset, void *buf, int len)
 	sc->sc_refcnt++;
 	err = usbd_do_request(sc->sc_udev, &req, buf);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err) {
 		DPRINTF(("%s: %s: read failed. off=%04x, err=%d\n",
-			 USBDEVNAME(sc->sc_dev), __func__, offset, err));
+			 device_get_nameunit(sc->sc_dev), __func__, offset, err));
 	}
 
 	return (err);
 }
 
 /* write register(s) */
-Static int
+static int
 udav_csr_write(struct udav_softc *sc, int offset, void *buf, int len)
 {
 	usb_device_request_t req;
@@ -728,7 +730,7 @@ udav_csr_write(struct udav_softc *sc, int offset, void *buf, int len)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -745,16 +747,16 @@ udav_csr_write(struct udav_softc *sc, int offset, void *buf, int len)
 	sc->sc_refcnt++;
 	err = usbd_do_request(sc->sc_udev, &req, buf);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err) {
 		DPRINTF(("%s: %s: write failed. off=%04x, err=%d\n",
-			 USBDEVNAME(sc->sc_dev), __func__, offset, err));
+			 device_get_nameunit(sc->sc_dev), __func__, offset, err));
 	}
 
 	return (err);
 }
 
-Static int
+static int
 udav_csr_read1(struct udav_softc *sc, int offset)
 {
 	u_int8_t val = 0;
@@ -763,7 +765,7 @@ udav_csr_read1(struct udav_softc *sc, int offset)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -772,7 +774,7 @@ udav_csr_read1(struct udav_softc *sc, int offset)
 }
 
 /* write a register */
-Static int
+static int
 udav_csr_write1(struct udav_softc *sc, int offset, unsigned char ch)
 {
 	usb_device_request_t req;
@@ -782,7 +784,7 @@ udav_csr_write1(struct udav_softc *sc, int offset, unsigned char ch)
 		return (0);
 
 	DPRINTFN(0x200,
-		("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+		("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -798,20 +800,20 @@ udav_csr_write1(struct udav_softc *sc, int offset, unsigned char ch)
 	sc->sc_refcnt++;
 	err = usbd_do_request(sc->sc_udev, &req, NULL);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err) {
 		DPRINTF(("%s: %s: write failed. off=%04x, err=%d\n",
-			 USBDEVNAME(sc->sc_dev), __func__, offset, err));
+			 device_get_nameunit(sc->sc_dev), __func__, offset, err));
 	}
 
 	return (err);
 }
 
 #if defined(__NetBSD__)
-Static int
+static int
 udav_init(struct ifnet *ifp)
 #elif defined(__FreeBSD__)
-Static void
+static void
 udav_init(void *xsc)
 #endif
 {
@@ -827,7 +829,7 @@ udav_init(void *xsc)
 	int s;
 #endif
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 #if defined(__NetBSD__)
@@ -848,7 +850,7 @@ udav_init(void *xsc)
 #if defined(__NetBSD__)
 	eaddr = LLADDR(ifp->if_sadl);
 #elif defined(__FreeBSD__)
-	eaddr = IFP2ENADDR(ifp);
+	eaddr = IF_LLADDR(ifp);
 #endif
 	udav_csr_write(sc, UDAV_PAR, eaddr, ETHER_ADDR_LEN);
 
@@ -868,7 +870,7 @@ udav_init(void *xsc)
 	/* Initialize transmit ring */
 	if (usb_ether_tx_list_init(sc, &sc->sc_cdata,
 	    sc->sc_udev) == ENOBUFS) {
-		printf("%s: tx list init failed\n", USBDEVNAME(sc->sc_dev));
+		printf("%s: tx list init failed\n", device_get_nameunit(sc->sc_dev));
 #if defined(__NetBSD__)
 		splx(s);
 		return (EIO);
@@ -882,7 +884,7 @@ udav_init(void *xsc)
 	/* Initialize receive ring */
 	if (usb_ether_rx_list_init(sc, &sc->sc_cdata,
 	    sc->sc_udev) == ENOBUFS) {
-		printf("%s: rx list init failed\n", USBDEVNAME(sc->sc_dev));
+		printf("%s: rx list init failed\n", device_get_nameunit(sc->sc_dev));
 #if defined(__NetBSD__)
 		splx(s);
 		return (EIO);
@@ -939,12 +941,12 @@ udav_init(void *xsc)
 #endif
 }
 
-Static void
+static void
 udav_reset(struct udav_softc *sc)
 {
 	int i;
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return;
@@ -976,11 +978,11 @@ udav_reset(struct udav_softc *sc)
 
 #if defined(__NetBSD__) || defined(__OpenBSD__)
 int
-udav_activate(device_ptr_t self, enum devact act)
+udav_activate(device_t self, enum devact act)
 {
 	struct udav_softc *sc = (struct udav_softc *)self;
 
-	DPRINTF(("%s: %s: enter, act=%d\n", USBDEVNAME(sc->sc_dev),
+	DPRINTF(("%s: %s: enter, act=%d\n", device_get_nameunit(sc->sc_dev),
 		 __func__, act));
 	switch (act) {
 	case DVACT_ACTIVATE:
@@ -1001,7 +1003,7 @@ udav_activate(device_ptr_t self, enum devact act)
 #define UDAV_CALCHASH(addr) \
 	(ether_crc32_le((addr), ETHER_ADDR_LEN) & ((1 << UDAV_BITS) - 1))
 
-Static void
+static void
 udav_setmulti(struct udav_softc *sc)
 {
 	struct ifnet *ifp;
@@ -1014,7 +1016,7 @@ udav_setmulti(struct udav_softc *sc)
 	u_int8_t hashes[8];
 	int h = 0;
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return;
@@ -1072,7 +1074,7 @@ udav_setmulti(struct udav_softc *sc)
 	udav_csr_write(sc, UDAV_MAR, hashes, sizeof(hashes));
 }
 
-Static int
+static int
 udav_openpipes(struct udav_softc *sc)
 {
 	struct ue_chain *c;
@@ -1090,7 +1092,7 @@ udav_openpipes(struct udav_softc *sc)
 			     USBD_EXCLUSIVE_USE, &sc->sc_pipe_rx);
 	if (err) {
 		printf("%s: open rx pipe failed: %s\n",
-		       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		error = EIO;
 		goto done;
 	}
@@ -1100,7 +1102,7 @@ udav_openpipes(struct udav_softc *sc)
 			     USBD_EXCLUSIVE_USE, &sc->sc_pipe_tx);
 	if (err) {
 		printf("%s: open tx pipe failed: %s\n",
-		       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		error = EIO;
 		goto done;
 	}
@@ -1114,7 +1116,7 @@ udav_openpipes(struct udav_softc *sc)
 				  udav_intr, UDAV_INTR_INTERVAL);
 	if (err) {
 		printf("%s: open intr pipe failed: %s\n",
-		       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		error = EIO;
 		goto done;
 	}
@@ -1129,24 +1131,24 @@ udav_openpipes(struct udav_softc *sc)
 				USBD_SHORT_XFER_OK | USBD_NO_COPY,
 				USBD_NO_TIMEOUT, udav_rxeof);
 		(void)usbd_transfer(c->ue_xfer);
-		DPRINTF(("%s: %s: start read\n", USBDEVNAME(sc->sc_dev),
+		DPRINTF(("%s: %s: start read\n", device_get_nameunit(sc->sc_dev),
 			 __func__));
 	}
 
  done:
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 
 	return (error);
 }
 
-Static void
+static void
 udav_start(struct ifnet *ifp)
 {
 	struct udav_softc *sc = ifp->if_softc;
 	struct mbuf *m_head = NULL;
 
-	DPRINTF(("%s: %s: enter, link=%d\n", USBDEVNAME(sc->sc_dev),
+	DPRINTF(("%s: %s: enter, link=%d\n", device_get_nameunit(sc->sc_dev),
 		 __func__, sc->sc_link));
 
 	if (sc->sc_dying)
@@ -1197,14 +1199,14 @@ udav_start(struct ifnet *ifp)
 	ifp->if_timer = 5;
 }
 
-Static int
+static int
 udav_send(struct udav_softc *sc, struct mbuf *m, int idx)
 {
 	int total_len;
 	struct ue_chain *c;
 	usbd_status err;
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev),__func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev),__func__));
 
 	c = &sc->sc_cdata.ue_tx_chain[idx];
 
@@ -1232,16 +1234,16 @@ udav_send(struct udav_softc *sc, struct mbuf *m, int idx)
 	sc->sc_refcnt++;
 	err = usbd_transfer(c->ue_xfer);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	if (err != USBD_IN_PROGRESS) {
-		printf("%s: udav_send error=%s\n", USBDEVNAME(sc->sc_dev),
+		printf("%s: udav_send error=%s\n", device_get_nameunit(sc->sc_dev),
 		       usbd_errstr(err));
 		/* Stop the interface */
-		usb_add_task(sc->sc_udev, &sc->sc_stop_task);
+		usb_add_task(sc->sc_udev, &sc->sc_stop_task, USB_TASKQ_DRIVER);
 		return (EIO);
 	}
 
-	DPRINTF(("%s: %s: send %d bytes\n", USBDEVNAME(sc->sc_dev),
+	DPRINTF(("%s: %s: send %d bytes\n", device_get_nameunit(sc->sc_dev),
 		 __func__, total_len));
 
 	sc->sc_cdata.ue_tx_cnt++;
@@ -1249,7 +1251,7 @@ udav_send(struct udav_softc *sc, struct mbuf *m, int idx)
 	return (0);
 }
 
-Static void
+static void
 udav_txeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 {
 	struct ue_chain *c = priv;
@@ -1268,7 +1270,7 @@ udav_txeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
         UDAV_LOCK(sc);
 #endif
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	ifp->if_timer = 0;
 #if defined(__FreeBSD__)
@@ -1287,13 +1289,13 @@ udav_txeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 			return;
 		}
 		ifp->if_oerrors++;
-		printf("%s: usb error on tx: %s\n", USBDEVNAME(sc->sc_dev),
+		printf("%s: usb error on tx: %s\n", device_get_nameunit(sc->sc_dev),
 		       usbd_errstr(status));
 		if (status == USBD_STALLED) {
 			sc->sc_refcnt++;
 			usbd_clear_endpoint_stall(sc->sc_pipe_tx);
 			if (--sc->sc_refcnt < 0)
-				usb_detach_wakeup(USBDEV(sc->sc_dev));
+				usb_detach_wakeup(sc->sc_dev);
 		}
 #if defined(__NetBSD__)
 		splx(s);
@@ -1322,7 +1324,7 @@ udav_txeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 #endif
 }
 
-Static void
+static void
 udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 {
 	struct ue_chain *c = priv;
@@ -1335,7 +1337,7 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 	int s;
 #endif
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev),__func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev),__func__));
 
 	if (sc->sc_dying)
 		return;
@@ -1346,7 +1348,7 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 		sc->sc_rx_errs++;
 		if (usbd_ratecheck(&sc->sc_rx_notice)) {
 			printf("%s: %u usb errors on rx: %s\n",
-			       USBDEVNAME(sc->sc_dev), sc->sc_rx_errs,
+			       device_get_nameunit(sc->sc_dev), sc->sc_rx_errs,
 			       usbd_errstr(status));
 			sc->sc_rx_errs = 0;
 		}
@@ -1354,7 +1356,7 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 			sc->sc_refcnt++;
 			usbd_clear_endpoint_stall(sc->sc_pipe_rx);
 			if (--sc->sc_refcnt < 0)
-				usb_detach_wakeup(USBDEV(sc->sc_dev));
+				usb_detach_wakeup(sc->sc_dev);
 		}
 		goto done;
 	}
@@ -1368,7 +1370,7 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 	/* first byte in received data */
 	pktstat = mtod(m, u_int8_t *);
 	m_adj(m, sizeof(u_int8_t));
-	DPRINTF(("%s: RX Status: 0x%02x\n", USBDEVNAME(sc->sc_dev), *pktstat));
+	DPRINTF(("%s: RX Status: 0x%02x\n", device_get_nameunit(sc->sc_dev), *pktstat));
 
 	total_len = UGETW(mtod(m, u_int8_t *));
 	m_adj(m, sizeof(u_int16_t));
@@ -1404,7 +1406,7 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 	c->ue_mbuf = usb_ether_newbuf();
 	if (c->ue_mbuf == NULL) {
 		printf("%s: no memory for rx list "
-		    "-- packet dropped!\n", USBDEVNAME(sc->sc_dev));
+		    "-- packet dropped!\n", device_get_nameunit(sc->sc_dev));
 		ifp->if_ierrors++;
 		goto done1;
 	}
@@ -1414,7 +1416,7 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 	BPF_MTAP(ifp, m);
 #endif
 
-	DPRINTF(("%s: %s: deliver %d\n", USBDEVNAME(sc->sc_dev),
+	DPRINTF(("%s: %s: deliver %d\n", device_get_nameunit(sc->sc_dev),
 		 __func__, m->m_len));
 #if defined(__NetBSD__)
 	IF_INPUT(ifp, m);
@@ -1439,18 +1441,18 @@ udav_rxeof(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 	sc->sc_refcnt++;
 	usbd_transfer(xfer);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 
-	DPRINTF(("%s: %s: start rx\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: start rx\n", device_get_nameunit(sc->sc_dev), __func__));
 }
 
 #if 0
-Static void udav_intr()
+static void udav_intr()
 {
 }
 #endif
 
-Static int
+static int
 udav_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 {
 	struct udav_softc *sc = ifp->if_softc;
@@ -1461,7 +1463,7 @@ udav_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 #endif
 	int error = 0;
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (EIO);
@@ -1528,7 +1530,7 @@ udav_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 	return (error);
 }
 
-Static void
+static void
 udav_watchdog(struct ifnet *ifp)
 {
 	struct udav_softc *sc = ifp->if_softc;
@@ -1538,10 +1540,10 @@ udav_watchdog(struct ifnet *ifp)
 	int s;
 #endif
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	ifp->if_oerrors++;
-	printf("%s: watchdog timeout\n", USBDEVNAME(sc->sc_dev));
+	printf("%s: watchdog timeout\n", device_get_nameunit(sc->sc_dev));
 
 #if defined(__NetBSD__)
 	s = splusb();
@@ -1565,20 +1567,20 @@ udav_watchdog(struct ifnet *ifp)
 #endif
 }
 
-Static void
+static void
 udav_stop_task(struct udav_softc *sc)
 {
 	udav_stop(GET_IFP(sc), 1);
 }
 
 /* Stop the adapter and free any mbufs allocated to the RX and TX lists. */
-Static void
+static void
 udav_stop(struct ifnet *ifp, int disable)
 {
 	struct udav_softc *sc = ifp->if_softc;
 	usbd_status err;
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	ifp->if_timer = 0;
 
@@ -1592,11 +1594,11 @@ udav_stop(struct ifnet *ifp, int disable)
 		err = usbd_abort_pipe(sc->sc_pipe_rx);
 		if (err)
 			printf("%s: abort rx pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+			       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		err = usbd_close_pipe(sc->sc_pipe_rx);
 		if (err)
 			printf("%s: close rx pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+			       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		sc->sc_pipe_rx = NULL;
 	}
 
@@ -1605,11 +1607,11 @@ udav_stop(struct ifnet *ifp, int disable)
 		err = usbd_abort_pipe(sc->sc_pipe_tx);
 		if (err)
 			printf("%s: abort tx pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+			       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		err = usbd_close_pipe(sc->sc_pipe_tx);
 		if (err)
 			printf("%s: close tx pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+			       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		sc->sc_pipe_tx = NULL;
 	}
 
@@ -1620,11 +1622,11 @@ udav_stop(struct ifnet *ifp, int disable)
 		err = usbd_abort_pipe(sc->sc_pipe_intr);
 		if (err)
 			printf("%s: abort intr pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+			       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		err = usbd_close_pipe(sc->sc_pipe_intr);
 		if (err)
 			printf("%s: close intr pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_dev), usbd_errstr(err));
+			       device_get_nameunit(sc->sc_dev), usbd_errstr(err));
 		sc->sc_pipe_intr = NULL;
 	}
 #endif
@@ -1643,13 +1645,13 @@ udav_stop(struct ifnet *ifp, int disable)
 }
 
 /* Set media options */
-Static int
+static int
 udav_ifmedia_change(struct ifnet *ifp)
 {
 	struct udav_softc *sc = ifp->if_softc;
 	struct mii_data *mii = GET_MII(sc);
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return (0);
@@ -1666,13 +1668,13 @@ udav_ifmedia_change(struct ifnet *ifp)
 }
 
 /* Report current media status. */
-Static void
+static void
 udav_ifmedia_status(struct ifnet *ifp, struct ifmediareq *ifmr)
 {
 	struct udav_softc *sc = ifp->if_softc;
 	struct mii_data *mii = GET_MII(sc);
 
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 
 	if (sc->sc_dying)
 		return;
@@ -1692,7 +1694,7 @@ udav_ifmedia_status(struct ifnet *ifp, struct ifmediareq *ifmr)
 	ifmr->ifm_status = mii->mii_media_status;
 }
 
-Static void
+static void
 udav_tick(void *xsc)
 {
 	struct udav_softc *sc = xsc;
@@ -1700,17 +1702,17 @@ udav_tick(void *xsc)
 	if (sc == NULL)
 		return;
 
-	DPRINTFN(0xff, ("%s: %s: enter\n", USBDEVNAME(sc->sc_dev),
+	DPRINTFN(0xff, ("%s: %s: enter\n", device_get_nameunit(sc->sc_dev),
 			__func__));
 
 	if (sc->sc_dying)
 		return;
 
 	/* Perform periodic stuff in process context */
-	usb_add_task(sc->sc_udev, &sc->sc_tick_task);
+	usb_add_task(sc->sc_udev, &sc->sc_tick_task, USB_TASKQ_DRIVER);
 }
 
-Static void
+static void
 udav_tick_task(void *xsc)
 {
 	struct udav_softc *sc = xsc;
@@ -1723,7 +1725,7 @@ udav_tick_task(void *xsc)
 	if (sc == NULL)
 		return;
 
-	DPRINTFN(0xff, ("%s: %s: enter\n", USBDEVNAME(sc->sc_dev),
+	DPRINTFN(0xff, ("%s: %s: enter\n", device_get_nameunit(sc->sc_dev),
 			__func__));
 
 	if (sc->sc_dying)
@@ -1747,7 +1749,7 @@ udav_tick_task(void *xsc)
 		if (mii->mii_media_status & IFM_ACTIVE &&
 		    IFM_SUBTYPE(mii->mii_media_active) != IFM_NONE) {
 			DPRINTF(("%s: %s: got link\n",
-				 USBDEVNAME(sc->sc_dev), __func__));
+				 device_get_nameunit(sc->sc_dev), __func__));
 			sc->sc_link++;
 #if defined(__NetBSD__)
 			if (IFQ_IS_EMPTY(&ifp->if_snd) == 0)
@@ -1768,10 +1770,10 @@ udav_tick_task(void *xsc)
 }
 
 /* Get exclusive access to the MII registers */
-Static void
+static void
 udav_lock_mii(struct udav_softc *sc)
 {
-	DPRINTFN(0xff, ("%s: %s: enter\n", USBDEVNAME(sc->sc_dev),
+	DPRINTFN(0xff, ("%s: %s: enter\n", device_get_nameunit(sc->sc_dev),
 			__func__));
 
 	sc->sc_refcnt++;
@@ -1782,10 +1784,10 @@ udav_lock_mii(struct udav_softc *sc)
 #endif
 }
 
-Static void
+static void
 udav_unlock_mii(struct udav_softc *sc)
 {
-	DPRINTFN(0xff, ("%s: %s: enter\n", USBDEVNAME(sc->sc_dev),
+	DPRINTFN(0xff, ("%s: %s: enter\n", device_get_nameunit(sc->sc_dev),
 		       __func__));
 
 #if defined(__NetBSD__)
@@ -1794,11 +1796,11 @@ udav_unlock_mii(struct udav_softc *sc)
 	lockmgr(&sc->sc_mii_lock, LK_RELEASE, NULL, NULL);
 #endif
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 }
 
-Static int
-udav_miibus_readreg(device_ptr_t dev, int phy, int reg)
+static int
+udav_miibus_readreg(device_t dev, int phy, int reg)
 {
 	struct udav_softc *sc;
 	u_int8_t val[2];
@@ -1810,11 +1812,11 @@ udav_miibus_readreg(device_ptr_t dev, int phy, int reg)
 	sc = USBGETSOFTC(dev);
 
 	DPRINTFN(0xff, ("%s: %s: enter, phy=%d reg=0x%04x\n",
-		 USBDEVNAME(sc->sc_dev), __func__, phy, reg));
+		 device_get_nameunit(sc->sc_dev), __func__, phy, reg));
 
 	if (sc->sc_dying) {
 #ifdef DIAGNOSTIC
-		printf("%s: %s: dying\n", USBDEVNAME(sc->sc_dev),
+		printf("%s: %s: dying\n", device_get_nameunit(sc->sc_dev),
 		       __func__);
 #endif
 		return (0);
@@ -1823,7 +1825,7 @@ udav_miibus_readreg(device_ptr_t dev, int phy, int reg)
 	/* XXX: one PHY only for the internal PHY */
 	if (phy != 0) {
 		DPRINTFN(0xff, ("%s: %s: phy=%d is not supported\n",
-			 USBDEVNAME(sc->sc_dev), __func__, phy));
+			 device_get_nameunit(sc->sc_dev), __func__, phy));
 		return (0);
 	}
 
@@ -1849,38 +1851,38 @@ udav_miibus_readreg(device_ptr_t dev, int phy, int reg)
 	data16 = val[0] | (val[1] << 8);
 
 	DPRINTFN(0xff, ("%s: %s: phy=%d reg=0x%04x => 0x%04x\n",
-		 USBDEVNAME(sc->sc_dev), __func__, phy, reg, data16));
+		 device_get_nameunit(sc->sc_dev), __func__, phy, reg, data16));
 
 	return (data16);
 }
 
-Static void
-udav_miibus_writereg(device_ptr_t dev, int phy, int reg, int data)
+static int
+udav_miibus_writereg(device_t dev, int phy, int reg, int data)
 {
 	struct udav_softc *sc;
 	u_int8_t val[2];
 
 	if (dev == NULL)
-		return;
+		return (0);	/* XXX real error? */
 
 	sc = USBGETSOFTC(dev);
 
 	DPRINTFN(0xff, ("%s: %s: enter, phy=%d reg=0x%04x data=0x%04x\n",
-		 USBDEVNAME(sc->sc_dev), __func__, phy, reg, data));
+		 device_get_nameunit(sc->sc_dev), __func__, phy, reg, data));
 
 	if (sc->sc_dying) {
 #ifdef DIAGNOSTIC
-		printf("%s: %s: dying\n", USBDEVNAME(sc->sc_dev),
+		printf("%s: %s: dying\n", device_get_nameunit(sc->sc_dev),
 		       __func__);
 #endif
-		return;
+		return (0);	/* XXX real error? */
 	}
 
 	/* XXX: one PHY only for the internal PHY */
 	if (phy != 0) {
 		DPRINTFN(0xff, ("%s: %s: phy=%d is not supported\n",
-			 USBDEVNAME(sc->sc_dev), __func__, phy));
-		return;
+			 device_get_nameunit(sc->sc_dev), __func__, phy));
+		return (0);	/* XXX real error? */
 	}
 
 	udav_lock_mii(sc);
@@ -1904,11 +1906,11 @@ udav_miibus_writereg(device_ptr_t dev, int phy, int reg, int data)
 
 	udav_unlock_mii(sc);
 
-	return;
+	return (0);
 }
 
-Static void
-udav_miibus_statchg(device_ptr_t dev)
+static void
+udav_miibus_statchg(device_t dev)
 {
 #ifdef UDAV_DEBUG
 	struct udav_softc *sc;
@@ -1917,7 +1919,7 @@ udav_miibus_statchg(device_ptr_t dev)
 		return;
 
 	sc = USBGETSOFTC(dev);
-	DPRINTF(("%s: %s: enter\n", USBDEVNAME(sc->sc_dev), __func__));
+	DPRINTF(("%s: %s: enter\n", device_get_nameunit(sc->sc_dev), __func__));
 #endif
 	/* Nothing to do */
 }
@@ -1927,8 +1929,8 @@ udav_miibus_statchg(device_ptr_t dev)
  * Stop all chip I/O so that the kernel's probe routines don't
  * get confused by errant DMAs when rebooting.
  */
-Static void
-udav_shutdown(device_ptr_t dev)
+static int
+udav_shutdown(device_t dev)
 {
 	struct udav_softc	*sc;
 
@@ -1936,10 +1938,10 @@ udav_shutdown(device_ptr_t dev)
 
 	udav_stop_task(sc);
 
-	return;
+	return (0);
 }
 
-Static void
+static void
 udav_rxstart(struct ifnet *ifp)
 {
 	struct udav_softc	*sc;
@@ -1952,7 +1954,7 @@ udav_rxstart(struct ifnet *ifp)
 	c->ue_mbuf = usb_ether_newbuf();
 	if (c->ue_mbuf == NULL) {
 		printf("%s: no memory for rx list "
-		    "-- packet dropped!\n", USBDEVNAME(sc->sc_dev));
+		    "-- packet dropped!\n", device_get_nameunit(sc->sc_dev));
 		ifp->if_ierrors++;
 		UDAV_UNLOCK(sc);
 		return;
