@@ -29,7 +29,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/dev/usb/kue_fw.h,v 1.3 2005/01/06 01:43:27 imp Exp $
+ * $FreeBSD: src/sys/dev/usb/kue_fw.h,v 1.4 2006/09/06 23:44:24 imp Exp $
  */
 
 /*
@@ -85,7 +85,7 @@
 #define KUE_QTINTR_LOAD_CODE_HIGH	0x9C
 
 /* Firmware code segment */
-Static unsigned char kue_code_seg[] =
+static unsigned char kue_code_seg[] =
 {
     /******************************************/
     /* NOTE: B6/C3 is data header signature   */
@@ -577,7 +577,7 @@ Static unsigned char kue_code_seg[] =
 };
 
 /* Firmware fixup (data?) segment */
-Static unsigned char kue_fix_seg[] =
+static unsigned char kue_fix_seg[] =
 {
     /******************************************/
     /* NOTE: B6/C3 is data header signature   */
@@ -680,6 +680,6 @@ Static unsigned char kue_fix_seg[] =
 
 /* Fixup command. */
 #define KUE_TRIGCMD_OFFSET	5
-Static unsigned char kue_trig_seg[] = {
+static unsigned char kue_trig_seg[] = {
 0xb6, 0xc3, 0x01, 0x00, 0x06, 0x64, 0x00, 0x00
 };
