@@ -38,25 +38,22 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/usb/ulpt.c,v 1.66.2.2 2007/06/17 09:38:26 brueffer Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/usb/ulpt.c,v 1.80 2007/06/21 14:42:34 imp Exp $");
 
 /*
  * Printer Class spec: http://www.usb.org/developers/data/devclass/usbprint109.PDF
  */
 
+/* XXXimp: need to migrate from devclass_get_softc */
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
 #include <sys/kernel.h>
 #include <sys/fcntl.h>
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-#include <sys/device.h>
-#include <sys/ioctl.h>
-#elif defined(__FreeBSD__)
 #include <sys/ioccom.h>
 #include <sys/module.h>
 #include <sys/bus.h>
-#endif
 #include <sys/uio.h>
 #include <sys/conf.h>
 #include <sys/syslog.h>
@@ -78,8 +75,8 @@ __FBSDID("$FreeBSD: src/sys/dev/usb/ulpt.c,v 1.66.2.2 2007/06/17 09:38:26 brueff
 #define ULPT_READ_TIMO 10
 
 #ifdef USB_DEBUG
-#define DPRINTF(x)	if (ulptdebug) logprintf x
-#define DPRINTFN(n,x)	if (ulptdebug>(n)) logprintf x
+#define DPRINTF(x)	if (ulptdebug) printf x
+#define DPRINTFN(n,x)	if (ulptdebug>(n)) printf x
 int	ulptdebug = 0;
 SYSCTL_NODE(_hw_usb, OID_AUTO, ulpt, CTLFLAG_RW, 0, "USB ulpt");
 SYSCTL_INT(_hw_usb_ulpt, OID_AUTO, debug, CTLFLAG_RW,
@@ -100,7 +97,7 @@ SYSCTL_INT(_hw_usb_ulpt, OID_AUTO, debug, CTLFLAG_RW,
 #define LPS_MASK        (LPS_SELECT|LPS_NERR|LPS_NOPAPER)
 
 struct ulpt_softc {
-	USBBASEDEVICE sc_dev;
+	device_t sc_dev;
 	usbd_device_handle sc_udev;	/* device */
 	usbd_interface_handle sc_iface;	/* interface */
 	int sc_ifaceno;
@@ -115,7 +112,7 @@ struct ulpt_softc {
 	usbd_xfer_handle sc_in_xfer;
 	void *sc_in_buf;
 
-	usb_callout_t sc_read_callout;
+	struct callout sc_read_callout;
 	int sc_has_callout;
 
 	u_char sc_state;
@@ -129,34 +126,18 @@ struct ulpt_softc {
 	int sc_refcnt;
 	u_char sc_dying;
 
-#if defined(__FreeBSD__)
 	struct cdev *dev;
 	struct cdev *dev_noprime;
-#endif
 };
 
-#if defined(__NetBSD__)
-dev_type_open(ulptopen);
-dev_type_close(ulptclose);
-dev_type_write(ulptwrite);
-dev_type_read(ulptread);
-dev_type_ioctl(ulptioctl);
-
-const struct cdevsw ulpt_cdevsw = {
-	ulptopen, ulptclose, ulptread, ulptwrite, ulptioctl,
-	nostop, notty, nopoll, nommap, nokqfilter,
-};
-#elif defined(__OpenBSD__)
-cdev_decl(ulpt);
-#elif defined(__FreeBSD__)
-Static d_open_t ulptopen;
-Static d_close_t ulptclose;
-Static d_write_t ulptwrite;
-Static d_read_t ulptread;
-Static d_ioctl_t ulptioctl;
+static d_open_t ulptopen;
+static d_close_t ulptclose;
+static d_write_t ulptwrite;
+static d_read_t ulptread;
+static d_ioctl_t ulptioctl;
 
 
-Static struct cdevsw ulpt_cdevsw = {
+static struct cdevsw ulpt_cdevsw = {
 	.d_version =	D_VERSION,
 	.d_flags =	D_NEEDGIANT,
 	.d_open =	ulptopen,
@@ -166,7 +147,6 @@ Static struct cdevsw ulpt_cdevsw = {
 	.d_ioctl =	ulptioctl,
 	.d_name =	"ulpt",
 };
-#endif
 
 void ulpt_disco(void *);
 
@@ -186,12 +166,34 @@ void ieee1284_print_id(char *);
 #define	ULPTUNIT(s)	(minor(s) & 0x1f)
 #define	ULPTFLAGS(s)	(minor(s) & 0xe0)
 
+static device_probe_t ulpt_match;
+static device_attach_t ulpt_attach;
+static device_detach_t ulpt_detach;
 
-USB_DECLARE_DRIVER(ulpt);
+static device_method_t ulpt_methods[] = {
+	/* Device interface */
+	DEVMETHOD(device_probe,		ulpt_match),
+	DEVMETHOD(device_attach,	ulpt_attach),
+	DEVMETHOD(device_detach,	ulpt_detach),
 
-USB_MATCH(ulpt)
+	{ 0, 0 }
+};
+
+static driver_t ulpt_driver = {
+	"ulpt",
+	ulpt_methods,
+	sizeof(struct ulpt_softc)
+};
+
+static devclass_t ulpt_devclass;
+
+MODULE_DEPEND(umass, usb, 1, 1, 1);
+DRIVER_MODULE(ulpt, uhub, ulpt_driver, ulpt_devclass, usbd_driver_load, 0);
+
+static int
+ulpt_match(device_t self)
 {
-	USB_MATCH_START(ulpt, uaa);
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 	usb_interface_descriptor_t *id;
 
 	DPRINTFN(10,("ulpt_match\n"));
@@ -208,23 +210,23 @@ USB_MATCH(ulpt)
 	return (UMATCH_NONE);
 }
 
-USB_ATTACH(ulpt)
+static int
+ulpt_attach(device_t self)
 {
-	USB_ATTACH_START(ulpt, sc, uaa);
+	struct ulpt_softc *sc = device_get_softc(self);
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 	usbd_device_handle dev = uaa->device;
 	usbd_interface_handle iface = uaa->iface;
 	usb_interface_descriptor_t *ifcd = usbd_get_interface_descriptor(iface);
 	usb_interface_descriptor_t *id, *iend;
 	usb_config_descriptor_t *cdesc;
 	usbd_status err;
-	char devinfo[1024];
 	usb_endpoint_descriptor_t *ed;
 	u_int8_t epcount;
 	int i, altno;
 
 	DPRINTFN(10,("ulpt_attach: sc=%p\n", sc));
-	usbd_devinfo(uaa->device, USBD_SHOW_INTERFACE_CLASS, devinfo);
-	USB_ATTACH_SETUP;
+	sc->sc_dev = self;
 
 	/* XXX
 	 * Stepping through the alternate settings needs to be abstracted out.
@@ -232,8 +234,8 @@ USB_ATTACH(ulpt)
 	cdesc = usbd_get_config_descriptor(dev);
 	if (cdesc == NULL) {
 		printf("%s: failed to get configuration descriptor\n",
-		       USBDEVNAME(sc->sc_dev));
-		USB_ATTACH_ERROR_RETURN;
+		       device_get_nameunit(sc->sc_dev));
+		return ENXIO;
 	}
 	iend = (usb_interface_descriptor_t *)
 		   ((char *)cdesc + UGETW(cdesc->wTotalLength));
@@ -264,9 +266,9 @@ USB_ATTACH(ulpt)
 		err = usbd_set_interface(iface, altno);
 		if (err) {
 			printf("%s: setting alternate interface failed\n",
-			       USBDEVNAME(sc->sc_dev));
+			       device_get_nameunit(sc->sc_dev));
 			sc->sc_dying = 1;
-			USB_ATTACH_ERROR_RETURN;
+			return ENXIO;
 		}
 	}
 
@@ -279,8 +281,8 @@ USB_ATTACH(ulpt)
 		ed = usbd_interface2endpoint_descriptor(iface, i);
 		if (ed == NULL) {
 			printf("%s: couldn't get ep %d\n",
-			    USBDEVNAME(sc->sc_dev), i);
-			USB_ATTACH_ERROR_RETURN;
+			    device_get_nameunit(sc->sc_dev), i);
+			return ENXIO;
 		}
 		if (UE_GET_DIR(ed->bEndpointAddress) == UE_DIR_IN &&
 		    UE_GET_XFERTYPE(ed->bmAttributes) == UE_BULK) {
@@ -292,9 +294,9 @@ USB_ATTACH(ulpt)
 	}
 	if (sc->sc_out == -1) {
 		printf("%s: could not find bulk out endpoint\n",
-		    USBDEVNAME(sc->sc_dev));
+		    device_get_nameunit(sc->sc_dev));
 		sc->sc_dying = 1;
-		USB_ATTACH_ERROR_RETURN;
+		return ENXIO;
 	}
 
 	if (usbd_get_quirks(dev)->uq_flags & UQ_BROKEN_BIDIR) {
@@ -302,7 +304,7 @@ USB_ATTACH(ulpt)
 		sc->sc_in = -1;
 	}
 
-	printf("%s: using %s-directional mode\n", USBDEVNAME(sc->sc_dev),
+	printf("%s: using %s-directional mode\n", device_get_nameunit(sc->sc_dev),
 	       sc->sc_in >= 0 ? "bi" : "uni");
 
 	DPRINTFN(10, ("ulpt_attach: bulk=%d\n", sc->sc_out));
@@ -330,68 +332,42 @@ USB_ATTACH(ulpt)
 	err = usbd_do_request_flags(dev, &req, devinfo, USBD_SHORT_XFER_OK,
 		  &alen, USBD_DEFAULT_TIMEOUT);
 	if (err) {
-		printf("%s: cannot get device id\n", USBDEVNAME(sc->sc_dev));
+		printf("%s: cannot get device id\n", device_get_nameunit(sc->sc_dev));
 	} else if (alen <= 2) {
 		printf("%s: empty device id, no printer connected?\n",
-		       USBDEVNAME(sc->sc_dev));
+		       device_get_nameunit(sc->sc_dev));
 	} else {
 		/* devinfo now contains an IEEE-1284 device ID */
 		len = ((devinfo[0] & 0xff) << 8) | (devinfo[1] & 0xff);
 		if (len > sizeof devinfo - 3)
 			len = sizeof devinfo - 3;
 		devinfo[len] = 0;
-		printf("%s: device id <", USBDEVNAME(sc->sc_dev));
+		printf("%s: device id <", device_get_nameunit(sc->sc_dev));
 		ieee1284_print_id(devinfo+2);
 		printf(">\n");
 	}
 	}
 #endif
 
-#if defined(__FreeBSD__)
 	sc->dev = make_dev(&ulpt_cdevsw, device_get_unit(self),
 		UID_ROOT, GID_OPERATOR, 0644, "ulpt%d", device_get_unit(self));
 	sc->dev_noprime = make_dev(&ulpt_cdevsw,
 		device_get_unit(self)|ULPT_NOPRIME,
 		UID_ROOT, GID_OPERATOR, 0644, "unlpt%d", device_get_unit(self));
-#endif
 
-	usbd_add_drv_event(USB_EVENT_DRIVER_ATTACH, sc->sc_udev,
-			   USBDEV(sc->sc_dev));
+	usbd_add_drv_event(USB_EVENT_DRIVER_ATTACH, sc->sc_udev, sc->sc_dev);
 
-	USB_ATTACH_SUCCESS_RETURN;
+	return 0;
 }
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-int
-ulpt_activate(device_ptr_t self, enum devact act)
+
+static int
+ulpt_detach(device_t self)
 {
-	struct ulpt_softc *sc = (struct ulpt_softc *)self;
-
-	switch (act) {
-	case DVACT_ACTIVATE:
-		return (EOPNOTSUPP);
-
-	case DVACT_DEACTIVATE:
-		sc->sc_dying = 1;
-		break;
-	}
-	return (0);
-}
-#endif
-
-USB_DETACH(ulpt)
-{
-	USB_DETACH_START(ulpt, sc);
+	struct ulpt_softc *sc = device_get_softc(self);
 	int s;
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-	int maj, mn;
-#endif
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-	DPRINTF(("ulpt_detach: sc=%p flags=%d\n", sc, flags));
-#elif defined(__FreeBSD__)
 	DPRINTF(("ulpt_detach: sc=%p\n", sc));
-#endif
 
 	sc->sc_dying = 1;
 	if (sc->sc_out_pipe != NULL)
@@ -403,30 +379,14 @@ USB_DETACH(ulpt)
 	if (--sc->sc_refcnt >= 0) {
 		/* There is noone to wake, aborting the pipe is enough */
 		/* Wait for processes to go away. */
-		usb_detach_wait(USBDEV(sc->sc_dev));
+		usb_detach_wait(sc->sc_dev);
 	}
 	splx(s);
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-	/* locate the major number */
-#if defined(__NetBSD__)
-	maj = cdevsw_lookup_major(&ulpt_cdevsw);
-#elif defined(__OpenBSD__)
-	for (maj = 0; maj < nchrdev; maj++)
-		if (cdevsw[maj].d_open == ulptopen)
-			break;
-#endif
-
-	/* Nuke the vnodes for any open instances (calls close). */
-	mn = self->dv_unit;
-	vdevgone(maj, mn, mn, VCHR);
-#elif defined(__FreeBSD__)
 	destroy_dev(sc->dev);
 	destroy_dev(sc->dev_noprime);
-#endif
 
-	usbd_add_drv_event(USB_EVENT_DRIVER_DETACH, sc->sc_udev,
-			   USBDEV(sc->sc_dev));
+	usbd_add_drv_event(USB_EVENT_DRIVER_DETACH, sc->sc_udev, sc->sc_dev);
 
 	return (0);
 }
@@ -501,14 +461,16 @@ int ulptusein = 1;
  * Reset the printer, then wait until it's selected and not busy.
  */
 int
-ulptopen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
+ulptopen(struct cdev *dev, int flag, int mode, struct thread *p)
 {
 	u_char flags = ULPTFLAGS(dev);
 	struct ulpt_softc *sc;
 	usbd_status err;
-	int spin, error;
+	int error;
 
-	USB_GET_SC_OPEN(ulpt, ULPTUNIT(dev), sc);
+	sc = devclass_get_softc(ulpt_devclass, ULPTUNIT(dev));
+	if (sc == NULL)
+		return (ENXIO);
 
 	if (sc == NULL || sc->sc_iface == NULL || sc->sc_dying)
 		return (ENXIO);
@@ -520,7 +482,7 @@ ulptopen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 	sc->sc_flags = flags;
 	DPRINTF(("ulptopen: flags=0x%x\n", (unsigned)flags));
 
-#if defined(USB_DEBUG) && defined(__FreeBSD__)
+#if defined(USB_DEBUG)
 	/* Ignoring these flags might not be a good idea */
 	if ((flags & ~ULPT_NOPRIME) != 0)
 		printf("ulptopen: flags ignored: %b\n", flags,
@@ -540,28 +502,6 @@ ulptopen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 		}
 	}
 
-	for (spin = 0; (ulpt_status(sc) & LPS_SELECT) == 0; spin += STEP) {
-		DPRINTF(("ulpt_open: waiting a while\n"));
-		if (spin >= TIMEOUT) {
-			error = EBUSY;
-			sc->sc_state = 0;
-			goto done;
-		}
-
-		/* wait 1/4 second, give up if we get a signal */
-		error = tsleep(sc, LPTPRI | PCATCH, "ulptop", STEP);
-		if (error != EWOULDBLOCK) {
-			sc->sc_state = 0;
-			goto done;
-		}
-
-		if (sc->sc_dying) {
-			error = ENXIO;
-			sc->sc_state = 0;
-			goto done;
-		}
-	}
-
 	err = usbd_open_pipe(sc->sc_iface, sc->sc_out, 0, &sc->sc_out_pipe);
 	if (err) {
 		error = EIO;
@@ -599,8 +539,9 @@ ulptopen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 		/* If it's not opened for read the set up a reader. */
 		if (!(flag & FREAD)) {
 			DPRINTF(("ulpt_open: start read callout\n"));
-			usb_callout_init(sc->sc_read_callout);
-			usb_callout(sc->sc_read_callout, hz/5, ulpt_tick, sc);
+			callout_init(&sc->sc_read_callout, 0);
+			callout_reset(&sc->sc_read_callout, hz/5, ulpt_tick,
+			    sc);
 			sc->sc_has_callout = 1;
 		}
 	}
@@ -625,7 +566,7 @@ ulptopen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 
  done:
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 
 	DPRINTF(("ulptopen: done, error=%d\n", error));
 	return (error);
@@ -641,28 +582,28 @@ ulpt_statusmsg(u_char status, struct ulpt_softc *sc)
 	sc->sc_laststatus = status;
 
 	if (new & LPS_SELECT)
-		log(LOG_NOTICE, "%s: offline\n", USBDEVNAME(sc->sc_dev));
+		log(LOG_NOTICE, "%s: offline\n", device_get_nameunit(sc->sc_dev));
 	else if (new & LPS_NOPAPER)
-		log(LOG_NOTICE, "%s: out of paper\n", USBDEVNAME(sc->sc_dev));
+		log(LOG_NOTICE, "%s: out of paper\n", device_get_nameunit(sc->sc_dev));
 	else if (new & LPS_NERR)
-		log(LOG_NOTICE, "%s: output error\n", USBDEVNAME(sc->sc_dev));
+		log(LOG_NOTICE, "%s: output error\n", device_get_nameunit(sc->sc_dev));
 
 	return (status);
 }
 
 int
-ulptclose(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
+ulptclose(struct cdev *dev, int flag, int mode, struct thread *p)
 {
 	struct ulpt_softc *sc;
 
-	USB_GET_SC(ulpt, ULPTUNIT(dev), sc);
+	sc = devclass_get_softc(ulpt_devclass, ULPTUNIT(dev));
 
 	if (sc->sc_state != ULPT_OPEN)
 		/* We are being forced to close before the open completed. */
 		return (0);
 
 	if (sc->sc_has_callout) {
-		usb_uncallout(sc->sc_read_callout, ulpt_tick, sc);
+		callout_stop(&sc->sc_read_callout);
 		sc->sc_has_callout = 0;
 	}
 
@@ -728,7 +669,7 @@ ulptwrite(struct cdev *dev, struct uio *uio, int flags)
 	struct ulpt_softc *sc;
 	int error;
 
-	USB_GET_SC(ulpt, ULPTUNIT(dev), sc);
+	sc = devclass_get_softc(ulpt_devclass, ULPTUNIT(dev));
 
 	if (sc->sc_dying)
 		return (EIO);
@@ -736,7 +677,7 @@ ulptwrite(struct cdev *dev, struct uio *uio, int flags)
 	sc->sc_refcnt++;
 	error = ulpt_do_write(sc, uio, flags);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	return (error);
 }
 
@@ -783,7 +724,7 @@ ulptread(struct cdev *dev, struct uio *uio, int flags)
 	struct ulpt_softc *sc;
 	int error;
 
-	USB_GET_SC(ulpt, ULPTUNIT(dev), sc);
+	sc = devclass_get_softc(ulpt_devclass, ULPTUNIT(dev));
 
 	if (sc->sc_dying)
 		return (EIO);
@@ -791,7 +732,7 @@ ulptread(struct cdev *dev, struct uio *uio, int flags)
 	sc->sc_refcnt++;
 	error = ulpt_do_read(sc, uio, flags);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	return (error);
 }
 
@@ -814,8 +755,8 @@ ulpt_read_cb(usbd_xfer_handle xfer, usbd_private_handle priv,
 		DPRINTF(("ulpt_tick: discarding %d bytes\n", n));
 #endif
 	if (!err || err == USBD_TIMEOUT)
-		usb_callout(sc->sc_read_callout, hz / ULPT_READS_PER_SEC,
-			    ulpt_tick, sc);
+		callout_reset(&sc->sc_read_callout, hz / ULPT_READS_PER_SEC,
+		    ulpt_tick, sc);
 }
 
 void
@@ -837,7 +778,7 @@ ulpt_tick(void *xsc)
 }
 
 int
-ulptioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, usb_proc_ptr p)
+ulptioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *p)
 {
 	int error = 0;
 
@@ -872,7 +813,3 @@ ieee1284_print_id(char *str)
 	}
 }
 #endif
-
-#if defined(__FreeBSD__)
-DRIVER_MODULE(ulpt, uhub, ulpt_driver, ulpt_devclass, usbd_driver_load, 0);
-#endif
