@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/usb/uplcom.c,v 1.29.2.5 2007/06/17 09:38:27 brueffer Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/usb/uplcom.c,v 1.50 2007/06/20 05:10:53 imp Exp $");
 
 /*-
  * Copyright (c) 2001 The NetBSD Foundation, Inc.
@@ -126,7 +126,7 @@ SYSCTL_INT(_hw_usb_uplcom, OID_AUTO, debug, CTLFLAG_RW,
 
 #define DPRINTFN(n, x)	do { \
 				if (uplcomdebug > (n)) \
-					logprintf x; \
+					printf x; \
 			} while (0)
 #else
 #define DPRINTFN(n, x)
@@ -184,25 +184,25 @@ struct	uplcom_softc {
 #define UPLCOMIBUFSIZE 256
 #define UPLCOMOBUFSIZE 256
 
-Static	usbd_status uplcom_reset(struct uplcom_softc *);
-Static	usbd_status uplcom_set_line_coding(struct uplcom_softc *,
+static	usbd_status uplcom_reset(struct uplcom_softc *);
+static	usbd_status uplcom_set_line_coding(struct uplcom_softc *,
 					   usb_cdc_line_state_t *);
-Static	usbd_status uplcom_set_crtscts(struct uplcom_softc *);
-Static	void uplcom_intr(usbd_xfer_handle, usbd_private_handle, usbd_status);
+static	usbd_status uplcom_set_crtscts(struct uplcom_softc *);
+static	void uplcom_intr(usbd_xfer_handle, usbd_private_handle, usbd_status);
 
-Static	void uplcom_set(void *, int, int, int);
-Static	void uplcom_dtr(struct uplcom_softc *, int);
-Static	void uplcom_rts(struct uplcom_softc *, int);
-Static	void uplcom_break(struct uplcom_softc *, int);
-Static	void uplcom_set_line_state(struct uplcom_softc *);
-Static	void uplcom_get_status(void *, int, u_char *, u_char *);
+static	void uplcom_set(void *, int, int, int);
+static	void uplcom_dtr(struct uplcom_softc *, int);
+static	void uplcom_rts(struct uplcom_softc *, int);
+static	void uplcom_break(struct uplcom_softc *, int);
+static	void uplcom_set_line_state(struct uplcom_softc *);
+static	void uplcom_get_status(void *, int, u_char *, u_char *);
 #if 0 /* TODO */
-Static	int  uplcom_ioctl(void *, int, u_long, caddr_t, int, usb_proc_ptr);
+static	int  uplcom_ioctl(void *, int, u_long, caddr_t, int, usb_proc_ptr);
 #endif
-Static	int  uplcom_param(void *, int, struct termios *);
-Static	int  uplcom_open(void *, int);
-Static	void uplcom_close(void *, int);
-Static	void uplcom_notify(void *, int);
+static	int  uplcom_param(void *, int, struct termios *);
+static	int  uplcom_open(void *, int);
+static	void uplcom_close(void *, int);
+static	void uplcom_notify(void *, int);
 
 struct ucom_callback uplcom_callback = {
 	uplcom_get_status,
@@ -245,8 +245,6 @@ static const struct uplcom_product {
 	  0x300, TYPE_PL2303X },
 	/* IOGEAR/ATEN UC-232A (also ST Lab USB-SERIAL-1) */
 	{ USB_VENDOR_PROLIFIC, USB_PRODUCT_PROLIFIC_PL2303, -1, TYPE_PL2303 },
-	/* HAMLET exagerate XURS232 */
-	{ USB_VENDOR_PROLIFIC, USB_PRODUCT_PROLIFIC_PL2303, -1, TYPE_PL2303X },
 	/* TDK USB-PHS Adapter UHA6400 */
 	{ USB_VENDOR_TDK, USB_PRODUCT_TDK_UHA6400, -1, TYPE_PL2303 },
 	/* RATOC REX-USB60 */
@@ -274,11 +272,11 @@ static const struct uplcom_product {
 	{ 0, 0 }
 };
 
-Static device_probe_t uplcom_match;
-Static device_attach_t uplcom_attach;
-Static device_detach_t uplcom_detach;
+static device_probe_t uplcom_match;
+static device_attach_t uplcom_attach;
+static device_detach_t uplcom_detach;
 
-Static device_method_t uplcom_methods[] = {
+static device_method_t uplcom_methods[] = {
 	/* Device interface */
 	DEVMETHOD(device_probe, uplcom_match),
 	DEVMETHOD(device_attach, uplcom_attach),
@@ -286,7 +284,7 @@ Static device_method_t uplcom_methods[] = {
 	{ 0, 0 }
 };
 
-Static driver_t uplcom_driver = {
+static driver_t uplcom_driver = {
 	"ucom",
 	uplcom_methods,
 	sizeof (struct uplcom_softc)
@@ -305,7 +303,7 @@ sysctl_hw_usb_uplcom_interval(SYSCTL_HANDLER_ARGS)
 	int err, val;
 
 	val = uplcominterval;
-	err = sysctl_handle_int(oidp, &val, sizeof(val), req);
+	err = sysctl_handle_int(oidp, &val, 0, req);
 	if (err != 0 || req->newptr == NULL)
 		return (err);
 	if (0 < val && val <= 1000)
@@ -320,9 +318,10 @@ SYSCTL_PROC(_hw_usb_uplcom, OID_AUTO, interval, CTLTYPE_INT | CTLFLAG_RW,
 	    0, sizeof(int), sysctl_hw_usb_uplcom_interval,
 	    "I", "uplcom interrupt pipe interval");
 
-USB_MATCH(uplcom)
+static int
+uplcom_match(device_t self)
 {
-	USB_MATCH_START(uplcom, uaa);
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 	int i;
 
 	if (uaa->iface != NULL)
@@ -339,35 +338,26 @@ USB_MATCH(uplcom)
 	return (UMATCH_NONE);
 }
 
-USB_ATTACH(uplcom)
+static int
+uplcom_attach(device_t self)
 {
-	USB_ATTACH_START(uplcom, sc, uaa);
+	struct uplcom_softc *sc = device_get_softc(self);
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 	usbd_device_handle dev = uaa->device;
 	struct ucom_softc *ucom;
 	usb_config_descriptor_t *cdesc;
 	usb_interface_descriptor_t *id;
 	usb_endpoint_descriptor_t *ed;
-	char *devinfo;
 	const char *devname;
 	usbd_status err;
 	int i;
 
-	devinfo = malloc(1024, M_USBDEV, M_WAITOK);
 	ucom = &sc->sc_ucom;
-
-	bzero(sc, sizeof (struct uplcom_softc));
-
-	usbd_devinfo(dev, 0, devinfo);
-	/* USB_ATTACH_SETUP; */
 	ucom->sc_dev = self;
-	device_set_desc_copy(self, devinfo);
-	/* USB_ATTACH_SETUP; */
-
 	ucom->sc_udev = dev;
 	ucom->sc_iface = uaa->iface;
 
-	devname = USBDEVNAME(ucom->sc_dev);
-	printf("%s: %s\n", devname, devinfo);
+	devname = device_get_nameunit(ucom->sc_dev);
 
 	DPRINTF(("uplcom attach: sc = %p\n", sc));
 
@@ -420,7 +410,7 @@ USB_ATTACH(uplcom)
 
 	if (cdesc == NULL) {
 		printf("%s: failed to get configuration descriptor\n",
-			USBDEVNAME(ucom->sc_dev));
+			device_get_nameunit(ucom->sc_dev));
 		ucom->sc_dying = 1;
 		goto error;
 	}
@@ -444,7 +434,7 @@ USB_ATTACH(uplcom)
 		ed = usbd_interface2endpoint_descriptor(ucom->sc_iface, i);
 		if (ed == NULL) {
 			printf("%s: no endpoint descriptor for %d\n",
-				USBDEVNAME(ucom->sc_dev), i);
+				device_get_nameunit(ucom->sc_dev), i);
 			ucom->sc_dying = 1;
 			goto error;
 		}
@@ -458,7 +448,7 @@ USB_ATTACH(uplcom)
 
 	if (sc->sc_intr_number == -1) {
 		printf("%s: Could not find interrupt in\n",
-			USBDEVNAME(ucom->sc_dev));
+			device_get_nameunit(ucom->sc_dev));
 		ucom->sc_dying = 1;
 		goto error;
 	}
@@ -499,7 +489,7 @@ USB_ATTACH(uplcom)
 		ed = usbd_interface2endpoint_descriptor(ucom->sc_iface, i);
 		if (ed == NULL) {
 			printf("%s: no endpoint descriptor for %d\n",
-				USBDEVNAME(ucom->sc_dev), i);
+				device_get_nameunit(ucom->sc_dev), i);
 			ucom->sc_dying = 1;
 			goto error;
 		}
@@ -515,14 +505,14 @@ USB_ATTACH(uplcom)
 
 	if (ucom->sc_bulkin_no == -1) {
 		printf("%s: Could not find data bulk in\n",
-			USBDEVNAME(ucom->sc_dev));
+			device_get_nameunit(ucom->sc_dev));
 		ucom->sc_dying = 1;
 		goto error;
 	}
 
 	if (ucom->sc_bulkout_no == -1) {
 		printf("%s: Could not find data bulk out\n",
-			USBDEVNAME(ucom->sc_dev));
+			device_get_nameunit(ucom->sc_dev));
 		ucom->sc_dying = 1;
 		goto error;
 	}
@@ -541,7 +531,7 @@ USB_ATTACH(uplcom)
 
 	if (err) {
 		printf("%s: reset failed: %s\n",
-		       USBDEVNAME(ucom->sc_dev), usbd_errstr(err));
+		       device_get_nameunit(ucom->sc_dev), usbd_errstr(err));
 		ucom->sc_dying = 1;
 		goto error;
 	}
@@ -551,18 +541,16 @@ USB_ATTACH(uplcom)
 
 	TASK_INIT(&sc->sc_task, 0, uplcom_notify, sc);
 	ucom_attach(&sc->sc_ucom);
-
-	free(devinfo, M_USBDEV);
-	USB_ATTACH_SUCCESS_RETURN;
+	return 0;
 
 error:
-	free(devinfo, M_USBDEV);
-	USB_ATTACH_ERROR_RETURN;
+	return ENXIO;
 }
 
-USB_DETACH(uplcom)
+static int
+uplcom_detach(device_t self)
 {
-	USB_DETACH_START(uplcom, sc);
+	struct uplcom_softc *sc = device_get_softc(self);
 	int rv = 0;
 
 	DPRINTF(("uplcom_detach: sc = %p\n", sc));
@@ -581,7 +569,7 @@ USB_DETACH(uplcom)
 	return (rv);
 }
 
-Static usbd_status
+static usbd_status
 uplcom_reset(struct uplcom_softc *sc)
 {
 	usb_device_request_t req;
@@ -596,7 +584,7 @@ uplcom_reset(struct uplcom_softc *sc)
 	err = usbd_do_request(sc->sc_ucom.sc_udev, &req, 0);
 	if (err) {
 		printf("%s: uplcom_reset: %s\n",
-		       USBDEVNAME(sc->sc_ucom.sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_ucom.sc_dev), usbd_errstr(err));
 		return (EIO);
 	}
 
@@ -611,7 +599,7 @@ struct pl2303x_init {
 	uint16_t	length;
 };
 
-Static const struct pl2303x_init pl2303x[] = {
+static const struct pl2303x_init pl2303x[] = {
 	{ UT_READ_VENDOR_DEVICE,  UPLCOM_SET_REQUEST, 0x8484,    0, 0 },
 	{ UT_WRITE_VENDOR_DEVICE, UPLCOM_SET_REQUEST, 0x0404,    0, 0 },
 	{ UT_READ_VENDOR_DEVICE,  UPLCOM_SET_REQUEST, 0x8484,    0, 0 },
@@ -626,7 +614,7 @@ Static const struct pl2303x_init pl2303x[] = {
 };
 #define N_PL2302X_INIT	(sizeof(pl2303x)/sizeof(pl2303x[0]))
 
-Static usbd_status
+static usbd_status
 uplcom_pl2303x_init(struct uplcom_softc *sc)
 {
 	usb_device_request_t req;
@@ -643,7 +631,7 @@ uplcom_pl2303x_init(struct uplcom_softc *sc)
 		err = usbd_do_request(sc->sc_ucom.sc_udev, &req, 0);
 		if (err) {
 			printf("%s: uplcom_pl2303x_init: %d: %s\n",
-				USBDEVNAME(sc->sc_ucom.sc_dev), i,
+				device_get_nameunit(sc->sc_ucom.sc_dev), i,
 				usbd_errstr(err));
 			return (EIO);
 		}
@@ -652,7 +640,7 @@ uplcom_pl2303x_init(struct uplcom_softc *sc)
 	return (0);
 }
 
-Static void
+static void
 uplcom_set_line_state(struct uplcom_softc *sc)
 {
 	usb_device_request_t req;
@@ -670,10 +658,10 @@ uplcom_set_line_state(struct uplcom_softc *sc)
 	err = usbd_do_request(sc->sc_ucom.sc_udev, &req, 0);
 	if (err)
 		printf("%s: uplcom_set_line_status: %s\n",
-		       USBDEVNAME(sc->sc_ucom.sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_ucom.sc_dev), usbd_errstr(err));
 }
 
-Static void
+static void
 uplcom_set(void *addr, int portno, int reg, int onoff)
 {
 	struct uplcom_softc *sc = addr;
@@ -693,7 +681,7 @@ uplcom_set(void *addr, int portno, int reg, int onoff)
 	}
 }
 
-Static void
+static void
 uplcom_dtr(struct uplcom_softc *sc, int onoff)
 {
 	DPRINTF(("uplcom_dtr: onoff = %d\n", onoff));
@@ -705,7 +693,7 @@ uplcom_dtr(struct uplcom_softc *sc, int onoff)
 	uplcom_set_line_state(sc);
 }
 
-Static void
+static void
 uplcom_rts(struct uplcom_softc *sc, int onoff)
 {
 	DPRINTF(("uplcom_rts: onoff = %d\n", onoff));
@@ -717,7 +705,7 @@ uplcom_rts(struct uplcom_softc *sc, int onoff)
 	uplcom_set_line_state(sc);
 }
 
-Static void
+static void
 uplcom_break(struct uplcom_softc *sc, int onoff)
 {
 	usb_device_request_t req;
@@ -734,10 +722,10 @@ uplcom_break(struct uplcom_softc *sc, int onoff)
 	err = usbd_do_request(sc->sc_ucom.sc_udev, &req, 0);
 	if (err)
 		printf("%s: uplcom_break: %s\n",
-		       USBDEVNAME(sc->sc_ucom.sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_ucom.sc_dev), usbd_errstr(err));
 }
 
-Static usbd_status
+static usbd_status
 uplcom_set_crtscts(struct uplcom_softc *sc)
 {
 	usb_device_request_t req;
@@ -757,14 +745,14 @@ uplcom_set_crtscts(struct uplcom_softc *sc)
 	err = usbd_do_request(sc->sc_ucom.sc_udev, &req, 0);
 	if (err) {
 		printf("%s: uplcom_set_crtscts: %s\n",
-		       USBDEVNAME(sc->sc_ucom.sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_ucom.sc_dev), usbd_errstr(err));
 		return (err);
 	}
 
 	return (USBD_NORMAL_COMPLETION);
 }
 
-Static usbd_status
+static usbd_status
 uplcom_set_line_coding(struct uplcom_softc *sc, usb_cdc_line_state_t *state)
 {
 	usb_device_request_t req;
@@ -789,7 +777,7 @@ uplcom_set_line_coding(struct uplcom_softc *sc, usb_cdc_line_state_t *state)
 	err = usbd_do_request(sc->sc_ucom.sc_udev, &req, state);
 	if (err) {
 		printf("%s: uplcom_set_line_coding: %s\n",
-		       USBDEVNAME(sc->sc_ucom.sc_dev), usbd_errstr(err));
+		       device_get_nameunit(sc->sc_ucom.sc_dev), usbd_errstr(err));
 		return (err);
 	}
 
@@ -798,7 +786,7 @@ uplcom_set_line_coding(struct uplcom_softc *sc, usb_cdc_line_state_t *state)
 	return (USBD_NORMAL_COMPLETION);
 }
 
-Static const int uplcom_rates[] = {
+static const int uplcom_rates[] = {
 	75, 150, 300, 600, 1200, 1800, 2400, 3600, 4800, 7200, 9600, 14400,
 	19200, 28800, 38400, 57600, 115200,
 	/*
@@ -809,7 +797,7 @@ Static const int uplcom_rates[] = {
 };
 #define N_UPLCOM_RATES	(sizeof(uplcom_rates)/sizeof(uplcom_rates[0]))
 
-Static int
+static int
 uplcom_param(void *addr, int portno, struct termios *t)
 {
 	struct uplcom_softc *sc = addr;
@@ -868,7 +856,7 @@ uplcom_param(void *addr, int portno, struct termios *t)
 	return (0);
 }
 
-Static int
+static int
 uplcom_open(void *addr, int portno)
 {
 	struct uplcom_softc *sc = addr;
@@ -893,7 +881,7 @@ uplcom_open(void *addr, int portno)
 					  uplcominterval);
 		if (err) {
 			printf("%s: cannot open interrupt pipe (addr %d)\n",
-			       USBDEVNAME(sc->sc_ucom.sc_dev),
+			       device_get_nameunit(sc->sc_ucom.sc_dev),
 			       sc->sc_intr_number);
 			return (EIO);
 		}
@@ -905,7 +893,7 @@ uplcom_open(void *addr, int portno)
 	return (0);
 }
 
-Static void
+static void
 uplcom_close(void *addr, int portno)
 {
 	struct uplcom_softc *sc = addr;
@@ -920,19 +908,19 @@ uplcom_close(void *addr, int portno)
 		err = usbd_abort_pipe(sc->sc_intr_pipe);
 		if (err)
 			printf("%s: abort interrupt pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_ucom.sc_dev),
+			       device_get_nameunit(sc->sc_ucom.sc_dev),
 			       usbd_errstr(err));
 		err = usbd_close_pipe(sc->sc_intr_pipe);
 		if (err)
 			printf("%s: close interrupt pipe failed: %s\n",
-			       USBDEVNAME(sc->sc_ucom.sc_dev),
+			       device_get_nameunit(sc->sc_ucom.sc_dev),
 			       usbd_errstr(err));
 		free(sc->sc_intr_buf, M_USBDEV);
 		sc->sc_intr_pipe = NULL;
 	}
 }
 
-Static void
+static void
 uplcom_intr(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 {
 	struct uplcom_softc *sc = priv;
@@ -947,14 +935,14 @@ uplcom_intr(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 			return;
 
 		DPRINTF(("%s: uplcom_intr: abnormal status: %s\n",
-			USBDEVNAME(sc->sc_ucom.sc_dev),
+			device_get_nameunit(sc->sc_ucom.sc_dev),
 			usbd_errstr(status)));
 		usbd_clear_endpoint_stall_async(sc->sc_intr_pipe);
 		return;
 	}
 
 	DPRINTF(("%s: uplcom status = %02x\n",
-		 USBDEVNAME(sc->sc_ucom.sc_dev), buf[8]));
+		 device_get_nameunit(sc->sc_ucom.sc_dev), buf[8]));
 
 	sc->sc_lsr = sc->sc_msr = 0;
 	pstatus = buf[8];
@@ -975,7 +963,7 @@ uplcom_intr(usbd_xfer_handle xfer, usbd_private_handle priv, usbd_status status)
 	taskqueue_enqueue(taskqueue_swi_giant, &sc->sc_task);
 }
 
-Static void
+static void
 uplcom_notify(void *arg, int count)
 {
 	struct uplcom_softc *sc;
@@ -986,7 +974,7 @@ uplcom_notify(void *arg, int count)
 	ucom_status_change(&sc->sc_ucom);
 }
 
-Static void
+static void
 uplcom_get_status(void *addr, int portno, u_char *lsr, u_char *msr)
 {
 	struct uplcom_softc *sc = addr;
@@ -1000,9 +988,9 @@ uplcom_get_status(void *addr, int portno, u_char *lsr, u_char *msr)
 }
 
 #if 0 /* TODO */
-Static int
+static int
 uplcom_ioctl(void *addr, int portno, u_long cmd, caddr_t data, int flag,
-	     usb_proc_ptr p)
+	     struct thread *p)
 {
 	struct uplcom_softc *sc = addr;
 	int error = 0;
