@@ -5,7 +5,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/usb/uscanner.c,v 1.63.2.7 2007/11/04 03:28:31 luigi Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/usb/uscanner.c,v 1.89 2007/10/05 07:26:39 luigi Exp $");
 
 /*-
  * Copyright (c) 2000 The NetBSD Foundation, Inc.
@@ -50,15 +50,11 @@ __FBSDID("$FreeBSD: src/sys/dev/usb/uscanner.c,v 1.63.2.7 2007/11/04 03:28:31 lu
 #include <sys/systm.h>
 #include <sys/kernel.h>
 #include <sys/malloc.h>
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-#include <sys/device.h>
-#elif defined(__FreeBSD__)
 #include <sys/module.h>
 #include <sys/bus.h>
 #include <sys/conf.h>
 #include <sys/fcntl.h>
 #include <sys/filio.h>
-#endif
 #include <sys/tty.h>
 #include <sys/file.h>
 #include <sys/selinfo.h>
@@ -75,8 +71,8 @@ __FBSDID("$FreeBSD: src/sys/dev/usb/uscanner.c,v 1.63.2.7 2007/11/04 03:28:31 lu
 #include "usbdevs.h"
 
 #ifdef USB_DEBUG
-#define DPRINTF(x)	if (uscannerdebug) logprintf x
-#define DPRINTFN(n,x)	if (uscannerdebug>(n)) logprintf x
+#define DPRINTF(x)	if (uscannerdebug) printf x
+#define DPRINTFN(n,x)	if (uscannerdebug>(n)) printf x
 int	uscannerdebug = 0;
 SYSCTL_NODE(_hw_usb, OID_AUTO, uscanner, CTLFLAG_RW, 0, "USB uscanner");
 SYSCTL_INT(_hw_usb_uscanner, OID_AUTO, debug, CTLFLAG_RW,
@@ -96,6 +92,7 @@ struct uscan_info {
 static const struct uscan_info uscanner_devs[] = {
   /* Acer Peripherals */
  {{ USB_VENDOR_ACERP, USB_PRODUCT_ACERP_ACERSCAN_320U }, 0 },
+ {{ USB_VENDOR_ACERP, USB_PRODUCT_ACERP_ACERSCAN_4300U }, 0 },
  {{ USB_VENDOR_ACERP, USB_PRODUCT_ACERP_ACERSCAN_640U }, 0 },
  {{ USB_VENDOR_ACERP, USB_PRODUCT_ACERP_ACERSCAN_640BT }, 0 },
  {{ USB_VENDOR_ACERP, USB_PRODUCT_ACERP_ACERSCAN_620U }, 0 },
@@ -242,12 +239,10 @@ static const struct uscan_info uscanner_devs[] = {
 #define	USCANNER_BUFFERSIZE	1024
 
 struct uscanner_softc {
-	USBBASEDEVICE		sc_dev;		/* base device */
+	device_t		sc_dev;		/* base device */
 	usbd_device_handle	sc_udev;
 	usbd_interface_handle	sc_iface;
-#if defined(__FreeBSD__)
 	struct cdev *dev;
-#endif
 
 	u_int			sc_dev_flags;
 
@@ -272,9 +267,6 @@ struct uscanner_softc {
 	u_char			sc_dying;
 };
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-cdev_decl(uscanner);
-#elif defined(__FreeBSD__)
 d_open_t  uscanneropen;
 d_close_t uscannerclose;
 d_read_t  uscannerread;
@@ -282,7 +274,7 @@ d_write_t uscannerwrite;
 d_poll_t  uscannerpoll;
 
 
-Static struct cdevsw uscanner_cdevsw = {
+static struct cdevsw uscanner_cdevsw = {
 	.d_version =	D_VERSION,
 	.d_flags =	D_NEEDGIANT,
 	.d_open =	uscanneropen,
@@ -292,20 +284,39 @@ Static struct cdevsw uscanner_cdevsw = {
 	.d_poll =	uscannerpoll,
 	.d_name =	"uscanner",
 };
-#endif
 
-Static int uscanner_do_read(struct uscanner_softc *, struct uio *, int);
-Static int uscanner_do_write(struct uscanner_softc *, struct uio *, int);
-Static void uscanner_do_close(struct uscanner_softc *);
+static int uscanner_do_read(struct uscanner_softc *, struct uio *, int);
+static int uscanner_do_write(struct uscanner_softc *, struct uio *, int);
+static void uscanner_do_close(struct uscanner_softc *);
 
 #define USCANNERUNIT(n) (minor(n))
 
-USB_DECLARE_DRIVER(uscanner);
+static device_probe_t uscanner_match;
+static device_attach_t uscanner_attach;
+static device_detach_t uscanner_detach;
 
-USB_MATCH(uscanner)
+static device_method_t uscanner_methods[] = {
+	/* Device interface */
+	DEVMETHOD(device_probe,		uscanner_match),
+	DEVMETHOD(device_attach,	uscanner_attach),
+	DEVMETHOD(device_detach,	uscanner_detach),
+
+	{ 0, 0 }
+};
+
+static driver_t uscanner_driver = {
+	"uscanner",
+	uscanner_methods,
+	sizeof(struct uscanner_softc)
+};
+
+static devclass_t uscanner_devclass;
+
+static int
+uscanner_match(device_t self)
 {
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 	usb_interface_descriptor_t *id;
-	USB_MATCH_START(uscanner, uaa);
 
 	if (uaa->iface == NULL)
 		return UMATCH_NONE;	/* do not grab the entire device */
@@ -325,25 +336,23 @@ USB_MATCH(uscanner)
 	if (id->bInterfaceClass == UICLASS_PRINTER ||
 	    id->bInterfaceClass == UICLASS_MASS)
 		return UMATCH_NONE;
- 
+
 	return UMATCH_VENDOR_PRODUCT;	/* ok we found it */
 }
 
-USB_ATTACH(uscanner)
+static int
+uscanner_attach(device_t self)
 {
-	USB_ATTACH_START(uscanner, sc, uaa);
+	struct uscanner_softc *sc = device_get_softc(self);
+	struct usb_attach_arg *uaa = device_get_ivars(self);
 	usb_interface_descriptor_t *id = 0;
 	usb_endpoint_descriptor_t *ed, *ed_bulkin = NULL, *ed_bulkout = NULL;
-	char devinfo[1024];
 	int i;
 	usbd_status err;
 	int ifnum;
 
-	usbd_devinfo(uaa->device, 0, devinfo);
-	USB_ATTACH_SETUP;
-
+	sc->sc_dev = self;
 	sc->sc_dev_flags = uscanner_lookup(uaa->vendor, uaa->product)->flags;
-
 	sc->sc_udev = uaa->device;
 
 	id = usbd_get_interface_descriptor(uaa->iface);
@@ -361,8 +370,8 @@ USB_ATTACH(uscanner)
 	err = usbd_set_config_no(uaa->device, 1, 1); /* XXX */
 	if (err) {
 		printf("%s: setting config no failed\n",
-		    USBDEVNAME(sc->sc_dev));
-		USB_ATTACH_ERROR_RETURN;
+		    device_get_nameunit(sc->sc_dev));
+		return ENXIO;
 	}
 #endif
 	err = usbd_device2interface_handle(sc->sc_udev, ifnum, &sc->sc_iface);
@@ -370,8 +379,8 @@ USB_ATTACH(uscanner)
 	    id = usbd_get_interface_descriptor(sc->sc_iface);
 	if (err || id == 0) {
 		printf("%s: could not get interface descriptor, err=%d,id=%p\n",
-		       USBDEVNAME(sc->sc_dev), err, id);
-		USB_ATTACH_ERROR_RETURN;
+		       device_get_nameunit(sc->sc_dev), err, id);
+		return ENXIO;
 	}
 
 	/* Find the two first bulk endpoints */
@@ -379,8 +388,8 @@ USB_ATTACH(uscanner)
 		ed = usbd_interface2endpoint_descriptor(sc->sc_iface, i);
 		if (ed == 0) {
 			printf("%s: could not read endpoint descriptor\n",
-			       USBDEVNAME(sc->sc_dev));
-			USB_ATTACH_ERROR_RETURN;
+			       device_get_nameunit(sc->sc_dev));
+			return ENXIO;
 		}
 
 		if (UE_GET_DIR(ed->bEndpointAddress) == UE_DIR_IN
@@ -398,33 +407,31 @@ USB_ATTACH(uscanner)
 	/* Verify that we goething sensible */
 	if (ed_bulkin == NULL || ed_bulkout == NULL) {
 		printf("%s: bulk-in and/or bulk-out endpoint not found\n",
-			USBDEVNAME(sc->sc_dev));
-		USB_ATTACH_ERROR_RETURN;
+			device_get_nameunit(sc->sc_dev));
+		return ENXIO;
 	}
 
 	sc->sc_bulkin = ed_bulkin->bEndpointAddress;
 	sc->sc_bulkout = ed_bulkout->bEndpointAddress;
 
-#ifdef __FreeBSD__
 	/* the main device, ctrl endpoint */
-	sc->dev = make_dev(&uscanner_cdevsw, USBDEVUNIT(sc->sc_dev),
-		UID_ROOT, GID_OPERATOR, 0644, "%s", USBDEVNAME(sc->sc_dev));
-#endif
+	sc->dev = make_dev(&uscanner_cdevsw, device_get_unit(sc->sc_dev),
+		UID_ROOT, GID_OPERATOR, 0644, "%s", device_get_nameunit(sc->sc_dev));
+	usbd_add_drv_event(USB_EVENT_DRIVER_ATTACH, sc->sc_udev,sc->sc_dev);
 
-	usbd_add_drv_event(USB_EVENT_DRIVER_ATTACH, sc->sc_udev,
-			   USBDEV(sc->sc_dev));
-
-	USB_ATTACH_SUCCESS_RETURN;
+	return 0;
 }
 
 int
-uscanneropen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
+uscanneropen(struct cdev *dev, int flag, int mode, struct thread *p)
 {
 	struct uscanner_softc *sc;
 	int unit = USCANNERUNIT(dev);
 	usbd_status err;
 
-	USB_GET_SC_OPEN(uscanner, unit, sc);
+	sc = devclass_get_softc(uscanner_devclass, unit);
+	if (sc == NULL)
+		return (ENXIO);
 
  	DPRINTFN(5, ("uscanneropen: flag=%d, mode=%d, unit=%d\n",
 		     flag, mode, unit));
@@ -450,7 +457,7 @@ uscanneropen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 				     USBD_EXCLUSIVE_USE, &sc->sc_bulkin_pipe);
 		if (err) {
 			printf("%s: cannot open bulk-in pipe (addr %d)\n",
-			       USBDEVNAME(sc->sc_dev), sc->sc_bulkin);
+			       device_get_nameunit(sc->sc_dev), sc->sc_bulkin);
 			uscanner_do_close(sc);
 			return (EIO);
 		}
@@ -460,7 +467,7 @@ uscanneropen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 				     USBD_EXCLUSIVE_USE, &sc->sc_bulkout_pipe);
 		if (err) {
 			printf("%s: cannot open bulk-out pipe (addr %d)\n",
-			       USBDEVNAME(sc->sc_dev), sc->sc_bulkout);
+			       device_get_nameunit(sc->sc_dev), sc->sc_bulkout);
 			uscanner_do_close(sc);
 			return (EIO);
 		}
@@ -481,12 +488,11 @@ uscanneropen(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
 }
 
 int
-uscannerclose(struct cdev *dev, int flag, int mode, usb_proc_ptr p)
+uscannerclose(struct cdev *dev, int flag, int mode, struct thread *p)
 {
 	struct uscanner_softc *sc;
 
-	USB_GET_SC(uscanner, USCANNERUNIT(dev), sc);
-
+	sc = devclass_get_softc(uscanner_devclass, USCANNERUNIT(dev));
 	DPRINTFN(5, ("uscannerclose: flag=%d, mode=%d, unit=%d\n",
 		     flag, mode, USCANNERUNIT(dev)));
 
@@ -539,14 +545,14 @@ uscanner_do_close(struct uscanner_softc *sc)
 	sc->sc_state &= ~USCANNER_OPEN;
 }
 
-Static int
+static int
 uscanner_do_read(struct uscanner_softc *sc, struct uio *uio, int flag)
 {
 	u_int32_t n, tn;
 	usbd_status err;
 	int error = 0;
 
-	DPRINTFN(5, ("%s: uscannerread\n", USBDEVNAME(sc->sc_dev)));
+	DPRINTFN(5, ("%s: uscannerread\n", device_get_nameunit(sc->sc_dev)));
 
 	if (sc->sc_dying)
 		return (EIO);
@@ -584,24 +590,23 @@ uscannerread(struct cdev *dev, struct uio *uio, int flag)
 	struct uscanner_softc *sc;
 	int error;
 
-	USB_GET_SC(uscanner, USCANNERUNIT(dev), sc);
-
+	sc = devclass_get_softc(uscanner_devclass, USCANNERUNIT(dev));
 	sc->sc_refcnt++;
 	error = uscanner_do_read(sc, uio, flag);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 
 	return (error);
 }
 
-Static int
+static int
 uscanner_do_write(struct uscanner_softc *sc, struct uio *uio, int flag)
 {
 	u_int32_t n;
 	int error = 0;
 	usbd_status err;
 
-	DPRINTFN(5, ("%s: uscanner_do_write\n", USBDEVNAME(sc->sc_dev)));
+	DPRINTFN(5, ("%s: uscanner_do_write\n", device_get_nameunit(sc->sc_dev)));
 
 	if (sc->sc_dying)
 		return (EIO);
@@ -634,46 +639,21 @@ uscannerwrite(struct cdev *dev, struct uio *uio, int flag)
 	struct uscanner_softc *sc;
 	int error;
 
-	USB_GET_SC(uscanner, USCANNERUNIT(dev), sc);
-
+	sc = devclass_get_softc(uscanner_devclass, USCANNERUNIT(dev));
 	sc->sc_refcnt++;
 	error = uscanner_do_write(sc, uio, flag);
 	if (--sc->sc_refcnt < 0)
-		usb_detach_wakeup(USBDEV(sc->sc_dev));
+		usb_detach_wakeup(sc->sc_dev);
 	return (error);
 }
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-int
-uscanner_activate(device_ptr_t self, enum devact act)
+static int
+uscanner_detach(device_t self)
 {
-	struct uscanner_softc *sc = (struct uscanner_softc *)self;
-
-	switch (act) {
-	case DVACT_ACTIVATE:
-		return (EOPNOTSUPP);
-
-	case DVACT_DEACTIVATE:
-		sc->sc_dying = 1;
-		break;
-	}
-	return (0);
-}
-#endif
-
-USB_DETACH(uscanner)
-{
-	USB_DETACH_START(uscanner, sc);
+	struct uscanner_softc *sc = device_get_softc(self);
 	int s;
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-	int maj, mn;
-#endif
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-	DPRINTF(("uscanner_detach: sc=%p flags=%d\n", sc, flags));
-#elif defined(__FreeBSD__)
 	DPRINTF(("uscanner_detach: sc=%p\n", sc));
-#endif
 
 	sc->sc_dying = 1;
 	sc->sc_dev_flags = 0;	/* make close really close device */
@@ -687,38 +667,24 @@ USB_DETACH(uscanner)
 	s = splusb();
 	if (--sc->sc_refcnt >= 0) {
 		/* Wait for processes to go away. */
-		usb_detach_wait(USBDEV(sc->sc_dev));
+		usb_detach_wait(sc->sc_dev);
 	}
 	splx(s);
 
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-	/* locate the major number */
-	for (maj = 0; maj < nchrdev; maj++)
-		if (cdevsw[maj].d_open == uscanneropen)
-			break;
-
-	/* Nuke the vnodes for any open instances (calls close). */
-	mn = self->dv_unit * USB_MAX_ENDPOINTS;
-	vdevgone(maj, mn, mn + USB_MAX_ENDPOINTS - 1, VCHR);
-#elif defined(__FreeBSD__)
 	/* destroy the device for the control endpoint */
 	destroy_dev(sc->dev);
-#endif
-
-	usbd_add_drv_event(USB_EVENT_DRIVER_DETACH, sc->sc_udev,
-			   USBDEV(sc->sc_dev));
+	usbd_add_drv_event(USB_EVENT_DRIVER_DETACH, sc->sc_udev, sc->sc_dev);
 
 	return (0);
 }
 
 int
-uscannerpoll(struct cdev *dev, int events, usb_proc_ptr p)
+uscannerpoll(struct cdev *dev, int events, struct thread *p)
 {
 	struct uscanner_softc *sc;
 	int revents = 0;
 
-	USB_GET_SC(uscanner, USCANNERUNIT(dev), sc);
-
+	sc = devclass_get_softc(uscanner_devclass, USCANNERUNIT(dev));
 	if (sc->sc_dying)
 		return (EIO);
 
@@ -733,6 +699,5 @@ uscannerpoll(struct cdev *dev, int events, usb_proc_ptr p)
 	return (revents);
 }
 
-#if defined(__FreeBSD__)
+MODULE_DEPEND(uscanner, usb, 1, 1, 1);
 DRIVER_MODULE(uscanner, uhub, uscanner_driver, uscanner_devclass, usbd_driver_load, 0);
-#endif
