@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/vge/if_vge.c,v 1.14.2.13 2007/07/14 20:25:20 csjp Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/vge/if_vge.c,v 1.31 2007/03/04 03:38:08 csjp Exp $");
 
 /*
  * VIA Networking Technologies VT612x PCI gigabit ethernet NIC driver.
@@ -120,7 +120,7 @@ MODULE_DEPEND(vge, pci, 1, 1, 1);
 MODULE_DEPEND(vge, ether, 1, 1, 1);
 MODULE_DEPEND(vge, miibus, 1, 1, 1);
 
-/* "controller miibus0" required.  See GENERIC if you get errors here. */
+/* "device miibus" required.  See GENERIC if you get errors here. */
 #include "miibus_if.h"
 
 #include <dev/vge/if_vgereg.h>
@@ -1037,7 +1037,7 @@ vge_attach(dev)
 
 	/* Hook interrupt last to avoid having to lock softc */
 	error = bus_setup_intr(dev, sc->vge_irq, INTR_TYPE_NET|INTR_MPSAFE,
-	    vge_intr, sc, &sc->vge_intrhand);
+	    NULL, vge_intr, sc, &sc->vge_intrhand);
 
 	if (error) {
 		printf("vge%d: couldn't set up irq\n", unit);
@@ -1094,8 +1094,6 @@ vge_detach(dev)
 		ifp->if_flags &= ~IFF_UP;
 		ether_ifdetach(ifp);
 	}
-	if (ifp)
-		if_free(ifp);
 	if (sc->vge_miibus)
 		device_delete_child(dev, sc->vge_miibus);
 	bus_generic_detach(dev);
@@ -1107,6 +1105,8 @@ vge_detach(dev)
 	if (sc->vge_res)
 		bus_release_resource(dev, SYS_RES_MEMORY,
 		    VGE_PCI_LOMEM, sc->vge_res);
+	if (ifp)
+		if_free(ifp);
 
 	/* Unload and free the RX DMA ring memory and map */
 
@@ -1460,10 +1460,9 @@ vge_rxeof(sc)
 			 * However, the 16-bit vlan tag is stored in big-endian,
 			 * so we have to byte swap it.
 			 */
-			VLAN_INPUT_TAG_NEW(ifp, m,
-			    bswap16(rxctl & VGE_RDCTL_VLANID));
-			if (m == NULL)
-				continue;
+			m->m_pkthdr.ether_vtag =
+			    bswap16(rxctl & VGE_RDCTL_VLANID);
+			m->m_flags |= M_VLANTAG;
 		}
 
 		VGE_UNLOCK(sc);
@@ -1715,7 +1714,6 @@ vge_encap(sc, m_head, idx)
 	struct vge_dmaload_arg	arg;
 	bus_dmamap_t		map;
 	int			error;
-	struct m_tag		*mtag;
 
 	if (sc->vge_ldata.vge_tx_free <= 2)
 		return (EFBIG);
@@ -1774,10 +1772,9 @@ vge_encap(sc, m_head, idx)
 	 * Set up hardware VLAN tagging.
 	 */
 
-	mtag = VLAN_OUTPUT_TAG(sc->vge_ifp, m_head);
-	if (mtag != NULL)
+	if (m_head->m_flags & M_VLANTAG)
 		sc->vge_ldata.vge_tx_list[idx].vge_ctl |=
-		    htole32((uint16_t)VLAN_TAG_VALUE(mtag) | VGE_TDCTL_VTAG);
+		    htole32(m_head->m_pkthdr.ether_vtag | VGE_TDCTL_VTAG);
 
 	sc->vge_ldata.vge_tx_list[idx].vge_sts |= htole32(VGE_TDSTS_OWN);
 
@@ -1917,7 +1914,7 @@ vge_init(xsc)
 
 	/* Set our station address */
 	for (i = 0; i < ETHER_ADDR_LEN; i++)
-		CSR_WRITE_1(sc, VGE_PAR0 + i, IFP2ENADDR(sc->vge_ifp)[i]);
+		CSR_WRITE_1(sc, VGE_PAR0 + i, IF_LLADDR(sc->vge_ifp)[i]);
 
 	/*
 	 * Set receive FIFO threshold. Also allow transmission and
