@@ -28,9 +28,10 @@
  * SUCH DAMAGE.
  *
  * $Id: vkbd.c,v 1.20 2004/11/15 23:53:30 max Exp $
- * $FreeBSD: src/sys/dev/vkbd/vkbd.c,v 1.7.2.4 2006/10/04 06:09:10 ru Exp $
+ * $FreeBSD: src/sys/dev/vkbd/vkbd.c,v 1.11 2006/09/27 19:57:00 ru Exp $
  */
 
+#include "opt_compat.h"
 #include "opt_kbd.h"
 
 #include <sys/param.h>
@@ -1092,7 +1093,9 @@ vkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 {
 	vkbd_state_t	*state = (vkbd_state_t *) kbd->kb_data;
 	int		 i;
+#ifdef COMPAT_FREEBSD6
 	int		 ival;
+#endif
 
 	VKBD_LOCK(state);
 
@@ -1101,10 +1104,12 @@ vkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		*(int *)arg = state->ks_mode;
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 7):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSKBMODE:		/* set keyboard mode */
 		switch (*(int *)arg) {
 		case K_XLATE:
@@ -1135,10 +1140,12 @@ vkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		*(int *)arg = KBD_LED_VAL(kbd);
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 66):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSETLED:		/* set keyboard LED */
 		/* NOTE: lock key state in ks_state won't be changed */
 		if (*(int *)arg & ~LOCK_MASK) {
@@ -1164,10 +1171,12 @@ vkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		*(int *)arg = state->ks_state & LOCK_MASK;
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 20):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSKBSTATE:	/* set lock key state */
 		if (*(int *)arg & ~LOCK_MASK) {
 			VKBD_UNLOCK(state);
@@ -1187,10 +1196,12 @@ vkbd_ioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		vkbd_status_changed(state);
 		break;
 
+#ifdef COMPAT_FREEBSD6
 	case _IO('K', 67):
 		ival = IOCPARM_IVAL(arg);
 		arg = (caddr_t)&ival;
 		/* FALLTHROUGH */
+#endif
 	case KDSETRAD:		/* set keyboard repeat rate (old interface) */
 		kbd->kb_delay1 = typematic_delay(*(int *)arg);
 		kbd->kb_delay2 = typematic_rate(*(int *)arg);
