@@ -62,7 +62,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/dev/wi/if_wi.c,v 1.180.2.12 2007/01/27 06:37:41 sam Exp $");
+__FBSDID("$FreeBSD: src/sys/dev/wi/if_wi.c,v 1.214 2007/09/16 20:02:29 thompsa Exp $");
 
 #define WI_HERMES_AUTOINC_WAR	/* Work around data write autoinc bug. */
 #define WI_HERMES_STATS_WAR	/* Work around stats counter bug. */
@@ -74,6 +74,7 @@ __FBSDID("$FreeBSD: src/sys/dev/wi/if_wi.c,v 1.180.2.12 2007/01/27 06:37:41 sam
 #include <sys/endian.h>
 #include <sys/sockio.h>
 #include <sys/mbuf.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/kernel.h>
 #include <sys/socket.h>
@@ -85,7 +86,6 @@ __FBSDID("$FreeBSD: src/sys/dev/wi/if_wi.c,v 1.180.2.12 2007/01/27 06:37:41 sam
 
 #include <machine/bus.h>
 #include <machine/resource.h>
-#include <machine/clock.h>
 #include <machine/atomic.h>
 #include <sys/rman.h>
 
@@ -114,8 +114,12 @@ __FBSDID("$FreeBSD: src/sys/dev/wi/if_wi.c,v 1.180.2.12 2007/01/27 06:37:41 sam
 
 static void wi_start_locked(struct ifnet *);
 static void wi_start(struct ifnet *);
-static int  wi_reset(struct wi_softc *);
-static void wi_watchdog(struct ifnet *);
+static int  wi_start_tx(struct ifnet *ifp, struct wi_frame *frmhdr,
+		struct mbuf *m0);
+static int  wi_raw_xmit(struct ieee80211_node *, struct mbuf *,
+		const struct ieee80211_bpf_params *);
+static int  wi_reset(struct ifnet *);
+static void wi_watchdog(void *);
 static int  wi_ioctl(struct ifnet *, u_long, caddr_t);
 static int  wi_media_change(struct ifnet *);
 static void wi_media_status(struct ifnet *, struct ifmediareq *);
@@ -128,8 +132,10 @@ static void wi_info_intr(struct wi_softc *);
 static int  wi_key_alloc(struct ieee80211com *, const struct ieee80211_key *,
 		ieee80211_keyix *, ieee80211_keyix *);
 
+#if 0
 static int  wi_get_cfg(struct ifnet *, u_long, caddr_t);
 static int  wi_set_cfg(struct ifnet *, u_long, caddr_t);
+#endif
 static int  wi_write_txrate(struct wi_softc *);
 static int  wi_write_wep(struct wi_softc *);
 static int  wi_write_multi(struct wi_softc *);
@@ -152,14 +158,26 @@ static void wi_scan_result(struct wi_softc *, int, int);
 
 static void wi_dump_pkt(struct wi_frame *, struct ieee80211_node *, int rssi);
 
+#if 0
 static int wi_get_debug(struct wi_softc *, struct wi_req *);
 static int wi_set_debug(struct wi_softc *, struct wi_req *);
+#endif
 
 /* support to download firmware for symbol CF card */
 static int wi_symbol_write_firm(struct wi_softc *, const void *, int,
 		const void *, int);
 static int wi_symbol_set_hcr(struct wi_softc *, int);
 
+static void wi_scan_start(struct ieee80211com *);
+static void wi_scan_curchan(struct ieee80211com *, unsigned long);
+static void wi_scan_mindwell(struct ieee80211com *);
+static void wi_scan_end(struct ieee80211com *);
+static void wi_set_channel(struct ieee80211com *);
+static void wi_update_slot(struct ifnet *);
+static struct ieee80211_node *wi_node_alloc(struct ieee80211_node_table *);
+static int wi_ioctl_get(struct ifnet *ifp, u_long command, caddr_t data);
+static int wi_ioctl_set(struct ifnet *ifp, u_long command, caddr_t data);
+	
 static __inline int
 wi_write_val(struct wi_softc *sc, int rid, u_int16_t val)
 {
@@ -254,13 +272,14 @@ wi_attach(device_t dev)
 		wi_free(dev);
 		return (ENOSPC);
 	}
+	ifp->if_softc = sc;
 
 	/*
 	 * NB: no locking is needed here; don't put it here
 	 *     unless you can prove it!
 	 */
 	error = bus_setup_intr(dev, sc->irq, INTR_TYPE_NET | INTR_MPSAFE,
-	    wi_intr, sc, &sc->wi_intrhand);
+	    NULL, wi_intr, sc, &sc->wi_intrhand);
 
 	if (error) {
 		device_printf(dev, "bus_setup_intr() failed! (%d)\n", error);
@@ -270,11 +289,12 @@ wi_attach(device_t dev)
 
 	mtx_init(&sc->sc_mtx, device_get_nameunit(dev), MTX_NETWORK_LOCK,
 	    MTX_DEF | MTX_RECURSE);
+	callout_init_mtx(&sc->sc_watchdog, &sc->sc_mtx, 0);
 
 	sc->sc_firmware_type = WI_NOTYPE;
 	sc->wi_cmd_count = 500;
 	/* Reset the NIC. */
-	if (wi_reset(sc) != 0)
+	if (wi_reset(ifp) != 0)
 		return ENXIO;		/* XXX */
 
 	/*
@@ -303,12 +323,10 @@ wi_attach(device_t dev)
 	/* Read NIC identification */
 	wi_read_nicid(sc);
 
-	ifp->if_softc = sc;
 	if_initname(ifp, device_get_name(dev), device_get_unit(dev));
 	ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
 	ifp->if_ioctl = wi_ioctl;
 	ifp->if_start = wi_start;
-	ifp->if_watchdog = wi_watchdog;
 	ifp->if_init = wi_init;
 	IFQ_SET_MAXLEN(&ifp->if_snd, IFQ_MAXLEN);
 	ifp->if_snd.ifq_drv_maxlen = IFQ_MAXLEN;
@@ -334,11 +352,14 @@ wi_attach(device_t dev)
 
 	val <<= 1;			/* shift for base 1 indices */
 	for (i = 1; i < 16; i++) {
+		struct ieee80211_channel *c;
+
 		if (!isset((u_int8_t*)&val, i))
 			continue;
-		ic->ic_channels[i].ic_freq =
-			ieee80211_ieee2mhz(i, IEEE80211_CHAN_B);
-		ic->ic_channels[i].ic_flags = IEEE80211_CHAN_B;
+		c = &ic->ic_channels[ic->ic_nchans++];
+		c->ic_freq = ieee80211_ieee2mhz(i, IEEE80211_CHAN_B);
+		c->ic_flags = IEEE80211_CHAN_B;
+		c->ic_ieee = i;
 	}
 
 	/*
@@ -352,14 +373,15 @@ wi_attach(device_t dev)
 	buflen = sizeof(val);
 	if (wi_read_rid(sc, WI_RID_OWN_CHNL, &val, &buflen) == 0) {
 		val = le16toh(val);
-		KASSERT(val < IEEE80211_CHAN_MAX &&
-			ic->ic_channels[val].ic_flags != 0,
-			("wi_attach: invalid own channel %u!", val));
-		ic->ic_ibss_chan = &ic->ic_channels[val];
+		ic->ic_bsschan = ieee80211_find_channel(ic,
+			ieee80211_ieee2mhz(val, IEEE80211_CHAN_B),
+			IEEE80211_CHAN_B);
+		if (ic->ic_bsschan == NULL)
+			ic->ic_bsschan = &ic->ic_channels[0];
 	} else {
 		device_printf(dev,
 			"WI_RID_OWN_CHNL failed, using first channel!\n");
-		ic->ic_ibss_chan = &ic->ic_channels[0];
+		ic->ic_bsschan = &ic->ic_channels[0];
 	}
 
 	/*
@@ -452,7 +474,6 @@ wi_attach(device_t dev)
 				rs->rs_rates[rs->rs_nrates++] = ratebuf[2+i];
 	} else {
 		/* XXX fallback on error? */
-		rs->rs_nrates = 0;
 	}
 
 	buflen = sizeof(val);
@@ -465,7 +486,7 @@ wi_attach(device_t dev)
 	sc->sc_system_scale = 1;
 	sc->sc_cnfauthmode = IEEE80211_AUTH_OPEN;
 	sc->sc_roaming_mode = 1;
-
+	sc->wi_channel = IEEE80211_CHAN_ANYC;
 	sc->sc_portnum = WI_DEFAULT_PORT;
 	sc->sc_authtype = WI_DEFAULT_AUTHTYPE;
 
@@ -486,6 +507,17 @@ wi_attach(device_t dev)
 	sc->sc_key_alloc = ic->ic_crypto.cs_key_alloc;
 	ic->ic_crypto.cs_key_alloc = wi_key_alloc;
 	ic->ic_newstate = wi_newstate;
+	ic->ic_raw_xmit = wi_raw_xmit;
+
+	ic->ic_scan_start = wi_scan_start;
+	ic->ic_scan_curchan = wi_scan_curchan;
+	ic->ic_scan_mindwell = wi_scan_mindwell;
+	ic->ic_scan_end = wi_scan_end;
+	ic->ic_set_channel = wi_set_channel;
+	ic->ic_node_alloc = wi_node_alloc;
+	ic->ic_updateslot = wi_update_slot;
+	ic->ic_reset = wi_reset;
+
 	ieee80211_media_init(ic, wi_media_change, wi_media_status);
 
 #if NBPFILTER > 0
@@ -534,8 +566,8 @@ wi_detach(device_t dev)
 	bpfdetach(ifp);
 #endif
 	ieee80211_ifdetach(&sc->sc_ic);
-	if_free(sc->sc_ifp);
 	bus_teardown_intr(dev, sc->irq, sc->wi_intrhand);
+	if_free(sc->sc_ifp);
 	wi_free(dev);
 	mtx_destroy(&sc->sc_mtx);
 	return (0);
@@ -576,7 +608,7 @@ wi_power(struct wi_softc *sc, int why)
 		break;
 	case PWR_RESUME:
 		if (ifp->if_flags & IFF_UP) {
-			wi_init(ifp);
+			wi_init(sc);
 			(void)wi_intr(sc);
 		}
 		break;
@@ -645,21 +677,20 @@ wi_init(void *arg)
 	struct ifnet *ifp = sc->sc_ifp;
 	struct ieee80211com *ic = &sc->sc_ic;
 	struct wi_joinreq join;
+	struct ieee80211_channel *chan;
 	int i;
 	int error = 0, wasenabled;
-	struct ifaddr *ifa;
-	struct sockaddr_dl *sdl;
 
-	WI_LOCK(sc);
 
-	if (sc->wi_gone) {
-		WI_UNLOCK(sc);
+
+	if (sc->wi_gone) 
 		return;
-	}
 
 	if ((wasenabled = sc->sc_enabled))
 		wi_stop(ifp, 1);
-	wi_reset(sc);
+
+	WI_LOCK(sc);
+	wi_reset(ifp);
 
 	/* common 802.11 configuration */
 	ic->ic_flags &= ~IEEE80211_F_IBSSON;
@@ -681,35 +712,45 @@ wi_init(void *arg)
 		 * HostAP mode the controller will lock up otherwise.
 		 */
 		if (sc->sc_firmware_type == WI_INTERSIL &&
-		    ic->ic_des_esslen == 0) {
-			ic->ic_des_essid[0] = ' ';
-			ic->ic_des_esslen = 1;
+		    ic->ic_des_ssid[0].len == 0) {
+			ic->ic_des_ssid[0].ssid[0] = ' ';
+			ic->ic_des_ssid[0].len = 1;
 		}
 		wi_write_val(sc, WI_RID_PORTTYPE, WI_PORTTYPE_HOSTAP);
 		break;
 	case IEEE80211_M_MONITOR:
-		if (sc->sc_firmware_type == WI_LUCENT)
+		switch (sc->sc_firmware_type) {
+		case WI_LUCENT:
 			wi_write_val(sc, WI_RID_PORTTYPE, WI_PORTTYPE_ADHOC);
+			break;
+		
+		case WI_INTERSIL:
+			wi_write_val(sc, WI_RID_PORTTYPE, WI_PORTTYPE_APSILENT);
+			break;
+		}
+
 		wi_cmd(sc, WI_CMD_DEBUG | (WI_TEST_MONITOR << 8), 0, 0, 0);
 		break;
+	case IEEE80211_M_WDS:
+		/* XXXX */
+		break;
 	}
 
 	/* Intersil interprets this RID as joining ESS even in IBSS mode */
 	if (sc->sc_firmware_type == WI_LUCENT &&
-	    (ic->ic_flags & IEEE80211_F_IBSSON) && ic->ic_des_esslen > 0)
+	    (ic->ic_flags & IEEE80211_F_IBSSON) && ic->ic_des_ssid[0].len > 0)
 		wi_write_val(sc, WI_RID_CREATE_IBSS, 1);
 	else
 		wi_write_val(sc, WI_RID_CREATE_IBSS, 0);
 	wi_write_val(sc, WI_RID_MAX_SLEEP, ic->ic_lintval);
-	wi_write_ssid(sc, WI_RID_DESIRED_SSID, ic->ic_des_essid,
-	    ic->ic_des_esslen);
+	wi_write_ssid(sc, WI_RID_DESIRED_SSID, ic->ic_des_ssid[0].ssid,
+	    ic->ic_des_ssid[0].len);
 	wi_write_val(sc, WI_RID_OWN_CHNL,
-		ieee80211_chan2ieee(ic, ic->ic_ibss_chan));
-	wi_write_ssid(sc, WI_RID_OWN_SSID, ic->ic_des_essid, ic->ic_des_esslen);
+		ieee80211_chan2ieee(ic, ic->ic_bsschan));
+	wi_write_ssid(sc, WI_RID_OWN_SSID, ic->ic_des_ssid[0].ssid,
+		ic->ic_des_ssid[0].len);
 
-	ifa = ifaddr_byindex(ifp->if_index);
-	sdl = (struct sockaddr_dl *) ifa->ifa_addr;
-	IEEE80211_ADDR_COPY(ic->ic_myaddr, LLADDR(sdl));
+	IEEE80211_ADDR_COPY(ic->ic_myaddr, IF_LLADDR(ifp));
 	wi_write_rid(sc, WI_RID_MAC_NODE, ic->ic_myaddr, IEEE80211_ADDR_LEN);
 
 	if (ic->ic_caps & IEEE80211_C_PMGT)
@@ -731,6 +772,7 @@ wi_init(void *arg)
 		wi_write_val(sc, WI_RID_MICROWAVE_OVEN, sc->sc_microwave_oven);
 	wi_write_txrate(sc);
 	wi_write_ssid(sc, WI_RID_NODENAME, sc->sc_nodename, sc->sc_nodelen);
+	wi_write_val(sc, WI_RID_ALT_RETRY_CNT, 0); /* for IEEE80211_BPF_NOACK */
 
 	if (ic->ic_opmode == IEEE80211_M_HOSTAP &&
 	    sc->sc_firmware_type == WI_INTERSIL) {
@@ -793,9 +835,11 @@ wi_init(void *arg)
 	if (ic->ic_opmode == IEEE80211_M_AHDEMO ||
 	    ic->ic_opmode == IEEE80211_M_IBSS ||
 	    ic->ic_opmode == IEEE80211_M_MONITOR ||
-	    ic->ic_opmode == IEEE80211_M_HOSTAP)
-		ieee80211_create_ibss(ic, ic->ic_ibss_chan);
-
+	    ic->ic_opmode == IEEE80211_M_HOSTAP) {
+		chan = (sc->wi_channel == IEEE80211_CHAN_ANYC) ? 
+			ic->ic_curchan : sc->wi_channel;
+		ieee80211_create_ibss(ic, chan);
+	}
 	/* Enable interrupts */
 	CSR_WRITE_2(sc, WI_INT_EN, WI_INTRS);
 
@@ -821,6 +865,8 @@ wi_init(void *arg)
 			wi_write_rid(sc, WI_RID_JOIN_REQ, &join, sizeof(join));
 	}
 
+	callout_reset(&sc->sc_watchdog, hz, wi_watchdog, sc);
+
 	WI_UNLOCK(sc);
 	return;
 out:
@@ -839,11 +885,10 @@ wi_stop(struct ifnet *ifp, int disable)
 	struct wi_softc *sc = ifp->if_softc;
 	struct ieee80211com *ic = &sc->sc_ic;
 
-	WI_LOCK(sc);
+	ieee80211_new_state(ic, IEEE80211_S_INIT, -1);
 
 	DELAY(100000);
-
-	ieee80211_new_state(ic, IEEE80211_S_INIT, -1);
+	WI_LOCK(sc);
 	if (sc->sc_enabled && !sc->wi_gone) {
 		CSR_WRITE_2(sc, WI_INT_EN, 0);
 		wi_cmd(sc, WI_CMD_DISABLE | sc->sc_portnum, 0, 0, 0);
@@ -857,12 +902,12 @@ wi_stop(struct ifnet *ifp, int disable)
 	} else if (sc->wi_gone && disable)	/* gone --> not enabled */
 	    sc->sc_enabled = 0;
 
+	callout_stop(&sc->sc_watchdog);		/* XXX drain */
 	sc->sc_tx_timer = 0;
 	sc->sc_scan_timer = 0;
 	sc->sc_false_syns = 0;
 	sc->sc_naps = 0;
 	ifp->if_drv_flags &= ~(IFF_DRV_OACTIVE | IFF_DRV_RUNNING);
-	ifp->if_timer = 0;
 
 	WI_UNLOCK(sc);
 }
@@ -877,7 +922,7 @@ wi_start_locked(struct ifnet *ifp)
 	struct ether_header *eh;
 	struct mbuf *m0;
 	struct wi_frame frmhdr;
-	int cur, fid, off, error;
+	int cur;
 
 	WI_LOCK_ASSERT(sc);
 
@@ -961,15 +1006,14 @@ wi_start_locked(struct ifnet *ifp)
 
 			k = ieee80211_crypto_encap(ic, ni, m0);
 			if (k == NULL) {
-				if (ni != NULL)
-					ieee80211_free_node(ni);
+				ieee80211_free_node(ni);
 				m_freem(m0);
 				continue;
 			}
 			frmhdr.wi_tx_ctl |= htole16(WI_TXCNTL_NOCRYPT);
 		}
 #if NBPFILTER > 0
-		if (bpf_peers_present(ic->ic_rawbpf)) {
+		if (bpf_peers_present(sc->sc_drvbpf)) {
 			sc->sc_tx_th.wt_rate =
 				ni->ni_rates.rs_rates[ni->ni_txrate];
 			bpf_mtap2(sc->sc_drvbpf,
@@ -982,27 +1026,9 @@ wi_start_locked(struct ifnet *ifp)
 		frmhdr.wi_dat_len = htole16(m0->m_pkthdr.len);
 		if (IFF_DUMPPKTS(ifp))
 			wi_dump_pkt(&frmhdr, NULL, -1);
-		fid = sc->sc_txd[cur].d_fid;
-		off = sizeof(frmhdr);
-		error = wi_write_bap(sc, fid, 0, &frmhdr, sizeof(frmhdr)) != 0
-		     || wi_mwrite_bap(sc, fid, off, m0, m0->m_pkthdr.len) != 0;
-		m_freem(m0);
-		if (ni != NULL)
-			ieee80211_free_node(ni);
-		if (error) {
-			ifp->if_oerrors++;
+		ieee80211_free_node(ni);
+		if (wi_start_tx(ifp, &frmhdr, m0))
 			continue;
-		}
-		sc->sc_txd[cur].d_len = off;
-		if (sc->sc_txcur == cur) {
-			if (wi_cmd(sc, WI_CMD_TX | WI_RECLAIM, fid, 0, 0)) {
-				if_printf(ifp, "xmit failed\n");
-				sc->sc_txd[cur].d_len = 0;
-				continue;
-			}
-			sc->sc_tx_timer = 5;
-			ifp->if_timer = 1;
-		}
 		sc->sc_txnext = cur = (cur + 1) % sc->sc_ntxbuf;
 	}
 }
@@ -1018,9 +1044,127 @@ wi_start(struct ifnet *ifp)
 }
 
 static int
-wi_reset(struct wi_softc *sc)
+wi_start_tx(struct ifnet *ifp, struct wi_frame *frmhdr, struct mbuf *m0)
 {
-	struct ifnet *ifp = sc->sc_ifp;
+	struct wi_softc	*sc = ifp->if_softc;
+	int cur = sc->sc_txnext;
+	int fid, off, error;
+
+	fid = sc->sc_txd[cur].d_fid;
+	off = sizeof(*frmhdr);
+	error = wi_write_bap(sc, fid, 0, frmhdr, sizeof(*frmhdr)) != 0
+	     || wi_mwrite_bap(sc, fid, off, m0, m0->m_pkthdr.len) != 0;
+	m_freem(m0);
+	if (error) {
+		ifp->if_oerrors++;
+		return -1;
+	}
+	sc->sc_txd[cur].d_len = off;
+	if (sc->sc_txcur == cur) {
+		if (wi_cmd(sc, WI_CMD_TX | WI_RECLAIM, fid, 0, 0)) {
+			if_printf(ifp, "xmit failed\n");
+			sc->sc_txd[cur].d_len = 0;
+			return -1;
+		}
+		sc->sc_tx_timer = 5;
+	}
+	return 0;
+}
+
+static int
+wi_raw_xmit(struct ieee80211_node *ni, struct mbuf *m0,
+	    const struct ieee80211_bpf_params *params)
+{
+	struct ieee80211com *ic = ni->ni_ic;
+	struct ifnet *ifp = ic->ic_ifp;
+	struct wi_softc	*sc = ifp->if_softc;
+	struct ieee80211_frame *wh;
+	struct wi_frame frmhdr;
+	int cur;
+	int rc = 0;
+
+	WI_LOCK(sc);
+
+	if (sc->wi_gone) {
+		rc = ENETDOWN;
+		goto out;
+	}
+	if (sc->sc_flags & WI_FLAGS_OUTRANGE) {
+		rc = ENETDOWN;
+		goto out;
+	}
+
+	memset(&frmhdr, 0, sizeof(frmhdr));
+	cur = sc->sc_txnext;
+	if (sc->sc_txd[cur].d_len != 0) {
+		ifp->if_drv_flags |= IFF_DRV_OACTIVE;
+		rc = ENOBUFS;
+		goto out;
+	}
+	m0->m_pkthdr.rcvif = NULL;
+
+	m_copydata(m0, 4, ETHER_ADDR_LEN * 2,
+	    (caddr_t)&frmhdr.wi_ehdr);
+	frmhdr.wi_ehdr.ether_type = 0;
+	wh = mtod(m0, struct ieee80211_frame *);
+			
+#if NBPFILTER > 0
+	if (bpf_peers_present(ic->ic_rawbpf))
+		bpf_mtap(ic->ic_rawbpf, m0);
+#endif
+	frmhdr.wi_tx_ctl = htole16(WI_ENC_TX_802_11|WI_TXCNTL_TX_EX);
+	if (params && (params->ibp_flags & IEEE80211_BPF_NOACK))
+		frmhdr.wi_tx_ctl |= htole16(WI_TXCNTL_ALTRTRY);
+	/* XXX check key for SWCRYPT instead of using operating mode */
+	if ((wh->i_fc[1] & IEEE80211_FC1_WEP) &&
+	    (sc->sc_encryption & HOST_ENCRYPT)) {
+	    	if (!params ||
+		    (params && (params->ibp_flags & IEEE80211_BPF_CRYPTO))) {
+			struct ieee80211_key *k;
+
+			k = ieee80211_crypto_encap(ic, ni, m0);
+			if (k == NULL) {
+				rc = ENOMEM;
+				goto out;
+			}
+			frmhdr.wi_tx_ctl |= htole16(WI_TXCNTL_NOCRYPT);
+		}
+	}
+#if NBPFILTER > 0
+	if (bpf_peers_present(sc->sc_drvbpf)) {
+		sc->sc_tx_th.wt_rate =
+			ni->ni_rates.rs_rates[ni->ni_txrate];
+		bpf_mtap2(sc->sc_drvbpf,
+			&sc->sc_tx_th, sc->sc_tx_th_len, m0);
+	}
+#endif
+	m_copydata(m0, 0, sizeof(struct ieee80211_frame),
+	    (caddr_t)&frmhdr.wi_whdr);
+	m_adj(m0, sizeof(struct ieee80211_frame));
+	frmhdr.wi_dat_len = htole16(m0->m_pkthdr.len);
+	if (IFF_DUMPPKTS(ifp))
+		wi_dump_pkt(&frmhdr, NULL, -1);
+	if (wi_start_tx(ifp, &frmhdr, m0) < 0) {
+		m0 = NULL;
+		rc = EIO;
+		goto out;
+	}
+	m0 = NULL;
+
+	sc->sc_txnext = cur = (cur + 1) % sc->sc_ntxbuf;
+out:
+	WI_UNLOCK(sc);
+
+	if (m0 != NULL)
+		m_freem(m0);
+	ieee80211_free_node(ni);
+	return rc;
+}
+
+static int
+wi_reset(struct ifnet *ifp)
+{
+	struct wi_softc *sc = ifp->if_softc;
 #define WI_INIT_TRIES 3
 	int i;
 	int error = 0;
@@ -1057,11 +1201,11 @@ wi_reset(struct wi_softc *sc)
 }
 
 static void
-wi_watchdog(struct ifnet *ifp)
+wi_watchdog(void *arg)
 {
-	struct wi_softc	*sc = ifp->if_softc;
+	struct wi_softc	*sc = arg;
+	struct ifnet *ifp = sc->sc_ifp;
 
-	ifp->if_timer = 0;
 	if (!sc->sc_enabled)
 		return;
 
@@ -1072,7 +1216,6 @@ wi_watchdog(struct ifnet *ifp)
 			wi_init(ifp->if_softc);
 			return;
 		}
-		ifp->if_timer = 1;
 	}
 
 	if (sc->sc_scan_timer) {
@@ -1081,12 +1224,11 @@ wi_watchdog(struct ifnet *ifp)
 			DPRINTF(("wi_watchdog: inquire scan\n"));
 			wi_cmd(sc, WI_CMD_INQUIRE, WI_INFO_SCAN_RESULTS, 0, 0);
 		}
-		if (sc->sc_scan_timer)
-			ifp->if_timer = 1;
 	}
 
 	/* TODO: rate control */
-	ieee80211_watchdog(&sc->sc_ic);
+
+	callout_reset(&sc->sc_watchdog, hz, wi_watchdog, sc);
 }
 
 static int
@@ -1094,12 +1236,12 @@ wi_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 {
 	struct wi_softc *sc = ifp->if_softc;
 	struct ieee80211com *ic = &sc->sc_ic;
-	struct ifreq *ifr = (struct ifreq *)data;
-	struct ieee80211req *ireq;
-	u_int8_t nodename[IEEE80211_NWID_LEN];
 	int error = 0;
 	struct thread *td = curthread;
+#if 0
+	struct ifreq *ifr = (struct ifreq *)data;
 	struct wi_req wreq;
+#endif
 
 	if (sc->wi_gone)
 		return (ENODEV);
@@ -1143,13 +1285,14 @@ wi_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 		error = wi_write_multi(sc);
 		WI_UNLOCK(sc);
 		break;
+#if 0
 	case SIOCGIFGENERIC:
 		WI_LOCK(sc);
 		error = wi_get_cfg(ifp, cmd, data);
 		WI_UNLOCK(sc);
 		break;
 	case SIOCSIFGENERIC:
-		error = suser(td);
+		error = priv_check(td, PRIV_DRIVER);
 		if (error == 0)
 			error = wi_set_cfg(ifp, cmd, data);
 		break;
@@ -1167,7 +1310,7 @@ wi_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 			error = copyout(&wreq, ifr->ifr_data, sizeof(wreq));
 		break;
 	case SIOCSPRISM2DEBUG:
-		if ((error = suser(td)))
+		if ((error = priv_check(td, PRIV_DRIVER)))
 			return (error);
 		error = copyin(ifr->ifr_data, &wreq, sizeof(wreq));
 		if (error)
@@ -1176,48 +1319,21 @@ wi_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 		error = wi_set_debug(sc, &wreq);
 		WI_UNLOCK(sc);
 		break;
+#endif
 	case SIOCG80211:
-		ireq = (struct ieee80211req *) data;
-		if (ireq->i_type == IEEE80211_IOC_STATIONNAME) {
-			ireq->i_len = sc->sc_nodelen + 1;
-			error = copyout(sc->sc_nodename, ireq->i_data,
-					ireq->i_len);
-			break;
-		}
-		goto ioctl_common;
+		error = wi_ioctl_get(ifp, cmd, data);
+		break;
 	case SIOCS80211:
-		ireq = (struct ieee80211req *) data;
-		if (ireq->i_type == IEEE80211_IOC_STATIONNAME) {
-			error = suser(td);
-			if (error)
-				break;
-			if (ireq->i_val != 0 ||
-			    ireq->i_len > IEEE80211_NWID_LEN) {
-				error = EINVAL;
-				break;
-			}
-			memset(nodename, 0, IEEE80211_NWID_LEN);
-			error = copyin(ireq->i_data, nodename, ireq->i_len);
-			if (error)
-				break;
-			WI_LOCK(sc);
-			if (sc->sc_enabled) {
-				error = wi_write_ssid(sc, WI_RID_NODENAME,
-					nodename, ireq->i_len);
-			}
-			if (error == 0) {
-				memcpy(sc->sc_nodename, nodename,
-					IEEE80211_NWID_LEN);
-				sc->sc_nodelen = ireq->i_len;
-			}
-			WI_UNLOCK(sc);
+		error = priv_check(td, PRIV_NET80211_MANAGE);
+		if (error)
+			break;
+		error = wi_ioctl_set(ifp, cmd, data);
+
+
 			break;
-		}
-		goto ioctl_common;
 	default:
-	ioctl_common:
-		WI_LOCK(sc);
 		error = ieee80211_ioctl(ic, cmd, data);
+		WI_LOCK(sc);
 		if (error == ENETRESET) {
 			if (sc->sc_enabled)
 				wi_init(sc);	/* XXX no error return */
@@ -1229,6 +1345,101 @@ wi_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 	return (error);
 }
 
+static int
+wi_ioctl_get(struct ifnet *ifp, u_long cmd, caddr_t data)
+{
+	int                 error;
+	struct wi_softc     *sc;
+	struct ieee80211req *ireq;
+	struct ieee80211com *ic;
+
+
+	sc = ifp->if_softc;
+	ic = &sc->sc_ic;
+	ireq = (struct ieee80211req *) data;
+
+	switch (ireq->i_type) {
+	case IEEE80211_IOC_STATIONNAME:
+		ireq->i_len = sc->sc_nodelen + 1;
+		error = copyout(sc->sc_nodename, ireq->i_data,
+				ireq->i_len);
+		break;
+	default:
+		error = ieee80211_ioctl(ic, cmd, data);
+		WI_LOCK(sc);
+		if (error == ENETRESET) {
+			if (sc->sc_enabled)
+				wi_init(sc);	/* XXX no error return */
+			error = 0;
+		}
+		WI_UNLOCK(sc);
+
+		break;
+	}
+	
+	return (error);
+}
+
+static int
+wi_ioctl_set(struct ifnet *ifp, u_long cmd, caddr_t data)
+{
+	int                 error;
+	struct wi_softc     *sc;
+	struct ieee80211req *ireq;
+	u_int8_t nodename[IEEE80211_NWID_LEN];
+		
+	sc = ifp->if_softc;
+	ireq = (struct ieee80211req *) data;
+	switch (ireq->i_type) {
+	case IEEE80211_IOC_STATIONNAME:
+		if (ireq->i_val != 0 ||
+		    ireq->i_len > IEEE80211_NWID_LEN) {
+			error = EINVAL;
+			break;
+		}
+		memset(nodename, 0, IEEE80211_NWID_LEN);
+		error = copyin(ireq->i_data, nodename, ireq->i_len);
+		if (error)
+			break;
+		WI_LOCK(sc);
+		if (sc->sc_enabled) {
+			error = wi_write_ssid(sc, WI_RID_NODENAME,
+					      nodename, ireq->i_len);
+		}
+		if (error == 0) {
+			memcpy(sc->sc_nodename, nodename,
+			       IEEE80211_NWID_LEN);
+			sc->sc_nodelen = ireq->i_len;
+		}
+		WI_UNLOCK(sc);
+		
+		break;
+	default:
+		error = ieee80211_ioctl(&sc->sc_ic, cmd, data);
+		WI_LOCK(sc);
+		if (error == ENETRESET) {
+			if (sc->sc_enabled)
+				wi_init(sc);	/* XXX no error return */
+			error = 0;
+		}
+		WI_UNLOCK(sc);
+		break;
+	}
+
+	return (error);
+}
+
+static struct ieee80211_node *
+wi_node_alloc(struct ieee80211_node_table *nt)
+{
+	struct wi_node *rn;
+
+	rn = malloc(sizeof (struct wi_node), M_80211_NODE,
+	    M_NOWAIT | M_ZERO);
+
+	return (rn != NULL) ? &rn->ni : NULL;
+}
+
 static int
 wi_media_change(struct ifnet *ifp)
 {
@@ -1301,6 +1512,9 @@ wi_media_status(struct ifnet *ifp, struct ifmediareq *imr)
 	case IEEE80211_M_MONITOR:
 		imr->ifm_active |= IFM_IEEE80211_MONITOR;
 		break;
+	case IEEE80211_M_WDS:
+		/* XXXX */
+		break;
 	}
 }
 
@@ -1327,6 +1541,7 @@ wi_sync_bssid(struct wi_softc *sc, u_int8_t new_bssid[IEEE80211_ADDR_LEN])
 		return;
 
 	sc->sc_false_syns = MAX(0, sc->sc_false_syns - 1);
+#if 0
 	/*
 	 * XXX hack; we should create a new node with the new bssid
 	 * and replace the existing ic_bss with it but since we don't
@@ -1335,6 +1550,7 @@ wi_sync_bssid(struct wi_softc *sc, u_int8_t new_bssid[IEEE80211_ADDR_LEN])
 	 * called and it will overwrite the node state.
 	 */
 	ieee80211_sta_join(ic, ieee80211_ref_node(ni));
+#endif
 }
 
 static void
@@ -1514,17 +1730,6 @@ wi_rx_intr(struct wi_softc *sc)
 
 	CSR_WRITE_2(sc, WI_EVENT_ACK, WI_EV_RX);
 
-	wh = mtod(m, struct ieee80211_frame *);
-	if (wh->i_fc[1] & IEEE80211_FC1_WEP) {
-		/*
-		 * WEP is decrypted by hardware and the IV
-		 * is stripped.  Clear WEP bit so we don't
-		 * try to process it in ieee80211_input.
-		 * XXX fix for TKIP, et. al.
-		 */
-		wh->i_fc[1] &= ~IEEE80211_FC1_WEP;
-	}
-
 #if NBPFILTER > 0
 	if (bpf_peers_present(sc->sc_drvbpf)) {
 		/* XXX replace divide by table */
@@ -1539,6 +1744,16 @@ wi_rx_intr(struct wi_softc *sc)
 			&sc->sc_rx_th, sc->sc_rx_th_len, m);
 	}
 #endif
+	wh = mtod(m, struct ieee80211_frame *);
+	if (wh->i_fc[1] & IEEE80211_FC1_WEP) {
+		/*
+		 * WEP is decrypted by hardware and the IV
+		 * is stripped.  Clear WEP bit so we don't
+		 * try to process it in ieee80211_input.
+		 * XXX fix for TKIP, et. al.
+		 */
+		wh->i_fc[1] &= ~IEEE80211_FC1_WEP;
+	}
 
 	/* synchronize driver's BSSID with firmware's BSSID */
 	dir = wh->i_fc[1] & IEEE80211_FC1_DIR_MASK;
@@ -1555,7 +1770,7 @@ wi_rx_intr(struct wi_softc *sc)
 	/*
 	 * Send frame up for processing.
 	 */
-	ieee80211_input(ic, m, ni, rssi, rstamp);
+	ieee80211_input(ic, m, ni, rssi, -95/*XXXXwi_rx_silence?*/, rstamp);
 	/*
 	 * The frame may have caused the node to be marked for
 	 * reclamation (e.g. in response to a DEAUTH message)
@@ -1640,7 +1855,6 @@ wi_tx_intr(struct wi_softc *sc)
 			sc->sc_txd[cur].d_len = 0;
 		} else {
 			sc->sc_tx_timer = 5;
-			ifp->if_timer = 1;
 		}
 	}
 }
@@ -1716,6 +1930,7 @@ wi_info_intr(struct wi_softc *sc)
 	case WI_INFO_SCAN_RESULTS:
 	case WI_INFO_HOST_SCAN_RESULTS:
 		wi_scan_result(sc, fid, le16toh(ltbuf[0]));
+		ieee80211_scan_done(ic);
 		break;
 
 	default:
@@ -1848,6 +2063,7 @@ wi_write_ssid(struct wi_softc *sc, int rid, u_int8_t *buf, int buflen)
 	return wi_write_rid(sc, rid, &ssid, sizeof(ssid));
 }
 
+#if 0
 static int
 wi_get_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 {
@@ -1882,7 +2098,7 @@ wi_get_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 	case WI_RID_TX_CRYPT_KEY:
 	case WI_RID_DEFLT_CRYPT_KEYS:
 	case WI_RID_TX_RATE:
-		return ieee80211_cfgget(ic, cmd, data);
+		return ieee80211_ioctl(ic, cmd, data);
 
 	case WI_RID_MICROWAVE_OVEN:
 		if (sc->sc_enabled && (sc->sc_flags & WI_FLAGS_HAS_MOR)) {
@@ -1936,7 +2152,7 @@ wi_get_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 
 	case WI_RID_READ_APS:
 		if (ic->ic_opmode == IEEE80211_M_HOSTAP)
-			return ieee80211_cfgget(ic, cmd, data);
+			return ieee80211_ioctl(ic, cmd, data);
 		if (sc->sc_scan_timer > 0) {
 			error = EINPROGRESS;
 			break;
@@ -1973,11 +2189,11 @@ wi_get_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 		break;
 
 	case WI_RID_READ_CACHE:
-		return ieee80211_cfgget(ic, cmd, data);
+		return ieee80211_ioctl(ic, cmd, data);
 
 	case WI_RID_SCAN_RES:		/* compatibility interface */
 		if (ic->ic_opmode == IEEE80211_M_HOSTAP)
-			return ieee80211_cfgget(ic, cmd, data);
+			return ieee80211_ioctl(ic, cmd, data);
 		if (sc->sc_scan_timer > 0) {
 			error = EINPROGRESS;
 			break;
@@ -2059,7 +2275,7 @@ wi_get_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 			    sc->sc_nodelen);
 			break;
 		default:
-			return ieee80211_cfgget(ic, cmd, data);
+			return ieee80211_ioctl(ic, cmd, data);
 		}
 		break;
 	}
@@ -2179,7 +2395,7 @@ wi_set_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 				WI_UNLOCK(sc);
 				return EINVAL;
 			}
-			ic->ic_fixed_rate = i;
+			ic->ic_fixed_rate = rs->rs_rates[i] & IEEE80211_RATE_VAL;
 		}
 		if (sc->sc_enabled)
 			error = wi_write_txrate(sc);
@@ -2237,9 +2453,10 @@ wi_set_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 			break;
 		}
 		WI_LOCK(sc);
-		memset(ic->ic_des_essid, 0, IEEE80211_NWID_LEN);
-		ic->ic_des_esslen = le16toh(wreq.wi_val[0]) * 2;
-		memcpy(ic->ic_des_essid, &wreq.wi_val[1], ic->ic_des_esslen);
+		memset(ic->ic_des_ssid[0].ssid, 0, IEEE80211_NWID_LEN);
+		ic->ic_des_ssid[0].len = le16toh(wreq.wi_val[0]) * 2;
+		memcpy(ic->ic_des_ssid[0].ssid, &wreq.wi_val[1],
+			ic->ic_des_ssid[0].len);
 		if (sc->sc_enabled)
 			wi_init(sc);	/* XXX no error return */
 		WI_UNLOCK(sc);
@@ -2251,8 +2468,8 @@ wi_set_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 			error = wi_write_rid(sc, wreq.wi_type, wreq.wi_val,
 			    len);
 		if (error == 0) {
-			/* XXX ieee80211_cfgset does a copyin */
-			error = ieee80211_cfgset(ic, cmd, data);
+			/* XXX ieee80211_ioctl does a copyin */
+			error = ieee80211_ioctl(ic, cmd, data);
 			if (error == ENETRESET) {
 				if (sc->sc_enabled)
 					wi_init(sc);
@@ -2264,6 +2481,7 @@ wi_set_cfg(struct ifnet *ifp, u_long cmd, caddr_t data)
 	}
 	return error;
 }
+#endif
 
 static int
 wi_write_txrate(struct wi_softc *sc)
@@ -2275,8 +2493,7 @@ wi_write_txrate(struct wi_softc *sc)
 	if (ic->ic_fixed_rate == IEEE80211_FIXED_RATE_NONE)
 		rate = 0;	/* auto */
 	else
-		rate = (ic->ic_sup_rates[IEEE80211_MODE_11B].rs_rates[ic->ic_fixed_rate] &
-		    IEEE80211_RATE_VAL) / 2;
+		rate = ic->ic_fixed_rate / 2;
 
 	/* rate: 0, 1, 2, 5, 11 */
 
@@ -2742,6 +2959,8 @@ wi_newstate(struct ieee80211com *ic, enum ieee80211_state nstate, int arg)
 		return (*sc->sc_newstate)(ic, nstate, arg);
 
 	case IEEE80211_S_SCAN:
+		return (*sc->sc_newstate)(ic, nstate, arg);
+
 	case IEEE80211_S_AUTH:
 	case IEEE80211_S_ASSOC:
 		ic->ic_state = nstate;	/* NB: skip normal ieee80211 handling */
@@ -2756,10 +2975,12 @@ wi_newstate(struct ieee80211com *ic, enum ieee80211_state nstate, int arg)
 		IEEE80211_ADDR_COPY(ni->ni_macaddr, ni->ni_bssid);
 		buflen = sizeof(val);
 		wi_read_rid(sc, WI_RID_CURRENT_CHAN, &val, &buflen);
-		/* XXX validate channel */
-		ni->ni_chan = &ic->ic_channels[le16toh(val)];
-		ic->ic_curchan = ni->ni_chan;
-		ic->ic_ibss_chan = ni->ni_chan;
+		ni->ni_chan = ieee80211_find_channel(ic,
+		    ieee80211_ieee2mhz(val, IEEE80211_CHAN_B),
+		    IEEE80211_CHAN_B);
+		if (ni->ni_chan == NULL)
+			ni->ni_chan = &ic->ic_channels[0];
+		ic->ic_curchan = ic->ic_bsschan = ni->ni_chan;
 #if NBPFILTER > 0
 		sc->sc_tx_th.wt_chan_freq = sc->sc_rx_th.wr_chan_freq =
 			htole16(ni->ni_chan->ic_freq);
@@ -2788,6 +3009,8 @@ wi_newstate(struct ieee80211com *ic, enum ieee80211_state nstate, int arg)
 			memcpy(ni->ni_essid, ssid.wi_ssid, ni->ni_esslen);
 		}
 		return (*sc->sc_newstate)(ic, nstate, arg);
+	default:
+		break;
 	}
 	return 0;
 }
@@ -2820,7 +3043,6 @@ wi_scan_ap(struct wi_softc *sc, u_int16_t chanmask, u_int16_t txrate)
 	}
 	if (error == 0) {
 		sc->sc_scan_timer = WI_SCAN_WAIT;
-		sc->sc_ifp->if_timer = 1;
 		DPRINTF(("wi_scan_ap: start scanning, "
 			"chamask 0x%x txrate 0x%x\n", chanmask, txrate));
 	}
@@ -2835,6 +3057,15 @@ wi_scan_result(struct wi_softc *sc, int fid, int cnt)
 	struct wi_scan_header ws_hdr;	/* Prism2 header */
 	struct wi_scan_data_p2 ws_dat;	/* Prism2 scantable*/
 	struct wi_apinfo *ap;
+	struct ieee80211_scanparams sp;
+	struct ieee80211_frame wh;
+	static long rstamp;
+	struct ieee80211com *ic;
+	uint8_t ssid[2+IEEE80211_NWID_LEN];
+
+	ic = &sc->sc_ic;
+	rstamp++;
+	memset(&sp, 0, sizeof(sp));
 
 	off = sizeof(u_int16_t) * 2;
 	memset(&ws_hdr, 0, sizeof(ws_hdr));
@@ -2864,25 +3095,52 @@ wi_scan_result(struct wi_softc *sc, int fid, int cnt)
 	/* Read Data */
 	ap = sc->sc_aps;
 	memset(&ws_dat, 0, sizeof(ws_dat));
+
 	for (i = 0; i < naps; i++, ap++) {
+		uint8_t rates[2 + IEEE80211_RATE_MAXSIZE];
+		uint16_t *bssid;
 		wi_read_bap(sc, fid, off, &ws_dat,
 		    (sizeof(ws_dat) < szbuf ? sizeof(ws_dat) : szbuf));
 		DPRINTF2(("wi_scan_result: #%d: off %d bssid %s\n", i, off,
 		    ether_sprintf(ws_dat.wi_bssid)));
+
 		off += szbuf;
-		ap->scanreason = le16toh(ws_hdr.wi_reason);
+		ap->scanreason = le16toh(ws_hdr.wi_reason);		
 		memcpy(ap->bssid, ws_dat.wi_bssid, sizeof(ap->bssid));
+		
+		bssid = (uint16_t *)&ap->bssid;
+		if (bssid[0] == 0 && bssid[1] == 0 && bssid[2] == 0)
+			break;
+
+		memcpy(wh.i_addr2, ws_dat.wi_bssid, sizeof(ap->bssid));
+		memcpy(wh.i_addr3, ws_dat.wi_bssid, sizeof(ap->bssid));
 		ap->channel = le16toh(ws_dat.wi_chid);
 		ap->signal  = le16toh(ws_dat.wi_signal);
 		ap->noise   = le16toh(ws_dat.wi_noise);
 		ap->quality = ap->signal - ap->noise;
-		ap->capinfo = le16toh(ws_dat.wi_capinfo);
-		ap->interval = le16toh(ws_dat.wi_interval);
-		ap->rate    = le16toh(ws_dat.wi_rate);
+		sp.capinfo = ap->capinfo = le16toh(ws_dat.wi_capinfo);
+		sp.bintval = ap->interval = le16toh(ws_dat.wi_interval);
+		ap->rate = le16toh(ws_dat.wi_rate);
+		rates[1] = 1;
+		rates[2] = (uint8_t)ap->rate / 5;
 		ap->namelen = le16toh(ws_dat.wi_namelen);
 		if (ap->namelen > sizeof(ap->name))
 			ap->namelen = sizeof(ap->name);
 		memcpy(ap->name, ws_dat.wi_name, ap->namelen);
+		sp.ssid = (uint8_t *)&ssid[0];
+		memcpy(sp.ssid + 2, ap->name, ap->namelen);
+		sp.ssid[1] = ap->namelen;
+		sp.bchan = ap->channel;
+		sp.curchan = ieee80211_find_channel(ic,
+			ieee80211_ieee2mhz(ap->channel, IEEE80211_CHAN_B),
+			IEEE80211_CHAN_B);
+		if (sp.curchan == NULL)
+			sp.curchan = &ic->ic_channels[0];
+		sp.rates = &rates[0];
+		sp.tstamp = (uint8_t *)&rstamp;
+		DPRINTF(("calling add_scan, bssid %s chan %d signal %d\n",
+		    ether_sprintf(ws_dat.wi_bssid), ap->channel, ap->signal));
+		ieee80211_add_scan(ic, &sp, &wh, 0, ap->signal, ap->noise, rstamp);
 	}
 done:
 	/* Done scanning */
@@ -2894,8 +3152,11 @@ done:
 static void
 wi_dump_pkt(struct wi_frame *wh, struct ieee80211_node *ni, int rssi)
 {
-	ieee80211_dump_pkt((u_int8_t *) &wh->wi_whdr, sizeof(wh->wi_whdr),
-	    ni ? ni->ni_rates.rs_rates[ni->ni_txrate] & IEEE80211_RATE_VAL : -1, rssi);
+	if (ni != NULL)
+		ieee80211_dump_pkt(ni->ni_ic,
+		    (u_int8_t *) &wh->wi_whdr, sizeof(wh->wi_whdr),
+		    ni->ni_rates.rs_rates[ni->ni_txrate] & IEEE80211_RATE_VAL,
+		    rssi);
 	printf(" status 0x%x rx_tstamp1 %u rx_tstamp0 0x%u rx_silence %u\n",
 		le16toh(wh->wi_status), le16toh(wh->wi_rx_tstamp1),
 		le16toh(wh->wi_rx_tstamp0), wh->wi_rx_silence);
@@ -2980,6 +3241,7 @@ wi_free(device_t dev)
 	return;
 }
 
+#if 0
 static int
 wi_get_debug(struct wi_softc *sc, struct wi_req *wreq)
 {
@@ -3118,6 +3380,7 @@ wi_set_debug(struct wi_softc *sc, struct wi_req *wreq)
 
 	return (error);
 }
+#endif
 
 /*
  * Special routines to download firmware for Symbol CF card.
@@ -3255,3 +3518,75 @@ wi_symbol_set_hcr(struct wi_softc *sc, int mode)
 	tsleep(sc, PWAIT, "wiinit", 1);
 	return 0;
 }
+
+/*
+ * This function can be called by ieee80211_set_shortslottime(). Refer to
+ * IEEE Std 802.11-1999 pp. 85 to know how these values are computed.
+ */
+static void
+wi_update_slot(struct ifnet *ifp)
+{
+	DPRINTF(("wi update slot unimplemented\n"));
+}
+
+static void
+wi_set_channel(struct ieee80211com *ic)
+{
+	struct ifnet *ifp = ic->ic_ifp;
+	struct wi_softc *sc = ifp->if_softc;
+
+	WI_LOCK(sc);
+	if (sc->sc_enabled && !(sc->sc_flags & WI_FLAGS_SCANNING)) {
+		DPRINTF(("wi_set_channel: %d (%dMHz)\n",
+		    ieee80211_chan2ieee(ic, ic->ic_curchan),
+		    ic->ic_curchan->ic_freq));
+
+		sc->wi_channel = ic->ic_curchan;
+		wi_write_val(sc, WI_RID_OWN_CHNL,
+		    ieee80211_chan2ieee(ic, ic->ic_curchan));
+
+		if (ic->ic_opmode == IEEE80211_M_HOSTAP &&
+		    sc->sc_firmware_type == WI_INTERSIL) {
+			/* XXX: some cards need to be re-enabled */
+			wi_cmd(sc, WI_CMD_DISABLE | WI_PORT0, 0, 0, 0);
+			wi_cmd(sc, WI_CMD_ENABLE | WI_PORT0, 0, 0, 0);
+		}
+	}
+	WI_UNLOCK(sc);
+}
+
+static void
+wi_scan_start(struct ieee80211com *ic)
+{
+	struct ifnet *ifp = ic->ic_ifp;
+	struct wi_softc *sc = ifp->if_softc;
+
+	WI_LOCK(sc);
+	sc->sc_flags |= WI_FLAGS_SCANNING;
+	wi_scan_ap(sc, 0x3fff, 0x000f);
+	WI_UNLOCK(sc);
+
+}
+
+static void
+wi_scan_curchan(struct ieee80211com *ic, unsigned long maxdwell)
+{
+	/* The firmware is not capable of scanning a single channel */
+}
+
+static void
+wi_scan_mindwell(struct ieee80211com *ic)
+{
+	/* NB: don't try to abort scan; wait for firmware to finish */
+}
+
+static void
+wi_scan_end(struct ieee80211com *ic)
+{
+	struct ifnet *ifp = ic->ic_ifp;
+	struct wi_softc *sc = ifp->if_softc;
+
+	WI_LOCK(sc);
+	sc->sc_flags &= ~WI_FLAGS_SCANNING;
+	WI_UNLOCK(sc);
+}
