@@ -25,10 +25,9 @@
  *
  * From: FreeBSD: src/sys/miscfs/kernfs/kernfs_vfsops.c 1.36
  *
- * $FreeBSD: src/sys/fs/devfs/devfs_devs.c,v 1.36.2.6 2006/10/30 15:40:53 kib Exp $
+ * $FreeBSD: src/sys/fs/devfs/devfs_devs.c,v 1.50 2007/04/23 13:36:52 rwatson Exp $
  */
 
-#include "opt_devfs.h"
 #include "opt_mac.h"
 
 #include <sys/param.h>
@@ -38,7 +37,6 @@
 #include <sys/kernel.h>
 #include <sys/limits.h>
 #include <sys/lock.h>
-#include <sys/mac.h>
 #include <sys/malloc.h>
 #include <sys/proc.h>
 #include <sys/sx.h>
@@ -50,6 +48,8 @@
 #include <fs/devfs/devfs.h>
 #include <fs/devfs/devfs_int.h>
 
+#include <security/mac/mac_framework.h>
+
 /*
  * The one true (but secret) list of active devices in the system.
  * Locked by dev_lock()/devmtx
@@ -182,7 +182,7 @@ devfs_newdirent(char *name, int namelen)
 	de->de_links = 1;
 	de->de_holdcnt = 1;
 #ifdef MAC
-	mac_init_devfsdirent(de);
+	mac_init_devfs(de);
 #endif
 	return (de);
 }
@@ -274,7 +274,7 @@ devfs_delete(struct devfs_mount *dm, struct devfs_dirent *de, int vp_locked)
 		de->de_symlink = NULL;
 	}
 #ifdef MAC
-	mac_destroy_devfsdirent(de);
+	mac_destroy_devfs(de);
 #endif
 	if (de->de_inode > DEVFS_ROOTINO) {
 		free_unr(devfs_inos, de->de_inode);
