@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/fs/hpfs/hpfs_vnops.c,v 1.66.2.1 2006/03/12 21:50:01 scottl Exp $
+ * $FreeBSD: src/sys/fs/hpfs/hpfs_vnops.c,v 1.70 2007/02/15 22:08:32 pjd Exp $
  */
 
 #include <sys/param.h>
@@ -78,6 +78,7 @@ static vop_remove_t	hpfs_remove;
 static vop_bmap_t	hpfs_bmap;
 static vop_fsync_t	hpfs_fsync;
 static vop_pathconf_t	hpfs_pathconf;
+static vop_vptofh_t	hpfs_vptofh;
 
 static int
 hpfs_fsync(ap)
@@ -501,11 +502,12 @@ hpfs_setattr(ap)
 	if (vap->va_atime.tv_sec != VNOVAL || vap->va_mtime.tv_sec != VNOVAL) {
 		if (vp->v_mount->mnt_flag & MNT_RDONLY)
 			return (EROFS);
-		if (cred->cr_uid != hp->h_uid &&
-		    (error = suser_cred(cred, SUSER_ALLOWJAIL)) &&
-		    ((vap->va_vaflags & VA_UTIMES_NULL) == 0 ||
-		    (error = VOP_ACCESS(vp, VWRITE, cred, td))))
-			return (error);
+		if (vap->va_vaflags & VA_UTIMES_NULL) {
+			error = VOP_ACCESS(vp, VADMIN, cred, td);
+			if (error)
+				error = VOP_ACCESS(vp, VWRITE, cred, td);
+		} else
+			error = VOP_ACCESS(vp, VADMIN, cred, td);
 		if (vap->va_atime.tv_sec != VNOVAL)
 			hp->h_atime = vap->va_atime.tv_sec;
 		if (vap->va_mtime.tv_sec != VNOVAL)
@@ -729,7 +731,7 @@ hpfs_open(ap)
 		struct thread *a_td;
 	} */ *ap;
 {
-#if HPFS_DEBUG
+#ifdef HPFS_DEBUG
 	register struct vnode *vp = ap->a_vp;
 	register struct hpfsnode *hp = VTOHP(vp);
 
@@ -758,7 +760,7 @@ hpfs_close(ap)
 		struct thread *a_td;
 	} */ *ap;
 {
-#if HPFS_DEBUG
+#ifdef HPFS_DEBUG
 	register struct vnode *vp = ap->a_vp;
 	register struct hpfsnode *hp = VTOHP(vp);
 
@@ -1207,6 +1209,24 @@ hpfs_pathconf(ap)
 	/* NOTREACHED */
 }
 
+int
+hpfs_vptofh(ap)
+	struct vop_vptofh_args /* {
+		struct vnode *a_vp;
+		struct fid *a_fhp;
+	} */ *ap;
+{
+	register struct hpfsnode *hpp;
+	register struct hpfid *hpfhp;
+
+	hpp = VTOHP(ap->a_vp);
+	hpfhp = (struct hpfid *)ap->a_fhp;
+	hpfhp->hpfid_len = sizeof(struct hpfid);
+	hpfhp->hpfid_ino = hpp->h_no;
+	/* hpfhp->hpfid_gen = hpp->h_gen; */
+	return (0);
+}
+
 
 /*
  * Global vfs data structures
@@ -1234,4 +1254,5 @@ struct vop_vector hpfs_vnodeops = {
 	.vop_setattr =		hpfs_setattr,
 	.vop_strategy =		hpfs_strategy,
 	.vop_write =		hpfs_write,
+	.vop_vptofh =		hpfs_vptofh,
 };
