@@ -25,7 +25,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/fs/ntfs/ntfs_vfsops.c,v 1.78.2.4 2006/10/09 19:47:14 tegge Exp $
+ * $FreeBSD: src/sys/fs/ntfs/ntfs_vfsops.c,v 1.88 2007/09/21 23:50:15 rodrigc Exp $
  */
 
 
@@ -60,10 +60,10 @@
 #include <fs/ntfs/ntfs_ihash.h>
 #include <fs/ntfs/ntfsmount.h>
 
-static MALLOC_DEFINE(M_NTFSMNT, "NTFS mount", "NTFS mount structure");
-MALLOC_DEFINE(M_NTFSNTNODE,"NTFS ntnode",  "NTFS ntnode information");
-MALLOC_DEFINE(M_NTFSFNODE,"NTFS fnode",  "NTFS fnode information");
-MALLOC_DEFINE(M_NTFSDIR,"NTFS dir",  "NTFS dir buffer");
+static MALLOC_DEFINE(M_NTFSMNT, "ntfs_mount", "NTFS mount structure");
+MALLOC_DEFINE(M_NTFSNTNODE,"ntfs_ntnode",  "NTFS ntnode information");
+MALLOC_DEFINE(M_NTFSFNODE,"ntfs_fnode",  "NTFS fnode information");
+MALLOC_DEFINE(M_NTFSDIR,"ntfs_dir",  "NTFS dir buffer");
 
 struct sockaddr;
 
@@ -80,7 +80,6 @@ static vfs_mount_t      ntfs_mount;
 static vfs_root_t       ntfs_root;
 static vfs_statfs_t     ntfs_statfs;
 static vfs_unmount_t    ntfs_unmount;
-static vfs_vptofh_t     ntfs_vptofh;
 
 static b_strategy_t     ntfs_bufstrategy;
 
@@ -262,12 +261,26 @@ ntfs_mountfs(devvp, mp, td)
 	int error, ronly, i, v;
 	struct vnode *vp;
 	struct g_consumer *cp;
+	struct g_provider *pp;
 	char *cs_ntfs, *cs_local;
 
 	ronly = (mp->mnt_flag & MNT_RDONLY) != 0;
 	DROP_GIANT();
 	g_topology_lock();
-	error = g_vfs_open(devvp, &cp, "ntfs", ronly ? 0 : 1);
+
+	/*
+	 * XXX: Do not allow more than one consumer to open a device
+	 *      associated with a particular GEOM provider.
+	 *      This disables multiple read-only mounts of a device,
+	 *      but it gets rid of panics in vget() when you try to
+	 *      mount the same device more than once.
+	 */
+	pp = g_dev_getprovider(devvp->v_rdev);
+ 	if ((pp != NULL) && ((pp->acr | pp->acw | pp->ace ) != 0)) 
+		error = EPERM;
+	else 
+		error = g_vfs_open(devvp, &cp, "ntfs", ronly ? 0 : 1);
+
 	g_topology_unlock();
 	PICKUP_GIANT();
 	VOP_UNLOCK(devvp, 0, td);
@@ -292,7 +305,7 @@ ntfs_mountfs(devvp, mp, td)
 	brelse( bp );
 	bp = NULL;
 
-	if (strncmp(ntmp->ntm_bootfile.bf_sysid, NTFS_BBID, NTFS_BBIDLEN)) {
+	if (strncmp((const char *)ntmp->ntm_bootfile.bf_sysid, NTFS_BBID, NTFS_BBIDLEN)) {
 		error = EINVAL;
 		dprintf(("ntfs_mountfs: invalid boot block\n"));
 		goto out;
@@ -327,10 +340,10 @@ ntfs_mountfs(devvp, mp, td)
 	ntmp->ntm_bo = &devvp->v_bufobj;
 
 	cs_local = vfs_getopts(mp->mnt_optnew, "cs_local", &error);
-	if (error)
+	if (error && error != ENOENT)
 		goto out;
 	cs_ntfs = vfs_getopts(mp->mnt_optnew, "cs_ntfs", &error);
-	if (error)
+	if (error && error != ENOENT)
 		goto out;
 	/* Copy in the 8-bit to Unicode conversion table */
 	/* Initialize Unicode to 8-bit table from 8toU table */
@@ -618,25 +631,7 @@ ntfs_fhtovp(
 	/* XXX as unlink/rmdir/mkdir/creat are not currently possible
 	 * with NTFS, we don't need to check anything else for now */
 	*vpp = nvp;
-	vnode_create_vobject_off(nvp, VTOF(nvp)->f_size, curthread);
-	return (0);
-}
-
-static int
-ntfs_vptofh(
-	struct vnode *vp,
-	struct fid *fhp)
-{
-	register struct ntnode *ntp;
-	register struct ntfid *ntfhp;
-
-	ddprintf(("ntfs_fhtovp(): %p\n", vp));
-
-	ntp = VTONT(vp);
-	ntfhp = (struct ntfid *)fhp;
-	ntfhp->ntfid_len = sizeof(struct ntfid);
-	ntfhp->ntfid_ino = ntp->i_number;
-	/* ntfhp->ntfid_gen = ntp->i_gen; */
+	vnode_create_vobject(nvp, VTOF(nvp)->f_size, curthread);
 	return (0);
 }
 
@@ -725,6 +720,13 @@ ntfs_vgetex(
 		ntfs_ntput(ip);
 		return (error);
 	}
+	/* XXX: Too early for mpsafe fs, lacks vnode lock */
+	error = insmntque(vp, ntmp->ntm_mountp);
+	if (error) {
+		ntfs_frele(fp);
+		ntfs_ntput(ip);
+		return (error);
+	}
 	dprintf(("ntfs_vget: vnode: %p for ntnode: %d\n", vp,ino));
 
 	fp->f_vp = vp;
@@ -787,7 +789,6 @@ static struct vfsops ntfs_vfsops = {
 	.vfs_uninit =	ntfs_uninit,
 	.vfs_unmount =	ntfs_unmount,
 	.vfs_vget =	ntfs_vget,
-	.vfs_vptofh =	ntfs_vptofh,
 };
 VFS_SET(ntfs_vfsops, ntfs, 0);
 MODULE_VERSION(ntfs, 1);
