@@ -29,7 +29,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/fs/nwfs/nwfs_io.c,v 1.41.2.1 2007/02/12 19:08:29 jhb Exp $
+ * $FreeBSD: src/sys/fs/nwfs/nwfs_io.c,v 1.46 2007/06/01 14:33:10 kib Exp $
  *
  */
 #include <sys/param.h>
@@ -226,7 +226,7 @@ nwfs_writevnode(vp, uiop, cred, ioflag)
 		/* We can relay only on local information about file size,
 		 * because until file is closed NetWare will not return
 		 * the correct size. */
-#if notyet
+#ifdef notyet
 			nwfs_attr_cacheremove(vp);
 			error = VOP_GETATTR(vp, &vattr, cred, td);
 			if (error) return (error);
@@ -483,7 +483,7 @@ nwfs_getpages(ap)
 			 * now tell them that it is ok to use.
 			 */
 			if (!error) {
-				if (m->flags & PG_WANTED)
+				if (m->oflags & VPO_WANTED)
 					vm_page_activate(m);
 				else
 					vm_page_deactivate(m);
@@ -524,7 +524,7 @@ nwfs_putpages(ap)
 #ifndef NWFS_RWCACHE
 	td = curthread;			/* XXX */
 	cred = td->td_ucred;		/* XXX */
-	VOP_OPEN(vp, FWRITE, cred, td, -1);
+	VOP_OPEN(vp, FWRITE, cred, td, NULL);
 	error = vop_stdputpages(ap);
 	VOP_CLOSE(vp, FWRITE, cred, td);
 	return error;
@@ -541,7 +541,7 @@ nwfs_putpages(ap)
 
 	td = curthread;			/* XXX */
 	cred = td->td_ucred;		/* XXX */
-/*	VOP_OPEN(vp, FWRITE, cred, td, -1);*/
+/*	VOP_OPEN(vp, FWRITE, cred, td, NULL);*/
 	np = VTONW(vp);
 	nmp = VFSTONWFS(vp->v_mount);
 	pages = ap->a_m;
