@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/fs/udf/udf_vnops.c,v 1.58.2.4 2007/06/26 06:59:24 remko Exp $
+ * $FreeBSD: src/sys/fs/udf/udf_vnops.c,v 1.66 2007/06/11 20:14:44 remko Exp $
  */
 
 /* udf_vnops.c */
@@ -67,6 +67,7 @@ static vop_strategy_t	udf_strategy;
 static vop_bmap_t	udf_bmap;
 static vop_cachedlookup_t	udf_lookup;
 static vop_reclaim_t	udf_reclaim;
+static vop_vptofh_t	udf_vptofh;
 static int udf_readatoffset(struct udf_node *node, int *size, off_t offset,
     struct buf **bp, uint8_t **data);
 static int udf_bmap_internal(struct udf_node *node, off_t offset,
@@ -88,10 +89,11 @@ static struct vop_vector udf_vnodeops = {
 	.vop_readlink =		udf_readlink,
 	.vop_reclaim =		udf_reclaim,
 	.vop_strategy =		udf_strategy,
+	.vop_vptofh =		udf_vptofh,
 };
 
-MALLOC_DEFINE(M_UDFFID, "UDF FID", "UDF FileId structure");
-MALLOC_DEFINE(M_UDFDS, "UDF DS", "UDF Dirstream structure");
+MALLOC_DEFINE(M_UDFFID, "udf_fid", "UDF FileId structure");
+MALLOC_DEFINE(M_UDFDS, "udf_ds", "UDF Dirstream structure");
 
 #define UDF_INVALID_BMAP	-1
 
@@ -167,7 +169,7 @@ udf_open(struct vop_open_args *ap) {
 	off_t fsize;
 
 	fsize = le64toh(np->fentry->inf_len);
-	vnode_create_vobject_off(ap->a_vp, fsize, ap->a_td);
+	vnode_create_vobject(ap->a_vp, fsize, ap->a_td);
 	return 0;
 }
 
@@ -900,7 +902,7 @@ udf_lookup(struct vop_cachedlookup_args *a)
 	 * If this is a LOOKUP and we've already partially searched through
 	 * the directory, pick up where we left off and flag that the
 	 * directory may need to be searched twice.  For a full description,
-	 * see /sys/isofs/cd9660/cd9660_lookup.c:cd9660_lookup()
+	 * see /sys/fs/cd9660/cd9660_lookup.c:cd9660_lookup()
 	 */
 	if (nameiop != LOOKUP || node->diroff == 0 || node->diroff > fsize) {
 		offset = 0;
@@ -1022,6 +1024,20 @@ udf_reclaim(struct vop_reclaim_args *a)
 	return (0);
 }
 
+static int
+udf_vptofh(struct vop_vptofh_args *a)
+{
+	struct udf_node *node;
+	struct ifid *ifhp;
+
+	node = VTON(a->a_vp);
+	ifhp = (struct ifid *)a->a_fhp;
+	ifhp->ifid_len = sizeof(struct ifid);
+	ifhp->ifid_ino = node->hash_id;
+
+	return (0);
+}
+
 /*
  * Read the block and then set the data pointer to correspond with the
  * offset passed in.  Only read in at most 'size' bytes, and then set 'size'
@@ -1132,7 +1148,7 @@ udf_bmap_internal(struct udf_node *node, off_t offset, daddr_t *sector,
 		} while(offset >= icblen);
 
 		lsector = (offset  >> udfmp->bshift) +
-		    ((struct short_ad *)(icb))->pos;
+		    le32toh(((struct short_ad *)(icb))->pos);
 
 		*max_size = icblen - offset;
 
