@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)union_vnops.c	8.32 (Berkeley) 6/23/95
- * $FreeBSD: src/sys/fs/unionfs/union_vnops.c,v 1.132.2.8 2007/10/23 03:37:10 daichi Exp $
+ * $FreeBSD: src/sys/fs/unionfs/union_vnops.c,v 1.142.2.7 2007/10/22 05:41:54 daichi Exp $
  *
  */
 
@@ -483,13 +483,13 @@ unionfs_open(struct vop_open_args *ap)
 			targetvp = uvp;
 	}
 
-	error = VOP_OPEN(targetvp, ap->a_mode, cred, td, ap->a_fdidx);
+	error = VOP_OPEN(targetvp, ap->a_mode, cred, td, ap->a_fp);
 	if (error == 0) {
 		if (targetvp == uvp) {
 			if (uvp->v_type == VDIR && lvp != NULLVP &&
 			    unsp->uns_lower_opencnt <= 0) {
 				/* open lower for readdir */
-				error = VOP_OPEN(lvp, FREAD, cred, td, -1);
+				error = VOP_OPEN(lvp, FREAD, cred, td, NULL);
 				if (error != 0) {
 					VOP_CLOSE(uvp, ap->a_mode, cred, td);
 					goto unionfs_open_abort;
@@ -501,7 +501,6 @@ unionfs_open(struct vop_open_args *ap)
 		} else {
 			unsp->uns_lower_opencnt++;
 			unsp->uns_lower_openmode = ap->a_mode;
-			unsp->uns_lower_fdidx = ap->a_fdidx;
 		}
 		ap->a_vp->v_object = targetvp->v_object;
 	}
@@ -1650,7 +1649,7 @@ unionfs_get_llt_revlock(int flags)
 }
 
 static int
-unionfs_lock(struct vop_lock_args *ap)
+unionfs_lock(struct vop_lock1_args *ap)
 {
 	int		error;
 	int		flags;
@@ -1888,7 +1887,8 @@ unionfs_advlock(struct vop_advlock_args *ap)
 		unionfs_get_node_status(unp, td, &unsp);
 		if (unsp->uns_lower_opencnt > 0) {
 			/* try reopen the vnode */
-			error = VOP_OPEN(uvp, unsp->uns_lower_openmode, td->td_ucred, td, unsp->uns_lower_fdidx);
+			error = VOP_OPEN(uvp, unsp->uns_lower_openmode,
+				td->td_ucred, td, NULL);
 			if (error)
 				goto unionfs_advlock_abort;
 			unsp->uns_upper_opencnt++;
@@ -2274,6 +2274,12 @@ unionfs_setlabel(struct vop_setlabel_args *ap)
 	return (error);
 }
 
+static int
+unionfs_vptofh(struct vop_vptofh_args *ap)
+{
+	return (EOPNOTSUPP);
+}
+
 struct vop_vector unionfs_vnodeops = {
 	.vop_default =		&default_vnodeops,
 
@@ -2296,7 +2302,7 @@ struct vop_vector unionfs_vnodeops = {
 	.vop_lease =		unionfs_lease,
 	.vop_link =		unionfs_link,
 	.vop_listextattr =	unionfs_listextattr,
-	.vop_lock =		unionfs_lock,
+	.vop_lock1 =		unionfs_lock,
 	.vop_lookup =		vfs_cache_lookup,
 	.vop_mkdir =		unionfs_mkdir,
 	.vop_mknod =		unionfs_mknod,
@@ -2321,4 +2327,5 @@ struct vop_vector unionfs_vnodeops = {
 	.vop_unlock =		unionfs_unlock,
 	.vop_whiteout =		unionfs_whiteout,
 	.vop_write =		unionfs_write,
+	.vop_vptofh =		unionfs_vptofh,
 };
