@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/geom/eli/g_eli.c,v 1.3.2.13 2006/12/02 08:49:40 pjd Exp $");
+__FBSDID("$FreeBSD: src/sys/geom/eli/g_eli.c,v 1.38 2007/06/05 00:00:51 jeff Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -258,6 +258,7 @@ g_eli_start(struct bio *bp)
 	case BIO_READ:
 	case BIO_WRITE:
 	case BIO_GETATTR:
+	case BIO_FLUSH:
 		break;
 	case BIO_DELETE:
 		/*
@@ -265,7 +266,7 @@ g_eli_start(struct bio *bp)
 		 * It could be done by overwritting requested sector with
 		 * random data g_eli_overwrites number of times.
 		 */
-	default:	
+	default:
 		g_io_deliver(bp, EOPNOTSUPP);
 		return;
 	}
@@ -298,6 +299,7 @@ g_eli_start(struct bio *bp)
 		wakeup(sc);
 		break;
 	case BIO_GETATTR:
+	case BIO_FLUSH:
 		cbp->bio_done = g_std_done;
 		cp = LIST_FIRST(&sc->sc_geom->consumer);
 		cbp->bio_to = cp->provider;
@@ -322,11 +324,19 @@ g_eli_worker(void *arg)
 
 	wr = arg;
 	sc = wr->w_softc;
-	mtx_lock_spin(&sched_lock);
+#ifdef SMP
+	/* Before sched_bind() to a CPU, wait for all CPUs to go on-line. */
+	if (mp_ncpus > 1 && sc->sc_crypto == G_ELI_CRYPTO_SW &&
+	    g_eli_threads == 0) {
+		while (!smp_started)
+			tsleep(wr, 0, "geli:smp", hz / 4);
+	}
+#endif
+	thread_lock(curthread);
 	sched_prio(curthread, PRIBIO);
 	if (sc->sc_crypto == G_ELI_CRYPTO_SW && g_eli_threads == 0)
 		sched_bind(curthread, wr->w_number);
-	mtx_unlock_spin(&sched_lock);
+	thread_unlock(curthread);
 
 	G_ELI_DEBUG(1, "Thread %s started.", curthread->td_proc->p_comm);
 
@@ -652,12 +662,14 @@ g_eli_create(struct gctl_req *req, struct g_class *mp, struct g_provider *bpp,
 		 * Use software cryptography, if we cannot get it.
 		 */
 		if (LIST_EMPTY(&sc->sc_workers)) {
-			error = crypto_newsession(&wr->w_sid, &crie, 1);
+			error = crypto_newsession(&wr->w_sid, &crie,
+					CRYPTOCAP_F_HARDWARE);
 			if (error == 0)
 				sc->sc_crypto = G_ELI_CRYPTO_HW;
 		}
 		if (sc->sc_crypto == G_ELI_CRYPTO_SW)
-			error = crypto_newsession(&wr->w_sid, &crie, 0);
+			error = crypto_newsession(&wr->w_sid, &crie,
+					CRYPTOCAP_F_SOFTWARE);
 		if (error != 0) {
 			free(wr, M_ELI);
 			if (req != NULL) {
@@ -869,7 +881,7 @@ g_eli_taste(struct g_class *mp, struct g_provider *pp, int flags __unused)
 	g_trace(G_T_TOPOLOGY, "%s(%s, %s)", __func__, mp->name, pp->name);
 	g_topology_assert();
 
-	if (rootvnode != NULL || g_eli_tries == 0)
+	if (root_mounted() || g_eli_tries == 0)
 		return (NULL);
 
 	G_ELI_DEBUG(3, "Tasting %s.", pp->name);
