@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/geom/eli/g_eli_ctl.c,v 1.1.2.6 2006/09/19 11:40:36 pjd Exp $");
+__FBSDID("$FreeBSD: src/sys/geom/eli/g_eli_ctl.c,v 1.13 2007/05/06 14:56:03 pjd Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -355,6 +355,10 @@ g_eli_ctl_onetime(struct gctl_req *req, struct g_class *mp)
 			gctl_error(req, "Invalid sector size.");
 			return;
 		}
+		if (*sectorsize > PAGE_SIZE) {
+			gctl_error(req, "warning: Using sectorsize bigger than "
+			    "the page size!");
+		}
 		md.md_sectorsize = *sectorsize;
 	}
 
@@ -638,7 +642,7 @@ g_eli_ctl_delkey(struct gctl_req *req, struct g_class *mp)
 			gctl_error(req, "No '%s' argument.", "force");
 			return;
 		}
-	
+
 		valp = gctl_get_paraml(req, "keyno", sizeof(*valp));
 		if (valp == NULL) {
 			gctl_error(req, "No '%s' argument.", "keyno");
@@ -680,6 +684,11 @@ g_eli_ctl_delkey(struct gctl_req *req, struct g_class *mp)
 			G_ELI_DEBUG(0, "Cannot store metadata on %s "
 			    "(error=%d).", pp->name, error);
 		}
+		/*
+		 * Flush write cache so we don't overwrite data N times in cache
+		 * and only once on disk.
+		 */
+		g_io_flush(cp);
 	}
 	bzero(&md, sizeof(md));
 	bzero(sector, sizeof(sector));
