@@ -41,7 +41,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/geom/geom_aes.c,v 1.28.2.1 2005/12/29 05:59:51 sobomax Exp $");
+__FBSDID("$FreeBSD: src/sys/geom/geom_aes.c,v 1.30 2005/11/30 22:15:00 sobomax Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
