@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/geom/shsec/g_shsec.c,v 1.3.4.1 2006/04/05 22:12:29 pjd Exp $");
+__FBSDID("$FreeBSD: src/sys/geom/shsec/g_shsec.c,v 1.6 2006/11/01 12:30:51 pjd Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -41,7 +41,7 @@ __FBSDID("$FreeBSD: src/sys/geom/shsec/g_shsec.c,v 1.3.4.1 2006/04/05 22:12:29 p
 #include <geom/shsec/g_shsec.h>
 
 
-static MALLOC_DEFINE(M_SHSEC, "shsec data", "GEOM_SHSEC Data");
+static MALLOC_DEFINE(M_SHSEC, "shsec_data", "GEOM_SHSEC Data");
 
 static uma_zone_t g_shsec_zone;
 
@@ -317,6 +317,7 @@ g_shsec_start(struct bio *bp)
 	switch (bp->bio_cmd) {
 	case BIO_READ:
 	case BIO_WRITE:
+	case BIO_FLUSH:
 		/*
 		 * Only those requests are supported.
 		 */
