@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/geom/uzip/g_uzip.c,v 1.4.2.5 2007/05/01 03:01:15 simokawa Exp $");
+__FBSDID("$FreeBSD: src/sys/geom/uzip/g_uzip.c,v 1.12 2007/04/24 06:30:06 simokawa Exp $");
 
 #include <sys/param.h>
 #include <sys/bio.h>
@@ -47,7 +47,7 @@ __FBSDID("$FreeBSD: src/sys/geom/uzip/g_uzip.c,v 1.4.2.5 2007/05/01 03:01:15 sim
 #define DPRINTF(a)
 #endif
 
-MALLOC_DEFINE(M_GEOM_UZIP, "GEOM UZIP", "GEOM UZIP data structures");
+MALLOC_DEFINE(M_GEOM_UZIP, "geom_uzip", "GEOM UZIP data structures");
 
 #define UZIP_CLASS_NAME	"UZIP"
 
@@ -529,5 +529,5 @@ static struct g_class g_uzip_class = {
 	.spoiled = g_uzip_spoiled,
 };
 
-DECLARE_GEOM_CLASS(g_uzip_class, geom_uzip);
-MODULE_DEPEND(geom_uzip, zlib, 1, 1, 1);
+DECLARE_GEOM_CLASS(g_uzip_class, g_uzip);
+MODULE_DEPEND(g_uzip, zlib, 1, 1, 1);
