@@ -30,7 +30,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)pcb.h	5.10 (Berkeley) 5/12/91
- * $FreeBSD: src/sys/i386/include/pcb.h,v 1.55 2005/04/14 04:13:27 peter Exp $
+ * $FreeBSD: src/sys/i386/include/pcb.h,v 1.56 2005/12/29 13:23:48 davidxu Exp $
  */
 
 #ifndef _I386_PCB_H_
@@ -74,7 +74,6 @@ struct pcb {
 	struct segment_descriptor pcb_gsd;
 	struct	pcb_ext	*pcb_ext;	/* optional pcb extension */
 	int	pcb_psl;	/* process status long */
-	void	(*pcb_switchout)(void);	/* Special switchout function. */
 	u_long	pcb_vm86[2];	/* vm86bios scratch space */
 };
 
