@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i386/isa/npx.c,v 1.162.2.4 2006/07/01 00:57:56 davidxu Exp $");
+__FBSDID("$FreeBSD: src/sys/i386/isa/npx.c,v 1.172 2007/06/05 00:00:52 jeff Exp $");
 
 #include "opt_cpu.h"
 #include "opt_isa.h"
@@ -62,7 +62,6 @@ __FBSDID("$FreeBSD: src/sys/i386/isa/npx.c,v 1.162.2.4 2006/07/01 00:57:56 david
 #include <machine/md_var.h>
 #include <machine/pcb.h>
 #include <machine/psl.h>
-#include <machine/clock.h>
 #include <machine/resource.h>
 #include <machine/specialreg.h>
 #include <machine/segments.h>
@@ -150,7 +149,7 @@ static	void	fpusave(union savefpu *);
 static	void	fpurstor(union savefpu *);
 static	int	npx_attach(device_t dev);
 static	void	npx_identify(driver_t *driver, device_t parent);
-static	void	npx_intr(void *);
+static	int	npx_intr(void *);
 static	int	npx_probe(device_t dev);
 #ifdef I586_CPU_XXX
 static	long	timezero(const char *funcname,
@@ -202,7 +201,7 @@ npx_identify(driver, parent)
 /*
  * Do minimal handling of npx interrupts to convert them to traps.
  */
-static void
+static int
 npx_intr(dummy)
 	void *dummy;
 {
@@ -231,10 +230,11 @@ npx_intr(dummy)
 	td = PCPU_GET(fpcurthread);
 	if (td != NULL) {
 		td->td_pcb->pcb_flags |= PCB_NPXTRAP;
-		mtx_lock_spin(&sched_lock);
+		thread_lock(td);
 		td->td_flags |= TDF_ASTPENDING;
-		mtx_unlock_spin(&sched_lock);
+		thread_unlock(td);
 	}
+	return (FILTER_HANDLED);
 }
 
 /*
@@ -280,8 +280,8 @@ npx_probe(dev)
 	irq_res = bus_alloc_resource(dev, SYS_RES_IRQ, &irq_rid, irq_num,
 	    irq_num, 1, RF_ACTIVE);
 	if (irq_res != NULL) {
-		if (bus_setup_intr(dev, irq_res, INTR_TYPE_MISC | INTR_FAST,
-			npx_intr, NULL, &irq_cookie) != 0)
+		if (bus_setup_intr(dev, irq_res, INTR_TYPE_MISC,
+			npx_intr, NULL, NULL, &irq_cookie) != 0)
 			panic("npx: can't create intr");
 	}
 
