@@ -28,7 +28,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i386/pci/pci_pir.c,v 1.117.2.2 2006/11/20 22:47:00 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/i386/pci/pci_pir.c,v 1.120 2006/11/09 18:03:36 jhb Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -107,6 +107,7 @@ static int pir_interrupt_weight[NUM_ISA_INTERRUPTS];
 /* sysctl vars */
 SYSCTL_DECL(_hw_pci);
 
+/* XXX this likely should live in a header file */
 #ifdef PC98
 /* IRQs 3, 5, 7, 9, 10, 11, 12, 13 */
 #define PCI_IRQ_OVERRIDE_MASK 0x3e68
