@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/capi/capi_msgs.c,v 1.4.4.1 2005/01/31 23:26:07 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/capi/capi_msgs.c,v 1.6 2007/07/06 07:17:17 bz Exp $");
 
 #include <sys/param.h>
 #include <sys/kernel.h>
@@ -36,9 +36,9 @@ __FBSDID("$FreeBSD: src/sys/i4b/capi/capi_msgs.c,v 1.4.4.1 2005/01/31 23:26:07 i
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include <machine/i4b_debug.h>
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_cause.h>
+#include <i4b/include/i4b_debug.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_cause.h>
 
 #include <i4b/include/i4b_l3l4.h>
 #include <i4b/include/i4b_mbuf.h>
