@@ -28,7 +28,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/capi/iavc/iavc_lli.c,v 1.8 2005/01/06 22:18:18 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/capi/iavc/iavc_lli.c,v 1.10 2007/07/06 07:17:17 bz Exp $");
 
 #include <sys/param.h>
 #include <sys/kernel.h>
@@ -38,7 +38,6 @@ __FBSDID("$FreeBSD: src/sys/i4b/capi/iavc/iavc_lli.c,v 1.8 2005/01/06 22:18:18 i
 #include <sys/malloc.h>
 #include <net/if.h>
 
-#include <machine/clock.h>
 
 #include <machine/bus.h>
 #include <sys/bus.h>
@@ -46,9 +45,9 @@ __FBSDID("$FreeBSD: src/sys/i4b/capi/iavc/iavc_lli.c,v 1.8 2005/01/06 22:18:18 i
 #include <vm/vm.h>
 #include <vm/pmap.h>
 
-#include <machine/i4b_debug.h>
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_trace.h>
+#include <i4b/include/i4b_debug.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_trace.h>
 
 #include <i4b/include/i4b_global.h>
 #include <i4b/include/i4b_l3l4.h>
