@@ -55,7 +55,7 @@
  *---------------------------------------------------------------------------*/ 
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_ipr.c,v 1.34.2.2 2007/01/19 23:01:31 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_ipr.c,v 1.40 2007/07/06 07:17:18 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -98,15 +98,15 @@ __FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_ipr.c,v 1.34.2.2 2007/01/19 23:01:31
 #include <sys/time.h>
 #include <net/bpf.h>
 
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_debug.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_debug.h>
 
 #include <i4b/include/i4b_global.h>
 #include <i4b/include/i4b_l3l4.h>
 
 #include <i4b/layer4/i4b_l4.h>
 
-NET_NEEDS_GIANT("i4b_ipr");
+#error "Cannot be used until I4B is locked."
 
 #define I4BIPRMTU	1500		/* regular MTU */
 #define I4BIPRMAXMTU	2000		/* max MTU */
@@ -490,7 +490,7 @@ i4biprioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 			{
 			struct thread *td = curthread;	/* XXX */
 
-			if((error = suser(td)))
+			if((error = priv_check(td, PRIV_DRIVER)))
 				return (error);
 		        sl_compress_setup(sc->sc_compr, *(int *)data);
 			}
