@@ -32,9 +32,9 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_rbch.c,v 1.33.2.1 2005/01/31 23:26:08 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_rbch.c,v 1.36 2007/07/06 07:17:18 bz Exp $");
 
-#include "i4brbch.h"
+#include "opt_i4b.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -48,9 +48,9 @@ __FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_rbch.c,v 1.33.2.1 2005/01/31 23:26:08
 
 #include <net/if.h>
 
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_rbch_ioctl.h>
-#include <machine/i4b_debug.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_rbch_ioctl.h>
+#include <i4b/include/i4b_debug.h>
 
 #include <i4b/include/i4b_global.h>
 #include <i4b/include/i4b_mbuf.h>
