@@ -32,7 +32,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_tel.c,v 1.36 2005/01/06 22:18:18 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_tel.c,v 1.37 2007/07/06 07:17:18 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -50,9 +50,9 @@ __FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_tel.c,v 1.36 2005/01/06 22:18:18 imp
 #include <net/if.h>
 #include <sys/tty.h>
 
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_tel_ioctl.h>
-#include <machine/i4b_debug.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_tel_ioctl.h>
+#include <i4b/include/i4b_debug.h>
 
 #include <i4b/include/i4b_global.h>
 #include <i4b/include/i4b_mbuf.h>
