@@ -34,7 +34,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_trace.c,v 1.32 2005/01/06 22:18:18 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_trace.c,v 1.33 2007/07/06 07:17:18 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -49,8 +49,8 @@ __FBSDID("$FreeBSD: src/sys/i4b/driver/i4b_trace.c,v 1.32 2005/01/06 22:18:18 im
 #include <net/if.h>
 #include <sys/tty.h>
 
-#include <machine/i4b_trace.h>
-#include <machine/i4b_ioctl.h>
+#include <i4b/include/i4b_trace.h>
+#include <i4b/include/i4b_ioctl.h>
 
 #include <i4b/include/i4b_mbuf.h>
 #include <i4b/include/i4b_global.h>
