@@ -32,7 +32,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_diva.c,v 1.4 2005/01/06 22:18:20 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_diva.c,v 1.5 2007/07/06 07:17:20 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -44,7 +44,7 @@ __FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_diva.c,v 1.4 2005/01/06 22:18:20
 
 #include <net/if.h>
 
-#include <machine/i4b_ioctl.h>
+#include <i4b/include/i4b_ioctl.h>
 
 #include <i4b/layer1/isic/i4b_isic.h>
 #include <i4b/layer1/isic/i4b_ipac.h>
