@@ -32,7 +32,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_drn_ngo.c,v 1.9 2005/01/06 22:18:20 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_drn_ngo.c,v 1.10 2007/07/06 07:17:20 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -43,7 +43,7 @@ __FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_drn_ngo.c,v 1.9 2005/01/06 22:18
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include <machine/i4b_ioctl.h>
+#include <i4b/include/i4b_ioctl.h>
 
 #include <i4b/layer1/isic/i4b_isic.h>
 #include <i4b/layer1/isic/i4b_hscx.h>
