@@ -59,7 +59,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_itk_ix1.c,v 1.10 2005/01/06 22:18:20 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_itk_ix1.c,v 1.13 2007/07/06 07:17:20 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -70,8 +70,8 @@ __FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_itk_ix1.c,v 1.10 2005/01/06 22:1
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_trace.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_trace.h>
 #include <i4b/layer1/i4b_l1.h>
 #include <i4b/layer1/isic/i4b_isic.h>
 #include <i4b/layer1/isic/i4b_hscx.h>
@@ -279,10 +279,15 @@ isic_probe_itkix1(device_t dev)
 	printf("Setting up access interupt...");
 	#endif
 
-	bus_setup_intr(dev, sc->sc_resources.irq,
-			INTR_TYPE_NET,
-			(void(*)(void *))(isicintr),
-			sc, &ih);
+	if (bus_setup_intr(dev, sc->sc_resources.irq, INTR_TYPE_NET, NULL,
+			(void(*)(void *))(isicintr), sc, &ih) != 0)
+	{
+		printf("isic%d: Could not setup irq for ITK IX1.\n", unit);
+		bus_release_resource(dev,SYS_RES_IOPORT,
+				sc->sc_resources.io_rid[0],
+				sc->sc_resources.io_base[0]);
+		return ENXIO;
+	}
 
 	#if defined(ITK_PROBE_DEBUG)
 	printf("done.\n");
