@@ -32,7 +32,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_l1fsm.c,v 1.8 2005/01/06 22:18:20 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_l1fsm.c,v 1.9 2007/07/06 07:17:20 bz Exp $");
 
 #include <sys/param.h>
 #include <sys/kernel.h>
@@ -41,9 +41,9 @@ __FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_l1fsm.c,v 1.8 2005/01/06 22:18:2
 
 #include <net/if.h>
 
-#include <machine/i4b_debug.h>
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_trace.h>
+#include <i4b/include/i4b_debug.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_trace.h>
 
 #include <i4b/layer1/isic/i4b_isic.h>
 
