@@ -32,7 +32,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_usr_sti.c,v 1.9 2005/01/06 22:18:20 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_usr_sti.c,v 1.11 2007/07/06 07:17:20 bz Exp $");
 
 #include "opt_i4b.h"
 
@@ -47,8 +47,8 @@ __FBSDID("$FreeBSD: src/sys/i4b/layer1/isic/i4b_usr_sti.c,v 1.9 2005/01/06 22:18
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include <machine/i4b_ioctl.h>
-#include <machine/i4b_trace.h>
+#include <i4b/include/i4b_ioctl.h>
+#include <i4b/include/i4b_trace.h>
 
 #include <i4b/layer1/i4b_l1.h>
 #include <i4b/layer1/isic/i4b_isic.h>
@@ -400,7 +400,7 @@ isic_probe_usrtai(device_t dev)
 
 	/* register interrupt routine */
 	bus_setup_intr(dev, sc->sc_resources.irq, INTR_TYPE_NET,
-			(void(*)(void *))(isicintr),
+			NULL, (void(*)(void *))(isicintr),
 			sc, &ih);
 
 	/* check IRQ validity */
