@@ -32,7 +32,7 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer2/i4b_iframe.c,v 1.14 2005/01/06 22:18:21 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer2/i4b_iframe.c,v 1.15 2007/07/06 07:17:21 bz Exp $");
 
 #include <sys/param.h>
 #include <sys/kernel.h>
@@ -41,8 +41,8 @@ __FBSDID("$FreeBSD: src/sys/i4b/layer2/i4b_iframe.c,v 1.14 2005/01/06 22:18:21 i
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include <machine/i4b_debug.h>
-#include <machine/i4b_trace.h>
+#include <i4b/include/i4b_debug.h>
+#include <i4b/include/i4b_trace.h>
 
 #include <i4b/include/i4b_global.h>
 #include <i4b/include/i4b_l1l2.h>
