@@ -32,14 +32,14 @@
  *---------------------------------------------------------------------------*/
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/i4b/layer2/i4b_lme.c,v 1.15 2005/01/06 22:18:21 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/i4b/layer2/i4b_lme.c,v 1.16 2007/07/06 07:17:21 bz Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include <machine/i4b_debug.h>
+#include <i4b/include/i4b_debug.h>
 
 #include <i4b/layer2/i4b_l2.h>
 
