@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/ia64/ia64/sal.c,v 1.14.2.1 2005/09/13 21:07:14 marcel Exp $");
+__FBSDID("$FreeBSD: src/sys/ia64/ia64/sal.c,v 1.15 2005/08/13 21:08:32 marcel Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
