@@ -25,10 +25,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/isa/syscons_isa.c,v 1.27.2.1 2006/08/22 16:52:42 rink Exp $");
+__FBSDID("$FreeBSD: src/sys/isa/syscons_isa.c,v 1.29 2007/02/27 17:22:30 jhb Exp $");
 
 #include "opt_syscons.h"
-#include "opt_xbox.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -48,10 +47,6 @@ __FBSDID("$FreeBSD: src/sys/isa/syscons_isa.c,v 1.27.2.1 2006/08/22 16:52:42 rin
 #include <machine/timerreg.h>
 #include <machine/pc/bios.h>
 
-#ifdef XBOX
-#include <machine/xbox.h>
-#endif
-
 #include <vm/vm.h>
 #include <vm/pmap.h>
 #include <vm/vm_param.h>
@@ -67,6 +62,12 @@ __FBSDID("$FreeBSD: src/sys/isa/syscons_isa.c,v 1.27.2.1 2006/08/22 16:52:42 rin
 
 #include <isa/isavar.h>
 
+#include "opt_xbox.h"
+
+#ifdef XBOX
+#include <machine/xbox.h>
+#endif
+
 static devclass_t	sc_devclass;
 
 static sc_softc_t main_softc;
@@ -109,7 +110,6 @@ static int
 scsuspend(device_t dev)
 {
 	int		retry = 10;
-	static int	dummy;
 	sc_softc_t	*sc;
 
 	sc = &main_softc;
@@ -127,7 +127,7 @@ scsuspend(device_t dev)
 		if (!sc->switch_in_progress) {
 			break;
 		}
-		tsleep(&dummy, 0, "scsuspend", 100);
+		pause("scsuspend", hz);
 	} while (retry--);
 
 	return (0);
