@@ -23,7 +23,7 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 #
-# $FreeBSD: src/sys/kern/bus_if.m,v 1.29.2.1 2006/07/12 22:57:57 imp Exp $
+# $FreeBSD: src/sys/kern/bus_if.m,v 1.34 2007/02/23 12:19:01 piso Exp $
 #
 
 #include <sys/bus.h>
@@ -326,6 +326,7 @@ METHOD int setup_intr {
 	device_t	_child;
 	struct resource *_irq;
 	int		_flags;
+	driver_filter_t	*_filter;
 	driver_intr_t	*_intr;
 	void		*_arg;
 	void		**_cookiep;
@@ -529,3 +530,14 @@ METHOD void hinted_child {
 	const char *	_dname;
 	int		_dunit;
 };
+
+/**
+ * @brief Returns bus_dma_tag_t for use w/ devices on the bus.
+ *
+ * @param _dev		the parent device of @p _child
+ * @param _child	the device to which the tag will belong
+ */
+METHOD bus_dma_tag_t get_dma_tag {
+	device_t	_dev;
+	device_t	_child;
+} DEFAULT bus_generic_get_dma_tag;
