@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/kern/kern_ntptime.c,v 1.54.2.2 2005/04/21 20:33:05 ps Exp $");
+__FBSDID("$FreeBSD: src/sys/kern/kern_ntptime.c,v 1.64 2007/06/14 18:37:58 rwatson Exp $");
 
 #include "opt_ntp.h"
 
@@ -39,6 +39,7 @@ __FBSDID("$FreeBSD: src/sys/kern/kern_ntptime.c,v 1.54.2.2 2005/04/21 20:33:05 p
 #include <sys/systm.h>
 #include <sys/sysproto.h>
 #include <sys/kernel.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/lock.h>
 #include <sys/mutex.h>
@@ -196,27 +197,22 @@ static long pps_errcnt;			/* calibration errors */
 
 static void ntp_init(void);
 static void hardupdate(long offset);
+static void ntp_gettime1(struct ntptimeval *ntvp);
 
-/*
- * ntp_gettime() - NTP user application interface
- *
- * See the timex.h header file for synopsis and API description. Note
- * that the TAI offset is returned in the ntvtimeval.tai structure
- * member.
- */
-static int
-ntp_sysctl(SYSCTL_HANDLER_ARGS)
+static void
+ntp_gettime1(struct ntptimeval *ntvp)
 {
-	struct ntptimeval ntv;	/* temporary structure */
 	struct timespec atv;	/* nanosecond time */
 
+	GIANT_REQUIRED;
+
 	nanotime(&atv);
-	ntv.time.tv_sec = atv.tv_sec;
-	ntv.time.tv_nsec = atv.tv_nsec;
-	ntv.maxerror = time_maxerror;
-	ntv.esterror = time_esterror;
-	ntv.tai = time_tai;
-	ntv.time_state = time_state;
+	ntvp->time.tv_sec = atv.tv_sec;
+	ntvp->time.tv_nsec = atv.tv_nsec;
+	ntvp->maxerror = time_maxerror;
+	ntvp->esterror = time_esterror;
+	ntvp->tai = time_tai;
+	ntvp->time_state = time_state;
 
 	/*
 	 * Status word error decode. If any of these conditions occur,
@@ -247,8 +243,42 @@ ntp_sysctl(SYSCTL_HANDLER_ARGS)
 	 */
 	    (time_status & STA_PPSFREQ &&
 	    time_status & (STA_PPSWANDER | STA_PPSERROR)))
-		ntv.time_state = TIME_ERROR;
-	return (sysctl_handle_opaque(oidp, &ntv, sizeof ntv, req));
+		ntvp->time_state = TIME_ERROR;
+}
+
+/*
+ * ntp_gettime() - NTP user application interface
+ *
+ * See the timex.h header file for synopsis and API description.  Note that
+ * the TAI offset is returned in the ntvtimeval.tai structure member.
+ */
+#ifndef _SYS_SYSPROTO_H_
+struct ntp_gettime_args {
+	struct ntptimeval *ntvp;
+};
+#endif
+/* ARGSUSED */
+int
+ntp_gettime(struct thread *td, struct ntp_gettime_args *uap)
+{	
+	struct ntptimeval ntv;
+
+	mtx_lock(&Giant);
+	ntp_gettime1(&ntv);
+	mtx_unlock(&Giant);
+
+	td->td_retval[0] = ntv.time_state;
+	return (copyout(&ntv, uap->ntvp, sizeof(ntv)));
+}
+
+static int
+ntp_sysctl(SYSCTL_HANDLER_ARGS)
+{
+	struct ntptimeval ntv;	/* temporary structure */
+
+	ntp_gettime1(&ntv);
+
+	return (sysctl_handle_opaque(oidp, &ntv, sizeof(ntv), req));
 }
 
 SYSCTL_NODE(_kern, OID_AUTO, ntp_pll, CTLFLAG_RW, 0, "");
@@ -263,12 +293,13 @@ SYSCTL_INT(_kern_ntp_pll, OID_AUTO, time_monitor, CTLFLAG_RD, &time_monitor, 0,
 SYSCTL_OPAQUE(_kern_ntp_pll, OID_AUTO, pps_freq, CTLFLAG_RD, &pps_freq, sizeof(pps_freq), "I", "");
 SYSCTL_OPAQUE(_kern_ntp_pll, OID_AUTO, time_freq, CTLFLAG_RD, &time_freq, sizeof(time_freq), "I", "");
 #endif
+
 /*
  * ntp_adjtime() - NTP daemon application interface
  *
- * See the timex.h header file for synopsis and API description. Note
- * that the timex.constant structure member has a dual purpose to set
- * the time constant and to set the TAI offset.
+ * See the timex.h header file for synopsis and API description.  Note that
+ * the timex.constant structure member has a dual purpose to set the time
+ * constant and to set the TAI offset.
  */
 #ifndef _SYS_SYSPROTO_H_
 struct ntp_adjtime_args {
@@ -276,9 +307,6 @@ struct ntp_adjtime_args {
 };
 #endif
 
-/*
- * MPSAFE
- */
 int
 ntp_adjtime(struct thread *td, struct ntp_adjtime_args *uap)
 {
@@ -304,7 +332,7 @@ ntp_adjtime(struct thread *td, struct ntp_adjtime_args *uap)
 	mtx_lock(&Giant);
 	modes = ntv.modes;
 	if (modes)
-		error = suser(td);
+		error = priv_check(td, PRIV_NTP_ADJTIME);
 	if (error)
 		goto done2;
 	s = splclock();
@@ -896,9 +924,6 @@ struct adjtime_args {
 	struct timeval *olddelta;
 };
 #endif
-/*
- * MPSAFE
- */
 /* ARGSUSED */
 int
 adjtime(struct thread *td, struct adjtime_args *uap)
@@ -925,9 +950,6 @@ kern_adjtime(struct thread *td, struct timeval *delta, struct timeval *olddelta)
 	struct timeval atv;
 	int error;
 
-	if ((error = suser(td)))
-		return (error);
-
 	mtx_lock(&Giant);
 	if (olddelta) {
 		atv.tv_sec = time_adjtime / 1000000;
@@ -938,10 +960,15 @@ kern_adjtime(struct thread *td, struct timeval *delta, struct timeval *olddelta)
 		}
 		*olddelta = atv;
 	}
-	if (delta)
+	if (delta) {
+		if ((error = priv_check(td, PRIV_ADJTIME))) {
+			mtx_unlock(&Giant);
+			return (error);
+		}
 		time_adjtime = (int64_t)delta->tv_sec * 1000000 +
 		    delta->tv_usec;
+	}
 	mtx_unlock(&Giant);
-	return (error);
+	return (0);
 }
 
