@@ -32,7 +32,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/kern/kern_rwlock.c,v 1.28.2.4 2007/09/27 20:24:55 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/kern/kern_rwlock.c,v 1.28.4.2 2007/12/01 11:28:37 attilio Exp $");
 
 #include "opt_ddb.h"
 #include "opt_no_adaptive_rwlocks.h"
@@ -59,6 +59,8 @@ CTASSERT((RW_RECURSE & LO_CLASSFLAGS) == RW_RECURSE);
 
 static void	db_show_rwlock(struct lock_object *lock);
 #endif
+static void	lock_rw(struct lock_object *lock, int how);
+static int	unlock_rw(struct lock_object *lock);
 
 struct lock_class lock_class_rw = {
 	.lc_name = "rw",
@@ -66,6 +68,8 @@ struct lock_class lock_class_rw = {
 #ifdef DDB
 	.lc_ddb_show = db_show_rwlock,
 #endif
+	.lc_lock = lock_rw,
+	.lc_unlock = unlock_rw,
 };
 
 /*
@@ -98,6 +102,34 @@ struct lock_class lock_class_rw = {
 #define	_rw_assert(rw, what, file, line)
 #endif
 
+void
+lock_rw(struct lock_object *lock, int how)
+{
+	struct rwlock *rw;
+
+	rw = (struct rwlock *)lock;
+	if (how)
+		rw_wlock(rw);
+	else
+		rw_rlock(rw);
+}
+
+int
+unlock_rw(struct lock_object *lock)
+{
+	struct rwlock *rw;
+
+	rw = (struct rwlock *)lock;
+	rw_assert(rw, RA_LOCKED | LA_NOTRECURSED);
+	if (rw->rw_lock & RW_LOCK_READ) {
+		rw_runlock(rw);
+		return (0);
+	} else {
+		rw_wunlock(rw);
+		return (1);
+	}
+}
+
 void
 rw_init_flags(struct rwlock *rw, const char *name, int opts)
 {
@@ -109,6 +141,8 @@ rw_init_flags(struct rwlock *rw, const char *name, int opts)
 	flags = LO_UPGRADABLE | LO_RECURSABLE;
 	if (opts & RW_DUPOK)
 		flags |= LO_DUPOK;
+	if (opts & RW_NOPROFILE)
+		flags |= LO_NOPROFILE;
 	if (!(opts & RW_NOWITNESS))
 		flags |= LO_WITNESS;
 	if (opts & RW_QUIET)
@@ -152,9 +186,6 @@ _rw_wlock(struct rwlock *rw, const char *file, int line)
 	MPASS(curthread != NULL);
 	KASSERT(rw->rw_lock != RW_DESTROYED,
 	    ("rw_wlock() of destroyed rwlock @ %s:%d", file, line));
-	KASSERT(rw_wowner(rw) != curthread,
-	    ("%s (%s): wlock already held @ %s:%d", __func__,
-	    rw->lock_object.lo_name, file, line));
 	WITNESS_CHECKORDER(&rw->lock_object, LOP_NEWORDER | LOP_EXCLUSIVE, file,
 	    line);
 	__rw_wlock(rw, curthread, file, line);
@@ -183,11 +214,14 @@ _rw_wunlock(struct rwlock *rw, const char *file, int line)
 void
 _rw_rlock(struct rwlock *rw, const char *file, int line)
 {
+	struct turnstile *ts;
 #ifdef ADAPTIVE_RWLOCKS
 	volatile struct thread *owner;
 #endif
-	//uint64_t waittime = 0; /* XXX: notsup */
-	//int contested = 0; /* XXX: notsup */
+#ifdef LOCK_PROFILING_SHARED
+	uint64_t waittime = 0;
+	int contested = 0;
+#endif
 	uintptr_t x;
 
 	KASSERT(rw->rw_lock != RW_DESTROYED,
@@ -230,59 +264,22 @@ _rw_rlock(struct rwlock *rw, const char *file, int line)
 			MPASS((x & RW_LOCK_READ_WAITERS) == 0);
 			if (atomic_cmpset_acq_ptr(&rw->rw_lock, x,
 			    x + RW_ONE_READER)) {
+#ifdef LOCK_PROFILING_SHARED
+				if (RW_READERS(x) == 0)
+					lock_profile_obtain_lock_success(
+					    &rw->lock_object, contested,
+					    waittime, file, line);
+#endif
 				if (LOCK_LOG_TEST(&rw->lock_object, 0))
 					CTR4(KTR_LOCK,
 					    "%s: %p succeed %p -> %p", __func__,
 					    rw, (void *)x,
 					    (void *)(x + RW_ONE_READER));
-				if (RW_READERS(x) == 0)
-					lock_profile_obtain_lock_success(
-					    &rw->lock_object, contested, waittime,
-					    file, line);
 				break;
 			}
 			cpu_spinwait();
 			continue;
 		}
-		lock_profile_obtain_lock_failed(&rw->lock_object, &contested,
-		    &waittime);
-
-		/*
-		 * Okay, now it's the hard case.  Some other thread already
-		 * has a write lock, so acquire the turnstile lock so we can
-		 * begin the process of blocking.
-		 */
-		turnstile_lock(&rw->lock_object);
-
-		/*
-		 * The lock might have been released while we spun, so
-		 * recheck its state and restart the loop if there is no
-		 * longer a write lock.
-		 */
-		x = rw->rw_lock;
-		if (x & RW_LOCK_READ) {
-			turnstile_release(&rw->lock_object);
-			cpu_spinwait();
-			continue;
-		}
-
-		/*
-		 * Ok, it's still a write lock.  If the RW_LOCK_READ_WAITERS
-		 * flag is already set, then we can go ahead and block.  If
-		 * it is not set then try to set it.  If we fail to set it
-		 * drop the turnstile lock and restart the loop.
-		 */
-		if (!(x & RW_LOCK_READ_WAITERS)) {
-			if (!atomic_cmpset_ptr(&rw->rw_lock, x,
-			    x | RW_LOCK_READ_WAITERS)) {
-				turnstile_release(&rw->lock_object);
-				cpu_spinwait();
-				continue;
-			}
-			if (LOCK_LOG_TEST(&rw->lock_object, 0))
-				CTR2(KTR_LOCK, "%s: %p set read waiters flag",
-				    __func__, rw);
-		}
 
 #ifdef ADAPTIVE_RWLOCKS
 		/*
@@ -292,17 +289,70 @@ _rw_rlock(struct rwlock *rw, const char *file, int line)
 		 */
 		owner = (struct thread *)RW_OWNER(x);
 		if (TD_IS_RUNNING(owner)) {
-			turnstile_release(&rw->lock_object);
 			if (LOCK_LOG_TEST(&rw->lock_object, 0))
 				CTR3(KTR_LOCK, "%s: spinning on %p held by %p",
 				    __func__, rw, owner);
-			while ((struct thread*)RW_OWNER(rw->rw_lock)== owner &&
+#ifdef LOCK_PROFILING_SHARED
+			lock_profile_obtain_lock_failed(&rw->lock_object,
+			    &contested, &waittime);
+#endif
+			while ((struct thread*)RW_OWNER(rw->rw_lock) == owner &&
 			    TD_IS_RUNNING(owner))
 				cpu_spinwait();
 			continue;
 		}
 #endif
 
+		/*
+		 * Okay, now it's the hard case.  Some other thread already
+		 * has a write lock, so acquire the turnstile lock so we can
+		 * begin the process of blocking.
+		 */
+		ts = turnstile_trywait(&rw->lock_object);
+
+		/*
+		 * The lock might have been released while we spun, so
+		 * recheck its state and restart the loop if there is no
+		 * longer a write lock.
+		 */
+		x = rw->rw_lock;
+		if (x & RW_LOCK_READ) {
+			turnstile_cancel(ts);
+			cpu_spinwait();
+			continue;
+		}
+
+#ifdef ADAPTIVE_RWLOCKS
+		/*
+		 * If the current owner of the lock is executing on another
+		 * CPU quit the hard path and try to spin.
+		 */
+		owner = (struct thread *)RW_OWNER(x);
+		if (TD_IS_RUNNING(owner)) {
+			turnstile_cancel(ts);
+			cpu_spinwait();
+			continue;
+		}
+#endif
+
+		/*
+		 * Ok, it's still a write lock.  If the RW_LOCK_READ_WAITERS
+		 * flag is already set, then we can go ahead and block.  If
+		 * it is not set then try to set it.  If we fail to set it
+		 * drop the turnstile lock and restart the loop.
+		 */
+		if (!(x & RW_LOCK_READ_WAITERS)) {
+			if (!atomic_cmpset_ptr(&rw->rw_lock, x,
+			    x | RW_LOCK_READ_WAITERS)) {
+				turnstile_cancel(ts);
+				cpu_spinwait();
+				continue;
+			}
+			if (LOCK_LOG_TEST(&rw->lock_object, 0))
+				CTR2(KTR_LOCK, "%s: %p set read waiters flag",
+				    __func__, rw);
+		}
+
 		/*
 		 * We were unable to acquire the lock and the read waiters
 		 * flag is set, so we must block on the turnstile.
@@ -310,8 +360,11 @@ _rw_rlock(struct rwlock *rw, const char *file, int line)
 		if (LOCK_LOG_TEST(&rw->lock_object, 0))
 			CTR2(KTR_LOCK, "%s: %p blocking on turnstile", __func__,
 			    rw);
-		turnstile_wait(&rw->lock_object, rw_owner(rw),
-		    TS_SHARED_QUEUE);
+#ifdef LOCK_PROFILING_SHARED
+		lock_profile_obtain_lock_failed(&rw->lock_object, &contested,
+		    &waittime);
+#endif
+		turnstile_wait(ts, rw_owner(rw), TS_SHARED_QUEUE);
 		if (LOCK_LOG_TEST(&rw->lock_object, 0))
 			CTR2(KTR_LOCK, "%s: %p resuming from turnstile",
 			    __func__, rw);
@@ -369,6 +422,9 @@ _rw_runlock(struct rwlock *rw, const char *file, int line)
 		 */
 		KASSERT(!(x & RW_LOCK_READ_WAITERS),
 		    ("%s: waiting readers", __func__));
+#ifdef LOCK_PROFILING_SHARED
+		lock_profile_release_lock(&rw->lock_object);
+#endif
 
 		/*
 		 * If there aren't any waiters for a write lock, then try
@@ -404,7 +460,7 @@ _rw_runlock(struct rwlock *rw, const char *file, int line)
 		 * Ok, we know we have a waiting writer and we think we
 		 * are the last reader, so grab the turnstile lock.
 		 */
-		turnstile_lock(&rw->lock_object);
+		turnstile_chain_lock(&rw->lock_object);
 
 		/*
 		 * Try to drop our lock leaving the lock in a unlocked
@@ -424,7 +480,7 @@ _rw_runlock(struct rwlock *rw, const char *file, int line)
 		 */
 		if (!atomic_cmpset_ptr(&rw->rw_lock,
 		    RW_READERS_LOCK(1) | RW_LOCK_WRITE_WAITERS, RW_UNLOCKED)) {
-			turnstile_release(&rw->lock_object);
+			turnstile_chain_unlock(&rw->lock_object);
 			continue;
 		}
 		if (LOCK_LOG_TEST(&rw->lock_object, 0))
@@ -442,9 +498,9 @@ _rw_runlock(struct rwlock *rw, const char *file, int line)
 		MPASS(ts != NULL);
 		turnstile_broadcast(ts, TS_EXCLUSIVE_QUEUE);
 		turnstile_unpend(ts, TS_SHARED_LOCK);
+		turnstile_chain_unlock(&rw->lock_object);
 		break;
 	}
-	lock_profile_release_lock(&rw->lock_object);
 }
 
 /*
@@ -455,10 +511,13 @@ _rw_runlock(struct rwlock *rw, const char *file, int line)
 void
 _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 {
+	struct turnstile *ts;
 #ifdef ADAPTIVE_RWLOCKS
 	volatile struct thread *owner;
 #endif
+	uint64_t waittime = 0;
 	uintptr_t v;
+	int contested = 0;
 
 	if (rw_wlocked(rw)) {
 		KASSERT(rw->lock_object.lo_flags & RW_RECURSE,
@@ -476,7 +535,28 @@ _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 		    rw->lock_object.lo_name, (void *)rw->rw_lock, file, line);
 
 	while (!_rw_write_lock(rw, tid)) {
-		turnstile_lock(&rw->lock_object);
+#ifdef ADAPTIVE_RWLOCKS
+		/*
+		 * If the lock is write locked and the owner is
+		 * running on another CPU, spin until the owner stops
+		 * running or the state of the lock changes.
+		 */
+		v = rw->rw_lock;
+		owner = (struct thread *)RW_OWNER(v);
+		if (!(v & RW_LOCK_READ) && TD_IS_RUNNING(owner)) {
+			if (LOCK_LOG_TEST(&rw->lock_object, 0))
+				CTR3(KTR_LOCK, "%s: spinning on %p held by %p",
+				    __func__, rw, owner);
+			lock_profile_obtain_lock_failed(&rw->lock_object,
+			    &contested, &waittime);
+			while ((struct thread*)RW_OWNER(rw->rw_lock) == owner &&
+			    TD_IS_RUNNING(owner))
+				cpu_spinwait();
+			continue;
+		}
+#endif
+
+		ts = turnstile_trywait(&rw->lock_object);
 		v = rw->rw_lock;
 
 		/*
@@ -484,11 +564,26 @@ _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 		 * turnstile chain lock, try again.
 		 */
 		if (v == RW_UNLOCKED) {
-			turnstile_release(&rw->lock_object);
+			turnstile_cancel(ts);
 			cpu_spinwait();
 			continue;
 		}
 
+#ifdef ADAPTIVE_RWLOCKS
+		/*
+		 * If the current owner of the lock is executing on another
+		 * CPU quit the hard path and try to spin.
+		 */
+		if (!(v & RW_LOCK_READ)) {
+			owner = (struct thread *)RW_OWNER(v);
+			if (TD_IS_RUNNING(owner)) {
+				turnstile_cancel(ts);
+				cpu_spinwait();
+				continue;
+			}
+		}
+#endif
+
 		/*
 		 * If the lock was released by a writer with both readers
 		 * and writers waiting and a reader hasn't woken up and
@@ -503,12 +598,12 @@ _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 			if (atomic_cmpset_acq_ptr(&rw->rw_lock,
 			    RW_UNLOCKED | RW_LOCK_WRITE_WAITERS,
 			    tid | RW_LOCK_WRITE_WAITERS)) {
-				turnstile_claim(&rw->lock_object);
+				turnstile_claim(ts);
 				CTR2(KTR_LOCK, "%s: %p claimed by new writer",
 				    __func__, rw);
 				break;
 			}
-			turnstile_release(&rw->lock_object);
+			turnstile_cancel(ts);
 			cpu_spinwait();
 			continue;
 		}
@@ -521,7 +616,7 @@ _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 		if (!(v & RW_LOCK_WRITE_WAITERS)) {
 			if (!atomic_cmpset_ptr(&rw->rw_lock, v,
 			    v | RW_LOCK_WRITE_WAITERS)) {
-				turnstile_release(&rw->lock_object);
+				turnstile_cancel(ts);
 				cpu_spinwait();
 				continue;
 			}
@@ -530,25 +625,6 @@ _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 				    __func__, rw);
 		}
 
-#ifdef ADAPTIVE_RWLOCKS
-		/*
-		 * If the lock is write locked and the owner is
-		 * running on another CPU, spin until the owner stops
-		 * running or the state of the lock changes.
-		 */
-		owner = (struct thread *)RW_OWNER(v);
-		if (!(v & RW_LOCK_READ) && TD_IS_RUNNING(owner)) {
-			turnstile_release(&rw->lock_object);
-			if (LOCK_LOG_TEST(&rw->lock_object, 0))
-				CTR3(KTR_LOCK, "%s: spinning on %p held by %p",
-				    __func__, rw, owner);
-			while ((struct thread*)RW_OWNER(rw->rw_lock)== owner &&
-			    TD_IS_RUNNING(owner))
-				cpu_spinwait();
-			continue;
-		}
-#endif
-
 		/*
 		 * We were unable to acquire the lock and the write waiters
 		 * flag is set, so we must block on the turnstile.
@@ -556,12 +632,15 @@ _rw_wlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 		if (LOCK_LOG_TEST(&rw->lock_object, 0))
 			CTR2(KTR_LOCK, "%s: %p blocking on turnstile", __func__,
 			    rw);
-		turnstile_wait(&rw->lock_object, rw_owner(rw),
-		    TS_EXCLUSIVE_QUEUE);
+		lock_profile_obtain_lock_failed(&rw->lock_object, &contested,
+		    &waittime);
+		turnstile_wait(ts, rw_owner(rw), TS_EXCLUSIVE_QUEUE);
 		if (LOCK_LOG_TEST(&rw->lock_object, 0))
 			CTR2(KTR_LOCK, "%s: %p resuming from turnstile",
 			    __func__, rw);
 	}
+	lock_profile_obtain_lock_success(&rw->lock_object, contested, waittime,
+	    file, line);
 }
 
 /*
@@ -590,25 +669,10 @@ _rw_wunlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 	if (LOCK_LOG_TEST(&rw->lock_object, 0))
 		CTR2(KTR_LOCK, "%s: %p contested", __func__, rw);
 
-	turnstile_lock(&rw->lock_object);
+	turnstile_chain_lock(&rw->lock_object);
 	ts = turnstile_lookup(&rw->lock_object);
 
-#ifdef ADAPTIVE_RWLOCKS
-	/*
-	 * There might not be a turnstile for this lock if all of
-	 * the waiters are adaptively spinning.  In that case, just
-	 * reset the lock to the unlocked state and return.
-	 */
-	if (ts == NULL) {
-		atomic_store_rel_ptr(&rw->rw_lock, RW_UNLOCKED);
-		if (LOCK_LOG_TEST(&rw->lock_object, 0))
-			CTR2(KTR_LOCK, "%s: %p no sleepers", __func__, rw);
-		turnstile_release(&rw->lock_object);
-		return;
-	}
-#else
 	MPASS(ts != NULL);
-#endif
 
 	/*
 	 * Use the same algo as sx locks for now.  Prefer waking up shared
@@ -625,42 +689,14 @@ _rw_wunlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 	 * above.  There is probably a potential priority inversion in
 	 * there that could be worked around either by waking both queues
 	 * of waiters or doing some complicated lock handoff gymnastics.
-	 *
-	 * Note that in the ADAPTIVE_RWLOCKS case, if both flags are
-	 * set, there might not be any actual writers on the turnstile
-	 * as they might all be spinning.  In that case, we don't want
-	 * to preserve the RW_LOCK_WRITE_WAITERS flag as the turnstile
-	 * is going to go away once we wakeup all the readers.
 	 */
 	v = RW_UNLOCKED;
 	if (rw->rw_lock & RW_LOCK_READ_WAITERS) {
 		queue = TS_SHARED_QUEUE;
-#ifdef ADAPTIVE_RWLOCKS
-		if (rw->rw_lock & RW_LOCK_WRITE_WAITERS &&
-		    !turnstile_empty(ts, TS_EXCLUSIVE_QUEUE))
-			v |= RW_LOCK_WRITE_WAITERS;
-#else
 		v |= (rw->rw_lock & RW_LOCK_WRITE_WAITERS);
-#endif
 	} else
 		queue = TS_EXCLUSIVE_QUEUE;
 
-#ifdef ADAPTIVE_RWLOCKS
-	/*
-	 * We have to make sure that we actually have waiters to
-	 * wakeup.  If they are all spinning, then we just need to
-	 * disown the turnstile and return.
-	 */
-	if (turnstile_empty(ts, queue)) {
-		if (LOCK_LOG_TEST(&rw->lock_object, 0))
-			CTR2(KTR_LOCK, "%s: %p no sleepers 2", __func__, rw);
-		atomic_store_rel_ptr(&rw->rw_lock, v);
-		turnstile_disown(ts);
-		turnstile_release(&rw->lock_object);
-		return;
-	}
-#endif
-
 	/* Wake up all waiters for the specific queue. */
 	if (LOCK_LOG_TEST(&rw->lock_object, 0))
 		CTR3(KTR_LOCK, "%s: %p waking up %s waiters", __func__, rw,
@@ -668,6 +704,7 @@ _rw_wunlock_hard(struct rwlock *rw, uintptr_t tid, const char *file, int line)
 	turnstile_broadcast(ts, queue);
 	atomic_store_rel_ptr(&rw->rw_lock, v);
 	turnstile_unpend(ts, TS_EXCLUSIVE_LOCK);
+	turnstile_chain_unlock(&rw->lock_object);
 }
 
 /*
@@ -679,6 +716,7 @@ int
 _rw_try_upgrade(struct rwlock *rw, const char *file, int line)
 {
 	uintptr_t v, tid;
+	struct turnstile *ts;
 	int success;
 
 	KASSERT(rw->rw_lock != RW_DESTROYED,
@@ -703,28 +741,21 @@ _rw_try_upgrade(struct rwlock *rw, const char *file, int line)
 	 * Ok, we think we have write waiters, so lock the
 	 * turnstile.
 	 */
-	turnstile_lock(&rw->lock_object);
+	ts = turnstile_trywait(&rw->lock_object);
 
 	/*
 	 * Try to switch from one reader to a writer again.  This time
 	 * we honor the current state of the RW_LOCK_WRITE_WAITERS
 	 * flag.  If we obtain the lock with the flag set, then claim
-	 * ownership of the turnstile.  In the ADAPTIVE_RWLOCKS case
-	 * it is possible for there to not be an associated turnstile
-	 * even though there are waiters if all of the waiters are
-	 * spinning.
+	 * ownership of the turnstile.
 	 */
 	v = rw->rw_lock & RW_LOCK_WRITE_WAITERS;
 	success = atomic_cmpset_ptr(&rw->rw_lock, RW_READERS_LOCK(1) | v,
 	    tid | v);
-#ifdef ADAPTIVE_RWLOCKS
-	if (success && v && turnstile_lookup(&rw->lock_object) != NULL)
-#else
 	if (success && v)
-#endif
-		turnstile_claim(&rw->lock_object);
+		turnstile_claim(ts);
 	else
-		turnstile_release(&rw->lock_object);
+		turnstile_cancel(ts);
 out:
 	LOCK_LOG_TRY("WUPGRADE", &rw->lock_object, 0, success, file, line);
 	if (success)
@@ -766,7 +797,7 @@ _rw_downgrade(struct rwlock *rw, const char *file, int line)
 	 * Ok, we think we have waiters, so lock the turnstile so we can
 	 * read the waiter flags without any races.
 	 */
-	turnstile_lock(&rw->lock_object);
+	turnstile_chain_lock(&rw->lock_object);
 	v = rw->rw_lock;
 	MPASS(v & (RW_LOCK_READ_WAITERS | RW_LOCK_WRITE_WAITERS));
 
@@ -774,36 +805,18 @@ _rw_downgrade(struct rwlock *rw, const char *file, int line)
 	 * Downgrade from a write lock while preserving
 	 * RW_LOCK_WRITE_WAITERS and give up ownership of the
 	 * turnstile.  If there are any read waiters, wake them up.
-	 *
-	 * For ADAPTIVE_RWLOCKS, we have to allow for the fact that
-	 * all of the read waiters might be spinning.  In that case,
-	 * act as if RW_LOCK_READ_WAITERS is not set.  Also, only
-	 * preserve the RW_LOCK_WRITE_WAITERS flag if at least one
-	 * writer is blocked on the turnstile.
 	 */
 	ts = turnstile_lookup(&rw->lock_object);
-#ifdef ADAPTIVE_RWLOCKS
-	if (ts == NULL)
-		v &= ~(RW_LOCK_READ_WAITERS | RW_LOCK_WRITE_WAITERS);
-	else if (v & RW_LOCK_READ_WAITERS &&
-	    turnstile_empty(ts, TS_SHARED_QUEUE))
-		v &= ~RW_LOCK_READ_WAITERS;
-	else if (v & RW_LOCK_WRITE_WAITERS &&
-	    turnstile_empty(ts, TS_EXCLUSIVE_QUEUE))
-		v &= ~RW_LOCK_WRITE_WAITERS;
-#else
 	MPASS(ts != NULL);
-#endif
 	if (v & RW_LOCK_READ_WAITERS)
 		turnstile_broadcast(ts, TS_SHARED_QUEUE);
 	atomic_store_rel_ptr(&rw->rw_lock, RW_READERS_LOCK(1) |
 	    (v & RW_LOCK_WRITE_WAITERS));
-	if (v & RW_LOCK_READ_WAITERS) {
+	if (v & RW_LOCK_READ_WAITERS)
 		turnstile_unpend(ts, TS_EXCLUSIVE_LOCK);
-	} else if (ts) {
+	else if (ts)
 		turnstile_disown(ts);
-		turnstile_release(&rw->lock_object);
-	}
+	turnstile_chain_unlock(&rw->lock_object);
 out:
 	LOCK_LOG_LOCK("WDOWNGRADE", &rw->lock_object, 0, 0, file, line);
 }
