@@ -40,7 +40,7 @@
 #include "opt_ddb.h"
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/kern/kern_sx.c,v 1.25.2.9 2007/09/27 20:24:55 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/kern/kern_sx.c,v 1.55 2007/10/02 14:48:48 pjd Exp $");
 
 #include <sys/param.h>
 #include <sys/ktr.h>
@@ -80,7 +80,7 @@ CTASSERT(((SX_ADAPTIVESPIN | SX_RECURSE) & LO_CLASSFLAGS) ==
 
 #define	GIANT_SAVE() do {						\
 	if (mtx_owned(&Giant)) {					\
-		WITNESS_SAVE(&Giant.mtx_object, Giant);		\
+		WITNESS_SAVE(&Giant.lock_object, Giant);		\
 		while (mtx_owned(&Giant)) {				\
 			_giantcnt++;					\
 			mtx_unlock(&Giant);				\
@@ -93,7 +93,7 @@ CTASSERT(((SX_ADAPTIVESPIN | SX_RECURSE) & LO_CLASSFLAGS) ==
 		mtx_assert(&Giant, MA_NOTOWNED);			\
 		while (_giantcnt--)					\
 			mtx_lock(&Giant);				\
-		WITNESS_RESTORE(&Giant.mtx_object, Giant);		\
+		WITNESS_RESTORE(&Giant.lock_object, Giant);		\
 	}								\
 } while (0)
 
@@ -106,6 +106,8 @@ CTASSERT(((SX_ADAPTIVESPIN | SX_RECURSE) & LO_CLASSFLAGS) ==
 #ifdef DDB
 static void	db_show_sx(struct lock_object *lock);
 #endif
+static void	lock_sx(struct lock_object *lock, int how);
+static int	unlock_sx(struct lock_object *lock);
 
 struct lock_class lock_class_sx = {
 	.lc_name = "sx",
@@ -113,12 +115,42 @@ struct lock_class lock_class_sx = {
 #ifdef DDB
 	.lc_ddb_show = db_show_sx,
 #endif
+	.lc_lock = lock_sx,
+	.lc_unlock = unlock_sx,
 };
 
 #ifndef INVARIANTS
 #define	_sx_assert(sx, what, file, line)
 #endif
 
+void
+lock_sx(struct lock_object *lock, int how)
+{
+	struct sx *sx;
+
+	sx = (struct sx *)lock;
+	if (how)
+		sx_xlock(sx);
+	else
+		sx_slock(sx);
+}
+
+int
+unlock_sx(struct lock_object *lock)
+{
+	struct sx *sx;
+
+	sx = (struct sx *)lock;
+	sx_assert(sx, SA_LOCKED | SA_NOTRECURSED);
+	if (sx_xlocked(sx)) {
+		sx_xunlock(sx);
+		return (1);
+	} else {
+		sx_sunlock(sx);
+		return (0);
+	}
+}
+
 void
 sx_sysinit(void *arg)
 {
@@ -127,13 +159,6 @@ sx_sysinit(void *arg)
 	sx_init(sargs->sa_sx, sargs->sa_desc);
 }
 
-void
-sx_init(struct sx *sx, const char *description)
-{
-
-	sx_init_flags(sx, description, 0);
-}
-
 void
 sx_init_flags(struct sx *sx, const char *description, int opts)
 {
@@ -145,6 +170,8 @@ sx_init_flags(struct sx *sx, const char *description, int opts)
 	flags = LO_RECURSABLE | LO_SLEEPABLE | LO_UPGRADABLE;
 	if (opts & SX_DUPOK)
 		flags |= LO_DUPOK;
+	if (opts & SX_NOPROFILE)
+		flags |= LO_NOPROFILE;
 	if (!(opts & SX_NOWITNESS))
 		flags |= LO_WITNESS;
 	if (opts & SX_QUIET)
@@ -190,15 +217,18 @@ _sx_try_slock(struct sx *sx, const char *file, int line)
 {
 	uintptr_t x;
 
-	x = sx->sx_lock;
-	KASSERT(x != SX_LOCK_DESTROYED,
-	    ("sx_try_slock() of destroyed sx @ %s:%d", file, line));
-	if ((x & SX_LOCK_SHARED) && atomic_cmpset_acq_ptr(&sx->sx_lock, x,
-	    x + SX_ONE_SHARER)) {
-		LOCK_LOG_TRY("SLOCK", &sx->lock_object, 0, 1, file, line);
-		WITNESS_LOCK(&sx->lock_object, LOP_TRYLOCK, file, line);
-		curthread->td_locks++;
-		return (1);
+	for (;;) {
+		x = sx->sx_lock;
+		KASSERT(x != SX_LOCK_DESTROYED,
+		    ("sx_try_slock() of destroyed sx @ %s:%d", file, line));
+		if (!(x & SX_LOCK_SHARED))
+			break;
+		if (atomic_cmpset_acq_ptr(&sx->sx_lock, x, x + SX_ONE_SHARER)) {
+			LOCK_LOG_TRY("SLOCK", &sx->lock_object, 0, 1, file, line);
+			WITNESS_LOCK(&sx->lock_object, LOP_TRYLOCK, file, line);
+			curthread->td_locks++;
+			return (1);
+		}
 	}
 
 	LOCK_LOG_TRY("SLOCK", &sx->lock_object, 0, 0, file, line);
@@ -390,9 +420,9 @@ _sx_xlock_hard(struct sx *sx, uintptr_t tid, int opts, const char *file,
 #ifdef ADAPTIVE_SX
 	volatile struct thread *owner;
 #endif
-	/* uint64_t waittime = 0; */
+	uint64_t waittime = 0;
 	uintptr_t x;
-	int /* contested = 0, */error = 0;
+	int contested = 0, error = 0;
 
 	/* If we already hold an exclusive lock, then recurse. */
 	if (sx_xlocked(sx)) {
