@@ -25,7 +25,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/kern/link_elf.c,v 1.81.8.6 2007/06/06 21:43:35 jhb Exp $");
+__FBSDID("$FreeBSD: src/sys/kern/link_elf.c,v 1.93 2007/05/31 11:51:51 kib Exp $");
 
 #include "opt_gdb.h"
 #include "opt_mac.h"
@@ -37,9 +37,9 @@ __FBSDID("$FreeBSD: src/sys/kern/link_elf.c,v 1.81.8.6 2007/06/06 21:43:35 jhb E
 #endif
 #include <sys/kernel.h>
 #include <sys/lock.h>
-#include <sys/mac.h>
 #include <sys/malloc.h>
 #include <sys/mutex.h>
+#include <sys/mount.h>
 #include <sys/proc.h>
 #include <sys/namei.h>
 #include <sys/fcntl.h>
@@ -48,6 +48,8 @@ __FBSDID("$FreeBSD: src/sys/kern/link_elf.c,v 1.81.8.6 2007/06/06 21:43:35 jhb E
 
 #include <machine/elf.h>
 
+#include <security/mac/mac_framework.h>
+
 #include <vm/vm.h>
 #include <vm/vm_param.h>
 #ifdef SPARSE_MAPPING
@@ -304,9 +306,10 @@ link_elf_init(void* arg)
 #endif
 
     (void)link_elf_link_common_finish(linker_kernel_file);
+    linker_kernel_file->flags |= LINKER_FILE_LINKED;
 }
 
-SYSINIT(link_elf, SI_SUB_KLD, SI_ORDER_SECOND, link_elf_init, 0);
+SYSINIT(link_elf, SI_SUB_KLD, SI_ORDER_THIRD, link_elf_init, 0);
 
 static int
 link_elf_preload_parse_symbols(elf_file_t ef)
@@ -556,17 +559,17 @@ link_elf_load_file(linker_class_t cls, const char* filename,
     int symstrindex;
     int symcnt;
     int strcnt;
-
-    GIANT_REQUIRED;
+    int vfslocked;
 
     shdr = NULL;
     lf = NULL;
 
-    NDINIT(&nd, LOOKUP, FOLLOW, UIO_SYSSPACE, filename, td);
+    NDINIT(&nd, LOOKUP, FOLLOW | MPSAFE, UIO_SYSSPACE, filename, td);
     flags = FREAD;
-    error = vn_open(&nd, &flags, 0, -1);
+    error = vn_open(&nd, &flags, 0, NULL);
     if (error)
 	return error;
+    vfslocked = NDHASGIANT(&nd);
     NDFREE(&nd, NDF_ONLY_PNBUF);
 #ifdef MAC
     error = mac_check_kld_load(curthread->td_ucred, nd.ni_vp);
@@ -758,8 +761,10 @@ link_elf_load_file(linker_class_t cls, const char* filename,
 
 #ifdef GPROF
     /* Update profiling information with the new text segment. */
+    mtx_lock(&Giant);
     kmupetext((uintfptr_t)(mapbase + segs[0]->p_vaddr - base_vaddr +
 	segs[0]->p_memsz));
+    mtx_unlock(&Giant);
 #endif
 
     ef->dynamic = (Elf_Dyn *) (mapbase + phdyn->p_vaddr - base_vaddr);
@@ -859,6 +864,7 @@ out:
 	free(firstpage, M_LINKER);
     VOP_UNLOCK(nd.ni_vp, 0, td);
     vn_close(nd.ni_vp, FREAD, td->td_ucred, td);
+    VFS_UNLOCK_GIANT(vfslocked);
 
     return error;
 }
