@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/kern/link_elf_obj.c,v 1.87.2.4 2006/12/20 14:05:41 kib Exp $");
+__FBSDID("$FreeBSD: src/sys/kern/link_elf_obj.c,v 1.95 2007/05/31 11:51:51 kib Exp $");
 
 #include "opt_ddb.h"
 #include "opt_mac.h"
@@ -35,9 +35,9 @@ __FBSDID("$FreeBSD: src/sys/kern/link_elf_obj.c,v 1.87.2.4 2006/12/20 14:05:41 k
 #include <sys/systm.h>
 #include <sys/kernel.h>
 #include <sys/lock.h>
-#include <sys/mac.h>
 #include <sys/malloc.h>
 #include <sys/mutex.h>
+#include <sys/mount.h>
 #include <sys/proc.h>
 #include <sys/namei.h>
 #include <sys/fcntl.h>
@@ -46,6 +46,8 @@ __FBSDID("$FreeBSD: src/sys/kern/link_elf_obj.c,v 1.87.2.4 2006/12/20 14:05:41 k
 
 #include <machine/elf.h>
 
+#include <security/mac/mac_framework.h>
+
 #include <vm/vm.h>
 #include <vm/vm_param.h>
 #include <vm/vm_object.h>
@@ -393,19 +395,19 @@ link_elf_load_file(linker_class_t cls, const char *filename,
 	int nsym;
 	int pb, rl, ra;
 	int alignmask;
-
-	GIANT_REQUIRED;
+	int vfslocked;
 
 	shdr = NULL;
 	lf = NULL;
 	mapsize = 0;
 	hdr = NULL;
 
-	NDINIT(&nd, LOOKUP, FOLLOW, UIO_SYSSPACE, filename, td);
+	NDINIT(&nd, LOOKUP, FOLLOW | MPSAFE, UIO_SYSSPACE, filename, td);
 	flags = FREAD;
-	error = vn_open(&nd, &flags, 0, -1);
+	error = vn_open(&nd, &flags, 0, NULL);
 	if (error)
 		return error;
+	vfslocked = NDHASGIANT(&nd);
 	NDFREE(&nd, NDF_ONLY_PNBUF);
 #ifdef MAC
 	error = mac_check_kld_load(td->td_ucred, nd.ni_vp);
@@ -788,6 +790,7 @@ out:
 		free(hdr, M_LINKER);
 	VOP_UNLOCK(nd.ni_vp, 0, td);
 	vn_close(nd.ni_vp, FREAD, td->td_ucred, td);
+	VFS_UNLOCK_GIANT(vfslocked);
 
 	return error;
 }
