@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/libkern/fnmatch.c,v 1.19.2.1 2006/02/13 23:52:29 rwatson Exp $");
+__FBSDID("$FreeBSD: src/sys/libkern/fnmatch.c,v 1.20 2006/01/22 00:46:40 rwatson Exp $");
 
 /*
  * Function fnmatch() as specified in POSIX 1003.2-1992, section B.6.
