@@ -28,11 +28,15 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/libkern/rindex.c,v 1.11 2005/01/07 00:24:32 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/libkern/rindex.c,v 1.12 2007/04/10 21:42:12 wkoszek Exp $");
 
 #include <sys/param.h>
 #include <sys/libkern.h>
 
+/*
+ * rindex() is also present as the strrchr() in the kernel; it does exactly the
+ * same thing as it's userland equivalent.
+ */
 char *
 rindex(p, ch)
 	const char *p;
