@@ -27,7 +27,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/net/ieee8023ad_lacp.c,v 1.7.2.3 2007/09/15 01:11:26 thompsa Exp $");
+__FBSDID("$FreeBSD: src/sys/net/ieee8023ad_lacp.c,v 1.12 2007/07/05 09:18:57 thompsa Exp $");
 
 #include <sys/param.h>
 #include <sys/callout.h>
@@ -278,6 +278,10 @@ lacp_pdu_input(struct lagg_port *lgp, struct mbuf *m)
 		goto bad;
 	}
 
+	if ((m->m_flags & M_MCAST) == 0) {
+		goto bad;
+	}
+
 	if (m->m_len < sizeof(*du)) {
 		m = m_pullup(m, sizeof(*du));
 		if (m == NULL) {
@@ -552,6 +556,7 @@ lacp_port_create(struct lagg_port *lgp)
 	lp->lp_ifp = ifp;
 	lp->lp_lagg = lgp;
 	lp->lp_lsc = lsc;
+	lp->lp_ifma = rifma;
 
 	LIST_INSERT_HEAD(&lsc->lsc_ports, lp, lp_next);
 
@@ -571,9 +576,7 @@ void
 lacp_port_destroy(struct lagg_port *lgp)
 {
 	struct lacp_port *lp = LACP_PORT(lgp);
-	struct ifnet *ifp = lgp->lp_ifp;
-	struct sockaddr_dl sdl;
-	int i, error;
+	int i;
 
 	LAGG_WLOCK_ASSERT(lgp->lp_softc);
 
@@ -586,18 +589,9 @@ lacp_port_destroy(struct lagg_port *lgp)
 	lacp_unselect(lp);
 	lgp->lp_flags &= ~LAGG_PORT_DISABLED;
 
-	bzero((char *)&sdl, sizeof(sdl));
-	sdl.sdl_len = sizeof(sdl);
-	sdl.sdl_family = AF_LINK;
-	sdl.sdl_index = ifp->if_index;
-	sdl.sdl_type = IFT_ETHER;
-	sdl.sdl_alen = ETHER_ADDR_LEN;
-
-	bcopy(&ethermulticastaddr_slowprotocols,
-	    LLADDR(&sdl), ETHER_ADDR_LEN);
-	error = if_delmulti(ifp, (struct sockaddr *)&sdl);
-	if (error)
-		printf("%s: DELMULTI failed on %s\n", __func__, lgp->lp_ifname);
+	/* The address may have already been removed by if_purgemaddrs() */
+	if (!lgp->lp_detaching)
+		if_delmulti_ifma(lp->lp_ifma);
 
 	LIST_REMOVE(lp, lp_next);
 	free(lp, M_DEVBUF);
@@ -1731,6 +1725,10 @@ lacp_marker_input(struct lagg_port *lgp, struct mbuf *m)
 		goto bad;
 	}
 
+	if ((m->m_flags & M_MCAST) == 0) {
+		goto bad;
+	}
+
 	if (m->m_len < sizeof(*mdu)) {
 		m = m_pullup(m, sizeof(*mdu));
 		if (m == NULL) {
