@@ -1,5 +1,5 @@
 /*	$NetBSD: if_arcsubr.c,v 1.36 2001/06/14 05:44:23 itojun Exp $	*/
-/*	$FreeBSD: src/sys/net/if_arcsubr.c,v 1.25.2.3 2006/06/11 16:31:19 rwatson Exp $ */
+/*	$FreeBSD: src/sys/net/if_arcsubr.c,v 1.30 2006/04/12 07:44:31 rwatson Exp $ */
 
 /*-
  * Copyright (c) 1994, 1995 Ignatios Souvatzis
@@ -549,14 +549,14 @@ arc_input(struct ifnet *ifp, struct mbuf *m)
 #ifdef INET
 	case ARCTYPE_IP:
 		m_adj(m, ARC_HDRNEWLEN);
-		if (ip_fastforward(m))
+		if ((m = ip_fastforward(m)) == NULL)
 			return;
 		isr = NETISR_IP;
 		break;
 
 	case ARCTYPE_IP_OLD:
 		m_adj(m, ARC_HDRLEN);
-		if (ip_fastforward(m))
+		if ((m = ip_fastforward(m)) == NULL)
 			return;
 		isr = NETISR_IP;
 		break;
@@ -635,7 +635,7 @@ arc_ifattach(struct ifnet *ifp, u_int8_t lla)
 #if __FreeBSD_version < 500000
 	ifa = ifnet_addrs[ifp->if_index - 1];
 #else
-	ifa = ifaddr_byindex(ifp->if_index);
+	ifa = ifp->if_addr;
 #endif
 	KASSERT(ifa != NULL, ("%s: no lladdr!\n", __func__));
 	sdl = (struct sockaddr_dl *)ifa->ifa_addr;
