@@ -2,6 +2,7 @@
 
 /*
  * Copyright (c) 2005, 2006 Reyk Floeter <reyk@openbsd.org>
+ * Copyright (c) 2007 Andrew Thompson <thompsa@FreeBSD.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -17,7 +18,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/net/if_lagg.c,v 1.11.2.8.2.2 2007/12/21 05:35:29 thompsa Exp $");
+__FBSDID("$FreeBSD: src/sys/net/if_lagg.c,v 1.19.2.5 2007/12/21 05:33:48 thompsa Exp $");
 
 #include "opt_inet.h"
 #include "opt_inet6.h"
@@ -31,8 +32,10 @@ __FBSDID("$FreeBSD: src/sys/net/if_lagg.c,v 1.11.2.8.2.2 2007/12/21 05:35:29 tho
 #include <sys/sockio.h>
 #include <sys/sysctl.h>
 #include <sys/module.h>
+#include <sys/priv.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
+#include <sys/hash.h>
 #include <sys/lock.h>
 #include <sys/rwlock.h>
 #include <sys/taskqueue.h>
@@ -77,7 +80,7 @@ SLIST_HEAD(__trhead, lagg_softc) lagg_list;	/* list of laggs */
 static struct mtx	lagg_list_mtx;
 eventhandler_tag	lagg_detach_cookie = NULL;
 
-static int	lagg_clone_create(struct if_clone *, int);
+static int	lagg_clone_create(struct if_clone *, int, caddr_t);
 static void	lagg_clone_destroy(struct ifnet *);
 static void	lagg_lladdr(struct lagg_softc *, uint8_t *);
 static void	lagg_capabilities(struct lagg_softc *);
@@ -99,7 +102,6 @@ static void	lagg_stop(struct lagg_softc *);
 static int	lagg_ioctl(struct ifnet *, u_long, caddr_t);
 static int	lagg_ether_setmulti(struct lagg_softc *);
 static int	lagg_ether_cmdmulti(struct lagg_port *, int);
-static void	lagg_ether_purgemulti(struct lagg_softc *);
 static	int	lagg_setflag(struct lagg_port *, int, int,
 		    int (*func)(struct ifnet *, int));
 static	int	lagg_setflags(struct lagg_port *, int status);
@@ -157,24 +159,6 @@ static const struct {
 	{ LAGG_PROTO_NONE,		NULL }
 };
 
-/*
- * Return a 32-bit hash of the given buffer.
- * XXX Taken from sys/sys/hash.h. This file can not be included directly due to
- * compiler warnings that need an API change to fix.
- */
-#define	HASHSTEP(x,c)	(((x << 5) + x) + (c))
-static __inline uint32_t
-hash32_buf(const void *buf, size_t len, uint32_t hash)
-{
-	const unsigned char *p = buf;
-
-	while (len--)
-		hash = HASHSTEP(hash, *p++);
-
-	return hash;
-}
-#undef HASHSTEP
-
 static int
 lagg_modevent(module_t mod, int type, void *data)
 {
@@ -194,8 +178,6 @@ lagg_modevent(module_t mod, int type, void *data)
 		EVENTHANDLER_DEREGISTER(ifnet_departure_event,
 		    lagg_detach_cookie);
 		if_clone_detach(&lagg_cloner);
-		while (!SLIST_EMPTY(&lagg_list))
-			lagg_clone_destroy(SLIST_FIRST(&lagg_list)->sc_ifp);
 		lagg_input_p = NULL;
 		lagg_linkstate_p = NULL;
 		mtx_destroy(&lagg_list_mtx);
@@ -215,7 +197,7 @@ static moduledata_t lagg_mod = {
 DECLARE_MODULE(if_lagg, lagg_mod, SI_SUB_PSEUDO, SI_ORDER_ANY);
 
 static int
-lagg_clone_create(struct if_clone *ifc, int unit)
+lagg_clone_create(struct if_clone *ifc, int unit, caddr_t params)
 {
 	struct lagg_softc *sc;
 	struct ifnet *ifp;
@@ -294,9 +276,6 @@ lagg_clone_destroy(struct ifnet *ifp)
 	/* Unhook the aggregation protocol */
 	(*sc->sc_detach)(sc);
 
-	/* Remove any multicast groups that we may have joined. */
-	lagg_ether_purgemulti(sc);
-
 	LAGG_WUNLOCK(sc);
 
 	ifmedia_removeall(&sc->sc_media);
@@ -320,7 +299,6 @@ lagg_lladdr(struct lagg_softc *sc, uint8_t *lladdr)
 	if (memcmp(lladdr, IF_LLADDR(ifp), ETHER_ADDR_LEN) == 0)
 		return;
 
-	bcopy(lladdr, IFP2ENADDR(ifp), ETHER_ADDR_LEN);
 	bcopy(lladdr, IF_LLADDR(ifp), ETHER_ADDR_LEN);
 	/* Let the protocol know the MAC has changed */
 	if (sc->sc_lladdr != NULL)
@@ -865,7 +843,7 @@ lagg_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 		free(outbuf, M_TEMP);
 		break;
 	case SIOCSLAGG:
-		error = suser(td);
+		error = priv_check(td, PRIV_NET_LAGG);
 		if (error)
 			break;
 		if (ra->ra_proto >= LAGG_PROTO_MAX) {
@@ -928,7 +906,7 @@ lagg_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 		LAGG_RUNLOCK(sc);
 		break;
 	case SIOCSLAGGPORT:
-		error = suser(td);
+		error = priv_check(td, PRIV_NET_LAGG);
 		if (error)
 			break;
 		if (rp->rp_portname[0] == '\0' ||
@@ -941,7 +919,7 @@ lagg_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 		LAGG_WUNLOCK(sc);
 		break;
 	case SIOCSLAGGDELPORT:
-		error = suser(td);
+		error = priv_check(td, PRIV_NET_LAGG);
 		if (error)
 			break;
 		if (rp->rp_portname[0] == '\0' ||
@@ -1014,46 +992,15 @@ lagg_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
 static int
 lagg_ether_setmulti(struct lagg_softc *sc)
 {
-	struct ifnet		*scifp = sc->sc_ifp;
-	struct ifnet		*ifp;
-	struct ifmultiaddr	*ifma, *rifma = NULL;
-	struct lagg_port	*lp;
-	struct lagg_mc		*mc;
-	struct sockaddr_dl	sdl;
-	int			error;
+	struct lagg_port *lp;
 
 	LAGG_WLOCK_ASSERT(sc);
 
-	bzero((char *)&sdl, sizeof(sdl));
-	sdl.sdl_len = sizeof(sdl);
-	sdl.sdl_family = AF_LINK;
-	sdl.sdl_type = IFT_ETHER;
-	sdl.sdl_alen = ETHER_ADDR_LEN;
-
-	/* First, remove any existing filter entries. */
-	lagg_ether_purgemulti(sc);
-
-	/* Now program new ones. */
-	TAILQ_FOREACH(ifma, &scifp->if_multiaddrs, ifma_link) {
-		if (ifma->ifma_addr->sa_family != AF_LINK)
-			continue;
-		mc = malloc(sizeof(struct lagg_mc), M_DEVBUF, M_NOWAIT);
-		if (mc == NULL)
-			return (ENOMEM);
-		bcopy(LLADDR((struct sockaddr_dl *)ifma->ifma_addr),
-		    (char *)&mc->mc_addr, ETHER_ADDR_LEN);
-		SLIST_INSERT_HEAD(&sc->sc_mc_head, mc, mc_entries);
-		bcopy(LLADDR((struct sockaddr_dl *)ifma->ifma_addr),
-		    LLADDR(&sdl), ETHER_ADDR_LEN);
-
-		/* do all the ports */
-		SLIST_FOREACH(lp, &sc->sc_ports, lp_entries) {
-			ifp = lp->lp_ifp;
-			sdl.sdl_index = ifp->if_index;
-			error = if_addmulti(ifp, (struct sockaddr *)&sdl, &rifma);
-			if (error)
-				return (error);
-		}
+	SLIST_FOREACH(lp, &sc->sc_ports, lp_entries) {
+		/* First, remove any existing filter entries. */
+		lagg_ether_cmdmulti(lp, 0);
+		/* copy all addresses from the lagg interface to the port */
+		lagg_ether_cmdmulti(lp, 1);
 	}
 	return (0);
 }
@@ -1062,11 +1009,12 @@ static int
 lagg_ether_cmdmulti(struct lagg_port *lp, int set)
 {
 	struct lagg_softc *sc = lp->lp_softc;
-	struct ifnet *ifp = lp->lp_ifp;;
-	struct lagg_mc		*mc;
-	struct ifmultiaddr	*rifma = NULL;
-	struct sockaddr_dl	sdl;
-	int			error;
+	struct ifnet *ifp = lp->lp_ifp;
+	struct ifnet *scifp = sc->sc_ifp;
+	struct lagg_mc *mc;
+	struct ifmultiaddr *ifma, *rifma = NULL;
+	struct sockaddr_dl sdl;
+	int error;
 
 	LAGG_WLOCK_ASSERT(sc);
 
@@ -1077,41 +1025,32 @@ lagg_ether_cmdmulti(struct lagg_port *lp, int set)
 	sdl.sdl_alen = ETHER_ADDR_LEN;
 	sdl.sdl_index = ifp->if_index;
 
-	SLIST_FOREACH(mc, &sc->sc_mc_head, mc_entries) {
-		bcopy((char *)&mc->mc_addr, LLADDR(&sdl), ETHER_ADDR_LEN);
+	if (set) {
+		TAILQ_FOREACH(ifma, &scifp->if_multiaddrs, ifma_link) {
+			if (ifma->ifma_addr->sa_family != AF_LINK)
+				continue;
+			bcopy(LLADDR((struct sockaddr_dl *)ifma->ifma_addr),
+			    LLADDR(&sdl), ETHER_ADDR_LEN);
 
-		if (set)
 			error = if_addmulti(ifp, (struct sockaddr *)&sdl, &rifma);
-		else
-			error = if_delmulti(ifp, (struct sockaddr *)&sdl);
-
-		if (error) {
-			printf("cmdmulti error on %s, set = %d\n",
-			    ifp->if_xname, set);
-			return (error);
+			if (error)
+				return (error);
+			mc = malloc(sizeof(struct lagg_mc), M_DEVBUF, M_NOWAIT);
+			if (mc == NULL)
+				return (ENOMEM);
+			mc->mc_ifma = rifma;
+			SLIST_INSERT_HEAD(&lp->lp_mc_head, mc, mc_entries);
+		}
+	} else {
+		while ((mc = SLIST_FIRST(&lp->lp_mc_head)) != NULL) {
+			SLIST_REMOVE(&lp->lp_mc_head, mc, lagg_mc, mc_entries);
+			if_delmulti_ifma(mc->mc_ifma);
+			free(mc, M_DEVBUF);
 		}
 	}
 	return (0);
 }
 
-static void
-lagg_ether_purgemulti(struct lagg_softc *sc)
-{
-	struct lagg_port *lp;
-	struct lagg_mc *mc;
-
-	LAGG_WLOCK_ASSERT(sc);
-
-	/* remove from ports */
-	SLIST_FOREACH(lp, &sc->sc_ports, lp_entries)
-		lagg_ether_cmdmulti(lp, 0);
-
-	while ((mc = SLIST_FIRST(&sc->sc_mc_head)) != NULL) {
-		SLIST_REMOVE(&sc->sc_mc_head, mc, lagg_mc, mc_entries);
-		free(mc, M_DEVBUF);
-	}
-}
-
 /* Handle a ref counted flag that should be set on the lagg port as well */
 static int
 lagg_setflag(struct lagg_port *lp, int flag, int status,
@@ -1178,7 +1117,7 @@ lagg_start(struct ifnet *ifp)
 		if (m == NULL)
 			break;
 
-		BPF_MTAP(ifp, m);
+		ETHER_BPF_MTAP(ifp, m);
 
 		if (sc->sc_proto != LAGG_PROTO_NONE)
 			error = (*sc->sc_start)(sc, m);
@@ -1210,13 +1149,18 @@ lagg_input(struct ifnet *ifp, struct mbuf *m)
 	}
 
 	LAGG_RLOCK(sc);
-	BPF_MTAP(scifp, m);
+	ETHER_BPF_MTAP(scifp, m);
 
 	m = (*sc->sc_input)(sc, lp, m);
 
 	if (m != NULL) {
 		scifp->if_ipackets++;
 		scifp->if_ibytes += m->m_pkthdr.len;
+
+		if (scifp->if_flags & IFF_MONITOR) {
+			m_freem(m);
+			m = NULL;
+		}
 	}
 
 	LAGG_RUNLOCK(sc);
@@ -1358,8 +1302,6 @@ lagg_hashmbuf(struct mbuf *m, uint32_t key)
 	struct ether_header *eh;
 	struct ether_vlan_header vlanbuf;
 	const struct ether_vlan_header *vlan;
-	struct m_tag *mtag;
-	u_int tag;
 #ifdef INET
 	const struct ip *ip;
 	struct ip ipbuf;
@@ -1380,11 +1322,8 @@ lagg_hashmbuf(struct mbuf *m, uint32_t key)
 
 	/* Special handling for encapsulating VLAN frames */
 	if (m->m_flags & M_VLANTAG) {
-		mtag = m_tag_locate(m, MTAG_VLAN, MTAG_VLAN_TAG, NULL);
-		KASSERT(mtag != NULL,
-			("%s: M_VLANTAG without m_tag", __func__));
-		tag = EVL_VLANOFTAG(VLAN_TAG_VALUE(mtag));
-		p = hash32_buf(&tag, sizeof(tag), p);
+		p = hash32_buf(&m->m_pkthdr.ether_vtag,
+		    sizeof(m->m_pkthdr.ether_vtag), p);
 	} else if (etype == ETHERTYPE_VLAN) {
 		vlan = lagg_gethdr(m, off,  sizeof(*vlan), &vlanbuf);
 		if (vlan == NULL)
