@@ -1,5 +1,5 @@
 /*	$NetBSD: if_media.h,v 1.3 1997/03/26 01:19:27 thorpej Exp $	*/
-/* $FreeBSD: src/sys/net/if_media.h,v 1.30.2.5 2007/06/14 20:02:41 davidch Exp $ */
+/* $FreeBSD: src/sys/net/if_media.h,v 1.40 2007/09/18 20:30:40 sam Exp $ */
 
 /*-
  * Copyright (c) 1997
@@ -144,7 +144,7 @@ uint64_t	ifmedia_baudrate(int);
 #define	IFM_10G_LR	18		/* 10GBase-LR 1310nm Single-mode */
 #define	IFM_10G_SR	19		/* 10GBase-SR 850nm Multi-mode */
 #define	IFM_10G_CX4	20		/* 10GBase CX4 copper */
-#define IFM_2500_SX	21		/* 2500baseSX - multi-mode fiber */
+#define IFM_2500_SX	21		/* 2500BaseSX - multi-mode fiber */
 
 /* note 31 is the max! */
 
@@ -202,6 +202,8 @@ uint64_t	ifmedia_baudrate(int);
 #define	IFM_IEEE80211_OFDM3	21	/* OFDM 3Mbps */
 #define	IFM_IEEE80211_OFDM4	22	/* OFDM 4.5Mbps */
 #define	IFM_IEEE80211_OFDM27	23	/* OFDM 27Mbps */
+/* NB: not enough bits to express MCS fully */
+#define	IFM_IEEE80211_MCS	24	/* HT MCS rate */
 
 #define	IFM_IEEE80211_ADHOC	0x00000100	/* Operate in Adhoc mode */
 #define	IFM_IEEE80211_HOSTAP	0x00000200	/* Operate in Host AP mode */
@@ -215,6 +217,8 @@ uint64_t	ifmedia_baudrate(int);
 #define	IFM_IEEE80211_11B	0x00020000	/* Direct Sequence mode */
 #define	IFM_IEEE80211_11G	0x00030000	/* 2Ghz, CCK mode */
 #define	IFM_IEEE80211_FH	0x00040000	/* 2Ghz, GFSK mode */
+#define	IFM_IEEE80211_11NA	0x00050000	/* 5Ghz, HT mode */
+#define	IFM_IEEE80211_11NG	0x00060000	/* 2Ghz, HT mode */
 
 /*
  * ATM
@@ -345,7 +349,7 @@ struct ifmedia_description {
 	{ IFM_10G_LR,	"10Gbase-LR" },					\
 	{ IFM_10G_SR,	"10Gbase-SR" },					\
 	{ IFM_10G_CX4,	"10Gbase-CX4" },				\
-	{ IFM_2500_SX,	"2500baseSX" },					\
+	{ IFM_2500_SX,	"2500BaseSX" },					\
 	{ 0, NULL },							\
 }
 
@@ -503,6 +507,8 @@ struct ifmedia_description {
 	{ IFM_IEEE80211_11B, "11b" },					\
 	{ IFM_IEEE80211_11G, "11g" },					\
 	{ IFM_IEEE80211_FH, "fh" },					\
+	{ IFM_IEEE80211_11NA, "11na" },					\
+	{ IFM_IEEE80211_11NG, "11ng" },					\
 	{ 0, NULL },							\
 }
 
