@@ -31,7 +31,7 @@
  */
 
 /*
- * $FreeBSD: src/sys/net/if_tap.c,v 1.55.2.11 2007/07/29 13:47:13 hm Exp $
+ * $FreeBSD: src/sys/net/if_tap.c,v 1.71 2007/03/19 18:17:31 bms Exp $
  * $Id: if_tap.c,v 0.21 2000/07/23 21:46:02 max Exp $
  */
 
@@ -47,6 +47,7 @@
 #include <sys/mbuf.h>
 #include <sys/module.h>
 #include <sys/poll.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/selinfo.h>
 #include <sys/signalvar.h>
@@ -61,7 +62,6 @@
 #include <net/bpf.h>
 #include <net/ethernet.h>
 #include <net/if.h>
-#include <net/if_arp.h>
 #include <net/if_clone.h>
 #include <net/if_dl.h>
 #include <net/route.h>
@@ -94,9 +94,9 @@ static void		tapifstart(struct ifnet *);
 static int		tapifioctl(struct ifnet *, u_long, caddr_t);
 static void		tapifinit(void *);
 
-static int		tap_clone_create(struct if_clone *, int);
+static int		tap_clone_create(struct if_clone *, int, caddr_t);
 static void		tap_clone_destroy(struct ifnet *);
-static int		vmnet_clone_create(struct if_clone *, int);
+static int		vmnet_clone_create(struct if_clone *, int, caddr_t);
 static void		vmnet_clone_destroy(struct ifnet *);
 
 IFC_SIMPLE_DECLARE(tap, 0);
@@ -176,7 +176,7 @@ TUNABLE_INT("net.link.tap.devfs_cloning", &tapdclone);
 DEV_MODULE(if_tap, tapmodevent, NULL);
 
 static int
-tap_clone_create(struct if_clone *ifc, int unit)
+tap_clone_create(struct if_clone *ifc, int unit, caddr_t params)
 {
 	struct cdev *dev;
 	int i;
@@ -204,9 +204,9 @@ tap_clone_create(struct if_clone *ifc, int unit)
 
 /* vmnet devices are tap devices in disguise */
 static int
-vmnet_clone_create(struct if_clone *ifc, int unit)
+vmnet_clone_create(struct if_clone *ifc, int unit, caddr_t params)
 {
-	return tap_clone_create(ifc, unit);
+	return tap_clone_create(ifc, unit, params);
 }
 
 static void
@@ -344,7 +344,7 @@ tapclone(void *arg, struct ucred *cred, char *name, int namelen, struct cdev **d
 		return;
 
 	if (!tapdclone ||
-	    (!tapuopen && suser_cred(cred, SUSER_ALLOWJAIL) != 0))
+	    (!tapuopen && priv_check_cred(cred, PRIV_NET_IFCREATE, 0) != 0))
 		return;
 
 	unit = 0;
@@ -389,7 +389,7 @@ tapclone(void *arg, struct ucred *cred, char *name, int namelen, struct cdev **d
 		}
 	}
 
-	if_clone_create(name, namelen);
+	if_clone_create(name, namelen, NULL);
 } /* tapclone */
 
 
@@ -480,8 +480,8 @@ tapopen(struct cdev *dev, int flag, int mode, struct thread *td)
 	int			 error, s;
 
 	if (tapuopen == 0) {
-		error = suser(td);
-		if (error != 0)
+		error = priv_check(td, PRIV_NET_TAP);
+		if (error)
 			return (error);
 	}
 
@@ -496,7 +496,7 @@ tapopen(struct cdev *dev, int flag, int mode, struct thread *td)
 		return (EBUSY);
 	}
 
-	bcopy(IFP2ENADDR(tp->tap_ifp), tp->ether_addr, sizeof(tp->ether_addr));
+	bcopy(IF_LLADDR(tp->tap_ifp), tp->ether_addr, sizeof(tp->ether_addr));
 	tp->tap_pid = td->td_proc->p_pid;
 	tp->tap_flags |= TAP_OPEN;
 	ifp = tp->tap_ifp;
@@ -927,7 +927,8 @@ tapwrite(struct cdev *dev, struct uio *uio, int flag)
 		return (EIO);
 	}
 
-	if ((m = m_uiotombuf(uio, M_DONTWAIT, 0, ETHER_ALIGN)) == NULL) {
+	if ((m = m_uiotombuf(uio, M_DONTWAIT, 0, ETHER_ALIGN,
+	    M_PKTHDR)) == NULL) {
 		ifp->if_ierrors ++;
 		return (ENOBUFS);
 	}
@@ -946,7 +947,7 @@ tapwrite(struct cdev *dev, struct uio *uio, int flag)
 
 	if (eh && (ifp->if_flags & IFF_PROMISC) == 0 &&
 	    !ETHER_IS_MULTICAST(eh->ether_dhost) &&
-	    bcmp(eh->ether_dhost, IFP2ENADDR(ifp), ETHER_ADDR_LEN) != 0) {
+	    bcmp(eh->ether_dhost, IF_LLADDR(ifp), ETHER_ADDR_LEN) != 0) {
 		m_freem(m);
 		return (0);
 	}
