@@ -13,7 +13,7 @@
  * UCL. This driver is based much more on read/write/poll mode of
  * operation though.
  *
- * $FreeBSD: src/sys/net/if_tun.c,v 1.152.2.6 2007/03/04 14:34:41 bms Exp $
+ * $FreeBSD: src/sys/net/if_tun.c,v 1.163 2007/02/05 11:15:52 bms Exp $
  */
 
 #include "opt_atalk.h"
@@ -23,9 +23,9 @@
 #include "opt_mac.h"
 
 #include <sys/param.h>
+#include <sys/priv.h>
 #include <sys/proc.h>
 #include <sys/systm.h>
-#include <sys/mac.h>
 #include <sys/mbuf.h>
 #include <sys/module.h>
 #include <sys/socket.h>
@@ -57,6 +57,8 @@
 
 #include <sys/queue.h>
 
+#include <security/mac/mac_framework.h>
+
 /*
  * tun_list is protected by global tunmtx.  Other mutable fields are
  * protected by tun->tun_mtx, or by their owning subsystem.  tun_dev is
@@ -127,7 +129,7 @@ static int	tunoutput(struct ifnet *, struct mbuf *, struct sockaddr *,
 		    struct rtentry *rt);
 static void	tunstart(struct ifnet *);
 
-static int	tun_clone_create(struct if_clone *, int);
+static int	tun_clone_create(struct if_clone *, int, caddr_t);
 static void	tun_clone_destroy(struct ifnet *);
 
 IFC_SIMPLE_DECLARE(tun, 0);
@@ -172,7 +174,7 @@ static struct cdevsw tun_cdevsw = {
 };
 
 static int
-tun_clone_create(struct if_clone *ifc, int unit)
+tun_clone_create(struct if_clone *ifc, int unit, caddr_t params)
 {
 	struct cdev *dev;
 	int i;
@@ -207,7 +209,7 @@ tunclone(void *arg, struct ucred *cred, char *name, int namelen,
 	 * If tun cloning is enabled, only the superuser can create an
 	 * interface.
 	 */
-	if (!tundclone || suser_cred(cred, SUSER_ALLOWJAIL) != 0)
+	if (!tundclone || priv_check_cred(cred, PRIV_NET_IFCREATE, 0) != 0)
 		return;
 
 	if (strcmp(name, TUNNAME) == 0) {
@@ -239,7 +241,7 @@ tunclone(void *arg, struct ucred *cred, char *name, int namelen,
 		}
 	}
 
-	if_clone_create(name, namelen);
+	if_clone_create(name, namelen, NULL);
 }
 
 static void
@@ -672,8 +674,8 @@ tunioctl(struct cdev *dev, u_long cmd, caddr_t data, int flag, struct thread *td
 		if (tunp->mtu < IF_MINMTU)
 			return (EINVAL);
 		if (TUN2IFP(tp)->if_mtu != tunp->mtu) {
-			error = suser(td);
-			if (error != 0)
+			error = priv_check(td, PRIV_NET_SETIFMTU);
+			if (error)
 				return (error);
 		}
 		TUN2IFP(tp)->if_mtu = tunp->mtu;
@@ -866,7 +868,7 @@ tunwrite(struct cdev *dev, struct uio *uio, int flag)
 		return (EIO);
 	}
 
-	if ((m = m_uiotombuf(uio, M_DONTWAIT, 0, 0)) == NULL) {
+	if ((m = m_uiotombuf(uio, M_DONTWAIT, 0, 0, M_PKTHDR)) == NULL) {
 		ifp->if_ierrors++;
 		return (error);
 	}
