@@ -23,7 +23,7 @@
  *	+1-313-764-2278
  *	netatalk@umich.edu
  *
- * $FreeBSD: src/sys/netatalk/at_control.c,v 1.42.2.1 2005/01/31 23:26:25 imp Exp $
+ * $FreeBSD: src/sys/netatalk/at_control.c,v 1.46 2007/02/19 22:40:02 rwatson Exp $
  */
 
 #include <sys/param.h>
@@ -31,6 +31,7 @@
 #include <sys/sockio.h>
 #include <sys/malloc.h>
 #include <sys/kernel.h>
+#include <sys/priv.h>
 #include <sys/socket.h>
 #include <net/if.h>
 #include <net/route.h>
@@ -118,8 +119,10 @@ at_control(struct socket *so, u_long cmd, caddr_t data, struct ifnet *ifp,
 	case SIOCSIFADDR:
 		/* 
 		 * If we are not superuser, then we don't get to do these ops.
+		 *
+		 * XXXRW: Layering?
 		 */
-		if (suser(td))
+		if (priv_check(td, PRIV_NET_ADDIFADDR))
 			return (EPERM);
 
 		sat = satosat(&ifr->ifr_addr);
@@ -158,6 +161,7 @@ at_control(struct socket *so, u_long cmd, caddr_t data, struct ifnet *ifp,
 		if (aa == NULL) {
 			aa0 = malloc(sizeof(struct at_ifaddr), M_IFADDR,
 			    M_WAITOK | M_ZERO);
+			callout_init(&aa0->aa_callout, CALLOUT_MPSAFE);
 			if ((aa = at_ifaddr_list) != NULL) {
 				/*
 				 * Don't let the loopback be first, since the
@@ -517,10 +521,12 @@ at_ifinit(struct ifnet *ifp, struct at_ifaddr *aa, struct sockaddr_at *sat)
 				 * start off the probes as an asynchronous
 				 * activity.  though why wait 200mSec?
 				 */
-				aa->aa_ch = timeout(aarpprobe, (caddr_t)ifp,
-				    hz / 5);
-				if (tsleep(aa, PPAUSE|PCATCH, "at_ifinit",
-				    0)) {
+				AARPTAB_LOCK();
+				callout_reset(&aa->aa_callout, hz / 5,
+				    aarpprobe, ifp);
+				if (msleep(aa, &aarptab_mtx, PPAUSE|PCATCH,
+				    "at_ifinit", 0)) {
+					AARPTAB_UNLOCK();
 					/*
 					 * theoretically we shouldn't time
 					 * out here so if we returned with an
@@ -533,6 +539,7 @@ at_ifinit(struct ifnet *ifp, struct at_ifaddr *aa, struct sockaddr_at *sat)
 					aa->aa_lastnet = onr.nr_lastnet;
 					return (EINTR);
 				}
+				AARPTAB_UNLOCK();
 
 				/* 
 				 * The async activity should have woken us
