@@ -23,7 +23,7 @@
  * Copies of this Software may be made, however, the above copyright
  * notice must be reproduced on all copies.
  *
- *	@(#) $FreeBSD: src/sys/netatm/atm.h,v 1.5.8.1 2005/01/31 23:26:25 imp Exp $
+ *	@(#) $FreeBSD: src/sys/netatm/atm.h,v 1.6 2005/01/07 01:45:36 imp Exp $
  *
  */
 
