@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/netatm/atm_if.c,v 1.30.2.1 2005/08/25 05:01:21 rwatson Exp $");
+__FBSDID("$FreeBSD: src/sys/netatm/atm_if.c,v 1.33 2007/04/03 12:45:10 rwatson Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -169,7 +169,7 @@ atm_physif_deregister(cup)
 	Cmn_unit	*cup;
 {
 	struct atm_pif	*pip = (struct atm_pif *)&cup->cu_pif;
-	Cmn_vcc		*cvp;
+	Cmn_vcc		*cvp, *cvp_next;
 	int	err;
 	int	s = splnet();
 
@@ -215,8 +215,9 @@ atm_physif_deregister(cup)
 	 */
 	cvp = cup->cu_vcc;
 	while (cvp) {
+		cvp_next = cvp->cv_next;
 		uma_zfree(cup->cu_vcc_zone, cvp);
-		cvp = cvp->cv_next;
+		cvp = cvp_next;
 	}
 	cup->cu_vcc = (Cmn_vcc *)NULL;
 
@@ -561,7 +562,7 @@ atm_physif_ioctl(code, data, arg)
 			 * Set macaddr in <Link> address
 			 */
 			ifp->if_addrlen = 6;
-			ifa = ifaddr_byindex(ifp->if_index);
+			ifa = ifp->if_addr;
 			if ( ifa ) {
 				sdl = (struct sockaddr_dl *)
 					ifa->ifa_addr;
