@@ -32,7 +32,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/netatm/sigpvc/sigpvc_if.c,v 1.17.4.1 2005/01/31 23:26:26 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/netatm/sigpvc/sigpvc_if.c,v 1.18 2005/01/07 01:45:38 imp Exp $");
 
 #ifndef ATM_SIGPVC_MODULE
 #include "opt_atm.h"
