@@ -31,7 +31,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/netatm/uni/unisig_subr.c,v 1.16.2.1 2005/01/31 23:26:27 imp Exp $");
+__FBSDID("$FreeBSD: src/sys/netatm/uni/unisig_subr.c,v 1.17 2005/01/07 01:45:37 imp Exp $");
 
 #include <sys/param.h>
 #include <sys/systm.h>
