@@ -28,7 +28,7 @@
  * SUCH DAMAGE.
  *
  * $Id: ng_btsocket_rfcomm.h,v 1.10 2003/03/29 22:27:42 max Exp $
- * $FreeBSD: src/sys/netgraph/bluetooth/include/ng_btsocket_rfcomm.h,v 1.4 2005/01/11 01:39:53 emax Exp $
+ * $FreeBSD: src/sys/netgraph/bluetooth/include/ng_btsocket_rfcomm.h,v 1.8 2006/07/21 17:11:13 rwatson Exp $
  */
 
 #ifndef _NETGRAPH_BTSOCKET_RFCOMM_H_
@@ -314,7 +314,8 @@ typedef struct ng_btsocket_rfcomm_pcb *	ng_btsocket_rfcomm_pcb_p;
 #ifdef _KERNEL
 
 void ng_btsocket_rfcomm_init       (void);
-int  ng_btsocket_rfcomm_abort      (struct socket *);
+void ng_btsocket_rfcomm_abort      (struct socket *);
+void ng_btsocket_rfcomm_close      (struct socket *);
 int  ng_btsocket_rfcomm_accept     (struct socket *, struct sockaddr **);
 int  ng_btsocket_rfcomm_attach     (struct socket *, int, struct thread *);
 int  ng_btsocket_rfcomm_bind       (struct socket *, struct sockaddr *,
@@ -324,9 +325,9 @@ int  ng_btsocket_rfcomm_connect    (struct socket *, struct sockaddr *,
 int  ng_btsocket_rfcomm_control    (struct socket *, u_long, caddr_t,
                                     struct ifnet *, struct thread *);
 int  ng_btsocket_rfcomm_ctloutput  (struct socket *, struct sockopt *);
-int  ng_btsocket_rfcomm_detach     (struct socket *);
+void ng_btsocket_rfcomm_detach     (struct socket *);
 int  ng_btsocket_rfcomm_disconnect (struct socket *);
-int  ng_btsocket_rfcomm_listen     (struct socket *, struct thread *);
+int  ng_btsocket_rfcomm_listen     (struct socket *, int, struct thread *);
 int  ng_btsocket_rfcomm_peeraddr   (struct socket *, struct sockaddr **);
 int  ng_btsocket_rfcomm_send       (struct socket *, int, struct mbuf *,
                                     struct sockaddr *, struct mbuf *,
