@@ -28,7 +28,7 @@
  * SUCH DAMAGE.
  *
  * $Id: ng_btsocket_rfcomm.c,v 1.28 2003/09/14 23:29:06 max Exp $
- * $FreeBSD: src/sys/netgraph/bluetooth/socket/ng_btsocket_rfcomm.c,v 1.17.2.3 2007/11/07 17:42:45 emax Exp $
+ * $FreeBSD: src/sys/netgraph/bluetooth/socket/ng_btsocket_rfcomm.c,v 1.26.2.1 2007/11/07 17:37:18 emax Exp $
  */
 
 #include <sys/param.h>
@@ -165,7 +165,7 @@ static int ng_btsocket_rfcomm_send_credits
 
 static int ng_btsocket_rfcomm_pcb_send
 	(ng_btsocket_rfcomm_pcb_p pcb, int limit);
-static int ng_btsocket_rfcomm_pcb_kill
+static void ng_btsocket_rfcomm_pcb_kill
 	(ng_btsocket_rfcomm_pcb_p pcb, int error);
 static ng_btsocket_rfcomm_pcb_p ng_btsocket_rfcomm_pcb_by_dlci
 	(ng_btsocket_rfcomm_session_p s, int dlci);
@@ -341,14 +341,21 @@ ng_btsocket_rfcomm_init(void)
  * Abort connection on socket
  */
 
-int
+void
 ng_btsocket_rfcomm_abort(struct socket *so)
 {
-	so->so_error = ECONNABORTED;
 
-	return (ng_btsocket_rfcomm_detach(so));
+	so->so_error = ECONNABORTED;
+	(void)ng_btsocket_rfcomm_disconnect(so);
 } /* ng_btsocket_rfcomm_abort */
 
+void
+ng_btsocket_rfcomm_close(struct socket *so)
+{
+
+	(void)ng_btsocket_rfcomm_disconnect(so);
+} /* ng_btsocket_rfcomm_close */
+
 /*
  * Accept connection on socket. Nothing to do here, socket must be connected
  * and ready, so just return peer address and be done with it.
@@ -690,13 +697,12 @@ ng_btsocket_rfcomm_ctloutput(struct socket *so, struct sockopt *sopt)
  * Detach and destroy socket
  */
 
-int
+void
 ng_btsocket_rfcomm_detach(struct socket *so)
 {
 	ng_btsocket_rfcomm_pcb_p	pcb = so2rfcomm_pcb(so);
 
-	if (pcb == NULL)
-		return (EINVAL);
+	KASSERT(pcb != NULL, ("ng_btsocket_rfcomm_detach: pcb == NULL"));
 
 	mtx_lock(&pcb->pcb_mtx);
 
@@ -742,12 +748,7 @@ ng_btsocket_rfcomm_detach(struct socket *so)
 	FREE(pcb, M_NETGRAPH_BTSOCKET_RFCOMM);
 
 	soisdisconnected(so);
-	ACCEPT_LOCK();
-	SOCK_LOCK(so);
 	so->so_pcb = NULL;
-	sotryfree(so);
-
-	return (0);
 } /* ng_btsocket_rfcomm_detach */
 
 /*
@@ -789,11 +790,10 @@ ng_btsocket_rfcomm_disconnect(struct socket *so)
 		ng_btsocket_rfcomm_task_wakeup();
 		break;
 
-/*
- * 	case NG_BTSOCKET_RFCOMM_DLC_CLOSED:
- *	case NG_BTSOCKET_RFCOMM_DLC_W4_CONNECT:
- *	case NG_BTSOCKET_RFCOMM_DLC_DISCONNECTING:
- */
+	case NG_BTSOCKET_RFCOMM_DLC_CLOSED:
+	case NG_BTSOCKET_RFCOMM_DLC_W4_CONNECT:
+		break;
+
 	default:
 		panic("%s: Invalid DLC state=%d, flags=%#x\n",
 			__func__, pcb->state, pcb->flags);
@@ -810,7 +810,7 @@ ng_btsocket_rfcomm_disconnect(struct socket *so)
  */
 
 int
-ng_btsocket_rfcomm_listen(struct socket *so, struct thread *td)
+ng_btsocket_rfcomm_listen(struct socket *so, int backlog, struct thread *td)
 {
 	ng_btsocket_rfcomm_pcb_p	 pcb = so2rfcomm_pcb(so), pcb1;
 	ng_btsocket_rfcomm_session_p	 s = NULL;
@@ -870,6 +870,7 @@ ng_btsocket_rfcomm_listen(struct socket *so, struct thread *td)
 	mtx_lock(&ng_btsocket_rfcomm_sessions_mtx);
 	SOCK_LOCK(so);
 	error = solisten_proto_check(so);
+	SOCK_UNLOCK(so);
 	if (error != 0)
 		goto out;
 
@@ -901,9 +902,10 @@ ng_btsocket_rfcomm_listen(struct socket *so, struct thread *td)
 			goto out;
 		l2so = NULL;
 	}
-	solisten_proto(so);
-out:
+	SOCK_LOCK(so);
+	solisten_proto(so, backlog);
 	SOCK_UNLOCK(so);
+out:
 	mtx_unlock(&ng_btsocket_rfcomm_sessions_mtx);
 	/*
 	 * If we still have an l2so reference here, it's unneeded, so release
@@ -1203,9 +1205,8 @@ ng_btsocket_rfcomm_connect_ind(ng_btsocket_rfcomm_session_p s, int channel)
 static void
 ng_btsocket_rfcomm_connect_cfm(ng_btsocket_rfcomm_session_p s)
 {
-	ng_btsocket_rfcomm_pcb_p	 pcb = NULL, pcb_next = NULL;
-	struct socket			*so = NULL;
-	int				 error;
+	ng_btsocket_rfcomm_pcb_p	pcb = NULL, pcb_next = NULL;
+	int				error;
 
 	mtx_assert(&s->session_mtx, MA_OWNED);
 
@@ -1229,17 +1230,12 @@ ng_btsocket_rfcomm_connect_cfm(ng_btsocket_rfcomm_session_p s)
 			error = ng_btsocket_rfcomm_send_pn(pcb);
 			if (error == 0)
 				pcb->state = NG_BTSOCKET_RFCOMM_DLC_CONFIGURING;
-			else if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-				so = pcb->so;
-			else 
-				so = NULL;
+			else
+				ng_btsocket_rfcomm_pcb_kill(pcb, error);
 		}
 
 		mtx_unlock(&pcb->pcb_mtx);
 		pcb = pcb_next;
-
-		if (so != NULL)
-			ng_btsocket_rfcomm_detach(so);
 	}
 } /* ng_btsocket_rfcomm_connect_cfm */
 
@@ -1601,8 +1597,8 @@ ng_btsocket_rfcomm_session_receive(ng_btsocket_rfcomm_session_p s)
 		flags = MSG_DONTWAIT;
 
 		m = NULL;
-		error = (*s->l2so->so_proto->pr_usrreqs->pru_soreceive)(s->l2so,
-				NULL, &uio, &m, (struct mbuf **) NULL, &flags);
+		error = soreceive(s->l2so, NULL, &uio, &m,
+		    (struct mbuf **) NULL, &flags);
 		if (error != 0) {
 			if (error == EWOULDBLOCK)
 				return (0); /* XXX can happen? */
@@ -1653,9 +1649,8 @@ ng_btsocket_rfcomm_session_send(ng_btsocket_rfcomm_session_p s)
 			return (0); /* we are done */
 
 		/* Call send function on the L2CAP socket */
-		error = (*s->l2so->so_proto->pr_usrreqs->pru_sosend)
-				(s->l2so, NULL, NULL, m, NULL, 0,
-				 curthread /* XXX */);
+		error = (*s->l2so->so_proto->pr_usrreqs->pru_send)(s->l2so,
+				0, m, NULL, NULL, curthread /* XXX */);
 		if (error != 0) {
 			NG_BTSOCKET_RFCOMM_ERR(
 "%s: Could not send data to L2CAP socket, error=%d\n", __func__, error);
@@ -1675,9 +1670,8 @@ ng_btsocket_rfcomm_session_send(ng_btsocket_rfcomm_session_p s)
 static void
 ng_btsocket_rfcomm_session_clean(ng_btsocket_rfcomm_session_p s)
 {
-	ng_btsocket_rfcomm_pcb_p	 pcb = NULL, pcb_next = NULL;
-	struct socket			*so = NULL;
-	int				 error;
+	ng_btsocket_rfcomm_pcb_p	pcb = NULL, pcb_next = NULL;
+	int				error;
 
 	mtx_assert(&s->session_mtx, MA_OWNED);
 
@@ -1699,16 +1693,10 @@ ng_btsocket_rfcomm_session_clean(ng_btsocket_rfcomm_session_p s)
 		else
 			error = ECONNREFUSED;
 
-		if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-			so = pcb->so;
-		else
-			so = NULL;
+		ng_btsocket_rfcomm_pcb_kill(pcb, error);
 
 		mtx_unlock(&pcb->pcb_mtx);
 		pcb = pcb_next;
-
-		if (so != NULL)
-			ng_btsocket_rfcomm_detach(so);
 	}
 } /* ng_btsocket_rfcomm_session_clean */
 
@@ -1719,9 +1707,8 @@ ng_btsocket_rfcomm_session_clean(ng_btsocket_rfcomm_session_p s)
 static void
 ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 {
-	ng_btsocket_rfcomm_pcb_p	 pcb = NULL, pcb_next = NULL;
-	struct socket			*so = NULL;
-	int				 error;
+	ng_btsocket_rfcomm_pcb_p	pcb = NULL, pcb_next = NULL;
+	int				error;
 
 	mtx_assert(&s->session_mtx, MA_OWNED);
 
@@ -1731,8 +1718,6 @@ ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 	 */
 
 	for (pcb = LIST_FIRST(&s->dlcs); pcb != NULL; ) {
-		so = NULL;
-
 		mtx_lock(&pcb->pcb_mtx);
 		pcb_next = LIST_NEXT(pcb, session_next);
 
@@ -1744,12 +1729,10 @@ ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 		 */
 
 		case NG_BTSOCKET_RFCOMM_DLC_W4_CONNECT:
-			if (pcb->flags & NG_BTSOCKET_RFCOMM_DLC_DETACHED) {
-				if (ng_btsocket_rfcomm_pcb_kill(pcb, 0))
-					so = pcb->so;
-			} else if (pcb->flags & NG_BTSOCKET_RFCOMM_DLC_TIMEDOUT)
-				if (ng_btsocket_rfcomm_pcb_kill(pcb, ETIMEDOUT))
-					so = pcb->so;
+			if (pcb->flags & NG_BTSOCKET_RFCOMM_DLC_DETACHED)
+				ng_btsocket_rfcomm_pcb_kill(pcb, 0);
+			else if (pcb->flags & NG_BTSOCKET_RFCOMM_DLC_TIMEDOUT)
+				ng_btsocket_rfcomm_pcb_kill(pcb, ETIMEDOUT);
 			break;
 
 		/*
@@ -1761,8 +1744,7 @@ ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 		case NG_BTSOCKET_RFCOMM_DLC_CONFIGURING:
 		case NG_BTSOCKET_RFCOMM_DLC_CONNECTING:
 			if (pcb->flags & NG_BTSOCKET_RFCOMM_DLC_TIMEDOUT)
-				if (ng_btsocket_rfcomm_pcb_kill(pcb, ETIMEDOUT))
-					so = pcb->so;
+				ng_btsocket_rfcomm_pcb_kill(pcb, ETIMEDOUT);
 			break;
 
 		/*
@@ -1777,8 +1759,7 @@ ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 		case NG_BTSOCKET_RFCOMM_DLC_CONNECTED:
 			error = ng_btsocket_rfcomm_pcb_send(pcb, ALOT);
 			if (error != 0)
-				if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-					so = pcb->so;
+				ng_btsocket_rfcomm_pcb_kill(pcb, error);
 			break;
 
 		/*
@@ -1797,11 +1778,10 @@ ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 						pcb->dlci);
 				if (error == 0)
 					ng_btsocket_rfcomm_timeout(pcb);
-				else if (ng_btsocket_rfcomm_pcb_kill(pcb,error))
-					so = pcb->so;
+				else
+					ng_btsocket_rfcomm_pcb_kill(pcb, error);
 			} else if (pcb->flags & NG_BTSOCKET_RFCOMM_DLC_TIMEDOUT)
-				if (ng_btsocket_rfcomm_pcb_kill(pcb, ETIMEDOUT))
-					so = pcb->so;
+				ng_btsocket_rfcomm_pcb_kill(pcb, ETIMEDOUT);
 			break;
 		
 /*		case NG_BTSOCKET_RFCOMM_DLC_CLOSED: */
@@ -1813,9 +1793,6 @@ ng_btsocket_rfcomm_session_process_pcb(ng_btsocket_rfcomm_session_p s)
 
 		mtx_unlock(&pcb->pcb_mtx);
 		pcb = pcb_next;
-
-		if (so != NULL)
-			ng_btsocket_rfcomm_detach(so);
 	}
 } /* ng_btsocket_rfcomm_session_process_pcb */
 
@@ -1998,9 +1975,8 @@ ng_btsocket_rfcomm_receive_frame(ng_btsocket_rfcomm_session_p s,
 static int
 ng_btsocket_rfcomm_receive_sabm(ng_btsocket_rfcomm_session_p s, int dlci)
 {
-	ng_btsocket_rfcomm_pcb_p	 pcb = NULL;
-	struct socket			*so = NULL;
-	int				 error = 0;
+	ng_btsocket_rfcomm_pcb_p	pcb = NULL;
+	int				error = 0;
 
 	mtx_assert(&s->session_mtx, MA_OWNED);
 
@@ -2071,14 +2047,11 @@ ng_btsocket_rfcomm_receive_sabm(ng_btsocket_rfcomm_session_p s, int dlci)
 		if (error == 0) {
 			pcb->state = NG_BTSOCKET_RFCOMM_DLC_CONNECTED;
 			soisconnected(pcb->so);
-		} else if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-			so = pcb->so;
+		} else
+			ng_btsocket_rfcomm_pcb_kill(pcb, error);
 
 		mtx_unlock(&pcb->pcb_mtx);
 
-		if (so != NULL)
-			ng_btsocket_rfcomm_detach(so);
-
 		return (error);
 	}
 
@@ -2100,13 +2073,10 @@ ng_btsocket_rfcomm_receive_sabm(ng_btsocket_rfcomm_session_p s, int dlci)
 		if (error == 0) {
 			pcb->state = NG_BTSOCKET_RFCOMM_DLC_CONNECTED;
 			soisconnected(pcb->so);
-		} else if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-			so = pcb->so;
+		} else
+			ng_btsocket_rfcomm_pcb_kill(pcb, error);
 
 		mtx_unlock(&pcb->pcb_mtx);
-
-		if (so != NULL)
-			ng_btsocket_rfcomm_detach(so);
 	} else
 		/* Nobody is listen()ing on the requested DLCI */
 		error = ng_btsocket_rfcomm_send_command(s,RFCOMM_FRAME_DM,dlci);
@@ -2146,8 +2116,7 @@ ng_btsocket_rfcomm_receive_disc(ng_btsocket_rfcomm_session_p s, int dlci)
 	} else {
 		pcb = ng_btsocket_rfcomm_pcb_by_dlci(s, dlci);
 		if (pcb != NULL) {
-			struct socket	*so = NULL;
-			int		 err;
+			int	err;
 
 			mtx_lock(&pcb->pcb_mtx);
 
@@ -2163,13 +2132,9 @@ ng_btsocket_rfcomm_receive_disc(ng_btsocket_rfcomm_session_p s, int dlci)
 			else
 				err = ECONNREFUSED;
 
-			if (ng_btsocket_rfcomm_pcb_kill(pcb, err))
-				so = pcb->so;
+			ng_btsocket_rfcomm_pcb_kill(pcb, err);
 
 			mtx_unlock(&pcb->pcb_mtx);
-
-			if (so != NULL)
-				ng_btsocket_rfcomm_detach(so);
 		} else {
 			NG_BTSOCKET_RFCOMM_WARN(
 "%s: Got DISC for non-existing dlci=%d\n", __func__, dlci);
@@ -2226,8 +2191,6 @@ ng_btsocket_rfcomm_receive_ua(ng_btsocket_rfcomm_session_p s, int dlci)
 	/* Check if we have this DLCI */
 	pcb = ng_btsocket_rfcomm_pcb_by_dlci(s, dlci);
 	if (pcb != NULL) {
-		struct socket	*so = NULL;
-
 		mtx_lock(&pcb->pcb_mtx);
 
 		NG_BTSOCKET_RFCOMM_INFO(
@@ -2246,8 +2209,7 @@ ng_btsocket_rfcomm_receive_ua(ng_btsocket_rfcomm_session_p s, int dlci)
 			break;
 
 		case NG_BTSOCKET_RFCOMM_DLC_DISCONNECTING:
-			if (ng_btsocket_rfcomm_pcb_kill(pcb, 0))
-				so = pcb->so;
+			ng_btsocket_rfcomm_pcb_kill(pcb, 0);
 			break;
 
 		default:
@@ -2259,9 +2221,6 @@ ng_btsocket_rfcomm_receive_ua(ng_btsocket_rfcomm_session_p s, int dlci)
 		}
 
 		mtx_unlock(&pcb->pcb_mtx);
-
-		if (so != NULL)
-			ng_btsocket_rfcomm_detach(so);
 	} else {
 		NG_BTSOCKET_RFCOMM_WARN(
 "%s: Got UA for non-existing dlci=%d\n", __func__, dlci);
@@ -2296,8 +2255,6 @@ ng_btsocket_rfcomm_receive_dm(ng_btsocket_rfcomm_session_p s, int dlci)
 	} else {
 		pcb = ng_btsocket_rfcomm_pcb_by_dlci(s, dlci);
 		if (pcb != NULL) {
-			struct socket	*so = NULL;
-
 			mtx_lock(&pcb->pcb_mtx);
 
 			NG_BTSOCKET_RFCOMM_INFO(
@@ -2309,13 +2266,9 @@ ng_btsocket_rfcomm_receive_dm(ng_btsocket_rfcomm_session_p s, int dlci)
 			else
 				error = ECONNREFUSED;
 
-			if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-				so = pcb->so;
+			ng_btsocket_rfcomm_pcb_kill(pcb, error);
 
 			mtx_unlock(&pcb->pcb_mtx);
-
-			if (so != NULL)
-				ng_btsocket_rfcomm_detach(so);
 		} else
 			NG_BTSOCKET_RFCOMM_WARN(
 "%s: Got DM for non-existing dlci=%d\n", __func__, dlci);
@@ -2924,8 +2877,6 @@ ng_btsocket_rfcomm_receive_pn(ng_btsocket_rfcomm_session_p s, struct mbuf *m0)
 		pcb = ng_btsocket_rfcomm_connect_ind(s,
 				RFCOMM_SRVCHANNEL(pn->dlci));
 		if (pcb != NULL) {
-			struct socket	*so = NULL;
-
 			mtx_lock(&pcb->pcb_mtx);
 
 			pcb->dlci = pn->dlci;
@@ -2950,13 +2901,10 @@ ng_btsocket_rfcomm_receive_pn(ng_btsocket_rfcomm_session_p s, struct mbuf *m0)
 				ng_btsocket_rfcomm_timeout(pcb);
 				pcb->state = NG_BTSOCKET_RFCOMM_DLC_CONNECTING;
 				soisconnecting(pcb->so);
-			} else if (ng_btsocket_rfcomm_pcb_kill(pcb, error))
-				so = pcb->so;
+			} else
+				ng_btsocket_rfcomm_pcb_kill(pcb, error);
 
 			mtx_unlock(&pcb->pcb_mtx);
-
-			if (so != NULL)
-				ng_btsocket_rfcomm_detach(so);
 		} else {
 			/* Nobody is listen()ing on this channel */
 			error = ng_btsocket_rfcomm_send_command(s,
@@ -3364,7 +3312,7 @@ ng_btsocket_rfcomm_pcb_send(ng_btsocket_rfcomm_pcb_p pcb, int limit)
  * Caller must hold pcb->pcb_mtx and pcb->session->session_mtx
  */
 
-static int
+static void
 ng_btsocket_rfcomm_pcb_kill(ng_btsocket_rfcomm_pcb_p pcb, int error)
 {
 	ng_btsocket_rfcomm_session_p	s = pcb->session;
@@ -3433,8 +3381,6 @@ ng_btsocket_rfcomm_pcb_kill(ng_btsocket_rfcomm_pcb_p pcb, int error)
 
 		ng_btsocket_rfcomm_task_wakeup();
 	}
-
-	return (pcb->so->so_state & SS_NOFDREF);
 } /* ng_btsocket_rfcomm_pcb_kill */
 
 /*
