@@ -28,7 +28,7 @@
  */
 
 static const char rcs_id[] =
-    "@(#) $FreeBSD: src/sys/netgraph/netflow/netflow.c,v 1.15.2.4 2007/01/25 21:39:00 glebius Exp $";
+    "@(#) $FreeBSD: src/sys/netgraph/netflow/netflow.c,v 1.25.4.1 2008/01/30 21:29:10 mav Exp $";
 
 #include <sys/param.h>
 #include <sys/kernel.h>
@@ -96,7 +96,7 @@ static const char rcs_id[] =
 			 ((t) << 3))	/* 8   */
 
 MALLOC_DECLARE(M_NETFLOW_HASH);
-MALLOC_DEFINE(M_NETFLOW_HASH, "NetFlow hash", "NetFlow hash");
+MALLOC_DEFINE(M_NETFLOW_HASH, "netflow_hash", "NetFlow hash");
 
 static int export_add(item_p, struct flow_entry *);
 static int export_send(priv_p, item_p, int flags);
@@ -616,8 +616,9 @@ export_send(priv_p priv, item_p item, int flags)
 	header->count = htons(header->count);
 
 	if (priv->export != NULL)
-		/* Should also NET_LOCK_GIANT(). */
 		NG_FWD_ITEM_HOOK_FLAGS(error, item, priv->export, flags);
+	else
+		NG_FREE_ITEM(item);
 
 	return (error);
 }
