@@ -25,7 +25,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/netgraph/ng_eiface.c,v 1.32.2.6 2007/03/11 10:11:16 ru Exp $
+ * $FreeBSD: src/sys/netgraph/ng_eiface.c,v 1.39 2007/07/26 10:54:33 glebius Exp $
  */
 
 #include <sys/param.h>
@@ -234,6 +234,12 @@ ng_eiface_start2(node_p node, hook_p hook, void *arg1, int arg2)
 		 */
 		BPF_MTAP(ifp, m);
 
+		if (ifp->if_flags & IFF_MONITOR) {
+			ifp->if_ipackets++;
+			m_freem(m);
+			continue;
+		}
+
 		/*
 		 * Send packet; if hook is not connected, mbuf will get
 		 * freed.
