@@ -34,7 +34,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/sys/netgraph/ng_fec.c,v 1.18.2.5 2007/06/10 07:41:25 dwmalone Exp $
+ * $FreeBSD: src/sys/netgraph/ng_fec.c,v 1.30 2007/05/18 15:05:49 dwmalone Exp $
  */
 /*-
  * Copyright (c) 1996-1999 Whistle Communications, Inc.
@@ -104,7 +104,6 @@
 #include <net/if.h>
 #include <net/if_dl.h>
 #include <net/if_types.h>
-#include <net/if_arp.h>
 #include <net/if_media.h>
 #include <net/bpf.h>
 #include <net/ethernet.h>
@@ -302,7 +301,7 @@ ng_fec_get_unit(int *unit)
 	}
 	bit = ffs(ng_fec_units[index]) - 1;
 	KASSERT(bit >= 0 && bit <= UNITS_BITSPERWORD - 1,
-	    ("%s: word=%d bit=%d", __FUNCTION__, ng_fec_units[index], bit));
+	    ("%s: word=%d bit=%d", __func__, ng_fec_units[index], bit));
 	ng_fec_units[index] &= ~(1 << bit);
 	*unit = (index * UNITS_BITSPERWORD) + bit;
 	ng_units_in_use++;
@@ -322,9 +321,9 @@ ng_fec_free_unit(int unit)
 	bit = unit % UNITS_BITSPERWORD;
 	mtx_lock(&ng_fec_mtx);
 	KASSERT(index < ng_fec_units_len,
-	    ("%s: unit=%d len=%d", __FUNCTION__, unit, ng_fec_units_len));
+	    ("%s: unit=%d len=%d", __func__, unit, ng_fec_units_len));
 	KASSERT((ng_fec_units[index] & (1 << bit)) == 0,
-	    ("%s: unit=%d is free", __FUNCTION__, unit));
+	    ("%s: unit=%d is free", __func__, unit));
 	ng_fec_units[index] |= (1 << bit);
 	/*
 	 * XXX We could think about reducing the size of ng_fec_units[]
@@ -419,7 +418,7 @@ ng_fec_addport(struct ng_fec_private *priv, char *iface)
 	 * by extension, all the other ports in the bundle).
 	 */
 	if (b->fec_ifcnt == 0)
-		if_setlladdr(ifp, IFP2ENADDR(bifp), ETHER_ADDR_LEN);
+		if_setlladdr(ifp, IF_LLADDR(bifp), ETHER_ADDR_LEN);
 
 	b->fec_btype = FEC_BTYPE_MAC;
 	new->fec_idx = b->fec_ifcnt;
@@ -429,11 +428,11 @@ ng_fec_addport(struct ng_fec_private *priv, char *iface)
 	SLIST_INIT(&new->fec_mc_head);
 
 	/* Save the real MAC address. */
-	bcopy(IFP2ENADDR(bifp),
+	bcopy(IF_LLADDR(bifp),
 	    (char *)&new->fec_mac, ETHER_ADDR_LEN);
 
 	/* Set up phony MAC address. */
-	if_setlladdr(bifp, IFP2ENADDR(ifp), ETHER_ADDR_LEN);
+	if_setlladdr(bifp, IF_LLADDR(ifp), ETHER_ADDR_LEN);
 
 	/* Save original input vector */
 	new->fec_if_input = bifp->if_input;
@@ -556,10 +555,7 @@ ng_fec_ether_cmdmulti(struct ifnet *trifp, struct ng_fec_portlist *p, int set)
 	} else {
 		while ((mc = SLIST_FIRST(&p->fec_mc_head)) != NULL) {
 			SLIST_REMOVE(&p->fec_mc_head, mc, ng_fec_mc, mc_entries);
-			ifma = mc->mc_ifma;
-			bcopy(LLADDR((struct sockaddr_dl *)ifma->ifma_addr),
-			    LLADDR(&sdl), ETHER_ADDR_LEN);
-			if_delmulti(ifp, (struct sockaddr *)&sdl);
+			if_delmulti_ifma(mc->mc_ifma);
 			free(mc, M_DEVBUF);
 		}
 	}
@@ -1195,6 +1191,7 @@ ng_fec_constructor(node_p node)
 	char ifname[NG_FEC_FEC_NAME_MAX + 1];
 	struct ifnet *ifp;
 	priv_p priv;
+	const uint8_t eaddr[ETHER_ADDR_LEN] = {0, 0, 0, 0, 0, 0};
 	struct ng_fec_bundle *b;
 	int error = 0;
 
@@ -1245,7 +1242,7 @@ ng_fec_constructor(node_p node)
 		log(LOG_WARNING, "%s: can't acquire netgraph name\n", ifname);
 
 	/* Attach the interface */
-	ether_ifattach(ifp, IFP2ENADDR(priv->ifp));
+	ether_ifattach(ifp, eaddr);
 	callout_handle_init(&priv->fec_ch);
 
 	/* Override output method with our own */
