@@ -39,7 +39,7 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/sys/netgraph/ng_source.c,v 1.25.2.2 2006/01/31 15:39:05 glebius Exp $");
+__FBSDID("$FreeBSD: src/sys/netgraph/ng_source.c,v 1.30 2007/03/02 14:36:19 emaste Exp $");
 
 /*
  * This node is used for high speed packet geneneration.  It queues
@@ -77,6 +77,8 @@ __FBSDID("$FreeBSD: src/sys/netgraph/ng_source.c,v 1.25.2.2 2006/01/31 15:39:05
 #define NG_SOURCE_INTR_TICKS		1
 #define NG_SOURCE_DRIVER_IFQ_MAXLEN	(4*1024)
 
+#define	mtod_off(m,off,t)	((t)(mtod((m),caddr_t)+(off)))
+
 /* Per node info */
 struct privdata {
 	node_p				node;
@@ -84,10 +86,13 @@ struct privdata {
 	hook_p				output;
 	struct ng_source_stats		stats;
 	struct ifqueue			snd_queue;	/* packets to send */
+	struct mbuf			*last_packet;	/* last pkt in queue */
 	struct ifnet			*output_ifp;
 	struct callout			intr_ch;
 	uint64_t			packets;	/* packets to send */
 	uint32_t			queueOctets;
+	struct ng_source_embed_info	embed_timestamp;
+	struct ng_source_embed_cnt_info	embed_counter[NG_SOURCE_COUNTERS];
 };
 typedef struct privdata *sc_p;
 
@@ -110,6 +115,13 @@ static int		ng_source_start (sc_p, uint64_t);
 static void		ng_source_stop (sc_p);
 static int		ng_source_send (sc_p, int, int *);
 static int		ng_source_store_output_ifp(sc_p, char *);
+static void		ng_source_packet_mod(sc_p, struct mbuf *,
+			    int, int, caddr_t, int);
+static void		ng_source_mod_counter(sc_p sc,
+			    struct ng_source_embed_cnt_info *cnt,
+			    struct mbuf *m, int increment);
+static int		ng_source_dup_mod(sc_p, struct mbuf *,
+			    struct mbuf **);
 
 /* Parse type for timeval */
 static const struct ng_parse_struct_field ng_source_timeval_type_fields[] = {
@@ -130,6 +142,22 @@ static const struct ng_parse_type ng_source_stats_type = {
 	&ng_source_stats_type_fields
 };
 
+/* Parse type for struct ng_source_embed_info */
+static const struct ng_parse_struct_field ng_source_embed_type_fields[] =
+	NG_SOURCE_EMBED_TYPE_INFO;
+static const struct ng_parse_type ng_source_embed_type = {
+	&ng_parse_struct_type,
+	&ng_source_embed_type_fields
+};
+
+/* Parse type for struct ng_source_embed_cnt_info */
+static const struct ng_parse_struct_field ng_source_embed_cnt_type_fields[] =
+	NG_SOURCE_EMBED_CNT_TYPE_INFO;
+static const struct ng_parse_type ng_source_embed_cnt_type = {
+	&ng_parse_struct_type,
+	&ng_source_embed_cnt_type_fields
+};
+
 /* List of commands and how to convert arguments to/from ASCII */
 static const struct ng_cmdlist ng_source_cmds[] = {
 	{
@@ -188,6 +216,34 @@ static const struct ng_cmdlist ng_source_cmds[] = {
 	  &ng_parse_uint32_type,
 	  NULL
 	},
+	{
+	  NGM_SOURCE_COOKIE,
+	  NGM_SOURCE_SET_TIMESTAMP,
+	  "settimestamp",
+	  &ng_source_embed_type,
+	  NULL
+	},
+	{
+	  NGM_SOURCE_COOKIE,
+	  NGM_SOURCE_GET_TIMESTAMP,
+	  "gettimestamp",
+	  NULL,
+	  &ng_source_embed_type
+	},
+	{
+	  NGM_SOURCE_COOKIE,
+	  NGM_SOURCE_SET_COUNTER,
+	  "setcounter",
+	  &ng_source_embed_cnt_type,
+	  NULL
+	},
+	{
+	  NGM_SOURCE_COOKIE,
+	  NGM_SOURCE_GET_COUNTER,
+	  "getcounter",
+	  &ng_parse_uint8_type,
+	  &ng_source_embed_cnt_type
+	},
 	{ 0 }
 };
 
@@ -372,6 +428,72 @@ ng_source_rcvmsg(node_p node, item_p item, hook_p lasthook)
 
 			sc->stats.maxPps = pps;
 
+			break;
+		    }
+		case NGM_SOURCE_SET_TIMESTAMP:
+		    {
+			struct ng_source_embed_info *embed;
+
+			if (msg->header.arglen != sizeof(*embed)) {
+				error = EINVAL;
+				goto done;
+			}
+			embed = (struct ng_source_embed_info *)msg->data;
+			bcopy(embed, &sc->embed_timestamp, sizeof(*embed));
+
+			break;
+		    }
+		case NGM_SOURCE_GET_TIMESTAMP:
+		    {
+			struct ng_source_embed_info *embed;
+
+			NG_MKRESPONSE(resp, msg, sizeof(*embed), M_DONTWAIT);
+			if (resp == NULL) {
+				error = ENOMEM;
+				goto done;
+			}
+			embed = (struct ng_source_embed_info *)resp->data;
+			bcopy(&sc->embed_timestamp, embed, sizeof(*embed));
+
+			break;
+		    }
+		case NGM_SOURCE_SET_COUNTER:
+		    {
+			struct ng_source_embed_cnt_info *embed;
+
+			if (msg->header.arglen != sizeof(*embed)) {
+				error = EINVAL;
+				goto done;
+			}
+			embed = (struct ng_source_embed_cnt_info *)msg->data;
+			if (embed->index >= NG_SOURCE_COUNTERS ||
+			    !(embed->width == 1 || embed->width == 2 ||
+			    embed->width == 4)) {
+				error = EINVAL;
+				goto done;
+			}
+			bcopy(embed, &sc->embed_counter[embed->index],
+			    sizeof(*embed));
+
+			break;
+		    }
+		case NGM_SOURCE_GET_COUNTER:
+		    {
+			uint8_t index = *(uint8_t *)msg->data;
+			struct ng_source_embed_cnt_info *embed;
+
+			if (index >= NG_SOURCE_COUNTERS) {
+				error = EINVAL;
+				goto done;
+			}
+			NG_MKRESPONSE(resp, msg, sizeof(*embed), M_DONTWAIT);
+			if (resp == NULL) {
+				error = ENOMEM;
+				goto done;
+			}
+			embed = (struct ng_source_embed_cnt_info *)resp->data;
+			bcopy(&sc->embed_counter[index], embed, sizeof(*embed));
+
 			break;
 		    }
 		default:
@@ -443,6 +565,7 @@ ng_source_rcvdata(hook_p hook, item_p item)
 	/* XXX should we check IF_QFULL() ? */
 	_IF_ENQUEUE(&sc->snd_queue, m);
 	sc->queueOctets += m->m_pkthdr.len;
+	sc->last_packet = m;
 
 	return (0);
 }
@@ -548,6 +671,7 @@ ng_source_clr_data (sc_p sc)
 		NG_FREE_M(m);
 	}
 	sc->queueOctets = 0;
+	sc->last_packet = NULL;
 }
 
 /*
@@ -662,11 +786,13 @@ ng_source_send(sc_p sc, int tosend, int *sent_p)
 		if (m == NULL)
 			break;
 
-		/* Duplicate the packet. */
-		m2 = m_copypacket(m, M_DONTWAIT);
-		if (m2 == NULL) {
-			_IF_PREPEND(&sc->snd_queue, m);
-			error = ENOBUFS;
+		/* Duplicate and modify the packet. */
+		error = ng_source_dup_mod(sc, m, &m2);
+		if (error) {
+			if (error == ENOBUFS)
+				_IF_PREPEND(&sc->snd_queue, m);
+			else
+				_IF_ENQUEUE(&sc->snd_queue, m);
 			break;
 		}
 
@@ -685,3 +811,110 @@ ng_source_send(sc_p sc, int tosend, int *sent_p)
 		*sent_p = sent;
 	return (error);
 }
+
+/*
+ * Modify packet in 'm' by changing 'len' bytes starting at 'offset'
+ * to data in 'cp'.
+ *
+ * The packet data in 'm' must be in a contiguous buffer in a single mbuf.
+ */
+static void
+ng_source_packet_mod(sc_p sc, struct mbuf *m, int offset, int len, caddr_t cp,
+    int flags)
+{
+	if (len == 0)
+		return;
+
+	/* Can't modify beyond end of packet. */
+	/* TODO: Pad packet for this case. */
+	if (offset + len > m->m_len)
+		return;
+
+	bcopy(cp, mtod_off(m, offset, caddr_t), len);
+}
+
+static void
+ng_source_mod_counter(sc_p sc, struct ng_source_embed_cnt_info *cnt,
+    struct mbuf *m, int increment)
+{
+	caddr_t cp;
+	uint32_t val;
+
+	val = htonl(cnt->next_val);
+	cp = (caddr_t)&val + sizeof(val) - cnt->width;
+	ng_source_packet_mod(sc, m, cnt->offset, cnt->width, cp, cnt->flags);
+
+	if (increment) {
+		cnt->next_val += increment;
+
+		if (increment > 0 && cnt->next_val > cnt->max_val) {
+			cnt->next_val = cnt->min_val - 1 +
+			    (cnt->next_val - cnt->max_val);
+			if (cnt->next_val > cnt->max_val)
+				cnt->next_val = cnt->max_val;
+		} else if (increment < 0 && cnt->next_val < cnt->min_val) {
+			cnt->next_val = cnt->max_val + 1 +
+			    (cnt->next_val - cnt->min_val);
+			if (cnt->next_val < cnt->min_val)
+				cnt->next_val = cnt->max_val;
+		}
+	}
+}
+
+static int
+ng_source_dup_mod(sc_p sc, struct mbuf *m0, struct mbuf **m_ptr)
+{
+	struct mbuf *m;
+	struct ng_source_embed_cnt_info *cnt;
+	struct ng_source_embed_info *ts;
+	int modify;
+	int error = 0;
+	int i, increment;
+
+	/* Are we going to modify packets? */
+	modify = sc->embed_timestamp.flags & NGM_SOURCE_EMBED_ENABLE;
+	for (i = 0; !modify && i < NG_SOURCE_COUNTERS; ++i)
+		modify = sc->embed_counter[i].flags & NGM_SOURCE_EMBED_ENABLE;
+
+	/* Duplicate the packet. */
+	if (modify)
+		m = m_dup(m0, M_DONTWAIT);
+	else
+		m = m_copypacket(m0, M_DONTWAIT);
+	if (m == NULL) {
+		error = ENOBUFS;
+		goto done;
+	}
+	*m_ptr = m;
+
+	if (!modify)
+		goto done;
+
+	/* Modify the copied packet for sending. */
+	KASSERT(M_WRITABLE(m), ("%s: packet not writable", __func__));
+
+	for (i = 0; i < NG_SOURCE_COUNTERS; ++i) {
+		cnt = &sc->embed_counter[i];
+		if (cnt->flags & NGM_SOURCE_EMBED_ENABLE) {
+			if ((cnt->flags & NGM_SOURCE_INC_CNT_PER_LIST) == 0 ||
+			    sc->last_packet == m0)
+				increment = cnt->increment;
+			else
+				increment = 0;
+			ng_source_mod_counter(sc, cnt, m, increment);
+		}
+	}
+
+	ts = &sc->embed_timestamp;
+	if (ts->flags & NGM_SOURCE_EMBED_ENABLE) {
+		struct timeval now;
+		getmicrotime(&now);
+		now.tv_sec = htonl(now.tv_sec);
+		now.tv_usec = htonl(now.tv_usec);
+		ng_source_packet_mod(sc, m, ts->offset, sizeof (now),
+		    (caddr_t)&now, ts->flags);
+	}
+
+done:
+	return(error);
+}
