@@ -14,7 +14,7 @@
  * as long as this message is kept with the software, all derivative
  * works or modified versions.
  *
- * $FreeBSD: src/sys/netgraph/ng_sppp.h,v 1.1.2.1 2005/01/31 23:26:29 imp Exp $
+ * $FreeBSD: src/sys/netgraph/ng_sppp.h,v 1.3 2005/02/03 13:03:31 ru Exp $
  * Cronyx Id: ng_sppp.h,v 1.1.2.6 2004/03/01 15:17:21 rik Exp $
  */
 
@@ -27,7 +27,6 @@
 
 /* Interface base name */
 #define NG_SPPP_IFACE_NAME		"sppp"
-#define NG_SPPP_IFACE_NAME_MAX		15
 
 /* My hook names */
 #define NG_SPPP_HOOK_DOWNSTREAM		"downstream"
@@ -37,8 +36,4 @@ enum {
 	NGM_SPPP_GET_IFNAME = 1,	/* returns struct ng_sppp_ifname */
 };
 
-struct ng_sppp_ifname {
-	char    ngif_name[NG_SPPP_IFACE_NAME_MAX + 1];
-};
-
 #endif /* _NETGRAPH_SPPP_H_ */
