@@ -23,10 +23,11 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: src/sys/netinet/ip_dummynet.c,v 1.93.2.6 2007/03/21 17:25:15 oleg Exp $
  */
 
+#include <sys/cdefs.h>
+__FBSDID("$FreeBSD: src/sys/netinet/ip_dummynet.c,v 1.110 2007/10/07 20:44:22 silby Exp $");
+
 #define	DUMMYNET_DEBUG
 
 #include "opt_inet6.h"
@@ -79,7 +80,6 @@
 #include <netinet/ip_var.h>
 
 #include <netinet/if_ether.h> /* for struct arpcom */
-#include <net/bridge.h>
 
 #include <netinet/ip6.h>       /* for ip6_input, ip6_output prototypes */
 #include <netinet6/ip6_var.h>
@@ -204,10 +204,7 @@ static struct mtx dummynet_mtx;
 #define	DUMMYNET_LOCK_DESTROY()	mtx_destroy(&dummynet_mtx)
 #define	DUMMYNET_LOCK()		mtx_lock(&dummynet_mtx)
 #define	DUMMYNET_UNLOCK()	mtx_unlock(&dummynet_mtx)
-#define	DUMMYNET_LOCK_ASSERT()	do {				\
-	mtx_assert(&dummynet_mtx, MA_OWNED);			\
-	NET_ASSERT_GIANT();					\
-} while (0)
+#define	DUMMYNET_LOCK_ASSERT()	mtx_assert(&dummynet_mtx, MA_OWNED)
 
 static int config_pipe(struct dn_pipe *p);
 static int ip_dn_ctl(struct sockopt *sopt);
@@ -507,8 +504,8 @@ transmit_event(struct dn_pipe *pipe, struct mbuf **head, struct mbuf **tail)
  * and put into delay line (p_queue)
  */
 static void
-move_pkt(struct mbuf *pkt, struct dn_flow_queue *q,
-	struct dn_pipe *p, int len)
+move_pkt(struct mbuf *pkt, struct dn_flow_queue *q, struct dn_pipe *p,
+    int len)
 {
     struct dn_pkt_tag *dt = dn_tag_get(pkt);
 
@@ -722,6 +719,7 @@ ready_event_wfq(struct dn_pipe *p, struct mbuf **head, struct mbuf **tail)
 static void
 dummynet(void * __unused unused)
 {
+
 	taskqueue_enqueue(dn_tq, &dn_task);
 }
 
@@ -731,7 +729,6 @@ dummynet(void * __unused unused)
 static void
 dummynet_task(void *context, int pending)
 {
-
 	struct mbuf *head = NULL, *tail = NULL;
 	struct dn_pipe *pipe;
 	struct dn_heap *heaps[3];
@@ -739,7 +736,6 @@ dummynet_task(void *context, int pending)
 	void *p;	/* generic parameter to handler */
 	int i;
 
-	NET_LOCK_GIANT();
 	DUMMYNET_LOCK();
 
 	heaps[0] = &ready_heap;			/* fixed-rate queues */
@@ -826,8 +822,6 @@ dummynet_task(void *context, int pending)
 		dummynet_send(head);
 
 	callout_reset(&dn_timeout, 1, dummynet, NULL);
-
-	NET_UNLOCK_GIANT();
 }
 
 static void
@@ -867,32 +861,6 @@ dummynet_send(struct mbuf *m)
 				printf("dummynet: if_bridge not loaded\n");
 
 			break;
-		case DN_TO_BDG_FWD :
-			/*
-			 * The bridge requires/assumes the Ethernet header is
-			 * contiguous in the first mbuf header.  Ensure this
-			 * is true.
-			 */
-			if (BDG_LOADED) {
-				if (m->m_len < ETHER_HDR_LEN &&
-				    (m = m_pullup(m, ETHER_HDR_LEN)) == NULL) {
-					printf("dummynet/bridge: pullup fail, "
-					    "dropping pkt\n");
-					break;
-				}
-				m = bdg_forward_ptr(m, pkt->ifp);
-			} else {
-				/*
-				 * somebody unloaded the bridge module.
-				 * Drop pkt
-				 */
-				/* XXX rate limit */
-				printf("dummynet: dropping bridged packet "
-				    "trapped in pipe\n");
-			}
-			if (m)
-				m_freem(m);
-			break;
 		case DN_TO_ETH_DEMUX:
 			/*
 			 * The Ethernet code assumes the Ethernet header is
@@ -928,9 +896,9 @@ expire_queues(struct dn_flow_set *fs)
     struct dn_flow_queue *q, *prev ;
     int i, initial_elements = fs->rq_elements ;
 
-    if (fs->last_expired == time_second)
+    if (fs->last_expired == time_uptime)
 	return 0 ;
-    fs->last_expired = time_second ;
+    fs->last_expired = time_uptime ;
     for (i = 0 ; i <= fs->rq_size ; i++) /* last one is overflow */
 	for (prev=NULL, q = fs->rq[i] ; q != NULL ; )
 	    if (q->head != NULL || q->S != q->F+1) {
@@ -1231,7 +1199,6 @@ locate_pipe(int pipe_nr)
  * m		the mbuf with the packet
  * ifp		the 'ifp' parameter from the caller.
  *		NULL in ip_input, destination interface in ip_output,
- *		real_dst in bdg_forward
  * rule		matching rule, in case of multiple passes
  *
  */
@@ -1579,7 +1546,7 @@ dn_rule_delete(void *r)
  * setup RED parameters
  */
 static int
-config_red(struct dn_flow_set *p, struct dn_flow_set * x)
+config_red(struct dn_flow_set *p, struct dn_flow_set *x)
 {
 	int i;
 
@@ -1724,7 +1691,7 @@ config_pipe(struct dn_pipe *p)
 			 */
 			pipe->idle_heap.size = pipe->idle_heap.elements = 0;
 			pipe->idle_heap.offset =
-			    OFFSET_OF(struct dn_flow_queue, heap_pos);
+			    offsetof(struct dn_flow_queue, heap_pos);
 		} else
 			/* Flush accumulated credit for all queues. */
 			for (i = 0; i <= pipe->fs.rq_size; i++)
@@ -1846,7 +1813,7 @@ pipe_remove_from_heap(struct dn_heap *h, struct dn_pipe *p)
  * drain all queues. Called in case of severe mbuf shortage.
  */
 void
-dummynet_drain()
+dummynet_drain(void)
 {
     struct dn_flow_set *fs;
     struct dn_pipe *pipe;
@@ -1883,6 +1850,7 @@ dummynet_drain()
 static int
 delete_pipe(struct dn_pipe *p)
 {
+
     if (p->pipe_nr == 0 && p->fs.fs_nr == 0)
 	return EINVAL ;
     if (p->pipe_nr != 0 && p->fs.fs_nr != 0)
@@ -2168,7 +2136,7 @@ ip_dn_init(void)
 	    taskqueue_thread_enqueue, &dn_tq);
 	taskqueue_start_threads(&dn_tq, 1, PI_NET, "dummynet");
 
-	callout_init(&dn_timeout, NET_CALLOUT_MPSAFE);
+	callout_init(&dn_timeout, CALLOUT_MPSAFE);
 	callout_reset(&dn_timeout, 1, dummynet, NULL);
 
 	/* Initialize curr_time adjustment mechanics. */
@@ -2198,6 +2166,7 @@ ip_dn_destroy(void)
 static int
 dummynet_modevent(module_t mod, int type, void *data)
 {
+
 	switch (type) {
 	case MOD_LOAD:
 		if (DUMMYNET_LOADED) {
