@@ -1,4 +1,3 @@
-/*	$FreeBSD: src/sys/netinet/ip_encap.c,v 1.20 2005/01/07 01:45:44 imp Exp $	*/
 /*	$KAME: ip_encap.c,v 1.41 2001/03/15 08:35:08 itojun Exp $	*/
 
 /*-
@@ -57,6 +56,9 @@
  */
 /* XXX is M_NETADDR correct? */
 
+#include <sys/cdefs.h>
+__FBSDID("$FreeBSD: src/sys/netinet/ip_encap.c,v 1.24 2007/10/07 20:44:23 silby Exp $");
+
 #include "opt_mrouting.h"
 #include "opt_inet.h"
 #include "opt_inet6.h"
@@ -87,11 +89,9 @@
 
 #include <machine/stdarg.h>
 
-#include <net/net_osdep.h>
-
 #include <sys/kernel.h>
 #include <sys/malloc.h>
-static MALLOC_DEFINE(M_NETADDR, "Export Host", "Export host address structure");
+static MALLOC_DEFINE(M_NETADDR, "encap_export_host", "Export host address structure");
 
 static void encap_add(struct encaptab *);
 static int mask_match(const struct encaptab *, const struct sockaddr *,
@@ -110,15 +110,13 @@ LIST_HEAD(, encaptab) encaptab = LIST_HEAD_INITIALIZER(&encaptab);
  * it's referenced by KAME pieces in netinet6.
  */
 void
-encap_init()
+encap_init(void)
 {
 }
 
 #ifdef INET
 void
-encap4_input(m, off)
-	struct mbuf *m;
-	int off;
+encap4_input(struct mbuf *m, int off)
 {
 	struct ip *ip;
 	int proto;
@@ -203,10 +201,7 @@ encap4_input(m, off)
 
 #ifdef INET6
 int
-encap6_input(mp, offp, proto)
-	struct mbuf **mp;
-	int *offp;
-	int proto;
+encap6_input(struct mbuf **mp, int *offp, int proto)
 {
 	struct mbuf *m = *mp;
 	struct ip6_hdr *ip6;
@@ -274,8 +269,7 @@ encap6_input(mp, offp, proto)
 
 /*lint -sem(encap_add, custodial(1)) */
 static void
-encap_add(ep)
-	struct encaptab *ep;
+encap_add(struct encaptab *ep)
 {
 
 	mtx_assert(&encapmtx, MA_OWNED);
@@ -288,13 +282,9 @@ encap_add(ep)
  * Return value will be necessary as input (cookie) for encap_detach().
  */
 const struct encaptab *
-encap_attach(af, proto, sp, sm, dp, dm, psw, arg)
-	int af;
-	int proto;
-	const struct sockaddr *sp, *sm;
-	const struct sockaddr *dp, *dm;
-	const struct protosw *psw;
-	void *arg;
+encap_attach(int af, int proto, const struct sockaddr *sp,
+    const struct sockaddr *sm, const struct sockaddr *dp,
+    const struct sockaddr *dm, const struct protosw *psw, void *arg)
 {
 	struct encaptab *ep;
 
@@ -348,12 +338,9 @@ encap_attach(af, proto, sp, sm, dp, dm, psw, arg)
 }
 
 const struct encaptab *
-encap_attach_func(af, proto, func, psw, arg)
-	int af;
-	int proto;
-	int (*func)(const struct mbuf *, int, int, void *);
-	const struct protosw *psw;
-	void *arg;
+encap_attach_func(int af, int proto,
+    int (*func)(const struct mbuf *, int, int, void *),
+    const struct protosw *psw, void *arg)
 {
 	struct encaptab *ep;
 
@@ -379,8 +366,7 @@ encap_attach_func(af, proto, func, psw, arg)
 }
 
 int
-encap_detach(cookie)
-	const struct encaptab *cookie;
+encap_detach(const struct encaptab *cookie)
 {
 	const struct encaptab *ep = cookie;
 	struct encaptab *p;
@@ -400,10 +386,8 @@ encap_detach(cookie)
 }
 
 static int
-mask_match(ep, sp, dp)
-	const struct encaptab *ep;
-	const struct sockaddr *sp;
-	const struct sockaddr *dp;
+mask_match(const struct encaptab *ep, const struct sockaddr *sp,
+    const struct sockaddr *dp)
 {
 	struct sockaddr_storage s;
 	struct sockaddr_storage d;
@@ -453,9 +437,7 @@ mask_match(ep, sp, dp)
 }
 
 static void
-encap_fillarg(m, ep)
-	struct mbuf *m;
-	const struct encaptab *ep;
+encap_fillarg(struct mbuf *m, const struct encaptab *ep)
 {
 	struct m_tag *tag;
 
@@ -467,8 +449,7 @@ encap_fillarg(m, ep)
 }
 
 void *
-encap_getarg(m)
-	struct mbuf *m;
+encap_getarg(struct mbuf *m)
 {
 	void *p = NULL;
 	struct m_tag *tag;
