@@ -1,4 +1,4 @@
-/*	$FreeBSD: src/sys/netipsec/ipsec_osdep.h,v 1.2.2.1 2006/09/04 10:36:47 pjd Exp $	*/
+/*	$FreeBSD: src/sys/netipsec/ipsec_osdep.h,v 1.6 2007/06/12 00:12:00 rwatson Exp $	*/
 /*	$NetBSD: ipsec_osdep.h,v 1.1 2003/08/13 20:06:51 jonathan Exp $	*/
 
 /*-
@@ -211,15 +211,16 @@ if_handoff(struct ifqueue *ifq, struct mbuf *m, struct ifnet *ifp, int adjust)
 
 /*
  * 8. Test for "privileged" socket opened by superuser.
- * FreeBSD tests  ((so)->so_cred != NULL && suser_cred((so)->so_cred, SUSER_ALLOWJAIL == 0))
+ * FreeBSD tests  ((so)->so_cred != NULL && priv_check_cred((so)->so_cred,
+ * PRIV_NETINET_IPSEC, 0) == 0).
  * NetBSD (1.6N) tests (so)->so_uid == 0).
  * This difference is wrapped inside  the IPSEC_PRIVILEGED_SO() macro.
- *
  */
 #ifdef __FreeBSD__ 
 #define IPSEC_IS_PRIVILEGED_SO(_so) \
 	((_so)->so_cred != NULL && \
-	 suser_cred((_so)->so_cred, SUSER_ALLOWJAIL) == 0)
+	 priv_check_cred((_so)->so_cred, PRIV_NETINET_IPSEC, 0) \
+	 == 0)
 #endif	/* __FreeBSD__ */
 
 #ifdef __NetBSD__
